# -*- coding: utf-8 -*-

from tiramisu import (UnicodeOption, OptionDescription, PortOption, IntOption,
                      ChoiceOption, SymLinkOption, IPOption, NetworkOption,
                      NetmaskOption, DomainnameOption, BroadcastOption,
                      URLOption, EmailOption, FilenameOption, DateOption,
                      PasswordOption, BoolOption, UsernameOption, Params,
                      ParamOption, ParamValue, Leadership,
                      tiramisu_copy, Config)
from examples.AmonEcole import eosfunc


def get_config(display_name=None):
    option_139789498664080 = UnicodeOption(**{'default': u'192.0.2.1', 'doc': u'Adresse IP du bridge', 'multi': False, 'properties': (), 'name': 'adresse_ip_br0'})
    option_139789498664352 = UnicodeOption(**{'default': u'255.255.255.0', 'doc': u'Adresse IP du netmask du bridge', 'multi': False, 'properties': (), 'name': 'adresse_netmask_br0'})
    option_139789498664624 = UnicodeOption(**{'default': u'192.0.2.0', 'doc': u'Adresse r\xe9seau du bridge br0', 'multi': False, 'properties': (), 'name': 'adresse_network_br0'})
    option_139789498664896 = UnicodeOption(**{'default': u'192.0.2.255', 'doc': u'Adresse IP du broadcast du bridge', 'multi': False, 'properties': (), 'name': 'adresse_broadcast_br0'})
    option_139789498665168 = UnicodeOption(**{'default': u'', 'doc': u'Chemin du conteneur root', 'multi': False, 'properties': (), 'name': 'container_path_root'})
    option_139789498665440 = UnicodeOption(**{'default': u'127.0.0.1', 'doc': u'Adresse IP du conteneur root', 'multi': False, 'properties': (), 'name': 'container_ip_root'})
    option_139789498665712 = UnicodeOption(**{'default': u'root', 'doc': u'Nom du groupe du conteneur root', 'multi': False, 'properties': (), 'name': 'container_name_root'})
    option_139789497844632 = SymLinkOption(**{'name': 'adresse_ip_root', "opt": option_139789498665440})
    option_139789498665984 = UnicodeOption(**{'doc': u'Chemin du conteneur all', 'multi': False, 'properties': (), 'name': 'container_path_all'})
    option_139789498666256 = UnicodeOption(**{'doc': u'Adresse IP du conteneur all', 'multi': False, 'properties': (), 'name': 'container_ip_all'})
    option_139789498666528 = UnicodeOption(**{'default': u'all', 'doc': u'Nom du groupe du conteneur all', 'multi': False, 'properties': (), 'name': 'container_name_all'})
    option_139789497846032 = SymLinkOption(**{'name': 'adresse_ip_all', "opt": option_139789498666256})
    option_139789497851984 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs', 'doc': u'Chemin du conteneur annuaire', 'multi': False, 'properties': (), 'name': 'container_path_annuaire'})
    option_139789497852256 = UnicodeOption(**{'default': u'192.0.2.50', 'doc': u'Adresse IP du conteneur annuaire', 'multi': False, 'properties': (), 'name': 'container_ip_annuaire'})
    option_139789497852528 = UnicodeOption(**{'default': u'bdd', 'doc': u'Nom du groupe du conteneur annuaire', 'multi': False, 'properties': (), 'name': 'container_name_annuaire'})
    option_139789497847432 = SymLinkOption(**{'name': 'adresse_ip_annuaire', "opt": option_139789497852256})
    option_139789497852800 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs', 'doc': u'Chemin du conteneur mail', 'multi': False, 'properties': (), 'name': 'container_path_mail'})
    option_139789497853072 = UnicodeOption(**{'default': u'192.0.2.51', 'doc': u'Adresse IP du conteneur mail', 'multi': False, 'properties': (), 'name': 'container_ip_mail'})
    option_139789497853344 = UnicodeOption(**{'default': u'reseau', 'doc': u'Nom du groupe du conteneur mail', 'multi': False, 'properties': (), 'name': 'container_name_mail'})
    option_139789497853616 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs', 'doc': u'Chemin du conteneur dhcp', 'multi': False, 'properties': (), 'name': 'container_path_dhcp'})
    option_139789497853888 = UnicodeOption(**{'default': u'192.0.2.52', 'doc': u'Adresse IP du conteneur dhcp', 'multi': False, 'properties': (), 'name': 'container_ip_dhcp'})
    option_139789497854160 = UnicodeOption(**{'default': u'partage', 'doc': u'Nom du groupe du conteneur dhcp', 'multi': False, 'properties': (), 'name': 'container_name_dhcp'})
    option_139789497858040 = SymLinkOption(**{'name': 'adresse_ip_dhcp', "opt": option_139789497853888})
    option_139789497854432 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs', 'doc': u'Chemin du conteneur fichier', 'multi': False, 'properties': (), 'name': 'container_path_fichier'})
    option_139789497854704 = UnicodeOption(**{'default': u'192.0.2.52', 'doc': u'Adresse IP du conteneur fichier', 'multi': False, 'properties': (), 'name': 'container_ip_fichier'})
    option_139789497854976 = UnicodeOption(**{'default': u'partage', 'doc': u'Nom du groupe du conteneur fichier', 'multi': False, 'properties': (), 'name': 'container_name_fichier'})
    option_139789497859440 = SymLinkOption(**{'name': 'adresse_ip_fichier', "opt": option_139789497854704})
    option_139789497855248 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs', 'doc': u'Chemin du conteneur web', 'multi': False, 'properties': (), 'name': 'container_path_web'})
    option_139789497855520 = UnicodeOption(**{'default': u'192.0.2.51', 'doc': u'Adresse IP du conteneur web', 'multi': False, 'properties': (), 'name': 'container_ip_web'})
    option_139789497864272 = UnicodeOption(**{'default': u'reseau', 'doc': u'Nom du groupe du conteneur web', 'multi': False, 'properties': (), 'name': 'container_name_web'})
    option_139789497861016 = SymLinkOption(**{'name': 'adresse_ip_web', "opt": option_139789497855520})
    option_139789497864544 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs', 'doc': u'Chemin du conteneur dns', 'multi': False, 'properties': (), 'name': 'container_path_dns'})
    option_139789497864816 = UnicodeOption(**{'default': u'192.0.2.53', 'doc': u'Adresse IP du conteneur dns', 'multi': False, 'properties': (), 'name': 'container_ip_dns'})
    option_139789497865088 = UnicodeOption(**{'default': u'internet', 'doc': u'Nom du groupe du conteneur dns', 'multi': False, 'properties': (), 'name': 'container_name_dns'})
    option_139789497865360 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs', 'doc': u'Chemin du conteneur mysql', 'multi': False, 'properties': (), 'name': 'container_path_mysql'})
    option_139789497865632 = UnicodeOption(**{'default': u'192.0.2.50', 'doc': u'Adresse IP du conteneur mysql', 'multi': False, 'properties': (), 'name': 'container_ip_mysql'})
    option_139789497865904 = UnicodeOption(**{'default': u'bdd', 'doc': u'Nom du groupe du conteneur mysql', 'multi': False, 'properties': (), 'name': 'container_name_mysql'})
    option_139789497863256 = SymLinkOption(**{'name': 'adresse_ip_mysql', "opt": option_139789497865632})
    option_139789497866176 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs', 'doc': u'Chemin du conteneur proxy', 'multi': False, 'properties': (), 'name': 'container_path_proxy'})
    option_139789497866448 = UnicodeOption(**{'default': u'192.0.2.53', 'doc': u'Adresse IP du conteneur proxy', 'multi': False, 'properties': (), 'name': 'container_ip_proxy'})
    option_139789497866720 = UnicodeOption(**{'default': u'internet', 'doc': u'Nom du groupe du conteneur proxy', 'multi': False, 'properties': (), 'name': 'container_name_proxy'})
    option_139789497868928 = SymLinkOption(**{'name': 'adresse_ip_proxy', "opt": option_139789497866448})
    option_139789497866992 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs', 'doc': u'Chemin du conteneur ftp', 'multi': False, 'properties': (), 'name': 'container_path_ftp'})
    option_139789497867264 = UnicodeOption(**{'default': u'192.0.2.52', 'doc': u'Adresse IP du conteneur ftp', 'multi': False, 'properties': (), 'name': 'container_ip_ftp'})
    option_139789497867536 = UnicodeOption(**{'default': u'partage', 'doc': u'Nom du groupe du conteneur ftp', 'multi': False, 'properties': (), 'name': 'container_name_ftp'})
    option_139789497870328 = SymLinkOption(**{'name': 'adresse_ip_ftp', "opt": option_139789497867264})
    option_139789497867808 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs', 'doc': u'Chemin du conteneur internet', 'multi': False, 'properties': (), 'name': 'container_path_internet'})
    option_139789497872464 = UnicodeOption(**{'default': u'192.0.2.53', 'doc': u'Adresse IP du conteneur internet', 'multi': False, 'properties': (), 'name': 'container_ip_internet'})
    option_139789497872736 = UnicodeOption(**{'default': u'internet', 'doc': u'Nom du groupe du conteneur internet', 'multi': False, 'properties': (), 'name': 'container_name_internet'})
    option_139789497871728 = SymLinkOption(**{'name': 'adresse_ip_internet', "opt": option_139789497872464})
    option_139789497873008 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs', 'doc': u'Chemin du conteneur bdd', 'multi': False, 'properties': (), 'name': 'container_path_bdd'})
    option_139789497873280 = UnicodeOption(**{'default': u'192.0.2.50', 'doc': u'Adresse IP du conteneur bdd', 'multi': False, 'properties': (), 'name': 'container_ip_bdd'})
    option_139789497873552 = UnicodeOption(**{'default': u'bdd', 'doc': u'Nom du groupe du conteneur bdd', 'multi': False, 'properties': (), 'name': 'container_name_bdd'})
    option_139789497877400 = SymLinkOption(**{'name': 'adresse_ip_bdd', "opt": option_139789497873280})
    option_139789497873824 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs', 'doc': u'Chemin du conteneur reseau', 'multi': False, 'properties': (), 'name': 'container_path_reseau'})
    option_139789497874096 = UnicodeOption(**{'default': u'192.0.2.51', 'doc': u'Adresse IP du conteneur reseau', 'multi': False, 'properties': (), 'name': 'container_ip_reseau'})
    option_139789497874368 = UnicodeOption(**{'default': u'reseau', 'doc': u'Nom du groupe du conteneur reseau', 'multi': False, 'properties': (), 'name': 'container_name_reseau'})
    option_139789497878800 = SymLinkOption(**{'name': 'adresse_ip_reseau', "opt": option_139789497874096})
    option_139789497874640 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs', 'doc': u'Chemin du conteneur partage', 'multi': False, 'properties': (), 'name': 'container_path_partage'})
    option_139789497874912 = UnicodeOption(**{'default': u'192.0.2.52', 'doc': u'Adresse IP du conteneur partage', 'multi': False, 'properties': (), 'name': 'container_ip_partage'})
    option_139789497875184 = UnicodeOption(**{'default': u'partage', 'doc': u'Nom du groupe du conteneur partage', 'multi': False, 'properties': (), 'name': 'container_name_partage'})
    option_139789497880200 = SymLinkOption(**{'name': 'adresse_ip_partage', "opt": option_139789497874912})
    option_139789498700496 = OptionDescription(children=[option_139789498664080, option_139789498664352, option_139789498664624, option_139789498664896, option_139789498665168, option_139789498665440, option_139789498665712, option_139789497844632, option_139789498665984, option_139789498666256, option_139789498666528, option_139789497846032, option_139789497851984, option_139789497852256, option_139789497852528, option_139789497847432, option_139789497852800, option_139789497853072, option_139789497853344, option_139789497853616, option_139789497853888, option_139789497854160, option_139789497858040, option_139789497854432, option_139789497854704, option_139789497854976, option_139789497859440, option_139789497855248, option_139789497855520, option_139789497864272, option_139789497861016, option_139789497864544, option_139789497864816, option_139789497865088, option_139789497865360, option_139789497865632, option_139789497865904, option_139789497863256, option_139789497866176, option_139789497866448, option_139789497866720, option_139789497868928, option_139789497866992, option_139789497867264, option_139789497867536, option_139789497870328, option_139789497867808, option_139789497872464, option_139789497872736, option_139789497871728, option_139789497873008, option_139789497873280, option_139789497873552, option_139789497877400, option_139789497873824, option_139789497874096, option_139789497874368, option_139789497878800, option_139789497874640, option_139789497874912, option_139789497875184, option_139789497880200], **{'doc': u'Information sur les conteneurs', 'properties': ('hidden', 'normal'), 'name': 'containers'})
    option_139789498700496.impl_set_information("icon", u"""puzzle-piece""")
    option_139789497875456 = UnicodeOption(**{'default': u'amonecole', 'doc': u'Nom du module EOLE', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'eole_module'})
    option_139789497875728 = UnicodeOption(**{'multi': False, 'name': 'ubuntu_version', 'doc': u'Nom de version de la distribution Ubuntu', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.get_version, 'callback_params': Params(args=(ParamValue("UBUNTU_VERSION"),), kwargs={})})
    option_139789497876000 = UnicodeOption(**{'multi': False, 'name': 'eole_version', 'doc': u'Num\xe9ro de version majeure du module EOLE', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.get_version, 'callback_params': Params(args=(ParamValue("EOLE_VERSION"),), kwargs={})})
    option_139789497888848 = UnicodeOption(**{'multi': False, 'name': 'eole_release', 'doc': u'Num\xe9ro de version fonctionnelle du module EOLE', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.get_version, 'callback_params': Params(args=(ParamValue("EOLE_RELEASE"),), kwargs={})})
    option_139789497889120 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'mode_conteneur_actif', 'default': 'oui', 'doc': u'Le serveur est en mode conteneur', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497889392 = ChoiceOption(**{'callback_params': None, 'multi': False, 'values': (u'oui', u'non'), 'name': 'module_instancie', 'doc': u'Le serveur a \xe9t\xe9 instanci\xe9', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.is_instanciate})
    option_139789497889664 = UnicodeOption(**{'doc': u"Identifiant de l'\xe9tablissement (exemple UAI)", 'multi': False, 'properties': ('auto_freeze', 'mandatory', 'basic'), 'name': 'numero_etab'})
    option_139789497889664.impl_set_information("separator", (u'\xc9tablissement', None))
    option_139789497889936 = UnicodeOption(**{'doc': u"Nom de l'\xe9tablissement", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'libelle_etab'})
    option_139789497890208 = DomainnameOption(**{'multi': False, 'name': 'nom_machine', 'allow_ip': False, 'type_': 'hostname', 'doc': u'Nom de la machine', 'properties': ('mandatory', 'basic'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497875456, False))})
    option_139789497890208.impl_set_information("separator", ('Nom DNS du serveur', None))
    option_139789497890208.impl_set_information("help", u"""Nom de la machine (exemple : eolebase)""")
    option_139789497890480 = DomainnameOption(**{'type_': 'netbios', 'name': 'smb_netbios_name', 'doc': u'Nom du contr\xf4leur de domaine (ex: monserveur)', 'multi': False, 'warnings_only': True, 'properties': ('auto_freeze', 'mandatory', 'basic')})
    option_139789497890480.impl_set_information("help", u"""Nom DNS du contrôleur de domaine. En mode conteneur, il doit être différent de celui du serveur Linux EOLE.""")
    option_139789497890208.impl_add_consistency("not_equal", option_139789497890480, **{'warnings_only': True})
    option_139789497890752 = DomainnameOption(**{'type_': 'domainname', 'name': 'nom_domaine_local', 'doc': u'Nom DNS du r\xe9seau local', 'allow_ip': True, 'allow_without_dot': True, 'multi': False, 'properties': ('mandatory', 'basic')})
    option_139789497890752.impl_set_information("help", u"""Nom de domaine privé du reseau local""")
    option_139789497891024 = DomainnameOption(**{'multi': False, 'name': 'nom_domaine_machine', 'allow_ip': False, 'type_': 'domainname', 'doc': u"Nom DNS associ\xe9 \xe0 l'interface 0", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.concat, 'callback_params': Params(args=(ParamOption(option_139789497890208, False), ParamValue("."), ParamOption(option_139789497890752, False)), kwargs={})})
    option_139789497891296 = DomainnameOption(**{'type_': 'hostname', 'name': 'nom_academie', 'doc': u'Nom de domaine acad\xe9mique (ex : ac-dijon)', 'allow_ip': False, 'multi': False, 'properties': ('mandatory', 'basic')})
    option_139789497891296.impl_set_information("separator", (u'Param\xe8tres r\xe9seau globaux', None))
    option_139789497891568 = DomainnameOption(**{'type_': 'hostname', 'name': 'suffixe_domaine_academique', 'doc': u'Suffixe du nom de domaine acad\xe9mique', 'default': u'fr', 'allow_ip': False, 'multi': False, 'properties': ('mandatory', 'basic')})
    option_139789497891840 = ChoiceOption(**{'multi': False, 'values': (u'2', u'3', u'4', u'5'), 'name': 'nombre_interfaces', 'default': u'2', 'doc': u"Nombre d'interfaces \xe0 activer", 'properties': ('mandatory', 'normal')})
    option_139789497892112 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_proxy_client', 'default': 'non', 'doc': u'Utiliser un serveur mandataire (proxy) pour acc\xe9der \xe0 Internet', 'properties': ('mandatory', 'basic')})
    option_139789497892112.impl_set_information("help", u"""Le module EOLE doit utiliser un serveur mandataire (proxy) pour accéder à Internet""")
    option_139789497892384 = DomainnameOption(**{'multi': False, 'name': 'proxy_client_adresse', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Nom ou adresse IP du serveur proxy', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497892112}]})
    option_139789497892384.impl_set_information("help", u"""Adresse IP ou nom de domaine du serveur proxy à utiliser par le module EOLE""")
    option_139789497938000 = PortOption(**{'multi': False, 'name': 'proxy_client_port', 'allow_private': True, 'default': '3128', 'doc': u'Port du serveur proxy', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497892112}]})
    option_139789497938000.impl_set_information("help", u"""Port du serveur proxy à utiliser par le module EOLE""")
    option_139789497938272 = ChoiceOption(**{'multi': False, 'values': (u'statique', u'dhcp', u'pppoe'), 'name': 'eth0_method', 'default': u'statique', 'doc': u"M\xe9thode d'attribution de l'adressage pour l'interface", 'properties': ('mandatory', 'basic')})
    option_139789497938272.impl_set_information("separator", ("Configuration de l'interface", None))
    option_139789497938544 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'adresse_ip_dns', 'doc': u'Adresse IP du serveur DNS', 'properties': ('basic',), 'callback': eosfunc.auto_dns, 'callback_params': Params(args=(ParamOption(option_139789497938272, False),), kwargs={})})
    option_139789497938544.impl_set_information("help", u"""Adresse IP du serveur de nom primaire, en général celui de votre fournisseur Internet""")
    option_139789497938816 = UnicodeOption(**{'default': u'Europe/Paris', 'doc': u'Fuseau horaire du serveur', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'time_zone'})
    option_139789497938816.impl_set_information("proposed_value", (u'Africa/Asmara', u'Africa/Cairo', u'Africa/Tripoli', u'America/Adak', u'America/Anchorage', u'America/Argentina/Buenos_Aires', u'America/Argentina/Catamarca', u'America/Argentina/Cordoba', u'America/Argentina/Jujuy', u'America/Argentina/Mendoza', u'America/Cayenne', u'America/Chicago', u'America/Denver', u'America/Detroit', u'America/Edmonton', u'America/Guadeloupe', u'America/Halifax', u'America/Havana', u'America/Indianapolis', u'America/Jamaica', u'America/Los_Angeles', u'America/Manaus', u'America/Martinique', u'America/Mazatlan', u'America/Mexico_City', u'America/Miquelon', u'America/New_York', u'America/Noronha', u'America/Phoenix', u'America/Regina', u'America/Rio_Branco', u'America/Santiago', u'America/Sao_Paulo', u'America/St_Johns', u'America/Tijuana', u'America/Toronto', u'America/Vancouver', u'America/Whitehorse', u'America/Winnipeg', u'Antarctica/McMurdo', u'Asia/Ashgabat', u'Asia/Chongqing', u'Asia/Dhaka', u'Asia/Ho_Chi_Minh', u'Asia/Hong_Kong', u'Asia/Kathmandu', u'Asia/Kolkata', u'Asia/Macau', u'Asia/Riyadh', u'Asia/Seoul', u'Asia/Shanghai', u'Asia/Singapore', u'Asia/Taipei', u'Asia/Tehran', u'Asia/Tel_Aviv', u'Asia/Thimphu', u'Asia/Tokyo', u'Asia/Ulaanbaatar', u'Atlantic/Faroe', u'Atlantic/Reykjavik', u'Australia/Adelaide', u'Australia/Brisbane', u'Australia/Darwin', u'Australia/Hobart', u'Australia/Lord_Howe', u'Australia/Melbourne', u'Australia/Perth', u'Australia/Sydney', u'Europe/Dublin', u'Europe/Helsinki', u'Europe/Istanbul', u'Europe/Lisbon', u'Europe/London', u'Europe/Moscow', u'Europe/Paris', u'Europe/Warsaw', u'Indian/Mayotte', u'Indian/Reunion', u'Pacific/Auckland', u'Pacific/Chatham', u'Pacific/Easter', u'Pacific/Gambier', u'Pacific/Honolulu', u'Pacific/Kwajalein', u'Pacific/Noumea', u'Pacific/Marquesas', u'Pacific/Pago_Pago', u'Pacific/Tahiti', u'Pacific/Wallis'))
    option_139789497939088 = DomainnameOption(**{'multi': True, 'name': 'serveur_ntp', 'allow_without_dot': True, 'default': [u'0.fr.pool.ntp.org', u'1.fr.pool.ntp.org', u'2.fr.pool.ntp.org', u'3.fr.pool.ntp.org'], 'doc': u'Adresse du serveur NTP', 'type_': 'domainname', 'allow_ip': True, 'default_multi': u'0.fr.pool.ntp.org', 'properties': ('mandatory', 'normal')})
    option_139789497939088.impl_set_information("help", u"""Nom ou adresse IP du serveur de temps""")
    option_139789497939360 = ChoiceOption(**{'multi': False, 'values': (u'autosign\xe9', u'letsencrypt', u'manuel'), 'name': 'cert_type', 'default': u'autosign\xe9', 'doc': u'Certificat SSL', 'properties': ('mandatory', 'basic')})
    option_139789497939360.impl_set_information("separator", ('Choix du certificat SSL', None))
    option_139789497939632 = FilenameOption(**{'multi': False, 'name': 'server_cert', 'doc': u'Chemin du fichier contenant le certificat SSL', 'properties': ('mandatory', 'basic'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("manuel"),), kwargs={'default_match': ParamValue("None"), 'condition_1': ParamOption(option_139789497939360, False), 'mismatch': ParamValue("/etc/ssl/certs/eole.crt")}), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'autosign\xe9', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'autosign\xe9', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}, {'expected': u'autosign\xe9', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}]})
    option_139789497939632.impl_set_information("help", u"""Le certificat SSL doit être au format PEM et est généralement fourni dans un fichier avec l'extension \"crt\"""")
    option_139789497939904 = FilenameOption(**{'multi': False, 'name': 'server_key', 'default': u'/etc/ssl/certs/eole.key', 'doc': u'Chemin du fichier contenant la cl\xe9 priv\xe9e du certificat SSL', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'autosign\xe9', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'autosign\xe9', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}, {'expected': u'autosign\xe9', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}]})
    option_139789497939904.impl_set_information("help", u"""La clé privé doit être au format PEM et est généralement fournie dans un fichier avec l'extension \"key\"""")
    option_139789497940176 = FilenameOption(**{'multi': False, 'name': 'server_pem', 'doc': u'Chemin du fichier contenant la cha\xeene de certification', 'properties': ('mandatory', 'basic'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("manuel"),), kwargs={'default_match': ParamValue("None"), 'condition_1': ParamOption(option_139789497939360, False), 'mismatch': ParamValue("/etc/ssl/certs/eole.pem")}), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'autosign\xe9', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'autosign\xe9', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}, {'expected': u'autosign\xe9', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}]})
    option_139789497940176.impl_set_information("help", u"""La chaîne de certification doit être au format PEM et est généralement fournie dans un fichier avec l'extension \"pem\"""")
    option_139789497940448 = DomainnameOption(**{'multi': True, 'name': 'serveur_maj', 'allow_without_dot': True, 'default': [u'eole.ac-dijon.fr', u'ftp.crihan.fr'], 'doc': u'Serveur de mise \xe0 jour', 'type_': 'domainname', 'allow_ip': True, 'default_multi': u'eole.ac-dijon.fr', 'properties': ('mandatory', 'normal')})
    option_139789497940448.impl_set_information("separator", (u'Mise \xe0 jour', None))
    option_139789497940448.impl_set_information("help", u"""Nom ou adresse IP du serveur de mise à jour""")
    option_139789497940720 = DomainnameOption(**{'multi': True, 'name': 'ubuntu_update_mirrors', 'allow_ip': True, 'allow_without_dot': True, 'type_': 'domainname', 'doc': u'Serveur de mise \xe0 jour Ubuntu', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789497940448, False),), kwargs={})})
    option_139789497940720.impl_set_information("help", u"""Nom ou adresse IP du serveur de mise à jour pour les paquets Ubuntu (utilise la valeur de serveur_maj par défaut)""")
    option_139789497940992 = DomainnameOption(**{'multi': True, 'name': 'envole_update_mirrors', 'allow_ip': True, 'allow_without_dot': True, 'type_': 'domainname', 'doc': u'Serveur de mise \xe0 jour Envole', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789497940448, False),), kwargs={})})
    option_139789497941264 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_filtrage_proxy', 'default': 'oui', 'doc': u'Activer le filtrage sur le proxy', 'properties': ('mandatory', 'expert')})
    option_139789497941264.impl_set_information("help", u"""Active le logiciel de filtrage e2guardian""")
    option_139789497941536 = URLOption(**{'multi': False, 'name': 'url_maj_blacklist', 'allow_without_dot': True, 'default': u'http://eole.orion.education.fr/maj/blacklists', 'allow_ip': True, 'doc': u'Adresse web de mise \xe0 jour des blacklists', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789497954384 = UnicodeOption(**{'default': u'eth1', 'doc': u'Interface associ\xe9e \xe0 adresse_ip_fichier_link', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'fichier_link_interface'})
    option_139789498700816 = OptionDescription(children=[option_139789497875456, option_139789497875728, option_139789497876000, option_139789497888848, option_139789497889120, option_139789497889392, option_139789497889664, option_139789497889936, option_139789497890208, option_139789497890752, option_139789497891024, option_139789497891296, option_139789497891568, option_139789497891840, option_139789497892112, option_139789497892384, option_139789497938000, option_139789497938544, option_139789497938816, option_139789497939088, option_139789497939360, option_139789497939632, option_139789497939904, option_139789497940176, option_139789497940448, option_139789497940720, option_139789497940992, option_139789497941536, option_139789497954384], **{'doc': u'G\xe9n\xe9ral', 'properties': ('basic',), 'name': 'general'})
    option_139789498700816.impl_set_information("help", u"""Paramètres de base du serveur""")
    option_139789498700816.impl_set_information("icon", u"""wrench""")
    option_139789497954656 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_log_distant', 'default': 'non', 'doc': u'Activer la gestion des logs centralis\xe9s', 'properties': ('mandatory', 'expert')})
    option_139789497954928 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_apt_cacher', 'default': 'oui', 'doc': u'Activer le cache APT', 'properties': ('mandatory', 'expert')})
    option_139789497954928.impl_set_information("help", u"""Apt-cacher est une solution proxy de mise en cache de paquets Debian""")
    option_139789497955200 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_eoledb', 'default': 'oui', 'doc': u'Activer le service de gestion de bases de donn\xe9es EoleDB', 'properties': ('mandatory', 'expert')})
    option_139789497955200.impl_set_information("help", u"""Utiliser ou non le service de gestion des bases de données EoleDB""")
    option_139789497955472 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_clam', 'default': 'oui', 'doc': u"Activer l'anti-virus ClamAV", 'properties': ('mandatory', 'normal')})
    option_139789497955472.impl_set_information("help", u"""ClamAV est un logiciel anti-virus pour Unix""")
    option_139789497955744 = ChoiceOption(**{'multi': False, 'values': (u'local', u'distant'), 'name': 'activer_client_ldap', 'default': u'local', 'doc': u'Emplacement du serveur LDAP', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497956016 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_dhcp', 'default': 'non', 'doc': u'Activer le serveur DHCP', 'properties': ('mandatory', 'basic')})
    option_139789497956016.impl_set_information("help", u"""DHCP (Dynamic Host Configuration Protocol) est un protocole réseau dont le rôle est d’assurer la configuration automatique des paramètres IP d’une station""")
    option_139789497956288 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_tftp', 'default': 'non', 'doc': u"Activer l'utilisation d'un serveur PXE/TFTP", 'properties': ('mandatory', 'expert')})
    option_139789497956288.impl_set_information("help", u"""TFTP (Trivial File Transfer Protocol) est un protocole simplifié de transfert de fichiers""")
    option_139789497956560 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_dhcprelay', 'default': 'non', 'doc': u'Activer le relai DHCP', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497956560.impl_set_information("help", u"""Le relai DHCP permet de relayer les trames DHCP d'un réseau vers d'autres réseaux""")
    option_139789497956832 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_esu', 'default': 'oui', 'doc': u'Utiliser le logiciel ESU', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497956832.impl_set_information("help", u"""ESU est une application de gestion avancée des postes clients""")
    option_139789497957104 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_nut', 'default': 'non', 'doc': u"Activer la gestion de l'onduleur NUT", 'properties': ('mandatory', 'normal')})
    option_139789497957104.impl_set_information("help", u"""NUT (Network UPS Tools) est un ensemble d'outils permettant de gérer et de superviser un système relié à un ou des onduleurs""")
    option_139789497957376 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_apache', 'default': 'oui', 'doc': u'Activer le serveur web Apache', 'properties': ('mandatory', 'normal')})
    option_139789497957376.impl_set_information("help", u"""Apache un serveur HTTP""")
    option_139789497957648 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'test_activer_nginx_web', 'doc': u'V\xe9rifier s\u2019il est possible d\u2019activer la publication d\u2019applications web par Nginx', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("['non', 'oui']"),), kwargs={'mismatch': ParamValue("oui"), 'condition_1': ParamOption(option_139789497889120, False), 'match': ParamValue("non"), 'condition_2': ParamOption(option_139789497957376, False)})})
    option_139789497957648.impl_set_information("help", u"""Vérifie que le mode conteneur est actif ou qu’apache est désactivé.""")
    option_139789497957920 = ChoiceOption(**{'multi': False, 'name': 'activer_nginx_web', 'doc': u'Activer la publication d\u2019applications web par Nginx', 'values': (u'oui', u'non'), 'properties': ('normal',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497957648, False)), "requires": [{'expected': u'non', 'action': 'frozen', 'inverse': False, "option": option_139789497957648}, {'expected': u'non', 'action': 'hidden', 'inverse': False, "option": option_139789497957648}, {'expected': u'non', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497957648}]})
    option_139789497957920.impl_set_information("help", u"""Publier les applications web par Nginx""")
    option_139789497962576 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'test_activer_ead3', 'doc': u'V\xe9rifier si l\u2019on peut activer l\u2019interface d\u2019administration du module (EAD3)', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.activate_master_only_web_app, 'callback_params': Params(args=None, kwargs={'activer_nginx_web': ParamOption(option_139789497957920, True), 'activer_apache': ParamOption(option_139789497957376, True), 'mode_conteneur_actif': ParamOption(option_139789497889120, False)})})
    option_139789497962576.impl_set_information("help", u"""Vérifier que la publication web par Nginx ou Apache est actif""")
    option_139789497962848 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'nginx_degrade_http', 'default': 'non', 'doc': u'D\xe9grader la s\xe9curit\xe9 en HTTP', 'properties': ('mandatory', 'expert')})
    option_139789497962848.impl_set_information("help", u"""Désactiver l’utilisation du HTTPS. Certaines applications comme gen_config sont désactivées dans ce cas.""")
    option_139789497963120 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'test_activer_ead3_https', 'doc': u'V\xe9rifier si HTTPS Nginx est d\xe9sactiv\xe9 pour l\u2019interface d\u2019administration du module (EAD3)', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("non"), 'condition_1': ParamOption(option_139789497962848, True), 'match': ParamValue("oui")})})
    option_139789497963392 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_ead3', 'default': 'non', 'doc': u'Activer l\u2019interface d\u2019administration du module (EAD3)', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497962576}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963120}]})
    option_139789497963392.impl_set_information("help", u"""Publier l’interface d’administration du module sur le port standard HTTPS (443)""")
    option_139789497963664 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_saltstack', 'doc': u'Activer SaltStack', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_val_first_value, 'callback_params': Params(args=(ParamOption(option_139789497963392, True), ParamValue("non")), kwargs={})})
    option_139789497963664.impl_set_information("help", u"""SaltStack est un logiciel de gestion de configuration écrit en Python""")
    option_139789497963936 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'install_rvp', 'default': 'non', 'doc': u'Activer le r\xe9seau virtuel priv\xe9 RVP', 'properties': ('mandatory', 'normal')})
    option_139789497963936.impl_set_information("help", u"""Le RVP utilise strongSwan qui est une implémentation libre et complète de VPN IPsec""")
    option_139789497964208 = ChoiceOption(**{'multi': False, 'values': (u'non', u'local', u'distant'), 'name': 'activer_sso', 'default': u'local', 'doc': u'Utiliser un serveur EoleSSO', 'properties': ('mandatory', 'normal')})
    option_139789497964208.impl_set_information("help", u"""EoleSSO est un serveur d'authentification développé pour répondre à la problématique du SSO (authentification unique)""")
    option_139789497964480 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_ead_web', 'default': 'oui', 'doc': u"Activer l'interface web de l'EAD", 'properties': ('mandatory', 'expert')})
    option_139789497964480.impl_set_information("help", u"""L'EAD est une interface simplifiée de gestion du serveur""")
    option_139789497964752 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'frontend_ead_distant_eth4', 'default': 'non', 'doc': u"Autoriser l'acc\xe8s au backend EAD depuis des frontend EAD distants", 'properties': ('mandatory', 'expert')})
    option_139789497964752.impl_set_information("separator", (u"Configuration de l'acc\xe8s au backend EAD", None))
    option_139789497964752.impl_set_information("help", u"""Ouverture des ports 4201 et 4202 depuis l'interface 4""")
    option_139789497965024 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'frontend_ead_distant_eth1', 'default': 'non', 'doc': u"Autoriser l'acc\xe8s au backend EAD depuis des frontend EAD distants", 'properties': ('mandatory', 'expert')})
    option_139789497965024.impl_set_information("separator", (u"Configuration de l'acc\xe8s au backend EAD", None))
    option_139789497965024.impl_set_information("help", u"""Ouverture des ports 4201 et 4202 depuis l'interface 1""")
    option_139789497965296 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'frontend_ead_distant_eth0', 'default': 'non', 'doc': u"Autoriser l'acc\xe8s au backend EAD depuis des frontend EAD distants", 'properties': ('mandatory', 'expert')})
    option_139789497965296.impl_set_information("separator", (u"Configuration de l'acc\xe8s au backend EAD", None))
    option_139789497965296.impl_set_information("help", u"""Ouverture des ports 4201 et 4202 depuis l'interface 0""")
    option_139789497965568 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'frontend_ead_distant_eth3', 'default': 'non', 'doc': u"Autoriser l'acc\xe8s au backend EAD depuis des frontend EAD distants", 'properties': ('mandatory', 'expert')})
    option_139789497965568.impl_set_information("separator", (u"Configuration de l'acc\xe8s au backend EAD", None))
    option_139789497965568.impl_set_information("help", u"""Ouverture des ports 4201 et 4202 depuis l'interface 3""")
    option_139789497965840 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'frontend_ead_distant_eth2', 'default': 'non', 'doc': u"Autoriser l'acc\xe8s au backend EAD depuis des frontend EAD distants", 'properties': ('mandatory', 'expert')})
    option_139789497965840.impl_set_information("separator", (u"Configuration de l'acc\xe8s au backend EAD", None))
    option_139789497965840.impl_set_information("help", u"""Ouverture des ports 4201 et 4202 depuis l'interface 2""")
    option_139789497966112 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_firewall', 'default': 'oui', 'doc': u'Activer le support du firewall', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789497970768 = ChoiceOption(**{'multi': False, 'name': 'acces_distant_backend_ead', 'doc': u"Autoriser l'utilisation du backend EAD par des frontend EAD distants", 'values': (u'oui', u'non'), 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("oui"), 'condition_4': ParamOption(option_139789497964752, True), 'condition_1': ParamOption(option_139789497965024, True), 'condition_0': ParamOption(option_139789497965296, False), 'condition_3': ParamOption(option_139789497965568, True), 'condition_2': ParamOption(option_139789497965840, True), 'match': ParamValue("non")}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497970768.impl_set_information("help", u"""Ouverture des ports 4201 et 4202 pour permettre à un frontend EAD distant d'utiliser le backend EAD de ce serveur""")
    option_139789497971312 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_mysql', 'default': 'oui', 'doc': u'Activer le serveur de bases de donn\xe9es MySQL', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497971312.impl_set_information("help", u"""MySQL est un serveur de bases de données relationnelles SQL""")
    option_139789497971584 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_bareos_dir', 'default': 'oui', 'doc': u'Activer la sauvegarde du serveur', 'properties': ('mandatory', 'normal')})
    option_139789497971040 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_bareos_sd', 'doc': u'Activer le support de stockage de la sauvegarde', 'properties': ('normal',), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789497971584, False),), kwargs={})})
    option_139789497971856 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_cups', 'default': 'oui', 'doc': u"Activer le serveur d'impression CUPS", 'properties': ('mandatory', 'normal')})
    option_139789497971856.impl_set_information("help", u"""Le Common Unix Printing System (CUPS) est un système modulaire d'impression informatique pour les systèmes d'exploitation Unix et assimilés""")
    option_139789497972128 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_wpad', 'default': 'oui', 'doc': u'Activer le support de WPAD', 'properties': ('mandatory', 'normal')})
    option_139789497972128.impl_set_information("help", u"""WPAD est un protocole qui permet la découverte automatique du proxy par les navigateurs""")
    option_139789497972400 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_revprox', 'doc': u'Activer le reverse proxy Nginx', 'properties': ('normal',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497957376, False))})
    option_139789497972400.impl_set_information("help", u"""Le Reverse Proxy permet de relayer des accès extérieurs vers des serveurs situés derrière le pare-feu""")
    option_139789497972672 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'test_hide_nginx_family', 'doc': u'V\xe9rifier s\u2019il faut cacher la famille Nginx', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("non"), 'condition_1': ParamOption(option_139789497957920, True), 'match': ParamValue("oui"), 'condition_2': ParamOption(option_139789497972400, False)})})
    option_139789497972672.impl_set_information("help", u"""Vérifie que la publication d’application web par Nginx et le reverse proxy sont désactivés.""")
    option_139789497972944 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_proftpd', 'default': 'oui', 'doc': u"Activer l'acc\xe8s FTP", 'properties': ('mandatory', 'normal')})
    option_139789497972944.impl_set_information("help", u"""ProFTPD est un serveur FTP""")
    option_139789497973216 = UnicodeOption(**{'default': u'/var/lib/eole/ead3files', 'doc': u'Chemin de t\xe9l\xe9versement des fichiers EAD3', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ead3_upload_path'})
    option_139789497973216.impl_set_information("help", u"""Chemin absolu (commençant par un /) vers lequel sont téléversés les fichiers provenant de l'action \"Gestion de fichier\"""")
    option_139789497973488 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'test_activer_genconfig', 'doc': u'V\xe9rifier si l\u2019on peut activer l\u2019interface de configuration du module (GenConfig)', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.activate_master_only_web_app, 'callback_params': Params(args=None, kwargs={'activer_nginx_web': ParamOption(option_139789497957920, True), 'activer_apache': ParamOption(option_139789497957376, True), 'mode_conteneur_actif': ParamOption(option_139789497889120, False)})})
    option_139789497973488.impl_set_information("help", u"""Vérifier que la publication web par Nginx ou Apache est active""")
    option_139789497973760 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'test_activer_genconfig_https', 'doc': u'V\xe9rifier si HTTPS Nginx est d\xe9sactiv\xe9 pour l\u2019interface de configuration du module (GenConfig)', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("non"), 'condition_1': ParamOption(option_139789497962848, True), 'match': ParamValue("oui")})})
    option_139789497974032 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_genconfig', 'default': 'oui', 'doc': u'Activer l\u2019interface de configuration du module (GenConfig)', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497973488}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497973760}]})
    option_139789497974032.impl_set_information("help", u"""Publier l’interface de configuration du module sur le port standard HTTPS (443)""")
    option_139789498701136 = OptionDescription(children=[option_139789497954656, option_139789497954928, option_139789497955200, option_139789497955472, option_139789497955744, option_139789497956016, option_139789497956288, option_139789497956560, option_139789497956832, option_139789497957104, option_139789497963664, option_139789497963936, option_139789497957376, option_139789497964208, option_139789497964480, option_139789497970768, option_139789497971312, option_139789497971584, option_139789497971040, option_139789497941264, option_139789497971856, option_139789497972128, option_139789497957648, option_139789497972672, option_139789497957920, option_139789497972400, option_139789497972944, option_139789497962576, option_139789497963120, option_139789497963392, option_139789497973216, option_139789497973488, option_139789497973760, option_139789497974032], **{'doc': u'Services', 'properties': ('basic',), 'name': 'services'})
    option_139789498701136.impl_set_information("help", u"""Paramètres des services fournis par le serveur""")
    option_139789498701136.impl_set_information("icon", u"""cogs""")
    option_139789497974304 = UnicodeOption(**{'multi': False, 'name': 'type_amon_concat', 'doc': u'Mod\xe8le de filtrage', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.concat, 'callback_params': Params(args=(ParamOption(option_139789497891840, False), ParamValue("zones")), kwargs={})})
    option_139789497983056 = UnicodeOption(**{'multi': False, 'name': 'type_amon', 'doc': u'Mod\xe8le de filtrage', 'properties': ('basic',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("amon"),), kwargs={'mismatch': ParamValue("2zones-amonecole"), 'condition_1': ParamOption(option_139789497875456, False), 'match': ParamOption(option_139789497974304, False)})})
    option_139789497983056.impl_set_information("proposed_value", (u'2zones', u'2zones-amonecole', u'3zones', u'3zones-dmz', u'4zones', u'5zones'))
    option_139789497983056.impl_set_information("help", u"""Modèle de pare-feu Era à appliquer sur le serveur""")
    option_139789497983328 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'install_scribe_dmz', 'default': 'non', 'doc': u"Activer la gestion d'un Scribe dans la DMZ", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497983872 = DomainnameOption(**{'type_': 'hostname', 'name': 'nom_serveur_scribe_dmz', 'doc': u'Nom du Scribe dans la DMZ (pour enregistrement DNS)', 'allow_ip': True, 'multi': False, 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497983328}]})
    option_139789497983600 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'ip_serveur_scribe_dmz', 'doc': u'Adresse IP du Scribe dans la DMZ', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497983328}]})
    option_139789498701456 = OptionDescription(children=[option_139789497983056, option_139789497974304, option_139789497983328, option_139789497983872, option_139789497983600], **{'doc': u'Firewall', 'properties': ('basic',), 'name': 'firewall'})
    option_139789498701456.impl_set_information("help", u"""Configuration du pare-feu""")
    option_139789498701456.impl_set_information("icon", u"""fire""")
    option_139789497984144 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_bash_completion', 'default': 'oui', 'doc': u"Activer l'auto-compl\xe9tion \xe9tendue sur la console (touche TAB)", 'properties': ('mandatory', 'expert')})
    option_139789497984144.impl_set_information("separator", ('Console', None))
    option_139789497984416 = IntOption(**{'default': 900, 'doc': u"Temps d'inactivit\xe9 avant d\xe9connexion bash (0 pour d\xe9sactiver)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'bash_tmout'})
    option_139789497984416.impl_set_information("help", u"""Durée en secondes""")
    option_139789497984688 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_ctrl_alt_suppr', 'default': 'oui', 'doc': u'Activer le reboot sur ctrl-alt-suppr', 'properties': ('mandatory', 'expert')})
    option_139789497984960 = ChoiceOption(**{'multi': False, 'values': (u'dark', u'light'), 'name': 'vim_background', 'default': u'dark', 'doc': u"Nuance du fond d'\xe9cran dans VIM", 'properties': ('mandatory', 'expert')})
    option_139789497984960.impl_set_information("help", u"""Indication de la valeur de la couleur d’arrière-plan de l’éditeur VIM pour améliorer la lisibilité de la coloration syntaxique (dark et light pour une coloration adaptée à un arrière-plan sombre et clair respectivement)""")
    option_139789497985504 = IntOption(**{'multi': False, 'name': 'vm_swappiness', 'default': 60, 'doc': u"Poids relatif de l'utilisation de la swap par rapport \xe0 la m\xe9moire vive", 'properties': ('mandatory', 'expert'), "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"0"), 'maxi': ParamValue(u"100")})})
    option_139789497985504.impl_set_information("separator", (u'Optimisations syst\xe8me', None))
    option_139789497985504.impl_set_information("help", u"""Le swappiness est un paramètre du noyau Linux permettant de définir avec quelle sensibilité il va écrire dans le swap si la quantité de RAM à utiliser devient trop importante""")
    option_139789497985232 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_rngd', 'default': 'non', 'doc': u'Activer le service de g\xe9n\xe9ration de nombres al\xe9atoires rng-tools', 'properties': ('mandatory', 'expert')})
    option_139789497985232.impl_set_information("help", u"""Le démon rngd agit comme une passerelle entre un vrai générateur de nombres aléatoires, matériel (TRNG), tel que ceux que l'on peut trouver dans les puces Intel/AMD/VIA et le pseudo-générateur de nombres aléatoires du noyau (PRNG)""")
    option_139789497985776 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'check_passwd', 'default': 'oui', 'doc': u'V\xe9rifier la complexit\xe9 des mots de passe', 'properties': ('mandatory', 'expert')})
    option_139789497985776.impl_set_information("separator", (u'Validation des mots de passe des utilisateurs syst\xe8me (root, eole, ...)', None))
    option_139789497986048 = IntOption(**{'multi': False, 'name': 'check_passwd_min_len_one_type', 'default': 0, 'doc': u'Taille minimum du mot de passe utilisant une seule classe de caract\xe8res', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497985776}]})
    option_139789497986048.impl_set_information("help", u"""Taille minimum du mot de passe compris seulement dans une classe, 0 pour interdire (les classes de caractères sont : chiffres, lettres minuscules, lettres majuscules, et autres caractères)""")
    option_139789497986320 = IntOption(**{'multi': False, 'name': 'check_passwd_min_len_two_type', 'default': 9, 'doc': u'Taille minimum du mot de passe utilisant deux classes de caract\xe8res', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497985776}]})
    option_139789497986320.impl_set_information("help", u"""Taille minimum du mot de passe compris seulement dans deux classes, 0 pour interdire (les classes de caractères sont : chiffres, lettres minuscules, lettres majuscules, et autres caractères)""")
    option_139789497986592 = IntOption(**{'multi': False, 'name': 'check_passwd_min_len_three_type', 'default': 8, 'doc': u'Taille minimum du mot de passe utilisant trois classes de caract\xe8res', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497985776}]})
    option_139789497986592.impl_set_information("help", u"""Taille minimum du mot de passe compris seulement dans trois classes, 0 pour interdire (les classes de caractères sont : chiffres, lettres minuscules, lettres majuscules, et autres caractères)""")
    option_139789497999440 = IntOption(**{'multi': False, 'name': 'check_passwd_min_len_four_type', 'default': 8, 'doc': u'Taille minimum du mot de passe utilisant quatre classes de caract\xe8res', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497985776}]})
    option_139789497999440.impl_set_information("help", u"""Taille minimum du mot de passe compris seulement dans quatre classes, 0 pour interdire (les classes de caractères sont : chiffres, lettres minuscules, lettres majuscules, et autres caractères)""")
    option_139789497999712 = IntOption(**{'multi': False, 'name': 'check_passwd_max', 'default': 40, 'doc': u'Taille maximale du mot de passe ', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497985776}]})
    option_139789497999984 = UnicodeOption(**{'callback_params': None, 'multi': True, 'name': 'cdrom_devices', 'doc': u'Liste des lecteurs de CDROM pr\xe9sent sur le serveur', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.list_cdrom_devices})
    option_139789498000256 = IntOption(**{'multi': True, 'name': 'cdrom_minor', 'doc': u'Minor du device CDROM pr\xe9sent sur le serveur', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.cdrom_minormajor, 'callback_params': Params(args=(ParamValue("minor"), ParamOption(option_139789497999984, False)), kwargs={})})
    option_139789498000528 = IntOption(**{'multi': True, 'name': 'cdrom_major', 'doc': u'Major du device CDROM pr\xe9sent sur le serveur', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.cdrom_minormajor, 'callback_params': Params(args=(ParamValue("major"), ParamOption(option_139789497999984, False)), kwargs={})})
    option_139789498701776 = Leadership(children=[option_139789497999984, option_139789498000256, option_139789498000528], **{'doc': u'Liste des lecteurs de CDROM pr\xe9sent sur le serveur', 'properties': ('expert',), 'name': 'cdrom_devices'})
    option_139789498000800 = IntOption(**{'callback_params': None, 'multi': False, 'name': 'partitioning_free_PE', 'doc': u'Nombre de blocs disponibles pour une nouvelle partition', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.calc_free_PE})
    option_139789498000800.impl_set_information("separator", ('Ajustement du partitionnement', None))
    option_139789498001072 = ChoiceOption(**{'callback_params': None, 'multi': False, 'values': (u'oui', u'non'), 'name': 'allow_manual_partitioning', 'doc': u'Permettre le partitionnement manuel', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.enable_lv_creation})
    option_139789498001072.impl_set_information("help", u"""Active automatiquement les champs de saisie si le serveur n’est pas encore instancié et que le groupe de volumes dispose de PE libres""")
    option_139789498001344 = UnicodeOption(**{'callback_params': None, 'multi': True, 'name': 'eole_created_lv_names', 'doc': u'Liste des noms de volume logique actuellement cr\xe9\xe9s sur le module', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.get_lv_names})
    option_139789498001344.impl_set_information("help", u"""Cette liste est utilisée pour calculer partitioning_lv_name et test_extend_standard_partitioning""")
    option_139789498001888 = UnicodeOption(**{'multi': True, 'name': 'eole_lv_names', 'doc': u'Liste des noms de volume logique du partitionnement EOLE standard', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.calc_multi_val, 'callback_params': Params(args=(ParamValue("root"), ParamValue("tmp"), ParamValue("var"), ParamValue("opt"), ParamValue("home")), kwargs={})})
    option_139789498001888.impl_set_information("help", u"""Cette liste est utilisée pour calculer test_extend_standard_partitioning""")
    option_139789498001616 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'test_extend_standard_partitioning', 'doc': u"Tester s'il faut \xe9tendre automatiquement le partitionnement EOLE", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.check_partitioning_auto_extend, 'callback_params': Params(args=None, kwargs={'created_lv_names': ParamOption(option_139789498001344, False), 'expected_lv_names': ParamOption(option_139789498001888, False)})})
    option_139789498001616.impl_set_information("help", u"""Vérifier que le partitionnement est compatible EOLE afin de pouvoir être étendu automatiquement""")
    option_139789498002160 = ChoiceOption(**{'multi': False, 'name': 'use_eole_standard_volume_extension', 'doc': u'Utiliser le mod\xe8le d\u2019extension standard EOLE', 'values': (u'oui', u'non'), 'properties': ('expert',), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789498001616, False),), kwargs={}), "requires": [{'expected': u'non', 'action': 'frozen', 'inverse': False, "option": option_139789498001616}, {'expected': u'non', 'action': 'hidden', 'inverse': False, "option": option_139789498001616}, {'expected': u'non', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789498001616}]})
    option_139789498002160.impl_set_information("help", u"""L’administrateur souhaite utiliser le modèle de répartition de l’espace standard d’EOLE""")
    option_139789498002432 = IntOption(**{'multi': True, 'name': 'eole_lv_standard_extends', 'doc': u'Liste des pourcentages d\u2019espace libre \xe0 attribuer aux volumes logiques', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.calc_multi_val, 'callback_params': Params(args=(ParamValue(0), ParamValue(0), ParamValue(0), ParamValue(0), ParamValue(100)), kwargs={})})
    option_139789498002432.impl_set_information("help", u"""Cette liste fournit le pourcentage d’espace libre à attribuer à chaque volume lors d’un partitionnement EOLE standard""")
    option_139789498002704 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'partitioning_create_lv', 'default': 'non', 'doc': u'Ajuster le partitionnement', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789498002160}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498001072}]})
    option_139789498002704.impl_set_information("help", u"""Activer les champs de saisie pour la création de volumes logiques ou l’extension de volumes logiques existants""")
    option_139789498002976 = UnicodeOption(**{'multi': True, 'name': 'partitioning_lv_name', 'doc': u'Nom du volume \xe0 cr\xe9er', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789498001344, False),), kwargs={}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498002704}]})
    option_139789498002976.impl_set_information("help", u"""Nom du volume logique LVM à créer dans le groupe de volume par défaut""")
    option_139789498015824 = IntOption(**{'multi': True, 'name': 'partitioning_lv_size', 'doc': u'Taille du volume en pourcentage de l\u2019espace disponible', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamValue(0),), kwargs={}), "validator": eosfunc.check_free_space})
    option_139789498015824.impl_set_information("help", u"""Taille du volume logique en pourcentage de l’espace disponible""")
    option_139789498016096 = UnicodeOption(**{'multi': True, 'name': 'partitioning_lv_format', 'default_multi': u'ext4', 'doc': u'Format du syst\xe8me de fichiers', 'properties': ('mandatory', 'expert'), "validator": eosfunc.is_fs_type})
    option_139789498016096.impl_set_information("help", u"""Format de la partition""")
    option_139789498016368 = FilenameOption(**{'doc': u'Point de montage du volume logique', 'multi': True, 'properties': ('expert',), 'name': 'partitioning_lv_mount_point'})
    option_139789498016368.impl_set_information("help", u"""Point de montage du volume logique""")
    option_139789498016640 = UnicodeOption(**{'doc': u'Options du montage', 'multi': True, 'properties': ('expert',), 'name': 'partitioning_lv_mount_options'})
    option_139789498016640.impl_set_information("help", u"""Options du montage""")
    option_139789498702096 = Leadership(children=[option_139789498002976, option_139789498015824, option_139789498016096, option_139789498016368, option_139789498016640], **{'doc': u'Nom du volume \xe0 cr\xe9er', 'properties': ('expert',), 'name': 'partitioning_lv_name'})
    option_139789498016912 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'partitioning_lv_extend', 'default': 'non', 'doc': u'Allouer l\u2019espace restant', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789498002160}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498001072}]})
    option_139789498016912.impl_set_information("help", u"""Allouer l’espace restant à un volume logique existant""")
    option_139789498017184 = FilenameOption(**{'multi': False, 'name': 'partitioning_lv_to_extend', 'doc': u'Volume logique \xe0 \xe9tendre', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498016912}], "validator": eosfunc.is_lv_name})
    option_139789498017184.impl_set_information("help", u"""Nom du volume logique à étendre avec l’espace restant""")
    option_139789498702416 = OptionDescription(children=[option_139789497984144, option_139789497984416, option_139789497984688, option_139789497984960, option_139789497985504, option_139789497985232, option_139789497985776, option_139789497986048, option_139789497986320, option_139789497986592, option_139789497999440, option_139789497999712, option_139789498701776, option_139789498000800, option_139789498001072, option_139789498001616, option_139789498002160, option_139789498001888, option_139789498002432, option_139789498001344, option_139789498002704, option_139789498702096, option_139789498016912, option_139789498017184], **{'doc': u'Syst\xe8me', 'properties': ('expert',), 'name': 'systeme'})
    option_139789498702416.impl_set_information("help", u"""Configuration système avancée""")
    option_139789498702416.impl_set_information("icon", u"""desktop""")
    option_139789498017456 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ssh_permit_root', 'default': 'oui', 'doc': u"Autoriser les connexions SSH pour l'utilisateur root", 'properties': ('mandatory', 'expert')})
    option_139789498017456.impl_set_information("separator", ('Configuration SSH', None))
    option_139789498017728 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ssh_allow_passwd', 'default': 'oui', 'doc': u'Autoriser les connexions SSH par mot de passe (si non clef RSA obligatoire)', 'properties': ('mandatory', 'expert')})
    option_139789498018000 = UsernameOption(**{'doc': u'Autoriser les connexions SSH pour les groupes', 'multi': True, 'properties': ('expert',), 'name': 'ssh_allow_groups'})
    option_139789498018272 = UnicodeOption(**{'default': u'5:30:10', 'doc': u'Crit\xe8res \xe0 appliquer pour le blocage des tentatives de connexions par force brute', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ssh_maxstartups'})
    option_139789498018272.impl_set_information("help", u"""La première valeur est le nombre de tentatives de connexions échouées tolérées. La dernière valeur est le nombre de connexions échouées maximales. La deuxième valeur fixe le pourcentage de refus aléatoire une fois le nombre de connexions tolérées atteint. Ce pourcentage de refus augmente à chaque échec supplémentaire jusqu'à ce que le nombre de connexions maximales soit atteint.""")
    option_139789498702736 = OptionDescription(children=[option_139789498017456, option_139789498017728, option_139789498018000, option_139789498018272], **{'doc': u'SSHd', 'properties': ('expert',), 'name': 'sshd'})
    option_139789498702736.impl_set_information("help", u"""Configuration des connexions SSH""")
    option_139789498702736.impl_set_information("icon", u"""shell""")
    option_139789498018544 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_reception_logs', 'default': 'non', 'doc': u'Activer la r\xe9ception des logs de machines distantes', 'properties': ('mandatory', 'normal')})
    option_139789498018544.impl_set_information("separator", (u'R\xe9ception', None))
    option_139789498018816 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_reception_logs_relp', 'default': 'non', 'doc': u'Activer la r\xe9ception des logs de machines distantes via le protocole RELP (fiable, non compatible TLS)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498018544}]})
    option_139789498019088 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'adresses_ip_clients_logs_relp', 'doc': u'IP des clients \xe0 autoriser pour le protocole RELP', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498018816}]})
    option_139789498019360 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_client_logs_relp', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498018816}]})
    option_139789498703056 = Leadership(children=[option_139789498019088, option_139789498019360], **{'doc': u'IP des clients \xe0 autoriser pour le protocole RELP', 'properties': ('basic',), 'name': 'adresses_ip_clients_logs_relp'})
    option_139789498032208 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_reception_logs_udp', 'default': 'non', 'doc': u'Activer la r\xe9ception des logs de machines distantes via le protocole UDP', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498018544}]})
    option_139789498032480 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'adresses_ip_clients_logs_udp', 'doc': u'Adresse IP \xe9quipement autoris\xe9', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498032208}]})
    option_139789498032752 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_client_logs_udp', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498032208}]})
    option_139789498703376 = Leadership(children=[option_139789498032480, option_139789498032752], **{'doc': u'Adresse IP \xe9quipement autoris\xe9', 'properties': ('basic',), 'name': 'adresses_ip_clients_logs_udp'})
    option_139789498033024 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_reception_logs_tcp', 'default': 'non', 'doc': u'Activer la r\xe9ception des logs de machines distantes via le protocole TCP (compatible TLS)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498018544}]})
    option_139789498033296 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'rsyslog_reception_tls', 'default': 'non', 'doc': u'Activer le chiffrement des transferts pour la r\xe9ception (TLS)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498033024}]})
    option_139789498033568 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'adresses_ip_clients_logs_tcp', 'doc': u'IP des clients \xe0 autoriser pour le protocole TCP', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498033024}]})
    option_139789498033840 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_client_logs_tcp', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498033024}]})
    option_139789498044496 = Leadership(children=[option_139789498033568, option_139789498033840], **{'doc': u'IP des clients \xe0 autoriser pour le protocole TCP', 'properties': ('basic',), 'name': 'adresses_ip_clients_logs_tcp'})
    option_139789498034112 = FilenameOption(**{'default': u'/etc/rsyslog.d/ssl/certs/zephirlog_ca.pem', 'doc': u'Emplacement de la CA', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'rsyslog_ca_file'})
    option_139789498034384 = FilenameOption(**{'default': u'/etc/rsyslog.d/ssl/private/rsyslog_privkey.pem', 'doc': u'Emplacement de la cl\xe9', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'rsyslog_privkey_file'})
    option_139789498034928 = FilenameOption(**{'default': u'/etc/rsyslog.d/ssl/certs/rsyslog_cert.pem', 'doc': u'Emplacement du certificat', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'rsyslog_cert_file'})
    option_139789498034656 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_envoi_logs', 'default': 'non', 'doc': u"Activer l'envoi des logs \xe0 une machine distante (TCP si TLS activ\xe9, RELP sinon)", 'properties': ('mandatory', 'normal')})
    option_139789498034656.impl_set_information("separator", ('Envoi', None))
    option_139789498035200 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_serveur_logs', 'doc': u'Adresse IP du serveur de log central', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498034656}]})
    option_139789498035472 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'rsyslog_envoi_tls', 'default': 'non', 'doc': u"Activer le chiffrement des transferts pour l'envoi (TLS)", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498034656}]})
    option_139789498035744 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'envoyer_tous_logs', 'default': 'oui', 'doc': u'Envoyer tous les journaux', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498034656}]})
    option_139789498035744.impl_set_information("separator", (u'Choix des journaux \xe0 envoyer', None))
    option_139789497528400 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'utiliser_rsyslog_plage_envoi_globale', 'default': 'non', 'doc': u'Utiliser une plage temporelle pour le transfert des logs', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789498035744}]})
    option_139789497528672 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2', u'3', u'4', u'5', u'6', u'7', u'8', u'9', u'10', u'11', u'12', u'13', u'14', u'15', u'16', u'17', u'18', u'19', u'20', u'21', u'22', u'23', u'24'), 'name': 'rsyslog_plage_globale_heure_debut', 'default': u'1', 'doc': u'Heure de d\xe9but du transfert des logs', 'properties': ('normal',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497528400}]})
    option_139789497528944 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2', u'3', u'4', u'5', u'6', u'7', u'8', u'9', u'10', u'11', u'12', u'13', u'14', u'15', u'16', u'17', u'18', u'19', u'20', u'21', u'22', u'23', u'24'), 'name': 'rsyslog_plage_globale_heure_fin', 'default': u'1', 'doc': u'Heure de fin du transfert des logs', 'properties': ('normal',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497528400}]})
    option_139789497529216 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'envoyer_logs_squid', 'default': 'non', 'doc': u'Activer le transfert des logs de Squid', 'properties': ('mandatory', 'normal')})
    option_139789497529216.impl_set_information("separator", ('Squid', None))
    option_139789497529488 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activate_squid_realtime', 'default': 'non', 'doc': u'Activer le transfert des logs de Squid en temps r\xe9el', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497529216}]})
    option_139789497529760 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2', u'3', u'4', u'5', u'6', u'7', u'8', u'9', u'10', u'11', u'12', u'13', u'14', u'15', u'16', u'17', u'18', u'19', u'20', u'21', u'22', u'23', u'24'), 'name': 'squid_heure_debut', 'default': u'1', 'doc': u'Heure de d\xe9but du transfert des logs', 'properties': ('normal',), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497529488}]})
    option_139789497530032 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2', u'3', u'4', u'5', u'6', u'7', u'8', u'9', u'10', u'11', u'12', u'13', u'14', u'15', u'16', u'17', u'18', u'19', u'20', u'21', u'22', u'23', u'24'), 'name': 'squid_heure_fin', 'default': u'1', 'doc': u'Heure de fin du transfert des logs', 'properties': ('normal',), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497529488}]})
    option_139789497530304 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'envoyer_logs_dansguardian', 'default': 'non', 'doc': u'Activer le transfert des logs de e2guardian', 'properties': ('mandatory', 'normal')})
    option_139789497530304.impl_set_information("separator", ('e2guardian', None))
    option_139789497530576 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activate_dansguardian_realtime', 'default': 'non', 'doc': u'Activer le transfert des logs de e2guardian en temps r\xe9el', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497530304}]})
    option_139789497530848 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2', u'3', u'4', u'5', u'6', u'7', u'8', u'9', u'10', u'11', u'12', u'13', u'14', u'15', u'16', u'17', u'18', u'19', u'20', u'21', u'22', u'23', u'24'), 'name': 'dansguardian_heure_debut', 'default': u'1', 'doc': u'Heure de d\xe9but du transfert des logs', 'properties': ('normal',), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497530576}]})
    option_139789497531120 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2', u'3', u'4', u'5', u'6', u'7', u'8', u'9', u'10', u'11', u'12', u'13', u'14', u'15', u'16', u'17', u'18', u'19', u'20', u'21', u'22', u'23', u'24'), 'name': 'dansguardian_heure_fin', 'default': u'1', 'doc': u'Heure de fin du transfert des logs', 'properties': ('normal',), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497530576}]})
    option_139789498044816 = OptionDescription(children=[option_139789498018544, option_139789498018816, option_139789498703056, option_139789498032208, option_139789498703376, option_139789498033024, option_139789498033296, option_139789498044496, option_139789498034112, option_139789498034384, option_139789498034928, option_139789498034656, option_139789498035200, option_139789498035472, option_139789498035744, option_139789497528400, option_139789497528672, option_139789497528944, option_139789497529216, option_139789497529488, option_139789497529760, option_139789497530032, option_139789497530304, option_139789497530576, option_139789497530848, option_139789497531120], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497954656}], **{'doc': u'Logs', 'properties': ('basic',), 'name': 'logs'})
    option_139789498044816.impl_set_information("icon", u"""edit""")
    option_139789497531392 = UsernameOption(**{'doc': u'Nom du compte PPPoE', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'pppoe_user', "requires": [{'expected': u'pppoe', 'action': 'disabled', 'inverse': True, "option": option_139789497938272}]})
    option_139789497531664 = PasswordOption(**{'doc': u'Mot de passe PPPoE', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'pppoe_passwd', "requires": [{'expected': u'pppoe', 'action': 'disabled', 'inverse': True, "option": option_139789497938272}]})
    option_139789497531936 = UnicodeOption(**{'multi': True, 'name': 'nom_carte_eth0', 'doc': u"Nom de l'interface r\xe9seau", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_net_device, 'callback_params': Params(args=(ParamValue(0),), kwargs={})})
    option_139789497531936.impl_set_information("proposed_value", (u'enp0s3',))
    option_139789497536592 = UnicodeOption(**{'multi': True, 'name': 'nom_carte_eth1', 'doc': u"Nom de l'interface r\xe9seau", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_net_device, 'callback_params': Params(args=(ParamValue(1),), kwargs={})})
    option_139789497536592.impl_set_information("proposed_value", (u'enp0s3',))
    option_139789497536864 = UnicodeOption(**{'multi': True, 'name': 'nom_carte_eth2', 'doc': u"Nom de l'interface r\xe9seau", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_net_device, 'callback_params': Params(args=(ParamValue(2),), kwargs={})})
    option_139789497536864.impl_set_information("proposed_value", (u'enp0s3',))
    option_139789497537136 = UnicodeOption(**{'multi': True, 'name': 'nom_carte_eth3', 'doc': u"Nom de l'interface r\xe9seau", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_net_device, 'callback_params': Params(args=(ParamValue(3),), kwargs={})})
    option_139789497537136.impl_set_information("proposed_value", (u'enp0s3',))
    option_139789497537408 = UnicodeOption(**{'multi': True, 'name': 'nom_carte_eth4', 'doc': u"Nom de l'interface r\xe9seau", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_net_device, 'callback_params': Params(args=(ParamValue(4),), kwargs={})})
    option_139789497537408.impl_set_information("proposed_value", (u'enp0s3',))
    option_139789497537136.impl_add_consistency("not_equal", option_139789497537408, **{'transitive': False, 'warnings_only': True})
    option_139789497536864.impl_add_consistency("not_equal", option_139789497537136, **{'transitive': False, 'warnings_only': True})
    option_139789497536864.impl_add_consistency("not_equal", option_139789497537408, **{'transitive': False, 'warnings_only': True})
    option_139789497536592.impl_add_consistency("not_equal", option_139789497536864, **{'transitive': False, 'warnings_only': True})
    option_139789497536592.impl_add_consistency("not_equal", option_139789497537136, **{'transitive': False, 'warnings_only': True})
    option_139789497536592.impl_add_consistency("not_equal", option_139789497537408, **{'transitive': False, 'warnings_only': True})
    option_139789497531936.impl_add_consistency("not_equal", option_139789497536592, **{'transitive': False, 'warnings_only': True})
    option_139789497531936.impl_add_consistency("not_equal", option_139789497536864, **{'transitive': False, 'warnings_only': True})
    option_139789497531936.impl_add_consistency("not_equal", option_139789497537136, **{'transitive': False, 'warnings_only': True})
    option_139789497531936.impl_add_consistency("not_equal", option_139789497537408, **{'transitive': False, 'warnings_only': True})
    option_139789497537680 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_bonding_eth0', 'doc': u'Param\xe9trer cette zone pour du bonding', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.list_len_gt, 'callback_params': Params(args=None, kwargs={'max_len': ParamValue("1"), 'mismatch': ParamValue("non"), 'match': ParamValue("oui"), 'param': ParamOption(option_139789497531936, False)})})
    option_139789497537952 = ChoiceOption(**{'multi': False, 'name': 'zone_is_bridge_eth0', 'doc': u"L'interface r\xe9seau de la zone est un bridge", 'values': (u'oui', u'non'), 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497537680}]})
    option_139789497538224 = UnicodeOption(**{'multi': False, 'name': 'nom_zone_eth0', 'doc': u"Nom de l'interface r\xe9seau de la zone", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_zone_name, 'callback_params': Params(args=None, kwargs={'nom_carte': ParamOption(option_139789497531936, False), 'interface_no': ParamValue("0"), 'method': ParamOption(option_139789497938272, False), 'zone_is_bridge': ParamOption(option_139789497537952, True)})})
    option_139789497538496 = UnicodeOption(**{'multi': False, 'name': 'pppoe_interface', 'doc': u'Interface o\xf9 PPPoE est connect\xe9', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497538224, False)), "requires": [{'expected': u'pppoe', 'action': 'disabled', 'inverse': True, "option": option_139789497938272}]})
    option_139789497538768 = IPOption(**{'multi': False, 'name': 'adresse_ip_eth0', 'doc': u'Adresse IP de la carte', 'warnings_only': True, 'properties': ('mandatory', 'basic'), 'private_only': True, 'callback': eosfunc.auto_eth, 'callback_params': Params(args=(ParamOption(option_139789497538224, False),), kwargs={'parametre': ParamOption(option_139789497938272, False), 'condition': ParamValue("dhcp")}), "requires": [{'expected': u'statique', 'action': 'frozen', 'inverse': True, "option": option_139789497938272}, {'expected': u'statique', 'action': 'hidden', 'inverse': True, "option": option_139789497938272}, {'expected': u'statique', 'action': 'force_default_on_freeze', 'inverse': True, "option": option_139789497938272}]})
    option_139789497538768.impl_set_information("help", u"""Adresse IP de la première carte réseau (ex: 10.21.5.1)""")
    option_139789497539040 = NetmaskOption(**{'multi': False, 'name': 'adresse_netmask_eth0', 'doc': u'Masque de sous r\xe9seau de la carte', 'properties': ('mandatory', 'basic'), 'callback': eosfunc.auto_netmask, 'callback_params': Params(args=(ParamOption(option_139789497538224, False),), kwargs={'parametre': ParamOption(option_139789497938272, False), 'condition': ParamValue("dhcp")}), "requires": [{'expected': u'statique', 'action': 'frozen', 'inverse': True, "option": option_139789497938272}, {'expected': u'statique', 'action': 'hidden', 'inverse': True, "option": option_139789497938272}, {'expected': u'statique', 'action': 'force_default_on_freeze', 'inverse': True, "option": option_139789497938272}]})
    option_139789497539040.impl_set_information("help", u"""Adresse du masque de sous réseau de la première carte réseau (ex: 255.255.255.0)""")
    option_139789497539040.impl_add_consistency("ip_netmask", option_139789497538768, **{'warnings_only': True})
    option_139789497539312 = NetworkOption(**{'multi': False, 'name': 'adresse_network_eth0', 'doc': u'Adresse r\xe9seau de la carte', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_or_auto_network, 'callback_params': Params(args=None, kwargs={'nom_carte': ParamOption(option_139789497538224, False), 'parametre': ParamOption(option_139789497938272, False), 'netmask': ParamOption(option_139789497539040, False), 'condition': ParamValue("dhcp"), 'ip': ParamOption(option_139789497538768, False)})})
    option_139789497539312.impl_set_information("help", u"""'Adresse réseau de la carte de la première carte réseau (ex: 10.21.5.0)""")
    option_139789497539584 = BroadcastOption(**{'multi': False, 'name': 'adresse_broadcast_eth0', 'doc': u'Adresse broadcast de sous r\xe9seau de la carte', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_or_auto_broadcast, 'callback_params': Params(args=None, kwargs={'nom_carte': ParamOption(option_139789497538224, False), 'parametre': ParamOption(option_139789497938272, False), 'netmask': ParamOption(option_139789497539040, False), 'condition': ParamValue("dhcp"), 'ip': ParamOption(option_139789497538768, False)})})
    option_139789497539584.impl_set_information("help", u"""Adresse de broadcast (diffusion) de la carte de la première carte réseau (ex: 10.21.5.255)""")
    option_139789497539856 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_gw', 'doc': u'Adresse IP de la passerelle par d\xe9faut', 'properties': ('basic',), 'callback': eosfunc.auto_defaultgw_ip, 'callback_params': Params(args=(ParamOption(option_139789497938272, False), ParamOption(option_139789497538224, False)), kwargs={}), "requires": [{'expected': u'statique', 'action': 'frozen', 'inverse': True, "option": option_139789497938272}, {'expected': u'statique', 'action': 'hidden', 'inverse': True, "option": option_139789497938272}, {'expected': u'statique', 'action': 'force_default_on_freeze', 'inverse': True, "option": option_139789497938272}]})
    option_139789497539856.impl_add_consistency("in_network", option_139789497539312, option_139789497539040, **{'warnings_only': True})
    option_139789497539856.impl_add_consistency("not_equal", option_139789497538768, **{})
    option_139789497540128 = UnicodeOption(**{'multi': False, 'name': 'interface_gw', 'doc': u'Interface de sortie', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789497538224, False),), kwargs={})})
    option_139789497557072 = ChoiceOption(**{'multi': False, 'values': (u'balance-rr', u'active-backup', u'balance-xor', u'broadcast', u'802.3ad', u'balance-tlb', u'balance-alb'), 'name': 'bonding_mode_eth0', 'default': u'balance-rr', 'doc': u'Mode de bonding', 'properties': ('expert',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497537680}]})
    option_139789497557072.impl_set_information("help", u"""Mode de fonctionnement du bond""")
    option_139789497557344 = IntOption(**{'multi': False, 'name': 'bonding_miimon_eth0', 'default': 100, 'doc': u'Fr\xe9quence des MII link monitoring en millisecondes', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497537680}]})
    option_139789497557344.impl_set_information("help", u"""Intervalle de temps pour le monitoring des interfaces""")
    option_139789497557616 = IntOption(**{'multi': False, 'name': 'bonding_downdelay_eth0', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e down", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497537680}]})
    option_139789497557888 = IntOption(**{'multi': False, 'name': 'bonding_updelay_eth0', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e comme active", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497537680}]})
    option_139789497558160 = ChoiceOption(**{'multi': False, 'values': (u'', u'speed 100 duplex full autoneg off', u'autoneg on', u'speed 10 duplex half autoneg off', u'speed 1000 duplex full autoneg off'), 'name': 'debit_carte_eth0', 'default': u'', 'doc': u"Mode de connexion pour l'interface", 'properties': ('expert',)})
    option_139789497558432 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ssh_eth0', 'default': 'oui', 'doc': u'Autoriser les connexions SSH', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497558432.impl_set_information("separator", ("Administration distante sur l'interface", True))
    option_139789497558704 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_ssh_eth0', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497558432}]})
    option_139789497558976 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_ssh_eth0', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497558432}]})
    option_139789497558976.impl_add_consistency("network_netmask", option_139789497558704, **{})
    option_139789498045456 = Leadership(children=[option_139789497558704, option_139789497558976], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'properties': ('basic',), 'name': 'ip_ssh_eth0'})
    option_139789497559248 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'admin_eth0', 'default': 'oui', 'doc': u'Autoriser les connexions pour administrer le serveur (EAD, phpMyAdmin, ...)', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497559520 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_admin_eth0', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497559248}]})
    option_139789497559792 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_admin_eth0', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497559248}]})
    option_139789497559792.impl_add_consistency("network_netmask", option_139789497559520, **{})
    option_139789498045136 = Leadership(children=[option_139789497559520, option_139789497559792], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'properties': ('basic',), 'name': 'ip_admin_eth0'})
    option_139789497560064 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'alias_eth0', 'default': 'non', 'doc': u"Ajouter des IP alias sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497560064.impl_set_information("separator", ("Configuration des alias sur l'interface", None))
    option_139789497560336 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'alias_ip_eth0', 'doc': u"Adresse IP alias pour l'interface 0", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497560064}]})
    option_139789497560608 = NetmaskOption(**{'multi': True, 'name': 'alias_netmask_eth0', 'doc': u'Masque de sous r\xe9seau correspondant \xe0 cet alias', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497539040, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497560064}]})
    option_139789497573456 = NetworkOption(**{'multi': True, 'name': 'alias_network_eth0', 'doc': u'Adresse r\xe9seau correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497560336, False), 'netmask': ParamOption(option_139789497560608, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497560064}]})
    option_139789497573728 = BroadcastOption(**{'multi': True, 'name': 'alias_broadcast_eth0', 'doc': u'Adresse broadcast correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497560336, False), 'netmask': ParamOption(option_139789497560608, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497560064}]})
    option_139789497574000 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_agregation', 'default': 'non', 'doc': u'R\xe9partition de charge entre 2 lignes Internet', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497560064}]})
    option_139789497574000.impl_set_information("separator", (u'Agr\xe9gation de liens', None))
    option_139789497574000.impl_set_information("help", u"""L'agrégation de liens permet la mise en place d'une répartition de charge ou d'une haute disponibilité pour les sorties Internet""")
    option_139789497574272 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'alias_gw_eth0', 'doc': u"Adresse de la passerelle pour cet alias (n\xe9cessaire \xe0 l'agr\xe9gation de liens)", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497574000}]})
    option_139789497574544 = ChoiceOption(**{'multi': False, 'values': (u'NTLM/SMB', u'Ldap', u'Ldap (Active Directory)', u'NTLM/KERBEROS', u'Fichier local'), 'name': 'type_squid_auth', 'default': u'NTLM/SMB', 'doc': u"Type d'authentification", 'properties': ('mandatory', 'normal')})
    option_139789497574816 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_cntlm', 'default': 'non', 'doc': u'Activer le proxy NTLM', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'NTLM/SMB', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}, {'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789497574816.impl_set_information("help", u"""CNTLM est un proxy permettant de s'authentifier en NTLM / NTLM Session Response / NTLMv2. L'authentification se fera sur le premier domaine spécifié.""")
    option_139789497575088 = PortOption(**{'multi': False, 'name': 'cntlm_port', 'allow_private': True, 'default': '3127', 'doc': u"Port d'\xe9coute du proxy NTLM", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497574816}]})
    option_139789497575360 = ChoiceOption(**{'multi': False, 'name': 'activer_cntlm_eth0', 'doc': u'Activer le proxy NTLM sur cette interface', 'values': (u'oui', u'non'), 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("1"),), kwargs={'condition_1': ParamOption(option_139789497891840, False)}), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497574816}]})
    option_139789497575632 = PortOption(**{'multi': False, 'name': 'wpad_proxyport_eth0', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD', 'properties': ('disabled', 'mandatory', 'expert'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamOption(option_139789497575088, True), 'condition_1': ParamOption(option_139789497575360, True), 'match': ParamValue("3128")})})
    option_139789497575632.impl_set_information("separator", (u'Configuration de la d\xe9tection automatique du proxy', None))
    option_139789497575904 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_alias_eth0', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour cet alias', 'properties': ('disabled', 'mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497575632, False))})
    option_139789498045776 = Leadership(children=[option_139789497560336, option_139789497560608, option_139789497573456, option_139789497573728, option_139789497574272, option_139789497575904], **{'doc': u"Adresse IP alias pour l'interface 0", 'properties': ('basic',), 'name': 'alias_ip_eth0'})
    option_139789497576176 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'vlan_eth0', 'default': 'non', 'doc': u"Activer le support des VLAN sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497576176.impl_set_information("separator", ("Configuration des VLAN sur l'interface", None))
    option_139789497576448 = IntOption(**{'doc': u"Num\xe9ro d'identifiant du VLAN", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'vlan_id_eth0', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497576176}]})
    option_139789497576720 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'vlan_ip_eth0', 'doc': u"Adresse IP de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497576176}]})
    option_139789497576992 = NetmaskOption(**{'multi': True, 'name': 'vlan_netmask_eth0', 'default_multi': u'255.255.255.0', 'doc': u"Masque de sous r\xe9seau de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497576176}]})
    option_139789497593936 = NetworkOption(**{'multi': True, 'name': 'vlan_network_eth0', 'doc': u"Adresse r\xe9seau de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497576720, False), 'netmask': ParamOption(option_139789497576992, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497576176}]})
    option_139789497594208 = BroadcastOption(**{'multi': True, 'name': 'vlan_broadcast_eth0', 'doc': u"Adresse de broadcast de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497576720, False), 'netmask': ParamOption(option_139789497576992, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497576176}]})
    option_139789497594480 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'vlan_gw_eth0', 'doc': u'Adresse de la passerelle pour ce VLAN', 'properties': ('normal',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497576176}]})
    option_139789497594752 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_vlan_eth0', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour ce VLAN', 'properties': ('disabled', 'mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497575632, False))})
    option_139789498046096 = Leadership(children=[option_139789497576448, option_139789497576720, option_139789497576992, option_139789497593936, option_139789497594208, option_139789497594480, option_139789497594752], **{'doc': u"Num\xe9ro d'identifiant du VLAN", 'properties': ('basic',), 'name': 'vlan_id_eth0'})
    option_139789497595024 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_bonding_eth4', 'doc': u'Param\xe9trer cette zone pour du bonding', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.list_len_gt, 'callback_params': Params(args=None, kwargs={'max_len': ParamValue("1"), 'mismatch': ParamValue("non"), 'match': ParamValue("oui"), 'param': ParamOption(option_139789497537408, False)})})
    option_139789497595296 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_bonding_eth3', 'doc': u'Param\xe9trer cette zone pour du bonding', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.list_len_gt, 'callback_params': Params(args=None, kwargs={'max_len': ParamValue("1"), 'mismatch': ParamValue("non"), 'match': ParamValue("oui"), 'param': ParamOption(option_139789497537136, False)})})
    option_139789497595568 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_bonding_eth2', 'doc': u'Param\xe9trer cette zone pour du bonding', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.list_len_gt, 'callback_params': Params(args=None, kwargs={'max_len': ParamValue("1"), 'mismatch': ParamValue("non"), 'match': ParamValue("oui"), 'param': ParamOption(option_139789497536864, False)})})
    option_139789497595840 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_bonding_eth1', 'doc': u'Param\xe9trer cette zone pour du bonding', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.list_len_gt, 'callback_params': Params(args=None, kwargs={'max_len': ParamValue("1"), 'mismatch': ParamValue("non"), 'match': ParamValue("oui"), 'param': ParamOption(option_139789497536592, False)})})
    option_139789497596112 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'bonding_is_active', 'doc': u'Au moins une interface est de type bonding', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("oui"),), kwargs={'mismatch': ParamValue("non"), 'condition_5': ParamOption(option_139789497595024, True), 'condition_4': ParamOption(option_139789497595296, True), 'condition_1': ParamOption(option_139789497537680, True), 'condition_3': ParamOption(option_139789497595568, True), 'condition_2': ParamOption(option_139789497595840, True), 'operator': ParamValue("OR"), 'match': ParamValue("oui")})})
    option_139789497596384 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_dns_eth0', 'default': 'non', 'doc': u'Activer le serveur DNS sur cette zone', 'properties': ('mandatory', 'expert')})
    option_139789497596384.impl_set_information("separator", ("Configuration DNS sur l'interface", None))
    option_139789497596656 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_master_eth0', 'default': 'non', 'doc': u'Serveur master DNS sur cette zone', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497596384}]})
    option_139789497596928 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2'), 'name': 'dansguardian_eth0', 'default': u'1', 'doc': u'Filtre web \xe0 appliquer sur cette interface', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789497596928.impl_set_information("help", u"""Instance e2guardian à utiliser pour l'interface 0 (1=Configuration 1, 2=Configuration 2)""")
    option_139789497597200 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_eth0_proxy_link', 'doc': u'Adresse IP pour le proxy', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497597200.impl_set_information("separator", ('Adresse pour le proxy', None))
    option_139789497597200.impl_set_information("help", u"""Adresse IP supplémentaire sur cette interface réseau, utilisée pour accèder au proxy. Le serveur aura donc plusieurs adresses IP sur le réseau local.""")
    option_139789497597200.impl_add_consistency("in_network", option_139789497539312, option_139789497539040, **{'transitive': False, 'warnings_only': True})
    option_139789497597200.impl_add_consistency("not_equal", option_139789497538768, **{'transitive': False})
    option_139789497597472 = NetmaskOption(**{'multi': False, 'name': 'adresse_mask_eth0_proxy_link', 'doc': u'Masque de sous r\xe9seau du proxy', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497539040, False)), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497602128 = BroadcastOption(**{'multi': False, 'name': 'adresse_bcast_eth0_proxy_link', 'doc': u'Broadcast pour le proxy', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497597200, False), 'netmask': ParamOption(option_139789497597472, False)}), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497602400 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_supp_proxy_eth0', 'default': 'non', 'doc': u'Autoriser des r\xe9seaux suppl\xe9mentaires \xe0 se connecter sur le proxy', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789497602400.impl_set_information("separator", (u'R\xe9seau suppl\xe9mentaire autoris\xe9 sur le proxy', None))
    option_139789497602672 = NetworkOption(**{'doc': u'R\xe9seaux suppl\xe9mentaires autoris\xe9s \xe0 se connecter sur le proxy', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'proxy_eth0_ip', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497602400}]})
    option_139789497602944 = NetmaskOption(**{'doc': u'Masque de sous r\xe9seau', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'proxy_eth0_netmask', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497602400}]})
    option_139789497602944.impl_add_consistency("network_netmask", option_139789497602672, **{})
    option_139789498046416 = Leadership(children=[option_139789497602672, option_139789497602944], **{'doc': u'R\xe9seaux suppl\xe9mentaires autoris\xe9s \xe0 se connecter sur le proxy', 'properties': ('basic',), 'name': 'proxy_eth0_ip'})
    option_139789497603216 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_frontend_ead_distant_eth0', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497965296}]})
    option_139789497603488 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau autoris\xe9 \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_frontend_ead_distant_eth0', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497965296}]})
    option_139789497603488.impl_add_consistency("network_netmask", option_139789497603216, **{})
    option_139789498046736 = Leadership(children=[option_139789497603216, option_139789497603488], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'properties': ('basic',), 'name': 'ip_frontend_ead_distant_eth0'})
    option_139789498047056 = OptionDescription(children=[option_139789497938272, option_139789497531392, option_139789497531664, option_139789497538496, option_139789497538768, option_139789497539040, option_139789497539312, option_139789497539584, option_139789497539856, option_139789497540128, option_139789497531936, option_139789497538224, option_139789497537680, option_139789497557072, option_139789497557344, option_139789497557616, option_139789497557888, option_139789497537952, option_139789497558160, option_139789497558432, option_139789498045456, option_139789497559248, option_139789498045136, option_139789497560064, option_139789498045776, option_139789497576176, option_139789498046096, option_139789497596112, option_139789497574000, option_139789497596384, option_139789497596656, option_139789497596928, option_139789497597200, option_139789497597472, option_139789497602128, option_139789497602400, option_139789498046416, option_139789497965296, option_139789498046736, option_139789497575632, option_139789497575360], **{'doc': u'Interface-0', 'properties': ('basic',), 'name': 'interface_0'})
    option_139789498047056.impl_set_information("help", u"""Configuration de l'interface 0""")
    option_139789498047056.impl_set_information("icon", u"""sitemap""")
    option_139789497603760 = ChoiceOption(**{'multi': False, 'values': (u'statique',), 'name': 'eth1_method', 'default': u'statique', 'doc': u"M\xe9thode d'attribution de l'adressage pour l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497603760.impl_set_information("separator", ("Configuration de l'interface", None))
    option_139789497604032 = IPOption(**{'multi': False, 'name': 'adresse_ip_eth1', 'doc': u'Adresse IP de la carte', 'properties': ('mandatory', 'basic'), 'private_only': True, 'warnings_only': True})
    option_139789497604304 = NetmaskOption(**{'default': u'255.255.255.0', 'doc': u'Masque de sous r\xe9seau de la carte', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'adresse_netmask_eth1'})
    option_139789497604304.impl_add_consistency("ip_netmask", option_139789497604032, **{'warnings_only': True})
    option_139789497604576 = NetworkOption(**{'multi': False, 'name': 'adresse_network_eth1', 'doc': u'Adresse r\xe9seau de la carte', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497604032, False), 'netmask': ParamOption(option_139789497604304, False)})})
    option_139789497604848 = BroadcastOption(**{'multi': False, 'name': 'adresse_broadcast_eth1', 'doc': u'Adresse broadcast de sous r\xe9seau de la carte', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497604032, False), 'netmask': ParamOption(option_139789497604304, False)})})
    option_139789497605120 = ChoiceOption(**{'multi': False, 'name': 'zone_is_bridge_eth1', 'doc': u"L'interface r\xe9seau de la zone est un bridge", 'values': (u'oui', u'non'), 'properties': ('mandatory', 'expert'), "default": "non", "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497595840}]})
    option_139789497605392 = UnicodeOption(**{'multi': False, 'name': 'nom_zone_eth1', 'doc': u"Nom de l'interface r\xe9seau de la zone", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_zone_name, 'callback_params': Params(args=None, kwargs={'nom_carte': ParamOption(option_139789497536592, False), 'interface_no': ParamValue("1"), 'method': ParamOption(option_139789497603760, False), 'zone_is_bridge': ParamOption(option_139789497605120, True)})})
    option_139789497605664 = ChoiceOption(**{'multi': False, 'values': (u'balance-rr', u'active-backup', u'balance-xor', u'broadcast', u'802.3ad', u'balance-tlb', u'balance-alb'), 'name': 'bonding_mode_eth1', 'default': u'balance-rr', 'doc': u'Mode de bonding', 'properties': ('expert',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595840}]})
    option_139789497605664.impl_set_information("help", u"""Mode de fonctionnement du bond""")
    option_139789497634896 = IntOption(**{'multi': False, 'name': 'bonding_miimon_eth1', 'default': 100, 'doc': u'Fr\xe9quence des MII link monitoring en millisecondes', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595840}]})
    option_139789497634896.impl_set_information("help", u"""Intervalle de temps pour le monitoring des interfaces""")
    option_139789497635168 = IntOption(**{'multi': False, 'name': 'bonding_downdelay_eth1', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e down", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595840}]})
    option_139789497635440 = IntOption(**{'multi': False, 'name': 'bonding_updelay_eth1', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e comme active", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595840}]})
    option_139789497635712 = ChoiceOption(**{'multi': False, 'values': (u'', u'speed 100 duplex full autoneg off', u'autoneg on', u'speed 10 duplex half autoneg off', u'speed 1000 duplex full autoneg off'), 'name': 'debit_carte_eth1', 'default': u'', 'doc': u"Mode de connexion pour l'interface interne 1", 'properties': ('expert',)})
    option_139789497635984 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ssh_eth1', 'default': 'oui', 'doc': u'Autoriser les connexions SSH', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497635984.impl_set_information("separator", ("Administration distante sur l'interface", None))
    option_139789497636256 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_ssh_eth1', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497635984}]})
    option_139789497636528 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_ssh_eth1', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497635984}]})
    option_139789497636528.impl_add_consistency("network_netmask", option_139789497636256, **{})
    option_139789498047376 = Leadership(children=[option_139789497636256, option_139789497636528], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'properties': ('basic',), 'name': 'ip_ssh_eth1'})
    option_139789497636800 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'admin_eth1', 'default': 'oui', 'doc': u'Autoriser les connexions pour administrer le serveur (EAD, phpMyAdmin, ...)', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497637072 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_admin_eth1', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497636800}]})
    option_139789497637344 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_admin_eth1', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497636800}]})
    option_139789497637344.impl_add_consistency("network_netmask", option_139789497637072, **{})
    option_139789498047696 = Leadership(children=[option_139789497637072, option_139789497637344], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'properties': ('basic',), 'name': 'ip_admin_eth1'})
    option_139789497637616 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'alias_eth1', 'default': 'non', 'doc': u"Ajouter des IP alias sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497637616.impl_set_information("separator", ("Configuration des alias sur l'interface", None))
    option_139789497637888 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'alias_ip_eth1', 'doc': u"Adresse IP alias pour l'interface interne 1", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497637616}]})
    option_139789497638160 = NetmaskOption(**{'multi': True, 'name': 'alias_netmask_eth1', 'doc': u'Masque de sous r\xe9seau correspondant \xe0 cet alias', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497604304, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497637616}]})
    option_139789497638432 = NetworkOption(**{'multi': True, 'name': 'alias_network_eth1', 'doc': u'Adresse r\xe9seau correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497637888, False), 'netmask': ParamOption(option_139789497638160, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497637616}]})
    option_139789497655376 = BroadcastOption(**{'multi': True, 'name': 'alias_broadcast_eth1', 'doc': u'Adresse broadcast correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497637888, False), 'netmask': ParamOption(option_139789497638160, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497637616}]})
    option_139789497655648 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'agriates_member', 'default': 'non', 'doc': u'Serveur membre du r\xe9seau AGRIATES', 'properties': ('mandatory', 'normal')})
    option_139789497655648.impl_set_information("separator", ('AGRIATES', None))
    option_139789497655920 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_rvp_alias_eth1', 'default_multi': 'oui', 'doc': u'Autoriser cet alias \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497637616}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789497656192 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_forward_alias_eth1', 'default_multi': 'oui', 'doc': u'Autoriser cet alias \xe0 utiliser les DNS de zones forward additionnelles', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497637616}]})
    option_139789497656464 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_squid_auth', 'default': 'oui', 'doc': u"Activer l'authentification web (proxy)", 'properties': ('mandatory', 'normal')})
    option_139789497656736 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'squid_auth_eth1', 'default': 'oui', 'doc': u"Activer l'authentification sur cette interface", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497656464}]})
    option_139789497656736.impl_set_information("help", u"""L'authentification sur cette interface s'applique aussi aux VLAN et alias""")
    option_139789497657008 = ChoiceOption(**{'multi': False, 'name': 'activer_cntlm_eth1', 'doc': u'Activer le proxy NTLM sur cette interface', 'values': (u'oui', u'non'), 'properties': ('expert',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497574816, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497574816}, {'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497656736}]})
    option_139789497657280 = PortOption(**{'multi': False, 'name': 'wpad_proxyport_eth1', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD', 'properties': ('mandatory', 'expert'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamOption(option_139789497575088, True), 'condition_1': ParamOption(option_139789497657008, True), 'match': ParamValue("3128")}), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497657280.impl_set_information("separator", (u'Configuration de la d\xe9tection automatique du proxy', None))
    option_139789497657552 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_alias_eth1', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour cet alias', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497657280, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789498048016 = Leadership(children=[option_139789497637888, option_139789497638160, option_139789497638432, option_139789497655376, option_139789497655920, option_139789497656192, option_139789497657552], **{'doc': u"Adresse IP alias pour l'interface interne 1", 'properties': ('basic',), 'name': 'alias_ip_eth1'})
    option_139789497657824 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'vlan_eth1', 'default': 'non', 'doc': u"Activer le support des VLAN sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497657824.impl_set_information("separator", ("Configuration des VLAN sur l'interface", None))
    option_139789497658096 = UnicodeOption(**{'doc': u"Num\xe9ro d'identifiant du VLAN", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'vlan_id_eth1', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497657824}]})
    option_139789497658368 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'vlan_ip_eth1', 'doc': u"Adresse IP de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497657824}]})
    option_139789497658640 = NetmaskOption(**{'multi': True, 'name': 'vlan_netmask_eth1', 'default_multi': u'255.255.255.0', 'doc': u"Masque de sous r\xe9seau de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497657824}]})
    option_139789497658912 = NetworkOption(**{'multi': True, 'name': 'vlan_network_eth1', 'doc': u"Adresse r\xe9seau de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497658368, False), 'netmask': ParamOption(option_139789497658640, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497657824}]})
    option_139789497671760 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'vlan_broadcast_eth1', 'doc': u"Adresse de broadcast de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497658368, False), 'netmask': ParamOption(option_139789497658640, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497657824}]})
    option_139789497672032 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_rvp_vlan_eth1', 'default_multi': 'oui', 'doc': u'Autoriser ce VLAN \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497657824}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}]})
    option_139789497672304 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_forward_vlan_eth1', 'default_multi': 'oui', 'doc': u'Autoriser ce VLAN \xe0 utiliser les DNS des zones forward additionnelles', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497657824}]})
    option_139789497672576 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_vlan_eth1', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour ce VLAN', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497657280, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497675856 = Leadership(children=[option_139789497658096, option_139789497658368, option_139789497658640, option_139789497658912, option_139789497671760, option_139789497672032, option_139789497672304, option_139789497672576], **{'doc': u"Num\xe9ro d'identifiant du VLAN", 'properties': ('basic',), 'name': 'vlan_id_eth1'})
    option_139789497672848 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_master_eth1', 'default': 'oui', 'doc': u'Serveur master DNS de cette zone', 'properties': ('mandatory', 'normal')})
    option_139789497672848.impl_set_information("separator", ("Configuration DNS sur l'interface", None))
    option_139789497673120 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_rvp_eth1', 'default': 'oui', 'doc': u'Autoriser cette zone \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789497673392 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_forward_eth1', 'default': 'oui', 'doc': u'Autoriser cette zone \xe0 utiliser les DNS des zones forward additionnelles', 'properties': ('mandatory', 'normal')})
    option_139789497673664 = DomainnameOption(**{'multi': False, 'name': 'nom_machine_eth1', 'allow_ip': False, 'type_': 'hostname', 'doc': u'Nom de la zone (pour r\xe9solution DNS)', 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("2"),), kwargs={'mismatch': ParamValue("admin"), 'condition_1': ParamOption(option_139789497891840, False), 'match': ParamValue("pedago")})})
    option_139789497673936 = DomainnameOption(**{'multi': False, 'name': 'nom_machine_eth2', 'allow_ip': False, 'type_': 'hostname', 'doc': u'Nom de la zone (pour r\xe9solution DNS)', 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("2"),), kwargs={'mismatch': ParamValue("pedago"), 'condition_1': ParamOption(option_139789497891840, False), 'match': ParamValue("notset")})})
    option_139789497674208 = DomainnameOption(**{'type_': 'hostname', 'name': 'nom_machine_eth3', 'doc': u'Nom de la zone (pour r\xe9solution DNS)', 'default': u'dmz-priv', 'allow_ip': False, 'multi': False, 'properties': ('mandatory', 'normal')})
    option_139789497674480 = DomainnameOption(**{'type_': 'hostname', 'name': 'nom_machine_eth4', 'doc': u'Nom de la zone (pour r\xe9solution DNS)', 'default': u'dmz-pub', 'allow_ip': False, 'multi': False, 'properties': ('mandatory', 'normal')})
    option_139789497674208.impl_add_consistency("not_equal", option_139789497674480, **{'transitive': False})
    option_139789497673936.impl_add_consistency("not_equal", option_139789497674208, **{'transitive': False})
    option_139789497673664.impl_add_consistency("not_equal", option_139789497673936, **{'transitive': False})
    option_139789497674752 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2'), 'name': 'dansguardian_eth1', 'default': u'1', 'doc': u'Filtre web \xe0 appliquer sur cette interface', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789497674752.impl_set_information("separator", ('Configuration de la politique de filtrage', None))
    option_139789497674752.impl_set_information("help", u"""Instance e2guardian à utiliser pour l'interface 1 (1=Configuration 1, 2=Configuration 2)""")
    option_139789497675024 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_eth1_proxy_link', 'doc': u'Adresse IP pour le proxy', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497675024.impl_set_information("separator", ('Adresse pour le proxy', None))
    option_139789497675024.impl_set_information("help", u"""Adresse IP supplémentaire sur cette interface réseau, utilisée pour accèder au proxy. Le serveur aura donc plusieurs adresses IP sur le réseau local.""")
    option_139789497675024.impl_add_consistency("in_network", option_139789497604576, option_139789497604304, **{'transitive': False, 'warnings_only': True})
    option_139789497675024.impl_add_consistency("not_equal", option_139789497604032, **{'transitive': False})
    option_139789497675296 = NetmaskOption(**{'multi': False, 'name': 'adresse_mask_eth1_proxy_link', 'doc': u'Masque de sous r\xe9seau du proxy', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497604304, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497692240 = BroadcastOption(**{'multi': False, 'name': 'adresse_bcast_eth1_proxy_link', 'doc': u'Broadcast pour le proxy', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497675024, False), 'netmask': ParamOption(option_139789497675296, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497692512 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_frontend_ead_distant_eth1', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497965024}]})
    option_139789497692784 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau autoris\xe9 \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_frontend_ead_distant_eth1', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497965024}]})
    option_139789497692784.impl_add_consistency("network_netmask", option_139789497692512, **{})
    option_139789497676176 = Leadership(children=[option_139789497692512, option_139789497692784], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'properties': ('basic',), 'name': 'ip_frontend_ead_distant_eth1'})
    option_139789497693056 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_fichier_link', 'doc': u'Adresse IP pour le serveur de fichiers', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497693056.impl_set_information("separator", ('Adresse pour le serveur de fichier', None))
    option_139789497693056.impl_set_information("help", u"""Adresse IP supplémentaire sur cette interface réseau, utilisée pour accèder aux serveurs de fichiers. Le serveur aura donc plusieurs adresses IP sur le réseau local""")
    option_139789497693056.impl_add_consistency("in_network", option_139789497604576, option_139789497604304, **{'warnings_only': True})
    option_139789497693056.impl_add_consistency("not_equal", option_139789497604032, **{})
    option_139789497693056.impl_add_consistency("not_equal", option_139789497675024, **{})
    option_139789497693328 = NetmaskOption(**{'multi': False, 'name': 'adresse_mask_fichier_link', 'doc': u'Masque de sous r\xe9seau pour le serveur de fichiers', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497539040, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497693600 = BroadcastOption(**{'multi': False, 'name': 'adresse_bcast_fichier_link', 'doc': u'Broadcast pour le serveur de fichiers', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497693056, False), 'netmask': ParamOption(option_139789497693328, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497676496 = OptionDescription(children=[option_139789497603760, option_139789497604032, option_139789497604304, option_139789497604576, option_139789497604848, option_139789497536592, option_139789497605392, option_139789497595840, option_139789497605664, option_139789497634896, option_139789497635168, option_139789497635440, option_139789497605120, option_139789497635712, option_139789497635984, option_139789498047376, option_139789497636800, option_139789498047696, option_139789497637616, option_139789498048016, option_139789497657824, option_139789497675856, option_139789497672848, option_139789497673120, option_139789497673392, option_139789497673664, option_139789497674752, option_139789497656736, option_139789497675024, option_139789497675296, option_139789497692240, option_139789497965024, option_139789497676176, option_139789497657280, option_139789497693056, option_139789497693328, option_139789497693600, option_139789497657008], **{'doc': u'Interface-1', 'properties': ('basic',), 'name': 'interface_1'})
    option_139789497676496.impl_set_information("help", u"""Configuration de l'interface 1""")
    option_139789497676496.impl_set_information("icon", u"""sitemap""")
    option_139789497693872 = ChoiceOption(**{'multi': False, 'values': (u'statique',), 'name': 'eth2_method', 'default': u'statique', 'doc': u"M\xe9thode d'attribution de l'adressage pour l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497693872.impl_set_information("separator", ("Configuration de l'interface", None))
    option_139789497694144 = IPOption(**{'multi': False, 'name': 'adresse_ip_eth2', 'doc': u"Adresse IP de l'interface", 'properties': ('mandatory', 'basic'), 'private_only': True, 'warnings_only': True})
    option_139789497694416 = NetmaskOption(**{'default': u'255.255.255.0', 'doc': u"Masque de sous r\xe9seau de l'interface", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'adresse_netmask_eth2'})
    option_139789497694416.impl_add_consistency("ip_netmask", option_139789497694144, **{'warnings_only': True})
    option_139789497694688 = NetworkOption(**{'multi': False, 'name': 'adresse_network_eth2', 'doc': u"Adresse r\xe9seau de l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497694144, False), 'netmask': ParamOption(option_139789497694416, False)})})
    option_139789497694960 = BroadcastOption(**{'multi': False, 'name': 'adresse_broadcast_eth2', 'doc': u"Adresse broadcast de sous r\xe9seau de l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497694144, False), 'netmask': ParamOption(option_139789497694416, False)})})
    option_139789497695232 = ChoiceOption(**{'multi': False, 'name': 'zone_is_bridge_eth2', 'doc': u"L'interface r\xe9seau de la zone est un bridge", 'values': (u'oui', u'non'), 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497595568}]})
    option_139789497695504 = UnicodeOption(**{'multi': False, 'name': 'nom_zone_eth2', 'doc': u"Nom de l'interface r\xe9seau de la zone", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_zone_name, 'callback_params': Params(args=None, kwargs={'nom_carte': ParamOption(option_139789497536864, False), 'interface_no': ParamValue("2"), 'method': ParamOption(option_139789497693872, False), 'zone_is_bridge': ParamOption(option_139789497695232, True)})})
    option_139789497695776 = ChoiceOption(**{'multi': False, 'values': (u'balance-rr', u'active-backup', u'balance-xor', u'broadcast', u'802.3ad', u'balance-tlb', u'balance-alb'), 'name': 'bonding_mode_eth2', 'default': u'balance-rr', 'doc': u'Mode de bonding', 'properties': ('expert',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595568}]})
    option_139789497695776.impl_set_information("help", u"""Mode de fonctionnement du bond""")
    option_139789497712720 = IntOption(**{'multi': False, 'name': 'bonding_miimon_eth2', 'default': 100, 'doc': u'Fr\xe9quence des MII link monitoring en millisecondes', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595568}]})
    option_139789497712720.impl_set_information("help", u"""Intervalle de temps pour le monitoring des interfaces""")
    option_139789497712992 = IntOption(**{'multi': False, 'name': 'bonding_downdelay_eth2', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e down", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595568}]})
    option_139789497713264 = IntOption(**{'multi': False, 'name': 'bonding_updelay_eth2', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e comme active", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595568}]})
    option_139789497713536 = ChoiceOption(**{'multi': False, 'values': (u'', u'speed 100 duplex full autoneg off', u'autoneg on', u'speed 10 duplex half autoneg off', u'speed 1000 duplex full autoneg off'), 'name': 'debit_carte_eth2', 'default': u'', 'doc': u"Mode de connexion pour l'interface interne 2", 'properties': ('expert',)})
    option_139789497713808 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ssh_eth2', 'default': 'non', 'doc': u'Autoriser les connexions SSH', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497713808.impl_set_information("separator", ("Administration distante sur l'interface", None))
    option_139789497714080 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_ssh_eth2', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497713808}]})
    option_139789497714352 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_ssh_eth2', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497713808}]})
    option_139789497714352.impl_add_consistency("network_netmask", option_139789497714080, **{})
    option_139789497676816 = Leadership(children=[option_139789497714080, option_139789497714352], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'properties': ('basic',), 'name': 'ip_ssh_eth2'})
    option_139789497714624 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'admin_eth2', 'default': 'non', 'doc': u'Autoriser les connexions pour administrer le serveur (EAD, phpMyAdmin, ...)', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497714896 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_admin_eth2', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497714624}]})
    option_139789497715168 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_admin_eth2', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497714624}]})
    option_139789497715168.impl_add_consistency("network_netmask", option_139789497714896, **{})
    option_139789497677136 = Leadership(children=[option_139789497714896, option_139789497715168], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'properties': ('basic',), 'name': 'ip_admin_eth2'})
    option_139789497715440 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'alias_eth2', 'default': 'non', 'doc': u"Ajouter des IP alias sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497715440.impl_set_information("separator", ("Configuration des alias sur l'interface", None))
    option_139789497715712 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'alias_ip_eth2', 'doc': u"Adresse IP alias pour l'interface", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497715440}]})
    option_139789497715984 = NetmaskOption(**{'multi': True, 'name': 'alias_netmask_eth2', 'doc': u'Masque de sous r\xe9seau correspondant \xe0 cet alias', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497694416, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497715440}]})
    option_139789497716256 = NetworkOption(**{'multi': True, 'name': 'alias_network_eth2', 'doc': u'Adresse r\xe9seau correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497715712, False), 'netmask': ParamOption(option_139789497715984, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497715440}]})
    option_139789497729104 = BroadcastOption(**{'multi': True, 'name': 'alias_broadcast_eth2', 'doc': u'Adresse broadcast correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497715712, False), 'netmask': ParamOption(option_139789497715984, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497715440}]})
    option_139789497729376 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_rvp_alias_eth2', 'default_multi': 'non', 'doc': u'Autoriser cet alias \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497715440}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789497729648 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_forward_alias_eth2', 'default_multi': 'non', 'doc': u'Autoriser cet alias \xe0 utiliser les DNS de zones forward additionnelles', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497715440}]})
    option_139789497729920 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'squid_auth_eth2', 'default': 'oui', 'doc': u"Activer l'authentification sur cette interface", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497656464}]})
    option_139789497729920.impl_set_information("help", u"""L'authentification sur cette interface s'applique aussi aux VLAN et alias""")
    option_139789497730192 = ChoiceOption(**{'multi': False, 'name': 'activer_cntlm_eth2', 'doc': u'Activer le proxy NTLM sur cette interface', 'values': (u'oui', u'non'), 'properties': ('expert',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497574816, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497574816}, {'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497729920}]})
    option_139789497730464 = PortOption(**{'multi': False, 'name': 'wpad_proxyport_eth2', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD', 'properties': ('mandatory', 'expert'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamOption(option_139789497575088, True), 'condition_1': ParamOption(option_139789497730192, True), 'match': ParamValue("3128")}), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497730464.impl_set_information("separator", (u'Configuration de la d\xe9tection automatique du proxy', None))
    option_139789497730736 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_alias_eth2', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour cet alias', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497730464, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497677456 = Leadership(children=[option_139789497715712, option_139789497715984, option_139789497716256, option_139789497729104, option_139789497729376, option_139789497729648, option_139789497730736], **{'doc': u"Adresse IP alias pour l'interface", 'properties': ('basic',), 'name': 'alias_ip_eth2'})
    option_139789497731008 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'vlan_eth2', 'default': 'non', 'doc': u"Activer le support des VLAN sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497731008.impl_set_information("separator", ("Configuration des VLAN sur l'interface", None))
    option_139789497731280 = UnicodeOption(**{'doc': u"Num\xe9ro d'identifiant du VLAN", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'vlan_id_eth2', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497731008}]})
    option_139789497731552 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'vlan_ip_eth2', 'doc': u"Adresse IP de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497731008}]})
    option_139789497731824 = NetmaskOption(**{'multi': True, 'name': 'vlan_netmask_eth2', 'default_multi': u'255.255.255.0', 'doc': u"Masque de sous r\xe9seau de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497731008}]})
    option_139789497732096 = NetworkOption(**{'multi': True, 'name': 'vlan_network_eth2', 'doc': u"Adresse r\xe9seau de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497731552, False), 'netmask': ParamOption(option_139789497731824, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497731008}]})
    option_139789497732368 = BroadcastOption(**{'multi': True, 'name': 'vlan_broadcast_eth2', 'doc': u"Adresse de broadcast de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497731552, False), 'netmask': ParamOption(option_139789497731824, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497731008}]})
    option_139789497732640 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_rvp_vlan_eth2', 'default_multi': 'non', 'doc': u'Autoriser ce VLAN \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497731008}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}]})
    option_139789497745488 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_forward_vlan_eth2', 'default_multi': 'non', 'doc': u'Autoriser ce VLAN \xe0 utiliser les DNS des zones forward additionnelles', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497731008}]})
    option_139789497745760 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_vlan_eth2', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour ce VLAN', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497730464, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497677776 = Leadership(children=[option_139789497731280, option_139789497731552, option_139789497731824, option_139789497732096, option_139789497732368, option_139789497732640, option_139789497745488, option_139789497745760], **{'doc': u"Num\xe9ro d'identifiant du VLAN", 'properties': ('basic',), 'name': 'vlan_id_eth2'})
    option_139789497746032 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_master_eth2', 'default': 'oui', 'doc': u'Serveur master DNS de cette zone', 'properties': ('mandatory', 'normal')})
    option_139789497746032.impl_set_information("separator", ("Configuration DNS sur l'interface", None))
    option_139789497746304 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_rvp_eth2', 'default': 'non', 'doc': u'Autoriser cette zone \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789497746576 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_forward_eth2', 'default': 'oui', 'doc': u'Autoriser cette zone \xe0 utiliser les DNS des zones forward additionnelles', 'properties': ('mandatory', 'normal')})
    option_139789497746848 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2'), 'name': 'dansguardian_eth2', 'default': u'1', 'doc': u'Filtre web \xe0 appliquer sur cette interface', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789497746848.impl_set_information("separator", ('Configuration de la politique de filtrage', None))
    option_139789497746848.impl_set_information("help", u"""Instance e2guardian à utiliser pour l'interface 2 (1=Configuration 1, 2=Configuration 2)""")
    option_139789497747120 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_eth2_proxy_link', 'doc': u'Adresse IP pour le proxy', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497747120.impl_set_information("separator", ('Adresse pour le proxy', None))
    option_139789497747120.impl_set_information("help", u"""Adresse IP supplémentaire sur cette interface réseau, utilisée pour accèder au proxy. Le serveur aura donc plusieurs adresses IP sur le réseau local.""")
    option_139789497747120.impl_add_consistency("in_network", option_139789497694688, option_139789497694416, **{'transitive': False, 'warnings_only': True})
    option_139789497747120.impl_add_consistency("not_equal", option_139789497694144, **{'transitive': False})
    option_139789497747392 = NetmaskOption(**{'multi': False, 'name': 'adresse_mask_eth2_proxy_link', 'doc': u'Masque de sous r\xe9seau du proxy', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497694416, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497747664 = BroadcastOption(**{'multi': False, 'name': 'adresse_bcast_eth2_proxy_link', 'doc': u'Broadcast pour le proxy', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497747120, False), 'netmask': ParamOption(option_139789497747392, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497747936 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_frontend_ead_distant_eth2', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497965840}]})
    option_139789497748208 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau autoris\xe9 \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_frontend_ead_distant_eth2', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497965840}]})
    option_139789497748208.impl_add_consistency("network_netmask", option_139789497747936, **{})
    option_139789497678096 = Leadership(children=[option_139789497747936, option_139789497748208], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'properties': ('basic',), 'name': 'ip_frontend_ead_distant_eth2'})
    option_139789497678416 = OptionDescription(children=[option_139789497693872, option_139789497694144, option_139789497694416, option_139789497694688, option_139789497694960, option_139789497536864, option_139789497695504, option_139789497595568, option_139789497695776, option_139789497712720, option_139789497712992, option_139789497713264, option_139789497695232, option_139789497713536, option_139789497713808, option_139789497676816, option_139789497714624, option_139789497677136, option_139789497715440, option_139789497677456, option_139789497731008, option_139789497677776, option_139789497746032, option_139789497746304, option_139789497746576, option_139789497673936, option_139789497746848, option_139789497729920, option_139789497747120, option_139789497747392, option_139789497747664, option_139789497965840, option_139789497678096, option_139789497730464, option_139789497730192], requires=[{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}], **{'doc': u'Interface-2', 'properties': ('basic',), 'name': 'interface_2'})
    option_139789497678416.impl_set_information("help", u"""Configuration de l'interface 2""")
    option_139789497678416.impl_set_information("icon", u"""sitemap""")
    option_139789497748480 = ChoiceOption(**{'multi': False, 'values': (u'statique',), 'name': 'eth3_method', 'default': u'statique', 'doc': u"M\xe9thode d'attribution de l'adressage pour l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497748480.impl_set_information("separator", ("Configuration de l'interface", None))
    option_139789497748752 = IPOption(**{'multi': False, 'name': 'adresse_ip_eth3', 'doc': u"Adresse IP de l'interface", 'properties': ('mandatory', 'basic'), 'private_only': True, 'warnings_only': True})
    option_139789497749024 = NetmaskOption(**{'default': u'255.255.255.0', 'doc': u"Masque de sous r\xe9seau de l'interface", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'adresse_netmask_eth3'})
    option_139789497749024.impl_add_consistency("ip_netmask", option_139789497748752, **{'warnings_only': True})
    option_139789497765968 = NetworkOption(**{'multi': False, 'name': 'adresse_network_eth3', 'doc': u"Adresse r\xe9seau de l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497748752, False), 'netmask': ParamOption(option_139789497749024, False)})})
    option_139789497766240 = BroadcastOption(**{'multi': False, 'name': 'adresse_broadcast_eth3', 'doc': u"Adresse broadcast de sous r\xe9seau de l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497748752, False), 'netmask': ParamOption(option_139789497749024, False)})})
    option_139789497766512 = ChoiceOption(**{'multi': False, 'name': 'zone_is_bridge_eth3', 'doc': u"L'interface r\xe9seau de la zone est un bridge", 'values': (u'oui', u'non'), 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497595296}]})
    option_139789497766784 = UnicodeOption(**{'multi': False, 'name': 'nom_zone_eth3', 'doc': u"Nom de l'interface r\xe9seau de la zone", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_zone_name, 'callback_params': Params(args=None, kwargs={'nom_carte': ParamOption(option_139789497537136, False), 'interface_no': ParamValue("3"), 'method': ParamOption(option_139789497748480, False), 'zone_is_bridge': ParamOption(option_139789497766512, True)})})
    option_139789497767056 = ChoiceOption(**{'multi': False, 'values': (u'balance-rr', u'active-backup', u'balance-xor', u'broadcast', u'802.3ad', u'balance-tlb', u'balance-alb'), 'name': 'bonding_mode_eth3', 'default': u'balance-rr', 'doc': u'Mode de bonding', 'properties': ('expert',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595296}]})
    option_139789497767056.impl_set_information("help", u"""Mode de fonctionnement du bond""")
    option_139789497767328 = IntOption(**{'multi': False, 'name': 'bonding_miimon_eth3', 'default': 100, 'doc': u'Fr\xe9quence des MII link monitoring en millisecondes', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595296}]})
    option_139789497767328.impl_set_information("help", u"""Intervalle de temps pour le monitoring des interfaces""")
    option_139789497767600 = IntOption(**{'multi': False, 'name': 'bonding_downdelay_eth3', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e down", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595296}]})
    option_139789497767872 = IntOption(**{'multi': False, 'name': 'bonding_updelay_eth3', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e comme active", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595296}]})
    option_139789497768144 = ChoiceOption(**{'multi': False, 'values': (u'', u'speed 100 duplex full autoneg off', u'autoneg on', u'speed 10 duplex half autoneg off', u'speed 1000 duplex full autoneg off'), 'name': 'debit_carte_eth3', 'default': u'', 'doc': u"Mode de connexion pour l'interface interne 3", 'properties': ('expert',)})
    option_139789497768416 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ssh_eth3', 'default': 'non', 'doc': u'Autoriser les connexions SSH', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497768416.impl_set_information("separator", ("Administration distante sur l'interface", None))
    option_139789497768688 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_ssh_eth3', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497768416}]})
    option_139789497768960 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_ssh_eth3', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497768416}]})
    option_139789497768960.impl_add_consistency("network_netmask", option_139789497768688, **{})
    option_139789497678736 = Leadership(children=[option_139789497768688, option_139789497768960], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'properties': ('basic',), 'name': 'ip_ssh_eth3'})
    option_139789497769232 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'admin_eth3', 'default': 'non', 'doc': u'Autoriser les connexions pour administrer le serveur (EAD, phpMyAdmin, ...)', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497769504 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_admin_eth3', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497769232}]})
    option_139789497782352 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_admin_eth3', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497769232}]})
    option_139789497782352.impl_add_consistency("network_netmask", option_139789497769504, **{})
    option_139789497679056 = Leadership(children=[option_139789497769504, option_139789497782352], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'properties': ('basic',), 'name': 'ip_admin_eth3'})
    option_139789497782624 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'alias_eth3', 'default': 'non', 'doc': u"Ajouter des IP alias sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497782624.impl_set_information("separator", ("Configuration des alias sur l'interface", None))
    option_139789497782896 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'alias_ip_eth3', 'doc': u"Adresse IP alias pour l'interface", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497782624}]})
    option_139789497783168 = NetmaskOption(**{'multi': True, 'name': 'alias_netmask_eth3', 'doc': u'Masque de sous r\xe9seau correspondant \xe0 cet alias', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497749024, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497782624}]})
    option_139789497783440 = NetworkOption(**{'multi': True, 'name': 'alias_network_eth3', 'doc': u'Adresse r\xe9seau correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497782896, False), 'netmask': ParamOption(option_139789497783168, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497782624}]})
    option_139789497783712 = BroadcastOption(**{'multi': True, 'name': 'alias_broadcast_eth3', 'doc': u'Adresse broadcast correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497782896, False), 'netmask': ParamOption(option_139789497783168, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497782624}]})
    option_139789497783984 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_rvp_alias_eth3', 'default_multi': 'non', 'doc': u'Autoriser cet alias \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497782624}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789497784256 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_forward_alias_eth3', 'default_multi': 'non', 'doc': u'Autoriser cet alias \xe0 utiliser les DNS de zones forward additionnelles', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497782624}]})
    option_139789497784528 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'squid_auth_eth3', 'default': 'oui', 'doc': u"Activer l'authentification sur cette interface", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497656464}]})
    option_139789497784528.impl_set_information("help", u"""L'authentification sur cette interface s'applique aussi aux VLAN et alias""")
    option_139789497784800 = ChoiceOption(**{'multi': False, 'name': 'activer_cntlm_eth3', 'doc': u'Activer le proxy NTLM sur cette interface', 'values': (u'oui', u'non'), 'properties': ('expert',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497574816, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497574816}, {'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497784528}]})
    option_139789497785072 = PortOption(**{'multi': False, 'name': 'wpad_proxyport_eth3', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD', 'properties': ('mandatory', 'expert'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamOption(option_139789497575088, True), 'condition_1': ParamOption(option_139789497784800, True), 'match': ParamValue("3128")}), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497785072.impl_set_information("separator", (u'Configuration de la d\xe9tection automatique du proxy', None))
    option_139789497785344 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_alias_eth3', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour cet alias', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497785072, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497679376 = Leadership(children=[option_139789497782896, option_139789497783168, option_139789497783440, option_139789497783712, option_139789497783984, option_139789497784256, option_139789497785344], **{'doc': u"Adresse IP alias pour l'interface", 'properties': ('basic',), 'name': 'alias_ip_eth3'})
    option_139789497785616 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'vlan_eth3', 'default': 'non', 'doc': u"Activer le support des VLAN sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497785616.impl_set_information("separator", ("Configuration des VLAN sur l'interface", None))
    option_139789497785888 = UnicodeOption(**{'doc': u"Num\xe9ro d'identifiant du VLAN", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'vlan_id_eth3', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497785616}]})
    option_139789497270352 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'vlan_ip_eth3', 'doc': u"Adresse IP de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497785616}]})
    option_139789497270624 = NetmaskOption(**{'multi': True, 'name': 'vlan_netmask_eth3', 'default_multi': u'255.255.255.0', 'doc': u"Masque de sous r\xe9seau de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497785616}]})
    option_139789497270896 = NetworkOption(**{'multi': True, 'name': 'vlan_network_eth3', 'doc': u"Adresse r\xe9seau de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497270352, False), 'netmask': ParamOption(option_139789497270624, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497785616}]})
    option_139789497271168 = BroadcastOption(**{'multi': True, 'name': 'vlan_broadcast_eth3', 'doc': u"Adresse de broadcast de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497270352, False), 'netmask': ParamOption(option_139789497270624, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497785616}]})
    option_139789497271440 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_rvp_vlan_eth3', 'default_multi': 'non', 'doc': u'Autoriser ce VLAN \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497785616}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}]})
    option_139789497271712 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_forward_vlan_eth3', 'default_multi': 'non', 'doc': u'Autoriser ce VLAN \xe0 utiliser les DNS des zones forward additionnelles', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497785616}]})
    option_139789497271984 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_vlan_eth3', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour ce VLAN', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497785072, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497278544 = Leadership(children=[option_139789497785888, option_139789497270352, option_139789497270624, option_139789497270896, option_139789497271168, option_139789497271440, option_139789497271712, option_139789497271984], **{'doc': u"Num\xe9ro d'identifiant du VLAN", 'properties': ('basic',), 'name': 'vlan_id_eth3'})
    option_139789497272256 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_master_eth3', 'default': 'oui', 'doc': u'Serveur master DNS de cette zone', 'properties': ('mandatory', 'normal')})
    option_139789497272256.impl_set_information("separator", ("Configuration DNS sur l'interface", None))
    option_139789497272528 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_rvp_eth3', 'default': 'non', 'doc': u'Autoriser cette zone \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789497272800 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_forward_eth3', 'default': 'oui', 'doc': u'Autoriser cette zone \xe0 utiliser les DNS des zones forward additionnelles', 'properties': ('mandatory', 'normal')})
    option_139789497273072 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2'), 'name': 'dansguardian_eth3', 'default': u'1', 'doc': u'Filtre web \xe0 appliquer sur cette interface', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789497273072.impl_set_information("separator", ('Configuration de la politique de filtrage', None))
    option_139789497273072.impl_set_information("help", u"""Instance e2guardian à utiliser pour l'interface 3 (1=Configuration 1, 2=Configuration 2)""")
    option_139789497273344 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_eth3_proxy_link', 'doc': u'Adresse IP pour le proxy', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497273344.impl_set_information("separator", ('Adresse pour le proxy', None))
    option_139789497273344.impl_add_consistency("in_network", option_139789497765968, option_139789497749024, **{'transitive': False, 'warnings_only': True})
    option_139789497273344.impl_add_consistency("not_equal", option_139789497748752, **{'transitive': False})
    option_139789497273616 = NetmaskOption(**{'multi': False, 'name': 'adresse_mask_eth3_proxy_link', 'doc': u'Masque de sous r\xe9seau du proxy', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497749024, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497273888 = BroadcastOption(**{'multi': False, 'name': 'adresse_bcast_eth3_proxy_link', 'doc': u'Broadcast pour le proxy', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497273344, False), 'netmask': ParamOption(option_139789497273616, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497299024 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_frontend_ead_distant_eth3', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497965568}]})
    option_139789497299296 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau autoris\xe9 \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_frontend_ead_distant_eth3', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497965568}]})
    option_139789497299296.impl_add_consistency("network_netmask", option_139789497299024, **{})
    option_139789497278864 = Leadership(children=[option_139789497299024, option_139789497299296], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'properties': ('basic',), 'name': 'ip_frontend_ead_distant_eth3'})
    option_139789497279184 = OptionDescription(children=[option_139789497748480, option_139789497748752, option_139789497749024, option_139789497765968, option_139789497766240, option_139789497537136, option_139789497766784, option_139789497595296, option_139789497767056, option_139789497767328, option_139789497767600, option_139789497767872, option_139789497766512, option_139789497768144, option_139789497768416, option_139789497678736, option_139789497769232, option_139789497679056, option_139789497782624, option_139789497679376, option_139789497785616, option_139789497278544, option_139789497272256, option_139789497272528, option_139789497272800, option_139789497674208, option_139789497273072, option_139789497784528, option_139789497273344, option_139789497273616, option_139789497273888, option_139789497965568, option_139789497278864, option_139789497785072, option_139789497784800], requires=[{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}], **{'doc': u'Interface-3', 'properties': ('basic',), 'name': 'interface_3'})
    option_139789497279184.impl_set_information("help", u"""Configuration de l'interface 3""")
    option_139789497279184.impl_set_information("icon", u"""sitemap""")
    option_139789497299568 = ChoiceOption(**{'multi': False, 'values': (u'statique',), 'name': 'eth4_method', 'default': u'statique', 'doc': u"M\xe9thode d'attribution de l'adressage pour l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497299568.impl_set_information("separator", ("Configuration de l'interface", None))
    option_139789497299840 = IPOption(**{'multi': False, 'name': 'adresse_ip_eth4', 'doc': u"Adresse IP de l'interface", 'properties': ('mandatory', 'basic'), 'private_only': True, 'warnings_only': True})
    option_139789497300112 = NetmaskOption(**{'default': u'255.255.255.0', 'doc': u"Masque de sous r\xe9seau de l'interface", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'adresse_netmask_eth4'})
    option_139789497300112.impl_add_consistency("ip_netmask", option_139789497299840, **{'warnings_only': True})
    option_139789497300384 = NetworkOption(**{'multi': False, 'name': 'adresse_network_eth4', 'doc': u"Adresse r\xe9seau de l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497299840, False), 'netmask': ParamOption(option_139789497300112, False)})})
    option_139789497300656 = BroadcastOption(**{'multi': False, 'name': 'adresse_broadcast_eth4', 'doc': u"Adresse broadcast de sous r\xe9seau de l'interface", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497299840, False), 'netmask': ParamOption(option_139789497300112, False)})})
    option_139789497300928 = ChoiceOption(**{'multi': False, 'name': 'zone_is_bridge_eth4', 'doc': u"L'interface r\xe9seau de la zone est un bridge", 'values': (u'oui', u'non'), 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497595024}]})
    option_139789497301200 = UnicodeOption(**{'multi': False, 'name': 'nom_zone_eth4', 'doc': u"Nom de l'interface r\xe9seau de la zone", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.get_zone_name, 'callback_params': Params(args=None, kwargs={'nom_carte': ParamOption(option_139789497537408, False), 'interface_no': ParamValue("4"), 'method': ParamOption(option_139789497299568, False), 'zone_is_bridge': ParamOption(option_139789497300928, True)})})
    option_139789497301472 = ChoiceOption(**{'multi': False, 'values': (u'balance-rr', u'active-backup', u'balance-xor', u'broadcast', u'802.3ad', u'balance-tlb', u'balance-alb'), 'name': 'bonding_mode_eth4', 'default': u'balance-rr', 'doc': u'Mode de bonding', 'properties': ('expert',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595024}]})
    option_139789497301472.impl_set_information("help", u"""Mode de fonctionnement du bond""")
    option_139789497301744 = IntOption(**{'multi': False, 'name': 'bonding_miimon_eth4', 'default': 100, 'doc': u'Fr\xe9quence des MII link monitoring en millisecondes', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595024}]})
    option_139789497301744.impl_set_information("help", u"""Intervalle de temps pour le monitoring des interfaces""")
    option_139789497302016 = IntOption(**{'multi': False, 'name': 'bonding_downdelay_eth4', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e down", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595024}]})
    option_139789497302288 = IntOption(**{'multi': False, 'name': 'bonding_updelay_eth4', 'default': 200, 'doc': u"Temps en millisecondes pour qu'une interface soit d\xe9tect\xe9e comme active", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497595024}]})
    option_139789497302560 = ChoiceOption(**{'multi': False, 'values': (u'', u'speed 100 duplex full autoneg off', u'autoneg on', u'speed 10 duplex half autoneg off', u'speed 1000 duplex full autoneg off'), 'name': 'debit_carte_eth4', 'default': u'', 'doc': u"Mode de connexion pour l'interface interne 4", 'properties': ('expert',)})
    option_139789497323600 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ssh_eth4', 'default': 'non', 'doc': u'Autoriser les connexions SSH', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497323600.impl_set_information("separator", ("Administration distante sur l'interface", None))
    option_139789497323872 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_ssh_eth4', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497323600}]})
    option_139789497324144 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour les connexions SSH', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_ssh_eth4', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497323600}]})
    option_139789497324144.impl_add_consistency("network_netmask", option_139789497323872, **{})
    option_139789497279504 = Leadership(children=[option_139789497323872, option_139789497324144], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions SSH', 'properties': ('basic',), 'name': 'ip_ssh_eth4'})
    option_139789497324416 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'admin_eth4', 'default': 'non', 'doc': u'Autoriser les connexions pour administrer le serveur (EAD, phpMyAdmin, ...)', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789497324688 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_admin_eth4', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497324416}]})
    option_139789497324960 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour administrer le serveur', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_admin_eth4', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497324416}]})
    option_139789497324960.impl_add_consistency("network_netmask", option_139789497324688, **{})
    option_139789497279824 = Leadership(children=[option_139789497324688, option_139789497324960], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour administrer le serveur', 'properties': ('basic',), 'name': 'ip_admin_eth4'})
    option_139789497325232 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'alias_eth4', 'default': 'non', 'doc': u"Ajouter des IP alias sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497325232.impl_set_information("separator", ("Configuration des alias sur l'interface", None))
    option_139789497325504 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'alias_ip_eth4', 'doc': u"Adresse IP alias pour l'interface", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497325232}]})
    option_139789497325776 = NetmaskOption(**{'multi': True, 'name': 'alias_netmask_eth4', 'doc': u'Masque de sous r\xe9seau correspondant \xe0 cet alias', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497300112, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497325232}]})
    option_139789497326048 = NetworkOption(**{'multi': True, 'name': 'alias_network_eth4', 'doc': u'Adresse r\xe9seau correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497325504, False), 'netmask': ParamOption(option_139789497325776, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497325232}]})
    option_139789497326320 = BroadcastOption(**{'multi': True, 'name': 'alias_broadcast_eth4', 'doc': u'Adresse broadcast correspondant \xe0 cet alias', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497325504, False), 'netmask': ParamOption(option_139789497325776, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497325232}]})
    option_139789497326592 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_rvp_alias_eth4', 'default_multi': 'non', 'doc': u'Autoriser cet alias \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497325232}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789497326864 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_forward_alias_eth4', 'default_multi': 'non', 'doc': u'Autoriser cet alias \xe0 utiliser les DNS de zones forward additionnelles', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497325232}]})
    option_139789497327136 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'squid_auth_eth4', 'default': 'oui', 'doc': u"Activer l'authentification sur cette interface", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497656464}]})
    option_139789497327136.impl_set_information("help", u"""L'authentification sur cette interface s'applique aussi aux VLAN et alias""")
    option_139789497335888 = ChoiceOption(**{'multi': False, 'name': 'activer_cntlm_eth4', 'doc': u'Activer le proxy NTLM sur cette interface', 'values': (u'oui', u'non'), 'properties': ('expert',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497574816, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497574816}, {'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497327136}]})
    option_139789497336160 = PortOption(**{'multi': False, 'name': 'wpad_proxyport_eth4', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD', 'properties': ('mandatory', 'expert'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamOption(option_139789497575088, True), 'condition_1': ParamOption(option_139789497335888, True), 'match': ParamValue("3128")}), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497336160.impl_set_information("separator", (u'Configuration de la d\xe9tection automatique du proxy', None))
    option_139789497336432 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_alias_eth4', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour cet alias', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497336160, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497280144 = Leadership(children=[option_139789497325504, option_139789497325776, option_139789497326048, option_139789497326320, option_139789497326592, option_139789497326864, option_139789497336432], **{'doc': u"Adresse IP alias pour l'interface", 'properties': ('basic',), 'name': 'alias_ip_eth4'})
    option_139789497336704 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'vlan_eth4', 'default': 'non', 'doc': u"Activer le support des VLAN sur l'interface", 'properties': ('mandatory', 'normal')})
    option_139789497336704.impl_set_information("separator", ("Configuration des VLAN sur l'interface", None))
    option_139789497336976 = UnicodeOption(**{'doc': u"Num\xe9ro d'identifiant du VLAN", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'vlan_id_eth4', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497336704}]})
    option_139789497337248 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'vlan_ip_eth4', 'doc': u"Adresse IP de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497336704}]})
    option_139789497337520 = NetmaskOption(**{'multi': True, 'name': 'vlan_netmask_eth4', 'default_multi': u'255.255.255.0', 'doc': u"Masque de sous r\xe9seau de l'interface dans ce VLAN", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497336704}]})
    option_139789497337792 = NetworkOption(**{'multi': True, 'name': 'vlan_network_eth4', 'doc': u"Adresse r\xe9seau de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_network, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497337248, False), 'netmask': ParamOption(option_139789497337520, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497336704}]})
    option_139789497338064 = BroadcastOption(**{'multi': True, 'name': 'vlan_broadcast_eth4', 'doc': u"Adresse de broadcast de l'interface dans ce VLAN", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497337248, False), 'netmask': ParamOption(option_139789497337520, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497336704}]})
    option_139789497338336 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_rvp_vlan_eth4', 'default_multi': 'non', 'doc': u'Autoriser ce VLAN \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497336704}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}]})
    option_139789497338608 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_forward_vlan_eth4', 'default_multi': 'non', 'doc': u'Autoriser ce VLAN \xe0 utiliser les DNS des zones forward additionnelles', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497336704}]})
    option_139789497338880 = PortOption(**{'multi': True, 'name': 'wpad_proxyport_vlan_eth4', 'allow_private': True, 'doc': u'Port du proxy diffus\xe9 par WPAD pour ce VLAN', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497336160, False)), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789497280464 = Leadership(children=[option_139789497336976, option_139789497337248, option_139789497337520, option_139789497337792, option_139789497338064, option_139789497338336, option_139789497338608, option_139789497338880], **{'doc': u"Num\xe9ro d'identifiant du VLAN", 'properties': ('basic',), 'name': 'vlan_id_eth4'})
    option_139789497339152 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_master_eth4', 'default': 'oui', 'doc': u'Serveur master DNS de cette zone', 'properties': ('mandatory', 'normal')})
    option_139789497339152.impl_set_information("separator", ("Configuration DNS sur l'interface", None))
    option_139789497339424 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_rvp_eth4', 'default': 'non', 'doc': u'Autoriser cette zone \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789497352272 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dns_forward_eth4', 'default': 'oui', 'doc': u'Autoriser cette zone \xe0 utiliser les DNS des zones forward additionnelles', 'properties': ('mandatory', 'normal')})
    option_139789497352544 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2'), 'name': 'dansguardian_eth4', 'default': u'1', 'doc': u'Filtre web \xe0 appliquer sur cette interface', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789497352544.impl_set_information("separator", ('Configuration de la politique de filtrage', None))
    option_139789497352544.impl_set_information("help", u"""Instance e2guardian à utiliser pour l'interface 4 (1=Configuration 1, 2=Configuration 2)""")
    option_139789497352816 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_eth4_proxy_link', 'doc': u'Adresse IP pour le proxy', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497352816.impl_set_information("separator", ('Adresse pour le proxy', None))
    option_139789497352816.impl_add_consistency("in_network", option_139789497300384, option_139789497300112, **{'transitive': False, 'warnings_only': True})
    option_139789497352816.impl_add_consistency("not_equal", option_139789497299840, **{'transitive': False})
    option_139789497353088 = NetmaskOption(**{'multi': False, 'name': 'adresse_mask_eth4_proxy_link', 'doc': u'Masque de sous r\xe9seau du proxy', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497300112, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497353360 = BroadcastOption(**{'multi': False, 'name': 'adresse_bcast_eth4_proxy_link', 'doc': u'Broadcast pour le proxy', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_broadcast, 'callback_params': Params(args=None, kwargs={'ip': ParamOption(option_139789497352816, False), 'netmask': ParamOption(option_139789497353088, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497353632 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_frontend_ead_distant_eth4', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964752}]})
    option_139789497353904 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau autoris\xe9 \xe0 acc\xe9der au backend EAD', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'netmask_frontend_ead_distant_eth4', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964752}]})
    option_139789497353904.impl_add_consistency("network_netmask", option_139789497353632, **{})
    option_139789497280784 = Leadership(children=[option_139789497353632, option_139789497353904], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e \xe0 acc\xe9der au backend EAD', 'properties': ('basic',), 'name': 'ip_frontend_ead_distant_eth4'})
    option_139789497281104 = OptionDescription(children=[option_139789497299568, option_139789497299840, option_139789497300112, option_139789497300384, option_139789497300656, option_139789497537408, option_139789497301200, option_139789497595024, option_139789497301472, option_139789497301744, option_139789497302016, option_139789497302288, option_139789497300928, option_139789497302560, option_139789497323600, option_139789497279504, option_139789497324416, option_139789497279824, option_139789497325232, option_139789497280144, option_139789497336704, option_139789497280464, option_139789497339152, option_139789497339424, option_139789497352272, option_139789497674480, option_139789497352544, option_139789497327136, option_139789497352816, option_139789497353088, option_139789497353360, option_139789497964752, option_139789497280784, option_139789497336160, option_139789497335888], requires=[{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}], **{'doc': u'Interface-4', 'properties': ('basic',), 'name': 'interface_4'})
    option_139789497281104.impl_set_information("help", u"""Configuration de l'interface 4""")
    option_139789497281104.impl_set_information("icon", u"""sitemap""")
    option_139789497354176 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'restrict_ping_request', 'default': 'non', 'doc': u'Restreindre le ping aux r\xe9seaux autoris\xe9s pour administrer le serveur', 'properties': ('disabled', 'mandatory', 'expert')})
    option_139789497354720 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_ipv6', 'default': 'non', 'doc': u'Activer le support IPV6', 'properties': ('mandatory', 'expert')})
    option_139789497354448 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_routage_ipv4', 'default': 'oui', 'doc': u'Activer le routage IPv4 entre les interfaces', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497354448.impl_set_information("help", u"""Permet d’activer le routage IPv4 au niveau du noyau en passant à 1 la valeur de /proc/sys/net/ipv4/ip_forward. Valeur par défaut: « non ».""")
    option_139789497354992 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_routage_ipv6', 'default': 'non', 'doc': u'Activer le routage IPv6 entre les interfaces', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497354720}]})
    option_139789497354992.impl_set_information("help", u"""Permet d’activer le routage IPv6 au niveau du noyau en passant à 1 la valeur de /proc/sys/net/ipv6/conf/all/forwarding. Valeur par défaut: « non ».""")
    option_139789497355264 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_log_martian', 'default': 'non', 'doc': u'Journaliser les "martian sources"', 'properties': ('mandatory', 'expert')})
    option_139789497355264.impl_set_information("separator", (u'S\xe9curit\xe9', None))
    option_139789497355536 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_antispoofing', 'default': 'non', 'doc': u"Activer l'anti-spoofing sur toutes les interfaces", 'properties': ('mandatory', 'expert')})
    option_139789497355808 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_ajout_hosts', 'default': 'non', 'doc': u"D\xe9clarer des noms d'h\xf4tes suppl\xe9mentaires", 'properties': ('mandatory', 'expert')})
    option_139789497355808.impl_set_information("separator", (u"Ajout d'h\xf4tes", None))
    option_139789497355808.impl_set_information("help", u"""Permet d'ajouter des entrées dans le fichier /etc/hosts""")
    option_139789497380944 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'adresse_ip_hosts', 'doc': u"Adresse IP de l'h\xf4te", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497355808}]})
    option_139789497381216 = DomainnameOption(**{'type_': 'domainname', 'name': 'nom_long_hosts', 'doc': u"Nom long de l'h\xf4te", 'allow_ip': False, 'multi': True, 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497355808}]})
    option_139789497381488 = DomainnameOption(**{'type_': 'hostname', 'name': 'nom_court_hosts', 'doc': u"Nom court de l'h\xf4te", 'allow_ip': False, 'multi': True, 'properties': ('expert',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497355808}]})
    option_139789497281424 = Leadership(children=[option_139789497380944, option_139789497381216, option_139789497381488], **{'doc': u"Adresse IP de l'h\xf4te", 'properties': ('expert',), 'name': 'adresse_ip_hosts'})
    option_139789497381760 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_route', 'default': 'non', 'doc': u'Ajouter des routes statiques', 'properties': ('mandatory', 'expert')})
    option_139789497381760.impl_set_information("separator", ('Ajout de routes statiques', None))
    option_139789497381760.impl_set_information("help", u"""Permet de déclarer des routes afin d'accéder à des adresses ou à des plages d'adresses par un chemin différent de celui par défaut""")
    option_139789497382032 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'route_adresse', 'doc': u'Adresse IP ou r\xe9seau \xe0 ajouter dans la table de routage', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497381760}]})
    option_139789497382304 = NetmaskOption(**{'doc': u'Masque de sous r\xe9seau (mettre \xe0 255.255.255.255 si adresse host)', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'route_netmask', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497381760}]})
    option_139789497382576 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'route_gw', 'doc': u'Adresse IP de la passerelle pour acc\xe9der \xe0 ce r\xe9seau', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497381760}]})
    option_139789497382848 = ChoiceOption(**{'multi': True, 'values': (u'0', u'1', u'2', u'3', u'4'), 'name': 'route_int', 'doc': u'Interface r\xe9seau reli\xe9e \xe0 la passerelle', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497381760}]})
    option_139789497383120 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_route', 'default_multi': 'oui', 'doc': u'Autoriser ce r\xe9seau \xe0 utiliser les DNS du serveur', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497381760}]})
    option_139789497383392 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_rvp_route', 'default_multi': 'oui', 'doc': u'Autoriser ce r\xe9seau \xe0 utiliser les DNS de Forward RVP/AGRIATES', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497381760}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789497383664 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dns_forward_route', 'default_multi': 'oui', 'doc': u'Autoriser ce r\xe9seau \xe0 utiliser les DNS des zones forward additionnelles', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497381760}]})
    option_139789497281744 = Leadership(children=[option_139789497382032, option_139789497382304, option_139789497382576, option_139789497382848, option_139789497383120, option_139789497383392, option_139789497383664], **{'doc': u'Adresse IP ou r\xe9seau \xe0 ajouter dans la table de routage', 'properties': ('expert',), 'name': 'route_adresse'})
    option_139789497383936 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ip_no_pmtu_disc', 'default': 'non', 'doc': u'D\xe9sactiver le path MTU discovery, le bit DF est positionn\xe9 \xe0 0', 'properties': ('mandatory', 'expert')})
    option_139789497383936.impl_set_information("separator", ('Configuration du MTU', None))
    option_139789497384208 = IntOption(**{'doc': u"Valeur du MTU pour l'interface 0 : rien = valeur par d\xe9faut de l'interface", 'multi': False, 'properties': ('expert',), 'name': 'valeur_mtu_eth0'})
    option_139789497384480 = IntOption(**{'doc': u"Valeur du MTU pour l'interface 1 : rien = valeur par d\xe9faut de l'interface", 'multi': False, 'properties': ('expert',), 'name': 'valeur_mtu_eth1'})
    option_139789497393232 = IntOption(**{'doc': u"Valeur du MTU pour l'interface 2 : rien = valeur par d\xe9faut de l'interface", 'multi': False, 'properties': ('expert',), 'name': 'valeur_mtu_eth2', "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789497393504 = IntOption(**{'doc': u"Valeur du MTU pour l'interface 3 : rien = valeur par d\xe9faut de l'interface", 'multi': False, 'properties': ('expert',), 'name': 'valeur_mtu_eth3', "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789497393776 = IntOption(**{'doc': u"Valeur du MTU pour l'interface 4 : rien = valeur par d\xe9faut de l'interface", 'multi': False, 'properties': ('expert',), 'name': 'valeur_mtu_eth4', "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789497394048 = IntOption(**{'doc': u"Valeur du MTU pour l'interface ppp0 : rien = valeur par d\xe9faut de l'interface", 'multi': False, 'properties': ('expert',), 'name': 'valeur_mtu_ppp0', "requires": [{'expected': u'pppoe', 'action': 'disabled', 'inverse': True, "option": option_139789497938272}]})
    option_139789497394320 = IntOption(**{'default': 128, 'doc': u'Neighbour table overflow stop culling limit', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ipv4_neigh_default_gc_thresh1'})
    option_139789497394320.impl_set_information("separator", ('Configuration de la "neighbour table"', None))
    option_139789497394592 = IntOption(**{'default': 512, 'doc': u'Neighbour table overflow soft limit', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ipv4_neigh_default_gc_thresh2'})
    option_139789497394864 = IntOption(**{'default': 1024, 'doc': u'Neighbour table overflow hard limit', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ipv4_neigh_default_gc_thresh3'})
    option_139789497395136 = DomainnameOption(**{'multi': True, 'name': 'test_distant_domaine', 'default': [u'bp-eole.ac-dijon.fr', u'archive.ubuntu.com'], 'doc': u"Domaine utilis\xe9 pour le test de l'acc\xe8s distant", 'type_': 'domainname', 'allow_ip': False, 'default_multi': u'bp-eole.ac-dijon.fr', 'properties': ('mandatory', 'expert')})
    option_139789497395136.impl_set_information("separator", (u"Test de l'acc\xe8s distant", None))
    option_139789497282064 = OptionDescription(children=[option_139789497966112, option_139789497354176, option_139789497354720, option_139789497354448, option_139789497354992, option_139789497355264, option_139789497355536, option_139789497355808, option_139789497281424, option_139789497381760, option_139789497281744, option_139789497383936, option_139789497384208, option_139789497384480, option_139789497393232, option_139789497393504, option_139789497393776, option_139789497394048, option_139789497394320, option_139789497394592, option_139789497394864, option_139789497395136], **{'doc': u'R\xe9seau avanc\xe9', 'properties': ('expert',), 'name': 'reseau_avance'})
    option_139789497282064.impl_set_information("icon", u"""sitemap""")
    option_139789497395408 = IntOption(**{'multi': False, 'name': 'ssl_default_key_bits', 'default': 2048, 'doc': u'Taille de la cl\xe9', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}]})
    option_139789497395408.impl_set_information("separator", (u'Param\xe8tres SSL', None))
    option_139789497395680 = IntOption(**{'multi': False, 'name': 'ssl_default_cert_time', 'default': 1096, 'doc': u'Dur\xe9e de validit\xe9 du certificat (en jours)', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}]})
    option_139789497395952 = UnicodeOption(**{'multi': False, 'name': 'ssl_country_name', 'doc': u'Nom du pays (C=)', 'properties': ('expert',), 'callback': eosfunc.calc_ssl_country_name, 'callback_params': Params(args=None, kwargs={'valeur': ParamOption(option_139789497891568, False)}), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}], "validator": eosfunc.valid_country})
    option_139789497395952.impl_set_information("separator", ('Subject (DN)', None))
    option_139789497396224 = UnicodeOption(**{'multi': False, 'name': 'ssl_organization_name', 'default': u'Ministere Education Nationale (MENESR)', 'doc': u"Nom de l'organisation (O=)", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}]})
    option_139789497396496 = UnicodeOption(**{'multi': True, 'name': 'ssl_organization_unit_name', 'doc': u"Nom de l'unit\xe9 de l'organisation (OU=)", 'properties': ('expert',), 'callback': eosfunc.calc_multi_val, 'callback_params': Params(args=(ParamValue("110 043 015"), ParamOption(option_139789497891296, False)), kwargs={}), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}]})
    option_139789497396768 = DomainnameOption(**{'multi': False, 'name': 'ssl_server_name', 'allow_ip': False, 'type_': 'domainname', 'doc': u'Nom DNS du serveur (de la forme nom-serveur.ac-acad.fr) (CN=)', 'properties': ('expert',), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789497891024, False),), kwargs={}), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}]})
    option_139789497413712 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'revprox_activate_http', 'default': 'non', 'doc': u'Activer le reverse proxy Nginx pour http/https', 'properties': ('mandatory', 'normal')})
    option_139789497413712.impl_set_information("separator", ('Redirection HTTP et HTTPS', None))
    option_139789497413984 = DomainnameOption(**{'multi': True, 'name': 'revprox_domainname', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Nom de domaine ou IP \xe0 rediriger', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497413712}]})
    option_139789497413984.impl_set_information("help", u"""Exemple : pour rediriger \"http://domaine/\" saisir \"domaine\"""")
    option_139789497414256 = DomainnameOption(**{'type_': 'domainname', 'name': 'web_url', 'doc': u'Nom de domaine des applications web (sans http://)', 'allow_ip': False, 'multi': False, 'properties': ('auto_freeze', 'mandatory', 'basic')})
    option_139789497414256.impl_set_information("help", u"""Nom de domaine de votre établissement ex : etablissement.ac-acad.fr""")
    option_139789497414256.impl_add_consistency("not_equal", option_139789497890480, **{'warnings_only': True})
    option_139789497414528 = DomainnameOption(**{'multi': False, 'name': 'eolesso_adresse', 'allow_ip': False, 'type_': 'domainname', 'doc': u"Nom de domaine du serveur d'authentification SSO", 'properties': ('normal',), 'callback': eosfunc.calc_val_first_value, 'callback_params': Params(args=(ParamOption(option_139789497414256, True), ParamOption(option_139789497891024, False)), kwargs={})})
    option_139789497414528.impl_set_information("help", u"""Nom de domaine du serveur d'authentification, ne pas utiliser d'adresse IP ou localhost""")
    option_139789497414800 = DomainnameOption(**{'multi': True, 'name': 'ssl_subjectaltname', 'allow_ip': True, 'allow_without_dot': True, 'type_': 'domainname', 'doc': u'Nom DNS/IP alternatif du serveur', 'properties': ('expert',), 'callback': eosfunc.calc_multi_val, 'callback_params': Params(args=(ParamOption(option_139789497538768, False), ParamOption(option_139789497891024, False), ParamOption(option_139789497560336, True), ParamOption(option_139789497413984, True), ParamOption(option_139789497414528, True), ParamOption(option_139789497414256, True)), kwargs={}), "requires": [{'expected': u'letsencrypt', 'action': 'frozen', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'hidden', 'inverse': False, "option": option_139789497939360}, {'expected': u'letsencrypt', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497939360}]})
    option_139789497414800.impl_set_information("separator", ('Nom Alternatif de la machine (SubjectAltName)', None))
    option_139789497415072 = DomainnameOption(**{'type_': 'domainname', 'name': 'le_extra_names', 'doc': u'Nom de domaines suppl\xe9mentaires', 'allow_ip': False, 'multi': True, 'properties': ('expert',), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497415072.impl_set_information("separator", (u'Certificats suppl\xe9mentaires \xe0 demander', None))
    option_139789497415072.impl_set_information("help", u"""Liste des noms de domaine pour les certificats supplémentaires a demander lors de la requête""")
    option_139789497415344 = FilenameOption(**{'multi': False, 'name': 'le_config_dir', 'default': u'/etc/ssl/letsencrypt/conf', 'doc': u"R\xe9pertoire de configuration du client Let's Encrypt", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497415344.impl_set_information("separator", (u"Param\xe8tres du client Let's Encrypt", None))
    option_139789497415344.impl_set_information("help", u"""Répertoire de configuration du client ACME (Let's Encrypt)""")
    option_139789497415616 = FilenameOption(**{'multi': False, 'name': 'le_work_dir', 'default': u'/tmp/letsencrypt/work', 'doc': u"R\xe9pertoire de travail du client Let's Encrypt", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497415616.impl_set_information("help", u"""Répertoire de travail du client ACME""")
    option_139789497415888 = FilenameOption(**{'multi': False, 'name': 'le_logs_dir', 'default': u'/var/log/letsencrypt/', 'doc': u"R\xe9pertoire de journalisation du client Let's Encrypt", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497415888.impl_set_information("help", u"""Répertoire de journalisation du client ACME""")
    option_139789497416160 = UnicodeOption(**{'doc': u"Adresse du serveur Let's Encrypt", 'multi': False, 'properties': ('expert',), 'name': 'le_server_addr', "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497416160.impl_set_information("help", u"""Adresse du serveur ACME utilisé pour la génération automatique des certificats""")
    option_139789497416432 = PortOption(**{'multi': False, 'name': 'le_server_port', 'allow_private': True, 'doc': u"Port d'\xe9coute du serveur Let's Encrypt", 'properties': ('expert',), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497416432.impl_set_information("help", u"""Port d'écoute du serveur ACME utilisé pour la génération automatique des certificats""")
    option_139789497416704 = ChoiceOption(**{'multi': False, 'values': (u'webroot', u'standalone'), 'name': 'le_client_mode', 'default': u'webroot', 'doc': u"Mode de fonctionnement du client Let's Encrypt", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497416704.impl_set_information("help", u"""Mode de fonctionnement du client ACME pour la demande de certificat, (standalone ou webroot)""")
    option_139789497416976 = PortOption(**{'multi': False, 'name': 'le_http_01_port', 'allow_private': True, 'default': '80', 'doc': u"Port d'\xe9coute pour la requ\xeate http-01", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497416976.impl_set_information("help", u"""Port d'écoute à ouvrir pour la vérification http 01 du serveur ACME lors de la demande de certificat""")
    option_139789497417248 = PortOption(**{'multi': False, 'name': 'le_tls_sni_port', 'allow_private': True, 'default': '443', 'doc': u"Port d'\xe9coute pour la requ\xeat TLS-SNI", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497417248.impl_set_information("help", u"""Port d'écoute à ouvrir pour la vérification TLS-SNI du serveur ACME lors de la demande de certificat""")
    option_139789497426000 = FilenameOption(**{'multi': False, 'name': 'le_ssl_cert', 'doc': u'Chemin du certificat SSL par d\xe9faut', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.concat, 'callback_params': Params(args=(ParamOption(option_139789497415344, False), ParamValue("/live/"), ParamOption(option_139789497891024, False), ParamValue("/cert.pem")), kwargs={}), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497426272 = FilenameOption(**{'multi': False, 'name': 'le_ssl_key', 'doc': u'Chemin de l\xe0 cl\xe9 priv\xe9e SSL par d\xe9faut', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.concat, 'callback_params': Params(args=(ParamOption(option_139789497415344, False), ParamValue("/live/"), ParamOption(option_139789497891024, False), ParamValue("/privkey.pem")), kwargs={}), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497426544 = FilenameOption(**{'multi': False, 'name': 'le_ssl_pem', 'doc': u'Chemin de la chaine compl\xe8te SSL par d\xe9faut', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.concat, 'callback_params': Params(args=(ParamOption(option_139789497415344, False), ParamValue("/live/"), ParamOption(option_139789497891024, False), ParamValue("/fullchain.pem")), kwargs={}), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789497434192 = OptionDescription(children=[option_139789497395408, option_139789497395680, option_139789497395952, option_139789497396224, option_139789497396496, option_139789497396768, option_139789497414800, option_139789497415072, option_139789497415344, option_139789497415616, option_139789497415888, option_139789497416160, option_139789497416432, option_139789497416704, option_139789497416976, option_139789497417248, option_139789497426000, option_139789497426272, option_139789497426544], **{'doc': u'Certificats SSL', 'properties': ('expert',), 'name': 'certificats_ssl'})
    option_139789497434192.impl_set_information("icon", u"""lock""")
    option_139789497426816 = UnicodeOption(**{'doc': u'Libell\xe9 du d\xe9p\xf4t', 'multi': True, 'properties': ('expert',), 'name': 'additional_repository_name'})
    option_139789497426816.impl_set_information("separator", (u'Configuration des d\xe9p\xf4ts tiers', None))
    option_139789497427088 = UnicodeOption(**{'multi': True, 'name': 'additional_repository_source', 'doc': u'D\xe9claration du d\xe9p\xf4t', 'properties': ('mandatory', 'expert'), "validator": eosfunc.valid_regexp, 'validator_params': Params(args=(ParamValue(u"^((deb)|(deb-src)) (\[ .* \] )?(((http)|(https)|(ftp)|(ftps))://\S+) (\S+)(( \S+)+)?"),), kwargs={'err_msg': ParamValue(u"La déclaration d'un dépôt doit respecter le format one-line-style (man 5 sources.list)")})})
    option_139789497427088.impl_set_information("help", u"""Déclaration du dépôt dans le style one-line attendu pour les fichiers .list : type options uri suite components (deb|deb-src [ option1=value1 option2=value2 ] uri suite [component1] [component2] [...])""")
    option_139789497427360 = ChoiceOption(**{'multi': True, 'values': (u'serveur de cl\xe9s', u'URL de la cl\xe9'), 'name': 'additional_repository_key_type', 'default_multi': u'serveur de cl\xe9s', 'doc': u'M\xe9thode de r\xe9cup\xe9ration de la cl\xe9 publique du d\xe9p\xf4t', 'properties': ('mandatory', 'expert')})
    option_139789497427632 = URLOption(**{'multi': True, 'name': 'additional_repository_key_url', 'doc': u'URL de la cl\xe9', 'allow_ip': True, 'allow_without_dot': True, 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'serveur de cl\xe9s', 'action': 'disabled', 'inverse': False, "option": option_139789497427360}]})
    option_139789497427904 = UnicodeOption(**{'doc': u'URL du serveur de cl\xe9s', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'additional_repository_key_signserver', "requires": [{'expected': u'URL de la cl\xe9', 'action': 'disabled', 'inverse': False, "option": option_139789497427360}]})
    option_139789497428176 = UnicodeOption(**{'doc': u'Empreinte de la cl\xe9', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'additional_repository_key_fingerprint', "requires": [{'expected': u'URL de la cl\xe9', 'action': 'disabled', 'inverse': False, "option": option_139789497427360}]})
    option_139789497434512 = Leadership(children=[option_139789497426816, option_139789497427088, option_139789497427360, option_139789497427632, option_139789497427904, option_139789497428176], **{'doc': u'Libell\xe9 du d\xe9p\xf4t', 'properties': ('expert',), 'name': 'additional_repository_name'})
    option_139789497434832 = OptionDescription(children=[option_139789497434512], **{'doc': u'D\xe9p\xf4t tiers', 'properties': ('expert',), 'name': 'depot_tiers'})
    option_139789497434832.impl_set_information("help", u"""Ajout de nouveaux paquets absents des dépôts officiels""")
    option_139789497434832.impl_set_information("icon", u"""debian""")
    option_139789497428448 = PortOption(**{'multi': False, 'name': 'apt_cacher_port', 'allow_private': True, 'default': '3142', 'doc': u'Port d\u2019\xe9coute du service apt-cacher-ng', 'properties': ('mandatory', 'expert')})
    option_139789497435152 = OptionDescription(children=[option_139789497428448], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497954928}], **{'doc': u'Apt-cacher', 'properties': ('expert',), 'name': 'apt_cacher'})
    option_139789497435152.impl_set_information("help", u"""Paramétrage avancé du service de mise en cache des paquets""")
    option_139789497435152.impl_set_information("icon", u"""debian""")
    option_139789497428720 = ChoiceOption(**{'multi': False, 'values': (u'mode_lb', u'mode_fo'), 'name': 'ag_mode', 'default': u'mode_lb', 'doc': u'Mode load balancing ou fail-over', 'properties': ('mandatory', 'normal')})
    option_139789497428720.impl_set_information("separator", (u"Mode d'agr\xe9gation", None))
    option_139789497428992 = IPOption(**{'doc': u'Destination forc\xe9e sur le lien 1', 'multi': True, 'allow_reserved': True, 'properties': ('normal',), 'name': 'ag_force_eth0'})
    option_139789497428992.impl_set_information("separator", ('Lien 1', None))
    option_139789497429264 = NetmaskOption(**{'doc': u'Masque pour destination forc\xe9e sur le lien 1', 'multi': True, 'properties': ('normal',), 'name': 'ag_force_netmask_eth0'})
    option_139789497435472 = Leadership(children=[option_139789497428992, option_139789497429264], **{'doc': u'Destination forc\xe9e sur le lien 1', 'properties': ('normal',), 'name': 'ag_force_eth0'})
    option_139789497429536 = IPOption(**{'doc': u'Adresse du DNS sur le lien 1', 'multi': True, 'allow_reserved': True, 'properties': ('mandatory', 'basic'), 'name': 'ag_dns_eth0'})
    option_139789497442384 = ChoiceOption(**{'multi': False, 'values': (u'actif', u'passif'), 'name': 'ag_fo_etat_eth0', 'default': u'actif', 'doc': u'Etat actif ou passif pour fail-over lien 1', 'properties': ('normal',), "requires": [{'expected': u'mode_lb', 'action': 'disabled', 'inverse': False, "option": option_139789497428720}]})
    option_139789497442656 = IntOption(**{'doc': u'D\xe9bit mesur\xe9 sur le lien 1 (entier en Mbps)', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'ag_weight_eth0', "requires": [{'expected': u'mode_fo', 'action': 'disabled', 'inverse': False, "option": option_139789497428720}]})
    option_139789497442928 = IPOption(**{'doc': u'Destination forc\xe9e sur le lien 2', 'multi': True, 'allow_reserved': True, 'properties': ('normal',), 'name': 'ag_force_eth0_0'})
    option_139789497442928.impl_set_information("separator", ('Lien 2', None))
    option_139789497443200 = NetmaskOption(**{'doc': u'Masque pour destination forc\xe9e sur le lien 2', 'multi': True, 'properties': ('normal',), 'name': 'ag_force_netmask_eth0_0'})
    option_139789497435792 = Leadership(children=[option_139789497442928, option_139789497443200], **{'doc': u'Destination forc\xe9e sur le lien 2', 'properties': ('normal',), 'name': 'ag_force_eth0_0'})
    option_139789497443472 = IPOption(**{'doc': u'Adresse du DNS sur le lien 2', 'multi': True, 'allow_reserved': True, 'properties': ('mandatory', 'basic'), 'name': 'ag_dns_eth0_0'})
    option_139789497443744 = ChoiceOption(**{'multi': False, 'values': (u'actif', u'passif'), 'name': 'ag_fo_etat_eth0_0', 'default': u'actif', 'doc': u'Etat actif ou passif pour fail-over lien 2', 'properties': ('normal',), "requires": [{'expected': u'mode_lb', 'action': 'disabled', 'inverse': False, "option": option_139789497428720}]})
    option_139789497444016 = IntOption(**{'doc': u'D\xe9bit mesur\xe9 sur le lien 2 (entier en Mbps)', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'ag_weight_eth0_0', "requires": [{'expected': u'mode_fo', 'action': 'disabled', 'inverse': False, "option": option_139789497428720}]})
    option_139789497444288 = IntOption(**{'default': 10, 'doc': u"D\xe9lai entre les tests d'\xe9tat (en secondes)", 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'ag_pause'})
    option_139789497444288.impl_set_information("separator", ('Divers', None))
    option_139789497444560 = IntOption(**{'default': 1, 'doc': u'Timeout de la requ\xeate DNS (en secondes)', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'ag_timeout'})
    option_139789497444832 = DomainnameOption(**{'multi': True, 'name': 'ag_testdns', 'default': [u'www.google.com'], 'doc': u'Adresse DNS test\xe9e', 'type_': 'domainname', 'allow_ip': False, 'default_multi': u'www.google.com', 'properties': ('mandatory', 'normal')})
    option_139789497445104 = IntOption(**{'default': 4, 'doc': u"Nombre de succ\xe8s avant changement d'\xe9tat", 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'ag_nbsucces'})
    option_139789497445376 = IntOption(**{'default': 1, 'doc': u"Nombre d'\xe9checs avant changement d'\xe9tat", 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'ag_nbechecs'})
    option_139789497445648 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ag_active_mail', 'default': 'non', 'doc': u'Activation des alertes mail', 'properties': ('mandatory', 'normal')})
    option_139789497445648.impl_set_information("separator", ('Alerte mail', None))
    option_139789497445920 = DomainnameOption(**{'multi': False, 'name': 'domaine_messagerie_etab', 'allow_ip': True, 'allow_without_dot': True, 'type_': 'domainname', 'doc': u"Nom de domaine de la messagerie de l'\xe9tablissement (ex : monetab.ac-aca.fr)", 'properties': ('mandatory', 'basic'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789497890752, False),), kwargs={})})
    option_139789497445920.impl_set_information("separator", (u"Serveur d'envoi/r\xe9ception (SMTP)", None))
    option_139789497445920.impl_set_information("help", u"""Nom de domaine de la messagerie de l’établissement, un domaine
            privé est automatiquement créé avec le préfixe « i- » si \"exim_mail_type\" est à \"satellite\".""")
    option_139789497462864 = UnicodeOption(**{'multi': False, 'name': 'system_mail_to', 'doc': u'Adresse \xe9lectronique recevant les courriers \xe9lectroniques \xe0 destination du compte root', 'properties': ('basic',), 'callback': eosfunc.concat, 'callback_params': Params(args=(ParamValue("admin@"), ParamOption(option_139789497445920, False)), kwargs={})})
    option_139789497462864.impl_set_information("help", u"""Adresse utilisée pour la réception des courriers électroniques à l'attention de l'utilisateur root""")
    option_139789497463136 = EmailOption(**{'multi': True, 'name': 'ag_mail_dest', 'doc': u"Adresse mail d'alerte", 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_val, 'callback_params': Params(args=(ParamOption(option_139789497462864, False),), kwargs={}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497445648}]})
    option_139789497436112 = OptionDescription(children=[option_139789497428720, option_139789497435472, option_139789497429536, option_139789497442384, option_139789497442656, option_139789497435792, option_139789497443472, option_139789497443744, option_139789497444016, option_139789497444288, option_139789497444560, option_139789497444832, option_139789497445104, option_139789497445376, option_139789497445648, option_139789497463136], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497574000}], **{'doc': u'agr\xe9gation', 'properties': ('basic',), 'name': 'agregation'})
    option_139789497436112.impl_set_information("help", u"""Paramétrage de la répartition de charge ou de la haute disponibilité pour les sorties Internet""")
    option_139789497436112.impl_set_information("icon", None)
    option_139789497463408 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'edb_local', 'default': 'oui', 'doc': u'Le serveur par d\xe9faut est local', 'properties': ('mandatory', 'expert')})
    option_139789497463680 = UnicodeOption(**{'doc': u'Adresse du serveur de base de donn\xe9es', 'multi': False, 'properties': ('expert',), 'name': 'edb_host', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497463408}]})
    option_139789497463680.impl_set_information("help", u"""Adresse du serveur de base de données par défaut de EoleDB, cette valeur sera utilisée pour toutes les applications qui ne définiront pas elles-mêmes un serveur de base de données""")
    option_139789497463952 = IntOption(**{'doc': u'Port du serveur de base de donn\xe9es', 'multi': False, 'properties': ('expert',), 'name': 'edb_port', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497463408}]})
    option_139789497463952.impl_set_information("help", u"""Port d'écoute du serveur de base de données par défaut de EoleDB""")
    option_139789497464224 = UnicodeOption(**{'doc': u"Compte d'administration du serveur de base de donn\xe9es", 'multi': False, 'properties': ('expert',), 'name': 'edb_root', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497463408}]})
    option_139789497464224.impl_set_information("help", u"""Identifiant du compte à utiliser pour administrer la base de données par défaut de EoleDB""")
    option_139789497464496 = UnicodeOption(**{'doc': u"Fichier contenant le mot de passe associ\xe9 au compte d'administration", 'multi': False, 'properties': ('expert',), 'name': 'edb_rootpw', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497463408}]})
    option_139789497464496.impl_set_information("help", u"""Chemin du fichier contenant le mot de passe de l'administrateur de la base de données par défaut de EoleDB""")
    option_139789497464768 = UnicodeOption(**{'doc': u'Machines qui peuvent utiliser le serveur de BDD', 'multi': True, 'properties': ('expert',), 'name': 'edb_allow_hosts', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497463408}]})
    option_139789497464768.impl_set_information("help", u"""Adresses IP avec lesquelles les utilisateurs de la base de données sont susceptibles de se connecter""")
    option_139789497436432 = OptionDescription(children=[option_139789497463408, option_139789497463680, option_139789497463952, option_139789497464224, option_139789497464496, option_139789497464768], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955200}], **{'doc': u'EoleDB', 'properties': ('expert',), 'name': 'eoledb'})
    option_139789497436432.impl_set_information("help", u"""Outil de gestion automatique des bases de données, simple et pratique""")
    option_139789497436432.impl_set_information("icon", u"""database""")
    option_139789497465040 = ChoiceOption(**{'multi': False, 'values': (u'3', u'4', u'5', u'6', u'7', u'8', u'9', u'10', u'11', u'12'), 'name': 'smb_min_password_length', 'default': u'5', 'doc': u'Longueur minimale des mots de passe', 'properties': ('mandatory', 'normal')})
    option_139789497465040.impl_set_information("separator", ('Politique de mot de passe pour les utilisateurs LDAP', None))
    option_139789497465584 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2', u'3', u'4'), 'name': 'smb_min_password_class', 'default': u'2', 'doc': u'Nombre minimum de classes de caract\xe8res', 'properties': ('mandatory', 'normal')})
    option_139789497465584.impl_set_information("help", u"""Les classes de caractères sont : chiffres, lettres minuscules, lettres majuscules, et autres caractères""")
    option_139789497436752 = OptionDescription(children=[option_139789497465040, option_139789497465584], **{'doc': u'Mots de passe', 'properties': ('normal',), 'name': 'mots_de_passe'})
    option_139789497436752.impl_set_information("help", u"""Politique générale pour les utilisateurs des applications web et de Samba""")
    option_139789497436752.impl_set_information("icon", u"""keyboard""")
    option_139789497465312 = IntOption(**{'default': 10, 'doc': u'Taille maximum pour un fichier \xe0 scanner (en Mo)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'clam_max_file_size'})
    option_139789497465312.impl_set_information("separator", ('ClamAV', None))
    option_139789497465856 = IntOption(**{'default': 50, 'doc': u'Quantit\xe9 de donn\xe9es maximum \xe0 scanner pour une archive (en Mo)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'clam_max_scan_size'})
    option_139789497466128 = IntOption(**{'default': 16, 'doc': u'Profondeur maximale pour le scan des archives', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'clam_max_recursion'})
    option_139789497466400 = IntOption(**{'default': 15, 'doc': u'Profondeur maximale pour le scan des r\xe9pertoires', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'clam_max_directory_recursion'})
    option_139789497475152 = IntOption(**{'default': 5000, 'doc': u'Nombre maximum de fichiers \xe0 scanner dans une archive', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'clam_max_files'})
    option_139789497475424 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'clam_exit_on_oom', 'default': 'no', 'doc': u'Arr\xeater le d\xe9mon en cas de surcharge m\xe9moire', 'properties': ('mandatory', 'expert')})
    option_139789497475696 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'clam_detect_pua', 'default': 'no', 'doc': u'D\xe9tection des applications ind\xe9sirables', 'properties': ('mandatory', 'expert')})
    option_139789497475968 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'clam_scan_elf', 'default': 'no', 'doc': u'Scan du contenu des fichiers ELF', 'properties': ('mandatory', 'expert')})
    option_139789497475968.impl_set_information("help", u"""ELF (Executable and Linkable Format) est un format de fichier binaire utilisé pour l'enregistrement de code compilé""")
    option_139789497476240 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'clam_scan_pdf', 'default': 'yes', 'doc': u'Scan du contenu des fichiers PDF', 'properties': ('mandatory', 'expert')})
    option_139789497476512 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_clamav', 'default': 'oui', 'doc': u"Activer l'anti-virus sur la messagerie", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}]})
    option_139789497476512.impl_set_information("help", u"""Gestion de l'antivirus sur la messagerie""")
    option_139789497476784 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'clam_scan_mail', 'default': 'no', 'doc': u'Scan des fichiers courriels', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497476512}]})
    option_139789497477056 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'clam_broken_exe', 'default': 'no', 'doc': u'D\xe9tection des fichiers ex\xe9cutables corrompus', 'properties': ('mandatory', 'expert')})
    option_139789497477328 = DomainnameOption(**{'multi': False, 'name': 'clam_dns_database_info', 'allow_without_dot': True, 'default': u'current.cvd.clamav.net', 'doc': u'Nom de domaine du serveur DNS de mise \xe0 jour', 'type_': 'domainname', 'allow_ip': True, 'properties': ('mandatory', 'expert')})
    option_139789497477328.impl_set_information("separator", ('Freshclam', None))
    option_139789497477600 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'clam_forcer_mirror_database', 'default': 'non', 'doc': u'Forcer un serveur de mise \xe0 jour freshclam', 'properties': ('mandatory', 'expert')})
    option_139789497477872 = DomainnameOption(**{'multi': True, 'name': 'clam_mirror_database', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Nom de domaine du serveur de mise \xe0 jour', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497477600}]})
    option_139789497478144 = DomainnameOption(**{'multi': False, 'name': 'clam_iana', 'default': u'fr', 'allow_ip': False, 'type_': 'hostname', 'doc': u'Code IANA pour la mise \xe0 jour de la base de signature', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497477600}]})
    option_139789497478416 = IntOption(**{'default': 5, 'doc': u'Nombre de tentatives de mise \xe0 jour par miroir', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'clam_max_attempts'})
    option_139789497478688 = IntOption(**{'default': 24, 'doc': u'Nombre de mises \xe0 jour quotidiennes', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'clam_checks'})
    option_139789497483344 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'clam_forcer_daemon', 'default': 'non', 'doc': u"Forcer l'activation du d\xe9mon clam sur le serveur", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789497483344.impl_set_information("separator", ('Services utilisant ClamAV', None))
    option_139789497483344.impl_set_information("help", u"""Permet d'activer le service ClamAV même s'il n'y a pas de scan temps réel activé""")
    option_139789497483616 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'smb_vscan', 'default': 'non', 'doc': u"Activer l'anti-virus temps r\xe9el sur SMB", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497483888 = IntOption(**{'multi': False, 'name': 'clam_purge', 'default': 20, 'doc': u'Dur\xe9e de conservation des fichiers en quarantaine (en jours)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497483616}]})
    option_139789497484160 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dansguardian_clam', 'default': 'non', 'doc': u"Activer l'anti-virus sur le proxy", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789497484432 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ftp_scan', 'default': 'oui', 'doc': u"Activer l'anti-virus temps r\xe9el sur FTP", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}]})
    option_139789497437072 = OptionDescription(children=[option_139789497465312, option_139789497465856, option_139789497466128, option_139789497466400, option_139789497475152, option_139789497475424, option_139789497475696, option_139789497475968, option_139789497476240, option_139789497476784, option_139789497477056, option_139789497477328, option_139789497477600, option_139789497477872, option_139789497478144, option_139789497478416, option_139789497478688, option_139789497483344, option_139789497483616, option_139789497483888, option_139789497484160, option_139789497484432, option_139789497476512], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}], **{'doc': u'ClamAV', 'properties': ('basic',), 'name': 'clamav'})
    option_139789497437072.impl_set_information("icon", u"""shield""")
    option_139789497484704 = DomainnameOption(**{'multi': False, 'name': 'adresse_ip_ldap', 'allow_ip': True, 'allow_without_dot': True, 'type_': 'domainname', 'doc': u'Adresse IP ou nom DNS du serveur LDAP', 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("local"),), kwargs={'condition_1': ParamOption(option_139789497955744, False), 'default_mismatch': ParamValue("None"), 'match': ParamOption(option_139789497852256, False)}), "requires": [{'expected': u'local', 'action': 'frozen', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'hidden', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'frozen', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'hidden', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497955744}]})
    option_139789497484704.impl_set_information("help", u"""Nom de domaine ou l'adresse IP de l'annuaire distant""")
    option_139789497484976 = UnicodeOption(**{'multi': False, 'name': 'ldap_base_dn', 'default': u'o=gouv,c=fr', 'doc': u"Base DN de l'annuaire", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), "validator": eosfunc.valid_regexp, 'validator_params': Params(args=(ParamValue(u"^o=|^dc=|^ou="),), kwargs={'err_msg': ParamValue(u"La racine doit débuter par une organisation (o=), une composante du domaine (dc=) ou une unité organisationnelle (ou=)")})})
    option_139789497485248 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ldap_tls', 'default': 'non', 'doc': u'Activer le support de TLS', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'frozen', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'hidden', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497955744}]})
    option_139789497485520 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ldap_nss', 'default': 'oui', 'doc': u'Ajouter les utilisateurs LDAP aux utilisateurs locaux', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'frozen', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'hidden', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497955744}]})
    option_139789497485792 = PortOption(**{'multi': False, 'name': 'ldap_port', 'allow_private': True, 'default': '389', 'doc': u'Port du serveur LDAP', 'properties': ('mandatory', 'normal')})
    option_139789497486064 = UnicodeOption(**{'multi': False, 'name': 'ldap_reader', 'doc': u'Utilisateur de lecture des comptes LDAP', 'properties': ('normal',), 'callback': eosfunc.concat, 'callback_params': Params(args=(ParamValue("cn=reader,"), ParamOption(option_139789497484976, False)), kwargs={}), "requires": [{'expected': u'local', 'action': 'frozen', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'hidden', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497955744}]})
    option_139789497486336 = FilenameOption(**{'multi': False, 'name': 'ldap_reader_passfile', 'default': u'/root/.reader', 'doc': u"Fichier de mot de passe de l'utilisateur de lecture", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'frozen', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'hidden', 'inverse': False, "option": option_139789497955744}, {'expected': u'local', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497955744}]})
    option_139789497486608 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_admin_passfile', 'default': 'non', 'doc': u'D\xe9finir le mot de passe admin de LDAP dans un fichier', 'properties': ('mandatory', 'normal')})
    option_139789497486880 = FilenameOption(**{'default': u'/root/.writer', 'doc': u"Fichier de mot de passe de l'utilisateur admin", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_admin_passfile'})
    option_139789497507920 = UnicodeOption(**{'default': u'uid', 'doc': u'Attribut de recherche des utilisateurs', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_match_attribute'})
    option_139789497508192 = UnicodeOption(**{'default': u'objectClass=person', 'doc': u"Filtre d'utilisateurs", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_filter_user'})
    option_139789497508464 = UnicodeOption(**{'default': u'objectClass=posixGroup', 'doc': u'Filtre de groupes', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_filter_group'})
    option_139789497508736 = UnicodeOption(**{'doc': u"DN racine de l'arbre utilisateurs", 'multi': False, 'properties': ('expert',), 'name': 'ldap_dntree_user'})
    option_139789497509008 = UnicodeOption(**{'doc': u"DN racine de l'arbre groupes", 'multi': False, 'properties': ('expert',), 'name': 'ldap_dntree_group'})
    option_139789497509280 = UnicodeOption(**{'default': u'displayName', 'doc': u"Champ 'nom d'affichage' de l'utilisateur", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_fill_displayname'})
    option_139789497509552 = UnicodeOption(**{'default': u'mail', 'doc': u"Champ 'mail' de l'utilisateur", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_fill_mail'})
    option_139789497509824 = UnicodeOption(**{'default': u'maildir', 'doc': u"Champ 'maildir' de l'utilisateur", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_fill_maildir'})
    option_139789497510096 = UnicodeOption(**{'doc': u"Champ 'fonction' de l'utilisateur", 'multi': False, 'properties': ('expert',), 'name': 'ldap_fill_fonction'})
    option_139789497510368 = UnicodeOption(**{'doc': u"Champ 'categorie' de l'utilisateur", 'multi': False, 'properties': ('expert',), 'name': 'ldap_fill_categorie'})
    option_139789497510640 = UnicodeOption(**{'doc': u"Champ 'rne' de l'utilisateur", 'multi': False, 'properties': ('expert',), 'name': 'ldap_fill_rne'})
    option_139789497510912 = UnicodeOption(**{'doc': u"Champ 'fredurne' de l'utilisateur", 'multi': False, 'properties': ('expert',), 'name': 'ldap_fill_fredurne'})
    option_139789497511184 = UnicodeOption(**{'default': u'cn', 'doc': u"Champ 'nom d'affichage' du groupe", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_fill_displaygroup'})
    option_139789497437392 = OptionDescription(children=[option_139789497484704, option_139789497484976, option_139789497485248, option_139789497485520, option_139789497485792, option_139789497486064, option_139789497486336, option_139789497486608, option_139789497486880, option_139789497507920, option_139789497508192, option_139789497508464, option_139789497508736, option_139789497509008, option_139789497509280, option_139789497509552, option_139789497509824, option_139789497510096, option_139789497510368, option_139789497510640, option_139789497510912, option_139789497511184], **{'doc': u'Annuaire', 'properties': ('normal',), 'name': 'annuaire'})
    option_139789497437392.impl_set_information("help", u"""Configuration générale de l'annuaire""")
    option_139789497437392.impl_set_information("icon", u"""book""")
    option_139789497511456 = DomainnameOption(**{'type_': 'domainname', 'name': 'global_nom_domaine_dhcp', 'doc': u'Nom de domaine \xe0 renvoyer aux clients DHCP', 'allow_ip': False, 'multi': False, 'properties': ('expert',)})
    option_139789497511456.impl_set_information("separator", (u'Param\xe8tres globaux (peuvent \xeatre surcharg\xe9s pour un r\xe9seau sp\xe9cifique)', None))
    option_139789497511456.impl_set_information("help", u"""Nom de domaine à renvoyer aux clients DHCP""")
    option_139789497516112 = IPOption(**{'doc': u'Adresse IP du DNS \xe0 renvoyer aux clients DHCP', 'multi': False, 'allow_reserved': True, 'properties': ('expert',), 'name': 'global_adresse_ip_dns_dhcp'})
    option_139789497516112.impl_set_information("help", u"""Adresse IP du DNS à renvoyer aux clients DHCP""")
    option_139789497516384 = IPOption(**{'doc': u'Adresse IP du DNS secondaire \xe0 renvoyer aux clients DHCP', 'multi': False, 'allow_reserved': True, 'properties': ('expert',), 'name': 'global_adresse_ip_dns_secondaire_dhcp'})
    option_139789497516656 = IPOption(**{'doc': u'Adresse IP du serveur primaire Wins \xe0 renvoyer aux clients', 'multi': False, 'allow_reserved': True, 'properties': ('expert',), 'name': 'global_adresse_ip_wins_primaire_dhcp'})
    option_139789497516656.impl_set_information("help", u"""Adresse IP du serveur primaire WINS à renvoyer aux clients DHCP""")
    option_139789497516928 = IPOption(**{'doc': u'Adresse IP du serveurs secondaire Wins \xe0 renvoyer aux clients', 'multi': False, 'allow_reserved': True, 'properties': ('expert',), 'name': 'global_adresse_ip_wins_secondaire_dhcp'})
    option_139789497516928.impl_set_information("help", u"""Adresse IP du serveur secondaire WINS à renvoyer aux clients DHCP""")
    option_139789497517200 = IPOption(**{'doc': u'Adresse IP du serveur NTP \xe0 renvoyer aux clients', 'multi': False, 'allow_reserved': True, 'properties': ('expert',), 'name': 'global_adresse_ip_ntp_dhcp'})
    option_139789497517200.impl_set_information("help", u"""Adresse IP du serveur NTP à renvoyer aux clients DHCP""")
    option_139789497517472 = IntOption(**{'default': 36000, 'doc': u'Temps du bail par d\xe9faut (sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'global_dhcp_lease_default'})
    option_139789497517472.impl_set_information("help", u"""Temps du bail par défaut, en secondes, offert à un client n'ayant pas fait la demande d'une durée spécifique""")
    option_139789497517744 = IntOption(**{'default': 144000, 'doc': u'Temps maximum du bail (sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'global_dhcp_lease_max'})
    option_139789497517744.impl_set_information("help", u"""Temps du bail maximal, en secondes, accordé à un client ayant fait la demande d'une durée spécifique""")
    option_139789497518016 = DomainnameOption(**{'multi': False, 'name': 'global_domain_wpad_dhcp', 'allow_ip': False, 'type_': 'domainname', 'doc': u'Nom de domaine du serveur WPAD', 'properties': ('expert',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'default_match': ParamValue("None"), 'condition_1': ParamOption(option_139789497972128, False), 'default_mismatch': ParamValue("None"), 'mismatch': ParamOption(option_139789497890752, False)})})
    option_139789497518016.impl_set_information("help", u"""Nom de domaine du serveur WPAD à renvoyer aux clients du sous-réseau""")
    option_139789497518288 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_dhcp_failover', 'default': 'non', 'doc': u'Activer la continuit\xe9 de service', 'properties': ('mandatory', 'normal')})
    option_139789497518288.impl_set_information("separator", (u'Configurer la continuit\xe9 de service', None))
    option_139789497518288.impl_set_information("help", u"""Affiche les paramètres pour la configuration de la continuité de service (failover) de DHCP.""")
    option_139789497518560 = UnicodeOption(**{'doc': u'Nom de la grappe', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'dhcp_failover_name', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789497518560.impl_set_information("help", u"""Nom utilisé pour identifier une configuration de continuité de service et l’associer aux pools d’adresses.""")
    option_139789497518832 = ChoiceOption(**{'multi': False, 'values': (u'primary', u'secondary'), 'name': 'dhcp_failover_rank', 'default': u'primary', 'doc': u'Rang du serveur dans la grappe', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789497518832.impl_set_information("help", u"""Rôle joué par le serveur dans le mécanisme de continuité de service (primaire ou secondaire).""")
    option_139789497519104 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'dhcp_failover_local_address', 'doc': u'Adresse IP du serveur DHCP local, en \xe9coute du serveur pair', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789497519104.impl_set_information("help", u"""Adresse IP du serveur local failover, en écoute du serveur pair.""")
    option_139789497519376 = IntOption(**{'multi': False, 'name': 'dhcp_failover_port', 'doc': u'Port de communication du serveur DHCP local, en \xe9coute du serveur pair', 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("primary"),), kwargs={'mismatch': ParamValue(847), 'condition_1': ParamOption(option_139789497518832, False), 'match': ParamValue(647)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789497519376.impl_set_information("help", u"""Port utilisé pour recevoir les informations du serveur pair.""")
    option_139789497519648 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'dhcp_failover_peer_address', 'doc': u'Adresse IP du serveur pair', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789497519648.impl_set_information("help", u"""Adresse IP du serveur pair.""")
    option_139789497008208 = IntOption(**{'multi': False, 'name': 'dhcp_failover_peer_port', 'doc': u'Port de communication du serveur pair', 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("secondary"),), kwargs={'mismatch': ParamValue(847), 'condition_1': ParamOption(option_139789497518832, False), 'match': ParamValue(647)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789497008208.impl_set_information("help", u"""Port utilisé pour contacter le serveur pair (peut être identique au port d’écoute du service hébergé).""")
    option_139789497008480 = IntOption(**{'multi': False, 'name': 'dhcp_failover_response_delay', 'default': 60, 'doc': u'D\xe9lai maximal de r\xe9ponse', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789497008480.impl_set_information("help", u"""Seuil de temps (en secondes) à partir duquel un serveur pair ne répondant pas aux requêtes est supposé défaillant.""")
    option_139789497008752 = IntOption(**{'multi': False, 'name': 'dhcp_failover_unacked_updates', 'default': 10, 'doc': u'Nombre de messages BNDUPD', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789497008752.impl_set_information("help", u"""Nombre limite de messages BNDUPD sans réponse BNDACK tolérés.""")
    option_139789497009024 = IntOption(**{'multi': False, 'name': 'dhcp_failover_balance_seconds', 'default': 5, 'doc': u'D\xe9lai avant passage de t\xe9moin', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789497009024.impl_set_information("help", u"""Temps limite (en secondes) après lequel un pair prend le pas sur l’autre si ce dernier ne répond plus aux requêtes des clients mais seulement aux messages d’appairage (outrepassement des paramètres de répartition de charge).""")
    option_139789497009296 = IntOption(**{'multi': False, 'name': 'dhcp_failover_mclt', 'default': 3600, 'doc': u'Temps maximum de d\xe9l\xe9gation', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'secondary', 'action': 'disabled', 'inverse': False, "option": option_139789497518832}]})
    option_139789497009296.impl_set_information("help", u"""Durée maximale (en secondes) pendant laquelle un serveur pair peut renouveler un bail sans concertation avec l’autre serveur (Maximum Client Lead Time).""")
    option_139789497009568 = IntOption(**{'multi': False, 'name': 'dhcp_failover_split', 'default': 128, 'doc': u'Seuil de prise en charge par le serveur primaire', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'secondary', 'action': 'disabled', 'inverse': False, "option": option_139789497518832}]})
    option_139789497009568.impl_set_information("help", u"""Valeur comprise entre 0 et 256 inclus faisant office de seuil et permettant d’associer les requêtes à l’un ou à l’autre serveur  pair (répartition de charge).""")
    option_139789497009840 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_omapi', 'default': 'non', 'doc': u'Activer OMAPI', 'properties': ('mandatory', 'expert')})
    option_139789497009840.impl_set_information("separator", ("Support de l'API OMAPI", None))
    option_139789497009840.impl_set_information("help", u"""Afficher les variables utilisées pour la configuration de OMAPI""")
    option_139789497010112 = IntOption(**{'multi': False, 'name': 'omapi_port', 'default': 7911, 'doc': u'Port d\u2019\xe9coute pour OMAPI', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497009840}]})
    option_139789497010112.impl_set_information("help", u"""Port à ouvrir et à utiliser pour OMAPI""")
    option_139789497010384 = PasswordOption(**{'callback_params': None, 'multi': False, 'name': 'omapi_secret', 'doc': u'Secret partag\xe9 pour OMAPI', 'properties': ('force_store_value', 'basic'), 'callback': eosfunc.gen_random, "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497009840}]})
    option_139789497010384.impl_set_information("help", u"""Clé secrète partagée à utiliser dans la configuration des différents serveurs utilisant OMAPI""")
    option_139789497010656 = NetworkOption(**{'multi': True, 'name': 'adresse_network_dhcp', 'doc': u'Adresse r\xe9seau de la plage DHCP', 'properties': ('mandatory', 'basic'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497604576, False))})
    option_139789497010656.impl_set_information("separator", (u'D\xe9finition des sous-r\xe9seaux', None))
    option_139789497010656.impl_set_information("help", u"""Adresse réseau du sous-réseau""")
    option_139789497010928 = NetmaskOption(**{'multi': True, 'name': 'adresse_netmask_dhcp', 'doc': u'Masque de sous-r\xe9seau de la plage DHCP', 'properties': ('mandatory', 'basic'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497604304, False))})
    option_139789497010928.impl_set_information("help", u"""Masque de sous-réseau""")
    option_139789497011200 = DomainnameOption(**{'type_': 'hostname', 'name': 'nom_plage_dhcp', 'doc': u'Nom de la plage DHCP', 'allow_ip': False, 'multi': True, 'properties': ('mandatory', 'basic'), "validator": eosfunc.check_name_uniq})
    option_139789497011200.impl_set_information("help", u"""Nom permettant d'identifier facilement la plage DHCP, notamment dans l'interface d'administration""")
    option_139789497011472 = IPOption(**{'doc': u'IP basse de la plage DHCP', 'multi': True, 'allow_reserved': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_basse_dhcp'})
    option_139789497011472.impl_set_information("help", u"""IP basse de la plage d'adresses attribuées dynamiquement pour le sous-réseau""")
    option_139789497011472.impl_add_consistency("in_network", option_139789497010656, option_139789497010928, **{})
    option_139789497011744 = IPOption(**{'doc': u'IP haute de la plage DHCP', 'multi': True, 'allow_reserved': True, 'properties': ('mandatory', 'basic'), 'name': 'ip_haute_dhcp'})
    option_139789497011744.impl_set_information("help", u"""IP haute de la plage d'adresses attribuées dynamiquement pour le sous-réseau""")
    option_139789497011744.impl_add_consistency("in_network", option_139789497010656, option_139789497010928, **{})
    option_139789497024592 = DomainnameOption(**{'multi': True, 'name': 'nom_domaine_dhcp', 'allow_ip': False, 'type_': 'domainname', 'doc': u'Nom de domaine \xe0 renvoyer aux clients DHCP', 'properties': ('basic',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497890752, False))})
    option_139789497024592.impl_set_information("help", u"""Nom de domaine à renvoyer aux clients DHCP du sous-réseau""")
    option_139789497024864 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'adresse_ip_gw_dhcp', 'doc': u'Adresse IP du routeur \xe0 renvoyer aux clients DHCP', 'properties': ('basic',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497604032, False))})
    option_139789497024864.impl_set_information("help", u"""Adresse IP du routeur à renvoyer aux clients DHCP du sous-réseau""")
    option_139789497025136 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'adresse_ip_dns_dhcp', 'doc': u'Adresse IP du DNS \xe0 renvoyer aux clients DHCP', 'properties': ('basic',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497675024, False))})
    option_139789497025136.impl_set_information("help", u"""Adresse IP du DNS à renvoyer aux clients DHCP du sous-réseau""")
    option_139789497025408 = IPOption(**{'doc': u'Adresse IP du DNS secondaire \xe0 renvoyer aux clients DHCP', 'multi': True, 'allow_reserved': True, 'properties': ('expert',), 'name': 'adresse_ip_dns_secondaire_dhcp'})
    option_139789497025408.impl_set_information("help", u"""Adresse IP d'un serveur DNS secondaire à renvoyer aux clients DHCP du sous-réseau""")
    option_139789497025680 = IPOption(**{'doc': u'Adresse IP du serveur primaire Wins \xe0 renvoyer aux clients', 'multi': True, 'allow_reserved': True, 'properties': ('expert',), 'name': 'adresse_ip_wins_primaire_dhcp'})
    option_139789497025680.impl_set_information("help", u"""Adresse IP du serveur primaire WINS à renvoyer aux clients du sous-réseau""")
    option_139789497025952 = IPOption(**{'doc': u'Adresse IP du serveur secondaire Wins \xe0 renvoyer aux clients', 'multi': True, 'allow_reserved': True, 'properties': ('expert',), 'name': 'adresse_ip_wins_secondaire_dhcp'})
    option_139789497025952.impl_set_information("help", u"""Adresse IP du serveur secondaire WINS à renvoyer aux clients du sous-réseau""")
    option_139789497026224 = IPOption(**{'doc': u'Adresse IP du serveur NTP \xe0 renvoyer aux clients', 'multi': True, 'allow_reserved': True, 'properties': ('expert',), 'name': 'adresse_ip_ntp_dhcp'})
    option_139789497026224.impl_set_information("help", u"""Adresse IP du serveur NTP à renvoyer aux clients du sous-réseau""")
    option_139789497026496 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'dhcp_pool_use_failover', 'doc': u'Utiliser la configuration de continuit\xe9 de service pour ce pool', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789497518288, False),), kwargs={})})
    option_139789497026768 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'interdire_hotes_inconnus', 'default_multi': 'non', 'doc': u'Interdire cette zone aux h\xf4tes inconnus', 'properties': ('mandatory', 'expert')})
    option_139789497026768.impl_set_information("help", u"""Le serveur attribuera des adresses uniquement aux stations déclarées""")
    option_139789497027040 = IntOption(**{'doc': u'Temps du bail par d\xe9faut (sec)', 'multi': True, 'properties': ('expert',), 'name': 'dhcp_lease_default'})
    option_139789497027040.impl_set_information("help", u"""Temps du bail par défaut, en secondes, offert à un client du sous-réseau n'ayant pas fait la demande d'une durée spécifique""")
    option_139789497027312 = IntOption(**{'doc': u'Temps maximum du bail (sec)', 'multi': True, 'properties': ('expert',), 'name': 'dhcp_lease_max'})
    option_139789497027312.impl_set_information("help", u"""Temps du bail maximal, en secondes, accordé à un client du sous-réseau ayant fait la demande d'une durée spécifique""")
    option_139789497437712 = Leadership(children=[option_139789497010656, option_139789497010928, option_139789497011200, option_139789497011472, option_139789497011744, option_139789497024592, option_139789497024864, option_139789497025136, option_139789497025408, option_139789497025680, option_139789497025952, option_139789497026224, option_139789497026496, option_139789497026768, option_139789497027040, option_139789497027312], **{'doc': u'Adresse r\xe9seau de la plage DHCP', 'properties': ('basic',), 'name': 'adresse_network_dhcp'})
    option_139789497036880 = OptionDescription(children=[option_139789497511456, option_139789497516112, option_139789497516384, option_139789497516656, option_139789497516928, option_139789497517200, option_139789497517472, option_139789497517744, option_139789497518016, option_139789497518288, option_139789497518560, option_139789497518832, option_139789497519104, option_139789497519376, option_139789497519648, option_139789497008208, option_139789497008480, option_139789497008752, option_139789497009024, option_139789497009296, option_139789497009568, option_139789497009840, option_139789497010112, option_139789497010384, option_139789497437712], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}], **{'doc': u'DHCP', 'properties': ('basic',), 'name': 'dhcp'})
    option_139789497036880.impl_set_information("help", u"""Paramétrage du serveur DHCP local""")
    option_139789497036880.impl_set_information("icon", u"""bolt""")
    option_139789497027584 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_tftp', 'doc': u'Adresse IP du serveur PXE/TFTP', 'properties': ('normal',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497693056, False))})
    option_139789497027856 = FilenameOption(**{'default': u'/var/lib/tftpboot/', 'doc': u'R\xe9pertoire sur le serveur PXE/TFTP', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'repertoire_tftp'})
    option_139789497028128 = FilenameOption(**{'multi': False, 'name': 'chemin_fichier_pxe', 'default': u'/pxelinux.0', 'doc': u'Chemin vers le fichier de boot PXE initial', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}]})
    option_139789497037200 = OptionDescription(children=[option_139789497027584, option_139789497027856, option_139789497028128], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956288}], **{'doc': u'TFTP', 'properties': ('basic',), 'name': 'tftp'})
    option_139789497037200.impl_set_information("help", u"""Configuration du serveur TFTP""")
    option_139789497037200.impl_set_information("icon", u"""cloud""")
    option_139789497045072 = ChoiceOption(**{'multi': True, 'values': (u'all', u'1', u'2', u'3', u'4'), 'name': 'dhcrelay_interfaces', 'default': [u'all'], 'doc': u"Num\xe9ro de l'interface derri\xe8re laquelle sont les clients DHCP", 'properties': ('mandatory', 'normal')})
    option_139789497045072.impl_set_information("help", u"""'all' relaiera les reqêtes DHCP depuis toutes les interfaces vers le serveur DHCP""")
    option_139789497045616 = IntOption(**{'doc': u'Num\xe9ro de VLAN des clients DHCP', 'multi': True, 'properties': ('normal',), 'name': 'dhcrelay_vlan'})
    option_139789497037520 = Leadership(children=[option_139789497045072, option_139789497045616], **{'doc': u"Num\xe9ro de l'interface derri\xe8re laquelle sont les clients DHCP", 'properties': ('normal',), 'name': 'dhcrelay_interfaces'})
    option_139789497045344 = ChoiceOption(**{'multi': False, 'values': (u'1', u'2', u'3', u'4'), 'name': 'dhcrelay_server_interface', 'default': u'1', 'doc': u"Num\xe9ro de l'interface derri\xe8re laquelle est le serveur DHCP", 'properties': ('mandatory', 'basic')})
    option_139789497045344.impl_set_information("help", u"""Numéro de l'interface vers où relayer les requêtes DHCP""")
    option_139789497045888 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_dhcp_dhcrelay', 'doc': u'Adresse IP du serveur DHCP \xe0 relayer', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789497983600, True),), kwargs={})})
    option_139789497037840 = OptionDescription(children=[option_139789497037520, option_139789497045344, option_139789497045888], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956560}], **{'doc': u'relai dhcp', 'properties': ('basic',), 'name': 'relai_dhcp'})
    option_139789497037840.impl_set_information("help", u"""Paramétrage du relai DHCP""")
    option_139789497037840.impl_set_information("icon", u"""forward""")
    option_139789497046160 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'esu_proxy', 'default': 'oui', 'doc': u'Activer le param\xe9trage du proxy dans ESU', 'properties': ('mandatory', 'normal')})
    option_139789497046432 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'esu_proxy_default', 'default': 'oui', 'doc': u'Activer le proxy ESU dans les mod\xe8les par d\xe9faut', 'properties': ('auto_freeze', 'mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497046160}]})
    option_139789497046704 = DomainnameOption(**{'multi': False, 'name': 'esu_proxy_server', 'allow_ip': True, 'allow_without_dot': True, 'type_': 'domainname', 'doc': u'Adresse du proxy par d\xe9faut dans ESU', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497675024, False)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497046160}]})
    option_139789497046976 = PortOption(**{'multi': False, 'name': 'esu_proxy_port', 'allow_private': True, 'default': '3128', 'doc': u'Port du proxy par d\xe9faut dans ESU', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497046160}]})
    option_139789497047248 = UnicodeOption(**{'multi': False, 'name': 'tmp_esu_proxy_bypass', 'doc': u'Valeur temporaire pour le calcul de esu_proxy_bypass', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_classe, 'callback_params': Params(args=(ParamOption(option_139789497604304, False),), kwargs={})})
    option_139789497047520 = UnicodeOption(**{'multi': False, 'name': 'tmp_esu_proxy_bypass2', 'doc': u'Valeur temporaire pour le calcul de esu_proxy_bypass2', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.concat, 'callback_params': Params(args=None, kwargs={'valeur2': ParamValue("/"), 'valeur3': ParamOption(option_139789497047248, False), 'valeur1': ParamOption(option_139789497604576, False)})})
    option_139789497047792 = UnicodeOption(**{'multi': True, 'name': 'esu_proxy_bypass', 'doc': u'Site accessible sans proxy dans ESU', 'properties': ('normal',), 'callback': eosfunc.calc_multi_val, 'callback_params': Params(args=(ParamValue("127.0.0.1"), ParamOption(option_139789497047520, False)), kwargs={}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497046160}]})
    option_139789497047792.impl_set_information("help", u"""Liste des sites qui sont accessibles sans passer par le proxy""")
    option_139789497038160 = OptionDescription(children=[option_139789497046160, option_139789497046432, option_139789497046704, option_139789497046976, option_139789497047792, option_139789497047248, option_139789497047520], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956832}], **{'doc': u'ESU', 'properties': ('basic',), 'name': 'esu'})
    option_139789497038160.impl_set_information("help", u"""Configuration du proxy ESU""")
    option_139789497038160.impl_set_information("icon", u"""windows""")
    option_139789497048064 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_winbind', 'default': 'non', 'doc': u'Activer winbind', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789497048336 = UnicodeOption(**{'doc': u'Suffixe de machine LDAP', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'name': 'smb_ldap_machine_suffix'})
    option_139789497048608 = UnicodeOption(**{'doc': u'Suffixe de groupe LDAP', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'name': 'smb_ldap_group_suffix'})
    option_139789497053264 = IntOption(**{'default': 0, 'doc': u'\xc2ge maximal par d\xe9faut des mots de passe', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'smb_ldap_max_pwd_age'})
    option_139789497053264.impl_set_information("help", u"""Définit la durée en jours avant expiration d’un mot de
            passe. Cette durée est compté à partir de la date
            d’enregistrement du mot de passe.""")
    option_139789497053536 = IntOption(**{'default': 120, 'doc': u'Dur\xe9e du cache des r\xe9sultats de requ\xeates n\xe9gatifs', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'smb_idmap_negative_cache_time'})
    option_139789497053536.impl_set_information("help", u"""Durée exprimée en secondes (une valeur de 1 désactive le cache)""")
    option_139789497053808 = IntOption(**{'doc': u'LDAP replication sleep', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'name': 'smb_ldap_replication_sleep'})
    option_139789497054080 = IntOption(**{'doc': u"Nombre d'entr\xe9es par page dans une requ\xeate LDAP", 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'name': 'smb_ldap_page_size'})
    option_139789497054352 = IntOption(**{'doc': u'D\xe9lai avant abandon pour la connexion au LDAP', 'multi': False, 'properties': ('expert',), 'name': 'smb_ldap_connection_timeout'})
    option_139789497054624 = UnicodeOption(**{'multi': False, 'name': 'smb_ldap_suffix', 'doc': u"Base DN de l'annuaire utilis\xe9 par Samba", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497484976, False))})
    option_139789497054896 = UnicodeOption(**{'default': u'cn=admin,o=gouv,c=fr', 'doc': u"Nom de l'administrateur du domaine LDAP", 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'smb_ldap_admin_dn'})
    option_139789497055168 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ead_support_multietab', 'default': 'non', 'doc': u'Support du multi-\xe9tablissement', 'properties': ('mandatory', 'normal')})
    option_139789497055168.impl_set_information("help", u"""Le mode multi-établissement permet de gérer plusieurs établissements au sein d'un seul module EOLE""")
    option_139789497055440 = UnicodeOption(**{'default': u'standard', 'doc': u'Mod\xe8le de partage par d\xe9faut', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'smb_share_model'})
    option_139789497055440.impl_set_information("proposed_value", (u'commun', u'devoirs', u'groupes', u'icones$', u'minedu', u'professeurs', u'standard'))
    option_139789497055712 = UnicodeOption(**{'default': u'\\\\%N\\%U', 'doc': u"R\xe9pertoire home distant de l'utilisateur", 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert'), 'name': 'smb_logon_home'})
    option_139789497055984 = UnicodeOption(**{'default': u'%U%a.bat', 'doc': u'Nom du script de logon', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert'), 'name': 'smb_logon_script'})
    option_139789497056256 = DomainnameOption(**{'type_': 'netbios', 'name': 'smb_workgroup', 'doc': u'Nom du domaine Samba (ex: mondomaine)', 'multi': False, 'warnings_only': True, 'properties': ('auto_freeze', 'mandatory', 'basic')})
    option_139789497056256.impl_set_information("help", u"""Nom du domaine (workgroup) Samba""")
    option_139789497056256.impl_add_consistency("not_equal", option_139789497890480, **{})
    option_139789497056528 = UnicodeOption(**{'default': u'445 139', 'doc': u'Ports sur lesquels Samba accepte les connexions', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'smb_ports'})
    option_139789497056800 = UnicodeOption(**{'multi': False, 'name': 'smb_server_string', 'doc': u'Libell\xe9 du serveur Samba', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497889936, False))})
    option_139789497061456 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'smb_trash', 'default': 'non', 'doc': u'Activer la corbeille Samba', 'properties': ('mandatory', 'expert')})
    option_139789497061728 = FilenameOption(**{'multi': False, 'name': 'smb_trash_dir', 'default': u'perso/.corbeille', 'doc': u'Nom du r\xe9pertoire corbeille', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497061456}]})
    option_139789497062000 = IntOption(**{'multi': False, 'name': 'smb_trash_purge', 'default': 8, 'doc': u'Dur\xe9e de conservation des fichiers dans la corbeille (en jours)', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497061456}]})
    option_139789497062272 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'smb_quotawarn', 'default': 'non', 'doc': u"Activer l'envoi de courriel en cas de d\xe9passement des quotas", 'properties': ('mandatory', 'expert')})
    option_139789497062544 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'smb_guest', 'default': 'oui', 'doc': u'Activer le mode invit\xe9 sur le partage', 'properties': ('mandatory', 'expert')})
    option_139789497062816 = IntOption(**{'default': 0, 'doc': u'Niveau de log', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'smb_log_level'})
    option_139789497063088 = IntOption(**{'default': 15, 'doc': u"Nombre de minutes d'inactivit\xe9 avant d\xe9connexion automatique d'acc\xe8s \xe0 un fichier", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'smb_deadtime'})
    option_139789497063360 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'smb_activer_hide_files', 'default': 'oui', 'doc': u'Activer le masquage de fichiers', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789497063632 = UnicodeOption(**{'multi': False, 'name': 'smb_hide_files', 'default': u'/desktop.ini/', 'doc': u'Fichiers \xe0 masquer dans le partage', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497063360}]})
    option_139789497063904 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'smb_wins_support', 'default': 'yes', 'doc': u'D\xe9marrer le serveur Wins', 'properties': ('mandatory', 'expert')})
    option_139789497064176 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'smb_wins_server', 'doc': u'Serveur WINS distant', 'properties': ('normal',), "requires": [{'expected': u'yes', 'action': 'disabled', 'inverse': False, "option": option_139789497063904}]})
    option_139789497064448 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'smb_dns_proxy', 'default': 'no', 'doc': u"Rechercher des noms d'h\xf4te dans le DNS", 'properties': ('mandatory', 'expert')})
    option_139789497064720 = UnicodeOption(**{'default': u'wins hosts bcast lmhosts', 'doc': u"Ordre de r\xe9solution de noms d'h\xf4te", 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'name': 'smb_name_resolve_order'})
    option_139789497064720.impl_set_information("proposed_value", (u'wins hosts bcast lmhosts', u'bcast wins hosts lmhosts', u'wins bcast hosts lmhosts'))
    option_139789497064992 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'smb_oplocks', 'default': 'no', 'doc': u'Activer les verrous opportunistes (oplocks)', 'properties': ('mandatory', 'expert')})
    option_139789497069648 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'smb_dos_attributes', 'default': 'no', 'doc': u'Activer le support des attributs DOS', 'properties': ('mandatory', 'expert')})
    option_139789497069920 = IntOption(**{'default': 99, 'doc': u"Niveau de candidature lors de l'\xe9lection d'un ma\xeetre explorateur", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'smb_os_level'})
    option_139789497070192 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'smb_domain_master', 'default': 'yes', 'doc': u'Contr\xf4leur de domaine principal', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789497070464 = UnicodeOption(**{'default': u'default', 'doc': u'Niveau de protocole maximum support\xe9 par le serveur', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'smb_max_proto'})
    option_139789497070464.impl_set_information("proposed_value", (u'NT1', u'SMB2', u'SMB3', u'default'))
    option_139789497070736 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'smb_unixextensions', 'default': 'no', 'doc': u'Activer les extensions Unix', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497071008 = FilenameOption(**{'default': u'/var/lib/samba/printers', 'doc': u"Chemin vers les drivers d'imprimantes Windows", 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'smb_printers_drivers_path'})
    option_139789497071552 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'smb_control_print_access_with_acl', 'default': 'non', 'doc': u"Les contr\xf4les d'acc\xe8s au partage print$ sont g\xe9r\xe9s par les ACL", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497071280 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'smb_spoolss_x64', 'default': 'non', 'doc': u'Annoncer Spoolss comme architecture x64', 'properties': ('mandatory', 'expert')})
    option_139789497071280.impl_set_information("help", u"""Spoolss est le service d'impression de Samba""")
    option_139789497071824 = FilenameOption(**{'default': u'/home', 'doc': u'Chemin de base vers les dossiers personnels', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'home_path'})
    option_139789497072096 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'smb_activer_partages', 'default': 'non', 'doc': u'Activer des partages suppl\xe9mentaires', 'properties': ('mandatory', 'expert')})
    option_139789497072368 = UnicodeOption(**{'doc': u'Nom du partage', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'smb_partage_nom', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497072096}]})
    option_139789497072640 = FilenameOption(**{'doc': u'Nom absolu du r\xe9pertoire \xe0 partager', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'smb_partage_path', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497072096}]})
    option_139789497072912 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'smb_partage_visibilite', 'default_multi': 'non', 'doc': u'Visibilit\xe9 du partage', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497072096}]})
    option_139789497073184 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'smb_partage_ecriture', 'default_multi': 'non', 'doc': u'Partage en lecture/\xe9criture', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497072096}]})
    option_139789497038800 = Leadership(children=[option_139789497072368, option_139789497072640, option_139789497072912, option_139789497073184], **{'doc': u'Nom du partage', 'properties': ('basic',), 'name': 'smb_partage_nom'})
    option_139789497081936 = UnicodeOption(**{'default_multi': u'@DomainAdmins', 'multi': True, 'name': 'smb_admin_users', 'default': [u'@DomainAdmins'], 'doc': u"Utilisateurs ayant des privil\xe8ges d'administration sur tous les partages", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789497082208 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'smb_lanman_auth', 'default': 'yes', 'doc': u'Authentification LanManager', 'properties': ('mandatory', 'expert')})
    option_139789497082480 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'smb_nt_acl_support', 'default': 'yes', 'doc': u'Support NT ACL', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789497082752 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_regles_filtrage_port_source', 'default': 'non', 'doc': u"Autoriser l'ouverture de flux \xe0 partir d'un port source ", 'properties': ('mandatory', 'expert')})
    option_139789497082752.impl_set_information("separator", (u"Autoriser l'ouverture de flux \xe0 partir d'un port source", None))
    option_139789497083024 = IntOption(**{'multi': True, 'name': 'fw_rule_port', 'default': [137], 'doc': u'Port source \xe0 partir duquel les flux sont autoris\xe9s', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497082752}]})
    option_139789497083296 = ChoiceOption(**{'multi': True, 'values': (u'tcp', u'udp'), 'name': 'fw_rule_protocole', 'default_multi': u'udp', 'doc': u'Protocole udp/tcp pour lequel les flux sont autoris\xe9s', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497082752}]})
    option_139789497083568 = ChoiceOption(**{'multi': True, 'values': (u'0', u'1', u'2', u'3', u'4'), 'name': 'fw_rule_int', 'default_multi': u'0', 'doc': u'Interface sur laquelle les flux sont autoris\xe9s', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497082752}]})
    option_139789497039120 = Leadership(children=[option_139789497083024, option_139789497083296, option_139789497083568], **{'doc': u'Port source \xe0 partir duquel les flux sont autoris\xe9s', 'properties': ('normal',), 'name': 'fw_rule_port'})
    option_139789497083840 = IntOption(**{'default': 128, 'doc': u"Nombre maximum d'instances inotify pour un UID r\xe9el", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'fs_inotify_max_user_instances'})
    option_139789497083840.impl_set_information("separator", (u'Param\xe8tres syst\xe8me', None))
    option_139789497084112 = IntOption(**{'default': 8192, 'doc': u'Nombre maximum de surveillants associ\xe9s \xe0 une instance inotify', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'fs_inotify_max_user_watches'})
    option_139789497084384 = IntOption(**{'default': 16384, 'doc': u"Nombre maximum d'\xe9v\xe9nements mis en file d'attente dans une instance inotify", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'fs_inotify_max_queued_events'})
    option_139789497084656 = IntOption(**{'doc': u'Nombre maximum de partage utilisateurs', 'multi': False, 'properties': ('expert',), 'name': 'smb_usershare_max_shares'})
    option_139789497084656.impl_set_information("help", u"""Nombre maximum de partages pouvant être définis par l'utilisateur""")
    option_139789497084928 = UnicodeOption(**{'doc': u'Optimisations r\xe9seau', 'multi': False, 'properties': ('expert',), 'name': 'smb_socket_options'})
    option_139789497038480 = OptionDescription(children=[option_139789497048064, option_139789497048336, option_139789497048608, option_139789497053264, option_139789497053536, option_139789497053808, option_139789497054080, option_139789497054352, option_139789497054624, option_139789497054896, option_139789497055168, option_139789497055440, option_139789497055712, option_139789497055984, option_139789497890480, option_139789497056256, option_139789497056528, option_139789497056800, option_139789497061456, option_139789497061728, option_139789497062000, option_139789497062272, option_139789497062544, option_139789497062816, option_139789497063088, option_139789497063360, option_139789497063632, option_139789497063904, option_139789497064176, option_139789497064448, option_139789497064720, option_139789497064992, option_139789497069648, option_139789497069920, option_139789497070192, option_139789497070464, option_139789497070736, option_139789497071008, option_139789497071552, option_139789497071280, option_139789497071824, option_139789497072096, option_139789497038800, option_139789497081936, option_139789497082208, option_139789497082480, option_139789497082752, option_139789497039120, option_139789497083840, option_139789497084112, option_139789497084384, option_139789497084656, option_139789497084928], **{'doc': u'Samba', 'properties': ('basic',), 'name': 'samba'})
    option_139789497038480.impl_set_information("help", u"""Configuration du serveur de fichier Samba""")
    option_139789497038480.impl_set_information("icon", u"""group""")
    option_139789497085200 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'nscd_passwd_enabled_cache', 'default': 'no', 'doc': u'Activer le cache NSCD pour passwd', 'properties': ('mandatory', 'expert')})
    option_139789497085472 = IntOption(**{'default': 0, 'doc': u'Dur\xe9e de vie du cache pour les groupes inexistants', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'nscd_group_negative_time_to_live'})
    option_139789497085472.impl_set_information("help", u"""Durée de vie en secondes""")
    option_139789497094224 = ChoiceOption(**{'multi': False, 'values': (u'yes', u'no'), 'name': 'nscd_group_persistent', 'default': 'no', 'doc': u'Activer la persistance pour les groupes', 'properties': ('mandatory', 'expert')})
    option_139789497039440 = OptionDescription(children=[option_139789497085200, option_139789497085472, option_139789497094224], **{'doc': u'Nscd', 'properties': ('expert',), 'name': 'nscd'})
    option_139789497039440.impl_set_information("help", u"""Paramétrage de la mise en cache des données LDAP""")
    option_139789497039440.impl_set_information("icon", u"""retweet""")
    option_139789497094496 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'nut_ups_daemon', 'default': 'oui', 'doc': u'Configuration sur un serveur ma\xeetre', 'properties': ('mandatory', 'normal')})
    option_139789497094496.impl_set_information("help", u"""Activer la gestion d'onduleur. Ce service est a activer si ce serveur est connecté à un onduleur (cable RS232 ou USB). Sinon, le service se connecte à un autre serveur NUT.""")
    option_139789497094768 = UnicodeOption(**{'doc': u"Nom de l'onduleur distant", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'nut_monitor_foreign_name', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497095040 = DomainnameOption(**{'multi': False, 'name': 'nut_monitor_foreign_host', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u"H\xf4te g\xe9rant l'onduleur", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497095312 = UsernameOption(**{'doc': u"Utilisateur de l'h\xf4te distant", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'nut_monitor_foreign_user', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497095584 = PasswordOption(**{'doc': u"Mot de passe de l'h\xf4te distant", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'nut_monitor_foreign_password', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497095856 = DomainnameOption(**{'type_': 'hostname', 'name': 'nut_ups_name', 'doc': u"Nom de l'onduleur", 'allow_ip': False, 'multi': True, 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497095856.impl_set_information("help", u"""Nom de l'onduleur géré par le service NUT.""")
    option_139789497096128 = UnicodeOption(**{'multi': True, 'name': 'nut_ups_driver', 'default_multi': u'usbhid-ups', 'doc': u"Pilote de communication de l'onduleur", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497096128.impl_set_information("proposed_value", (u'al175', u'apcsmart', u'apcsmart-old', u'apcupsd-ups', u'bcmxcp', u'bcmxcp_usb', u'belkin', u'belkinunv', u'bestfcom', u'bestfortress', u'bestuferrups', u'bestups', u'blazer_ser', u'blazer_usb', u'clone', u'clone-outlet', u'dummy-ups', u'etapro', u'everups', u'gamatronic', u'genericups', u'isbmex', u'ivtscd', u'liebert', u'liebert-esp2', u'masterguard', u'metasys', u'mge-shut', u'mge-utalk', u'microdowell', u'nutdrv_atcl_usb', u'nutdrv_qx', u'oldmge-shut', u'oneac', u'optiups', u'powercom', u'powerpanel', u'rhino', u'richcomm_usb', u'riello_ser', u'riello_usb', u'safenet', u'solis', u'tripplite', u'tripplite_usb', u'tripplitesu', u'upscode2', u'usbhid-ups', u'victronups'))
    option_139789497096128.impl_set_information("help", u"""Pilote de communication de l'onduleur.""")
    option_139789497096400 = UnicodeOption(**{'multi': True, 'name': 'nut_ups_port', 'default_multi': u'auto', 'doc': u"Port de communication de l'onduleur", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497096400.impl_set_information("proposed_value", (u'auto', u'/dev/ttyS0', u'/dev/ttyS1'))
    option_139789497096400.impl_set_information("help", u"""Port de communication de l'onduleur. Pour les onduleurs USB ce port doit être \"auto\".""")
    option_139789497096672 = UnicodeOption(**{'doc': u"Num\xe9ro de s\xe9rie de l'onduleur (facultatif)", 'multi': True, 'properties': ('normal',), 'name': 'nut_ups_serial', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497096672.impl_set_information("help", u"""Valeur spécifique pour le champ \"serial\" associé à l'onduleur dans /etc/ups/ups.conf""")
    option_139789497096944 = UnicodeOption(**{'doc': u"Productid de l'onduleur (facultatif)", 'multi': True, 'properties': ('normal',), 'name': 'nut_ups_productid', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497096944.impl_set_information("help", u"""Valeur spécifique pour le champ \"productid\" associé à l'onduleur dans /etc/ups/ups.conf""")
    option_139789497097216 = UnicodeOption(**{'doc': u"Upstype de l'onduleur (facultatif)", 'multi': True, 'properties': ('normal',), 'name': 'nut_ups_upstype', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497097216.impl_set_information("help", u"""Valeur spécifique pour le champ \"upstype\" associé à l'onduleur dans /etc/ups/ups.conf""")
    option_139789497039760 = Leadership(children=[option_139789497095856, option_139789497096128, option_139789497096400, option_139789497096672, option_139789497096944, option_139789497097216], **{'doc': u"Nom de l'onduleur", 'properties': ('basic',), 'name': 'nut_ups_name'})
    option_139789497097488 = PasswordOption(**{'callback_params': None, 'multi': False, 'name': 'nut_ups_password', 'doc': u"Mot de passe de l'administrateur local de l'onduleur", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.gen_random})
    option_139789497097760 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'nut_monitor', 'default': 'non', 'doc': u'Autoriser des esclaves distants \xe0 se connecter', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789497097760.impl_set_information("separator", ('Esclaves distants', None))
    option_139789497114704 = UsernameOption(**{'doc': u"Utilisateur de surveillance de l'onduleur", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'nut_monitor_user', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497097760}]})
    option_139789497114704.impl_set_information("help", u"""Nom d'utilisateur de surveillance de l'onduleur.""")
    option_139789497114976 = PasswordOption(**{'doc': u"Mot de passe de surveillance de l'onduleur", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'nut_monitor_password', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497097760}]})
    option_139789497114976.impl_set_information("help", u"""Mot de passe de l'utilisateur de surveillance de l'onduleur.""")
    option_139789497115248 = NetworkOption(**{'doc': u"Adresse IP du r\xe9seau de l'esclave", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'nut_monitor_host', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497097760}]})
    option_139789497115520 = NetmaskOption(**{'doc': u"Masque de l'IP du r\xe9seau de l'esclave", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'nut_monitor_netmask', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497097760}]})
    option_139789497115520.impl_add_consistency("network_netmask", option_139789497115248, **{})
    option_139789497040080 = Leadership(children=[option_139789497114704, option_139789497114976, option_139789497115248, option_139789497115520], **{'doc': u"Utilisateur de surveillance de l'onduleur", 'properties': ('basic',), 'name': 'nut_monitor_user'})
    option_139789497040400 = OptionDescription(children=[option_139789497094496, option_139789497094768, option_139789497095040, option_139789497095312, option_139789497095584, option_139789497039760, option_139789497097488, option_139789497097760, option_139789497040080], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957104}], **{'doc': u'Onduleur', 'properties': ('basic',), 'name': 'onduleur'})
    option_139789497040400.impl_set_information("help", u"""Paramétrage de la gestion des onduleurs par NUT""")
    option_139789497040400.impl_set_information("icon", u"""bolt""")
    option_139789497115792 = UnicodeOption(**{'default': u'local', 'doc': u'Nom de domaine du minion', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'minion_domain_name'})
    option_139789497115792.impl_set_information("help", u"""La machine cliente SaltStack est appelé « minion »""")
    option_139789497116064 = PortOption(**{'multi': False, 'name': 'salt_api_port', 'allow_private': True, 'default': '8880', 'doc': u"Port d'acc\xe8s \xe0 l'API SaltStack", 'properties': ('mandatory', 'expert')})
    option_139789497126992 = OptionDescription(children=[option_139789497115792, option_139789497116064], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963664}], **{'doc': u'SaltStack', 'properties': ('expert',), 'name': 'saltstack'})
    option_139789497126992.impl_set_information("help", u"""Personnalisation de la configuration SaltStack""")
    option_139789497126992.impl_set_information("icon", u"""cloud""")
    option_139789497116336 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'sw_database_mode', 'doc': u'Configuration des tunnels en mode database', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamValue("non"))})
    option_139789497116336.impl_set_information("separator", (u'Param\xe8tres strongSwan', None))
    option_139789497116336.impl_set_information("help", u"""Configuration des tunnels en mode database (oui -> ipsec.db)
ou fichier plat (non -> ipsec.conf)""")
    option_139789497116608 = IntOption(**{'default': 11, 'doc': u"Nombre d'essais de retransmission avant Dead Peer Detection", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'sw_retransmit_tries'})
    option_139789497116880 = IntOption(**{'default': 0, 'doc': u'Timeout pour le process stroke', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'sw_stroke_timeout'})
    option_139789497116880.impl_set_information("help", u"""Exemples de process pouvant se figer : \"ipsec stroke statusall\", \"ipsec stroke delete connexion_name\"""")
    option_139789497117152 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'sw_install_vpn_route', 'default': 'non', 'doc': u'Gestion des routes par strongSwan', 'properties': ('mandatory', 'expert')})
    option_139789497117152.impl_set_information("separator", ('Gestion des Routes VPN', None))
    option_139789497117152.impl_set_information("help", u"""Si 'oui', l'adresse IP d'une des interfaces sera mise en source route selon l'ordre de montée des tunnels;
Si 'non', l'adresse IP de l'interface spécifiée sera forcée comme source lors de l'utilisation des tunnels par le serveur lui-même.""")
    option_139789497117424 = ChoiceOption(**{'multi': False, 'values': (u'non', u'1', u'2', u'3', u'4'), 'name': 'sw_force_ip_src', 'default': u'1', 'doc': u"Forcer l'adresse IP source de l'interface", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497117152}]})
    option_139789497117424.impl_set_information("help", u"""Lorsque le serveur aura besoin de passer par un tunnel, il sera vu avec cette adresse IP à l'autre extrémité du tunnel (dans la mesure où le reseau source inclu cette adresse)""")
    option_139789497117696 = IntOption(**{'default': 32, 'doc': u'Nombre de threads disponibles pour strongSwan', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'sw_threads'})
    option_139789497117696.impl_set_information("separator", ('Gestion des threads', None))
    option_139789497117968 = IntOption(**{'default': 2, 'doc': u'Nombre de threads \xe0 r\xe9server pour les jobs HIGH priority', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'sw_high_priority_threads'})
    option_139789497118240 = IntOption(**{'default': 4, 'doc': u'Nombre de threads \xe0 r\xe9server pour les jobs MEDIUM priority', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'sw_medium_priority_threads'})
    option_139789497135184 = IntOption(**{'default': 1280, 'doc': u"Taille maximum d'un fragment IKE", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'sw_fragment_size'})
    option_139789497135184.impl_set_information("separator", ('Gestion de la fragmentation', None))
    option_139789497135184.impl_set_information("help", u"""Cette taille sera utilisée uniquement si la fragmentation est autorisée dans la configuration IPSec""")
    option_139789497135456 = IntOption(**{'default': 0, 'doc': u'MSS \xe0 positionner sur les routes (0 pour d\xe9sactiver)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'sw_mss'})
    option_139789497135456.impl_set_information("help", u"""Contournement en cas de problème de MTU/MSS.""")
    option_139789497135728 = IntOption(**{'default': 0, 'doc': u'MTU \xe0 positionner sur les routes (0 pour d\xe9sactiver)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'sw_mtu'})
    option_139789497135728.impl_set_information("help", u"""Contournement en cas de problème de MTU/MSS.""")
    option_139789497136000 = IPOption(**{'doc': u'Adresses IP \xe0 tester dans test-rvp', 'multi': True, 'allow_reserved': True, 'properties': ('normal',), 'name': 'ip_test_rvp'})
    option_139789497136000.impl_set_information("separator", (u'Param\xe8tres agent Z\xe9phir RVP et diagnose', None))
    option_139789497136272 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'sw_crl_check', 'default': 'oui', 'doc': u'Contr\xf4le du status des certificats dans la CRL', 'properties': ('mandatory', 'expert')})
    option_139789497136272.impl_set_information("separator", (u'Param\xe8tres IPsec', None))
    option_139789497136544 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'acces_proxy_zone_rvp', 'default': 'non', 'doc': u'Acc\xe8s RVP par le proxy', 'properties': ('mandatory', 'normal')})
    option_139789497136544.impl_set_information("separator", (u'Acc\xe8s RVP par le proxy', None))
    option_139789497136544.impl_set_information("help", u"""Autoriser les réseaux locaux à accéder à des zones RVP via le proxy""")
    option_139789497136816 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'adresse_network_zone_rvp', 'doc': u'Adresse r\xe9seau de la zone RVP', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497136544}]})
    option_139789497137088 = NetmaskOption(**{'doc': u'Masque de sous-r\xe9seau de la zone RVP', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'adresse_netmask_zone_rvp'})
    option_139789497137360 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'autoriser_rvp_autres', 'default_multi': 'non', 'doc': u"Autoriser d'autres zones que l'interface 1 \xe0 acc\xe9der \xe0 ce r\xe9seau", 'properties': ('mandatory', 'normal')})
    option_139789497127312 = Leadership(children=[option_139789497136816, option_139789497137088, option_139789497137360], **{'doc': u'Adresse r\xe9seau de la zone RVP', 'properties': ('basic',), 'name': 'adresse_network_zone_rvp'})
    option_139789497137632 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'adresse_dns_agriates', 'doc': u'Adresse du DNS permettant de resoudre les in.ac-acad.fr', 'properties': ('normal',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}]})
    option_139789497137904 = DomainnameOption(**{'type_': 'domainname', 'name': 'nom_zone_forward_rvp', 'doc': u'Nom DNS de la zone resolue par le DNS AGRIATES', 'allow_ip': False, 'multi': True, 'properties': ('normal',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}]})
    option_139789497127632 = OptionDescription(children=[option_139789497116336, option_139789497116608, option_139789497116880, option_139789497117152, option_139789497117424, option_139789497117696, option_139789497117968, option_139789497118240, option_139789497135184, option_139789497135456, option_139789497135728, option_139789497136000, option_139789497136272, option_139789497136544, option_139789497127312, option_139789497655648, option_139789497137632, option_139789497137904], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}], **{'doc': u'RVP', 'properties': ('basic',), 'name': 'rvp'})
    option_139789497127632.impl_set_information("help", u"""Paramétrage du réseau virtuel privé""")
    option_139789497127632.impl_set_information("icon", u"""lock""")
    option_139789497138176 = UnicodeOption(**{'multi': False, 'name': 'envole_version', 'doc': u'Num\xe9ro de version majeure d\u2019Envole', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.get_version, 'callback_params': Params(args=(ParamValue("ENVOLE_VERSION"),), kwargs={})})
    option_139789497138448 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'force_envole', 'default': 'non', 'doc': u'Utiliser Envole comme application par d\xe9faut en frontal', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497138720 = ChoiceOption(**{'multi': False, 'values': (u'non', u'local'), 'name': 'activer_recuperation_courriel', 'default': u'local', 'doc': u'Activer le serveur de r\xe9cup\xe9ration de courrier \xe9lectronique', 'properties': ('mandatory', 'normal')})
    option_139789497138720.impl_set_information("separator", (u'Serveur de r\xe9cup\xe9ration de courrier \xe9lectronique (POP/IMAP)', None))
    option_139789497143376 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_roundcube', 'default': 'oui', 'doc': u'Activer Roundcube (webmail)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497138720}]})
    option_139789497143376.impl_set_information("help", u"""RoundCube est un client webmail Open Source""")
    option_139789497143648 = FilenameOption(**{'multi': False, 'name': 'web_redirection', 'doc': u'Application web par d\xe9faut (redirection)', 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_webredirection, 'callback_params': Params(args=None, kwargs={'force_envole': ParamOption(option_139789497138448, True), 'roundcube': ParamOption(option_139789497143376, True)}), "validator": eosfunc.valid_regexp, 'validator_params': Params(args=(ParamValue(u"^/"),), kwargs={'err_msg': ParamValue(u"Les alias doivent débuter par un \"/\"")})})
    option_139789497143648.impl_set_information("help", u"""Application proposée par défaut lors de l'accès du serveur web""")
    option_139789497143920 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_web_behind_revproxy', 'default': 'oui', 'doc': u'Le serveur web est derri\xe8re un reverse proxy', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497144192 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'web_behind_revproxy_ip', 'doc': u'Adresse IP du serveur reverse proxy', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789498664080, False),), kwargs={}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497143920}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497143920}]})
    option_139789497144464 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_web_valider_ca', 'default': 'non', 'doc': u"Activer la v\xe9rification de l'autorit\xe9 de certification pour les applications web cassifi\xe9es", 'properties': ('mandatory', 'expert')})
    option_139789497144736 = FilenameOption(**{'multi': False, 'name': 'apache_cert', 'doc': u'Certificat utilis\xe9 par apache', 'properties': ('expert',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497939632, False))})
    option_139789497145008 = ChoiceOption(**{'multi': False, 'values': (u'mysql', u'sqlite3'), 'name': 'bareos_db_type', 'default': u'mysql', 'doc': u'Type de la base de donn\xe9es du catalogue', 'properties': ('auto_freeze', 'mandatory', 'basic')})
    option_139789497145280 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_bareoswebui', 'default': 'non', 'doc': u'Activer Bareos WebUI (gestion de la sauvegarde)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497971584}, {'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497971312}, {'expected': u'mysql', 'action': 'disabled', 'inverse': True, "option": option_139789497145008}]})
    option_139789497145280.impl_set_information("help", u"""Bareos WebUI est une application web permettant de surveiller et gérer les sauvegardes Bareos""")
    option_139789497145552 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_phpmyadmin', 'default': 'non', 'doc': u'Activer phpMyAdmin (administration des bases MySQL)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971312}]})
    option_139789497145552.impl_set_information("help", u"""phpMyAdmin permet d'administrer en ligne les bases de données MySQL du serveur""")
    option_139789497145824 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_eoleapps', 'default': 'oui', 'doc': u'Activer le service eoleapps', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789497145824.impl_set_information("help", u"""Serveur d'applications flask EOLE""")
    option_139789497146096 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_eoe', 'default': 'oui', 'doc': u'Activer EOE (gestion de mot de passe \xe9l\xe8ve)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145824}]})
    option_139789497146096.impl_set_information("help", u"""EOE est une application web à destination des élèves""")
    option_139789497146368 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_eop', 'default': 'oui', 'doc': u'Activer EOP (EOLE Outils Profs)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145824}]})
    option_139789497146368.impl_set_information("help", u"""EOP est une application web indépendante de l'EAD destinée aux profs""")
    option_139789497146640 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'eop_profs_can_change_all_passwords', 'default': 'non', 'doc': u'Permettre aux enseignants de changer le mot de passe de tous les \xe9l\xe8ves', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497146368}]})
    option_139789497146640.impl_set_information("help", u"""Par défaut, seuls les professeurs principaux peuvent changer le mot de passe de leurs élèves. En choisissant \"oui\" , les professeurs pourront changer le mot de passe de n'importe quel élève.""")
    option_139789497146912 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_roundcubepop', 'default': 'oui', 'doc': u"Permettre aux utilisateurs d'ajouter des comptes de courrier \xe9lectronique personnels (au travers d'un serveur pop)", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497143376}]})
    option_139789497146912.impl_set_information("help", u"""RoundCube permet aux utilisateurs d'ajouter des comptes de courrier électronique POP autres que ceux gérés par l'annuaire.""")
    option_139789497155664 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_pydio', 'default': 'oui', 'doc': u'Activer Pydio', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}]})
    option_139789497155664.impl_set_information("help", u"""Pydio est un gestionnaire de fichiers en ligne""")
    option_139789497155936 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_pydio_local', 'default': 'non', 'doc': u'Activer un repository Local', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789497156208 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_pydio_ftp', 'default': 'non', 'doc': u'Activer un repository distant en FTP', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789497156480 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'pydio_repository_ftp', 'doc': u'IP du repository distant FTP', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497156208}]})
    option_139789497156752 = UnicodeOption(**{'multi': False, 'name': 'pydio_ftp', 'doc': u'Adresse du serveur FTP utilis\xe9 par Pydio', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'callback': eosfunc.calc_container, 'callback_params': Params(args=(ParamOption(option_139789497889120, False), ParamOption(option_139789497854704, False)), kwargs={})})
    option_139789497127952 = OptionDescription(children=[option_139789497138176, option_139789497414256, option_139789497138448, option_139789497143648, option_139789497143920, option_139789497144192, option_139789497144464, option_139789497144736, option_139789497145280, option_139789497145552, option_139789497146096, option_139789497146368, option_139789497146640, option_139789497143376, option_139789497146912, option_139789497155664, option_139789497155936, option_139789497156208, option_139789497156480, option_139789497156752], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}], **{'doc': u'Applications web', 'properties': ('basic',), 'name': 'applications_web'})
    option_139789497127952.impl_set_information("help", u"""Paramètrage et gestion des applications web""")
    option_139789497127952.impl_set_information("icon", u"""php-alt""")
    option_139789497157024 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'apache_plus', 'default': 'non', 'doc': u'D\xe9clarer des applications web suppl\xe9mentaires', 'properties': ('mandatory', 'expert')})
    option_139789497157024.impl_set_information("separator", ('Applications supplementaires', None))
    option_139789497157296 = FilenameOption(**{'doc': u"Chemin complet l'application (exemple : /var/www/html/appli)", 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'apache_dir', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497157024}]})
    option_139789497157568 = FilenameOption(**{'multi': True, 'name': 'apache_alias', 'doc': u"Alias de l'application (exemple : /appli)", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497157024}], "validator": eosfunc.valid_regexp, 'validator_params': Params(args=(ParamValue(u"^/"),), kwargs={'err_msg': ParamValue(u"Les alias doivent débuter par un \"/\"")})})
    option_139789497157840 = FilenameOption(**{'default_multi': u'index.php', 'doc': u"Fichier \xe0 rechercher lorsqu'un client envoie une requ\xeate pour l'index d'un r\xe9pertoire", 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'apache_directory_index'})
    option_139789497157840.impl_set_information("proposed_value", (u'index.php', u'index.html'))
    option_139789497128272 = Leadership(children=[option_139789497157296, option_139789497157568, option_139789497157840], **{'doc': u"Chemin complet l'application (exemple : /var/www/html/appli)", 'properties': ('expert',), 'name': 'apache_dir'})
    option_139789497158112 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'apache_lister_repertoires', 'default': 'non', 'doc': u'Permettre de lister les r\xe9pertoires et leur contenu', 'properties': ('mandatory', 'expert')})
    option_139789497158112.impl_set_information("separator", ('Configuration Apache', None))
    option_139789497158384 = IntOption(**{'default': 300, 'doc': u"Temps en secondes pendant lequel le serveur va attendre des entr\xe9es/sorties avant de consid\xe9rer qu'une requ\xeate a \xe9chou\xe9", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'apache_timeout'})
    option_139789497158656 = ChoiceOption(**{'multi': False, 'values': (u'On', u'Off'), 'name': 'apache_keepalive', 'default': u'On', 'doc': u'Autoriser les connexions persistantes', 'properties': ('mandatory', 'expert')})
    option_139789497158928 = IntOption(**{'default': 5, 'doc': u'Nombre de processus enfants du serveur cr\xe9\xe9s au d\xe9marrage', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'mpm_startservers'})
    option_139789497159200 = IntOption(**{'default': 5, 'doc': u'Nombre minimum de processus serveurs enfants inactifs', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'mpm_minspareservers'})
    option_139789497167952 = IntOption(**{'default': 10, 'doc': u'Nombre maximum de processus serveurs enfants inactifs', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'mpm_maxspareservers'})
    option_139789497167952.impl_set_information("help", u"""S'il y a plus de MaxSpareServers processus inactifs, le processus parent arrêtera les processus excédentaires""")
    option_139789497168224 = IntOption(**{'default': 150, 'doc': u'Nombre maximum de connexions pouvant \xeatre trait\xe9es simultan\xe9ment', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'mpm_maxrequestworkers'})
    option_139789497168224.impl_set_information("help", u"""Si la limite MaxRequestWorkers est atteinte, toute tentative de connexion sera normalement mise dans une file d'attente""")
    option_139789497168496 = IntOption(**{'default': 0, 'doc': u"Nombre maximum de connexions qu'un processus enfant va traiter au cours de son fonctionnement", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'mpm_maxconnectionsperchild'})
    option_139789497168496.impl_set_information("help", u"""Si MaxConnectionsPerChild est définie à 0, il n'y a plus aucune limite sur le nombre de connexions que le processus pourra traiter""")
    option_139789497168768 = IntOption(**{'default': 32, 'doc': u'Taille maximale des donn\xe9es re\xe7ues par la m\xe9thode POST (en Mo)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'php_post_max_size'})
    option_139789497168768.impl_set_information("separator", ('Configuration PHP', None))
    option_139789497169040 = IntOption(**{'default': 16, 'doc': u"Taille maximale d'un fichier \xe0 charger (en Mo)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'php_upload_max_filesize'})
    option_139789497169312 = IntOption(**{'default': 30, 'doc': u"Temps maximal d'ex\xe9cution d'un script (en secondes)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'php_max_execution_time'})
    option_139789497169584 = IntOption(**{'default': 60, 'doc': u"Dur\xe9e maximale pour analyser les donn\xe9es d'entr\xe9e (en secondes)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'php_max_input_time'})
    option_139789497169856 = IntOption(**{'default': 128, 'doc': u"Taille m\xe9moire maximale qu'un script est autoris\xe9 \xe0 allouer (en Mo)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'php_memory_limit'})
    option_139789497170128 = ChoiceOption(**{'multi': False, 'values': (u'On', u'Off'), 'name': 'php_display_errors', 'default': u'Off', 'doc': u"Affichage des erreurs \xe0 l'\xe9cran", 'properties': ('mandatory', 'expert')})
    option_139789497170400 = IntOption(**{'default': 3600, 'doc': u'Dur\xe9e de vie des donn\xe9es sur le serveur (en secondes)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'php_session_gc_maxlifetime'})
    option_139789497170672 = IntOption(**{'default': 16, 'doc': u"Nombre d'octets \xe0 lire dans le fichier utilis\xe9 comme source additionnelle d'entropie", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'php_session_entropy_length'})
    option_139789497170672.impl_set_information("help", u"""Spécifie le nombre d'octets qui seront lus dans le fichier utilisé comme source additionnelle d'entropie pour la création de l'identifiant de session. Mettre à 0 pour le désactiver.""")
    option_139789497170944 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'php_browscap', 'default': 'non', 'doc': u'Activer la directive de configuration browscap', 'properties': ('mandatory', 'expert')})
    option_139789497170944.impl_set_information("help", u"""La directive de configuration browscap permet d'obtenir plus d'information sur les capacités du navigateur client grâce à la fonction get_browser()""")
    option_139789497128592 = OptionDescription(children=[option_139789497157024, option_139789497128272, option_139789497158112, option_139789497158384, option_139789497158656, option_139789497158928, option_139789497159200, option_139789497167952, option_139789497168224, option_139789497168496, option_139789497168768, option_139789497169040, option_139789497169312, option_139789497169584, option_139789497169856, option_139789497170128, option_139789497170400, option_139789497170672, option_139789497170944], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}], **{'doc': u'Apache', 'properties': ('expert',), 'name': 'apache'})
    option_139789497128592.impl_set_information("help", u"""Paramètrage avancé du serveur web Apache""")
    option_139789497128592.impl_set_information("icon", u"""apache""")
    option_139789497171216 = UsernameOption(**{'doc': u"Utilisateur autoris\xe9 \xe0 se connecter \xe0 l'interface web de gestion de la sauvegarde", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'bareoswebui_user'})
    option_139789497171488 = PasswordOption(**{'doc': u"Mot de passe de l'utilisateur autoris\xe9 \xe0 se connecter \xe0 l'interface web de gestion de la sauvegarde", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'bareoswebui_password'})
    option_139789497128912 = Leadership(children=[option_139789497171216, option_139789497171488], **{'doc': u"Utilisateur autoris\xe9 \xe0 se connecter \xe0 l'interface web de gestion de la sauvegarde", 'properties': ('basic',), 'name': 'bareoswebui_user'})
    option_139789497180240 = PasswordOption(**{'callback_params': None, 'multi': False, 'name': 'bareoswebui_db_mysql_password', 'doc': u'Mot de passe de la base de donn\xe9es MySQL de la sauvegarde', 'properties': ('auto_freeze', 'basic'), 'callback': eosfunc.gen_random})
    option_139789497129232 = OptionDescription(children=[option_139789497128912, option_139789497180240], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145280}], **{'doc': u'bareos webui', 'properties': ('basic',), 'name': 'bareos_webui'})
    option_139789497129232.impl_set_information("help", u"""Paramétrage de l'application web Bareos WebUI""")
    option_139789497129232.impl_set_information("icon", u"""archive""")
    option_139789497180512 = PortOption(**{'multi': False, 'name': 'eolesso_port', 'allow_private': True, 'default': '8443', 'doc': u'Port utilis\xe9 par le service EoleSSO', 'properties': ('mandatory', 'normal')})
    option_139789497180784 = FilenameOption(**{'doc': u"Alias d'acc\xe8s au service SSO (param\xe8tre : __CAS_FOLDER)", 'multi': False, 'properties': ('expert',), 'name': 'eolesso_cas_folder'})
    option_139789497180784.impl_set_information("help", u"""Certains serveurs SSO tels que lemonLDAP ou keycloak ont besoin d'un alias spécifique en plus du domaine et du port, exemple : https://monserveursso:8443/CAS""")
    option_139789497181056 = UnicodeOption(**{'default': u'EoleSSOServer', 'doc': u'Nom du cookie EoleSSO', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'eolesso_cookie_name'})
    option_139789497181056.impl_set_information("help", u"""Nom du cookie identifiant la session SSO""")
    option_139789497181328 = UnicodeOption(**{'doc': u'Domaine du cookie EoleSSO', 'multi': False, 'properties': ('expert',), 'name': 'eolesso_cookie_domain'})
    option_139789497181328.impl_set_information("help", u"""Domaine du cookie SSO ( si rien: utilise le nom d'hôte du serveur)""")
    option_139789497181600 = DomainnameOption(**{'multi': True, 'name': 'eolesso_ldap', 'allow_without_dot': True, 'default': [u'localhost'], 'doc': u'Adresse du serveur LDAP utilis\xe9 par EoleSSO', 'type_': 'domainname', 'allow_ip': True, 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497182144 = PortOption(**{'multi': True, 'name': 'eolesso_port_ldap', 'allow_private': True, 'default_multi': '389', 'doc': u'Port du serveur LDAP utilis\xe9 par EoleSSO', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497181872 = UnicodeOption(**{'multi': True, 'name': 'eolesso_base_ldap', 'doc': u"Chemin de recherche dans l'annuaire", 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_val_first_value, 'callback_params': Params(args=(ParamOption(option_139789497484976, True), ParamValue("o=gouv,c=fr")), kwargs={}), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497182416 = UnicodeOption(**{'multi': True, 'name': 'eolesso_ldap_label', 'doc': u"Libell\xe9 \xe0 pr\xe9senter aux utilisateurs en cas d'homonymes", 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_libelle_annuaire, 'callback_params': Params(args=(ParamOption(option_139789497181600, False), ParamOption(option_139789497890208, False), ParamOption(option_139789497890752, False)), kwargs={}), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497182416.impl_set_information("help", u"""Dans le cas ou un utilisateur aurait un homonyme dans un des annuaires configurés, une liste déroulante lui permet de choisir sa provenance (un des annuaires configurés, ou un établissement en cas d'annuaire multi établissements). Ce libellé est utilisé pour décrire cet annuaire. (si rien: Annuaire de 'adresse_serveur')""")
    option_139789497182688 = UnicodeOption(**{'doc': u"Informations suppl\xe9mentaire dans le cadre d'information sur les homonymes", 'multi': True, 'properties': ('normal',), 'name': 'eolesso_ldap_infos', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497182688.impl_set_information("help", u"""Dans le cas ou des homonymes sont détectés, le contenu du fichier indiqué sera ajouté au code html du cadre affiché (seulement si l'utilisateur est présent dans cet annuaire). Le fichier doit être dans le répertoire /usr/share/sso/interface/infos_homonymes""")
    option_139789497182960 = UnicodeOption(**{'multi': True, 'name': 'eolesso_ldap_reader', 'doc': u'Utilisateur de lecture des comptes LDAP (n\xe9cessaire pour la f\xe9d\xe9ration)', 'properties': ('normal',), 'callback': eosfunc.concat, 'callback_params': Params(args=(ParamValue("cn=reader,"), ParamOption(option_139789497181872, False)), kwargs={}), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497182960.impl_set_information("help", u"""La fédération nécessite un compte LDAP ayant accès en lecture aux attributs de tous les utilisateurs. Dans le cas de l'utilisation d'un annuaire fourni par le paquet eole-annuaire, l'utilisateur \"cn=reader,o=gouv,c=fr\" est disponible""")
    option_139789497183232 = FilenameOption(**{'multi': True, 'name': 'eolesso_ldap_reader_passfile', 'default_multi': u'/root/.reader', 'doc': u"Fichier de mot de passe de l'utilisateur de lecture", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497183232.impl_set_information("help", u"""Fichier contenant le mot de passe de l'utilisateur de lecture des comptes LDAP. si le fichier ou le compte n'existent pas, la recherche d'attributs en cas de fédération se fera de façon anonyme""")
    option_139789497183504 = UnicodeOption(**{'multi': True, 'name': 'eolesso_ldap_match_attribute', 'default_multi': u'uid', 'doc': u'Attribut de recherche des utilisateurs', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497183504.impl_set_information("help", u"""Attribut utilisé comme identifiant des utilisateurs pour cet annuaire ('uid' par défaut. Si autre valeur, les groupes de l'utilisateur ne seront pas détectés)""")
    option_139789497183776 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'eolesso_pam_securid', 'default': 'non', 'doc': u"Gestion de l'authentification OTP (RSA SecurID)", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497196624 = ChoiceOption(**{'multi': True, 'values': (u'inactifs', u'identiques', u'configurables'), 'name': 'eolesso_ldap_login_otp', 'default_multi': u'inactifs', 'doc': u'Identifiants OTP pour cet annuaire', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497183776}]})
    option_139789497196624.impl_set_information("help", u"""Type d'identifiants OTP gérés pour les utilisateurs de cet annuaire. Inactifs : OTP non disponible; identiques : identifiant identique à l'identifiant LDAP; configurables : identifiant personnalisé renseigné par l'utilisateur à la première connexion en mode OTP""")
    option_139789497196896 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'eolesso_ldap_apps_params', 'default': 'non', 'doc': u'Information LDAP suppl\xe9mentaires (applications)', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497196896.impl_set_information("help", u"""Permet de configurer des paramètres supplémentaires pour chaque annuaire LDAP. Utilisé par les applications accédant directement aux annuaires (ex: ownCloud)""")
    option_139789497197168 = UnicodeOption(**{'multi': True, 'name': 'eolesso_ldap_filter_user', 'default_multi': u'objectClass=person', 'doc': u"Filtre d'utilisateurs", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497197168.impl_set_information("help", u"""Sans élément de substitution, par exemple \"objectClass=person\".""")
    option_139789497197440 = UnicodeOption(**{'multi': True, 'name': 'eolesso_ldap_filter_group', 'default_multi': u'objectClass=posixGroup', 'doc': u'Filtre de groupes', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497197440.impl_set_information("help", u"""Sans élément de substitution, par exemple \"objectClass=posixGroup\".""")
    option_139789497197712 = UnicodeOption(**{'doc': u"DN racine de l'arbre utilisateurs", 'multi': True, 'properties': ('normal',), 'name': 'eolesso_ldap_dntree_user', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497197984 = UnicodeOption(**{'doc': u"DN racine de l'arbre groupes", 'multi': True, 'properties': ('normal',), 'name': 'eolesso_ldap_dntree_group', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497198256 = UnicodeOption(**{'multi': True, 'name': 'eolesso_ldap_fill_displayname', 'default_multi': u'displayName', 'doc': u"Champ 'nom d'affichage' de l'utilisateur", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497198256.impl_set_information("help", u"""Attribut utilisé comme nom complet pour l'utilisateur""")
    option_139789497198528 = UnicodeOption(**{'multi': True, 'name': 'eolesso_ldap_fill_mail', 'default_multi': u'mail', 'doc': u"Champ 'mail' de l'utilisateur", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497198528.impl_set_information("help", u"""Attribut utilisé comme mail pour l'utilisateur""")
    option_139789497198800 = UnicodeOption(**{'doc': u"Champ 'fonction' de l'utilisateur", 'multi': True, 'properties': ('normal',), 'name': 'eolesso_ldap_fill_fonction', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497198800.impl_set_information("help", u"""Attribut utilisé comme fonction pour l'utilisateur""")
    option_139789497199072 = UnicodeOption(**{'doc': u"Champ 'categorie' de l'utilisateur", 'multi': True, 'properties': ('normal',), 'name': 'eolesso_ldap_fill_categorie', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497199072.impl_set_information("help", u"""Attribut utilisé comme catégorie pour l'utilisateur""")
    option_139789497199344 = UnicodeOption(**{'doc': u"Champ 'rne' de l'utilisateur", 'multi': True, 'properties': ('normal',), 'name': 'eolesso_ldap_fill_rne', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497199344.impl_set_information("help", u"""Attribut utilisé comme rne pour l'utilisateur""")
    option_139789497199616 = UnicodeOption(**{'doc': u"Champ 'fredurne' de l'utilisateur", 'multi': True, 'properties': ('normal',), 'name': 'eolesso_ldap_fill_fredurne', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497199616.impl_set_information("help", u"""Attribut utilisé comme fredurne pour l'utisateur""")
    option_139789497199888 = UnicodeOption(**{'multi': True, 'name': 'eolesso_ldap_fill_displaygroup', 'default_multi': u'cn', 'doc': u"Champ 'nom d'affichage' du groupe", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497196896}]})
    option_139789497199888.impl_set_information("help", u"""Attribut utilisé comme nom du groupe de l'utilisateur""")
    option_139789497129552 = Leadership(children=[option_139789497181600, option_139789497182144, option_139789497181872, option_139789497182416, option_139789497182688, option_139789497182960, option_139789497183232, option_139789497183504, option_139789497196624, option_139789497197168, option_139789497197440, option_139789497197712, option_139789497197984, option_139789497198256, option_139789497198528, option_139789497198800, option_139789497199072, option_139789497199344, option_139789497199616, option_139789497199888], **{'doc': u'Adresse du serveur LDAP utilis\xe9 par EoleSSO', 'properties': ('normal',), 'name': 'eolesso_ldap'})
    option_139789497200160 = UnicodeOption(**{'doc': u'Adresse du serveur SSO parent', 'multi': False, 'properties': ('normal',), 'name': 'eolesso_adresse_parent', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497208912 = IntOption(**{'multi': False, 'name': 'eolesso_port_parent', 'default': 8443, 'doc': u'Port du serveur SSO parent', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497209184 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'eolesso_metrics', 'default': 'non', 'doc': u"G\xe9n\xe9rer des statistiques d'usage du service", 'properties': ('mandatory', 'expert')})
    option_139789497209184.impl_set_information("help", u"""Génère des statistiques d'usage du service sur l'URL /metrics""")
    option_139789497209456 = UnicodeOption(**{'doc': u"Nom d'entit\xe9 SAML du serveur eole-sso (ou rien)", 'multi': False, 'properties': ('normal',), 'name': 'eolesso_entity_name', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497209456.impl_set_information("help", u"""Nom d'entité du serveur EoleSSO local à indiquer dans les messages SAML (si rien, une valeur est calculée depuis le nom de l'académie et le nom de machine)""")
    option_139789497209728 = IntOption(**{'multi': False, 'name': 'eolesso_otppass_minsize', 'default': 10, 'doc': u'Taille minimum du passcode OTP', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497183776}]})
    option_139789497209728.impl_set_information("help", u"""Taille minimum des mots de passe OTP (code PIN + clé)""")
    option_139789497210000 = IntOption(**{'multi': False, 'name': 'eolesso_otppass_maxsize', 'default': 12, 'doc': u'Taille maximum du passcode OTP', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497183776}]})
    option_139789497210000.impl_set_information("help", u"""Taille maximum des mots de passe OTP (code PIN + clé)""")
    option_139789497210272 = UnicodeOption(**{'multi': False, 'name': 'eolesso_otppass_regx', 'default': u'^[0-9]{10,12}$', 'doc': u'Expression r\xe9guli\xe8re de d\xe9tection des passcodes OTP', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497183776}]})
    option_139789497210272.impl_set_information("help", u"""Expression régulière pour valider que le mot de passe saisi est de type OTP (vérifié quand la longueursaisie correspond au tailles min/max). Comprend le code PIN + la clé. Par défaut : chaîne de 12 caractères numériques.""")
    option_139789497210816 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'eolesso_otp_desync', 'default': 'non', 'doc': u'Gestion locale des cl\xe9s OTP d\xe9synchronis\xe9es', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497183776}]})
    option_139789497210816.impl_set_information("help", u"""Choix du mode de fonctionnement en cas de clé OTP désynchronisée (par ex. suite à plusieurs tentatives échouées).Si non, l'utilisateur est invité à débloquer la situation en allant sur une URL particulière. Si oui, EoleSSO va demander la saisie d'un deuxième mot de passe pour débloquer la situation""")
    option_139789497210544 = UnicodeOption(**{'doc': u'Adresse de la mire OTP en cas d\xe9synchronisation de cl\xe9', 'multi': False, 'properties': ('normal',), 'name': 'eolesso_otp_portal', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497210816}]})
    option_139789497210544.impl_set_information("help", u"""En cas de désynchronisation d'une clé OTP, l'utilisateur sera redirigé sur cette adresse pour débloquer la situation. Si vide, un message par défaut est affiché""")
    option_139789497211088 = UnicodeOption(**{'multi': False, 'name': 'eolesso_cert', 'doc': u'Chemin du certificat SSL', 'properties': ('normal',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497939632, False)), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497211088.impl_set_information("help", u"""Certificat utilisé par le serveur SSO""")
    option_139789497211360 = UnicodeOption(**{'multi': False, 'name': 'eolesso_key', 'doc': u'Chemin de la cl\xe9 priv\xe9e li\xe9e au certificat SSL', 'properties': ('normal',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497939904, False)), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497211360.impl_set_information("help", u"""Valeur ignorée si le certificat n'est pas défini. si rien et certificat défini: 'nom_certificat.key' dans le même répertoire.""")
    option_139789497211632 = UnicodeOption(**{'doc': u"Chemin de l'autorit\xe9 de certification (ou rien)", 'multi': False, 'properties': ('normal',), 'name': 'eolesso_ca_location', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497211632.impl_set_information("help", u"""Certificat de l'autorité de certification validant le certificat du serveur SSO (dans le cas d'une chaine de certification, tous les certificats de la chaine doivent être intégés)""")
    option_139789497211904 = IntOption(**{'default': 7200, 'doc': u"Dur\xe9e de vie d'une session sur le serveur SSO (en secondes)", 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'eolesso_session_timeout'})
    option_139789497212176 = UnicodeOption(**{'doc': u'CSS par d\xe9faut du service SSO (sans le .css)', 'multi': False, 'properties': ('normal',), 'name': 'eolesso_css', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497212176.impl_set_information("help", u"""CSS par défaut pour les pages du service SSO (sans le .css, le fichier est à placer dans /usr/share/sso/interface/[nom_fichier].css)""")
    option_139789497212448 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'eolesso_responsive', 'default': 'non', 'doc': u'Activer la balise meta viewport (CSS responsive)', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497212448.impl_set_information("help", u"""Inclut la balise meta 'viewport' dans les pages de l'application (avec content=\"width=device-width, initial-scale=1\"). A activer en cas d'utilisation d'une CSS 'responsive'.""")
    option_139789497229392 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'federation_transparente', 'default': 'non', 'doc': u"Cacher le formulaire lors de l'envoi des informations de f\xe9d\xe9ration", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789497229392.impl_set_information("help", u"""Lors de l'envoi d'attributs de l'utilisateur à un fournisseur tiers dans le cadre d'une fédération d'identité, un formulaire est présenté pour le prévenir. En choisissant 'oui', le formulaire sera envoyé automatiquement sans être affiché.""")
    option_139789497229664 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'cas_send_logout', 'default': 'oui', 'doc': u'Activer le logout centralis\xe9 du serveur SSO', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789497230208 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'cas_verify_service', 'default': 'non', 'doc': u'Ne pas r\xe9pondre aux demandes CAS des applications inconnues', 'properties': ('mandatory', 'expert')})
    option_139789497230208.impl_set_information("help", u"""Si ce paramètre est à oui, seules les applications renseignées dans les fichiers d'applications (/usr/share/sso/app_filters/*_apps.ini) sont autorisées à recevoir des réponses du serveur en mode CAS. Si il est à non, le filtre par défaut leur sera appliqué""")
    option_139789497229936 = IntOption(**{'default': -300, 'doc': u'D\xe9calage de temps (en secondes) dans les messages de f\xe9d\xe9ration SAML', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'sso_saml_time_adjust'})
    option_139789497229936.impl_set_information("help", u"""Décalage en secondes appliqué aux dates dans les messages de fédération SAML. Permet d'éviter le rejet des messages lorsque le serveur partenaire n'est pas tout à fait synchrone (par défaut, on décale de 5 minutes dans le passé). Ce délai est aussi pris en compte pour la validation des messages reçus.""")
    option_139789497230480 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ead_sso', 'default': 'oui', 'doc': u"Utiliser l'authentification SSO pour l'EAD", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}]})
    option_139789497230752 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_openid', 'default': 'non', 'doc': u"Autoriser l'authentification OpenID Connect", 'properties': ('mandatory', 'expert')})
    option_139789497230752.impl_set_information("separator", ('Authentification OpenID', None))
    option_139789497231024 = UnicodeOption(**{'doc': u"R\xe9f\xe9rence du fournisseur d'identit\xe9 OpenID", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'openid_providers', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497230752}]})
    option_139789497231024.impl_set_information("proposed_value", (u'fconnect',))
    option_139789497231024.impl_set_information("help", u"""nom du fournisseur d'identité. Non affiché, utilisé dans le nom des images (logos)""")
    option_139789497231296 = UnicodeOption(**{'multi': True, 'name': 'openid_providers_label', 'doc': u"Libell\xe9 du fournisseur d'identit\xe9 OpenID", 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("fconnect"),), kwargs={'condition_1': ParamOption(option_139789497231024, False), 'default_mismatch': ParamValue("None"), 'match': ParamValue("France Connect")})})
    option_139789497231296.impl_set_information("help", u"""Libellé présenté aux utilisateurs pour décrire le fournisseur""")
    option_139789497231568 = URLOption(**{'multi': True, 'name': 'openid_providers_issuer', 'allow_ip': True, 'allow_without_dot': True, 'doc': u"URL d'acc\xe8s (issuer)", 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("fconnect"),), kwargs={'condition_1': ParamOption(option_139789497231024, False), 'default_mismatch': ParamValue("None"), 'match': ParamValue("https://fcp.integ01.dev-franceconnect.fr/api/v1/")})})
    option_139789497231568.impl_set_information("help", u"""URL de base du fournisseur d'identité (généralement appelée issuer)""")
    option_139789497231840 = URLOption(**{'multi': True, 'name': 'openid_providers_about_url', 'allow_ip': True, 'allow_without_dot': True, 'doc': u"URL d'information (A propos)", 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("fconnect"),), kwargs={'condition_1': ParamOption(option_139789497231024, False), 'default_mismatch': ParamValue("None"), 'match': ParamValue("https://fcp.integ01.dev-franceconnect.fr/a-propos")})})
    option_139789497232112 = UnicodeOption(**{'multi': True, 'name': 'openid_providers_about_label', 'doc': u"Libell\xe9 de l'URL d'information (A propos)", 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("fconnect"),), kwargs={'condition_1': ParamOption(option_139789497231024, False), 'default_mismatch': ParamValue("None"), 'match': ParamValue("Qu'est ce que FranceConnect ?")})})
    option_139789497232384 = URLOption(**{'multi': True, 'name': 'openid_providers_auth_endp', 'allow_ip': True, 'allow_without_dot': True, 'doc': u"URL de demande d'autorisation (authorization endpoint)", 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("fconnect"),), kwargs={'condition_1': ParamOption(option_139789497231024, False), 'default_mismatch': ParamValue("None"), 'match': ParamValue("https://fcp.integ01.dev-franceconnect.fr/api/v1/authorize")})})
    option_139789497232384.impl_set_information("help", u"""URL de réception des requêtes d'authentification.""")
    option_139789497232656 = URLOption(**{'multi': True, 'name': 'openid_providers_reg_endp', 'allow_ip': True, 'allow_without_dot': True, 'doc': u"URL d'enregistrement automatique (register endpoint)", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("fconnect"),), kwargs={'condition_1': ParamOption(option_139789497231024, False), 'default_mismatch': ParamValue("None"), 'match': ParamValue("https://fcp.integ01.dev-franceconnect.fr/api/v1/register")})})
    option_139789497232656.impl_set_information("help", u"""URL d'enregistrement automatique du client (non géré)""")
    option_139789497232928 = URLOption(**{'multi': True, 'name': 'openid_providers_token_endp', 'allow_ip': True, 'allow_without_dot': True, 'doc': u"URL de r\xe9cup\xe9ration de jeton d'acc\xe8s (token endpoint)", 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("fconnect"),), kwargs={'condition_1': ParamOption(option_139789497231024, False), 'default_mismatch': ParamValue("None"), 'match': ParamValue("https://fcp.integ01.dev-franceconnect.fr/api/v1/token")})})
    option_139789497232928.impl_set_information("help", u"""URL de récupération de jeton d'accès (access token)""")
    option_139789497233488 = URLOption(**{'multi': True, 'name': 'openid_providers_logout_endp', 'allow_ip': True, 'allow_without_dot': True, 'doc': u'URL de d\xe9connexion (logout endpoint)', 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("fconnect"),), kwargs={'condition_1': ParamOption(option_139789497231024, False), 'default_mismatch': ParamValue("None"), 'match': ParamValue("https://fcp.integ01.dev-franceconnect.fr/api/v1/logout")})})
    option_139789497233488.impl_set_information("help", u"""URL de déconnexion du fournisseur d'identité (non utilisé pour Google,Facebook,Live)""")
    option_139789497233760 = URLOption(**{'multi': True, 'name': 'openid_providers_userinfo_endp', 'allow_ip': True, 'allow_without_dot': True, 'doc': u'URL de lecture des informations (userinfo endpoint)', 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("fconnect"),), kwargs={'condition_1': ParamOption(option_139789497231024, False), 'default_mismatch': ParamValue("None"), 'match': ParamValue("https://fcp.integ01.dev-franceconnect.fr/api/v1/userinfo")})})
    option_139789497233760.impl_set_information("help", u"""URL de récupération des informations sur l'utilisateur (non utilisé si renvoyées avec le jeton d'accès)""")
    option_139789497234032 = URLOption(**{'multi': True, 'name': 'openid_providers_jwks_uri', 'doc': u'URL de description des certificats de signature (jwks URI)', 'allow_ip': True, 'allow_without_dot': True, 'properties': ('normal',)})
    option_139789497234032.impl_set_information("help", u"""URL décrivant les certificats utilisés pour la signature des jetons (si disponible)""")
    option_139789497129872 = Leadership(children=[option_139789497231024, option_139789497231296, option_139789497231568, option_139789497231840, option_139789497232112, option_139789497232384, option_139789497232656, option_139789497232928, option_139789497233488, option_139789497233760, option_139789497234032], **{'doc': u"R\xe9f\xe9rence du fournisseur d'identit\xe9 OpenID", 'properties': ('basic',), 'name': 'openid_providers'})
    option_139789497130192 = OptionDescription(children=[option_139789497414528, option_139789497180512, option_139789497180784, option_139789497181056, option_139789497181328, option_139789497129552, option_139789497196896, option_139789497200160, option_139789497208912, option_139789497209184, option_139789497209456, option_139789497183776, option_139789497209728, option_139789497210000, option_139789497210272, option_139789497210816, option_139789497210544, option_139789497211088, option_139789497211360, option_139789497211632, option_139789497211904, option_139789497212176, option_139789497212448, option_139789497229392, option_139789497229664, option_139789497230208, option_139789497229936, option_139789497230480, option_139789497230752, option_139789497129872], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}], **{'doc': u'EOLE SSO', 'properties': ('basic',), 'name': 'eole_sso'})
    option_139789497130192.impl_set_information("help", u"""Configuration du serveur d'authentification EoleSSO""")
    option_139789497130192.impl_set_information("icon", u"""group""")
    option_139789497234304 = DomainnameOption(**{'type_': 'domainname', 'name': 'nom_domaine_local_supp', 'doc': u'Nom domaine local suppl\xe9mentaire ou rien', 'allow_ip': False, 'multi': True, 'properties': ('expert',)})
    option_139789497234304.impl_set_information("separator", (u'Ajout de domaines locaux suppl\xe9mentaires (DNS master sur ces domaines)', None))
    option_139789497234576 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_zone_forward', 'default': 'non', 'doc': u'D\xe9clarer des zones DNS \xe0 forwarder', 'properties': ('mandatory', 'expert')})
    option_139789497234576.impl_set_information("separator", ('Forward de zones DNS', None))
    option_139789497234848 = DomainnameOption(**{'type_': 'domainname', 'name': 'nom_zone_forward', 'doc': u'Nom DNS de la zone', 'allow_ip': False, 'multi': True, 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497234576}]})
    option_139789497235120 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'ip_dns_zone_forward', 'doc': u'Adresse IP du serveur DNS de la zone', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497234576}]})
    option_139789497130512 = Leadership(children=[option_139789497234848, option_139789497235120], **{'doc': u'Nom DNS de la zone', 'properties': ('expert',), 'name': 'nom_zone_forward'})
    option_139789497258064 = OptionDescription(children=[option_139789497234304, option_139789497234576, option_139789497130512], **{'doc': u'Zones-DNS', 'properties': ('expert',), 'name': 'zones_dns'})
    option_139789497258064.impl_set_information("help", u"""Personnalisation des zones DNS""")
    option_139789497258064.impl_set_information("icon", u"""random""")
    option_139789497235392 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_ead_reverseproxy', 'default': 'non', 'doc': u"Activer l'interface web de l'EAD sur un second port", 'properties': ('mandatory', 'expert')})
    option_139789497235392.impl_set_information("help", u"""Ce second port peut, par exemple, permettre d'accéder à l'interface depuis l'extérieur si un proxy inverse est utilisé""")
    option_139789497235664 = PortOption(**{'multi': False, 'name': 'port_ead_reverseproxy', 'allow_private': True, 'default': '4203', 'doc': u"Port d'acc\xe8s EAD personnalis\xe9", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497235392}], "validator": eosfunc.valid_differ, 'validator_params': Params(args=(ParamValue(u"4200"),), kwargs={})})
    option_139789497258384 = OptionDescription(children=[option_139789497235392, option_139789497235664], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964480}], **{'doc': u'EAD-web', 'properties': ('expert',), 'name': 'ead_web'})
    option_139789497258384.impl_set_information("help", u"""Paramétrage avancé de l'interface web EAD""")
    option_139789497258384.impl_set_information("icon", u"""gear""")
    option_139789497235936 = IntOption(**{'default': 200, 'doc': u'Nombre maximum de connexions simultan\xe9es', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'mysql_max_connexions'})
    option_139789497235936.impl_set_information("separator", ('Configuration', None))
    option_139789497236208 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'mysql_open_firewall', 'default': 'non', 'doc': u"Permettre l'acc\xe8s au serveur depuis l'ext\xe9rieur", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789497236480 = NetworkOption(**{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions distantes au serveur MYSQL', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'ip_my_dist', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497236208}]})
    option_139789497236752 = NetmaskOption(**{'doc': u'Masque du sous r\xe9seau pour les connexions distantes au serveur MySQL', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'mask_my_dist', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497236208}]})
    option_139789497236752.impl_add_consistency("network_netmask", option_139789497236480, **{'warnings_only': True})
    option_139789497237024 = ChoiceOption(**{'multi': True, 'values': (u'auto', u'all', u'eth0', u'eth1', u'eth2', u'eth3', u'eth4'), 'name': 'if_my_dist', 'default_multi': u'auto', 'doc': u'Interface pour les connexions distantes au serveur MySQL', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497236208}]})
    option_139789497258704 = Leadership(children=[option_139789497236480, option_139789497236752, option_139789497237024], **{'doc': u'Adresse IP r\xe9seau autoris\xe9e pour les connexions distantes au serveur MYSQL', 'properties': ('expert',), 'name': 'ip_my_dist'})
    option_139789497259024 = OptionDescription(children=[option_139789497235936, option_139789497236208, option_139789497258704], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971312}], **{'doc': u'MySQL', 'properties': ('expert',), 'name': 'mysql'})
    option_139789497259024.impl_set_information("help", u"""Paramétrage avancé du serveur de gestion de bases de données MySQL""")
    option_139789497259024.impl_set_information("icon", u"""mysql-alt""")
    option_139789496746064 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_spamassassin', 'default': 'oui', 'doc': u'Activer le service anti-spam SpamAssassin', 'properties': ('mandatory', 'normal')})
    option_139789496746064.impl_set_information("separator", ('Service anti-spam', None))
    option_139789496746064.impl_set_information("help", u"""Le but de ce logiciel est de filtrer le trafic des courriels pour éradiquer les courriels reconnus comme pourriels ou courriels non sollicités""")
    option_139789496746336 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'adresse_ip_mail', 'doc': u'Adresse IP du serveur de mail local', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("oui"),), kwargs={'mismatch': ParamOption(option_139789497538768, False), 'condition_1': ParamOption(option_139789497889120, False), 'default_mismatch': ParamValue("None"), 'match': ParamOption(option_139789497853072, False)})})
    option_139789496746608 = DomainnameOption(**{'multi': False, 'name': 'adresse_imap', 'allow_ip': True, 'allow_without_dot': True, 'type_': 'domainname', 'doc': u'Adresse du serveur de r\xe9cup\xe9ration de courrier \xe9lectronique IMAP', 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("local"),), kwargs={'condition_1': ParamOption(option_139789497138720, False), 'default_mismatch': ParamValue("None"), 'match': ParamOption(option_139789496746336, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497138720}, {'expected': u'local', 'action': 'frozen', 'inverse': False, "option": option_139789497138720}, {'expected': u'local', 'action': 'hidden', 'inverse': False, "option": option_139789497138720}, {'expected': u'local', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497138720}]})
    option_139789496747152 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_courier_imap', 'default': 'oui', 'doc': u'Activer le serveur IMAP de Courier', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497138720}]})
    option_139789496747152.impl_set_information("help", u"""Courier est un serveur d'échange de courrier électronique via le protocole IMAP""")
    option_139789496746880 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_courier_pop', 'default': 'non', 'doc': u'Activer le serveur POP de Courier', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497138720}]})
    option_139789496746880.impl_set_information("help", u"""Courier est un serveur d'échange de courrier électronique via le protocole POP""")
    option_139789496747424 = ChoiceOption(**{'multi': False, 'name': 'activer_courier_commun', 'doc': u'Activer les services communs pour Courier', 'values': (u'oui', u'non'), 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("oui"), 'condition_1': ParamOption(option_139789496747152, False), 'match': ParamValue("non"), 'condition_2': ParamOption(option_139789496746880, False)}), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497138720}]})
    option_139789496747696 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_courier_imap_sso', 'default': 'oui', 'doc': u"Autoriser l'authentification CAS pour le serveur IMAP de Courier", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496747152}]})
    option_139789496747968 = DomainnameOption(**{'multi': False, 'name': 'adresse_imap_sso', 'allow_ip': True, 'allow_without_dot': True, 'type_': 'domainname', 'doc': u'Adresse du serveur IMAP utilis\xe9 pour le client CAS/IMAP', 'properties': ('mandatory', 'expert'), 'callback': tiramisu_copy, 'callback_params': Params(args=(ParamOption(option_139789496746608, False),), kwargs={}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496747696}]})
    option_139789496747968.impl_set_information("help", u"""Il faut renseigner l'adresse du serveur IMAP définit sur le client IMAP avec authentification CAS (par exemple roundcube)""")
    option_139789496748240 = UnicodeOption(**{'multi': False, 'name': 'proxy_cachemaster', 'doc': u'Adresse courriel du cachemaster', 'properties': ('normal',), 'callback': eosfunc.concat, 'callback_params': Params(args=(ParamValue("cachemaster@"), ParamOption(option_139789497891296, False), ParamValue("."), ParamOption(option_139789497891568, False)), kwargs={}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789496748240.impl_set_information("help", u"""Adresse électronique proposée pour les réclamations liées au filtrage des sites""")
    option_139789496748512 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_smarthost', 'default': 'oui', 'doc': u'G\xe9rer la distribution locale', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789496748512.impl_set_information("help", u"""Gestion d’un domaine local et d’un domaine privé (préfixe \"i-\")""")
    option_139789496748784 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_sympa', 'default': 'oui', 'doc': u'Activer le serveur de listes de diffusion Sympa', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748512}]})
    option_139789496748784.impl_set_information("separator", ('Serveur de listes', None))
    option_139789496748784.impl_set_information("help", u"""Sympa (Système de Multi-Postage Automatique) est un gestionnaire de listes électroniques""")
    option_139789496749056 = EmailOption(**{'doc': u"Adresse \xe9lectronique d'envoi pour le compte root", 'multi': False, 'properties': ('normal',), 'name': 'system_mail_from'})
    option_139789496749056.impl_set_information("help", u"""Adresse utilisée comme expéditeur des courriers électroniques de l'utilisateur root""")
    option_139789496749328 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'system_mail_from_for_headers', 'default': 'non', 'doc': u"\xc9craser les ent\xeates 'From:', 'Reply-To:' et 'Sender:' du message", 'properties': ('mandatory', 'expert')})
    option_139789496749600 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'system_mail_to_for_headers', 'default': 'non', 'doc': u"\xc9craser les ent\xeates 'To:', 'Cc:' et 'Bcc:' du message", 'properties': ('mandatory', 'expert')})
    option_139789496758352 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_ldap_delivery', 'default': 'oui', 'doc': u'G\xe9rer la distribution pour les comptes LDAP', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789496758624 = IntOption(**{'multi': False, 'name': 'exim_quota', 'default': 20, 'doc': u'Quota des bo\xeetes aux lettres en Mo', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748512}], "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"0")})})
    option_139789496758624.impl_set_information("help", u"""Taille par défaut, en Mo, des boîtes aux lettres.""")
    option_139789496758896 = IntOption(**{'multi': False, 'name': 'exim_quota_warn', 'default': 80, 'doc': u'Pourcentage d\u2019utilisation des bo\xeetes entra\xeenant un warning', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748512}], "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"0")})})
    option_139789496758896.impl_set_information("help", u"""Limite d’utilisation, en pourcent, d’une boîte aux lettres entraînant
            l’envoi d’un message d’alerte.""")
    option_139789496759168 = IntOption(**{'multi': False, 'name': 'exim_spam_score', 'default': 50, 'doc': u"Seuil de d\xe9tection d'un spam (multipli\xe9 par 10)", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496746064}]})
    option_139789496759168.impl_set_information("help", u"""Seuil à partir duquel un courrier électronique est considéré
            comme spam. Le seuil fourni par SpamAssassin est multiplié
            par 10 afin de faire des comparaisons sur des entiers.""")
    option_139789496759440 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_use_tls', 'default': 'oui', 'doc': u'Activer le TLS pour les clients', 'properties': ('mandatory', 'expert')})
    option_139789496759440.impl_set_information("help", u"""Activer le support du TLS pour les sessions SMTP.
            Le serveur de courrier supportera alors STARTTLS sur le port 25 et écoutera en TLS sur le port 465 pour les clients qui le supportent.
            Passer cette variable à non rend l'authentification SMTP impossible ce qui empêche les utilisateurs d'envoyer des messages.""")
    option_139789496759712 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_exim_relay_smtp', 'default': 'non', 'doc': u'Router les courriels par une passerelle SMTP', 'properties': ('mandatory', 'basic')})
    option_139789496759712.impl_set_information("separator", ('Relai des messages', None))
    option_139789496759712.impl_set_information("help", u"""Activer l’utilisation d’une passerelle SMTP pour l’envoi
            des courriers électroniques vers l’extérieurs. Si cette option est
            désactivée, le réseau doit autoriser le protocole SMTP
            depuis ce serveur vers Internet.""")
    option_139789496759984 = DomainnameOption(**{'multi': False, 'name': 'exim_relay_smtp', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Passerelle SMTP', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496759712}]})
    option_139789496759984.impl_set_information("help", u"""Passerelle SMTP utilisée pour l’envoi des courriers électroniques vers
            l’extérieur. Le protocole SMTP doit être autorisé entre
            le serveur local et cette passerelle.""")
    option_139789496760256 = ChoiceOption(**{'multi': False, 'values': (u'non', u'port 25', u'port 465'), 'name': 'tls_smtp', 'default': u'non', 'doc': u'Utilisation du TLS (SSL) par la passerelle SMTP', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496759712}]})
    option_139789496760256.impl_set_information("help", u"""Si la passerelle SMTP accepte le TLS, choisir le port en
            fonction du support de la commande STARTTLS (port 25) ou
            non (port 465)""")
    option_139789496760528 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_relay', 'default': 'non', 'doc': u'Activer le relai des messages', 'properties': ('mandatory', 'expert')})
    option_139789496760528.impl_set_information("help", u"""Activer la configuration manuelle du relai des courriers électroniques.
            Cette variable permet d’activer :
            - Les listes d’adresses IP autorisées à utiliser ce serveur comme relai de messagerie.
            - La liste des noms de domaines autorisées à être relayés par ce serveur.
            Si vous n’êtes pas sûr, mettre la valeur « non ».""")
    option_139789496760800 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_relay_dhcp', 'default': 'non', 'doc': u'Relayer les courriers \xe9lectroniques pour toutes les plages d\xe9finies dans le DHCP', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496760528}]})
    option_139789496760800.impl_set_information("help", u"""Activer le relai des courriers électroniques pour les plages d’adresses
            définies dans la configuration DHCP.
            Si toutes les plages d’adresses ne sont pas autoriser à
            utiliser ce serveur comme relai de messagerie, vous devez
            spécifier « non » et activer le relai manuel.""")
    option_139789496761072 = UnicodeOption(**{'doc': u'Relayer les courriers \xe9lectroniques pour des plages d\u2019adresses IPv4', 'multi': True, 'properties': ('expert',), 'name': 'exim_relay_cidrv4', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496760528}]})
    option_139789496761072.impl_set_information("help", u"""Liste des adresses IPv4 autorisées à utiliser ce serveur comme
            relai de messagerie.
            Les adresses doivent être spécifiés en notation CIDR.""")
    option_139789496761344 = UnicodeOption(**{'doc': u'Relayer les courriers \xe9lectroniques pour des plages d\u2019adresses IPv6', 'multi': True, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'name': 'exim_relay_cidrv6'})
    option_139789496761344.impl_set_information("help", u"""Liste des adresses IPv6 autorisées à utiliser ce serveur comme
            relai de messagerie.
            Les adresses doivent être spécifiés en notation CIDR.
            À l’heure actuelle, ce paramètre n’ayant aucun effet il est caché.""")
    option_139789496761616 = UnicodeOption(**{'doc': u'Relayer les courriers \xe9lectroniques pour des nom de domaines', 'multi': True, 'properties': ('expert',), 'name': 'exim_relay_domains', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496760528}]})
    option_139789496761616.impl_set_information("help", u"""Liste des noms de domaines autorisées à être relayés par ce
            serveur.
            Ce paramètre est utilisé pour des MX secondaires ou un frontal
            à plusieurs domaines de messagerie.
            La syntaxe autorise le joker « * » et bien plus encore
            (http://exim.org/exim-html-current/doc/html/spec_html/ch10.html#SECTdomainlist)
            Si vous n’êtes pas sûr, laissez vide.""")
    option_139789496761888 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_relay_manual_routes', 'default': 'non', 'doc': u'Activer le relai avec des routes manuelles (EAD)', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789496761888.impl_set_information("help", u"""Activation du routage manuel des messages en utilisant la
            liste de correspondance « domaine: passerelle SMTP »
            définie dans l’EAD.""")
    option_139789496770640 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'test_smtp_access', 'doc': u"V\xe9rifie si les ports SMTP doivent \xeatre accessibles depuis l'ext\xe9rieur", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("oui"),), kwargs={'operator': ParamValue("OR"), 'condition_1': ParamOption(option_139789496748512, False), 'condition_2': ParamOption(option_139789496760528, False)})})
    option_139789496771184 = UnicodeOption(**{'default': u'automatique', 'doc': u'FQDN utilis\xe9 par Exim', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'exim_primary_hostname'})
    option_139789496771184.impl_set_information("proposed_value", (u'automatique', u'nom_machine.domaine_messagerie_etab', u'nom_machine.nom_domaine_local'))
    option_139789496771184.impl_set_information("separator", ('Configuration experte', None))
    option_139789496771184.impl_set_information("help", u"""Nom de domaine complètement qualifié utilisé par Exim dans
            le protocole SMTP. C'est utile pour les
            vérifications anti-spam des MX externes.
            Les valeurs possibles sont :
            - automatique : laisser Exim décider ;
            - nom_machine.domaine_messagerie_etab : utiliser le
              nom de la machine complété par le nom de domaine de la
              messagerie établissement ;
            - nom_machine.nom_domaine_local : utiliser le
              nom de la machine complété par le nom de domaine local.""")
    option_139789496770912 = ChoiceOption(**{'multi': False, 'values': (u'nom de domaine local', u'domaine priv\xe9 de messagerie \xe9tablissement', u'domaine public de messagerie \xe9tablissement'), 'name': 'exim_qualify_domain', 'default': u'nom de domaine local', 'doc': u'Domaine utilis\xe9 pour qualifier les adresses', 'properties': ('mandatory', 'expert')})
    option_139789496770912.impl_set_information("help", u"""Nom de domaine ajouté aux adresses :
            - non qualifiées (sans @DOMAIN) ;
            - qualifié avec un nom de machine sans domaine (@NOM_MACHINE).
            Les valeurs configurables sont :
            - nom de domaine local : correspond à « %%nom_domain_local » ;
            - domaine privé de messagerie établissement : correspond à « i-%%domaine_messagerie_etab » ;
            - domaine public de messagerie établissement : correspond à « %%domaine_messagerie_etab ».
            Si « %%domaine_messagerie_etab » est vide, « %%nom_domain_local » est utilisé.""")
    option_139789496771456 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_use_syslog', 'default': 'oui', 'doc': u'Envoyer les logs par rsyslog', 'properties': ('mandatory', 'expert')})
    option_139789496771456.impl_set_information("help", u"""Activer l’envoi des logs par rsyslog.""")
    option_139789496771728 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_use_log_files', 'default': 'non', 'doc': u'Dupliquer les logs dans des fichiers', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496771456}]})
    option_139789496771728.impl_set_information("help", u"""Dupliquer les logs dans des fichiers gérés directement par
            Exim.
            Si vous envoyez les logs par rsyslog, vous pouvez conserver la
            gestion des fichiers traditionnelles d’Exim.
            Ces fichiers étant gérés directement par Exim, ils se
            trouveront dans le conteneurs du service.""")
    option_139789496772000 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'exim_address_rewrite', 'default': 'non', 'doc': u'Activer les r\xe8gles de r\xe9\xe9criture \xe9tendue', 'properties': ('mandatory', 'expert')})
    option_139789496772000.impl_set_information("help", u"""Permettre de définir des règles de réécriture personnalisées. Si non, seuls les courriers électroniques en 'localhost' sont réécris avec le 'nom_domain_local'. Voir http://exim.org/exim-html-current/doc/html/spec_html/ch31.html.""")
    option_139789496772272 = UnicodeOption(**{'doc': u'Sch\xe9ma de r\xe9\xe9criture', 'multi': True, 'properties': ('expert',), 'name': 'exim_address_rewrite_pattern', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496772000}]})
    option_139789496772272.impl_set_information("help", u"""Modèle de correspondance des adresses courriers électroniques à réécrire.
            http://exim.org/exim-html-current/doc/html/spec_html/ch31.html#SECID151""")
    option_139789496772544 = UnicodeOption(**{'doc': u'Remplacement de r\xe9\xe9criture', 'multi': True, 'properties': ('expert',), 'name': 'exim_address_rewrite_replacement', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496772000}]})
    option_139789496772544.impl_set_information("help", u"""Valeur de remplacement des adresses électroniques.
            http://exim.org/exim-html-current/doc/html/spec_html/ch31.html#SECID152""")
    option_139789496772816 = UnicodeOption(**{'multi': True, 'name': 'exim_address_rewrite_flags', 'doc': u'Drapeau de r\xe9\xe9criture', 'properties': ('expert',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496772000}], "validator": eosfunc.valid_regexp, 'validator_params': Params(args=(ParamValue(u"^[EFTbcfhrstSQqRw\s]*$"),), kwargs={'err_msg': ParamValue(u"Les flags doivent être une série des caractères suivants: EFTbcfhrst S QqRw")})})
    option_139789496772816.impl_set_information("help", u"""Drapeau contrôlant la réécriture des adresses électroniques.
            Voir http://exim.org/exim-html-current/doc/html/spec_html/ch31.html#SECID153""")
    option_139789497259344 = Leadership(children=[option_139789496772272, option_139789496772544, option_139789496772816], **{'doc': u'Sch\xe9ma de r\xe9\xe9criture', 'properties': ('expert',), 'name': 'exim_address_rewrite_pattern'})
    option_139789497259664 = OptionDescription(children=[option_139789496746064, option_139789497138720, option_139789496746608, option_139789496747152, option_139789496746880, option_139789496747424, option_139789496747696, option_139789496747968, option_139789496748240, option_139789496748784, option_139789496746336, option_139789497445920, option_139789496749056, option_139789496749328, option_139789497462864, option_139789496749600, option_139789496748512, option_139789496758352, option_139789496758624, option_139789496758896, option_139789496759168, option_139789496759440, option_139789496759712, option_139789496759984, option_139789496760256, option_139789496760528, option_139789496760800, option_139789496761072, option_139789496761344, option_139789496761616, option_139789496761888, option_139789496770640, option_139789496771184, option_139789496770912, option_139789496771456, option_139789496771728, option_139789496772000, option_139789497259344], **{'doc': u'Messagerie', 'properties': ('basic',), 'name': 'messagerie'})
    option_139789497259664.impl_set_information("help", u"""Paramétrage du serveur de messagerie""")
    option_139789497259664.impl_set_information("icon", u"""envelope""")
    option_139789496773088 = ChoiceOption(**{'multi': False, 'values': (u'eole', u'sdet', u'zephir'), 'name': 'ldap_schema', 'default': u'eole', 'doc': u'Sch\xe9ma LDAP utilis\xe9', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789496773360 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ldap_replication', 'default': 'non', 'doc': u'Activer la r\xe9plication LDAP (fournisseur)', 'properties': ('mandatory', 'expert')})
    option_139789496773632 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ldap_replication_client', 'default': 'non', 'doc': u'Activer la r\xe9plication LDAP (client)', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789496774176 = IntOption(**{'default': 0, 'doc': u'Niveau de log', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_loglevel'})
    option_139789496773904 = IntOption(**{'default': 5000, 'doc': u"Nombre maximum d'entr\xe9es \xe0 retourner lors d'une requ\xeate", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_sizelimit'})
    option_139789496782928 = IntOption(**{'default': 3600, 'doc': u'Temps de r\xe9ponse maximum \xe0 une requ\xeate (en secondes)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_timelimit'})
    option_139789496783200 = IntOption(**{'default': 1000, 'doc': u"Taille du cache (en nombre d'entr\xe9es)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ldap_cachesize'})
    option_139789496783472 = ChoiceOption(**{'multi': False, 'values': (u'non', u'oui', u'uniquement'), 'name': 'ldap_ssl', 'default': u'non', 'doc': u'Activer LDAP sur le port SSL', 'properties': ('mandatory', 'expert')})
    option_139789496783472.impl_set_information("help", u"""Permet d'activer LDAP sur le port SSL (636). Si 'uniquement' est choisi, le firewall ne rend accessible que ce port. Le service reste néanmoins accessible en local sur le port 389.""")
    option_139789496783744 = ChoiceOption(**{'multi': False, 'values': (u'tous', u'authentifi\xe9', u'aucun'), 'name': 'ldap_restrict_access', 'default': u'tous', 'doc': u'Utilisateur autoris\xe9 \xe0 acc\xe9der \xe0 distance au serveur LDAP', 'properties': ('mandatory', 'expert')})
    option_139789497259984 = OptionDescription(children=[option_139789496773088, option_139789496773360, option_139789496773632, option_139789496774176, option_139789496773904, option_139789496782928, option_139789496783200, option_139789496783472, option_139789496783744], requires=[{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497955744}], **{'doc': u'OpenLDAP', 'properties': ('expert',), 'name': 'openldap'})
    option_139789497259984.impl_set_information("help", u"""Paramétrage de l'annuaire OpenLDAP local""")
    option_139789497259984.impl_set_information("icon", u"""book""")
    option_139789496784016 = FilenameOption(**{'default': u'/var/lib/bareos', 'doc': u"R\xe9pertoire d'extraction de la base de donn\xe9es du catalogue", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'bareos_db_extract_dir'})
    option_139789496784288 = PasswordOption(**{'callback_params': None, 'multi': False, 'name': 'bareos_db_mysql_password', 'doc': u'Mot de passe de la base de donn\xe9es MySQL de la sauvegarde', 'properties': ('auto_freeze', 'mandatory', 'expert'), 'callback': eosfunc.gen_random, "requires": [{'expected': u'mysql', 'action': 'disabled', 'inverse': True, "option": option_139789497145008}]})
    option_139789496784560 = UnicodeOption(**{'multi': False, 'name': 'bareos_dir_name', 'doc': u'Nom du directeur local', 'properties': ('auto_freeze', 'mandatory', 'basic'), 'callback': eosfunc.concat, 'callback_params': Params(args=None, kwargs={'valeur2': ParamValue("-dir"), 'valeur1': ParamOption(option_139789497890208, False)})})
    option_139789496784560.impl_set_information("help", u"""Nom identifiant le service bareos-dir""")
    option_139789496784832 = IntOption(**{'default': 6, 'doc': u'P\xe9riode de r\xe9tention des sauvegardes compl\xe8tes', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'bareos_full_retention'})
    option_139789496784832.impl_set_information("help", u"""Durée pendant laquelle le contenu des sauvegardes complètes est conservé avant écrasement""")
    option_139789496785104 = ChoiceOption(**{'multi': False, 'values': (u'years', u'months', u'weeks', u'days'), 'name': 'bareos_full_retention_unit', 'default': u'months', 'doc': u'Unit\xe9 de valeur pour la r\xe9tention des sauvegardes compl\xe8tes', 'properties': ('mandatory', 'normal')})
    option_139789496785104.impl_set_information("help", u"""Unité de la durée de rétention pour les sauvegardes complètes""")
    option_139789496785376 = IntOption(**{'default': 5, 'doc': u'P\xe9riode de r\xe9tention des sauvegardes diff\xe9rentielles', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'bareos_diff_retention'})
    option_139789496785376.impl_set_information("help", u"""Durée pendant laquelle le contenu des sauvegardes différentielles est conservé avant écrasement""")
    option_139789496785648 = ChoiceOption(**{'multi': False, 'values': (u'years', u'months', u'weeks', u'days'), 'name': 'bareos_diff_retention_unit', 'default': u'weeks', 'doc': u'Unit\xe9 de valeur pour la r\xe9tention des sauvegardes diff\xe9rentielles', 'properties': ('mandatory', 'normal')})
    option_139789496785648.impl_set_information("help", u"""Unité de la durée de rétention pour les sauvegardes différentielles""")
    option_139789496785920 = IntOption(**{'default': 10, 'doc': u'P\xe9riode de r\xe9tention des sauvegardes incr\xe9mentales', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'bareos_inc_retention'})
    option_139789496785920.impl_set_information("help", u"""Durée pendant laquelle le contenu des sauvegardes incrémentales est conservé avant écrasement""")
    option_139789496786192 = ChoiceOption(**{'multi': False, 'values': (u'years', u'months', u'weeks', u'days'), 'name': 'bareos_inc_retention_unit', 'default': u'days', 'doc': u'Unit\xe9 de valeur pour la r\xe9tention des sauvegardes incr\xe9mentales', 'properties': ('mandatory', 'normal')})
    option_139789496786192.impl_set_information("help", u"""Unité de la durée de rétention pour les sauvegardes incrémentales""")
    option_139789496786464 = IntOption(**{'default': 86400, 'doc': u"Dur\xe9e maximale pour l'ex\xe9cution compl\xe8te d'une sauvegarde (en secondes)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'bareos_max_run_time'})
    option_139789496786464.impl_set_information("help", u"""Spécifie le délai, en secondes, alloué pour l'exécution complète d'un job depuis son lancement (pas nécessairement à l'heure de sa programmation) jusqu'à sa fin (0 pour pas de limite)""")
    option_139789496791120 = ChoiceOption(**{'multi': False, 'values': (u'Aucune', u'GZIP1', u'GZIP2', u'GZIP3', u'GZIP4', u'GZIP5', u'GZIP6', u'GZIP7', u'GZIP8', u'GZIP9'), 'name': 'bareos_compression', 'default': u'GZIP6', 'doc': u'Niveau de compression des sauvegardes', 'properties': ('mandatory', 'expert')})
    option_139789496791120.impl_set_information("help", u"""Niveau de compression gzip appliqué aux données sauvegardées""")
    option_139789496791392 = PasswordOption(**{'callback_params': None, 'multi': False, 'name': 'bareos_dir_password', 'doc': u'Mot de passe du directeur', 'properties': ('force_store_value', 'expert'), 'callback': eosfunc.gen_random})
    option_139789496791392.impl_set_information("help", u"""Mot de passe à transmettre aux applications distantes pour leur permettre de s'authentifier auprès du directeur""")
    option_139789496791664 = PasswordOption(**{'callback_params': None, 'multi': False, 'name': 'bareos_fd_password', 'doc': u'Mot de passe du serveur de fichiers', 'properties': ('force_store_value', 'frozen', 'hidden', 'basic'), 'callback': eosfunc.gen_random})
    option_139789496791936 = ChoiceOption(**{'multi': False, 'name': 'bareos_sd_local', 'doc': u'Le gestionnaire du stockage est local', 'values': (u'oui', u'non'), 'properties': ('normal',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497971040, False)), "requires": [{'expected': u'non', 'action': 'frozen', 'inverse': False, "option": option_139789497971040, 'same_action': False}, {'expected': u'non', 'action': 'hidden', 'inverse': False, "option": option_139789497971040, 'same_action': False}, {'expected': u'non', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497971040, 'same_action': False}]})
    option_139789496791936.impl_set_information("separator", ('Gestion du stockage', None))
    option_139789496791936.impl_set_information("help", u"""Choix du service bareos-sd gérant l'écriture des volumes de sauvegardes""")
    option_139789496792208 = DomainnameOption(**{'multi': False, 'name': 'bareos_sd_adresse', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Adresse du serveur de stockage distant', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789496791936}]})
    option_139789496792208.impl_set_information("help", u"""Adresse du service bareos-sd gérant l'écriture des volumes de sauvegardes""")
    option_139789496792480 = PasswordOption(**{'callback_params': None, 'multi': False, 'name': 'bareos_sd_password', 'doc': u'Mot de passe du serveur de stockage distant', 'properties': ('force_store_value', 'mandatory', 'basic'), 'callback': eosfunc.gen_random, "requires": [{'expected': u'oui', 'action': 'frozen', 'inverse': False, "option": option_139789496791936, 'same_action': False}, {'expected': u'oui', 'action': 'hidden', 'inverse': False, "option": option_139789496791936, 'same_action': False}]})
    option_139789496792480.impl_set_information("help", u"""Mot de passe permettant d'identifier le service bareos-sd du serveur distant (ressource Storage de la configuration du service bareos-dir local)""")
    option_139789497260304 = OptionDescription(children=[option_139789497145008, option_139789496784016, option_139789496784288, option_139789496784560, option_139789496784832, option_139789496785104, option_139789496785376, option_139789496785648, option_139789496785920, option_139789496786192, option_139789496786464, option_139789496791120, option_139789496791392, option_139789496791664, option_139789496791936, option_139789496792208, option_139789496792480], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}], **{'doc': u'Directeur Bareos', 'properties': ('basic',), 'name': 'directeur_bareos'})
    option_139789497260304.impl_set_information("help", u"""Configuration de la sauvegarde du serveur""")
    option_139789497260304.impl_set_information("icon", u"""archive""")
    option_139789496792752 = UnicodeOption(**{'multi': False, 'name': 'bareos_sd_name', 'doc': u'Nom du serveur de stockage', 'properties': ('mandatory', 'normal'), 'callback': eosfunc.concat, 'callback_params': Params(args=None, kwargs={'valeur2': ParamValue("-sd"), 'valeur1': ParamOption(option_139789497890208, False)})})
    option_139789496793024 = UnicodeOption(**{'doc': u'Nom du directeur Bareos distant', 'multi': True, 'properties': ('normal',), 'name': 'bareos_sd_remote_dir_name', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971040}]})
    option_139789496793024.impl_set_information("separator", (u'Configuration des acc\xe8s distants au stockage', None))
    option_139789496793024.impl_set_information("help", u"""Nom du service bareos-dir du serveur distant à sauvegarder""")
    option_139789496793296 = DomainnameOption(**{'multi': True, 'name': 'bareos_sd_remote_ip', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Adresse IP du directeur distant', 'properties': ('normal',), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971040}]})
    option_139789496793296.impl_set_information("help", u"""Adresse du serveur distant à sauvegarder""")
    option_139789496793568 = PasswordOption(**{'doc': u'Mot de passe Bareos distant', 'multi': True, 'properties': ('normal',), 'name': 'bareos_sd_remote_password'})
    option_139789496793568.impl_set_information("help", u"""Mot de passe permettant d'identifier le service bareos-dir du serveur distant à sauvegarder (ressource Director de la configuration du service bareos-sd local)""")
    option_139789497260624 = Leadership(children=[option_139789496793024, option_139789496793296, option_139789496793568], **{'doc': u'Nom du directeur Bareos distant', 'properties': ('normal',), 'name': 'bareos_sd_remote_dir_name'})
    option_139789497260944 = OptionDescription(children=[option_139789496792752, option_139789497260624], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971040}], **{'doc': u'Stockage Bareos', 'properties': ('normal',), 'name': 'stockage_bareos'})
    option_139789497260944.impl_set_information("help", u"""Configuration du service bareos-sd""")
    option_139789497260944.impl_set_information("icon", u"""archive""")
    option_139789496793840 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_squid2', 'default': 'non', 'doc': u'Activer une deuxi\xe8me instance de Squid', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497656464}]})
    option_139789496794112 = UnicodeOption(**{'multi': False, 'name': 'num_dansguardian_instance', 'doc': u'Nombre de filtres web (filtrage web et pare-feu) configurables', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("3"), 'condition_1': ParamOption(option_139789496793840, True), 'match': ParamValue("2")})})
    option_139789496794384 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_freeradius', 'default': 'non', 'doc': u'Activer le service FreeRADIUS', 'properties': ('mandatory', 'normal')})
    option_139789496794384.impl_set_information("help", u"""RADIUS (Remote Authentication Dial-In User Service) est un protocole client-serveur permettant de centraliser des données d'authentification""")
    option_139789497261264 = OptionDescription(children=[option_139789496794112, option_139789497656464, option_139789496793840, option_139789496794384], **{'doc': u'Authentification', 'properties': ('normal',), 'name': 'authentification'})
    option_139789497261264.impl_set_information("help", u"""Configuration des authentifications proxy et FreeRADIUS""")
    option_139789497261264.impl_set_information("icon", u"""key""")
    option_139789496794656 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dans_instance_1_active', 'doc': u'dans_instance_1_active', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("1"),), kwargs={'mismatch': ParamValue("non"), 'condition_5': ParamOption(option_139789497352544, True), 'condition_4': ParamOption(option_139789497273072, True), 'condition_1': ParamOption(option_139789497596928, True), 'condition_3': ParamOption(option_139789497746848, True), 'condition_2': ParamOption(option_139789497674752, True), 'operator': ParamValue("OR"), 'match': ParamValue("oui")})})
    option_139789496811600 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dans_instance_2_active', 'doc': u'dans_instance_2_active', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("2"),), kwargs={'mismatch': ParamValue("non"), 'condition_5': ParamOption(option_139789497352544, True), 'condition_4': ParamOption(option_139789497273072, True), 'condition_1': ParamOption(option_139789497596928, True), 'condition_3': ParamOption(option_139789497746848, True), 'condition_2': ParamOption(option_139789497674752, True), 'operator': ParamValue("OR"), 'match': ParamValue("oui")})})
    option_139789496811872 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dans_instance_3_active', 'doc': u'dans_instance_3_active', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("oui"),), kwargs={'mismatch': ParamValue("non"), 'condition_1': ParamOption(option_139789497656464, False), 'match': ParamValue("oui"), 'condition_2': ParamOption(option_139789496793840, True)})})
    option_139789496812144 = ChoiceOption(**{'multi': False, 'values': (u'0', u'1', u'2', u'3', u'4'), 'name': 'dans_num_opt_filters', 'default': u'3', 'doc': u'Nombre de politiques optionnelles de filtrage par zone', 'properties': ('mandatory', 'expert')})
    option_139789496812144.impl_set_information("separator", ('Configuration commune aux deux zones', None))
    option_139789496812144.impl_set_information("help", u"""Nombre de politiques optionnelles (1, 2 et 3) à activer, le diminuer permet un gain de performance""")
    option_139789496812416 = IntOption(**{'multi': False, 'name': 'docbodytimeout', 'default': 20, 'doc': u'Proxy Timeout (ex: Doc Body Timeout)', 'properties': ('mandatory', 'expert'), "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"5"), 'maxi': ParamValue(u"100")})})
    option_139789496812416.impl_set_information("help", u"""Délai d'attente TCP entre le proxy et le guardian (Min 5, Max 100)""")
    option_139789496812688 = IntOption(**{'multi': False, 'name': 'docheadertimeout', 'default': 20, 'doc': u'Proxy header exchange (ex: Doc Header Timeout)', 'properties': ('mandatory', 'expert'), "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"20"), 'maxi': ParamValue(u"300")})})
    option_139789496812688.impl_set_information("help", u"""Délai d'attente entre le proxy et le guardian (Min 20, Max 300)""")
    option_139789496812960 = IntOption(**{'multi': False, 'name': 'headertimeout', 'default': 55, 'doc': u'Pconn timeout (ex: Header Timeout)', 'properties': ('mandatory', 'expert'), "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"5"), 'maxi': ParamValue(u"300")})})
    option_139789496812960.impl_set_information("help", u"""Combien de secondes une connection persistante attend de nouvelles requêtes (Min 5, Max 300)""")
    option_139789496813232 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non', u'admin seulement'), 'name': 'dansguardian_ead_log', 'default': u'non', 'doc': u"Autoriser la consultation des logs li\xe9s au filtrage web dans l'EAD", 'properties': ('mandatory', 'expert')})
    option_139789496813776 = UnicodeOption(**{'default': u'Filtre web 1', 'doc': u"Libell\xe9 du filtre web 1 dans l'EAD", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'dansguardian_ead_filtre1'})
    option_139789496813776.impl_set_information("separator", ('Filtre web 1', None))
    option_139789496813504 = IntOption(**{'multi': False, 'name': 'guardian_httpworkers1', 'default': 1000, 'doc': u'Nombre de processus disponibles pour traiter les connexions', 'properties': ('mandatory', 'expert'), "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"20"), 'maxi': ParamValue(u"20000")})})
    option_139789496813504.impl_set_information("help", u"""1000 processus convient pour une centaine d'utilisateurs, il est possible d'augmenter le nombre de processus à 5000 pour une structure de taille moyenne, 10000 pour une grosse structure. La valeur est également dépendante du nombre de règles et de la quantité de mémoire RAM disponible. Le nombre maximum de processus est limité à 20000.""")
    option_139789496814048 = FilenameOption(**{'default': u'/var/spool/guardian1', 'doc': u'R\xe9pertoire de cache', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'dans_cachedir1'})
    option_139789496814320 = IntOption(**{'default': 5000, 'doc': u'Taille maximum de fichier \xe0 scanner conserv\xe9 en m\xe9moire (en Kibibytes)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'dans_maxramsize1'})
    option_139789496814592 = IntOption(**{'default': 20000, 'doc': u'Taille maximum de fichier \xe0 scanner conserv\xe9 sur le disque (en Kibibytes)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'dans_maxfilesize1'})
    option_139789496814864 = ChoiceOption(**{'multi': False, 'values': (u'---', u'oui', u'non'), 'name': 'unactive_dansguardian_auth1', 'default': u'---', 'doc': u"D\xe9sactiver les plugins qui logguent l'authentification sur la zone 1", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789496815136 = IntOption(**{'default': 50, 'doc': u'Limite de pond\xe9ration du filtrage', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'naughtynesslimit1'})
    option_139789496815136.impl_set_information("help", u"""Limite de pondération du filtrage des pages, chaque élément de la page reçoit un score, la somme est faite et si le score est supérieur à cette limite la page est bloquée (jeunes enfants : 50, enfants : 100, jeunes adultes : 160)""")
    option_139789496819792 = UnicodeOption(**{'default': u'Filtre web 2', 'doc': u"Libell\xe9 du filtre web 2 dans l'EAD", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'dansguardian_ead_filtre2'})
    option_139789496819792.impl_set_information("separator", ('Filtre web 2', None))
    option_139789496820064 = IntOption(**{'multi': False, 'name': 'guardian_httpworkers2', 'default': 1000, 'doc': u'Nombre de processus disponibles pour traiter les connexions', 'properties': ('mandatory', 'expert'), "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"20"), 'maxi': ParamValue(u"20000")})})
    option_139789496820064.impl_set_information("help", u"""1000 processus convient pour une centaine d'utilisateurs, il est possible d'augmenter le nombre de processus à 5000 pour une structure de taille moyenne, 10000 pour une grosse structure. La valeur est également dépendante du nombre de règles et de la quantité de mémoire RAM disponible. Le nombre maximum de processus est limité à 20000.""")
    option_139789496820336 = FilenameOption(**{'default': u'/var/spool/guardian2', 'doc': u'R\xe9pertoire de cache', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'dans_cachedir2'})
    option_139789496820608 = IntOption(**{'default': 5000, 'doc': u'Taille maximum de fichier \xe0 scanner conserv\xe9 en m\xe9moire (en Kibibytes)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'dans_maxramsize2'})
    option_139789496820880 = IntOption(**{'default': 20000, 'doc': u'Taille maximum de fichier \xe0 scanner conserv\xe9 sur le disque (en Kibibytes)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'dans_maxfilesize2'})
    option_139789496821152 = ChoiceOption(**{'multi': False, 'values': (u'---', u'oui', u'non'), 'name': 'unactive_dansguardian_auth2', 'default': u'---', 'doc': u"D\xe9sactiver les plugins qui logguent l'authentification sur la zone 2", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789496821424 = IntOption(**{'default': 50, 'doc': u'Limite de pond\xe9ration du filtrage', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'naughtynesslimit2'})
    option_139789496821424.impl_set_information("help", u"""Limite de pondération du filtrage des pages, chaque élément de la page reçoit un score, la somme est faite et si le score est supérieur à cette limite la page est bloquée (jeunes enfants : 50, enfants : 100, jeunes adultes : 160)""")
    option_139789496821696 = UnicodeOption(**{'multi': False, 'name': 'dansguardian_ead_filtre3', 'default': u'Filtre web 3', 'doc': u"Libell\xe9 du filtre web 3 dans l'EAD", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}]})
    option_139789496821696.impl_set_information("separator", ('Filtre web 3', None))
    option_139789496821968 = IntOption(**{'multi': False, 'name': 'guardian_httpworkers3', 'default': 1000, 'doc': u'Nombre de processus disponibles pour traiter les connexions', 'properties': ('mandatory', 'expert'), "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"20"), 'maxi': ParamValue(u"20000")})})
    option_139789496821968.impl_set_information("help", u"""1000 processus convient pour une centaine d'utilisateurs, il est possible d'augmenter le nombre de processus à 5000 pour une structure de taille moyenne, 10000 pour une grosse structure. La valeur est également dépendante du nombre de règles et de la quantité de mémoire RAM disponible. Le nombre maximum de processus est limité à 20000.""")
    option_139789496822240 = PortOption(**{'multi': False, 'name': 'dansguardian_port3', 'allow_private': True, 'default': '3129', 'doc': u"Port d'\xe9coute pour le 3\xe8me filtre web", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}]})
    option_139789496822512 = FilenameOption(**{'multi': False, 'name': 'dans_cachedir3', 'default': u'/var/spool/guardian3', 'doc': u'R\xe9pertoire de cache', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}]})
    option_139789496822784 = IntOption(**{'multi': False, 'name': 'dans_maxramsize3', 'default': 5000, 'doc': u'Taille maximum de fichier \xe0 scanner conserv\xe9 en m\xe9moire (en Kibibytes)', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}]})
    option_139789496823056 = IntOption(**{'multi': False, 'name': 'dans_maxfilesize3', 'default': 20000, 'doc': u'Taille maximum de fichier \xe0 scanner conserv\xe9 sur le disque (en Kibibytes)', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}]})
    option_139789496823328 = ChoiceOption(**{'multi': False, 'values': (u'---', u'oui', u'non'), 'name': 'unactive_dansguardian_auth3', 'default': u'---', 'doc': u"D\xe9sactiver les plugins qui logguent l'authentification sur la zone 3", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert')})
    option_139789496832080 = IntOption(**{'multi': False, 'name': 'naughtynesslimit3', 'default': 50, 'doc': u'Limite de pond\xe9ration du filtrage', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}]})
    option_139789496832080.impl_set_information("help", u"""Limite de pondération du filtrage des pages, chaque élément de la page reçoit un score, la somme est faite et si le score est supérieur à cette limite la page est bloquée (jeunes enfants : 50, enfants : 100, jeunes adultes : 160)""")
    option_139789497261584 = OptionDescription(children=[option_139789496794656, option_139789496811600, option_139789496811872, option_139789496812144, option_139789496812416, option_139789496812688, option_139789496812960, option_139789496813232, option_139789496813776, option_139789496813504, option_139789496814048, option_139789496814320, option_139789496814592, option_139789496814864, option_139789496815136, option_139789496819792, option_139789496820064, option_139789496820336, option_139789496820608, option_139789496820880, option_139789496821152, option_139789496821424, option_139789496821696, option_139789496821968, option_139789496822240, option_139789496822512, option_139789496822784, option_139789496823056, option_139789496823328, option_139789496832080], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}], **{'doc': u'Filtrage web', 'properties': ('expert',), 'name': 'filtrage_web'})
    option_139789497261584.impl_set_information("help", u"""Configuration avancée du logiciel de filtrage e2guardian""")
    option_139789497261584.impl_set_information("icon", u"""code-fork""")
    option_139789496832352 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'lightsquid_auto', 'default': 'non', 'doc': u'G\xe9n\xe9rer les statistiques Squid automatiquement', 'properties': ('mandatory', 'expert')})
    option_139789496832352.impl_set_information("help", u"""LightSquid est un analyseur de le log Squid permettant d'afficher sous forme de pages web l'utilisation du proxy""")
    option_139789496832624 = PortOption(**{'multi': False, 'name': 'lightsquid_port', 'allow_private': True, 'default': '8062', 'doc': u"Port d'\xe9coute du CGI LightSquid", 'properties': ('mandatory', 'expert')})
    option_139789496832896 = ChoiceOption(**{'multi': False, 'values': (u'aucune', u'par IP', u'anonyme'), 'name': 'lightsquid_anon_mode', 'default': u'aucune', 'doc': u'M\xe9thode d\u2019anonymisation des rapports LightSquid', 'properties': ('mandatory', 'expert')})
    option_139789496832896.impl_set_information("help", u"""Méthode d’anonimisation des rapports LightSquid :
        - aucune : le nom d’utilisateur est utilisé s’il est disponible
        - par IP : les rapports sont générés par adresse IP
        - anonyme : le nom « anonyme » est utilisé à la place du nom et de l’adresse IP""")
    option_139789496833168 = PortOption(**{'doc': u"Port d'\xe9coute HTTPS de Squid", 'multi': False, 'properties': ('expert',), 'allow_private': True, 'name': 'https_port'})
    option_139789496833440 = PortOption(**{'doc': u'"SSL_ports" suppl\xe9mentaire', 'multi': True, 'properties': ('expert',), 'allow_private': True, 'name': 'ssl_ports'})
    option_139789496833712 = PortOption(**{'doc': u'"Safe_ports" suppl\xe9mentaire', 'multi': True, 'properties': ('expert',), 'allow_private': True, 'name': 'safe_ports'})
    option_139789496833984 = IntOption(**{'default': 20, 'doc': u"Nombre de processus associ\xe9s au module d'authentification basique", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'number_of_basic_children'})
    option_139789496834256 = IntOption(**{'default': 20, 'doc': u"Nombre de processus associ\xe9s au module d'authentification NTLM", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'number_of_ntlm_children'})
    option_139789496834528 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'squid_nopeerproxy_url_regex', 'default': 'non', 'doc': u'Activer les exceptions de cache p\xe8re de type regexp', 'properties': ('mandatory', 'expert')})
    option_139789496834800 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_proxy_refresh_pattern', 'default': 'non', 'doc': u'Personnaliser s\xe9lectivement la dur\xe9e des caches', 'properties': ('mandatory', 'expert')})
    option_139789496834800.impl_set_information("help", u"""La directive refresh_pattern permet de personnaliser l'algorithme de gestion du rafraîchissement du cache par site""")
    option_139789496835072 = UnicodeOption(**{'doc': u'Expression rationnelle pour le site', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'proxy_refresh_pattern_regexp', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496834800}]})
    option_139789496835344 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'proxy_refresh_pattern_case_sensitive', 'default_multi': 'non', 'doc': u"L'expression rationnelle est sensible \xe0 la casse", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496834800}]})
    option_139789496835616 = IntOption(**{'doc': u'Temps minimum de cache (en minutes)', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'proxy_refresh_pattern_min', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496834800}]})
    option_139789496835616.impl_set_information("help", u"""Temps minimum de cache d'un objet sans expiration explicite en-deça duquel l'objet est déclaré \"frais\"""")
    option_139789496840272 = IntOption(**{'multi': True, 'name': 'proxy_refresh_pattern_pourcent', 'doc': u"Rapport entre l'\xe2ge de l'objet dans le cache et son \xe2ge sur le site (en pourcent)", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496834800}], "validator": eosfunc.valid_entier, 'validator_params': Params(args=None, kwargs={'mini': ParamValue(u"0"), 'maxi': ParamValue(u"100")})})
    option_139789496840272.impl_set_information("help", u"""Rapport entre l'âge de l'objet dans le cache et son âge sur le site en-deça duquel l'objet est déclaré \"frais\"""")
    option_139789496840544 = IntOption(**{'doc': u'Temps maximum de cache (en minutes)', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'proxy_refresh_pattern_max', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496834800}]})
    option_139789496840544.impl_set_information("help", u"""Temps maximum de cache d'un objet sans expiration explicite au-delà duquel l'objet est déclaré \"périmé\"""")
    option_139789496840816 = UnicodeOption(**{'doc': u'Options', 'multi': True, 'properties': ('expert',), 'name': 'proxy_refresh_pattern_options', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496834800}]})
    option_139789496852560 = Leadership(children=[option_139789496835072, option_139789496835344, option_139789496835616, option_139789496840272, option_139789496840544, option_139789496840816], **{'doc': u'Expression rationnelle pour le site', 'properties': ('expert',), 'name': 'proxy_refresh_pattern_regexp'})
    option_139789496841088 = PortOption(**{'multi': False, 'name': 'icp_port', 'allow_private': True, 'default': '3130', 'doc': u"Port d'\xe9coute pour les requ\xeates ICP", 'properties': ('mandatory', 'expert')})
    option_139789496841360 = PortOption(**{'multi': False, 'name': 'htcp_port', 'allow_private': True, 'default': '4827', 'doc': u"Port d'\xe9coute pour les requ\xeates HTCP", 'properties': ('mandatory', 'expert')})
    option_139789496841632 = IntOption(**{'default': 0, 'doc': u'Timout pour les requ\xeates ICP (en msec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'icp_query_timeout'})
    option_139789496841904 = IntOption(**{'default': 2000, 'doc': u'Maximum_icp_query_timeout (en msec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'maximum_icp_query_timeout'})
    option_139789496842176 = UnicodeOption(**{'doc': u'Groupe de serveurs multicast', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'expert'), 'name': 'mcast_groups'})
    option_139789496842448 = IntOption(**{'default': 2000, 'doc': u'Mcast_icp_query_timeout (en msec)', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert'), 'name': 'mcast_icp_query_timeout'})
    option_139789496842720 = IntOption(**{'default': 10, 'doc': u'Dead_peer_timeout (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'dead_peer_timeout'})
    option_139789496842992 = IntOption(**{'default': 25, 'doc': u'Nombre maximum de redirections test\xe9es', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'forward_max_tries'})
    option_139789496843264 = IntOption(**{'default': 256, 'doc': u'Taille maximale de la m\xe9moire cache (en MB)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'cache_mem'})
    option_139789496843536 = IntOption(**{'default': 90, 'doc': u'Cache_swap_low (en %)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'cache_swap_low'})
    option_139789496843808 = IntOption(**{'default': 95, 'doc': u'Cache_swap_high (en %)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'cache_swap_high'})
    option_139789496856656 = IntOption(**{'default': 0, 'doc': u'Minimum_object_size (en KB)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'minimum_object_size'})
    option_139789496856928 = IntOption(**{'default': 32768, 'doc': u'Maximum_object_size (en KB)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'maximum_object_size'})
    option_139789496857200 = IntOption(**{'default': 512, 'doc': u"Taille maximum d'un objet \xe0 mettre en cache (en KB)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'maximum_object_size_in_memory'})
    option_139789496857472 = IntOption(**{'default': 1024, 'doc': u"Ipcache_size (en nombre d'entr\xe9es)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ipcache_size'})
    option_139789496857744 = IntOption(**{'default': 90, 'doc': u'Ipcache_low (en %)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ipcache_low'})
    option_139789496858016 = IntOption(**{'default': 95, 'doc': u'Ipcache_high (en %)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ipcache_high'})
    option_139789496858288 = IntOption(**{'default': 1024, 'doc': u"Fqdncache_size (en nombre d'entr\xe9es)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'fqdncache_size'})
    option_139789496858560 = ChoiceOption(**{'multi': False, 'values': (u'lru', u'heap GDSF', u'heap LFUDA', u'heap LRU'), 'name': 'cache_replacement_policy', 'default': u'lru', 'doc': u'Cache_replacement_policy', 'properties': ('mandatory', 'expert')})
    option_139789496858832 = ChoiceOption(**{'multi': False, 'values': (u'lru', u'heap GDSF', u'heap LFUDA', u'heap LRU'), 'name': 'memory_replacement_policy', 'default': u'lru', 'doc': u'Memory_replacement_policy', 'properties': ('mandatory', 'expert')})
    option_139789496859104 = FilenameOption(**{'default': [u'/var/spool/squid'], 'doc': u'Emplacement du cache', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'cache_dir'})
    option_139789496859376 = ChoiceOption(**{'multi': True, 'values': (u'ufs',), 'name': 'cache_dir_type', 'default_multi': u'ufs', 'doc': u'Type de stockage utilis\xe9', 'properties': ('mandatory', 'expert')})
    option_139789496859648 = UnicodeOption(**{'default_multi': u'1000', 'doc': u'Taille du cache (en MBytes)', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'cache_dir_disk_space'})
    option_139789496859920 = UnicodeOption(**{'default_multi': u'16', 'doc': u'Nombre maximum de r\xe9pertoires de niveau 1', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'cache_dir_firstLevel'})
    option_139789496860192 = UnicodeOption(**{'default_multi': u'256', 'doc': u'Nombre maximum de r\xe9pertoires de niveau 2', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'cache_dir_secondLevel'})
    option_139789496852880 = Leadership(children=[option_139789496859104, option_139789496859376, option_139789496859648, option_139789496859920, option_139789496860192], **{'doc': u'Emplacement du cache', 'properties': ('expert',), 'name': 'cache_dir'})
    option_139789496868944 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'log_mime_hdrs', 'default': 'off', 'doc': u'Log_mime_hdrs', 'properties': ('mandatory', 'expert')})
    option_139789496869216 = NetmaskOption(**{'default': u'255.255.255.255', 'doc': u'Masque r\xe9seau des clients ', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'client_netmask'})
    option_139789496869488 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'strip_query_terms', 'default': 'off', 'doc': u'Activer la r\xe9duction URLs dans les logs', 'properties': ('mandatory', 'expert')})
    option_139789496869760 = UsernameOption(**{'default': 'ftp', 'doc': u'Utilisateur FTP', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ftp_user'})
    option_139789496870032 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'ftp_passive', 'default': 'on', 'doc': u'Autoriser les connexions FTP passives', 'properties': ('mandatory', 'expert')})
    option_139789496870304 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'ftp_sanitycheck', 'default': 'on', 'doc': u'Activer la v\xe9rification des connexions FTP', 'properties': ('mandatory', 'expert')})
    option_139789496870576 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'ftp_telnet_protocol', 'default': 'on', 'doc': u'Ftp_telnet_protocol', 'properties': ('mandatory', 'expert')})
    option_139789496870848 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'check_hostname', 'default': 'off', 'doc': u"Activer la v\xe9rification des noms d'h\xf4te", 'properties': ('mandatory', 'expert')})
    option_139789496871120 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'allow_underscore', 'default': 'on', 'doc': u"Autoriser les underscore dans les noms d'h\xf4te", 'properties': ('mandatory', 'expert')})
    option_139789496871392 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'dns_defnames', 'default': 'off', 'doc': u'Dns_defnames', 'properties': ('mandatory', 'expert')})
    option_139789496871664 = DomainnameOption(**{'type_': 'domainname', 'name': 'dns_nameservers', 'doc': u'Utiliser un DNS sp\xe9cifique', 'allow_ip': True, 'allow_without_dot': True, 'multi': True, 'properties': ('expert',)})
    option_139789496871936 = FilenameOption(**{'default': u'/etc/hosts', 'doc': u'Emplacement du fichier hosts', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'expert'), 'name': 'hosts_file'})
    option_139789496872208 = IntOption(**{'default': 64, 'doc': u'Taille maximum des en-t\xeates HTTP dans une requ\xeate (en KB)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'request_header_max_size'})
    option_139789496872480 = IntOption(**{'default': 0, 'doc': u"Taille maximum d'un corps de requ\xeate HTTP (en MB)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'request_body_max_size'})
    option_139789496877136 = IntOption(**{'default': 16, 'doc': u'Quick_abort_min (en KB)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'quick_abort_min'})
    option_139789496877408 = IntOption(**{'default': 16, 'doc': u'Quick_abort_max (en KB)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'quick_abort_max'})
    option_139789496877680 = IntOption(**{'default': 95, 'doc': u'Quick_abort_pct (en %)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'quick_abort_pct'})
    option_139789496877952 = IntOption(**{'default': 16, 'doc': u'Read_ahead_gap (en KB)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'read_ahead_gap'})
    option_139789496878224 = IntOption(**{'default': 0, 'doc': u'Time-to-Live par d\xe9faut pour les requ\xeates en \xe9chec (en min)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'negative_ttl'})
    option_139789496878496 = IntOption(**{'default': 6, 'doc': u'Positive_dns_ttl (en heures)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'positive_dns_ttl'})
    option_139789496878768 = IntOption(**{'default': 1, 'doc': u'Negative_dns_ttl (en min)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'negative_dns_ttl'})
    option_139789496879040 = IntOption(**{'default': 0, 'doc': u'Range_offset_limit (en KB)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'range_offset_limit'})
    option_139789496879312 = IntOption(**{'default': 240, 'doc': u'Forward timeout (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'forward_timeout'})
    option_139789496879584 = IntOption(**{'default': 60, 'doc': u'Connect timeout (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'connect_timeout'})
    option_139789496879856 = IntOption(**{'default': 30, 'doc': u'Peer connect timeout (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'peer_connect_timeout'})
    option_139789496880128 = IntOption(**{'default': 900, 'doc': u'Read timeout (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'read_timeout'})
    option_139789496880400 = IntOption(**{'default': 300, 'doc': u'Request timeout (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'request_timeout'})
    option_139789496880672 = IntOption(**{'default': 120, 'doc': u'Persistent request timeout (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'persistent_request_timeout'})
    option_139789496885328 = IntOption(**{'default': 1440, 'doc': u'Dur\xe9e maximum pendant laquelle un client peut rester connect\xe9 au cache (en min)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'client_lifetime'})
    option_139789496885600 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'half_closed_clients', 'default': 'off', 'doc': u'Half_closed_clients', 'properties': ('mandatory', 'expert')})
    option_139789496885872 = IntOption(**{'default': 60, 'doc': u'Timout pour les connexions persistantes (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'pconn_timeout'})
    option_139789496886144 = IntOption(**{'default': 10, 'doc': u'Ident_timeout (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ident_timeout'})
    option_139789496886416 = IntOption(**{'default': 30, 'doc': u'Shutdown_lifetime (en sec)', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'shutdown_lifetime'})
    option_139789496853200 = OptionDescription(children=[option_139789496832352, option_139789496832624, option_139789496832896, option_139789496833168, option_139789496833440, option_139789496833712, option_139789496833984, option_139789496834256, option_139789496834528, option_139789496834800, option_139789496852560, option_139789496841088, option_139789496841360, option_139789496841632, option_139789496841904, option_139789496842176, option_139789496842448, option_139789496842720, option_139789496842992, option_139789496843264, option_139789496843536, option_139789496843808, option_139789496856656, option_139789496856928, option_139789496857200, option_139789496857472, option_139789496857744, option_139789496858016, option_139789496858288, option_139789496858560, option_139789496858832, option_139789496852880, option_139789496868944, option_139789496869216, option_139789496869488, option_139789496869760, option_139789496870032, option_139789496870304, option_139789496870576, option_139789496870848, option_139789496871120, option_139789496871392, option_139789496871664, option_139789496871936, option_139789496872208, option_139789496872480, option_139789496877136, option_139789496877408, option_139789496877680, option_139789496877952, option_139789496878224, option_139789496878496, option_139789496878768, option_139789496879040, option_139789496879312, option_139789496879584, option_139789496879856, option_139789496880128, option_139789496880400, option_139789496880672, option_139789496885328, option_139789496885600, option_139789496885872, option_139789496886144, option_139789496886416], **{'doc': u'Squid', 'properties': ('expert',), 'name': 'squid'})
    option_139789496853200.impl_set_information("help", u"""Configuration du proxy Squid""")
    option_139789496853200.impl_set_information("icon", u"""code-fork""")
    option_139789496886688 = PortOption(**{'multi': False, 'name': 'proxy2_port', 'allow_private': True, 'doc': u"Port d'\xe9coute de la seconde instance de Squid", 'properties': ('mandatory', 'expert'), 'callback': eosfunc.calc_val_first_value, 'callback_params': Params(args=(ParamOption(option_139789496822240, True), ParamValue("3129")), kwargs={}), "requires": [{'expected': u'oui', 'action': 'frozen', 'inverse': False, "option": option_139789497941264}, {'expected': u'oui', 'action': 'hidden', 'inverse': False, "option": option_139789497941264}, {'expected': u'oui', 'action': 'force_default_on_freeze', 'inverse': False, "option": option_139789497941264}]})
    option_139789496886960 = FilenameOption(**{'default': [u'/var/spool/squid2'], 'doc': u'Emplacement du cache', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'cache_dir_2'})
    option_139789496887232 = ChoiceOption(**{'multi': True, 'values': (u'ufs',), 'name': 'cache_dir_type_2', 'default_multi': u'ufs', 'doc': u'Type de stockage utilis\xe9', 'properties': ('mandatory', 'expert')})
    option_139789496887504 = IntOption(**{'default_multi': 1000, 'doc': u'Taille du cache (en MBytes)', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'cache_dir_disk_space_2'})
    option_139789496887776 = IntOption(**{'default_multi': 16, 'doc': u'Nombre maximum de r\xe9pertoires de niveau 1', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'cache_dir_firstLevel_2'})
    option_139789496888048 = IntOption(**{'default_multi': 256, 'doc': u'Nombre maximum de r\xe9pertoires de niveau 2', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'cache_dir_secondLevel_2'})
    option_139789496853520 = Leadership(children=[option_139789496886960, option_139789496887232, option_139789496887504, option_139789496887776, option_139789496888048], **{'doc': u'Emplacement du cache', 'properties': ('expert',), 'name': 'cache_dir_2'})
    option_139789496853840 = OptionDescription(children=[option_139789496886688, option_139789496853520], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}], **{'doc': u'squid2', 'properties': ('expert',), 'name': 'squid2'})
    option_139789496853840.impl_set_information("help", u"""Configuration de la seconde instance du proxy Squid""")
    option_139789496853840.impl_set_information("icon", u"""code-fork""")
    option_139789496888320 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'ip_serveur_ldap1', 'doc': u'Adresse du premier serveur LDAP', 'properties': ('mandatory', 'basic'), 'callback': eosfunc.calc_container, 'callback_params': Params(args=(ParamOption(option_139789497889120, False), ParamOption(option_139789497852256, False)), kwargs={}), "requires": [{'expected': u'Ldap', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496888592 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'ip_serveur_ldap2', 'doc': u'Adresse du second serveur LDAP (si le 1er ne r\xe9pond pas)', 'properties': ('normal',), "requires": [{'expected': u'Ldap', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496888864 = UnicodeOption(**{'multi': False, 'name': 'proxy_ldap_base_dn', 'doc': u"Suffixe racine de l'annuaire LDAP (base DN)", 'properties': ('mandatory', 'basic'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497484976, False)), "requires": [{'expected': u'Ldap', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496893520 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'ip_serveur_ldapad', 'doc': u'Adresse IP du serveur LDAP (Active Directory)', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Ldap (Active Directory)', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496893792 = UnicodeOption(**{'doc': u"Suffixe racine de l'annuaire LDAP (base DN Active Directory)", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'ldapad_base_dn', "requires": [{'expected': u'Ldap (Active Directory)', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496894064 = UnicodeOption(**{'doc': u"Nom du compte n\xe9cessaire pour l'interrogation LDAP (Active Directory)", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'ldapad_user', "requires": [{'expected': u'Ldap (Active Directory)', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496894336 = PasswordOption(**{'doc': u"Mot de passe du compte n\xe9cessaire pour l'interrogation LDAP (Active Directory)", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'ldapad_passwd', "requires": [{'expected': u'Ldap (Active Directory)', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496894608 = DomainnameOption(**{'multi': False, 'name': 'nom_serveur_smb', 'doc': u'Nom du contr\xf4leur de domaine SMB', 'warnings_only': True, 'type_': 'netbios', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497890480, False)), "requires": [{'expected': u'NTLM/SMB', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496894880 = DomainnameOption(**{'multi': False, 'name': 'nom_domaine_smb', 'doc': u'Nom du domaine SMB', 'warnings_only': True, 'type_': 'netbios', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497056256, False)), "requires": [{'expected': u'NTLM/SMB', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496895152 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'ip_serveur_smb', 'doc': u'Adresse IP du contr\xf4leur de domaine SMB', 'properties': ('mandatory', 'normal'), 'callback': eosfunc.calc_container, 'callback_params': Params(args=(ParamOption(option_139789497889120, False), ParamOption(option_139789497854704, False)), kwargs={}), "requires": [{'expected': u'NTLM/SMB', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496895424 = DomainnameOption(**{'type_': 'hostname', 'name': 'nom_serveur_krb', 'doc': u'Nom du contr\xf4leur de domaine KERBEROS', 'allow_ip': False, 'multi': False, 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496895696 = DomainnameOption(**{'multi': False, 'name': 'nom_domaine_krb', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Nom du domaine KERBEROS (fqdn)', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496895968 = DomainnameOption(**{'type_': 'netbios', 'name': 'nom_domaine_windows', 'doc': u'Nom du domaine Windows', 'multi': False, 'warnings_only': True, 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496896240 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'ip_serveur_krb', 'doc': u'Adresse IP du contr\xf4leur de domaine KERBEROS', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496854160 = OptionDescription(children=[option_139789497574544, option_139789496888320, option_139789496888592, option_139789496888864, option_139789496893520, option_139789496893792, option_139789496894064, option_139789496894336, option_139789496894608, option_139789496894880, option_139789496895152, option_139789496895424, option_139789496895696, option_139789496895968, option_139789496896240, option_139789497574816, option_139789497575088], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497656464}], **{'doc': u'Proxy authentifi\xe9', 'properties': ('basic',), 'name': 'proxy_authentifie'})
    option_139789496854160.impl_set_information("help", u"""Configuration du proxy authentifié""")
    option_139789496854160.impl_set_information("icon", u"""key""")
    option_139789496896512 = ChoiceOption(**{'multi': False, 'values': (u'Ldap', u'Ldap (Active Directory)'), 'name': 'type_squid_auth_2', 'default': u'Ldap', 'doc': u"Type d'authentification", 'properties': ('mandatory', 'normal')})
    option_139789496896784 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'ip_serveur_ldap1_2', 'doc': u'Adresse du premier serveur LDAP', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497484704, False)), "requires": [{'expected': u'Ldap (Active Directory)', 'action': 'disabled', 'inverse': False, "option": option_139789496896512}]})
    option_139789496897056 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'ip_serveur_ldap2_2', 'doc': u'Adresse du second serveur LDAP (si le 1er ne r\xe9pond pas)', 'properties': ('normal',), "requires": [{'expected': u'Ldap (Active Directory)', 'action': 'disabled', 'inverse': False, "option": option_139789496896512}]})
    option_139789496918096 = UnicodeOption(**{'multi': False, 'name': 'proxy_ldap_base_dn_2', 'doc': u"Suffixe racine de l'annuaire LDAP (base DN)", 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497484976, False)), "requires": [{'expected': u'Ldap (Active Directory)', 'action': 'disabled', 'inverse': False, "option": option_139789496896512}]})
    option_139789496918368 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'ip_serveur_ldapad_2', 'doc': u'Adresse IP du serveur LDAP (Active Directory)', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Ldap', 'action': 'disabled', 'inverse': False, "option": option_139789496896512}]})
    option_139789496918640 = UnicodeOption(**{'doc': u"Suffixe racine de l'annuaire LDAP (base DN Active Directory)", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'ldapad_base_dn_2', "requires": [{'expected': u'Ldap', 'action': 'disabled', 'inverse': False, "option": option_139789496896512}]})
    option_139789496918912 = UnicodeOption(**{'doc': u"Nom du compte n\xe9cessaire pour l'interrogation LDAP (Active Directory)", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'ldapad_user_2', "requires": [{'expected': u'Ldap', 'action': 'disabled', 'inverse': False, "option": option_139789496896512}]})
    option_139789496919184 = PasswordOption(**{'doc': u"Mot de passe du compte n\xe9cessaire pour l'interrogation LDAP (Active Directory)", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'ldapad_passwd_2', "requires": [{'expected': u'Ldap', 'action': 'disabled', 'inverse': False, "option": option_139789496896512}]})
    option_139789496854480 = OptionDescription(children=[option_139789496896512, option_139789496896784, option_139789496897056, option_139789496918096, option_139789496918368, option_139789496918640, option_139789496918912, option_139789496919184], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}], **{'doc': u'proxy authentifi\xe9 2', 'properties': ('basic',), 'name': 'proxy_authentifie_2'})
    option_139789496854480.impl_set_information("help", u"""Configuration de l'authentification sur la seconde instance du proxy Squid""")
    option_139789496854480.impl_set_information("icon", u"""key""")
    option_139789496919456 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'cups_pdf', 'default': 'oui', 'doc': u"Activation de l'imprimante virtuelle PDF", 'properties': ('mandatory', 'expert')})
    option_139789496919728 = UnicodeOption(**{'multi': False, 'name': 'cups_pdf_name', 'default': u'PDF', 'doc': u"Nom de l'imprimante virtuelle PDF", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496919456}], "validator": eosfunc.valid_regexp, 'validator_params': Params(args=(ParamValue(u"^[A-Za-z0-9_-]*$"),), kwargs={'err_msg': ParamValue(u"Seuls les caractères alphanumériques, _ et - sont autorisés")})})
    option_139789496920000 = ChoiceOption(**{'multi': False, 'values': (u'true', u'false'), 'name': 'cups_pdf_shared', 'default': u'true', 'doc': u"L'imprimante virtuelle PDF est partag\xe9e", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496919456}]})
    option_139789496920272 = ChoiceOption(**{'multi': False, 'values': (u'alert', u'crit', u'debug2', u'debug', u'emerg', u'error', u'info', u'none', u'notice', u'warn'), 'name': 'cups_loglevel', 'default': u'warn', 'doc': u'Niveau de log', 'properties': ('mandatory', 'expert')})
    option_139789496920816 = ChoiceOption(**{'multi': False, 'values': (u'on', u'off'), 'name': 'cups_browsing', 'default': 'on', 'doc': u'Activer la r\xe9cup\xe9ration des informations des imprimantes distantes', 'properties': ('mandatory', 'expert')})
    option_139789496920544 = IntOption(**{'default': 100, 'doc': u"Nombre maximum de copies qu'un utilisateur peut effectuer pour un travail d'impression", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'cups_maxcopies'})
    option_139789496921088 = IntOption(**{'default': 500, 'doc': u'Nombre maximum de travaux simultan\xe9s', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'cups_maxjobs'})
    option_139789496921360 = IntOption(**{'default': 100, 'doc': u'Nombre maximum de clients simultan\xe9s', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'cups_maxclients'})
    option_139789496921632 = ChoiceOption(**{'multi': False, 'values': (u'Yes', u'No'), 'name': 'cups_preservejobhistory', 'default': u'Yes', 'doc': u"Conserver l'historique des demandes d'impression", 'properties': ('mandatory', 'expert')})
    option_139789496930384 = ChoiceOption(**{'multi': False, 'values': (u'Yes', u'No'), 'name': 'cups_preservejobfiles', 'default': u'No', 'doc': u'Conserver les fichiers apr\xe8s impression', 'properties': ('mandatory', 'expert')})
    option_139789496930656 = ChoiceOption(**{'multi': False, 'values': (u'Yes', u'No'), 'name': 'cups_autopurgejobs', 'default': u'No', 'doc': u"Purger automatiquement l'historique des travaux", 'properties': ('mandatory', 'expert')})
    option_139789496930928 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'cups_printcap', 'default': 'non', 'doc': u'G\xe9n\xe9rer le fichier printcap', 'properties': ('mandatory', 'expert')})
    option_139789496931200 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'cups_lp', 'default': 'non', 'doc': u"Charger le module d'impression d'imprimante sur port parall\xe8le (incompatible avec les conteneurs)", 'properties': ('mandatory', 'expert')})
    option_139789496854800 = OptionDescription(children=[option_139789496919456, option_139789496919728, option_139789496920000, option_139789496920272, option_139789496920816, option_139789496920544, option_139789496921088, option_139789496921360, option_139789496921632, option_139789496930384, option_139789496930656, option_139789496930928, option_139789496931200], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}], **{'doc': u'CUPS', 'properties': ('expert',), 'name': 'cups'})
    option_139789496854800.impl_set_information("help", u"""Paramétrage avancé du serveur d'impression CUPS""")
    option_139789496854800.impl_set_information("icon", u"""print""")
    option_139789496931744 = DomainnameOption(**{'type_': 'domainname', 'name': 'proxy_bypass_domain_eth0', 'doc': u"Exceptions de type nom de domaine pour l'interface 0", 'allow_ip': False, 'multi': True, 'properties': ('disabled', 'normal')})
    option_139789496931744.impl_set_information("separator", ('Exceptions de proxy pour des domaines de destination', True))
    option_139789496931472 = DomainnameOption(**{'type_': 'domainname', 'name': 'proxy_bypass_domain_eth1', 'doc': u"Exceptions de type nom de domaine pour l'interface 1", 'allow_ip': False, 'multi': True, 'properties': ('normal',)})
    option_139789496932016 = DomainnameOption(**{'type_': 'domainname', 'name': 'proxy_bypass_domain_eth2', 'doc': u"Exceptions de type nom de domaine pour l'interface 2", 'allow_ip': False, 'multi': True, 'properties': ('normal',), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496932288 = DomainnameOption(**{'type_': 'domainname', 'name': 'proxy_bypass_domain_eth3', 'doc': u"Exceptions de type nom de domaine pour l'interface 3", 'allow_ip': False, 'multi': True, 'properties': ('normal',), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496932560 = DomainnameOption(**{'type_': 'domainname', 'name': 'proxy_bypass_domain_eth4', 'doc': u"Exceptions de type nom de domaine pour l'interface 4", 'allow_ip': False, 'multi': True, 'properties': ('normal',), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496932832 = UnicodeOption(**{'doc': u'Liste des domaines de destination \xe0 ne pas authentifier', 'multi': True, 'properties': ('normal',), 'name': 'proxy_noauth'})
    option_139789496932832.impl_set_information("separator", ('Ne pas authentifier des domaines de destination', None))
    option_139789496933104 = UnicodeOption(**{'multi': True, 'name': 'proxy_noauth_auto', 'doc': u'Agr\xe9gation des domaines de destination \xe0 ne pas authentifier', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_val, 'callback_params': Params(args=(ParamOption(option_139789497940448, False), ParamOption(option_139789497940720, False), ParamValue("eole.ac-dijon.fr"), ParamValue("eoleng.ac-dijon.fr"), ParamValue("bp-eole.ac-dijon.fr"), ParamValue("test-eole.ac-dijon.fr"), ParamValue("test-eoleng.ac-dijon.fr"), ParamValue("ftp.crihan.fr"), ParamValue(".archive.canonical.com"), ParamValue(".ubuntu.com"), ParamValue(".ppa.launchpad.net"), ParamValue(".database.clamav.net"), ParamValue(".db.fr.clamav.net"), ParamValue(".pool.ntp.org"), ParamValue(".services.openoffice.org"), ParamValue(".windowsupdate.com"), ParamValue(".wustat.windows.com"), ParamValue(".microsoft.com"), ParamValue(".pack.google.com"), ParamValue("eqos.orion.education.fr")), kwargs={})})
    option_139789496933376 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'proxy_bypass_eth0', 'default': 'non', 'doc': u"Ajouter des exceptions de destination pour l'interface 0", 'properties': ('disabled', 'mandatory', 'normal')})
    option_139789496933376.impl_set_information("separator", (u'Exceptions de proxy pour des IP et adresses r\xe9seaux de destination', True))
    option_139789496933648 = NetworkOption(**{'doc': u"Exceptions de type r\xe9seau pour l'interface 0", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'proxy_bypass_network_eth0', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496933376}]})
    option_139789496933920 = NetmaskOption(**{'multi': True, 'name': 'proxy_bypass_netmask_eth0', 'default_multi': u'255.255.255.255', 'doc': u'Masque de sous-r\xe9seau associ\xe9', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496933376}]})
    option_139789496933920.impl_add_consistency("network_netmask", option_139789496933648, **{})
    option_139789496855120 = Leadership(children=[option_139789496933648, option_139789496933920], **{'doc': u"Exceptions de type r\xe9seau pour l'interface 0", 'properties': ('basic',), 'name': 'proxy_bypass_network_eth0'})
    option_139789496942672 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'proxy_bypass_eth1', 'default': 'non', 'doc': u"Ajouter des exceptions de destination pour l'interface 1", 'properties': ('mandatory', 'normal')})
    option_139789496942944 = NetworkOption(**{'doc': u"Exceptions de type r\xe9seau pour l'interface 1", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'proxy_bypass_network_eth1', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496942672}]})
    option_139789496943216 = NetmaskOption(**{'multi': True, 'name': 'proxy_bypass_netmask_eth1', 'default_multi': u'255.255.255.255', 'doc': u'Masque de sous-r\xe9seau associ\xe9', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496942672}]})
    option_139789496943216.impl_add_consistency("network_netmask", option_139789496942944, **{})
    option_139789496855440 = Leadership(children=[option_139789496942944, option_139789496943216], **{'doc': u"Exceptions de type r\xe9seau pour l'interface 1", 'properties': ('basic',), 'name': 'proxy_bypass_network_eth1'})
    option_139789496943488 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'proxy_bypass_eth2', 'default': 'non', 'doc': u"Ajouter des exceptions de destination pour l'interface 2", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496943760 = NetworkOption(**{'doc': u"Exceptions de type r\xe9seau pour l'interface 2", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'proxy_bypass_network_eth2', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496943488}]})
    option_139789496944032 = NetmaskOption(**{'multi': True, 'name': 'proxy_bypass_netmask_eth2', 'default_multi': u'255.255.255.255', 'doc': u'Masque de sous-r\xe9seau associ\xe9', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496943488}]})
    option_139789496944032.impl_add_consistency("network_netmask", option_139789496943760, **{})
    option_139789496855760 = Leadership(children=[option_139789496943760, option_139789496944032], **{'doc': u"Exceptions de type r\xe9seau pour l'interface 2", 'properties': ('basic',), 'name': 'proxy_bypass_network_eth2'})
    option_139789496944304 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'proxy_bypass_eth3', 'default': 'non', 'doc': u"Ajouter des exceptions de destination pour l'interface 3", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496944576 = NetworkOption(**{'doc': u"Exceptions de type r\xe9seau pour l'interface 3", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'proxy_bypass_network_eth3', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496944304}]})
    option_139789496944848 = NetmaskOption(**{'multi': True, 'name': 'proxy_bypass_netmask_eth3', 'default_multi': u'255.255.255.255', 'doc': u'Masque de sous-r\xe9seau associ\xe9', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496944304}]})
    option_139789496944848.impl_add_consistency("network_netmask", option_139789496944576, **{})
    option_139789496856080 = Leadership(children=[option_139789496944576, option_139789496944848], **{'doc': u"Exceptions de type r\xe9seau pour l'interface 3", 'properties': ('basic',), 'name': 'proxy_bypass_network_eth3'})
    option_139789496945120 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'proxy_bypass_eth4', 'default': 'non', 'doc': u"Ajouter des exceptions de destination pour l'interface 4", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496945392 = NetworkOption(**{'doc': u"Exceptions de type r\xe9seau pour l'interface 4", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'proxy_bypass_network_eth4', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496945120}]})
    option_139789496945664 = NetmaskOption(**{'multi': True, 'name': 'proxy_bypass_netmask_eth4', 'default_multi': u'255.255.255.255', 'doc': u'Masque de sous-r\xe9seau associ\xe9', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496945120}]})
    option_139789496945664.impl_add_consistency("network_netmask", option_139789496945392, **{})
    option_139789496967248 = Leadership(children=[option_139789496945392, option_139789496945664], **{'doc': u"Exceptions de type r\xe9seau pour l'interface 4", 'properties': ('basic',), 'name': 'proxy_bypass_network_eth4'})
    option_139789496945936 = NetworkOption(**{'doc': u"Exception source de type r\xe9seau pour l'interface 1", 'multi': True, 'properties': ('normal',), 'name': 'proxy_bypass_src_network_eth1'})
    option_139789496945936.impl_set_information("separator", (u'Exceptions de proxy pour des IP et adresses r\xe9seaux source', None))
    option_139789496946208 = NetmaskOption(**{'default_multi': u'255.255.255.255', 'doc': u'Masque de sous-r\xe9seau associ\xe9', 'multi': True, 'properties': ('mandatory', 'normal'), 'name': 'proxy_bypass_src_netmask_eth1'})
    option_139789496946208.impl_add_consistency("network_netmask", option_139789496945936, **{})
    option_139789496967568 = Leadership(children=[option_139789496945936, option_139789496946208], **{'doc': u"Exception source de type r\xe9seau pour l'interface 1", 'properties': ('normal',), 'name': 'proxy_bypass_src_network_eth1'})
    option_139789496979536 = NetworkOption(**{'doc': u"Exception source de type r\xe9seau pour l'interface 2", 'multi': True, 'properties': ('normal',), 'name': 'proxy_bypass_src_network_eth2', "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496979808 = NetmaskOption(**{'multi': True, 'name': 'proxy_bypass_src_netmask_eth2', 'default_multi': u'255.255.255.255', 'doc': u'Masque de sous-r\xe9seau associ\xe9', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496979808.impl_add_consistency("network_netmask", option_139789496979536, **{})
    option_139789496967888 = Leadership(children=[option_139789496979536, option_139789496979808], **{'doc': u"Exception source de type r\xe9seau pour l'interface 2", 'properties': ('normal',), 'name': 'proxy_bypass_src_network_eth2'})
    option_139789496980080 = NetworkOption(**{'doc': u"Exception source de type r\xe9seau pour l'interface 3", 'multi': True, 'properties': ('normal',), 'name': 'proxy_bypass_src_network_eth3', "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496980352 = NetmaskOption(**{'multi': True, 'name': 'proxy_bypass_src_netmask_eth3', 'default_multi': u'255.255.255.255', 'doc': u'Masque de sous-r\xe9seau associ\xe9', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496980352.impl_add_consistency("network_netmask", option_139789496980080, **{})
    option_139789496968208 = Leadership(children=[option_139789496980080, option_139789496980352], **{'doc': u"Exception source de type r\xe9seau pour l'interface 3", 'properties': ('normal',), 'name': 'proxy_bypass_src_network_eth3'})
    option_139789496980624 = NetworkOption(**{'doc': u"Exception source de type r\xe9seau pour l'interface 4", 'multi': True, 'properties': ('normal',), 'name': 'proxy_bypass_src_network_eth4', "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496980896 = NetmaskOption(**{'multi': True, 'name': 'proxy_bypass_src_netmask_eth4', 'default_multi': u'255.255.255.255', 'doc': u'Masque de sous-r\xe9seau associ\xe9', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789496980896.impl_add_consistency("network_netmask", option_139789496980624, **{})
    option_139789496968528 = Leadership(children=[option_139789496980624, option_139789496980896], **{'doc': u"Exception source de type r\xe9seau pour l'interface 4", 'properties': ('normal',), 'name': 'proxy_bypass_src_network_eth4'})
    option_139789496981168 = UnicodeOption(**{'doc': u"Ne pas passer par le proxy pour l'h\xf4te ou le domaine", 'multi': True, 'properties': ('normal',), 'name': 'nom_host_wpad_exclude', "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789496981168.impl_set_information("separator", (u"Exceptions d'h\xf4te ou de domaine sp\xe9cifiques \xe0 WPAD", None))
    option_139789496981168.impl_set_information("help", u"""Les exclusions associées sont générées avec la fonction localHostOrDomainIs()""")
    option_139789496981440 = ChoiceOption(**{'multi': True, 'values': (u'Toutes', u'1', u'2', u'3', u'4'), 'name': 'nom_host_interface_wpad_exclude', 'doc': u'Interface(s) concern\xe9e(s)', 'properties': ('normal',), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': True, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': True, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': True, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': True, "option": option_139789497891840}, {'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789496968848 = Leadership(children=[option_139789496981168, option_139789496981440], **{'doc': u"Ne pas passer par le proxy pour l'h\xf4te ou le domaine", 'properties': ('normal',), 'name': 'nom_host_wpad_exclude'})
    option_139789496969168 = OptionDescription(children=[option_139789496931744, option_139789496931472, option_139789496932016, option_139789496932288, option_139789496932560, option_139789496932832, option_139789496933104, option_139789496933376, option_139789496855120, option_139789496942672, option_139789496855440, option_139789496943488, option_139789496855760, option_139789496944304, option_139789496856080, option_139789496945120, option_139789496967248, option_139789496967568, option_139789496967888, option_139789496968208, option_139789496968528, option_139789496968848], **{'doc': u'Exceptions proxy', 'properties': ('basic',), 'name': 'exceptions_proxy'})
    option_139789496969168.impl_set_information("help", u"""Gestion des exceptions de proxy""")
    option_139789496969168.impl_set_information("icon", u"""code-fork""")
    option_139789496981712 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_cache_pere', 'default': 'non', 'doc': u'Utiliser un proxy web parent global', 'properties': ('mandatory', 'expert')})
    option_139789496981712.impl_set_information("separator", ('Proxy parent global', None))
    option_139789496981712.impl_set_information("help", u"""Déclarer un ou plusieurs serveurs proxy parents qui seront utilisés pour toutes les zones DNS""")
    option_139789496981984 = DomainnameOption(**{'multi': True, 'name': 'nom_cache_pere', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Adresse du serveur proxy web parent', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496981712}]})
    option_139789496982256 = PortOption(**{'multi': True, 'name': 'port_cache_pere', 'allow_private': True, 'doc': u'Port du serveur proxy web parent', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496981712}]})
    option_139789496982528 = PortOption(**{'multi': True, 'name': 'port_icp_cache_pere', 'allow_private': True, 'default_multi': '3130', 'doc': u'Port ICP du serveur proxy web parent', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496981712}]})
    option_139789496982800 = UnicodeOption(**{'multi': True, 'name': 'options_cache_pere', 'default_multi': u'no-query', 'doc': u'Options du serveur proxy web parent', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496981712}]})
    option_139789496982800.impl_set_information("help", u"""Options, séparées par des espaces, ajoutées à la configuration du serveur cache parent (exemple \"no-query no-netdb-exchange no-digest\"). Attention, l'option \"round-robin\" est ajoutée automatiquement si plusieurs serveurs père sont définis.""")
    option_139789496969488 = Leadership(children=[option_139789496981984, option_139789496982256, option_139789496982528, option_139789496982800], **{'doc': u'Adresse du serveur proxy web parent', 'properties': ('expert',), 'name': 'nom_cache_pere'})
    option_139789496983072 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'proxy_pere_rvp', 'default': 'non', 'doc': u'Acc\xe8s aux zones RVP en passant par le serveur proxy web parent global', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496981712}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789496483920 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_cache_pere_zone', 'default': 'non', 'doc': u'Utiliser un proxy web parent par zone', 'properties': ('mandatory', 'expert')})
    option_139789496483920.impl_set_information("separator", ('Proxy parent par zone', None))
    option_139789496483920.impl_set_information("help", u"""Déclarer un ou plusieurs serveurs proxy parents associés à des zones DNS particulières""")
    option_139789496484192 = DomainnameOption(**{'multi': True, 'name': 'nom_cache_pere_zone', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Adresse du serveur proxy web parent pour la zone', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496483920}]})
    option_139789496484464 = PortOption(**{'multi': True, 'name': 'port_cache_pere_zone', 'allow_private': True, 'doc': u'Port du serveur web parent pour la zone', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496483920}]})
    option_139789496484736 = PortOption(**{'multi': True, 'name': 'port_icp_cache_pere_zone', 'allow_private': True, 'default_multi': '3130', 'doc': u'Port ICP du serveur web parent pour la zone', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496483920}]})
    option_139789496485008 = ChoiceOption(**{'multi': True, 'values': (u'DNS', u'nom fichier'), 'name': 'type_nom_zone_dns_cache', 'default_multi': u'DNS', 'doc': u"M\xe9thode d'utilisation de la zone accessible via ce serveur web parent", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496483920}]})
    option_139789496485008.impl_set_information("help", u"""Les zones DNS de destination peuvent être soit renseignées directement dans la variable, soit renseignées dans un fichier texte""")
    option_139789496485552 = UnicodeOption(**{'doc': u'Nom DNS ou nom de fichier de la zone accessible via ce serveur web parent', 'multi': True, 'properties': ('mandatory', 'expert'), 'name': 'nom_zone_dns_cache', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496483920}]})
    option_139789496485552.impl_set_information("help", u"""Le nom DNS commence généralement par un '.' (.example.net)""")
    option_139789496485280 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'autoriser_proxy_autres', 'default_multi': 'non', 'doc': u"Autoriser son utilisation par d'autres zones que celle de l'interface 1", 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496483920}]})
    option_139789496485824 = UnicodeOption(**{'multi': True, 'name': 'options_cache_pere_zone', 'default_multi': u'proxy-only no-query', 'doc': u'Options du serveur proxy web parent', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496483920}]})
    option_139789496485824.impl_set_information("help", u"""Options, séparées par des espaces, ajoutées à la configuration du serveur cache parent de la zone (exemple \"no-query no-netdb-exchange no-digest\").""")
    option_139789496969808 = Leadership(children=[option_139789496484192, option_139789496484464, option_139789496484736, option_139789496485008, option_139789496485552, option_139789496485280, option_139789496485824], **{'doc': u'Adresse du serveur proxy web parent pour la zone', 'properties': ('expert',), 'name': 'nom_cache_pere_zone'})
    option_139789496486096 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_proxy_sibling', 'default': 'non', 'doc': u'Activer la coop\xe9ration des caches', 'properties': ('mandatory', 'expert')})
    option_139789496486096.impl_set_information("separator", (u'Coop\xe9ration des caches', None))
    option_139789496486096.impl_set_information("help", u"""Les serveurs frères (siblings) sont des serveurs de cache d'un niveau hiérarchique égal dont le but est de répartir la charge""")
    option_139789496486368 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'proxy_sibling_ip', 'doc': u'Adresse IP du serveur proxy distant', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496486096}]})
    option_139789496486640 = PortOption(**{'multi': True, 'name': 'proxy_sibling_port', 'allow_private': True, 'doc': u'Port du serveur proxy distant', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496486096}]})
    option_139789496970128 = Leadership(children=[option_139789496486368, option_139789496486640], **{'doc': u'Adresse IP du serveur proxy distant', 'properties': ('expert',), 'name': 'proxy_sibling_ip'})
    option_139789496970448 = OptionDescription(children=[option_139789496981712, option_139789496969488, option_139789496983072, option_139789496483920, option_139789496969808, option_139789496486096, option_139789496970128], **{'doc': u'Proxy parent', 'properties': ('expert',), 'name': 'proxy_parent'})
    option_139789496970448.impl_set_information("help", u"""Configuration du chaînage du proxy""")
    option_139789496970448.impl_set_information("icon", u"""code-fork""")
    option_139789496486912 = DomainnameOption(**{'multi': True, 'name': 'wpad_domains', 'allow_ip': False, 'type_': 'domainname', 'doc': u'Nom de domaine du service WPAD', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497890752, False))})
    option_139789496970768 = OptionDescription(children=[option_139789496486912], requires=[{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}], **{'doc': u'WPAD', 'properties': ('normal',), 'name': 'wpad'})
    option_139789496970768.impl_set_information("help", u"""Paramètrage de la découverte automatique du proxy""")
    option_139789496970768.impl_set_information("icon", u"""code-fork""")
    option_139789496487184 = ChoiceOption(**{'multi': False, 'name': 'revprox_auto_config_local_web', 'doc': u'Activer la configuration automatique pour les applications locales', 'values': (u'oui', u'non'), 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("oui"),), kwargs={'mismatch': ParamValue("non"), 'condition_1': ParamOption(option_139789497957376, False), 'match': ParamValue("oui"), 'condition_2': ParamOption(option_139789497972400, False)}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789496487184.impl_set_information("help", u"""Configure automatiquement le reverse proxy afin de faire pointer web_url sur le conteneur web""")
    option_139789496487456 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'test_nginx_open_port', 'doc': u'Ouvrir les ports Nginx', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("oui"),), kwargs={'mismatch': ParamValue("non"), 'condition_1': ParamOption(option_139789497413712, True), 'condition_3': ParamOption(option_139789497957920, True), 'condition_2': ParamOption(option_139789496487184, True), 'operator': ParamValue("OR"), 'match': ParamValue("oui")})})
    option_139789496487456.impl_set_information("help", u"""Vérifie que la publication d’application web par Nginx ou le reverse proxy HTTP ou la configuration automatique des applications locales est actif.""")
    option_139789496504400 = DomainnameOption(**{'type_': 'domainname', 'name': 'revprox_default', 'doc': u'Nom de domaine par d\xe9faut', 'allow_ip': False, 'multi': False, 'properties': ('normal',)})
    option_139789496504400.impl_set_information("help", u"""Si un client accède au serveur avec un nom de domaine non déclaré, le flux est redirigé vers ce domaine""")
    option_139789496504672 = ChoiceOption(**{'multi': False, 'values': (u'128', u'64', u'32'), 'name': 'revprox_hash_bucket_size', 'default': u'128', 'doc': u'Longueur maximum pour un nom de domaine', 'properties': ('mandatory', 'expert')})
    option_139789496508496 = OptionDescription(children=[option_139789496487456, option_139789496504400, option_139789497962848, option_139789496504672], requires=[{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497972672}], **{'doc': u'Nginx', 'properties': ('normal',), 'name': 'nginx'})
    option_139789496508496.impl_set_information("help", u"""Paramétrage global de Nginx""")
    option_139789496508496.impl_set_information("icon", u"""nginx""")
    option_139789496504944 = UnicodeOption(**{'multi': False, 'name': 'ead3_http_path', 'default': u'/ead', 'doc': u'Chemin d\u2019acc\xe8s de l\u2019application EAD3', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963392}]})
    option_139789496504944.impl_set_information("separator", ('EAD', None))
    option_139789496504944.impl_set_information("help", u"""Chemin absolu (commençant par un /) sous lequel le service EAD3 sera accessible""")
    option_139789496505216 = UnicodeOption(**{'multi': False, 'name': 'genconfig_http_path', 'default': u'/genconfig', 'doc': u'Chemin d\u2019acc\xe8s de l\u2019application GenConfig', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497974032}]})
    option_139789496505216.impl_set_information("separator", ('GenConfig', None))
    option_139789496505216.impl_set_information("help", u"""Chemin absolu (commençant par un /) sous lequel le service GenConfig sera accessible""")
    option_139789496508816 = OptionDescription(children=[option_139789496504944, option_139789496505216], **{'doc': u'Applications web Nginx', 'properties': ('expert',), 'name': 'applications_web_nginx'})
    option_139789496508816.impl_set_information("help", u"""Paramétrage des applications web publiées par Nginx""")
    option_139789496508816.impl_set_information("icon", u"""nginx""")
    option_139789496505488 = DomainnameOption(**{'multi': False, 'name': 'revprox_sso', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Nom de domaine du serveur SSO', 'properties': ('normal',), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}]})
    option_139789496505488.impl_set_information("separator", ('Redirection de services particuliers', True))
    option_139789496505488.impl_set_information("help", u"""Pour rediriger vers le SSO de la machine \"domainelocal\", taper \"domainelocal\"""")
    option_139789496505760 = DomainnameOption(**{'multi': False, 'name': 'revprox_eop', 'allow_ip': True, 'allow_without_dot': True, 'type_': 'domainname', 'doc': u'Nom de domaine du serveur EOP', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'normal'), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("oui"),), kwargs={'operator': ParamValue("OR"), 'condition_1': ParamOption(option_139789497146368, True), 'default_mismatch': ParamValue("None"), 'match': ParamOption(option_139789497854704, False)})})
    option_139789496505760.impl_set_information("help", u"""Pour rediriger vers EOP de la machine \"domainelocal\", taper \"domainelocal\"""")
    option_139789496506032 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_revprox_ead', 'default': 'non', 'doc': u"Activer la redirection de l'EAD Scribe", 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789496506576 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'revprox_ead', 'doc': u'IP du Scribe pour la redirection EAD', 'properties': ('mandatory', 'normal'), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497983600, True)), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496506032}]})
    option_139789496506304 = PortOption(**{'multi': False, 'name': 'revprox_ead_port', 'allow_private': True, 'default': '4203', 'doc': u"Port de l'EAD sur le Scribe", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496506032}]})
    option_139789496506848 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'revprox_domain_wildcard', 'default_multi': 'non', 'doc': u'Activer la redirection pour tous les sous-domaines', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497413712}]})
    option_139789496506848.impl_set_information("help", u"""Exemple pour \"domaine\" : tous les sous-domaines de \"domaine\" seront redirigés""")
    option_139789496507120 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'revprox_le_cert', 'default_multi': 'oui', 'doc': u"Demander un certificat \xe0 Let's Encrypt pour ce domaine ?", 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'autosign\xe9', 'action': 'disabled', 'inverse': False, "option": option_139789497939360}, {'expected': u'manuel', 'action': 'disabled', 'inverse': False, "option": option_139789497939360}, {'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789496506848}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497413712}]})
    option_139789496507120.impl_set_information("help", u"""Demander un certificat SSL au serveur ACME (Let's Encrypt) pour ce domaine""")
    option_139789496507392 = UnicodeOption(**{'multi': True, 'name': 'revprox_rep', 'default_multi': u'/', 'doc': u'R\xe9pertoire ou nom de la page \xe0 rediriger', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497413712}]})
    option_139789496507392.impl_set_information("help", u"""URL relative (sans le nom de domaine) redirigée pour l'adresse définie dans la variable ci-dessus (exemple \"/mail\")""")
    option_139789496507664 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non', u'redirige vers https'), 'name': 'revprox_http', 'default_multi': u'redirige vers https', 'doc': u'Reverse proxy HTTP', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497413712}]})
    option_139789496507936 = ChoiceOption(**{'multi': True, 'values': (u'oui', u'non'), 'name': 'revprox_https', 'default_multi': 'oui', 'doc': u'Reverse proxy HTTPS', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497413712}]})
    option_139789496520784 = URLOption(**{'multi': True, 'name': 'revprox_url', 'doc': u'IP ou domaine de destination (avec http:// ou https://) ou URI compl\xe8te', 'allow_ip': True, 'allow_without_dot': True, 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497413712}]})
    option_139789496520784.impl_set_information("help", u"""Nom de domaine ou IP de destination, par exemple \"http://domainelocal\" ou URI, par exemple \"http://domainelocal/dir/\"""")
    option_139789496509136 = Leadership(children=[option_139789497413984, option_139789496506848, option_139789496507120, option_139789496507392, option_139789496507664, option_139789496507936, option_139789496520784], **{'doc': u'Nom de domaine ou IP \xe0 rediriger', 'properties': ('basic',), 'name': 'revprox_domainname'})
    option_139789496521056 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_revprox_rewrite', 'default': 'non', 'doc': u"Activer la r\xe9\xe9criture d'URL", 'properties': ('mandatory', 'expert')})
    option_139789496521328 = DomainnameOption(**{'multi': True, 'name': 'revprox_rewrite_domaine', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Nom de domaine concern\xe9 par la r\xe9\xe9criture', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496521056}]})
    option_139789496521600 = ChoiceOption(**{'multi': True, 'values': (u'http/https', u'http', u'https'), 'name': 'revprox_rewrite_proto', 'default_multi': u'http/https', 'doc': u'Protocole', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496521056}]})
    option_139789496521872 = UnicodeOption(**{'doc': u'R\xe9pertoire de la r\xe9\xe9criture', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'revprox_rewrite_location', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496521056}]})
    option_139789496522144 = UnicodeOption(**{'doc': u'Regex de la r\xe9\xe9criture', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'revprox_rewrite_regex', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496521056}]})
    option_139789496522416 = UnicodeOption(**{'doc': u'La valeur de remplacement', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'revprox_rewrite_replacement', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496521056}]})
    option_139789496509456 = Leadership(children=[option_139789496521328, option_139789496521600, option_139789496521872, option_139789496522144, option_139789496522416], **{'doc': u'Nom de domaine concern\xe9 par la r\xe9\xe9criture', 'properties': ('basic',), 'name': 'revprox_rewrite_domaine'})
    option_139789496522688 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'revprox_redirection_http', 'default': 'non', 'doc': u'Activer la redirection des domaines', 'properties': ('mandatory', 'normal')})
    option_139789496522960 = DomainnameOption(**{'multi': True, 'name': 'revprox_redirection', 'allow_without_dot': True, 'allow_ip': True, 'type_': 'domainname', 'doc': u'Nom de domaine \xe0 rediriger', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496522688}]})
    option_139789496523232 = UnicodeOption(**{'multi': True, 'name': 'revprox_redirection_rep', 'default_multi': u'/', 'doc': u'Nom du r\xe9pertoire du domaine \xe0 rediriger', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496522688}]})
    option_139789496523504 = URLOption(**{'multi': True, 'name': 'revprox_redirection_destination', 'doc': u'Nom de domaine de destination', 'allow_ip': True, 'allow_without_dot': True, 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496522688}]})
    option_139789496509776 = Leadership(children=[option_139789496522960, option_139789496523232, option_139789496523504], **{'doc': u'Nom de domaine \xe0 rediriger', 'properties': ('basic',), 'name': 'revprox_redirection'})
    option_139789496510096 = OptionDescription(children=[option_139789496505488, option_139789496505760, option_139789496506032, option_139789496506576, option_139789496506304, option_139789496487184, option_139789497413712, option_139789496509136, option_139789496521056, option_139789496509456, option_139789496522688, option_139789496509776], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972400}], **{'doc': u'Reverse proxy', 'properties': ('basic',), 'name': 'reverse_proxy'})
    option_139789496510096.impl_set_information("help", u"""Paramètrage du proxy inverse""")
    option_139789496510096.impl_set_information("icon", u"""nginx""")
    option_139789496523776 = UnicodeOption(**{'multi': False, 'name': 'ftp_servername', 'doc': u'Nom du serveur FTP', 'properties': ('expert',), 'callback': tiramisu_copy, 'callback_params': Params(args=ParamOption(option_139789497889936, False))})
    option_139789496524048 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ftp_tls', 'default': 'non', 'doc': u'Activer le chiffrement TLS', 'properties': ('mandatory', 'expert')})
    option_139789496524320 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_ftp_anonymous_access', 'default': 'non', 'doc': u"Activer l'acc\xe8s anonyme", 'properties': ('mandatory', 'expert')})
    option_139789496545360 = FilenameOption(**{'multi': False, 'name': 'ftp_anonymous_directory', 'doc': u'Chemin du r\xe9pertoire anonyme', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496524320}], "validator": eosfunc.valid_regexp, 'validator_params': Params(args=(ParamValue(u"^/"),), kwargs={'err_msg': ParamValue(u"Renseigner un chemin absolu")})})
    option_139789496545632 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_ftp_access', 'default': 'non', 'doc': u'Activer des acc\xe8s FTP suppl\xe9mentaires', 'properties': ('mandatory', 'expert')})
    option_139789496545904 = FilenameOption(**{'multi': True, 'name': 'ftp_access_directory', 'doc': u'Chemin du r\xe9pertoire FTP suppl\xe9mentaire', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496545632}], "validator": eosfunc.valid_regexp, 'validator_params': Params(args=(ParamValue(u"^/"),), kwargs={'err_msg': ParamValue(u"Renseigner un chemin absolu")})})
    option_139789496546176 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ftp_pam_ldap', 'default': 'oui', 'doc': u'Autoriser les comptes LDAP \xe0 se connecter en acc\xe8s FTP', 'properties': ('mandatory', 'expert')})
    option_139789496546448 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ftp_pam_sso', 'default': 'oui', 'doc': u'Autoriser CAS en acc\xe8s FTP', 'properties': ('mandatory', 'expert'), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496546176}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}]})
    option_139789496546720 = IntOption(**{'default': 50, 'doc': u"Nombre maximum d'utilisateurs simultan\xe9s", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ftp_maxclients'})
    option_139789496546992 = IntOption(**{'default': 40, 'doc': u'Nombre maximum de processus pour ProFTPD', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ftp_maxinstances'})
    option_139789496547264 = IntOption(**{'default': 500, 'doc': u'Taille maximum du fichier r\xe9cup\xe9r\xe9 (download) en Mb', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ftp_maxretrievefilesize'})
    option_139789496547536 = IntOption(**{'default': 100, 'doc': u'Taille maximum du fichier d\xe9pos\xe9 (upload) en Mb', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ftp_maxstorefilesize'})
    option_139789496547808 = IntOption(**{'default': 1200, 'doc': u"Temps maximum d'inactivit\xe9 avant d\xe9connexion (en secondes)", 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'ftp_timeoutidle'})
    option_139789496548080 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ftp_perso_ele', 'default': 'non', 'doc': u"Activer l'acc\xe8s aux dossiers personnels des \xe9l\xe8ves pour les professeurs", 'properties': ('mandatory', 'expert')})
    option_139789496510416 = OptionDescription(children=[option_139789496523776, option_139789496524048, option_139789496524320, option_139789496545360, option_139789496545632, option_139789496545904, option_139789496546176, option_139789496546448, option_139789496546720, option_139789496546992, option_139789496547264, option_139789496547536, option_139789496547808, option_139789496548080], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}], **{'doc': u'ProFTPD', 'properties': ('expert',), 'name': 'proftpd'})
    option_139789496510416.impl_set_information("help", u"""Paramétrage avancé du serveur FTP""")
    option_139789496510416.impl_set_information("icon", u"""cloud-download""")
    option_139789496548352 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'activer_proxy_radius', 'default': 'non', 'doc': u'Activer le proxy radius', 'properties': ('mandatory', 'normal')})
    option_139789496548352.impl_set_information("help", u"""Permet de transmettre les requêtes du client à d'autres serveurs RADIUS""")
    option_139789496548624 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ttls_copy_request_to_tunnel', 'default': 'oui', 'doc': u"Copier les attributs dans la requ\xeate d'authentification du tunnel pour le protocole TTLS", 'properties': ('mandatory', 'expert')})
    option_139789496548896 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'ttls_use_tunneled_reply', 'default': 'oui', 'doc': u'R\xe9pondre dans la requ\xeate de tunnel pour le protocole TTLS', 'properties': ('mandatory', 'expert')})
    option_139789496549456 = ChoiceOption(**{'multi': False, 'values': (u'802.1x', u'accounting'), 'name': 'freerad_mode', 'default': u'802.1x', 'doc': u"Mode d'utilisation de FreeRADIUS", 'properties': ('mandatory', 'normal')})
    option_139789496549456.impl_set_information("help", u"""802.1x : mode filaire
accounting : mode wifi""")
    option_139789496549728 = ChoiceOption(**{'multi': False, 'values': (u'0', u'1', u'2', u'3', u'4'), 'name': 'freerad_listen_int', 'default': u'0', 'doc': u"Num\xe9ro de l'interface sur laquelle FreeRADIUS \xe9coutera", 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'accounting', 'action': 'disabled', 'inverse': False, "option": option_139789496549456}]})
    option_139789496550000 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'freerad_listen_addr', 'doc': u'Adresse IP sur laquelle FreeRADIUS \xe9coutera', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'802.1x', 'action': 'disabled', 'inverse': False, "option": option_139789496549456}]})
    option_139789496550272 = IPOption(**{'doc': u"Adresse IP du serveur d'acc\xe8s (NAS)", 'multi': True, 'allow_reserved': True, 'properties': ('mandatory', 'basic'), 'name': 'freerad_nas_ip'})
    option_139789496550272.impl_set_information("separator", ('Configuration des NAS', None))
    option_139789496550544 = IntOption(**{'doc': u"Masque de sous r\xe9seau (notation CIDR) du serveur d'acc\xe8s (NAS)", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'freerad_nas_cidr_netmask', "requires": [{'expected': u'802.1x', 'action': 'disabled', 'inverse': False, "option": option_139789496549456}]})
    option_139789496550544.impl_set_information("help", u"""Masque au format CIDR (255.255.255.0 -> 24)""")
    option_139789496550816 = UnicodeOption(**{'doc': u"Nom court du serveur d'acc\xe8s (NAS)", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'freerad_nas_name'})
    option_139789496551088 = PasswordOption(**{'doc': u"Secret partag\xe9 avec le serveur d'acc\xe8s (NAS)", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'freerad_nas_passwd'})
    option_139789496551360 = ChoiceOption(**{'multi': True, 'values': (u'cisco', u'computone', u'livingston', u'max40xx', u'multitech', u'netserver', u'pathras', u'patton', u'portslave', u'tc', u'usrhiper', u'other'), 'name': 'freerad_nas_type', 'default_multi': u'other', 'doc': u"Type du serveur d'acc\xe8s (NAS)", 'properties': ('mandatory', 'basic')})
    option_139789496510736 = Leadership(children=[option_139789496550272, option_139789496550544, option_139789496550816, option_139789496551088, option_139789496551360], **{'doc': u"Adresse IP du serveur d'acc\xe8s (NAS)", 'properties': ('basic',), 'name': 'freerad_nas_ip'})
    option_139789496551632 = IPOption(**{'doc': u'Adresse IP du serveur LDAP permettant de r\xe9cup\xe9rer les comptes utilisateurs', 'multi': False, 'allow_reserved': True, 'properties': ('mandatory', 'basic'), 'name': 'freerad_ldap_ip'})
    option_139789496551632.impl_set_information("separator", ('Configuration LDAP', None))
    option_139789496551904 = UnicodeOption(**{'default': u'o=gouv,c=fr', 'doc': u"Suffixe racine de l'annuaire LDAP (base DN)", 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'freerad_ldap_base_dn'})
    option_139789496552176 = PasswordOption(**{'doc': u"Cl\xe9 d'acc\xe8s reader \xe0 la base LDAP sur Scribe (/root/.reader)", 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'freerad_cle_ldap_scribe', "requires": [{'expected': u'802.1x', 'action': 'disabled', 'inverse': False, "option": option_139789496549456}]})
    option_139789496552448 = UsernameOption(**{'doc': u"Groupe d'utilisateurs \xe0 r\xe9cup\xe9rer dans l'annuaire LDAP", 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'freerad_ldap_group', "requires": [{'expected': u'accounting', 'action': 'disabled', 'inverse': False, "option": option_139789496549456}]})
    option_139789496552448.impl_set_information("proposed_value", (u'eleves', u'professeurs'))
    option_139789496552448.impl_set_information("separator", ('Configuration des groupes et des VLAN', None))
    option_139789496552720 = IntOption(**{'doc': u'Num\xe9ro de VLAN \xe0 attribuer \xe0 ce groupe', 'multi': True, 'properties': ('mandatory', 'basic'), 'name': 'freerad_group_vlan', "requires": [{'expected': u'accounting', 'action': 'disabled', 'inverse': False, "option": option_139789496549456}]})
    option_139789496511056 = Leadership(children=[option_139789496552448, option_139789496552720], **{'doc': u"Groupe d'utilisateurs \xe0 r\xe9cup\xe9rer dans l'annuaire LDAP", 'properties': ('basic',), 'name': 'freerad_ldap_group'})
    option_139789496511376 = OptionDescription(children=[option_139789496548352, option_139789496548624, option_139789496548896, option_139789496549456, option_139789496549728, option_139789496550000, option_139789496510736, option_139789496551632, option_139789496551904, option_139789496552176, option_139789496511056], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}], **{'doc': u'freeradius', 'properties': ('basic',), 'name': 'freeradius'})
    option_139789496511376.impl_set_information("help", u"""Paramétrage de l'authentification RADIUS""")
    option_139789496511376.impl_set_information("icon", u"""bullseye""")
    option_139789496552992 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'listen_from_network', 'default': 'oui', 'doc': u"En \xe9coute depuis l'ext\xe9rieur", 'properties': ('mandatory', 'expert')})
    option_139789496552992.impl_set_information("help", u"""Le serveur eoleflask doit il être accessible depuis l'extérieur ?""")
    option_139789496511696 = OptionDescription(children=[option_139789496552992, option_139789497145824], **{'doc': u'EOLEFlask', 'properties': ('expert',), 'name': 'eoleflask'})
    option_139789496511696.impl_set_information("help", u"""Paramétrage du framework d'application web Flask""")
    option_139789496511696.impl_set_information("icon", u"""python""")
    option_139789496561744 = ChoiceOption(**{'multi': False, 'values': (u'automatique', u'manuel'), 'name': 'synchro_aaf', 'default': u'manuel', 'doc': u'Mode de synchronisation AAF', 'properties': ('mandatory', 'expert')})
    option_139789496561744.impl_set_information("help", u"""Le module Zéphir permet de synchroniser les comptes depuis l'Annuaire Académique Fédérateur""")
    option_139789496562016 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'synchro_aaf_sendmail', 'default': 'non', 'doc': u"Envoi d'un courrier \xe9lectronique en cas d'erreur", 'properties': ('mandatory', 'expert')})
    option_139789496562016.impl_set_information("help", u"""Active l'envoi d'un courrier électronique en cas d'erreur lors de l'import AAF automatisé""")
    option_139789496562288 = EmailOption(**{'multi': True, 'name': 'synchro_aaf_mail', 'doc': u'Adresse(s) \xe9lectronique(s) \xe0 utiliser', 'properties': ('mandatory', 'expert'), 'callback': eosfunc.calc_multi_val, 'callback_params': Params(args=(ParamOption(option_139789497462864, False),), kwargs={}), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496562016}]})
    option_139789496562560 = UnicodeOption(**{'multi': False, 'name': 'code_ent', 'doc': u"Code de l'ENT (tel que d\xe9fini par le SDET v2.0)", 'properties': ('expert',), "validator": eosfunc.valid_regexp, 'validator_params': Params(args=(ParamValue(u"^[A-Z][0-9]$"),), kwargs={'err_msg': ParamValue(u"L'identifiant doit etre compose d'une lettre et d'un chiffre")})})
    option_139789496562560.impl_set_information("help", u"""Le module Zéphir permet de distribuer des plages d'identifiants ENT aux établissements""")
    option_139789496512016 = OptionDescription(children=[option_139789496561744, option_139789496562016, option_139789496562288, option_139789496562560], **{'doc': u'ENT', 'properties': ('expert',), 'name': 'ent'})
    option_139789496512016.impl_set_information("icon", None)
    option_139789496562832 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'roundcube_tls', 'default': 'non', 'doc': u'Serveur de courrier \xe9lectronique est en TLS', 'properties': ('mandatory', 'expert')})
    option_139789496563104 = ChoiceOption(**{'multi': False, 'values': (u'default', u'externe', u'local'), 'name': 'roundcube_db_mode', 'default': u'default', 'doc': u'Serveur de base de donn\xe9es \xe0 utiliser', 'properties': ('expert',)})
    option_139789496563104.impl_set_information("separator", (u'Base de donn\xe9es', None))
    option_139789496563104.impl_set_information("help", u"""Utilisation ou non d'un paramètrage spécifique de base de données pour roundcube""")
    option_139789496563376 = UnicodeOption(**{'doc': u'Adresse du serveur de base de donn\xe9es', 'multi': False, 'properties': ('expert',), 'name': 'roundcube_dbserver', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': False, "option": option_139789496563104}, {'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496563104}]})
    option_139789496563376.impl_set_information("help", u"""Adresse du serveur de base de données par défaut de roundcube""")
    option_139789496563648 = IntOption(**{'doc': u"Port d'\xe9coute du serveur de base de donn\xe9es", 'multi': False, 'properties': ('expert',), 'name': 'roundcube_dbport', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': False, "option": option_139789496563104}, {'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496563104}]})
    option_139789496563648.impl_set_information("help", u"""Port d'écoute du serveur de base de données par défaut de roundcube""")
    option_139789496563920 = UnicodeOption(**{'doc': u'H\xf4tes autoris\xe9s \xe0 utiliser la base de donn\xe9es', 'multi': True, 'properties': ('expert',), 'name': 'roundcube_allow_hosts', "requires": [{'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496563104}]})
    option_139789496563920.impl_set_information("help", u"""Adresses IP avec lesquelles les utilisateurs de la base de données de roundcube sont suceptibles de se connecter""")
    option_139789496564192 = UnicodeOption(**{'doc': u'Utilisateur du serveur de base de donn\xe9es', 'multi': False, 'properties': ('expert',), 'name': 'roundcube_dbuser', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': False, "option": option_139789496563104}, {'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496563104}]})
    option_139789496564192.impl_set_information("help", u"""Nom d'utilisateur d'administration du serveur de base de données par défaut de roundcube""")
    option_139789496564464 = UnicodeOption(**{'doc': u'Fichier de mot de passe du serveur', 'multi': False, 'properties': ('expert',), 'name': 'roundcube_dbpass', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': False, "option": option_139789496563104}, {'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496563104}]})
    option_139789496564464.impl_set_information("help", u"""Fichier contenant le mot de passe de l'administrateur du serveur de base de données par défaut de roundcube""")
    option_139789496574032 = OptionDescription(children=[option_139789496562832, option_139789496563104, option_139789496563376, option_139789496563648, option_139789496563920, option_139789496564192, option_139789496564464], **{'doc': u'Roundcube', 'properties': ('expert',), 'name': 'roundcube'})
    option_139789496574032.impl_set_information("help", u"""Outil de gestion automatique des base de données de roundcube""")
    option_139789496574032.impl_set_information("icon", u"""envelope""")
    option_139789496564736 = ChoiceOption(**{'multi': False, 'values': (u'default', u'externe', u'local'), 'name': 'pydio_db_mode', 'default': u'default', 'doc': u'Serveur de bases de donn\xe9es \xe0 utiliser', 'properties': ('expert',)})
    option_139789496565008 = UnicodeOption(**{'doc': u'Adresse du serveur de base de donn\xe9es', 'multi': False, 'properties': ('expert',), 'name': 'pydio_dbserver', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': False, "option": option_139789496564736}, {'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496564736}]})
    option_139789496565280 = IntOption(**{'doc': u"Port d'\xe9coute du serveur de base de donn\xe9es", 'multi': False, 'properties': ('expert',), 'name': 'pydio_dbport', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': False, "option": option_139789496564736}, {'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496564736}]})
    option_139789496582224 = UnicodeOption(**{'doc': u'H\xf4tes authoris\xe9s \xe0 utiliser la base de donn\xe9es', 'multi': True, 'properties': ('expert',), 'name': 'pydio_allow_hosts', "requires": [{'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496564736}]})
    option_139789496582496 = UnicodeOption(**{'doc': u'Utilisateur du serveur de base de donn\xe9es', 'multi': False, 'properties': ('expert',), 'name': 'pydio_dbuser', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': False, "option": option_139789496564736}, {'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496564736}]})
    option_139789496582768 = UnicodeOption(**{'doc': u'Fichier de mot de passe du serveur', 'multi': False, 'properties': ('expert',), 'name': 'pydio_dbpass', "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': False, "option": option_139789496564736}, {'expected': u'default', 'action': 'disabled', 'inverse': False, "option": option_139789496564736}]})
    option_139789496574352 = OptionDescription(children=[option_139789496564736, option_139789496565008, option_139789496565280, option_139789496582224, option_139789496582496, option_139789496582768], **{'doc': u'Pydio', 'properties': ('expert',), 'name': 'pydio'})
    option_139789496574352.impl_set_information("icon", None)
    option_139789496574672 = OptionDescription(children=[option_139789498700496, option_139789498700816, option_139789498701136, option_139789498701456, option_139789498702416, option_139789498702736, option_139789498044816, option_139789498047056, option_139789497676496, option_139789497678416, option_139789497279184, option_139789497281104, option_139789497282064, option_139789497434192, option_139789497434832, option_139789497435152, option_139789497436112, option_139789497436432, option_139789497436752, option_139789497437072, option_139789497437392, option_139789497036880, option_139789497037200, option_139789497037840, option_139789497038160, option_139789497038480, option_139789497039440, option_139789497040400, option_139789497126992, option_139789497127632, option_139789497127952, option_139789497128592, option_139789497129232, option_139789497130192, option_139789497258064, option_139789497258384, option_139789497259024, option_139789497259664, option_139789497259984, option_139789497260304, option_139789497260944, option_139789497261264, option_139789497261584, option_139789496853200, option_139789496853840, option_139789496854160, option_139789496854480, option_139789496854800, option_139789496969168, option_139789496970448, option_139789496970768, option_139789496508496, option_139789496508816, option_139789496510096, option_139789496510416, option_139789496511376, option_139789496511696, option_139789496512016, option_139789496574032, option_139789496574352], **{'doc': u'', 'properties': (), 'name': 'creole'})
    option_139789496583040 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496583312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496583584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496583856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496584128 = IntOption(**{'default': 1, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496578968 = SymLinkOption(**{'name': 'ip', "opt": option_139789498665440})
    option_139789496579528 = SymLinkOption(**{'name': 'path', "opt": option_139789498665168})
    option_139789496574992 = OptionDescription(children=[option_139789496583040, option_139789496583312, option_139789496583584, option_139789496583856, option_139789496584128, option_139789496578968, option_139789496579528], **{'doc': u'', 'properties': (), 'name': 'container0'})
    option_139789496584400 = UnicodeOption(**{'default': u'all', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496584672 = UnicodeOption(**{'default': u'all', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496584944 = UnicodeOption(**{'default': u'all', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496585216 = UnicodeOption(**{'default': u'all', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496580088 = SymLinkOption(**{'name': 'ip', "opt": option_139789498666256})
    option_139789496580648 = SymLinkOption(**{'name': 'path', "opt": option_139789498665984})
    option_139789496575312 = OptionDescription(children=[option_139789496584400, option_139789496584672, option_139789496584944, option_139789496585216, option_139789496580088, option_139789496580648], **{'doc': u'', 'properties': (), 'name': 'container1'})
    option_139789496585488 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496585760 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496586320 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496586592 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496586864 = IntOption(**{'default': 52, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496581208 = SymLinkOption(**{'name': 'ip', "opt": option_139789497867264})
    option_139789496581768 = SymLinkOption(**{'name': 'path', "opt": option_139789497866992})
    option_139789496575632 = OptionDescription(children=[option_139789496585488, option_139789496585760, option_139789496586320, option_139789496586592, option_139789496586864, option_139789496581208, option_139789496581768], **{'doc': u'', 'properties': (), 'name': 'container10'})
    option_139789496587136 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496587408 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496587680 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496587952 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496588224 = IntOption(**{'default': 53, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496591256 = SymLinkOption(**{'name': 'ip', "opt": option_139789497872464})
    option_139789496591816 = SymLinkOption(**{'name': 'path', "opt": option_139789497867808})
    option_139789496575952 = OptionDescription(children=[option_139789496587136, option_139789496587408, option_139789496587680, option_139789496587952, option_139789496588224, option_139789496591256, option_139789496591816], **{'doc': u'', 'properties': (), 'name': 'container11'})
    option_139789496588496 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496588768 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496589040 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496589312 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496589584 = IntOption(**{'default': 50, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496594056 = SymLinkOption(**{'name': 'ip', "opt": option_139789497873280})
    option_139789496594792 = SymLinkOption(**{'name': 'path', "opt": option_139789497873008})
    option_139789496576272 = OptionDescription(children=[option_139789496588496, option_139789496588768, option_139789496589040, option_139789496589312, option_139789496589584, option_139789496594056, option_139789496594792], **{'doc': u'', 'properties': (), 'name': 'container12'})
    option_139789496589856 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496598608 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496598880 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496599152 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496599424 = IntOption(**{'default': 51, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496597032 = SymLinkOption(**{'name': 'ip', "opt": option_139789497874096})
    option_139789496597592 = SymLinkOption(**{'name': 'path', "opt": option_139789497873824})
    option_139789496576592 = OptionDescription(children=[option_139789496589856, option_139789496598608, option_139789496598880, option_139789496599152, option_139789496599424, option_139789496597032, option_139789496597592], **{'doc': u'', 'properties': (), 'name': 'container13'})
    option_139789496599696 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496599968 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496600240 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496600512 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496600784 = IntOption(**{'default': 52, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496604104 = SymLinkOption(**{'name': 'ip', "opt": option_139789497874912})
    option_139789496604664 = SymLinkOption(**{'name': 'path', "opt": option_139789497874640})
    option_139789496576912 = OptionDescription(children=[option_139789496599696, option_139789496599968, option_139789496600240, option_139789496600512, option_139789496600784, option_139789496604104, option_139789496604664], **{'doc': u'', 'properties': (), 'name': 'container14'})
    option_139789496601056 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496601328 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496601600 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496601872 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496602144 = IntOption(**{'default': 50, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496607080 = SymLinkOption(**{'name': 'ip', "opt": option_139789497852256})
    option_139789496607640 = SymLinkOption(**{'name': 'path', "opt": option_139789497851984})
    option_139789496577232 = OptionDescription(children=[option_139789496601056, option_139789496601328, option_139789496601600, option_139789496601872, option_139789496602144, option_139789496607080, option_139789496607640], **{'doc': u'', 'properties': (), 'name': 'container2'})
    option_139789496610896 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496611168 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496611440 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496611712 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496611984 = IntOption(**{'default': 51, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496609880 = SymLinkOption(**{'name': 'ip', "opt": option_139789497853072})
    option_139789496610440 = SymLinkOption(**{'name': 'path', "opt": option_139789497852800})
    option_139789496577552 = OptionDescription(children=[option_139789496610896, option_139789496611168, option_139789496611440, option_139789496611712, option_139789496611984, option_139789496609880, option_139789496610440], **{'doc': u'', 'properties': (), 'name': 'container3'})
    option_139789496612256 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496612528 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496612800 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496613072 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496613344 = IntOption(**{'default': 52, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496616952 = SymLinkOption(**{'name': 'ip', "opt": option_139789497853888})
    option_139789496617512 = SymLinkOption(**{'name': 'path', "opt": option_139789497853616})
    option_139789496619088 = OptionDescription(children=[option_139789496612256, option_139789496612528, option_139789496612800, option_139789496613072, option_139789496613344, option_139789496616952, option_139789496617512], **{'doc': u'', 'properties': (), 'name': 'container4'})
    option_139789496613616 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496613888 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496614160 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496614432 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496627280 = IntOption(**{'default': 52, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496624024 = SymLinkOption(**{'name': 'ip', "opt": option_139789497854704})
    option_139789496624584 = SymLinkOption(**{'name': 'path', "opt": option_139789497854432})
    option_139789496619408 = OptionDescription(children=[option_139789496613616, option_139789496613888, option_139789496614160, option_139789496614432, option_139789496627280, option_139789496624024, option_139789496624584], **{'doc': u'', 'properties': (), 'name': 'container5'})
    option_139789496627552 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496627824 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496628096 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496628368 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496628640 = IntOption(**{'default': 51, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496626824 = SymLinkOption(**{'name': 'ip', "opt": option_139789497855520})
    option_139789496631656 = SymLinkOption(**{'name': 'path', "opt": option_139789497855248})
    option_139789496619728 = OptionDescription(children=[option_139789496627552, option_139789496627824, option_139789496628096, option_139789496628368, option_139789496628640, option_139789496626824, option_139789496631656], **{'doc': u'', 'properties': (), 'name': 'container6'})
    option_139789496628912 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496629184 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496629456 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496629728 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496630000 = IntOption(**{'default': 53, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496633896 = SymLinkOption(**{'name': 'ip', "opt": option_139789497864816})
    option_139789496634456 = SymLinkOption(**{'name': 'path', "opt": option_139789497864544})
    option_139789496620048 = OptionDescription(children=[option_139789496628912, option_139789496629184, option_139789496629456, option_139789496629728, option_139789496630000, option_139789496633896, option_139789496634456], **{'doc': u'', 'properties': (), 'name': 'container7'})
    option_139789496630272 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496630544 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496630816 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496643664 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496643936 = IntOption(**{'default': 50, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496640968 = SymLinkOption(**{'name': 'ip', "opt": option_139789497865632})
    option_139789496641528 = SymLinkOption(**{'name': 'path', "opt": option_139789497865360})
    option_139789496620368 = OptionDescription(children=[option_139789496630272, option_139789496630544, option_139789496630816, option_139789496643664, option_139789496643936, option_139789496640968, option_139789496641528], **{'doc': u'', 'properties': (), 'name': 'container8'})
    option_139789496644208 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496644480 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496644752 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789496645024 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496645296 = IntOption(**{'default': 53, 'doc': u'', 'multi': False, 'properties': (), 'name': 'id'})
    option_139789496648040 = SymLinkOption(**{'name': 'ip', "opt": option_139789497866448})
    option_139789496648600 = SymLinkOption(**{'name': 'path', "opt": option_139789497866176})
    option_139789496620688 = OptionDescription(children=[option_139789496644208, option_139789496644480, option_139789496644752, option_139789496645024, option_139789496645296, option_139789496648040, option_139789496648600], **{'doc': u'', 'properties': (), 'name': 'container9'})
    option_139789496621008 = OptionDescription(children=[option_139789496574992, option_139789496575312, option_139789496575632, option_139789496575952, option_139789496576272, option_139789496576592, option_139789496576912, option_139789496577232, option_139789496577552, option_139789496619088, option_139789496619408, option_139789496619728, option_139789496620048, option_139789496620368, option_139789496620688], **{'doc': u'', 'properties': (), 'name': 'containers'})
    option_139789496649720 = SymLinkOption(**{'name': 'adresse_ip_br0', "opt": option_139789498664080})
    option_139789496650280 = SymLinkOption(**{'name': 'adresse_netmask_br0', "opt": option_139789498664352})
    option_139789496650840 = SymLinkOption(**{'name': 'adresse_network_br0', "opt": option_139789498664624})
    option_139789496651400 = SymLinkOption(**{'name': 'adresse_broadcast_br0', "opt": option_139789498664896})
    option_139789496621328 = OptionDescription(children=[option_139789496649720, option_139789496650280, option_139789496650840, option_139789496651400], **{'doc': u'', 'properties': (), 'name': 'network'})
    option_139789496645568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496645840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496646112 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496646384 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'network', 'doc': u'', 'properties': ()})
    option_139789496646656 = UnicodeOption(**{'default': u'networking', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496646928 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496621648 = OptionDescription(children=[option_139789496645568, option_139789496645840, option_139789496646112, option_139789496646384, option_139789496646656, option_139789496646928], **{'doc': u'', 'properties': (), 'name': 'service0'})
    option_139789496647200 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496655952 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496656224 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496656496 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496656768 = UnicodeOption(**{'default': u'cron', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496657040 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496621968 = OptionDescription(children=[option_139789496647200, option_139789496655952, option_139789496656224, option_139789496656496, option_139789496656768, option_139789496657040], **{'doc': u'', 'properties': (), 'name': 'service1'})
    option_139789496657312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496657584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496657856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496658128 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496658400 = UnicodeOption(**{'default': u'rng-tools', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496658672 = UnicodeOption(**{'default': u'rngd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496658944 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497985232}]})
    option_139789496622288 = OptionDescription(children=[option_139789496657312, option_139789496657584, option_139789496657856, option_139789496658128, option_139789496658400, option_139789496658672, option_139789496658944], **{'doc': u'', 'properties': (), 'name': 'service2'})
    option_139789496659216 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496659488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496664144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496664416 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496664688 = UnicodeOption(**{'default': u'ntp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496664960 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496622608 = OptionDescription(children=[option_139789496659216, option_139789496659488, option_139789496664144, option_139789496664416, option_139789496664688, option_139789496664960], **{'doc': u'', 'properties': (), 'name': 'service3'})
    option_139789496665232 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496665504 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496665776 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496666048 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496666320 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496666592 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496666864 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497955744}]})
    option_139789496676432 = OptionDescription(children=[option_139789496665232, option_139789496665504, option_139789496665776, option_139789496666048, option_139789496666320, option_139789496666592, option_139789496666864], **{'doc': u'', 'properties': (), 'name': 'service4'})
    option_139789496667136 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496667408 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496667680 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496680528 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496680800 = UnicodeOption(**{'default': u'clamav-freshclam', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496681072 = UnicodeOption(**{'default': u'clamav', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496681344 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}]})
    option_139789496676752 = OptionDescription(children=[option_139789496667136, option_139789496667408, option_139789496667680, option_139789496680528, option_139789496680800, option_139789496681072, option_139789496681344], **{'doc': u'', 'properties': (), 'name': 'service5'})
    option_139789496681616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496681888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496682160 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496682432 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496682704 = UnicodeOption(**{'default': u'clamav-daemon', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496682976 = UnicodeOption(**{'default': u'force_clamav', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496683248 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497483344}]})
    option_139789496677072 = OptionDescription(children=[option_139789496681616, option_139789496681888, option_139789496682160, option_139789496682432, option_139789496682704, option_139789496682976, option_139789496683248], **{'doc': u'', 'properties': (), 'name': 'service6'})
    option_139789496683520 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496683792 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496684064 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496692816 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496693088 = UnicodeOption(**{'default': u'clamav-daemon', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496693360 = UnicodeOption(**{'default': u'clamav_mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496693632 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497476512}]})
    option_139789496677392 = OptionDescription(children=[option_139789496683520, option_139789496683792, option_139789496684064, option_139789496692816, option_139789496693088, option_139789496693360, option_139789496693632], **{'doc': u'', 'properties': (), 'name': 'service7'})
    option_139789496693904 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496694176 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496694448 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496694720 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496694992 = UnicodeOption(**{'default': u'clamav-daemon', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496695264 = UnicodeOption(**{'default': u'antivirus_samba', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496695536 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497483616}]})
    option_139789496677712 = OptionDescription(children=[option_139789496693904, option_139789496694176, option_139789496694448, option_139789496694720, option_139789496694992, option_139789496695264, option_139789496695536], **{'doc': u'', 'properties': (), 'name': 'service8'})
    option_139789496695808 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496696080 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496696352 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496705104 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496705376 = UnicodeOption(**{'default': u'clamav-daemon', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496705648 = UnicodeOption(**{'default': u'clamav_proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496705920 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497484160}]})
    option_139789496678032 = OptionDescription(children=[option_139789496695808, option_139789496696080, option_139789496696352, option_139789496705104, option_139789496705376, option_139789496705648, option_139789496705920], **{'doc': u'', 'properties': (), 'name': 'service9'})
    option_139789496706192 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496706464 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496706736 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496707008 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496707280 = UnicodeOption(**{'default': u'clamav-daemon', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496707552 = UnicodeOption(**{'default': u'antivirus_ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496707824 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497484432}]})
    option_139789496678352 = OptionDescription(children=[option_139789496706192, option_139789496706464, option_139789496706736, option_139789496707008, option_139789496707280, option_139789496707552, option_139789496707824], **{'doc': u'', 'properties': (), 'name': 'service10'})
    option_139789496708096 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496708368 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496708640 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496713296 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496713568 = UnicodeOption(**{'default': u'isc-dhcp-server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496713840 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496714112 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}]})
    option_139789496678672 = OptionDescription(children=[option_139789496708096, option_139789496708368, option_139789496708640, option_139789496713296, option_139789496713568, option_139789496713840, option_139789496714112], **{'doc': u'', 'properties': (), 'name': 'service11'})
    option_139789496714384 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496714656 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496714928 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496715200 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496715472 = UnicodeOption(**{'default': u'tftpd-hpa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496715744 = UnicodeOption(**{'default': u'tftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496716016 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956288}]})
    option_139789496678992 = OptionDescription(children=[option_139789496714384, option_139789496714656, option_139789496714928, option_139789496715200, option_139789496715472, option_139789496715744, option_139789496716016], **{'doc': u'', 'properties': (), 'name': 'service12'})
    option_139789496716288 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496716560 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496716832 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496721488 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496721760 = UnicodeOption(**{'default': u'isc-dhcp-relay', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496722032 = UnicodeOption(**{'default': u'dhrelay', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496722304 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956560}]})
    option_139789496679312 = OptionDescription(children=[option_139789496716288, option_139789496716560, option_139789496716832, option_139789496721488, option_139789496721760, option_139789496722032, option_139789496722304], **{'doc': u'', 'properties': (), 'name': 'service13'})
    option_139789496722576 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496722848 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496723120 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496723392 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496723664 = UnicodeOption(**{'default': u'winbind', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496723936 = UnicodeOption(**{'default': u'winbind', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496724208 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497048064}]})
    option_139789496679632 = OptionDescription(children=[option_139789496722576, option_139789496722848, option_139789496723120, option_139789496723392, option_139789496723664, option_139789496723936, option_139789496724208], **{'doc': u'', 'properties': (), 'name': 'service14'})
    option_139789496724480 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496724752 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496725024 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496733776 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496734048 = UnicodeOption(**{'default': u'winbind', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496734320 = UnicodeOption(**{'default': u'ntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496734592 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}, {'expected': u'NTLM/SMB', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496679952 = OptionDescription(children=[option_139789496724480, option_139789496724752, option_139789496725024, option_139789496733776, option_139789496734048, option_139789496734320, option_139789496734592], **{'doc': u'', 'properties': (), 'name': 'service15'})
    option_139789496734864 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496735136 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496735408 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496735680 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496735952 = UnicodeOption(**{'default': u'smbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496736224 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496217680 = OptionDescription(children=[option_139789496734864, option_139789496735136, option_139789496735408, option_139789496735680, option_139789496735952, option_139789496736224], **{'doc': u'', 'properties': (), 'name': 'service16'})
    option_139789496736496 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496736768 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496737040 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496737312 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496221776 = UnicodeOption(**{'default': u'smbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496222048 = UnicodeOption(**{'default': u'ntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496222320 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}, {'expected': u'NTLM/SMB', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496218000 = OptionDescription(children=[option_139789496736496, option_139789496736768, option_139789496737040, option_139789496737312, option_139789496221776, option_139789496222048, option_139789496222320], **{'doc': u'', 'properties': (), 'name': 'service17'})
    option_139789496222592 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496222864 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496223136 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496223408 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496223680 = UnicodeOption(**{'default': u'nmbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496223952 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496218320 = OptionDescription(children=[option_139789496222592, option_139789496222864, option_139789496223136, option_139789496223408, option_139789496223680, option_139789496223952], **{'doc': u'', 'properties': (), 'name': 'service18'})
    option_139789496224224 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496224496 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496224768 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496225040 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496225312 = UnicodeOption(**{'default': u'nmbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496229968 = UnicodeOption(**{'default': u'ntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496230240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}, {'expected': u'NTLM/SMB', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789496218640 = OptionDescription(children=[option_139789496224224, option_139789496224496, option_139789496224768, option_139789496225040, option_139789496225312, option_139789496229968, option_139789496230240], **{'doc': u'', 'properties': (), 'name': 'service19'})
    option_139789496230512 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496230784 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496231056 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496231328 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496231600 = UnicodeOption(**{'default': u'nscd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496231872 = UnicodeOption(**{'default': u'nscd_service', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496232144 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496218960 = OptionDescription(children=[option_139789496230512, option_139789496230784, option_139789496231056, option_139789496231328, option_139789496231600, option_139789496231872, option_139789496232144], **{'doc': u'', 'properties': (), 'name': 'service20'})
    option_139789496232416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496232688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496232960 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496233232 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496233504 = UnicodeOption(**{'default': u'nut-server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496246352 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496246624 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957104}]})
    option_139789496219280 = OptionDescription(children=[option_139789496232416, option_139789496232688, option_139789496232960, option_139789496233232, option_139789496233504, option_139789496246352, option_139789496246624], **{'doc': u'', 'properties': (), 'name': 'service21'})
    option_139789496246896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496247168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496247440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496247712 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496247984 = UnicodeOption(**{'default': u'salt-api', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496248256 = UnicodeOption(**{'default': u'saltstack', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496248528 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963664}]})
    option_139789496219600 = OptionDescription(children=[option_139789496246896, option_139789496247168, option_139789496247440, option_139789496247712, option_139789496247984, option_139789496248256, option_139789496248528], **{'doc': u'', 'properties': (), 'name': 'service22'})
    option_139789496248800 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496249072 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496249344 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496249616 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496249888 = UnicodeOption(**{'default': u'salt-master', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496258640 = UnicodeOption(**{'default': u'saltstack', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496258912 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963664}]})
    option_139789496219920 = OptionDescription(children=[option_139789496248800, option_139789496249072, option_139789496249344, option_139789496249616, option_139789496249888, option_139789496258640, option_139789496258912], **{'doc': u'', 'properties': (), 'name': 'service23'})
    option_139789496259184 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496259456 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496259728 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496260000 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496260272 = UnicodeOption(**{'default': u'salt-minion', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496260544 = UnicodeOption(**{'default': u'saltstack', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496260816 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963664}]})
    option_139789496220240 = OptionDescription(children=[option_139789496259184, option_139789496259456, option_139789496259728, option_139789496260000, option_139789496260272, option_139789496260544, option_139789496260816], **{'doc': u'', 'properties': (), 'name': 'service24'})
    option_139789496261088 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496261360 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496261632 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496261904 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496262176 = UnicodeOption(**{'default': u'apache2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496266832 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496267104 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789496220560 = OptionDescription(children=[option_139789496261088, option_139789496261360, option_139789496261632, option_139789496261904, option_139789496262176, option_139789496266832, option_139789496267104], **{'doc': u'', 'properties': (), 'name': 'service25'})
    option_139789496267376 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496267648 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496267920 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496268192 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496268464 = UnicodeOption(**{'default': u'apache2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496268736 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496269008 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789496220880 = OptionDescription(children=[option_139789496267376, option_139789496267648, option_139789496267920, option_139789496268192, option_139789496268464, option_139789496268736, option_139789496269008], **{'doc': u'', 'properties': (), 'name': 'service26'})
    option_139789496269280 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496269552 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496269824 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496270096 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496270368 = UnicodeOption(**{'default': u'default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496275024 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496275296 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789496221200 = OptionDescription(children=[option_139789496269280, option_139789496269552, option_139789496269824, option_139789496270096, option_139789496270368, option_139789496275024, option_139789496275296], **{'doc': u'', 'properties': (), 'name': 'service27'})
    option_139789496275568 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496275840 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496276112 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496276384 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496276656 = UnicodeOption(**{'default': u'Vhost', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496276928 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496277200 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789496283216 = OptionDescription(children=[option_139789496275568, option_139789496275840, option_139789496276112, option_139789496276384, option_139789496276656, option_139789496276928, option_139789496277200], **{'doc': u'', 'properties': (), 'name': 'service28'})
    option_139789496277472 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496277744 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496278016 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496278288 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496278560 = UnicodeOption(**{'default': u'Vhost-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496287312 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496287584 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789496283536 = OptionDescription(children=[option_139789496277472, option_139789496277744, option_139789496278016, option_139789496278288, option_139789496278560, option_139789496287312, option_139789496287584], **{'doc': u'', 'properties': (), 'name': 'service29'})
    option_139789496287856 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496288128 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496288400 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496288672 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496288944 = UnicodeOption(**{'default': u'eole', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496289216 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496289488 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789496283856 = OptionDescription(children=[option_139789496287856, option_139789496288128, option_139789496288400, option_139789496288672, option_139789496288944, option_139789496289216, option_139789496289488], **{'doc': u'', 'properties': (), 'name': 'service30'})
    option_139789496289760 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496290032 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496290304 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496290576 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496290848 = UnicodeOption(**{'default': u'bareos-webui', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496295504 = UnicodeOption(**{'default': u'bareoswebui', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496295776 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145280}]})
    option_139789496284176 = OptionDescription(children=[option_139789496289760, option_139789496290032, option_139789496290304, option_139789496290576, option_139789496290848, option_139789496295504, option_139789496295776], **{'doc': u'', 'properties': (), 'name': 'service31'})
    option_139789496296048 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496296320 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496296592 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496296864 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496297136 = UnicodeOption(**{'default': u'eole-sso', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496297408 = UnicodeOption(**{'default': u'sso', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496297680 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789496284496 = OptionDescription(children=[option_139789496296048, option_139789496296320, option_139789496296592, option_139789496296864, option_139789496297136, option_139789496297408, option_139789496297680], **{'doc': u'', 'properties': (), 'name': 'service32'})
    option_139789496297952 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496298224 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496298496 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496298768 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496299040 = UnicodeOption(**{'default': u'bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496303696 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496284816 = OptionDescription(children=[option_139789496297952, option_139789496298224, option_139789496298496, option_139789496298768, option_139789496299040, option_139789496303696], **{'doc': u'', 'properties': (), 'name': 'service33'})
    option_139789496303968 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496304240 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496304512 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496304784 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496305056 = UnicodeOption(**{'default': u'ead-server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496305328 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496285136 = OptionDescription(children=[option_139789496303968, option_139789496304240, option_139789496304512, option_139789496304784, option_139789496305056, option_139789496305328], **{'doc': u'', 'properties': (), 'name': 'service34'})
    option_139789496305600 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496305872 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496306144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496306416 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496306688 = UnicodeOption(**{'default': u'ead-web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496306960 = UnicodeOption(**{'default': u'ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496307232 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964480}]})
    option_139789496285456 = OptionDescription(children=[option_139789496305600, option_139789496305872, option_139789496306144, option_139789496306416, option_139789496306688, option_139789496306960, option_139789496307232], **{'doc': u'', 'properties': (), 'name': 'service35'})
    option_139789496320080 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496320352 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496320624 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496320896 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496321168 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496321440 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496321712 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971312}]})
    option_139789496285776 = OptionDescription(children=[option_139789496320080, option_139789496320352, option_139789496320624, option_139789496320896, option_139789496321168, option_139789496321440, option_139789496321712], **{'doc': u'', 'properties': (), 'name': 'service36'})
    option_139789496321984 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496322256 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496322528 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496322800 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496323072 = UnicodeOption(**{'default': u'spamassassin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496323344 = UnicodeOption(**{'default': u'spamassassin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496323616 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496746064}]})
    option_139789496286096 = OptionDescription(children=[option_139789496321984, option_139789496322256, option_139789496322528, option_139789496322800, option_139789496323072, option_139789496323344, option_139789496323616], **{'doc': u'', 'properties': (), 'name': 'service37'})
    option_139789496332368 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496332640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496332912 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496333184 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496333456 = UnicodeOption(**{'default': u'bareos-dir', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496333728 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496334000 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789496286416 = OptionDescription(children=[option_139789496332368, option_139789496332640, option_139789496332912, option_139789496333184, option_139789496333456, option_139789496333728, option_139789496334000], **{'doc': u'', 'properties': (), 'name': 'service38'})
    option_139789496334272 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496334544 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496334816 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496335088 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496335360 = UnicodeOption(**{'default': u'bareos-fd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496335632 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496335904 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789496286736 = OptionDescription(children=[option_139789496334272, option_139789496334544, option_139789496334816, option_139789496335088, option_139789496335360, option_139789496335632, option_139789496335904], **{'doc': u'', 'properties': (), 'name': 'service39'})
    option_139789496344656 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496344928 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496345200 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496345472 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496345744 = UnicodeOption(**{'default': u'bareos-sd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496346016 = UnicodeOption(**{'default': u'bareos_sd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496346288 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971040}]})
    option_139789496348752 = OptionDescription(children=[option_139789496344656, option_139789496344928, option_139789496345200, option_139789496345472, option_139789496345744, option_139789496346016, option_139789496346288], **{'doc': u'', 'properties': (), 'name': 'service40'})
    option_139789496346560 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496346832 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496347104 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496347376 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496347648 = UnicodeOption(**{'default': u'courier-imap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496347920 = UnicodeOption(**{'default': u'courier_imap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496348192 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496747152}]})
    option_139789496349072 = OptionDescription(children=[option_139789496346560, option_139789496346832, option_139789496347104, option_139789496347376, option_139789496347648, option_139789496347920, option_139789496348192], **{'doc': u'', 'properties': (), 'name': 'service41'})
    option_139789496356944 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496357216 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496357488 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496357760 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496358032 = UnicodeOption(**{'default': u'courier-imap-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496358304 = UnicodeOption(**{'default': u'courier_imap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496358576 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496747152}]})
    option_139789496349392 = OptionDescription(children=[option_139789496356944, option_139789496357216, option_139789496357488, option_139789496357760, option_139789496358032, option_139789496358304, option_139789496358576], **{'doc': u'', 'properties': (), 'name': 'service42'})
    option_139789496358848 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496359120 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496359392 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496359664 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496359936 = UnicodeOption(**{'default': u'courier-pop', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496360208 = UnicodeOption(**{'default': u'courier_pop', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496360480 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496746880}]})
    option_139789496349712 = OptionDescription(children=[option_139789496358848, option_139789496359120, option_139789496359392, option_139789496359664, option_139789496359936, option_139789496360208, option_139789496360480], **{'doc': u'', 'properties': (), 'name': 'service43'})
    option_139789496365136 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496365408 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496365680 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496365952 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496366224 = UnicodeOption(**{'default': u'courier-pop-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496366496 = UnicodeOption(**{'default': u'courier_pop', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496366768 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496746880}]})
    option_139789496350032 = OptionDescription(children=[option_139789496365136, option_139789496365408, option_139789496365680, option_139789496365952, option_139789496366224, option_139789496366496, option_139789496366768], **{'doc': u'', 'properties': (), 'name': 'service44'})
    option_139789496367040 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496367312 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496367584 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496367856 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496368128 = UnicodeOption(**{'default': u'courier-authdaemon', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496368400 = UnicodeOption(**{'default': u'courier_authdaemon', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496368672 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496747424}]})
    option_139789496350352 = OptionDescription(children=[option_139789496367040, option_139789496367312, option_139789496367584, option_139789496367856, option_139789496368128, option_139789496368400, option_139789496368672], **{'doc': u'', 'properties': (), 'name': 'service45'})
    option_139789496377424 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496377696 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496377968 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496378240 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496378512 = UnicodeOption(**{'default': u'phpmyadmin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496378784 = UnicodeOption(**{'default': u'myadmin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496379056 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145552}]})
    option_139789496350672 = OptionDescription(children=[option_139789496377424, option_139789496377696, option_139789496377968, option_139789496378240, option_139789496378512, option_139789496378784, option_139789496379056], **{'doc': u'', 'properties': (), 'name': 'service46'})
    option_139789496379328 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496379600 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496379872 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496380144 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496380416 = UnicodeOption(**{'default': u'squid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496380688 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496350992 = OptionDescription(children=[option_139789496379328, option_139789496379600, option_139789496379872, option_139789496380144, option_139789496380416, option_139789496380688], **{'doc': u'', 'properties': (), 'name': 'service47'})
    option_139789496380960 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496389712 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496389984 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496390256 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496390528 = UnicodeOption(**{'default': u'squid3-2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496390800 = UnicodeOption(**{'default': u'squid2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496391072 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}]})
    option_139789496351312 = OptionDescription(children=[option_139789496380960, option_139789496389712, option_139789496389984, option_139789496390256, option_139789496390528, option_139789496390800, option_139789496391072], **{'doc': u'', 'properties': (), 'name': 'service48'})
    option_139789496391344 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496391616 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496391888 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496392160 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496392432 = UnicodeOption(**{'default': u'eole-guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496392704 = UnicodeOption(**{'default': u'guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496392976 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789496351632 = OptionDescription(children=[option_139789496391344, option_139789496391616, option_139789496391888, option_139789496392160, option_139789496392432, option_139789496392704, option_139789496392976], **{'doc': u'', 'properties': (), 'name': 'service49'})
    option_139789496393248 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496397904 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496398176 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496398448 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496398720 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496398992 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496399264 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789496351952 = OptionDescription(children=[option_139789496393248, option_139789496397904, option_139789496398176, option_139789496398448, option_139789496398720, option_139789496398992, option_139789496399264], **{'doc': u'', 'properties': (), 'name': 'service50'})
    option_139789496399536 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496399808 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496400080 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496400352 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496400624 = UnicodeOption(**{'default': u'cups-browsed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496400896 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496401168 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789496352272 = OptionDescription(children=[option_139789496399536, option_139789496399808, option_139789496400080, option_139789496400352, option_139789496400624, option_139789496400896, option_139789496401168], **{'doc': u'', 'properties': (), 'name': 'service51'})
    option_139789496401440 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496406096 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496406368 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496406640 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496406912 = UnicodeOption(**{'default': u'proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496407184 = UnicodeOption(**{'default': u'proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496407456 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}]})
    option_139789496410192 = OptionDescription(children=[option_139789496401440, option_139789496406096, option_139789496406368, option_139789496406640, option_139789496406912, option_139789496407184, option_139789496407456], **{'doc': u'', 'properties': (), 'name': 'service52'})
    option_139789496407728 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496408000 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496408272 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496408544 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496408816 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496409088 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496409360 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789496410512 = OptionDescription(children=[option_139789496407728, option_139789496408000, option_139789496408272, option_139789496408544, option_139789496408816, option_139789496409088, option_139789496409360], **{'doc': u'', 'properties': (), 'name': 'service53'})
    option_139789496409632 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496418384 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496418656 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496418928 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496419200 = UnicodeOption(**{'default': u'wwsympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496419472 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496419744 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789496410832 = OptionDescription(children=[option_139789496409632, option_139789496418384, option_139789496418656, option_139789496418928, option_139789496419200, option_139789496419472, option_139789496419744], **{'doc': u'', 'properties': (), 'name': 'service54'})
    option_139789496420016 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496420288 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496420560 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496420832 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496421104 = UnicodeOption(**{'default': u'controle-vnc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496421376 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496411152 = OptionDescription(children=[option_139789496420016, option_139789496420288, option_139789496420560, option_139789496420832, option_139789496421104, option_139789496421376], **{'doc': u'', 'properties': (), 'name': 'service55'})
    option_139789496421648 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496421920 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496430672 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496430944 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496431216 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496431488 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496431760 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789496411472 = OptionDescription(children=[option_139789496421648, option_139789496421920, option_139789496430672, option_139789496430944, option_139789496431216, option_139789496431488, option_139789496431760], **{'doc': u'', 'properties': (), 'name': 'service56'})
    option_139789496432032 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496432304 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496432576 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496432848 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496433120 = UnicodeOption(**{'default': u'exim4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496433392 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496411792 = OptionDescription(children=[option_139789496432032, option_139789496432304, option_139789496432576, option_139789496432848, option_139789496433120, option_139789496433392], **{'doc': u'', 'properties': (), 'name': 'service57'})
    option_139789496433664 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496433936 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496434208 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496438864 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496439136 = UnicodeOption(**{'default': u'eoleflask', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496439408 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496412112 = OptionDescription(children=[option_139789496433664, option_139789496433936, option_139789496434208, option_139789496438864, option_139789496439136, option_139789496439408], **{'doc': u'', 'properties': (), 'name': 'service58'})
    option_139789496439680 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496439952 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496440224 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496440496 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496440768 = UnicodeOption(**{'default': u'nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496441040 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496412432 = OptionDescription(children=[option_139789496439680, option_139789496439952, option_139789496440224, option_139789496440496, option_139789496440768, option_139789496441040], **{'doc': u'', 'properties': (), 'name': 'service59'})
    option_139789496441312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496441584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496441856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496442128 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496442400 = UnicodeOption(**{'default': u'ead3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496451152 = UnicodeOption(**{'default': u'ead3_apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496451424 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963392}, {'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497957648}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789496412752 = OptionDescription(children=[option_139789496441312, option_139789496441584, option_139789496441856, option_139789496442128, option_139789496442400, option_139789496451152, option_139789496451424], **{'doc': u'', 'properties': (), 'name': 'service60'})
    option_139789496451696 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496451968 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496452240 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496452512 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496452784 = UnicodeOption(**{'default': u'eoleapps', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496453056 = UnicodeOption(**{'default': u'eoleapps', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496453328 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145824}]})
    option_139789496413072 = OptionDescription(children=[option_139789496451696, option_139789496451968, option_139789496452240, option_139789496452512, option_139789496452784, option_139789496453056, option_139789496453328], **{'doc': u'', 'properties': (), 'name': 'service61'})
    option_139789496453600 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496453872 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496454144 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496454416 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496454688 = UnicodeOption(**{'default': u'eoleapps-apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496463440 = UnicodeOption(**{'default': u'eoleapps', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496463712 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145824}]})
    option_139789496413392 = OptionDescription(children=[option_139789496453600, option_139789496453872, option_139789496454144, option_139789496454416, option_139789496454688, option_139789496463440, option_139789496463712], **{'doc': u'', 'properties': (), 'name': 'service62'})
    option_139789496463984 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496464256 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496464528 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496464800 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789496465072 = UnicodeOption(**{'default': u'genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496465344 = UnicodeOption(**{'default': u'genconfig_apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789496465616 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497974032}, {'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497957920}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789496413712 = OptionDescription(children=[option_139789496463984, option_139789496464256, option_139789496464528, option_139789496464800, option_139789496465072, option_139789496465344, option_139789496465616], **{'doc': u'', 'properties': (), 'name': 'service63'})
    option_139789496465888 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496466160 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496466432 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496466704 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789496466976 = UnicodeOption(**{'default': u'websockify', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495955536 = UnicodeOption(**{'default': u'websockify', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789495955808 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497146368}]})
    option_139789495959632 = OptionDescription(children=[option_139789496465888, option_139789496466160, option_139789496466432, option_139789496466704, option_139789496466976, option_139789495955536, option_139789495955808], **{'doc': u'', 'properties': (), 'name': 'service64'})
    option_139789495956080 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495956352 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495956624 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495956896 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789495957168 = UnicodeOption(**{'default': u'roundcube', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495957440 = UnicodeOption(**{'default': u'roundcube', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789495957712 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497143376}]})
    option_139789495959952 = OptionDescription(children=[option_139789495956080, option_139789495956352, option_139789495956624, option_139789495956896, option_139789495957168, option_139789495957440, option_139789495957712], **{'doc': u'', 'properties': (), 'name': 'service65'})
    option_139789495957984 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495958256 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495958528 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495958800 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789495959072 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495967824 = UnicodeOption(**{'default': u'cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789495968096 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497574816}]})
    option_139789495960272 = OptionDescription(children=[option_139789495957984, option_139789495958256, option_139789495958528, option_139789495958800, option_139789495959072, option_139789495967824, option_139789495968096], **{'doc': u'', 'properties': (), 'name': 'service66'})
    option_139789495968368 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495968640 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495968912 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495969184 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789495969456 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495969728 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'servicelist'})
    option_139789495970000 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789495960592 = OptionDescription(children=[option_139789495968368, option_139789495968640, option_139789495968912, option_139789495969184, option_139789495969456, option_139789495969728, option_139789495970000], **{'doc': u'', 'properties': (), 'name': 'service67'})
    option_139789495970272 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495970544 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495970816 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495971088 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789495971360 = UnicodeOption(**{'default': u'bastion', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495976016 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495960912 = OptionDescription(children=[option_139789495970272, option_139789495970544, option_139789495970816, option_139789495971088, option_139789495971360, option_139789495976016], **{'doc': u'', 'properties': (), 'name': 'service68'})
    option_139789495976288 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495976560 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495976832 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495977104 = ChoiceOption(**{'multi': False, 'values': (u'systemd', u'upstart', u'apache', u'network'), 'name': 'method', 'default': 'systemd', 'doc': u'', 'properties': ()})
    option_139789495977376 = UnicodeOption(**{'default': u'z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495977648 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495961232 = OptionDescription(children=[option_139789495976288, option_139789495976560, option_139789495976832, option_139789495977104, option_139789495977376, option_139789495977648], **{'doc': u'', 'properties': (), 'name': 'service69'})
    option_139789495961552 = OptionDescription(children=[option_139789496621648, option_139789496621968, option_139789496622288, option_139789496622608, option_139789496676432, option_139789496676752, option_139789496677072, option_139789496677392, option_139789496677712, option_139789496678032, option_139789496678352, option_139789496678672, option_139789496678992, option_139789496679312, option_139789496679632, option_139789496679952, option_139789496217680, option_139789496218000, option_139789496218320, option_139789496218640, option_139789496218960, option_139789496219280, option_139789496219600, option_139789496219920, option_139789496220240, option_139789496220560, option_139789496220880, option_139789496221200, option_139789496283216, option_139789496283536, option_139789496283856, option_139789496284176, option_139789496284496, option_139789496284816, option_139789496285136, option_139789496285456, option_139789496285776, option_139789496286096, option_139789496286416, option_139789496286736, option_139789496348752, option_139789496349072, option_139789496349392, option_139789496349712, option_139789496350032, option_139789496350352, option_139789496350672, option_139789496350992, option_139789496351312, option_139789496351632, option_139789496351952, option_139789496352272, option_139789496410192, option_139789496410512, option_139789496410832, option_139789496411152, option_139789496411472, option_139789496411792, option_139789496412112, option_139789496412432, option_139789496412752, option_139789496413072, option_139789496413392, option_139789496413712, option_139789495959632, option_139789495959952, option_139789495960272, option_139789495960592, option_139789495960912, option_139789495961232], **{'doc': u'', 'properties': (), 'name': 'services'})
    option_139789495977920 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495978192 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495978464 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495978736 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '123', 'doc': u'', 'properties': ()})
    option_139789495979008 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495979280 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'udp', 'doc': u'', 'properties': ()})
    option_139789495979552 = UnicodeOption(**{'default': u'ntp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495984208 = UnicodeOption(**{'default': u'___auto_ntp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495984480 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495961872 = OptionDescription(children=[option_139789495977920, option_139789495978192, option_139789495978464, option_139789495978736, option_139789495979008, option_139789495979280, option_139789495979552, option_139789495984208, option_139789495984480], **{'doc': u'', 'properties': (), 'name': 'service_access0'})
    option_139789495984752 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495985024 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495985296 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495985568 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '123', 'doc': u'', 'properties': ()})
    option_139789495985840 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495986112 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'udp', 'doc': u'', 'properties': ()})
    option_139789495986384 = UnicodeOption(**{'default': u'ntp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495986656 = UnicodeOption(**{'default': u'___auto_ntp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495986928 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495962192 = OptionDescription(children=[option_139789495984752, option_139789495985024, option_139789495985296, option_139789495985568, option_139789495985840, option_139789495986112, option_139789495986384, option_139789495986656, option_139789495986928], **{'doc': u'', 'properties': (), 'name': 'service_access1'})
    option_139789495987200 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495987472 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495987744 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495992400 = UnicodeOption(**{'default': u'ntpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495992672 = UnicodeOption(**{'default': u'tcpwrapper', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495992944 = UnicodeOption(**{'default': u'ntp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495993216 = UnicodeOption(**{'default': u'___auto_ntp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495993488 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495962512 = OptionDescription(children=[option_139789495987200, option_139789495987472, option_139789495987744, option_139789495992400, option_139789495992672, option_139789495992944, option_139789495993216, option_139789495993488], **{'doc': u'', 'properties': (), 'name': 'service_access2'})
    option_139789495993760 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495994032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495994304 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495994576 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '514', 'doc': u'', 'properties': ()})
    option_139789495994848 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495995120 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'udp', 'doc': u'', 'properties': ()})
    option_139789495995392 = UnicodeOption(**{'default': u'rsyslog_udp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495995664 = UnicodeOption(**{'default': u'___auto_rsyslog_udp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495995936 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495962832 = OptionDescription(children=[option_139789495993760, option_139789495994032, option_139789495994304, option_139789495994576, option_139789495994848, option_139789495995120, option_139789495995392, option_139789495995664, option_139789495995936], **{'doc': u'', 'properties': (), 'name': 'service_access3'})
    option_139789496004688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496004960 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496005232 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496005504 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '10514', 'doc': u'', 'properties': ()})
    option_139789496005776 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496006048 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496006320 = UnicodeOption(**{'default': u'rsyslog_tcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496006592 = UnicodeOption(**{'default': u'___auto_rsyslog_tcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496006864 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495963152 = OptionDescription(children=[option_139789496004688, option_139789496004960, option_139789496005232, option_139789496005504, option_139789496005776, option_139789496006048, option_139789496006320, option_139789496006592, option_139789496006864], **{'doc': u'', 'properties': (), 'name': 'service_access4'})
    option_139789496007136 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496007408 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496007680 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496007952 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '20514', 'doc': u'', 'properties': ()})
    option_139789496008224 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496012880 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496013152 = UnicodeOption(**{'default': u'rsyslog_relp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496013424 = UnicodeOption(**{'default': u'___auto_rsyslog_relp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496013696 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496016976 = OptionDescription(children=[option_139789496007136, option_139789496007408, option_139789496007680, option_139789496007952, option_139789496008224, option_139789496012880, option_139789496013152, option_139789496013424, option_139789496013696], **{'doc': u'', 'properties': (), 'name': 'service_access5'})
    option_139789496013968 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496014240 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496014512 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496014784 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '22', 'doc': u'', 'properties': ()})
    option_139789496015056 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496015328 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496015600 = UnicodeOption(**{'default': u'sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496015872 = UnicodeOption(**{'default': u'___auto_sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496016144 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496017296 = OptionDescription(children=[option_139789496013968, option_139789496014240, option_139789496014512, option_139789496014784, option_139789496015056, option_139789496015328, option_139789496015600, option_139789496015872, option_139789496016144], **{'doc': u'', 'properties': (), 'name': 'service_access6'})
    option_139789496016416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496025168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496025440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496025712 = UnicodeOption(**{'default': u'sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496025984 = UnicodeOption(**{'default': u'tcpwrapper', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496026256 = UnicodeOption(**{'default': u'sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496026528 = UnicodeOption(**{'default': u'___auto_sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496026800 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496017616 = OptionDescription(children=[option_139789496016416, option_139789496025168, option_139789496025440, option_139789496025712, option_139789496025984, option_139789496026256, option_139789496026528, option_139789496026800], **{'doc': u'', 'properties': (), 'name': 'service_access7'})
    option_139789496027072 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496027344 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496027616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496027888 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '7000', 'doc': u'', 'properties': ()})
    option_139789496028160 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496028432 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496028704 = UnicodeOption(**{'default': u'genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496033360 = UnicodeOption(**{'default': u'___auto_genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496033632 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496017936 = OptionDescription(children=[option_139789496027072, option_139789496027344, option_139789496027616, option_139789496027888, option_139789496028160, option_139789496028432, option_139789496028704, option_139789496033360, option_139789496033632], **{'doc': u'', 'properties': (), 'name': 'service_access8'})
    option_139789496033904 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496034176 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496034448 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496034720 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '389', 'doc': u'', 'properties': ()})
    option_139789496034992 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496035264 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496035536 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496035808 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496036080 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'tous', 'action': 'disabled', 'inverse': True, "option": option_139789496783744}, {'expected': u'authentifi\xe9', 'action': 'disabled', 'inverse': True, "option": option_139789496783744}, {'expected': u'non', 'action': 'disabled', 'inverse': True, "option": option_139789496783472}, {'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496783472}, {'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497955744}]})
    option_139789496018256 = OptionDescription(children=[option_139789496033904, option_139789496034176, option_139789496034448, option_139789496034720, option_139789496034992, option_139789496035264, option_139789496035536, option_139789496035808, option_139789496036080], **{'doc': u'', 'properties': (), 'name': 'service_access9'})
    option_139789496036352 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496036624 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496036896 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496041552 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '636', 'doc': u'', 'properties': ()})
    option_139789496041824 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496042096 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496042368 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496042640 = UnicodeOption(**{'default': u'slapd_ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496042912 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'tous', 'action': 'disabled', 'inverse': True, "option": option_139789496783744}, {'expected': u'authentifi\xe9', 'action': 'disabled', 'inverse': True, "option": option_139789496783744}, {'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496783472}, {'expected': u'uniquement', 'action': 'disabled', 'inverse': True, "option": option_139789496783472}, {'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497955744}]})
    option_139789496018576 = OptionDescription(children=[option_139789496036352, option_139789496036624, option_139789496036896, option_139789496041552, option_139789496041824, option_139789496042096, option_139789496042368, option_139789496042640, option_139789496042912], **{'doc': u'', 'properties': (), 'name': 'service_access10'})
    option_139789496043184 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496043456 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496043728 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496044000 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496044272 = UnicodeOption(**{'default': u'tcpwrapper', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496044544 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496044816 = UnicodeOption(**{'default': u'tcpw_slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496045088 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'tous', 'action': 'disabled', 'inverse': True, "option": option_139789496783744}, {'expected': u'authentifi\xe9', 'action': 'disabled', 'inverse': True, "option": option_139789496783744}, {'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497955744}]})
    option_139789496018896 = OptionDescription(children=[option_139789496043184, option_139789496043456, option_139789496043728, option_139789496044000, option_139789496044272, option_139789496044544, option_139789496044816, option_139789496045088], **{'doc': u'', 'properties': (), 'name': 'service_access11'})
    option_139789496053840 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496054112 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496054384 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496051424 = SymLinkOption(**{'name': 'name', "opt": option_139789497519376})
    option_139789496054656 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496054928 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496055200 = UnicodeOption(**{'default': u'failover_dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496055472 = UnicodeOption(**{'default': u'failover_dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496055744 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789496019216 = OptionDescription(children=[option_139789496053840, option_139789496054112, option_139789496054384, option_139789496051424, option_139789496054656, option_139789496054928, option_139789496055200, option_139789496055472, option_139789496055744], **{'doc': u'', 'properties': (), 'name': 'service_access12'})
    option_139789496056016 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496056288 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496056560 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496056832 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '69', 'doc': u'', 'properties': ()})
    option_139789496057104 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496057376 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'udp', 'doc': u'', 'properties': ()})
    option_139789496066128 = UnicodeOption(**{'default': u'tftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496066400 = UnicodeOption(**{'default': u'___auto_tftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496066672 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496019536 = OptionDescription(children=[option_139789496056016, option_139789496056288, option_139789496056560, option_139789496056832, option_139789496057104, option_139789496057376, option_139789496066128, option_139789496066400, option_139789496066672], **{'doc': u'', 'properties': (), 'name': 'service_access13'})
    option_139789496066944 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496067216 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496067488 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496067760 = UnicodeOption(**{'default': u'in.tftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496068032 = UnicodeOption(**{'default': u'tcpwrapper', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496068304 = UnicodeOption(**{'default': u'tftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496068576 = UnicodeOption(**{'default': u'___auto_tftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496068848 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496019856 = OptionDescription(children=[option_139789496066944, option_139789496067216, option_139789496067488, option_139789496067760, option_139789496068032, option_139789496068304, option_139789496068576, option_139789496068848], **{'doc': u'', 'properties': (), 'name': 'service_access14'})
    option_139789496069120 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496069392 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496069664 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496072744 = SymLinkOption(**{'name': 'name', "opt": option_139789497010112})
    option_139789496074320 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496074592 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496074864 = UnicodeOption(**{'default': u'omap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496075136 = UnicodeOption(**{'default': u'omapi', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496075408 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497009840}]})
    option_139789496020176 = OptionDescription(children=[option_139789496069120, option_139789496069392, option_139789496069664, option_139789496072744, option_139789496074320, option_139789496074592, option_139789496074864, option_139789496075136, option_139789496075408], **{'doc': u'', 'properties': (), 'name': 'service_access15'})
    option_139789496075680 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496075952 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496076224 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496076496 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '139', 'doc': u'', 'properties': ()})
    option_139789496076768 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496077040 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496077312 = UnicodeOption(**{'default': u'smbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496077584 = UnicodeOption(**{'default': u'___auto_smbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496077856 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496020496 = OptionDescription(children=[option_139789496075680, option_139789496075952, option_139789496076224, option_139789496076496, option_139789496076768, option_139789496077040, option_139789496077312, option_139789496077584, option_139789496077856], **{'doc': u'', 'properties': (), 'name': 'service_access16'})
    option_139789496086608 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496086880 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496087152 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496087424 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '445', 'doc': u'', 'properties': ()})
    option_139789496087696 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496087968 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496088240 = UnicodeOption(**{'default': u'smbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496088512 = UnicodeOption(**{'default': u'___auto_smbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496088784 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496094800 = OptionDescription(children=[option_139789496086608, option_139789496086880, option_139789496087152, option_139789496087424, option_139789496087696, option_139789496087968, option_139789496088240, option_139789496088512, option_139789496088784], **{'doc': u'', 'properties': (), 'name': 'service_access17'})
    option_139789496089056 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496089328 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496089600 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496089872 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '137', 'doc': u'', 'properties': ()})
    option_139789496090144 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496102992 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'udp', 'doc': u'', 'properties': ()})
    option_139789496103264 = UnicodeOption(**{'default': u'nmbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496103536 = UnicodeOption(**{'default': u'___auto_nmbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496103808 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496095120 = OptionDescription(children=[option_139789496089056, option_139789496089328, option_139789496089600, option_139789496089872, option_139789496090144, option_139789496102992, option_139789496103264, option_139789496103536, option_139789496103808], **{'doc': u'', 'properties': (), 'name': 'service_access18'})
    option_139789496104080 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496104352 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496104624 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496104896 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '138', 'doc': u'', 'properties': ()})
    option_139789496105168 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496105440 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'udp', 'doc': u'', 'properties': ()})
    option_139789496105712 = UnicodeOption(**{'default': u'nmbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496105984 = UnicodeOption(**{'default': u'___auto_nmbd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496106256 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496095440 = OptionDescription(children=[option_139789496104080, option_139789496104352, option_139789496104624, option_139789496104896, option_139789496105168, option_139789496105440, option_139789496105712, option_139789496105984, option_139789496106256], **{'doc': u'', 'properties': (), 'name': 'service_access19'})
    option_139789496106528 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496111184 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496111456 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496111728 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '3493', 'doc': u'', 'properties': ()})
    option_139789496112000 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496112272 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496112544 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496112816 = UnicodeOption(**{'default': u'___auto_nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496113088 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957104}]})
    option_139789496095760 = OptionDescription(children=[option_139789496106528, option_139789496111184, option_139789496111456, option_139789496111728, option_139789496112000, option_139789496112272, option_139789496112544, option_139789496112816, option_139789496113088], **{'doc': u'', 'properties': (), 'name': 'service_access20'})
    option_139789496113360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496113632 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496113904 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496114176 = UnicodeOption(**{'default': u'upsd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496114448 = UnicodeOption(**{'default': u'tcpwrapper', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496114720 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496119376 = UnicodeOption(**{'default': u'___auto_nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496119648 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957104}]})
    option_139789496096080 = OptionDescription(children=[option_139789496113360, option_139789496113632, option_139789496113904, option_139789496114176, option_139789496114448, option_139789496114720, option_139789496119376, option_139789496119648], **{'doc': u'', 'properties': (), 'name': 'service_access21'})
    option_139789496119920 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496120192 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496120464 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496120736 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '80', 'doc': u'', 'properties': ()})
    option_139789496121008 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496121280 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496121552 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496121824 = UnicodeOption(**{'default': u'___auto_apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496122096 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789496096400 = OptionDescription(children=[option_139789496119920, option_139789496120192, option_139789496120464, option_139789496120736, option_139789496121008, option_139789496121280, option_139789496121552, option_139789496121824, option_139789496122096], **{'doc': u'', 'properties': (), 'name': 'service_access22'})
    option_139789496122368 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496122640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496122912 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496131664 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '80', 'doc': u'', 'properties': ()})
    option_139789496131936 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496132208 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496132480 = UnicodeOption(**{'default': u'nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496132752 = UnicodeOption(**{'default': u'wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496133024 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789496096720 = OptionDescription(children=[option_139789496122368, option_139789496122640, option_139789496122912, option_139789496131664, option_139789496131936, option_139789496132208, option_139789496132480, option_139789496132752, option_139789496133024], **{'doc': u'', 'properties': (), 'name': 'service_access23'})
    option_139789496133296 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496133568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496133840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496134112 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '80', 'doc': u'', 'properties': ()})
    option_139789496134384 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496134656 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496134928 = UnicodeOption(**{'default': u'nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496135200 = UnicodeOption(**{'default': u'nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496139856 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496487456}]})
    option_139789496097040 = OptionDescription(children=[option_139789496133296, option_139789496133568, option_139789496133840, option_139789496134112, option_139789496134384, option_139789496134656, option_139789496134928, option_139789496135200, option_139789496139856], **{'doc': u'', 'properties': (), 'name': 'service_access24'})
    option_139789496140128 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496140400 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496140672 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496140944 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '443', 'doc': u'', 'properties': ()})
    option_139789496141216 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496141488 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496141760 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496142032 = UnicodeOption(**{'default': u'___auto_apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496142304 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789496097360 = OptionDescription(children=[option_139789496140128, option_139789496140400, option_139789496140672, option_139789496140944, option_139789496141216, option_139789496141488, option_139789496141760, option_139789496142032, option_139789496142304], **{'doc': u'', 'properties': (), 'name': 'service_access25'})
    option_139789496142576 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496142848 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496143120 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496143392 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '443', 'doc': u'', 'properties': ()})
    option_139789496152144 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496152416 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496152688 = UnicodeOption(**{'default': u'nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496152960 = UnicodeOption(**{'default': u'nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496153232 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496487456}]})
    option_139789496097680 = OptionDescription(children=[option_139789496142576, option_139789496142848, option_139789496143120, option_139789496143392, option_139789496152144, option_139789496152416, option_139789496152688, option_139789496152960, option_139789496153232], **{'doc': u'', 'properties': (), 'name': 'service_access26'})
    option_139789496153504 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496153776 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496154048 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496151688 = SymLinkOption(**{'name': 'name', "opt": option_139789497180512})
    option_139789496154320 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496154592 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496154864 = UnicodeOption(**{'default': u'sso', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496155136 = UnicodeOption(**{'default': u'___auto_sso', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496155408 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497964208}]})
    option_139789496098000 = OptionDescription(children=[option_139789496153504, option_139789496153776, option_139789496154048, option_139789496151688, option_139789496154320, option_139789496154592, option_139789496154864, option_139789496155136, option_139789496155408], **{'doc': u'', 'properties': (), 'name': 'service_access27'})
    option_139789496155680 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496160336 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496160608 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496160880 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '53', 'doc': u'', 'properties': ()})
    option_139789496161152 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496161424 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496161696 = UnicodeOption(**{'default': u'bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496161968 = UnicodeOption(**{'default': u'___auto_bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496162240 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496098320 = OptionDescription(children=[option_139789496155680, option_139789496160336, option_139789496160608, option_139789496160880, option_139789496161152, option_139789496161424, option_139789496161696, option_139789496161968, option_139789496162240], **{'doc': u'', 'properties': (), 'name': 'service_access28'})
    option_139789496162512 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496162784 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496163056 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496163328 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '53', 'doc': u'', 'properties': ()})
    option_139789496163600 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496163872 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'udp', 'doc': u'', 'properties': ()})
    option_139789496168528 = UnicodeOption(**{'default': u'bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496168800 = UnicodeOption(**{'default': u'___auto_bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496169072 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789496172624 = OptionDescription(children=[option_139789496162512, option_139789496162784, option_139789496163056, option_139789496163328, option_139789496163600, option_139789496163872, option_139789496168528, option_139789496168800, option_139789496169072], **{'doc': u'', 'properties': (), 'name': 'service_access29'})
    option_139789496169344 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496169616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496169888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496170160 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '4201', 'doc': u'', 'properties': ()})
    option_139789496170432 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496170704 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496170976 = UnicodeOption(**{'default': u'ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496171248 = UnicodeOption(**{'default': u'ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496171520 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497970768}]})
    option_139789496172944 = OptionDescription(children=[option_139789496169344, option_139789496169616, option_139789496169888, option_139789496170160, option_139789496170432, option_139789496170704, option_139789496170976, option_139789496171248, option_139789496171520], **{'doc': u'', 'properties': (), 'name': 'service_access30'})
    option_139789496171792 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496172064 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496180816 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496181088 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '4202', 'doc': u'', 'properties': ()})
    option_139789496181360 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496181632 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496181904 = UnicodeOption(**{'default': u'ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496182176 = UnicodeOption(**{'default': u'ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496182448 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497970768}]})
    option_139789496173264 = OptionDescription(children=[option_139789496171792, option_139789496172064, option_139789496180816, option_139789496181088, option_139789496181360, option_139789496181632, option_139789496181904, option_139789496182176, option_139789496182448], **{'doc': u'', 'properties': (), 'name': 'service_access31'})
    option_139789496182720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496182992 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496183264 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496183536 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '4200', 'doc': u'', 'properties': ()})
    option_139789496183808 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496184080 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496184352 = UnicodeOption(**{'default': u'ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496193104 = UnicodeOption(**{'default': u'___auto_ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496193376 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964480}]})
    option_139789496173584 = OptionDescription(children=[option_139789496182720, option_139789496182992, option_139789496183264, option_139789496183536, option_139789496183808, option_139789496184080, option_139789496184352, option_139789496193104, option_139789496193376], **{'doc': u'', 'properties': (), 'name': 'service_access32'})
    option_139789496193648 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496193920 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496194192 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496191528 = SymLinkOption(**{'name': 'name', "opt": option_139789497235664})
    option_139789496194464 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496194736 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496195008 = UnicodeOption(**{'default': u'ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496195280 = UnicodeOption(**{'default': u'___auto_ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496195552 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964480}]})
    option_139789496173904 = OptionDescription(children=[option_139789496193648, option_139789496193920, option_139789496194192, option_139789496191528, option_139789496194464, option_139789496194736, option_139789496195008, option_139789496195280, option_139789496195552], **{'doc': u'', 'properties': (), 'name': 'service_access33'})
    option_139789496195824 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496196096 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496196368 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496196640 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '3306', 'doc': u'', 'properties': ()})
    option_139789496205392 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496205664 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789496205936 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496206208 = UnicodeOption(**{'default': u'myPort', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496206480 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971312}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497236208}]})
    option_139789496174224 = OptionDescription(children=[option_139789496195824, option_139789496196096, option_139789496196368, option_139789496196640, option_139789496205392, option_139789496205664, option_139789496205936, option_139789496206208, option_139789496206480], **{'doc': u'', 'properties': (), 'name': 'service_access34'})
    option_139789496206752 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789496207024 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789496207296 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789496207568 = UnicodeOption(**{'default': u'mysqld', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789496207840 = UnicodeOption(**{'default': u'tcpwrapper', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789496208112 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789496208384 = UnicodeOption(**{'default': u'myPort', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789496208656 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971312}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497236208}]})
    option_139789496174544 = OptionDescription(children=[option_139789496206752, option_139789496207024, option_139789496207296, option_139789496207568, option_139789496207840, option_139789496208112, option_139789496208384, option_139789496208656], **{'doc': u'', 'properties': (), 'name': 'service_access35'})
    option_139789496208928 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495693392 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495693664 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495693936 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '9103', 'doc': u'', 'properties': ()})
    option_139789495694208 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495694480 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495694752 = UnicodeOption(**{'default': u'bareos_sd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495695024 = UnicodeOption(**{'default': u'___auto_bareos_sd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495695296 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971040}]})
    option_139789496174864 = OptionDescription(children=[option_139789496208928, option_139789495693392, option_139789495693664, option_139789495693936, option_139789495694208, option_139789495694480, option_139789495694752, option_139789495695024, option_139789495695296], **{'doc': u'', 'properties': (), 'name': 'service_access36'})
    option_139789495695568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495695840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495696112 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495700008 = SymLinkOption(**{'name': 'name', "opt": option_139789496792752})
    option_139789495696384 = UnicodeOption(**{'default': u'tcpwrapper', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495696656 = UnicodeOption(**{'default': u'bareos_sd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495696928 = UnicodeOption(**{'default': u'___auto_bareos_sd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495701584 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971040}]})
    option_139789496175184 = OptionDescription(children=[option_139789495695568, option_139789495695840, option_139789495696112, option_139789495700008, option_139789495696384, option_139789495696656, option_139789495696928, option_139789495701584], **{'doc': u'', 'properties': (), 'name': 'service_access37'})
    option_139789495701856 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495702128 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495702400 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495702672 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '143', 'doc': u'', 'properties': ()})
    option_139789495702944 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495703216 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495703488 = UnicodeOption(**{'default': u'courier_imap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495703760 = UnicodeOption(**{'default': u'___auto_courier_imap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495704032 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496747152}]})
    option_139789496175504 = OptionDescription(children=[option_139789495701856, option_139789495702128, option_139789495702400, option_139789495702672, option_139789495702944, option_139789495703216, option_139789495703488, option_139789495703760, option_139789495704032], **{'doc': u'', 'properties': (), 'name': 'service_access38'})
    option_139789495704304 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495704576 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495704848 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495705120 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '993', 'doc': u'', 'properties': ()})
    option_139789495717968 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495718240 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495718512 = UnicodeOption(**{'default': u'courier_imap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495718784 = UnicodeOption(**{'default': u'___auto_courier_imap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495719056 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496747152}]})
    option_139789496175824 = OptionDescription(children=[option_139789495704304, option_139789495704576, option_139789495704848, option_139789495705120, option_139789495717968, option_139789495718240, option_139789495718512, option_139789495718784, option_139789495719056], **{'doc': u'', 'properties': (), 'name': 'service_access39'})
    option_139789495719328 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495719600 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495719872 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495720144 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '110', 'doc': u'', 'properties': ()})
    option_139789495720416 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495720688 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495720960 = UnicodeOption(**{'default': u'courier_pop', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495721232 = UnicodeOption(**{'default': u'___auto_courier_pop', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495721504 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496746880}]})
    option_139789496176144 = OptionDescription(children=[option_139789495719328, option_139789495719600, option_139789495719872, option_139789495720144, option_139789495720416, option_139789495720688, option_139789495720960, option_139789495721232, option_139789495721504], **{'doc': u'', 'properties': (), 'name': 'service_access40'})
    option_139789495726160 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495726432 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495726704 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495726976 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '995', 'doc': u'', 'properties': ()})
    option_139789495727248 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495727520 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495727792 = UnicodeOption(**{'default': u'courier_pop', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495728064 = UnicodeOption(**{'default': u'___auto_courier_pop', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495728336 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496746880}]})
    option_139789495742544 = OptionDescription(children=[option_139789495726160, option_139789495726432, option_139789495726704, option_139789495726976, option_139789495727248, option_139789495727520, option_139789495727792, option_139789495728064, option_139789495728336], **{'doc': u'', 'properties': (), 'name': 'service_access41'})
    option_139789495728608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495728880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495729152 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495732216 = SymLinkOption(**{'name': 'name', "opt": option_139789496832624})
    option_139789495729424 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495729696 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495746640 = UnicodeOption(**{'default': u'lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495746912 = UnicodeOption(**{'default': u'___auto_lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495747184 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495742864 = OptionDescription(children=[option_139789495728608, option_139789495728880, option_139789495729152, option_139789495732216, option_139789495729424, option_139789495729696, option_139789495746640, option_139789495746912, option_139789495747184], **{'doc': u'', 'properties': (), 'name': 'service_access42'})
    option_139789495747456 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495747728 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495748000 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495748272 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '3128', 'doc': u'', 'properties': ()})
    option_139789495748544 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495748816 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495749088 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495749360 = UnicodeOption(**{'default': u'___auto_squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495749632 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495743184 = OptionDescription(children=[option_139789495747456, option_139789495747728, option_139789495748000, option_139789495748272, option_139789495748544, option_139789495748816, option_139789495749088, option_139789495749360, option_139789495749632], **{'doc': u'', 'properties': (), 'name': 'service_access43'})
    option_139789495749904 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495750176 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495754832 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495759208 = SymLinkOption(**{'name': 'name', "opt": option_139789496886688})
    option_139789495755104 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495755376 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495755648 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495755920 = UnicodeOption(**{'default': u'squid3-2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495756192 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495743504 = OptionDescription(children=[option_139789495749904, option_139789495750176, option_139789495754832, option_139789495759208, option_139789495755104, option_139789495755376, option_139789495755648, option_139789495755920, option_139789495756192], **{'doc': u'', 'properties': (), 'name': 'service_access44'})
    option_139789495756464 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495756736 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495757008 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495757280 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '631', 'doc': u'', 'properties': ()})
    option_139789495757552 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495757824 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495758096 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495758368 = UnicodeOption(**{'default': u'___auto_cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495767120 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789495743824 = OptionDescription(children=[option_139789495756464, option_139789495756736, option_139789495757008, option_139789495757280, option_139789495757552, option_139789495757824, option_139789495758096, option_139789495758368, option_139789495767120], **{'doc': u'', 'properties': (), 'name': 'service_access45'})
    option_139789495767392 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495767664 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495767936 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495765544 = SymLinkOption(**{'name': 'name', "opt": option_139789496841088})
    option_139789495768208 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495768480 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'udp', 'doc': u'', 'properties': ()})
    option_139789495768752 = UnicodeOption(**{'default': u'squid3sibling', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495769024 = UnicodeOption(**{'default': u'___auto_squid3sibling', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495769296 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495744144 = OptionDescription(children=[option_139789495767392, option_139789495767664, option_139789495767936, option_139789495765544, option_139789495768208, option_139789495768480, option_139789495768752, option_139789495769024, option_139789495769296], **{'doc': u'', 'properties': (), 'name': 'service_access46'})
    option_139789495769568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495769840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495770112 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495770384 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '8443', 'doc': u'', 'properties': ()})
    option_139789495770656 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495775312 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495775584 = UnicodeOption(**{'default': u'nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495775856 = UnicodeOption(**{'default': u'revsso', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495776128 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': None, 'action': 'disabled', 'inverse': False, "option": option_139789496505488}]})
    option_139789495744464 = OptionDescription(children=[option_139789495769568, option_139789495769840, option_139789495770112, option_139789495770384, option_139789495770656, option_139789495775312, option_139789495775584, option_139789495775856, option_139789495776128], **{'doc': u'', 'properties': (), 'name': 'service_access47'})
    option_139789495776400 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495776672 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495776944 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495777216 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '21', 'doc': u'', 'properties': ()})
    option_139789495777488 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495777760 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495778032 = UnicodeOption(**{'default': u'proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495778304 = UnicodeOption(**{'default': u'proftpd_container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495778576 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}, {'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495744784 = OptionDescription(children=[option_139789495776400, option_139789495776672, option_139789495776944, option_139789495777216, option_139789495777488, option_139789495777760, option_139789495778032, option_139789495778304, option_139789495778576], **{'doc': u'', 'properties': (), 'name': 'service_access48'})
    option_139789495778848 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495783504 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495783776 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495784048 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '8787', 'doc': u'', 'properties': ()})
    option_139789495784320 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495784592 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495784864 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495785136 = UnicodeOption(**{'default': u'___auto_sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495785408 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789495745104 = OptionDescription(children=[option_139789495778848, option_139789495783504, option_139789495783776, option_139789495784048, option_139789495784320, option_139789495784592, option_139789495784864, option_139789495785136, option_139789495785408], **{'doc': u'', 'properties': (), 'name': 'service_access49'})
    option_139789495785680 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495785952 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495786224 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495786496 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '8888', 'doc': u'', 'properties': ()})
    option_139789495786768 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495787040 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495795792 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495796064 = UnicodeOption(**{'default': u'___auto_sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495796336 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789495745424 = OptionDescription(children=[option_139789495785680, option_139789495785952, option_139789495786224, option_139789495786496, option_139789495786768, option_139789495787040, option_139789495795792, option_139789495796064, option_139789495796336], **{'doc': u'', 'properties': (), 'name': 'service_access50'})
    option_139789495796608 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495796880 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495797152 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495797424 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '8789', 'doc': u'', 'properties': ()})
    option_139789495797696 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495797968 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495798240 = UnicodeOption(**{'default': u'controle-vnc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495798512 = UnicodeOption(**{'default': u'___auto_controle-vnc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495798784 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495745744 = OptionDescription(children=[option_139789495796608, option_139789495796880, option_139789495797152, option_139789495797424, option_139789495797696, option_139789495797968, option_139789495798240, option_139789495798512, option_139789495798784], **{'doc': u'', 'properties': (), 'name': 'service_access51'})
    option_139789495799056 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495799328 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495808080 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495808352 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '8790', 'doc': u'', 'properties': ()})
    option_139789495808624 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495808896 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495809168 = UnicodeOption(**{'default': u'controle-vnc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495809440 = UnicodeOption(**{'default': u'___auto_controle-vnc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495809712 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495746064 = OptionDescription(children=[option_139789495799056, option_139789495799328, option_139789495808080, option_139789495808352, option_139789495808624, option_139789495808896, option_139789495809168, option_139789495809440, option_139789495809712], **{'doc': u'', 'properties': (), 'name': 'service_access52'})
    option_139789495809984 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495810256 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495810528 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495810800 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '25', 'doc': u'', 'properties': ()})
    option_139789495811072 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495811344 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495811616 = UnicodeOption(**{'default': u'exim4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495816272 = UnicodeOption(**{'default': u'smtp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495816544 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496770640}]})
    option_139789495820368 = OptionDescription(children=[option_139789495809984, option_139789495810256, option_139789495810528, option_139789495810800, option_139789495811072, option_139789495811344, option_139789495811616, option_139789495816272, option_139789495816544], **{'doc': u'', 'properties': (), 'name': 'service_access53'})
    option_139789495816816 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495817088 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495817360 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495817632 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '465', 'doc': u'', 'properties': ()})
    option_139789495817904 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495818176 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495818448 = UnicodeOption(**{'default': u'exim4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495818720 = UnicodeOption(**{'default': u'smtp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495818992 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496770640}]})
    option_139789495820688 = OptionDescription(children=[option_139789495816816, option_139789495817088, option_139789495817360, option_139789495817632, option_139789495817904, option_139789495818176, option_139789495818448, option_139789495818720, option_139789495818992], **{'doc': u'', 'properties': (), 'name': 'service_access54'})
    option_139789495819264 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495819536 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495819808 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495828560 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '6080', 'doc': u'', 'properties': ()})
    option_139789495828832 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495829104 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495829376 = UnicodeOption(**{'default': u'websockify', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495829648 = UnicodeOption(**{'default': u'___auto_websockify', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495829920 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497146368}]})
    option_139789495821008 = OptionDescription(children=[option_139789495819264, option_139789495819536, option_139789495819808, option_139789495828560, option_139789495828832, option_139789495829104, option_139789495829376, option_139789495829648, option_139789495829920], **{'doc': u'', 'properties': (), 'name': 'service_access55'})
    option_139789495830192 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495830464 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495830736 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495834616 = SymLinkOption(**{'name': 'name', "opt": option_139789497575088})
    option_139789495831008 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495831280 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495831552 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495831824 = UnicodeOption(**{'default': u'___auto_eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495832096 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495821328 = OptionDescription(children=[option_139789495830192, option_139789495830464, option_139789495830736, option_139789495834616, option_139789495831008, option_139789495831280, option_139789495831552, option_139789495831824, option_139789495832096], **{'doc': u'', 'properties': (), 'name': 'service_access56'})
    option_139789495840848 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495841120 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495841392 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495841664 = PortOption(**{'multi': False, 'name': 'name', 'allow_private': True, 'default': '8090', 'doc': u'', 'properties': ()})
    option_139789495841936 = UnicodeOption(**{'default': u'port', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495842208 = ChoiceOption(**{'multi': False, 'values': (u'tcp', u'udp'), 'name': 'protocol', 'default': 'tcp', 'doc': u'', 'properties': ()})
    option_139789495842480 = UnicodeOption(**{'default': u'z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495842752 = UnicodeOption(**{'default': u'___auto_z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_accesslist'})
    option_139789495843024 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495821648 = OptionDescription(children=[option_139789495840848, option_139789495841120, option_139789495841392, option_139789495841664, option_139789495841936, option_139789495842208, option_139789495842480, option_139789495842752, option_139789495843024], **{'doc': u'', 'properties': (), 'name': 'service_access57'})
    option_139789495821968 = OptionDescription(children=[option_139789495961872, option_139789495962192, option_139789495962512, option_139789495962832, option_139789495963152, option_139789496016976, option_139789496017296, option_139789496017616, option_139789496017936, option_139789496018256, option_139789496018576, option_139789496018896, option_139789496019216, option_139789496019536, option_139789496019856, option_139789496020176, option_139789496020496, option_139789496094800, option_139789496095120, option_139789496095440, option_139789496095760, option_139789496096080, option_139789496096400, option_139789496096720, option_139789496097040, option_139789496097360, option_139789496097680, option_139789496098000, option_139789496098320, option_139789496172624, option_139789496172944, option_139789496173264, option_139789496173584, option_139789496173904, option_139789496174224, option_139789496174544, option_139789496174864, option_139789496175184, option_139789496175504, option_139789496175824, option_139789496176144, option_139789495742544, option_139789495742864, option_139789495743184, option_139789495743504, option_139789495743824, option_139789495744144, option_139789495744464, option_139789495744784, option_139789495745104, option_139789495745424, option_139789495745744, option_139789495746064, option_139789495820368, option_139789495820688, option_139789495821008, option_139789495821328, option_139789495821648], **{'doc': u'', 'properties': (), 'name': 'service_accesss'})
    option_139789495843296 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495843568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495843840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495844112 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495845784 = SymLinkOption(**{'name': 'name', "opt": option_139789498032480})
    option_139789495846344 = SymLinkOption(**{'name': 'netmask', "opt": option_139789498032752})
    option_139789495844384 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495849040 = UnicodeOption(**{'default': u'rsyslog_udp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495849312 = UnicodeOption(**{'default': u'___auto_rsyslog_udp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495849584 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495822288 = OptionDescription(children=[option_139789495843296, option_139789495843568, option_139789495843840, option_139789495844112, option_139789495845784, option_139789495846344, option_139789495844384, option_139789495849040, option_139789495849312, option_139789495849584], **{'doc': u'', 'properties': (), 'name': 'service_restriction0'})
    option_139789495849856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495850128 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495850400 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495850672 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495853696 = SymLinkOption(**{'name': 'name', "opt": option_139789498033568})
    option_139789495854256 = SymLinkOption(**{'name': 'netmask', "opt": option_139789498033840})
    option_139789495850944 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495851216 = UnicodeOption(**{'default': u'rsyslog_tcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495851488 = UnicodeOption(**{'default': u'___auto_rsyslog_tcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495851760 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495822608 = OptionDescription(children=[option_139789495849856, option_139789495850128, option_139789495850400, option_139789495850672, option_139789495853696, option_139789495854256, option_139789495850944, option_139789495851216, option_139789495851488, option_139789495851760], **{'doc': u'', 'properties': (), 'name': 'service_restriction1'})
    option_139789495852032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495852304 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495852576 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495861328 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495865704 = SymLinkOption(**{'name': 'name', "opt": option_139789498019088})
    option_139789495866264 = SymLinkOption(**{'name': 'netmask', "opt": option_139789498019360})
    option_139789495861600 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495861872 = UnicodeOption(**{'default': u'rsyslog_relp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495862144 = UnicodeOption(**{'default': u'___auto_rsyslog_relp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495862416 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495822928 = OptionDescription(children=[option_139789495852032, option_139789495852304, option_139789495852576, option_139789495861328, option_139789495865704, option_139789495866264, option_139789495861600, option_139789495861872, option_139789495862144, option_139789495862416], **{'doc': u'', 'properties': (), 'name': 'service_restriction2'})
    option_139789495862688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495862960 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495863232 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495863504 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495869520 = SymLinkOption(**{'name': 'name', "opt": option_139789497558704})
    option_139789495870080 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497558976})
    option_139789495863776 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495864048 = UnicodeOption(**{'default': u'sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495864320 = UnicodeOption(**{'default': u'___auto_sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495864592 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495823248 = OptionDescription(children=[option_139789495862688, option_139789495862960, option_139789495863232, option_139789495863504, option_139789495869520, option_139789495870080, option_139789495863776, option_139789495864048, option_139789495864320, option_139789495864592], **{'doc': u'', 'properties': (), 'name': 'service_restriction3'})
    option_139789495864864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495873616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495873888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495874160 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495873160 = SymLinkOption(**{'name': 'name', "opt": option_139789497558704})
    option_139789495877992 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497558976})
    option_139789495874432 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495874704 = UnicodeOption(**{'default': u'genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495874976 = UnicodeOption(**{'default': u'___auto_genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495875248 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495823568 = OptionDescription(children=[option_139789495864864, option_139789495873616, option_139789495873888, option_139789495874160, option_139789495873160, option_139789495877992, option_139789495874432, option_139789495874704, option_139789495874976, option_139789495875248], **{'doc': u'', 'properties': (), 'name': 'service_restriction4'})
    option_139789495875520 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495875792 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495876064 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495876336 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495881072 = SymLinkOption(**{'name': 'name', "opt": option_139789497636256})
    option_139789495881808 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497636528})
    option_139789495876608 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495876880 = UnicodeOption(**{'default': u'sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495877152 = UnicodeOption(**{'default': u'___auto_sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495885904 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495823888 = OptionDescription(children=[option_139789495875520, option_139789495875792, option_139789495876064, option_139789495876336, option_139789495881072, option_139789495881808, option_139789495876608, option_139789495876880, option_139789495877152, option_139789495885904], **{'doc': u'', 'properties': (), 'name': 'service_restriction5'})
    option_139789495886176 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495886448 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495886720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495886992 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495884888 = SymLinkOption(**{'name': 'name', "opt": option_139789497636256})
    option_139789495885448 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497636528})
    option_139789495887264 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495887536 = UnicodeOption(**{'default': u'genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495887808 = UnicodeOption(**{'default': u'___auto_genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495888080 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495894096 = OptionDescription(children=[option_139789495886176, option_139789495886448, option_139789495886720, option_139789495886992, option_139789495884888, option_139789495885448, option_139789495887264, option_139789495887536, option_139789495887808, option_139789495888080], **{'doc': u'', 'properties': (), 'name': 'service_restriction6'})
    option_139789495888352 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495888624 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495888896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495889168 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495892800 = SymLinkOption(**{'name': 'name', "opt": option_139789497714080})
    option_139789495893360 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497714352})
    option_139789495889440 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495902288 = UnicodeOption(**{'default': u'sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495902560 = UnicodeOption(**{'default': u'___auto_sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495902832 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495894416 = OptionDescription(children=[option_139789495888352, option_139789495888624, option_139789495888896, option_139789495889168, option_139789495892800, option_139789495893360, option_139789495889440, option_139789495902288, option_139789495902560, option_139789495902832], **{'doc': u'', 'properties': (), 'name': 'service_restriction7'})
    option_139789495903104 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495903376 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495903648 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495903920 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495900712 = SymLinkOption(**{'name': 'name', "opt": option_139789497714080})
    option_139789495901272 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497714352})
    option_139789495904192 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495904464 = UnicodeOption(**{'default': u'genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495904736 = UnicodeOption(**{'default': u'___auto_genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495905008 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495894736 = OptionDescription(children=[option_139789495903104, option_139789495903376, option_139789495903648, option_139789495903920, option_139789495900712, option_139789495901272, option_139789495904192, option_139789495904464, option_139789495904736, option_139789495905008], **{'doc': u'', 'properties': (), 'name': 'service_restriction8'})
    option_139789495905280 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495905552 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495905824 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495910480 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495908624 = SymLinkOption(**{'name': 'name', "opt": option_139789497768688})
    option_139789495909184 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497768960})
    option_139789495910752 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495911024 = UnicodeOption(**{'default': u'sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495911296 = UnicodeOption(**{'default': u'___auto_sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495911568 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495895056 = OptionDescription(children=[option_139789495905280, option_139789495905552, option_139789495905824, option_139789495910480, option_139789495908624, option_139789495909184, option_139789495910752, option_139789495911024, option_139789495911296, option_139789495911568], **{'doc': u'', 'properties': (), 'name': 'service_restriction9'})
    option_139789495911840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495912112 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495912384 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495912656 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495916536 = SymLinkOption(**{'name': 'name', "opt": option_139789497768688})
    option_139789495917096 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497768960})
    option_139789495912928 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495913200 = UnicodeOption(**{'default': u'genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495913472 = UnicodeOption(**{'default': u'___auto_genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495913744 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495895376 = OptionDescription(children=[option_139789495911840, option_139789495912112, option_139789495912384, option_139789495912656, option_139789495916536, option_139789495917096, option_139789495912928, option_139789495913200, option_139789495913472, option_139789495913744], **{'doc': u'', 'properties': (), 'name': 'service_restriction10'})
    option_139789495914016 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495922768 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495923040 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495923312 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495920352 = SymLinkOption(**{'name': 'name', "opt": option_139789497323872})
    option_139789495920912 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497324144})
    option_139789495923584 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495923856 = UnicodeOption(**{'default': u'sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495924128 = UnicodeOption(**{'default': u'___auto_sshd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495924400 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495895696 = OptionDescription(children=[option_139789495914016, option_139789495922768, option_139789495923040, option_139789495923312, option_139789495920352, option_139789495920912, option_139789495923584, option_139789495923856, option_139789495924128, option_139789495924400], **{'doc': u'', 'properties': (), 'name': 'service_restriction11'})
    option_139789495924672 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495924944 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495925216 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495925488 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495928264 = SymLinkOption(**{'name': 'name', "opt": option_139789497323872})
    option_139789495928824 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497324144})
    option_139789495925760 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495926032 = UnicodeOption(**{'default': u'genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495926304 = UnicodeOption(**{'default': u'___auto_genconfig', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495930960 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495896016 = OptionDescription(children=[option_139789495924672, option_139789495924944, option_139789495925216, option_139789495925488, option_139789495928264, option_139789495928824, option_139789495925760, option_139789495926032, option_139789495926304, option_139789495930960], **{'doc': u'', 'properties': (), 'name': 'service_restriction12'})
    option_139789495931232 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495931504 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495931776 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495932048 = UnicodeOption(**{'default': u'auto', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495936176 = SymLinkOption(**{'name': 'name', "opt": option_139789497519648})
    option_139789495932320 = NetmaskOption(**{'default': u'255.255.255.255', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495932592 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495932864 = UnicodeOption(**{'default': u'failover_dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495933136 = UnicodeOption(**{'default': u'failover_dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495933408 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497518288}]})
    option_139789495896336 = OptionDescription(children=[option_139789495931232, option_139789495931504, option_139789495931776, option_139789495932048, option_139789495936176, option_139789495932320, option_139789495932592, option_139789495932864, option_139789495933136, option_139789495933408], **{'doc': u'', 'properties': (), 'name': 'service_restriction13'})
    option_139789495933680 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495933952 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495934224 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495934496 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495943248 = NetworkOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495943520 = NetmaskOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495943792 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495944064 = UnicodeOption(**{'default': u'tftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495944336 = UnicodeOption(**{'default': u'tftp_eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495944608 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': ('disabled',), 'name': 'activate'})
    option_139789495896656 = OptionDescription(children=[option_139789495933680, option_139789495933952, option_139789495934224, option_139789495934496, option_139789495943248, option_139789495943520, option_139789495943792, option_139789495944064, option_139789495944336, option_139789495944608], **{'doc': u'', 'properties': (), 'name': 'service_restriction14'})
    option_139789495944880 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495945152 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495945424 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495945696 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495945968 = NetworkOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495946240 = NetmaskOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495946512 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495946784 = UnicodeOption(**{'default': u'tftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495951440 = UnicodeOption(**{'default': u'tftp_eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495951712 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495896976 = OptionDescription(children=[option_139789495944880, option_139789495945152, option_139789495945424, option_139789495945696, option_139789495945968, option_139789495946240, option_139789495946512, option_139789495946784, option_139789495951440, option_139789495951712], **{'doc': u'', 'properties': (), 'name': 'service_restriction15'})
    option_139789495951984 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495952256 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495952528 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495952800 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495953072 = NetworkOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495953344 = NetmaskOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495953616 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495953888 = UnicodeOption(**{'default': u'bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495954160 = UnicodeOption(**{'default': u'bind9_eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495954432 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497596384}]})
    option_139789495897296 = OptionDescription(children=[option_139789495951984, option_139789495952256, option_139789495952528, option_139789495952800, option_139789495953072, option_139789495953344, option_139789495953616, option_139789495953888, option_139789495954160, option_139789495954432], **{'doc': u'', 'properties': (), 'name': 'service_restriction16'})
    option_139789495954704 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495954976 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495439440 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495439712 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495439984 = NetworkOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495440256 = NetmaskOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495440528 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495440800 = UnicodeOption(**{'default': u'bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495441072 = UnicodeOption(**{'default': u'___auto_bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495441344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495897616 = OptionDescription(children=[option_139789495954704, option_139789495954976, option_139789495439440, option_139789495439712, option_139789495439984, option_139789495440256, option_139789495440528, option_139789495440800, option_139789495441072, option_139789495441344], **{'doc': u'', 'properties': (), 'name': 'service_restriction17'})
    option_139789495441616 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495441888 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495442160 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495442432 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495442704 = NetworkOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495442976 = NetmaskOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495447632 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495447904 = UnicodeOption(**{'default': u'bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495448176 = UnicodeOption(**{'default': u'___auto_bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495448448 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495451728 = OptionDescription(children=[option_139789495441616, option_139789495441888, option_139789495442160, option_139789495442432, option_139789495442704, option_139789495442976, option_139789495447632, option_139789495447904, option_139789495448176, option_139789495448448], **{'doc': u'', 'properties': (), 'name': 'service_restriction18'})
    option_139789495448720 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495448992 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495449264 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495449536 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495449808 = NetworkOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495450080 = NetmaskOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495450352 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495450624 = UnicodeOption(**{'default': u'bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495450896 = UnicodeOption(**{'default': u'___auto_bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495451168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495452048 = OptionDescription(children=[option_139789495448720, option_139789495448992, option_139789495449264, option_139789495449536, option_139789495449808, option_139789495450080, option_139789495450352, option_139789495450624, option_139789495450896, option_139789495451168], **{'doc': u'', 'properties': (), 'name': 'service_restriction19'})
    option_139789495459920 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495460192 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495460464 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495460736 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495461008 = NetworkOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495461280 = NetmaskOption(**{'default': u'0.0.0.0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495461552 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495461824 = UnicodeOption(**{'default': u'bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495462096 = UnicodeOption(**{'default': u'___auto_bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495462368 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495452368 = OptionDescription(children=[option_139789495459920, option_139789495460192, option_139789495460464, option_139789495460736, option_139789495461008, option_139789495461280, option_139789495461552, option_139789495461824, option_139789495462096, option_139789495462368], **{'doc': u'', 'properties': (), 'name': 'service_restriction20'})
    option_139789495462640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495462912 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495463184 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495463456 = UnicodeOption(**{'default': u'auto', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495466536 = SymLinkOption(**{'name': 'name', "opt": option_139789497115248})
    option_139789495467096 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497115520})
    option_139789495468112 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495468384 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495468656 = UnicodeOption(**{'default': u'___auto_nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495468928 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957104}]})
    option_139789495452688 = OptionDescription(children=[option_139789495462640, option_139789495462912, option_139789495463184, option_139789495463456, option_139789495466536, option_139789495467096, option_139789495468112, option_139789495468384, option_139789495468656, option_139789495468928], **{'doc': u'', 'properties': (), 'name': 'service_restriction21'})
    option_139789495469200 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495469472 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495469744 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495470016 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495474448 = SymLinkOption(**{'name': 'name', "opt": option_139789497603216})
    option_139789495475008 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497603488})
    option_139789495470288 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495470560 = UnicodeOption(**{'default': u'ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495470832 = UnicodeOption(**{'default': u'___auto_ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495471104 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495453008 = OptionDescription(children=[option_139789495469200, option_139789495469472, option_139789495469744, option_139789495470016, option_139789495474448, option_139789495475008, option_139789495470288, option_139789495470560, option_139789495470832, option_139789495471104], **{'doc': u'', 'properties': (), 'name': 'service_restriction22'})
    option_139789495471376 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495471648 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495480400 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495480672 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495478264 = SymLinkOption(**{'name': 'name', "opt": option_139789497692512})
    option_139789495478824 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497692784})
    option_139789495480944 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495481216 = UnicodeOption(**{'default': u'ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495481488 = UnicodeOption(**{'default': u'___auto_ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495481760 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495453328 = OptionDescription(children=[option_139789495471376, option_139789495471648, option_139789495480400, option_139789495480672, option_139789495478264, option_139789495478824, option_139789495480944, option_139789495481216, option_139789495481488, option_139789495481760], **{'doc': u'', 'properties': (), 'name': 'service_restriction23'})
    option_139789495482032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495482304 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495482576 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495482848 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495486176 = SymLinkOption(**{'name': 'name', "opt": option_139789497747936})
    option_139789495486736 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497748208})
    option_139789495483120 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495483392 = UnicodeOption(**{'default': u'ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495483664 = UnicodeOption(**{'default': u'___auto_ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495483936 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495453648 = OptionDescription(children=[option_139789495482032, option_139789495482304, option_139789495482576, option_139789495482848, option_139789495486176, option_139789495486736, option_139789495483120, option_139789495483392, option_139789495483664, option_139789495483936], **{'doc': u'', 'properties': (), 'name': 'service_restriction24'})
    option_139789495492688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495492960 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495493232 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495493504 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495489992 = SymLinkOption(**{'name': 'name', "opt": option_139789497299024})
    option_139789495490552 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497299296})
    option_139789495493776 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495494048 = UnicodeOption(**{'default': u'ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495494320 = UnicodeOption(**{'default': u'___auto_ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495494592 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495453968 = OptionDescription(children=[option_139789495492688, option_139789495492960, option_139789495493232, option_139789495493504, option_139789495489992, option_139789495490552, option_139789495493776, option_139789495494048, option_139789495494320, option_139789495494592], **{'doc': u'', 'properties': (), 'name': 'service_restriction25'})
    option_139789495494864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495495136 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495495408 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495495680 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495497904 = SymLinkOption(**{'name': 'name', "opt": option_139789497353632})
    option_139789495498464 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497353904})
    option_139789495495952 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495496224 = UnicodeOption(**{'default': u'ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495500880 = UnicodeOption(**{'default': u'___auto_ead_server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495501152 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495454288 = OptionDescription(children=[option_139789495494864, option_139789495495136, option_139789495495408, option_139789495495680, option_139789495497904, option_139789495498464, option_139789495495952, option_139789495496224, option_139789495500880, option_139789495501152], **{'doc': u'', 'properties': (), 'name': 'service_restriction26'})
    option_139789495501424 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495501696 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495501968 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495502240 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495505816 = SymLinkOption(**{'name': 'name', "opt": option_139789497559520})
    option_139789495506376 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497559792})
    option_139789495502512 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495502784 = UnicodeOption(**{'default': u'ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495503056 = UnicodeOption(**{'default': u'___auto_ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495503328 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964480}]})
    option_139789495454608 = OptionDescription(children=[option_139789495501424, option_139789495501696, option_139789495501968, option_139789495502240, option_139789495505816, option_139789495506376, option_139789495502512, option_139789495502784, option_139789495503056, option_139789495503328], **{'doc': u'', 'properties': (), 'name': 'service_restriction27'})
    option_139789495503600 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495503872 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495504144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495504416 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495509632 = SymLinkOption(**{'name': 'name', "opt": option_139789497559520})
    option_139789495510192 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497559792})
    option_139789495513168 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495513440 = UnicodeOption(**{'default': u'lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495513712 = UnicodeOption(**{'default': u'___auto_lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495513984 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495454928 = OptionDescription(children=[option_139789495503600, option_139789495503872, option_139789495504144, option_139789495504416, option_139789495509632, option_139789495510192, option_139789495513168, option_139789495513440, option_139789495513712, option_139789495513984], **{'doc': u'', 'properties': (), 'name': 'service_restriction28'})
    option_139789495514256 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495514528 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495514800 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495515072 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495517544 = SymLinkOption(**{'name': 'name', "opt": option_139789497559520})
    option_139789495518104 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497559792})
    option_139789495515344 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495515616 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495515888 = UnicodeOption(**{'default': u'___auto_sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495516160 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789495455248 = OptionDescription(children=[option_139789495514256, option_139789495514528, option_139789495514800, option_139789495515072, option_139789495517544, option_139789495518104, option_139789495515344, option_139789495515616, option_139789495515888, option_139789495516160], **{'doc': u'', 'properties': (), 'name': 'service_restriction29'})
    option_139789495516432 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495516704 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495525456 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495525728 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495529552 = SymLinkOption(**{'name': 'name', "opt": option_139789497559520})
    option_139789495530112 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497559792})
    option_139789495526000 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495526272 = UnicodeOption(**{'default': u'z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495526544 = UnicodeOption(**{'default': u'___auto_z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495526816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495533648 = OptionDescription(children=[option_139789495516432, option_139789495516704, option_139789495525456, option_139789495525728, option_139789495529552, option_139789495530112, option_139789495526000, option_139789495526272, option_139789495526544, option_139789495526816], **{'doc': u'', 'properties': (), 'name': 'service_restriction30'})
    option_139789495527088 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495527360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495527632 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495527904 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495533192 = SymLinkOption(**{'name': 'name', "opt": option_139789497637072})
    option_139789495542120 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497637344})
    option_139789495528176 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495528448 = UnicodeOption(**{'default': u'ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495528720 = UnicodeOption(**{'default': u'___auto_ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495528992 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964480}]})
    option_139789495533968 = OptionDescription(children=[option_139789495527088, option_139789495527360, option_139789495527632, option_139789495527904, option_139789495533192, option_139789495542120, option_139789495528176, option_139789495528448, option_139789495528720, option_139789495528992], **{'doc': u'', 'properties': (), 'name': 'service_restriction31'})
    option_139789495545936 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495546208 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495546480 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495546752 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495545200 = SymLinkOption(**{'name': 'name', "opt": option_139789497637072})
    option_139789495550032 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497637344})
    option_139789495547024 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495547296 = UnicodeOption(**{'default': u'lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495547568 = UnicodeOption(**{'default': u'___auto_lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495547840 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495534288 = OptionDescription(children=[option_139789495545936, option_139789495546208, option_139789495546480, option_139789495546752, option_139789495545200, option_139789495550032, option_139789495547024, option_139789495547296, option_139789495547568, option_139789495547840], **{'doc': u'', 'properties': (), 'name': 'service_restriction32'})
    option_139789495548112 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495548384 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495548656 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495548928 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495553112 = SymLinkOption(**{'name': 'name', "opt": option_139789497637072})
    option_139789495553672 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497637344})
    option_139789495549200 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495549472 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495558224 = UnicodeOption(**{'default': u'___auto_sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495558496 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789495534608 = OptionDescription(children=[option_139789495548112, option_139789495548384, option_139789495548656, option_139789495548928, option_139789495553112, option_139789495553672, option_139789495549200, option_139789495549472, option_139789495558224, option_139789495558496], **{'doc': u'', 'properties': (), 'name': 'service_restriction33'})
    option_139789495558768 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495559040 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495559312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495559584 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495556928 = SymLinkOption(**{'name': 'name', "opt": option_139789497637072})
    option_139789495557488 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497637344})
    option_139789495559856 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495560128 = UnicodeOption(**{'default': u'z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495560400 = UnicodeOption(**{'default': u'___auto_z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495560672 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495534928 = OptionDescription(children=[option_139789495558768, option_139789495559040, option_139789495559312, option_139789495559584, option_139789495556928, option_139789495557488, option_139789495559856, option_139789495560128, option_139789495560400, option_139789495560672], **{'doc': u'', 'properties': (), 'name': 'service_restriction34'})
    option_139789495560944 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495561216 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495561488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495561760 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495564840 = SymLinkOption(**{'name': 'name', "opt": option_139789497714896})
    option_139789495565400 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497715168})
    option_139789495566416 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495566688 = UnicodeOption(**{'default': u'ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495566960 = UnicodeOption(**{'default': u'___auto_ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495567232 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964480}]})
    option_139789495535248 = OptionDescription(children=[option_139789495560944, option_139789495561216, option_139789495561488, option_139789495561760, option_139789495564840, option_139789495565400, option_139789495566416, option_139789495566688, option_139789495566960, option_139789495567232], **{'doc': u'', 'properties': (), 'name': 'service_restriction35'})
    option_139789495567504 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495567776 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495568048 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495568320 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495572752 = SymLinkOption(**{'name': 'name', "opt": option_139789497714896})
    option_139789495573312 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497715168})
    option_139789495568592 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495568864 = UnicodeOption(**{'default': u'lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495569136 = UnicodeOption(**{'default': u'___auto_lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495569408 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495535568 = OptionDescription(children=[option_139789495567504, option_139789495567776, option_139789495568048, option_139789495568320, option_139789495572752, option_139789495573312, option_139789495568592, option_139789495568864, option_139789495569136, option_139789495569408], **{'doc': u'', 'properties': (), 'name': 'service_restriction36'})
    option_139789495569680 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495569952 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495578704 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495578976 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495576568 = SymLinkOption(**{'name': 'name', "opt": option_139789497714896})
    option_139789495577128 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497715168})
    option_139789495579248 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495579520 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495579792 = UnicodeOption(**{'default': u'___auto_sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495580064 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789495535888 = OptionDescription(children=[option_139789495569680, option_139789495569952, option_139789495578704, option_139789495578976, option_139789495576568, option_139789495577128, option_139789495579248, option_139789495579520, option_139789495579792, option_139789495580064], **{'doc': u'', 'properties': (), 'name': 'service_restriction37'})
    option_139789495580336 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495580608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495580880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495581152 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495588576 = SymLinkOption(**{'name': 'name', "opt": option_139789497714896})
    option_139789495589136 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497715168})
    option_139789495581424 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495581696 = UnicodeOption(**{'default': u'z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495581968 = UnicodeOption(**{'default': u'___auto_z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495582240 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495536208 = OptionDescription(children=[option_139789495580336, option_139789495580608, option_139789495580880, option_139789495581152, option_139789495588576, option_139789495589136, option_139789495581424, option_139789495581696, option_139789495581968, option_139789495582240], **{'doc': u'', 'properties': (), 'name': 'service_restriction38'})
    option_139789495595088 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495595360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495595632 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495595904 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495592392 = SymLinkOption(**{'name': 'name', "opt": option_139789497769504})
    option_139789495592952 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497782352})
    option_139789495596176 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495596448 = UnicodeOption(**{'default': u'ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495596720 = UnicodeOption(**{'default': u'___auto_ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495596992 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964480}]})
    option_139789495536528 = OptionDescription(children=[option_139789495595088, option_139789495595360, option_139789495595632, option_139789495595904, option_139789495592392, option_139789495592952, option_139789495596176, option_139789495596448, option_139789495596720, option_139789495596992], **{'doc': u'', 'properties': (), 'name': 'service_restriction39'})
    option_139789495597264 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495597536 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495597808 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495598080 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495600304 = SymLinkOption(**{'name': 'name', "opt": option_139789497769504})
    option_139789495600864 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497782352})
    option_139789495598352 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495598624 = UnicodeOption(**{'default': u'lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495603280 = UnicodeOption(**{'default': u'___auto_lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495603552 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495536848 = OptionDescription(children=[option_139789495597264, option_139789495597536, option_139789495597808, option_139789495598080, option_139789495600304, option_139789495600864, option_139789495598352, option_139789495598624, option_139789495603280, option_139789495603552], **{'doc': u'', 'properties': (), 'name': 'service_restriction40'})
    option_139789495603824 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495604096 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495604368 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495604640 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495608216 = SymLinkOption(**{'name': 'name', "opt": option_139789497769504})
    option_139789495608776 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497782352})
    option_139789495604912 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495605184 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495605456 = UnicodeOption(**{'default': u'___auto_sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495605728 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789495537168 = OptionDescription(children=[option_139789495603824, option_139789495604096, option_139789495604368, option_139789495604640, option_139789495608216, option_139789495608776, option_139789495604912, option_139789495605184, option_139789495605456, option_139789495605728], **{'doc': u'', 'properties': (), 'name': 'service_restriction41'})
    option_139789495606000 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495606272 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495606544 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495606816 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495612032 = SymLinkOption(**{'name': 'name', "opt": option_139789497769504})
    option_139789495612592 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497782352})
    option_139789495615568 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495615840 = UnicodeOption(**{'default': u'z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495616112 = UnicodeOption(**{'default': u'___auto_z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495616384 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495619664 = OptionDescription(children=[option_139789495606000, option_139789495606272, option_139789495606544, option_139789495606816, option_139789495612032, option_139789495612592, option_139789495615568, option_139789495615840, option_139789495616112, option_139789495616384], **{'doc': u'', 'properties': (), 'name': 'service_restriction42'})
    option_139789495616656 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495616928 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495617200 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495617472 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495624040 = SymLinkOption(**{'name': 'name', "opt": option_139789497324688})
    option_139789495624600 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497324960})
    option_139789495617744 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495618016 = UnicodeOption(**{'default': u'ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495618288 = UnicodeOption(**{'default': u'___auto_ead_web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495618560 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964480}]})
    option_139789495619984 = OptionDescription(children=[option_139789495616656, option_139789495616928, option_139789495617200, option_139789495617472, option_139789495624040, option_139789495624600, option_139789495617744, option_139789495618016, option_139789495618288, option_139789495618560], **{'doc': u'', 'properties': (), 'name': 'service_restriction43'})
    option_139789495618832 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495619104 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495627856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495628128 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495631952 = SymLinkOption(**{'name': 'name', "opt": option_139789497324688})
    option_139789495632512 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497324960})
    option_139789495628400 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495628672 = UnicodeOption(**{'default': u'lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495628944 = UnicodeOption(**{'default': u'___auto_lightsquid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495629216 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495620304 = OptionDescription(children=[option_139789495618832, option_139789495619104, option_139789495627856, option_139789495628128, option_139789495631952, option_139789495632512, option_139789495628400, option_139789495628672, option_139789495628944, option_139789495629216], **{'doc': u'', 'properties': (), 'name': 'service_restriction44'})
    option_139789495629488 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495629760 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495630032 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495630304 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495635592 = SymLinkOption(**{'name': 'name', "opt": option_139789497324688})
    option_139789495640424 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497324960})
    option_139789495630576 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495630848 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495631120 = UnicodeOption(**{'default': u'___auto_sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495631392 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789495620624 = OptionDescription(children=[option_139789495629488, option_139789495629760, option_139789495630032, option_139789495630304, option_139789495635592, option_139789495640424, option_139789495630576, option_139789495630848, option_139789495631120, option_139789495631392], **{'doc': u'', 'properties': (), 'name': 'service_restriction45'})
    option_139789495648336 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495648608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495648880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495649152 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495643504 = SymLinkOption(**{'name': 'name', "opt": option_139789497324688})
    option_139789495652432 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497324960})
    option_139789495649424 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495649696 = UnicodeOption(**{'default': u'z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495649968 = UnicodeOption(**{'default': u'___auto_z_stats', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495650240 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495620944 = OptionDescription(children=[option_139789495648336, option_139789495648608, option_139789495648880, option_139789495649152, option_139789495643504, option_139789495652432, option_139789495649424, option_139789495649696, option_139789495649968, option_139789495650240], **{'doc': u'', 'properties': (), 'name': 'service_restriction46'})
    option_139789495650512 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495650784 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495651056 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495655232 = SymLinkOption(**{'name': 'interface', "opt": option_139789497237024})
    option_139789495655792 = SymLinkOption(**{'name': 'name', "opt": option_139789497236480})
    option_139789495656528 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497236752})
    option_139789495651328 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495651600 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495651872 = UnicodeOption(**{'default': u'___auto_mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495660624 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971312}]})
    option_139789495621264 = OptionDescription(children=[option_139789495650512, option_139789495650784, option_139789495651056, option_139789495655232, option_139789495655792, option_139789495656528, option_139789495651328, option_139789495651600, option_139789495651872, option_139789495660624], **{'doc': u'', 'properties': (), 'name': 'service_restriction47'})
    option_139789495660896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495661168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495661440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495661712 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495659608 = SymLinkOption(**{'name': 'name', "opt": option_139789496793296})
    option_139789495661984 = NetmaskOption(**{'default': u'255.255.255.255', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495662256 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495662528 = UnicodeOption(**{'default': u'bareos_sd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495662800 = UnicodeOption(**{'default': u'___auto_bareos_sd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495663072 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971040}]})
    option_139789495621584 = OptionDescription(children=[option_139789495660896, option_139789495661168, option_139789495661440, option_139789495661712, option_139789495659608, option_139789495661984, option_139789495662256, option_139789495662528, option_139789495662800, option_139789495663072], **{'doc': u'', 'properties': (), 'name': 'service_restriction48'})
    option_139789495663344 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495663616 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495663888 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495664160 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495667240 = SymLinkOption(**{'name': 'name', "opt": option_139789497539312})
    option_139789495667800 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497539040})
    option_139789495668816 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495669088 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495669360 = UnicodeOption(**{'default': u'proxy_eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495669632 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789495621904 = OptionDescription(children=[option_139789495663344, option_139789495663616, option_139789495663888, option_139789495664160, option_139789495667240, option_139789495667800, option_139789495668816, option_139789495669088, option_139789495669360, option_139789495669632], **{'doc': u'', 'properties': (), 'name': 'service_restriction49'})
    option_139789495669904 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495670176 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495670448 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495670720 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495675152 = SymLinkOption(**{'name': 'name', "opt": option_139789497539312})
    option_139789495675712 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497539040})
    option_139789495670992 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495671264 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495671536 = UnicodeOption(**{'default': u'cntlm_eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495671808 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': ('disabled',), 'name': 'activate'})
    option_139789495622224 = OptionDescription(children=[option_139789495669904, option_139789495670176, option_139789495670448, option_139789495670720, option_139789495675152, option_139789495675712, option_139789495670992, option_139789495671264, option_139789495671536, option_139789495671808], **{'doc': u'', 'properties': (), 'name': 'service_restriction50'})
    option_139789495672080 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495672352 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495681104 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495681376 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495678968 = SymLinkOption(**{'name': 'name', "opt": option_139789497602672})
    option_139789495679528 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497602944})
    option_139789495681648 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495681920 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495682192 = UnicodeOption(**{'default': u'proxy_eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495682464 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789495622544 = OptionDescription(children=[option_139789495672080, option_139789495672352, option_139789495681104, option_139789495681376, option_139789495678968, option_139789495679528, option_139789495681648, option_139789495681920, option_139789495682192, option_139789495682464], **{'doc': u'', 'properties': (), 'name': 'service_restriction51'})
    option_139789495682736 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495683008 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495683280 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495683552 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495686880 = SymLinkOption(**{'name': 'name', "opt": option_139789497602672})
    option_139789495687440 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497602944})
    option_139789495683824 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495684096 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495684368 = UnicodeOption(**{'default': u'cntlm_eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495684640 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': ('disabled',), 'name': 'activate'})
    option_139789495622864 = OptionDescription(children=[option_139789495682736, option_139789495683008, option_139789495683280, option_139789495683552, option_139789495686880, option_139789495687440, option_139789495683824, option_139789495684096, option_139789495684368, option_139789495684640], **{'doc': u'', 'properties': (), 'name': 'service_restriction52'})
    option_139789495169104 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495169376 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495169648 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495169920 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495690696 = SymLinkOption(**{'name': 'name', "opt": option_139789497604576})
    option_139789495691256 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497604304})
    option_139789495170192 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495170464 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495170736 = UnicodeOption(**{'default': u'___auto_squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495171008 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495623184 = OptionDescription(children=[option_139789495169104, option_139789495169376, option_139789495169648, option_139789495169920, option_139789495690696, option_139789495691256, option_139789495170192, option_139789495170464, option_139789495170736, option_139789495171008], **{'doc': u'', 'properties': (), 'name': 'service_restriction53'})
    option_139789495171280 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495171552 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495171824 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495172096 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495174320 = SymLinkOption(**{'name': 'name', "opt": option_139789497604576})
    option_139789495174880 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497604304})
    option_139789495172368 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495172640 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495177296 = UnicodeOption(**{'default': u'___auto_eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495177568 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495181392 = OptionDescription(children=[option_139789495171280, option_139789495171552, option_139789495171824, option_139789495172096, option_139789495174320, option_139789495174880, option_139789495172368, option_139789495172640, option_139789495177296, option_139789495177568], **{'doc': u'', 'properties': (), 'name': 'service_restriction54'})
    option_139789495177840 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495178112 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495178384 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495178656 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495186328 = SymLinkOption(**{'name': 'name', "opt": option_139789497694688})
    option_139789495186888 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497694416})
    option_139789495178928 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495179200 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495179472 = UnicodeOption(**{'default': u'___auto_squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495179744 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495181712 = OptionDescription(children=[option_139789495177840, option_139789495178112, option_139789495178384, option_139789495178656, option_139789495186328, option_139789495186888, option_139789495178928, option_139789495179200, option_139789495179472, option_139789495179744], **{'doc': u'', 'properties': (), 'name': 'service_restriction55'})
    option_139789495180016 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495180288 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495180560 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495180832 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495190144 = SymLinkOption(**{'name': 'name', "opt": option_139789497694688})
    option_139789495190704 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497694416})
    option_139789495193680 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495193952 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495194224 = UnicodeOption(**{'default': u'___auto_eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495194496 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495182032 = OptionDescription(children=[option_139789495180016, option_139789495180288, option_139789495180560, option_139789495180832, option_139789495190144, option_139789495190704, option_139789495193680, option_139789495193952, option_139789495194224, option_139789495194496], **{'doc': u'', 'properties': (), 'name': 'service_restriction56'})
    option_139789495194768 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495195040 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495195312 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495195584 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495198056 = SymLinkOption(**{'name': 'name', "opt": option_139789497765968})
    option_139789495198616 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497749024})
    option_139789495195856 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495196128 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495196400 = UnicodeOption(**{'default': u'___auto_squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495196672 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495182352 = OptionDescription(children=[option_139789495194768, option_139789495195040, option_139789495195312, option_139789495195584, option_139789495198056, option_139789495198616, option_139789495195856, option_139789495196128, option_139789495196400, option_139789495196672], **{'doc': u'', 'properties': (), 'name': 'service_restriction57'})
    option_139789495196944 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495197216 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495201872 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495202144 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495205968 = SymLinkOption(**{'name': 'name', "opt": option_139789497765968})
    option_139789495206528 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497749024})
    option_139789495202416 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495202688 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495202960 = UnicodeOption(**{'default': u'___auto_eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495203232 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495182672 = OptionDescription(children=[option_139789495196944, option_139789495197216, option_139789495201872, option_139789495202144, option_139789495205968, option_139789495206528, option_139789495202416, option_139789495202688, option_139789495202960, option_139789495203232], **{'doc': u'', 'properties': (), 'name': 'service_restriction58'})
    option_139789495203504 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495203776 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495204048 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495204320 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495209608 = SymLinkOption(**{'name': 'name', "opt": option_139789497300384})
    option_139789495210344 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497300112})
    option_139789495204592 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495204864 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495205136 = UnicodeOption(**{'default': u'___auto_squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495205408 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495182992 = OptionDescription(children=[option_139789495203504, option_139789495203776, option_139789495204048, option_139789495204320, option_139789495209608, option_139789495210344, option_139789495204592, option_139789495204864, option_139789495205136, option_139789495205408], **{'doc': u'', 'properties': (), 'name': 'service_restriction59'})
    option_139789495218256 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495218528 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495218800 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495219072 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495213424 = SymLinkOption(**{'name': 'name', "opt": option_139789497300384})
    option_139789495222352 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497300112})
    option_139789495219344 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495219616 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495219888 = UnicodeOption(**{'default': u'___auto_eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495220160 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495183312 = OptionDescription(children=[option_139789495218256, option_139789495218528, option_139789495218800, option_139789495219072, option_139789495213424, option_139789495222352, option_139789495219344, option_139789495219616, option_139789495219888, option_139789495220160], **{'doc': u'', 'properties': (), 'name': 'service_restriction60'})
    option_139789495220432 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495220704 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495220976 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495221248 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495225432 = SymLinkOption(**{'name': 'name', "opt": option_139789497593936})
    option_139789495225992 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497576992})
    option_139789495221520 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495221792 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495230544 = UnicodeOption(**{'default': u'proxy_eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495230816 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789495183632 = OptionDescription(children=[option_139789495220432, option_139789495220704, option_139789495220976, option_139789495221248, option_139789495225432, option_139789495225992, option_139789495221520, option_139789495221792, option_139789495230544, option_139789495230816], **{'doc': u'', 'properties': (), 'name': 'service_restriction61'})
    option_139789495231088 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495231360 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495231632 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495231904 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495229248 = SymLinkOption(**{'name': 'name', "opt": option_139789497593936})
    option_139789495229808 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497576992})
    option_139789495232176 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495232448 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495232720 = UnicodeOption(**{'default': u'cntlm_eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495232992 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': ('disabled',), 'name': 'activate'})
    option_139789495183952 = OptionDescription(children=[option_139789495231088, option_139789495231360, option_139789495231632, option_139789495231904, option_139789495229248, option_139789495229808, option_139789495232176, option_139789495232448, option_139789495232720, option_139789495232992], **{'doc': u'', 'properties': (), 'name': 'service_restriction62'})
    option_139789495233264 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495233536 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495233808 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495234080 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495237160 = SymLinkOption(**{'name': 'name', "opt": option_139789497658912})
    option_139789495237720 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497658640})
    option_139789495238736 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495239008 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495239280 = UnicodeOption(**{'default': u'___auto_squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495239552 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495184272 = OptionDescription(children=[option_139789495233264, option_139789495233536, option_139789495233808, option_139789495234080, option_139789495237160, option_139789495237720, option_139789495238736, option_139789495239008, option_139789495239280, option_139789495239552], **{'doc': u'', 'properties': (), 'name': 'service_restriction63'})
    option_139789495239824 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495240096 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495240368 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495240640 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495245072 = SymLinkOption(**{'name': 'name', "opt": option_139789497658912})
    option_139789495245632 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497658640})
    option_139789495240912 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495241184 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495241456 = UnicodeOption(**{'default': u'___auto_eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495241728 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495184592 = OptionDescription(children=[option_139789495239824, option_139789495240096, option_139789495240368, option_139789495240640, option_139789495245072, option_139789495245632, option_139789495240912, option_139789495241184, option_139789495241456, option_139789495241728], **{'doc': u'', 'properties': (), 'name': 'service_restriction64'})
    option_139789495242000 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495242272 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495251024 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495251296 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495248888 = SymLinkOption(**{'name': 'name', "opt": option_139789497732096})
    option_139789495249448 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497731824})
    option_139789495251568 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495251840 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495252112 = UnicodeOption(**{'default': u'___auto_squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495252384 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495184912 = OptionDescription(children=[option_139789495242000, option_139789495242272, option_139789495251024, option_139789495251296, option_139789495248888, option_139789495249448, option_139789495251568, option_139789495251840, option_139789495252112, option_139789495252384], **{'doc': u'', 'properties': (), 'name': 'service_restriction65'})
    option_139789495252656 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495252928 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495253200 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495253472 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495256800 = SymLinkOption(**{'name': 'name', "opt": option_139789497732096})
    option_139789495257360 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497731824})
    option_139789495253744 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495254016 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495254288 = UnicodeOption(**{'default': u'___auto_eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495254560 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495259216 = OptionDescription(children=[option_139789495252656, option_139789495252928, option_139789495253200, option_139789495253472, option_139789495256800, option_139789495257360, option_139789495253744, option_139789495254016, option_139789495254288, option_139789495254560], **{'doc': u'', 'properties': (), 'name': 'service_restriction66'})
    option_139789495267408 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495267680 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495267952 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495268224 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495264712 = SymLinkOption(**{'name': 'name', "opt": option_139789497270896})
    option_139789495265272 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497270624})
    option_139789495268496 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495268768 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495269040 = UnicodeOption(**{'default': u'___auto_squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495269312 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495259536 = OptionDescription(children=[option_139789495267408, option_139789495267680, option_139789495267952, option_139789495268224, option_139789495264712, option_139789495265272, option_139789495268496, option_139789495268768, option_139789495269040, option_139789495269312], **{'doc': u'', 'properties': (), 'name': 'service_restriction67'})
    option_139789495269584 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495269856 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495270128 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495270400 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495272624 = SymLinkOption(**{'name': 'name', "opt": option_139789497270896})
    option_139789495273184 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497270624})
    option_139789495270672 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495270944 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495275600 = UnicodeOption(**{'default': u'___auto_eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495275872 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495259856 = OptionDescription(children=[option_139789495269584, option_139789495269856, option_139789495270128, option_139789495270400, option_139789495272624, option_139789495273184, option_139789495270672, option_139789495270944, option_139789495275600, option_139789495275872], **{'doc': u'', 'properties': (), 'name': 'service_restriction68'})
    option_139789495276144 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495276416 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495276688 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495276960 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495280536 = SymLinkOption(**{'name': 'name', "opt": option_139789497337792})
    option_139789495281096 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497337520})
    option_139789495277232 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495277504 = UnicodeOption(**{'default': u'squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495277776 = UnicodeOption(**{'default': u'___auto_squid3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495278048 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495260176 = OptionDescription(children=[option_139789495276144, option_139789495276416, option_139789495276688, option_139789495276960, option_139789495280536, option_139789495281096, option_139789495277232, option_139789495277504, option_139789495277776, option_139789495278048], **{'doc': u'', 'properties': (), 'name': 'service_restriction69'})
    option_139789495278320 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495278592 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495278864 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495279136 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495284352 = SymLinkOption(**{'name': 'name', "opt": option_139789497337792})
    option_139789495284912 = SymLinkOption(**{'name': 'netmask', "opt": option_139789497337520})
    option_139789495287888 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495288160 = UnicodeOption(**{'default': u'eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495288432 = UnicodeOption(**{'default': u'___auto_eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495288704 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495260496 = OptionDescription(children=[option_139789495278320, option_139789495278592, option_139789495278864, option_139789495279136, option_139789495284352, option_139789495284912, option_139789495287888, option_139789495288160, option_139789495288432, option_139789495288704], **{'doc': u'', 'properties': (), 'name': 'service_restriction70'})
    option_139789495288976 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495289248 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495289520 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495289792 = UnicodeOption(**{'default': u'auto', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interface'})
    option_139789495292264 = SymLinkOption(**{'name': 'name', "opt": option_139789496486368})
    option_139789495290064 = NetmaskOption(**{'default': u'255.255.255.255', 'doc': u'', 'multi': False, 'properties': (), 'name': 'netmask'})
    option_139789495290336 = UnicodeOption(**{'default': u'ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'node_name'})
    option_139789495290608 = UnicodeOption(**{'default': u'squid3sibling', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service'})
    option_139789495290880 = UnicodeOption(**{'default': u'___auto_squid3sibling', 'doc': u'', 'multi': False, 'properties': (), 'name': 'service_restrictionlist'})
    option_139789495291152 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495260816 = OptionDescription(children=[option_139789495288976, option_139789495289248, option_139789495289520, option_139789495289792, option_139789495292264, option_139789495290064, option_139789495290336, option_139789495290608, option_139789495290880, option_139789495291152], **{'doc': u'', 'properties': (), 'name': 'service_restriction71'})
    option_139789495261136 = OptionDescription(children=[option_139789495822288, option_139789495822608, option_139789495822928, option_139789495823248, option_139789495823568, option_139789495823888, option_139789495894096, option_139789495894416, option_139789495894736, option_139789495895056, option_139789495895376, option_139789495895696, option_139789495896016, option_139789495896336, option_139789495896656, option_139789495896976, option_139789495897296, option_139789495897616, option_139789495451728, option_139789495452048, option_139789495452368, option_139789495452688, option_139789495453008, option_139789495453328, option_139789495453648, option_139789495453968, option_139789495454288, option_139789495454608, option_139789495454928, option_139789495455248, option_139789495533648, option_139789495533968, option_139789495534288, option_139789495534608, option_139789495534928, option_139789495535248, option_139789495535568, option_139789495535888, option_139789495536208, option_139789495536528, option_139789495536848, option_139789495537168, option_139789495619664, option_139789495619984, option_139789495620304, option_139789495620624, option_139789495620944, option_139789495621264, option_139789495621584, option_139789495621904, option_139789495622224, option_139789495622544, option_139789495622864, option_139789495623184, option_139789495181392, option_139789495181712, option_139789495182032, option_139789495182352, option_139789495182672, option_139789495182992, option_139789495183312, option_139789495183632, option_139789495183952, option_139789495184272, option_139789495184592, option_139789495184912, option_139789495259216, option_139789495259536, option_139789495259856, option_139789495260176, option_139789495260496, option_139789495260816], **{'doc': u'', 'properties': (), 'name': 'service_restrictions'})
    option_139789495294784 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495291424 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495296080 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495296352 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495300456 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495301016 = SymLinkOption(**{'name': 'ip', "opt": option_139789497852256})
    option_139789495296624 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495301856 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495296896 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495297168 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495297440 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495261456 = OptionDescription(children=[option_139789495294784, option_139789495291424, option_139789495296080, option_139789495296352, option_139789495300456, option_139789495301016, option_139789495296624, option_139789495301856, option_139789495296896, option_139789495297168, option_139789495297440], **{'doc': u'', 'properties': (), 'name': 'interface0'})
    option_139789495303536 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495297712 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495297984 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495298256 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495305112 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495305672 = SymLinkOption(**{'name': 'ip', "opt": option_139789497853072})
    option_139789495298528 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495306512 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495298800 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495299072 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495299344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495261776 = OptionDescription(children=[option_139789495303536, option_139789495297712, option_139789495297984, option_139789495298256, option_139789495305112, option_139789495305672, option_139789495298528, option_139789495306512, option_139789495298800, option_139789495299072, option_139789495299344], **{'doc': u'', 'properties': (), 'name': 'interface1'})
    option_139789495308368 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495299616 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495312464 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495312736 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495309768 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495310328 = SymLinkOption(**{'name': 'ip', "opt": option_139789497853888})
    option_139789495313008 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495311168 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495313280 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495313552 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495313824 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495262096 = OptionDescription(children=[option_139789495308368, option_139789495299616, option_139789495312464, option_139789495312736, option_139789495309768, option_139789495310328, option_139789495313008, option_139789495311168, option_139789495313280, option_139789495313552, option_139789495313824], **{'doc': u'', 'properties': (), 'name': 'interface2'})
    option_139789495317120 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495314096 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495314368 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495314640 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495318520 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495319080 = SymLinkOption(**{'name': 'ip', "opt": option_139789497854704})
    option_139789495314912 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495319920 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495315184 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495315456 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495315728 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495262416 = OptionDescription(children=[option_139789495317120, option_139789495314096, option_139789495314368, option_139789495314640, option_139789495318520, option_139789495319080, option_139789495314912, option_139789495319920, option_139789495315184, option_139789495315456, option_139789495315728], **{'doc': u'', 'properties': (), 'name': 'interface3'})
    option_139789495321776 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495316000 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495324752 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495325024 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495323176 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495323736 = SymLinkOption(**{'name': 'ip', "opt": option_139789497855520})
    option_139789495325296 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495328848 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495325568 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495325840 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495326112 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495262736 = OptionDescription(children=[option_139789495321776, option_139789495316000, option_139789495324752, option_139789495325024, option_139789495323176, option_139789495323736, option_139789495325296, option_139789495328848, option_139789495325568, option_139789495325840, option_139789495326112], **{'doc': u'', 'properties': (), 'name': 'interface4'})
    option_139789495330528 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495326384 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495326656 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495326928 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495331928 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495332488 = SymLinkOption(**{'name': 'ip', "opt": option_139789497864816})
    option_139789495327200 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495333504 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495327472 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495327744 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495328016 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495337040 = OptionDescription(children=[option_139789495330528, option_139789495326384, option_139789495326656, option_139789495326928, option_139789495331928, option_139789495332488, option_139789495327200, option_139789495333504, option_139789495327472, option_139789495327744, option_139789495328016], **{'doc': u'', 'properties': (), 'name': 'interface5'})
    option_139789495335184 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495328288 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495341136 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495341408 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495336584 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495345512 = SymLinkOption(**{'name': 'ip', "opt": option_139789497865632})
    option_139789495341680 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495346352 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495341952 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495342224 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495342496 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495337360 = OptionDescription(children=[option_139789495335184, option_139789495328288, option_139789495341136, option_139789495341408, option_139789495336584, option_139789495345512, option_139789495341680, option_139789495346352, option_139789495341952, option_139789495342224, option_139789495342496], **{'doc': u'', 'properties': (), 'name': 'interface6'})
    option_139789495348032 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495342768 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495343040 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495343312 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495353704 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495354264 = SymLinkOption(**{'name': 'ip', "opt": option_139789497866448})
    option_139789495343584 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495355104 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495343856 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495344128 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495344400 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495337680 = OptionDescription(children=[option_139789495348032, option_139789495342768, option_139789495343040, option_139789495343312, option_139789495353704, option_139789495354264, option_139789495343584, option_139789495355104, option_139789495343856, option_139789495344128, option_139789495344400], **{'doc': u'', 'properties': (), 'name': 'interface7'})
    option_139789495356784 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495344672 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495361616 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495361888 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495358360 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495358920 = SymLinkOption(**{'name': 'ip', "opt": option_139789497867264})
    option_139789495362160 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495359760 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495362432 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495362704 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495362976 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495338000 = OptionDescription(children=[option_139789495356784, option_139789495344672, option_139789495361616, option_139789495361888, option_139789495358360, option_139789495358920, option_139789495362160, option_139789495359760, option_139789495362432, option_139789495362704, option_139789495362976], **{'doc': u'', 'properties': (), 'name': 'interface8'})
    option_139789495365712 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495363248 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495363520 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495363792 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495367112 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495367672 = SymLinkOption(**{'name': 'ip', "opt": option_139789497872464})
    option_139789495364064 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495368512 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495364336 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495364608 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495364880 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495338320 = OptionDescription(children=[option_139789495365712, option_139789495363248, option_139789495363520, option_139789495363792, option_139789495367112, option_139789495367672, option_139789495364064, option_139789495368512, option_139789495364336, option_139789495364608, option_139789495364880], **{'doc': u'', 'properties': (), 'name': 'interface9'})
    option_139789495370368 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495365152 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495373904 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495374176 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495371768 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495372328 = SymLinkOption(**{'name': 'ip', "opt": option_139789497873280})
    option_139789495374448 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495373168 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495374720 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495374992 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495375264 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495338640 = OptionDescription(children=[option_139789495370368, option_139789495365152, option_139789495373904, option_139789495374176, option_139789495371768, option_139789495372328, option_139789495374448, option_139789495373168, option_139789495374720, option_139789495374992, option_139789495375264], **{'doc': u'', 'properties': (), 'name': 'interface10'})
    option_139789495379120 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495375536 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495375808 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495376080 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495380520 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495381080 = SymLinkOption(**{'name': 'ip', "opt": option_139789497874096})
    option_139789495376352 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495382096 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495376624 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495376896 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495377168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495338960 = OptionDescription(children=[option_139789495379120, option_139789495375536, option_139789495375808, option_139789495376080, option_139789495380520, option_139789495381080, option_139789495376352, option_139789495382096, option_139789495376624, option_139789495376896, option_139789495377168], **{'doc': u'', 'properties': (), 'name': 'interface11'})
    option_139789495383776 = SymLinkOption(**{'name': 'bcast', "opt": option_139789498664896})
    option_139789495377440 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495386192 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495386464 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495385176 = SymLinkOption(**{'name': 'gateway', "opt": option_139789498664080})
    option_139789495385736 = SymLinkOption(**{'name': 'ip', "opt": option_139789497874912})
    option_139789495386736 = UnicodeOption(**{'default': u'br0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495390848 = SymLinkOption(**{'name': 'mask', "opt": option_139789498664352})
    option_139789495387008 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'bridge', 'doc': u'', 'properties': ()})
    option_139789495387280 = UnicodeOption(**{'default': u'containers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495387552 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495339280 = OptionDescription(children=[option_139789495383776, option_139789495377440, option_139789495386192, option_139789495386464, option_139789495385176, option_139789495385736, option_139789495386736, option_139789495390848, option_139789495387008, option_139789495387280, option_139789495387552], **{'doc': u'', 'properties': (), 'name': 'interface12'})
    option_139789495392528 = SymLinkOption(**{'name': 'bcast', "opt": option_139789497693600})
    option_139789495387824 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495388096 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495388368 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495393928 = SymLinkOption(**{'name': 'ip', "opt": option_139789497693056})
    option_139789495388640 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495394944 = SymLinkOption(**{'name': 'mask', "opt": option_139789497693328})
    option_139789495388912 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'macvlan', 'doc': u'', 'properties': ()})
    option_139789495389184 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495389456 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495339600 = OptionDescription(children=[option_139789495392528, option_139789495387824, option_139789495388096, option_139789495388368, option_139789495393928, option_139789495388640, option_139789495394944, option_139789495388912, option_139789495389184, option_139789495389456], **{'doc': u'', 'properties': (), 'name': 'interface13'})
    option_139789495396624 = SymLinkOption(**{'name': 'bcast', "opt": option_139789497692240})
    option_139789495389728 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495398480 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495398752 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495399024 = UnicodeOption(**{'default': u'eth1_proxy_link', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interfacelist'})
    option_139789495402576 = SymLinkOption(**{'name': 'ip', "opt": option_139789497675024})
    option_139789495399296 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495403416 = SymLinkOption(**{'name': 'mask', "opt": option_139789497675296})
    option_139789495399568 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'macvlan', 'doc': u'', 'properties': ()})
    option_139789495399840 = UnicodeOption(**{'default': u'eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495400112 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495339920 = OptionDescription(children=[option_139789495396624, option_139789495389728, option_139789495398480, option_139789495398752, option_139789495399024, option_139789495402576, option_139789495399296, option_139789495403416, option_139789495399568, option_139789495399840, option_139789495400112], **{'doc': u'', 'properties': (), 'name': 'interface14'})
    option_139789495405096 = SymLinkOption(**{'name': 'bcast', "opt": option_139789497602128})
    option_139789495400384 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495400656 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495400928 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495401200 = UnicodeOption(**{'default': u'eth0_proxy_link', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interfacelist'})
    option_139789495411048 = SymLinkOption(**{'name': 'ip', "opt": option_139789497597200})
    option_139789495401472 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495411888 = SymLinkOption(**{'name': 'mask', "opt": option_139789497597472})
    option_139789495401744 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'macvlan', 'doc': u'', 'properties': ()})
    option_139789495402016 = UnicodeOption(**{'default': u'eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495414864 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'5', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789495340240 = OptionDescription(children=[option_139789495405096, option_139789495400384, option_139789495400656, option_139789495400928, option_139789495401200, option_139789495411048, option_139789495401472, option_139789495411888, option_139789495401744, option_139789495402016, option_139789495414864], **{'doc': u'', 'properties': (), 'name': 'interface15'})
    option_139789495413568 = SymLinkOption(**{'name': 'bcast', "opt": option_139789497747664})
    option_139789495415136 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495415408 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495415680 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495415952 = UnicodeOption(**{'default': u'eth2_proxy_link', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interfacelist'})
    option_139789495419520 = SymLinkOption(**{'name': 'ip', "opt": option_139789497747120})
    option_139789495416224 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495420360 = SymLinkOption(**{'name': 'mask', "opt": option_139789497747392})
    option_139789495416496 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'macvlan', 'doc': u'', 'properties': ()})
    option_139789495416768 = UnicodeOption(**{'default': u'eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495417040 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789495340560 = OptionDescription(children=[option_139789495413568, option_139789495415136, option_139789495415408, option_139789495415680, option_139789495415952, option_139789495419520, option_139789495416224, option_139789495420360, option_139789495416496, option_139789495416768, option_139789495417040], **{'doc': u'', 'properties': (), 'name': 'interface16'})
    option_139789495422040 = SymLinkOption(**{'name': 'bcast', "opt": option_139789497273888})
    option_139789495417312 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495417584 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495417856 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495418128 = UnicodeOption(**{'default': u'eth3_proxy_link', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interfacelist'})
    option_139789495423896 = SymLinkOption(**{'name': 'ip', "opt": option_139789497273344})
    option_139789495418400 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789495424736 = SymLinkOption(**{'name': 'mask', "opt": option_139789497273616})
    option_139789495427152 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'macvlan', 'doc': u'', 'properties': ()})
    option_139789495427424 = UnicodeOption(**{'default': u'eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495427696 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789494906960 = OptionDescription(children=[option_139789495422040, option_139789495417312, option_139789495417584, option_139789495417856, option_139789495418128, option_139789495423896, option_139789495418400, option_139789495424736, option_139789495427152, option_139789495427424, option_139789495427696], **{'doc': u'', 'properties': (), 'name': 'interface17'})
    option_139789495426416 = SymLinkOption(**{'name': 'bcast', "opt": option_139789497353360})
    option_139789495427968 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495428240 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495428512 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495428784 = UnicodeOption(**{'default': u'eth4_proxy_link', 'doc': u'', 'multi': False, 'properties': (), 'name': 'interfacelist'})
    option_139789494912176 = SymLinkOption(**{'name': 'ip', "opt": option_139789497352816})
    option_139789495429056 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'linkto'})
    option_139789494913016 = SymLinkOption(**{'name': 'mask', "opt": option_139789497353088})
    option_139789495429328 = ChoiceOption(**{'multi': False, 'values': (u'bridge', u'macvlan'), 'name': 'method', 'default': 'macvlan', 'doc': u'', 'properties': ()})
    option_139789495429600 = UnicodeOption(**{'default': u'eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495429872 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'2', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'3', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}, {'expected': u'4', 'action': 'disabled', 'inverse': False, "option": option_139789497891840}]})
    option_139789494907280 = OptionDescription(children=[option_139789495426416, option_139789495427968, option_139789495428240, option_139789495428512, option_139789495428784, option_139789494912176, option_139789495429056, option_139789494913016, option_139789495429328, option_139789495429600, option_139789495429872], **{'doc': u'', 'properties': (), 'name': 'interface18'})
    option_139789494907600 = OptionDescription(children=[option_139789495261456, option_139789495261776, option_139789495262096, option_139789495262416, option_139789495262736, option_139789495337040, option_139789495337360, option_139789495337680, option_139789495338000, option_139789495338320, option_139789495338640, option_139789495338960, option_139789495339280, option_139789495339600, option_139789495339920, option_139789495340240, option_139789495340560, option_139789494906960, option_139789494907280], **{'doc': u'', 'properties': (), 'name': 'interfaces'})
    option_139789495430144 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495430416 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495430688 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494919248 = UnicodeOption(**{'default': u'eole-antivirus-container-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494907920 = OptionDescription(children=[option_139789495430144, option_139789495430416, option_139789495430688, option_139789494919248], **{'doc': u'', 'properties': (), 'name': 'package0'})
    option_139789494919520 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494919792 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494920064 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494920336 = UnicodeOption(**{'default': u'eole-antivirus-container-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494908240 = OptionDescription(children=[option_139789494919520, option_139789494919792, option_139789494920064, option_139789494920336], **{'doc': u'', 'properties': (), 'name': 'package1'})
    option_139789494920608 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494920880 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494921152 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494921424 = UnicodeOption(**{'default': u'eole-antivirus-container-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494908560 = OptionDescription(children=[option_139789494920608, option_139789494920880, option_139789494921152, option_139789494921424], **{'doc': u'', 'properties': (), 'name': 'package2'})
    option_139789494921696 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494921968 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494922240 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494922512 = UnicodeOption(**{'default': u'eole-antivirus-container-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494908880 = OptionDescription(children=[option_139789494921696, option_139789494921968, option_139789494922240, option_139789494922512], **{'doc': u'', 'properties': (), 'name': 'package3'})
    option_139789494922784 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494927440 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494927712 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494927984 = UnicodeOption(**{'default': u'eole-dhcp-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494909200 = OptionDescription(children=[option_139789494922784, option_139789494927440, option_139789494927712, option_139789494927984], **{'doc': u'', 'properties': (), 'name': 'package4'})
    option_139789494928256 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494928528 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494928800 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494929072 = UnicodeOption(**{'default': u'eole-fichier-primaire-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494909520 = OptionDescription(children=[option_139789494928256, option_139789494928528, option_139789494928800, option_139789494929072], **{'doc': u'', 'properties': (), 'name': 'package5'})
    option_139789494929344 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494929616 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494929888 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494930160 = UnicodeOption(**{'default': u'eole-fichier-common-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494909840 = OptionDescription(children=[option_139789494929344, option_139789494929616, option_139789494929888, option_139789494930160], **{'doc': u'', 'properties': (), 'name': 'package6'})
    option_139789494930432 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494930704 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494930976 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494935632 = UnicodeOption(**{'default': u'eole-web-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494910160 = OptionDescription(children=[option_139789494930432, option_139789494930704, option_139789494930976, option_139789494935632], **{'doc': u'', 'properties': (), 'name': 'package7'})
    option_139789494935904 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494936176 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494936448 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494936720 = UnicodeOption(**{'default': u'eole-bareoswebui-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494910480 = OptionDescription(children=[option_139789494935904, option_139789494936176, option_139789494936448, option_139789494936720], **{'doc': u'', 'properties': (), 'name': 'package8'})
    option_139789494936992 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494937264 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494937536 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494937808 = UnicodeOption(**{'default': u'eole-dns-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494943824 = OptionDescription(children=[option_139789494936992, option_139789494937264, option_139789494937536, option_139789494937808], **{'doc': u'', 'properties': (), 'name': 'package9'})
    option_139789494938080 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494938352 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494938624 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494938896 = UnicodeOption(**{'default': u'eole-mysql-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494944144 = OptionDescription(children=[option_139789494938080, option_139789494938352, option_139789494938624, option_139789494938896], **{'doc': u'', 'properties': (), 'name': 'package10'})
    option_139789494939168 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494952016 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494952288 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494952560 = UnicodeOption(**{'default': u'eole-spamassassin-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494944464 = OptionDescription(children=[option_139789494939168, option_139789494952016, option_139789494952288, option_139789494952560], **{'doc': u'', 'properties': (), 'name': 'package11'})
    option_139789494952832 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494953104 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494953376 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494953648 = UnicodeOption(**{'default': u'eole-annuaire-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494944784 = OptionDescription(children=[option_139789494952832, option_139789494953104, option_139789494953376, option_139789494953648], **{'doc': u'', 'properties': (), 'name': 'package12'})
    option_139789494953920 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494954192 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494954464 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494954736 = UnicodeOption(**{'default': u'eole-courier-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494945104 = OptionDescription(children=[option_139789494953920, option_139789494954192, option_139789494954464, option_139789494954736], **{'doc': u'', 'properties': (), 'name': 'package13'})
    option_139789494955008 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494955280 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494955552 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494964304 = UnicodeOption(**{'default': u'eole-phpmyadmin-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494945424 = OptionDescription(children=[option_139789494955008, option_139789494955280, option_139789494955552, option_139789494964304], **{'doc': u'', 'properties': (), 'name': 'package14'})
    option_139789494964576 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494964848 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494965120 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494965392 = UnicodeOption(**{'default': u'eole-proxy-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494945744 = OptionDescription(children=[option_139789494964576, option_139789494964848, option_139789494965120, option_139789494965392], **{'doc': u'', 'properties': (), 'name': 'package15'})
    option_139789494965664 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494965936 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494966208 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494966480 = UnicodeOption(**{'default': u'eole-cups-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494946064 = OptionDescription(children=[option_139789494965664, option_139789494965936, option_139789494966208, option_139789494966480], **{'doc': u'', 'properties': (), 'name': 'package16'})
    option_139789494966752 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494967024 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494967296 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494967568 = UnicodeOption(**{'default': u'eole-proftpd-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494946384 = OptionDescription(children=[option_139789494966752, option_139789494967024, option_139789494967296, option_139789494967568], **{'doc': u'', 'properties': (), 'name': 'package17'})
    option_139789494967840 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494972496 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494972768 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494973040 = UnicodeOption(**{'default': u'eole-sympa-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494946704 = OptionDescription(children=[option_139789494967840, option_139789494972496, option_139789494972768, option_139789494973040], **{'doc': u'', 'properties': (), 'name': 'package18'})
    option_139789494973312 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494973584 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494973856 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494974128 = UnicodeOption(**{'default': u'eole-controle-vnc-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494947024 = OptionDescription(children=[option_139789494973312, option_139789494973584, option_139789494973856, option_139789494974128], **{'doc': u'', 'properties': (), 'name': 'package19'})
    option_139789494974400 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494974672 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494974944 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494975216 = UnicodeOption(**{'default': u'eole-exim-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494947344 = OptionDescription(children=[option_139789494974400, option_139789494974672, option_139789494974944, option_139789494975216], **{'doc': u'', 'properties': (), 'name': 'package20'})
    option_139789494975488 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494975760 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494976032 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494980688 = UnicodeOption(**{'default': u'eole-exim-auth-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494984784 = OptionDescription(children=[option_139789494975488, option_139789494975760, option_139789494976032, option_139789494980688], **{'doc': u'', 'properties': (), 'name': 'package21'})
    option_139789494980960 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494981232 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494981504 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494981776 = UnicodeOption(**{'default': u'eole-scribe-backend', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494985104 = OptionDescription(children=[option_139789494980960, option_139789494981232, option_139789494981504, option_139789494981776], **{'doc': u'', 'properties': (), 'name': 'package22'})
    option_139789494982048 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494982320 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494982592 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494982864 = UnicodeOption(**{'default': u'eole-eoe-apps', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494985424 = OptionDescription(children=[option_139789494982048, option_139789494982320, option_139789494982592, option_139789494982864], **{'doc': u'', 'properties': (), 'name': 'package23'})
    option_139789494983136 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494983408 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494983680 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494983952 = UnicodeOption(**{'default': u'eole-eop-apps', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494985744 = OptionDescription(children=[option_139789494983136, option_139789494983408, option_139789494983680, option_139789494983952], **{'doc': u'', 'properties': (), 'name': 'package24'})
    option_139789494984224 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494992976 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494993248 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494993520 = UnicodeOption(**{'default': u'eole-websockify-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494986064 = OptionDescription(children=[option_139789494984224, option_139789494992976, option_139789494993248, option_139789494993520], **{'doc': u'', 'properties': (), 'name': 'package25'})
    option_139789494993792 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494994064 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494994336 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494994608 = UnicodeOption(**{'default': u'roundcube-apps', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494986384 = OptionDescription(children=[option_139789494993792, option_139789494994064, option_139789494994336, option_139789494994608], **{'doc': u'', 'properties': (), 'name': 'package26'})
    option_139789494994880 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494995152 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494995424 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494995696 = UnicodeOption(**{'default': u'eole-cntlm-pkg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494986704 = OptionDescription(children=[option_139789494994880, option_139789494995152, option_139789494995424, option_139789494995696], **{'doc': u'', 'properties': (), 'name': 'package27'})
    option_139789494995968 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494996240 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494996512 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495005264 = UnicodeOption(**{'default': u'libpam-cas-eole', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494987024 = OptionDescription(children=[option_139789494995968, option_139789494996240, option_139789494996512, option_139789495005264], **{'doc': u'', 'properties': (), 'name': 'package28'})
    option_139789495005536 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495005808 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495006080 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495006352 = UnicodeOption(**{'default': u'pydio-apps', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494987344 = OptionDescription(children=[option_139789495005536, option_139789495005808, option_139789495006080, option_139789495006352], **{'doc': u'', 'properties': (), 'name': 'package29'})
    option_139789494987664 = OptionDescription(children=[option_139789494907920, option_139789494908240, option_139789494908560, option_139789494908880, option_139789494909200, option_139789494909520, option_139789494909840, option_139789494910160, option_139789494910480, option_139789494943824, option_139789494944144, option_139789494944464, option_139789494944784, option_139789494945104, option_139789494945424, option_139789494945744, option_139789494946064, option_139789494946384, option_139789494946704, option_139789494947024, option_139789494947344, option_139789494984784, option_139789494985104, option_139789494985424, option_139789494985744, option_139789494986064, option_139789494986384, option_139789494986704, option_139789494987024, option_139789494987344], **{'doc': u'', 'properties': (), 'name': 'packages'})
    option_139789495006624 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495006896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495007168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495007440 = UnicodeOption(**{'default': u'/etc/eole/release', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495007712 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495007984 = UnicodeOption(**{'default': u'/etc/eole/release', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495008256 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495008528 = UnicodeOption(**{'default': u'/var/lib/creole/release', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495008800 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494987984 = OptionDescription(children=[option_139789495006624, option_139789495006896, option_139789495007168, option_139789495007440, option_139789495007712, option_139789495007984, option_139789495008256, option_139789495008528, option_139789495008800], **{'doc': u'', 'properties': (), 'name': 'file0'})
    option_139789495013456 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495013728 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495014000 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495014272 = UnicodeOption(**{'default': u'container_disabled', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495014544 = UnicodeOption(**{'default': u'/etc/eole/.VirtDisabled.lock', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495014816 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495015088 = UnicodeOption(**{'default': u'/etc/eole/.VirtDisabled.lock', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495015360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495015632 = UnicodeOption(**{'default': u'/var/lib/creole/container.lockfile', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495015904 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494988304 = OptionDescription(children=[option_139789495013456, option_139789495013728, option_139789495014000, option_139789495014272, option_139789495014544, option_139789495014816, option_139789495015088, option_139789495015360, option_139789495015632, option_139789495015904], **{'doc': u'', 'properties': (), 'name': 'file1'})
    option_139789495016176 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495016448 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495016720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495016992 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495025744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495026016 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495026288 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495026560 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495026832 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495029840 = OptionDescription(children=[option_139789495016176, option_139789495016448, option_139789495016720, option_139789495016992, option_139789495025744, option_139789495026016, option_139789495026288, option_139789495026560, option_139789495026832], **{'doc': u'', 'properties': (), 'name': 'file2'})
    option_139789495027104 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495027376 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495027648 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495027920 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495028192 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495028464 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495028736 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495029008 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495029280 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495038032 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495030160 = OptionDescription(children=[option_139789495027104, option_139789495027376, option_139789495027648, option_139789495027920, option_139789495028192, option_139789495028464, option_139789495028736, option_139789495029008, option_139789495029280, option_139789495038032], **{'doc': u'', 'properties': (), 'name': 'file3'})
    option_139789495038304 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495038576 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495038848 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495039120 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495039392 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495039664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495039936 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495040208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495040480 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495040752 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495030480 = OptionDescription(children=[option_139789495038304, option_139789495038576, option_139789495038848, option_139789495039120, option_139789495039392, option_139789495039664, option_139789495039936, option_139789495040208, option_139789495040480, option_139789495040752], **{'doc': u'', 'properties': (), 'name': 'file4'})
    option_139789495041024 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495041296 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495041568 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495050320 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495050592 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495050864 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495051136 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495051408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495051680 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495051952 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495030800 = OptionDescription(children=[option_139789495041024, option_139789495041296, option_139789495041568, option_139789495050320, option_139789495050592, option_139789495050864, option_139789495051136, option_139789495051408, option_139789495051680, option_139789495051952], **{'doc': u'', 'properties': (), 'name': 'file5'})
    option_139789495052224 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495052496 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495052768 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495053040 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495053312 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495053584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495053856 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495062608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495062880 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495063152 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495031120 = OptionDescription(children=[option_139789495052224, option_139789495052496, option_139789495052768, option_139789495053040, option_139789495053312, option_139789495053584, option_139789495053856, option_139789495062608, option_139789495062880, option_139789495063152], **{'doc': u'', 'properties': (), 'name': 'file6'})
    option_139789495063424 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495063696 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495063968 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495064240 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495064512 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495064784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495065056 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495065328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495065600 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495065872 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495031440 = OptionDescription(children=[option_139789495063424, option_139789495063696, option_139789495063968, option_139789495064240, option_139789495064512, option_139789495064784, option_139789495065056, option_139789495065328, option_139789495065600, option_139789495065872], **{'doc': u'', 'properties': (), 'name': 'file7'})
    option_139789495066144 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495070800 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495071072 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495071344 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495071616 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495071888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495072160 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495072432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495072704 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495072976 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495031760 = OptionDescription(children=[option_139789495066144, option_139789495070800, option_139789495071072, option_139789495071344, option_139789495071616, option_139789495071888, option_139789495072160, option_139789495072432, option_139789495072704, option_139789495072976], **{'doc': u'', 'properties': (), 'name': 'file8'})
    option_139789495073248 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495073520 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495073792 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495074064 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495074336 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495083088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495083360 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495083632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495083904 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495084176 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495032080 = OptionDescription(children=[option_139789495073248, option_139789495073520, option_139789495073792, option_139789495074064, option_139789495074336, option_139789495083088, option_139789495083360, option_139789495083632, option_139789495083904, option_139789495084176], **{'doc': u'', 'properties': (), 'name': 'file9'})
    option_139789495084448 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495084720 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495084992 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495085264 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495085536 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495085808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495086080 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495086352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495086624 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495091280 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495032400 = OptionDescription(children=[option_139789495084448, option_139789495084720, option_139789495084992, option_139789495085264, option_139789495085536, option_139789495085808, option_139789495086080, option_139789495086352, option_139789495086624, option_139789495091280], **{'doc': u'', 'properties': (), 'name': 'file10'})
    option_139789495091552 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495091824 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495092096 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495092368 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495092640 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495092912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495093184 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495093456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495093728 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495094000 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495032720 = OptionDescription(children=[option_139789495091552, option_139789495091824, option_139789495092096, option_139789495092368, option_139789495092640, option_139789495092912, option_139789495093184, option_139789495093456, option_139789495093728, option_139789495094000], **{'doc': u'', 'properties': (), 'name': 'file11'})
    option_139789495094272 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495094544 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495094816 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495099472 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495099744 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495100016 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495100288 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495100560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495100832 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495101104 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495033040 = OptionDescription(children=[option_139789495094272, option_139789495094544, option_139789495094816, option_139789495099472, option_139789495099744, option_139789495100016, option_139789495100288, option_139789495100560, option_139789495100832, option_139789495101104], **{'doc': u'', 'properties': (), 'name': 'file12'})
    option_139789495101376 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495101648 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495101920 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495102192 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495102464 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495102736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495103008 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495107664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495107936 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495108208 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495033360 = OptionDescription(children=[option_139789495101376, option_139789495101648, option_139789495101920, option_139789495102192, option_139789495102464, option_139789495102736, option_139789495103008, option_139789495107664, option_139789495107936, option_139789495108208], **{'doc': u'', 'properties': (), 'name': 'file13'})
    option_139789495108480 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495108752 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495109024 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495109296 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495109568 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495109840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495110112 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495110384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495110656 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495110928 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495115856 = OptionDescription(children=[option_139789495108480, option_139789495108752, option_139789495109024, option_139789495109296, option_139789495109568, option_139789495109840, option_139789495110112, option_139789495110384, option_139789495110656, option_139789495110928], **{'doc': u'', 'properties': (), 'name': 'file14'})
    option_139789495111200 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495119952 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495120224 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495120496 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495120768 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495121040 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495121312 = UnicodeOption(**{'default': u'/etc/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495121584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495121856 = UnicodeOption(**{'default': u'/var/lib/creole/timezone', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495122128 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495116176 = OptionDescription(children=[option_139789495111200, option_139789495119952, option_139789495120224, option_139789495120496, option_139789495120768, option_139789495121040, option_139789495121312, option_139789495121584, option_139789495121856, option_139789495122128], **{'doc': u'', 'properties': (), 'name': 'file15'})
    option_139789495122400 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495122672 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495122944 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495123216 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495123488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495132240 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495132512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495132784 = UnicodeOption(**{'default': u'/var/lib/creole/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495133056 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789495116496 = OptionDescription(children=[option_139789495122400, option_139789495122672, option_139789495122944, option_139789495123216, option_139789495123488, option_139789495132240, option_139789495132512, option_139789495132784, option_139789495133056], **{'doc': u'', 'properties': (), 'name': 'file16'})
    option_139789495133328 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495133600 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495133872 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495134144 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495134416 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495134688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495134960 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495135232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495135504 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495135776 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495116816 = OptionDescription(children=[option_139789495133328, option_139789495133600, option_139789495133872, option_139789495134144, option_139789495134416, option_139789495134688, option_139789495134960, option_139789495135232, option_139789495135504, option_139789495135776], **{'doc': u'', 'properties': (), 'name': 'file17'})
    option_139789495144528 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495144800 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495145072 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495145344 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495145616 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495145888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495146160 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495146432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495146704 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495146976 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495117136 = OptionDescription(children=[option_139789495144528, option_139789495144800, option_139789495145072, option_139789495145344, option_139789495145616, option_139789495145888, option_139789495146160, option_139789495146432, option_139789495146704, option_139789495146976], **{'doc': u'', 'properties': (), 'name': 'file18'})
    option_139789495147248 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495147520 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495147792 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495148064 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495156816 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495157088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495157360 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495157632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495157904 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495158176 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495117456 = OptionDescription(children=[option_139789495147248, option_139789495147520, option_139789495147792, option_139789495148064, option_139789495156816, option_139789495157088, option_139789495157360, option_139789495157632, option_139789495157904, option_139789495158176], **{'doc': u'', 'properties': (), 'name': 'file19'})
    option_139789495158448 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495158720 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495158992 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495159264 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495159536 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495159808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495160080 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495160352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495165008 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495165280 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495117776 = OptionDescription(children=[option_139789495158448, option_139789495158720, option_139789495158992, option_139789495159264, option_139789495159536, option_139789495159808, option_139789495160080, option_139789495160352, option_139789495165008, option_139789495165280], **{'doc': u'', 'properties': (), 'name': 'file20'})
    option_139789495165552 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495165824 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789495166096 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789495166368 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789495166640 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789495166912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789495167184 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789495167456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789495167728 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789495168000 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495118096 = OptionDescription(children=[option_139789495165552, option_139789495165824, option_139789495166096, option_139789495166368, option_139789495166640, option_139789495166912, option_139789495167184, option_139789495167456, option_139789495167728, option_139789495168000], **{'doc': u'', 'properties': (), 'name': 'file21'})
    option_139789495168272 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789495168544 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494648912 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494649184 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494649456 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494649728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494650000 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494650272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494650544 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494650816 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495118416 = OptionDescription(children=[option_139789495168272, option_139789495168544, option_139789494648912, option_139789494649184, option_139789494649456, option_139789494649728, option_139789494650000, option_139789494650272, option_139789494650544, option_139789494650816], **{'doc': u'', 'properties': (), 'name': 'file22'})
    option_139789494651088 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494651360 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494651632 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494651904 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494652176 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494652448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494661200 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494661472 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494661744 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494662016 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495118736 = OptionDescription(children=[option_139789494651088, option_139789494651360, option_139789494651632, option_139789494651904, option_139789494652176, option_139789494652448, option_139789494661200, option_139789494661472, option_139789494661744, option_139789494662016], **{'doc': u'', 'properties': (), 'name': 'file23'})
    option_139789494662288 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494662560 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494662832 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494663104 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494663376 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494663648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494663920 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494664192 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494664464 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494664736 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495119056 = OptionDescription(children=[option_139789494662288, option_139789494662560, option_139789494662832, option_139789494663104, option_139789494663376, option_139789494663648, option_139789494663920, option_139789494664192, option_139789494664464, option_139789494664736], **{'doc': u'', 'properties': (), 'name': 'file24'})
    option_139789494673488 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494673760 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494674032 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494674304 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494674576 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494674848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494675120 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494675392 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494675664 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494675936 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789495119376 = OptionDescription(children=[option_139789494673488, option_139789494673760, option_139789494674032, option_139789494674304, option_139789494674576, option_139789494674848, option_139789494675120, option_139789494675392, option_139789494675664, option_139789494675936], **{'doc': u'', 'properties': (), 'name': 'file25'})
    option_139789494676208 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494676480 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494676752 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494677024 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494681680 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494681952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494682224 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494682496 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494682768 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494683040 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494685776 = OptionDescription(children=[option_139789494676208, option_139789494676480, option_139789494676752, option_139789494677024, option_139789494681680, option_139789494681952, option_139789494682224, option_139789494682496, option_139789494682768, option_139789494683040], **{'doc': u'', 'properties': (), 'name': 'file26'})
    option_139789494683312 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494683584 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494683856 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494684128 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494684400 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494684672 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494684944 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494685216 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494693968 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494694240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494686096 = OptionDescription(children=[option_139789494683312, option_139789494683584, option_139789494683856, option_139789494684128, option_139789494684400, option_139789494684672, option_139789494684944, option_139789494685216, option_139789494693968, option_139789494694240], **{'doc': u'', 'properties': (), 'name': 'file27'})
    option_139789494694512 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494694784 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494695056 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494695328 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494695600 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494695872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494696144 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494696416 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494696688 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494696960 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494686416 = OptionDescription(children=[option_139789494694512, option_139789494694784, option_139789494695056, option_139789494695328, option_139789494695600, option_139789494695872, option_139789494696144, option_139789494696416, option_139789494696688, option_139789494696960], **{'doc': u'', 'properties': (), 'name': 'file28'})
    option_139789494697232 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494697504 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494702160 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494702432 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494702704 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494702976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494703248 = UnicodeOption(**{'default': u'/etc/network/interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494703520 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494703792 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.interfaces', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494704064 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494686736 = OptionDescription(children=[option_139789494697232, option_139789494697504, option_139789494702160, option_139789494702432, option_139789494702704, option_139789494702976, option_139789494703248, option_139789494703520, option_139789494703792, option_139789494704064], **{'doc': u'', 'properties': (), 'name': 'file29'})
    option_139789494704336 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494704608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494704880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494705152 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494705424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494705696 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494710352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494710624 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494710896 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494687056 = OptionDescription(children=[option_139789494704336, option_139789494704608, option_139789494704880, option_139789494705152, option_139789494705424, option_139789494705696, option_139789494710352, option_139789494710624, option_139789494710896], **{'doc': u'', 'properties': (), 'name': 'file30'})
    option_139789494711168 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494711440 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494711712 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494711984 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494712256 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494712528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494712800 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494713072 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494713344 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494713616 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494687376 = OptionDescription(children=[option_139789494711168, option_139789494711440, option_139789494711712, option_139789494711984, option_139789494712256, option_139789494712528, option_139789494712800, option_139789494713072, option_139789494713344, option_139789494713616], **{'doc': u'', 'properties': (), 'name': 'file31'})
    option_139789494713888 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494718544 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494718816 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494719088 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494719360 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494719632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494719904 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494720176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494720448 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494720720 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494687696 = OptionDescription(children=[option_139789494713888, option_139789494718544, option_139789494718816, option_139789494719088, option_139789494719360, option_139789494719632, option_139789494719904, option_139789494720176, option_139789494720448, option_139789494720720], **{'doc': u'', 'properties': (), 'name': 'file32'})
    option_139789494720992 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494721264 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494721536 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494721808 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494722080 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494726736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494727008 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494727280 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494727552 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494727824 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494688016 = OptionDescription(children=[option_139789494720992, option_139789494721264, option_139789494721536, option_139789494721808, option_139789494722080, option_139789494726736, option_139789494727008, option_139789494727280, option_139789494727552, option_139789494727824], **{'doc': u'', 'properties': (), 'name': 'file33'})
    option_139789494728096 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494728368 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494728640 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494728912 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494729184 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494729456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494729728 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494730000 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494730272 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494739024 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494688336 = OptionDescription(children=[option_139789494728096, option_139789494728368, option_139789494728640, option_139789494728912, option_139789494729184, option_139789494729456, option_139789494729728, option_139789494730000, option_139789494730272, option_139789494739024], **{'doc': u'', 'properties': (), 'name': 'file34'})
    option_139789494739296 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494739568 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494739840 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494740112 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494740384 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494740656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494740928 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494741200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494741472 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494741744 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494688656 = OptionDescription(children=[option_139789494739296, option_139789494739568, option_139789494739840, option_139789494740112, option_139789494740384, option_139789494740656, option_139789494740928, option_139789494741200, option_139789494741472, option_139789494741744], **{'doc': u'', 'properties': (), 'name': 'file35'})
    option_139789494742016 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494742288 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494742560 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494747216 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494747488 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494747760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494748032 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494748304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494748576 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494748848 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494688976 = OptionDescription(children=[option_139789494742016, option_139789494742288, option_139789494742560, option_139789494747216, option_139789494747488, option_139789494747760, option_139789494748032, option_139789494748304, option_139789494748576, option_139789494748848], **{'doc': u'', 'properties': (), 'name': 'file36'})
    option_139789494749120 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494749392 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494749664 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494749936 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494750208 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494750480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494750752 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494759504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494759776 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494760048 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494689296 = OptionDescription(children=[option_139789494749120, option_139789494749392, option_139789494749664, option_139789494749936, option_139789494750208, option_139789494750480, option_139789494750752, option_139789494759504, option_139789494759776, option_139789494760048], **{'doc': u'', 'properties': (), 'name': 'file37'})
    option_139789494760320 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494760592 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494760864 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494761136 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494761408 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494761680 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494761952 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494762224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494762496 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494762768 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494767696 = OptionDescription(children=[option_139789494760320, option_139789494760592, option_139789494760864, option_139789494761136, option_139789494761408, option_139789494761680, option_139789494761952, option_139789494762224, option_139789494762496, option_139789494762768], **{'doc': u'', 'properties': (), 'name': 'file38'})
    option_139789494763040 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494775888 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494776160 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494776432 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494776704 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494776976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494777248 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494777520 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494777792 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494778064 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494768016 = OptionDescription(children=[option_139789494763040, option_139789494775888, option_139789494776160, option_139789494776432, option_139789494776704, option_139789494776976, option_139789494777248, option_139789494777520, option_139789494777792, option_139789494778064], **{'doc': u'', 'properties': (), 'name': 'file39'})
    option_139789494778336 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494778608 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494778880 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494779152 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494779424 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494784080 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494784352 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494784624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494784896 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494785168 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494768336 = OptionDescription(children=[option_139789494778336, option_139789494778608, option_139789494778880, option_139789494779152, option_139789494779424, option_139789494784080, option_139789494784352, option_139789494784624, option_139789494784896, option_139789494785168], **{'doc': u'', 'properties': (), 'name': 'file40'})
    option_139789494785440 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494785712 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494785984 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494786256 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494786528 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494786800 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494787072 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494787344 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494787616 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494792272 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494768656 = OptionDescription(children=[option_139789494785440, option_139789494785712, option_139789494785984, option_139789494786256, option_139789494786528, option_139789494786800, option_139789494787072, option_139789494787344, option_139789494787616, option_139789494792272], **{'doc': u'', 'properties': (), 'name': 'file41'})
    option_139789494792544 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494792816 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494793088 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494793360 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494793632 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494793904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494794176 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494794448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494794720 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494794992 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494768976 = OptionDescription(children=[option_139789494792544, option_139789494792816, option_139789494793088, option_139789494793360, option_139789494793632, option_139789494793904, option_139789494794176, option_139789494794448, option_139789494794720, option_139789494794992], **{'doc': u'', 'properties': (), 'name': 'file42'})
    option_139789494795264 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494795536 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494795808 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494804560 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494804832 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494805104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494805376 = UnicodeOption(**{'default': u'/etc/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494805648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494805920 = UnicodeOption(**{'default': u'/var/lib/creole/resolv.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494806192 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494769296 = OptionDescription(children=[option_139789494795264, option_139789494795536, option_139789494795808, option_139789494804560, option_139789494804832, option_139789494805104, option_139789494805376, option_139789494805648, option_139789494805920, option_139789494806192], **{'doc': u'', 'properties': (), 'name': 'file43'})
    option_139789494806464 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494806736 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494807008 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494807280 = UnicodeOption(**{'default': u'/etc/hostname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494807552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494807824 = UnicodeOption(**{'default': u'/etc/hostname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494808096 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494812752 = UnicodeOption(**{'default': u'/var/lib/creole/hostname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494813024 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494769616 = OptionDescription(children=[option_139789494806464, option_139789494806736, option_139789494807008, option_139789494807280, option_139789494807552, option_139789494807824, option_139789494808096, option_139789494812752, option_139789494813024], **{'doc': u'', 'properties': (), 'name': 'file44'})
    option_139789494813296 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494813568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494813840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494814112 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494814384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494814656 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494814928 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494815200 = UnicodeOption(**{'default': u'/var/lib/creole/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494815472 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494769936 = OptionDescription(children=[option_139789494813296, option_139789494813568, option_139789494813840, option_139789494814112, option_139789494814384, option_139789494814656, option_139789494814928, option_139789494815200, option_139789494815472], **{'doc': u'', 'properties': (), 'name': 'file45'})
    option_139789494815744 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494816016 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494816288 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494820944 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494821216 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494821488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494821760 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494822032 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494822304 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494822576 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494770256 = OptionDescription(children=[option_139789494815744, option_139789494816016, option_139789494816288, option_139789494820944, option_139789494821216, option_139789494821488, option_139789494821760, option_139789494822032, option_139789494822304, option_139789494822576], **{'doc': u'', 'properties': (), 'name': 'file46'})
    option_139789494822848 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494823120 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494823392 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494823664 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494823936 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494824208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494824480 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494833232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494833504 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494833776 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494770576 = OptionDescription(children=[option_139789494822848, option_139789494823120, option_139789494823392, option_139789494823664, option_139789494823936, option_139789494824208, option_139789494824480, option_139789494833232, option_139789494833504, option_139789494833776], **{'doc': u'', 'properties': (), 'name': 'file47'})
    option_139789494834048 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494834320 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494834592 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494834864 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494835136 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494835408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494835680 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494835952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494836224 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494836496 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494770896 = OptionDescription(children=[option_139789494834048, option_139789494834320, option_139789494834592, option_139789494834864, option_139789494835136, option_139789494835408, option_139789494835680, option_139789494835952, option_139789494836224, option_139789494836496], **{'doc': u'', 'properties': (), 'name': 'file48'})
    option_139789494836768 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494841424 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494841696 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494841968 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494842240 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494842512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494842784 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494843056 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494843328 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494843600 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494771216 = OptionDescription(children=[option_139789494836768, option_139789494841424, option_139789494841696, option_139789494841968, option_139789494842240, option_139789494842512, option_139789494842784, option_139789494843056, option_139789494843328, option_139789494843600], **{'doc': u'', 'properties': (), 'name': 'file49'})
    option_139789494843872 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494844144 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494844416 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494844688 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494844960 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494849616 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494849888 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494850160 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494850432 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494850704 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494857808 = OptionDescription(children=[option_139789494843872, option_139789494844144, option_139789494844416, option_139789494844688, option_139789494844960, option_139789494849616, option_139789494849888, option_139789494850160, option_139789494850432, option_139789494850704], **{'doc': u'', 'properties': (), 'name': 'file50'})
    option_139789494850976 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494851248 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494851520 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494851792 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494852064 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494852336 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494852608 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494852880 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494853152 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494870096 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494858128 = OptionDescription(children=[option_139789494850976, option_139789494851248, option_139789494851520, option_139789494851792, option_139789494852064, option_139789494852336, option_139789494852608, option_139789494852880, option_139789494853152, option_139789494870096], **{'doc': u'', 'properties': (), 'name': 'file51'})
    option_139789494870368 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494870640 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494870912 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494871184 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494871456 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494871728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494872000 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494872272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494872544 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494872816 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494858448 = OptionDescription(children=[option_139789494870368, option_139789494870640, option_139789494870912, option_139789494871184, option_139789494871456, option_139789494871728, option_139789494872000, option_139789494872272, option_139789494872544, option_139789494872816], **{'doc': u'', 'properties': (), 'name': 'file52'})
    option_139789494873088 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494873360 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494873632 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494878288 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494878560 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494878832 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494879104 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494879376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494879648 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494879920 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494858768 = OptionDescription(children=[option_139789494873088, option_139789494873360, option_139789494873632, option_139789494878288, option_139789494878560, option_139789494878832, option_139789494879104, option_139789494879376, option_139789494879648, option_139789494879920], **{'doc': u'', 'properties': (), 'name': 'file53'})
    option_139789494880192 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494880464 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494880736 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494881008 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494881280 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494881552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494881824 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494886480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494886752 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494887024 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494859088 = OptionDescription(children=[option_139789494880192, option_139789494880464, option_139789494880736, option_139789494881008, option_139789494881280, option_139789494881552, option_139789494881824, option_139789494886480, option_139789494886752, option_139789494887024], **{'doc': u'', 'properties': (), 'name': 'file54'})
    option_139789494887296 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494887568 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494887840 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494888112 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494888384 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494888656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494888928 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494889200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494889472 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494889744 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494859408 = OptionDescription(children=[option_139789494887296, option_139789494887568, option_139789494887840, option_139789494888112, option_139789494888384, option_139789494888656, option_139789494888928, option_139789494889200, option_139789494889472, option_139789494889744], **{'doc': u'', 'properties': (), 'name': 'file55'})
    option_139789494890016 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494894672 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494894944 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494895216 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494895488 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494895760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494896032 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494896304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494896576 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494896848 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494859728 = OptionDescription(children=[option_139789494890016, option_139789494894672, option_139789494894944, option_139789494895216, option_139789494895488, option_139789494895760, option_139789494896032, option_139789494896304, option_139789494896576, option_139789494896848], **{'doc': u'', 'properties': (), 'name': 'file56'})
    option_139789494897120 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494897392 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494897664 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494897936 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494898208 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494902864 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494903136 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494903408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494903680 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494903952 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494860048 = OptionDescription(children=[option_139789494897120, option_139789494897392, option_139789494897664, option_139789494897936, option_139789494898208, option_139789494902864, option_139789494903136, option_139789494903408, option_139789494903680, option_139789494903952], **{'doc': u'', 'properties': (), 'name': 'file57'})
    option_139789494904224 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494904496 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494904768 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494905040 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494905312 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494905584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494905856 = UnicodeOption(**{'default': u'/etc/hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494906128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494906400 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.hosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494386768 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494860368 = OptionDescription(children=[option_139789494904224, option_139789494904496, option_139789494904768, option_139789494905040, option_139789494905312, option_139789494905584, option_139789494905856, option_139789494906128, option_139789494906400, option_139789494386768], **{'doc': u'', 'properties': (), 'name': 'file58'})
    option_139789494387040 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494387312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494387584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494387856 = UnicodeOption(**{'default': u'/etc/sysctl.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494388128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494388400 = UnicodeOption(**{'default': u'/etc/sysctl.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494388672 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494388944 = UnicodeOption(**{'default': u'/var/lib/creole/sysctl.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494389216 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494860688 = OptionDescription(children=[option_139789494387040, option_139789494387312, option_139789494387584, option_139789494387856, option_139789494388128, option_139789494388400, option_139789494388672, option_139789494388944, option_139789494389216], **{'doc': u'', 'properties': (), 'name': 'file59'})
    option_139789494389488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494389760 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494390032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494390304 = UnicodeOption(**{'default': u'/etc/sudoers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494394960 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494395232 = UnicodeOption(**{'default': u'0440', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494395504 = UnicodeOption(**{'default': u'/etc/sudoers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494395776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494396048 = UnicodeOption(**{'default': u'/var/lib/creole/sudoers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494396320 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494861008 = OptionDescription(children=[option_139789494389488, option_139789494389760, option_139789494390032, option_139789494390304, option_139789494394960, option_139789494395232, option_139789494395504, option_139789494395776, option_139789494396048, option_139789494396320], **{'doc': u'', 'properties': (), 'name': 'file60'})
    option_139789494396592 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494396864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494397136 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494397408 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494397680 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494397952 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494398224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494398496 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494403152 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494861328 = OptionDescription(children=[option_139789494396592, option_139789494396864, option_139789494397136, option_139789494397408, option_139789494397680, option_139789494397952, option_139789494398224, option_139789494398496, option_139789494403152], **{'doc': u'', 'properties': (), 'name': 'file61'})
    option_139789494403424 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494403696 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494403968 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494404240 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494404512 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494404784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494405056 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494405328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494405600 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494405872 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494411344 = OptionDescription(children=[option_139789494403424, option_139789494403696, option_139789494403968, option_139789494404240, option_139789494404512, option_139789494404784, option_139789494405056, option_139789494405328, option_139789494405600, option_139789494405872], **{'doc': u'', 'properties': (), 'name': 'file62'})
    option_139789494406144 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494406416 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494406688 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494415440 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494415712 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494415984 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494416256 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494416528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494416800 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494417072 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494411664 = OptionDescription(children=[option_139789494406144, option_139789494406416, option_139789494406688, option_139789494415440, option_139789494415712, option_139789494415984, option_139789494416256, option_139789494416528, option_139789494416800, option_139789494417072], **{'doc': u'', 'properties': (), 'name': 'file63'})
    option_139789494417344 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494417616 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494417888 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494418160 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494418432 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494418704 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494418976 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494427728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494428000 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494428272 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494411984 = OptionDescription(children=[option_139789494417344, option_139789494417616, option_139789494417888, option_139789494418160, option_139789494418432, option_139789494418704, option_139789494418976, option_139789494427728, option_139789494428000, option_139789494428272], **{'doc': u'', 'properties': (), 'name': 'file64'})
    option_139789494428544 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494428816 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494429088 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494429360 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494429632 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494429904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494430176 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494430448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494430720 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494430992 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494412304 = OptionDescription(children=[option_139789494428544, option_139789494428816, option_139789494429088, option_139789494429360, option_139789494429632, option_139789494429904, option_139789494430176, option_139789494430448, option_139789494430720, option_139789494430992], **{'doc': u'', 'properties': (), 'name': 'file65'})
    option_139789494431264 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494444112 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494444384 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494444656 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494444928 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494445200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494445472 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494445744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494446016 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494446288 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494412624 = OptionDescription(children=[option_139789494431264, option_139789494444112, option_139789494444384, option_139789494444656, option_139789494444928, option_139789494445200, option_139789494445472, option_139789494445744, option_139789494446016, option_139789494446288], **{'doc': u'', 'properties': (), 'name': 'file66'})
    option_139789494446560 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494446832 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494447104 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494447376 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494447648 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494456400 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494456672 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494456944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494457216 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494457488 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494412944 = OptionDescription(children=[option_139789494446560, option_139789494446832, option_139789494447104, option_139789494447376, option_139789494447648, option_139789494456400, option_139789494456672, option_139789494456944, option_139789494457216, option_139789494457488], **{'doc': u'', 'properties': (), 'name': 'file67'})
    option_139789494457760 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494458032 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494458304 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494458576 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494458848 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494459120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494459392 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494459664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494459936 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494464592 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494413264 = OptionDescription(children=[option_139789494457760, option_139789494458032, option_139789494458304, option_139789494458576, option_139789494458848, option_139789494459120, option_139789494459392, option_139789494459664, option_139789494459936, option_139789494464592], **{'doc': u'', 'properties': (), 'name': 'file68'})
    option_139789494464864 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494465136 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494465408 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494465680 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494465952 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494466224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494466496 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494466768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494467040 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494467312 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494413584 = OptionDescription(children=[option_139789494464864, option_139789494465136, option_139789494465408, option_139789494465680, option_139789494465952, option_139789494466224, option_139789494466496, option_139789494466768, option_139789494467040, option_139789494467312], **{'doc': u'', 'properties': (), 'name': 'file69'})
    option_139789494467584 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494467856 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494468128 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494472784 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494473056 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494473328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494473600 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494473872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494474144 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494474416 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494413904 = OptionDescription(children=[option_139789494467584, option_139789494467856, option_139789494468128, option_139789494472784, option_139789494473056, option_139789494473328, option_139789494473600, option_139789494473872, option_139789494474144, option_139789494474416], **{'doc': u'', 'properties': (), 'name': 'file70'})
    option_139789494474688 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494474960 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494475232 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494475504 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494475776 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494476048 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494476320 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494480976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494481248 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494481520 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494414224 = OptionDescription(children=[option_139789494474688, option_139789494474960, option_139789494475232, option_139789494475504, option_139789494475776, option_139789494476048, option_139789494476320, option_139789494480976, option_139789494481248, option_139789494481520], **{'doc': u'', 'properties': (), 'name': 'file71'})
    option_139789494481792 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494482064 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494482336 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494482608 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494482880 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494483152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494483424 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494483696 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494483968 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494484240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494414544 = OptionDescription(children=[option_139789494481792, option_139789494482064, option_139789494482336, option_139789494482608, option_139789494482880, option_139789494483152, option_139789494483424, option_139789494483696, option_139789494483968, option_139789494484240], **{'doc': u'', 'properties': (), 'name': 'file72'})
    option_139789494484512 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494493264 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494493536 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494493808 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494494080 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494494352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494494624 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494494896 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494495168 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494495440 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494414864 = OptionDescription(children=[option_139789494484512, option_139789494493264, option_139789494493536, option_139789494493808, option_139789494494080, option_139789494494352, option_139789494494624, option_139789494494896, option_139789494495168, option_139789494495440], **{'doc': u'', 'properties': (), 'name': 'file73'})
    option_139789494495712 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494495984 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494496256 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494496528 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494496800 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494501456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494501728 = UnicodeOption(**{'default': u'/etc/vim/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494502000 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494502272 = UnicodeOption(**{'default': u'/var/lib/creole/vimrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494502544 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494509648 = OptionDescription(children=[option_139789494495712, option_139789494495984, option_139789494496256, option_139789494496528, option_139789494496800, option_139789494501456, option_139789494501728, option_139789494502000, option_139789494502272, option_139789494502544], **{'doc': u'', 'properties': (), 'name': 'file74'})
    option_139789494502816 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494503088 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494503360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494503632 = UnicodeOption(**{'default': u'/etc/profile.d/eolerc.sh', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494503904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494504176 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494504448 = UnicodeOption(**{'default': u'/etc/profile.d/eolerc.sh', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494504720 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494504992 = UnicodeOption(**{'default': u'/var/lib/creole/eolerc.sh', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494513744 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494509968 = OptionDescription(children=[option_139789494502816, option_139789494503088, option_139789494503360, option_139789494503632, option_139789494503904, option_139789494504176, option_139789494504448, option_139789494504720, option_139789494504992, option_139789494513744], **{'doc': u'', 'properties': (), 'name': 'file75'})
    option_139789494514016 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494514288 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494514560 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494514832 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494515104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494515376 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494515648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494515920 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494516192 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494510288 = OptionDescription(children=[option_139789494514016, option_139789494514288, option_139789494514560, option_139789494514832, option_139789494515104, option_139789494515376, option_139789494515648, option_139789494515920, option_139789494516192], **{'doc': u'', 'properties': (), 'name': 'file76'})
    option_139789494516464 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494516736 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494517008 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494517280 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494526032 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494526304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494526576 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494526848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494527120 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494527392 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494510608 = OptionDescription(children=[option_139789494516464, option_139789494516736, option_139789494517008, option_139789494517280, option_139789494526032, option_139789494526304, option_139789494526576, option_139789494526848, option_139789494527120, option_139789494527392], **{'doc': u'', 'properties': (), 'name': 'file77'})
    option_139789494527664 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494527936 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494528208 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494528480 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494528752 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494529024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494529296 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494529568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494534224 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494534496 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494510928 = OptionDescription(children=[option_139789494527664, option_139789494527936, option_139789494528208, option_139789494528480, option_139789494528752, option_139789494529024, option_139789494529296, option_139789494529568, option_139789494534224, option_139789494534496], **{'doc': u'', 'properties': (), 'name': 'file78'})
    option_139789494534768 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494535040 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494535312 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494535584 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494535856 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494536128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494536400 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494536672 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494536944 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494537216 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494511248 = OptionDescription(children=[option_139789494534768, option_139789494535040, option_139789494535312, option_139789494535584, option_139789494535856, option_139789494536128, option_139789494536400, option_139789494536672, option_139789494536944, option_139789494537216], **{'doc': u'', 'properties': (), 'name': 'file79'})
    option_139789494537488 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494537760 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494546512 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494546784 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494547056 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494547328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494547600 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494547872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494548144 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494548416 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494511568 = OptionDescription(children=[option_139789494537488, option_139789494537760, option_139789494546512, option_139789494546784, option_139789494547056, option_139789494547328, option_139789494547600, option_139789494547872, option_139789494548144, option_139789494548416], **{'doc': u'', 'properties': (), 'name': 'file80'})
    option_139789494548688 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494548960 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494549232 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494549504 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494549776 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494550048 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494554704 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494554976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494555248 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494555520 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494511888 = OptionDescription(children=[option_139789494548688, option_139789494548960, option_139789494549232, option_139789494549504, option_139789494549776, option_139789494550048, option_139789494554704, option_139789494554976, option_139789494555248, option_139789494555520], **{'doc': u'', 'properties': (), 'name': 'file81'})
    option_139789494555792 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494556064 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494556336 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494556608 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494556880 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494557152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494557424 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494557696 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494557968 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494558240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494512208 = OptionDescription(children=[option_139789494555792, option_139789494556064, option_139789494556336, option_139789494556608, option_139789494556880, option_139789494557152, option_139789494557424, option_139789494557696, option_139789494557968, option_139789494558240], **{'doc': u'', 'properties': (), 'name': 'file82'})
    option_139789494571088 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494571360 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494571632 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494571904 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494572176 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494572448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494572720 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494572992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494573264 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494573536 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494512528 = OptionDescription(children=[option_139789494571088, option_139789494571360, option_139789494571632, option_139789494571904, option_139789494572176, option_139789494572448, option_139789494572720, option_139789494572992, option_139789494573264, option_139789494573536], **{'doc': u'', 'properties': (), 'name': 'file83'})
    option_139789494573808 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494574080 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494574352 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494574624 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494579280 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494579552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494579824 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494580096 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494580368 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494580640 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494512848 = OptionDescription(children=[option_139789494573808, option_139789494574080, option_139789494574352, option_139789494574624, option_139789494579280, option_139789494579552, option_139789494579824, option_139789494580096, option_139789494580368, option_139789494580640], **{'doc': u'', 'properties': (), 'name': 'file84'})
    option_139789494580912 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494581184 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494581456 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494581728 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494582000 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494582272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494582544 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494582816 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494587472 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494587744 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494513168 = OptionDescription(children=[option_139789494580912, option_139789494581184, option_139789494581456, option_139789494581728, option_139789494582000, option_139789494582272, option_139789494582544, option_139789494582816, option_139789494587472, option_139789494587744], **{'doc': u'', 'properties': (), 'name': 'file85'})
    option_139789494588016 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494588288 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494588560 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494588832 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494589104 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494589376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494589648 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494589920 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494590192 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494590464 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494595664 = OptionDescription(children=[option_139789494588016, option_139789494588288, option_139789494588560, option_139789494588832, option_139789494589104, option_139789494589376, option_139789494589648, option_139789494589920, option_139789494590192, option_139789494590464], **{'doc': u'', 'properties': (), 'name': 'file86'})
    option_139789494590736 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494591008 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494599760 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494600032 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494600304 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494600576 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494600848 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494601120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494601392 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494601664 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494595984 = OptionDescription(children=[option_139789494590736, option_139789494591008, option_139789494599760, option_139789494600032, option_139789494600304, option_139789494600576, option_139789494600848, option_139789494601120, option_139789494601392, option_139789494601664], **{'doc': u'', 'properties': (), 'name': 'file87'})
    option_139789494601936 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494602208 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494602480 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494602752 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494603024 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494603296 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494612048 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494612320 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494612592 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494612864 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494596304 = OptionDescription(children=[option_139789494601936, option_139789494602208, option_139789494602480, option_139789494602752, option_139789494603024, option_139789494603296, option_139789494612048, option_139789494612320, option_139789494612592, option_139789494612864], **{'doc': u'', 'properties': (), 'name': 'file88'})
    option_139789494613136 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494613408 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494613680 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494613952 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494614224 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494614496 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494614768 = UnicodeOption(**{'default': u'/etc/ssh/sshd_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494615040 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494615312 = UnicodeOption(**{'default': u'/var/lib/creole/sshd_config.lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494615584 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494596624 = OptionDescription(children=[option_139789494613136, option_139789494613408, option_139789494613680, option_139789494613952, option_139789494614224, option_139789494614496, option_139789494614768, option_139789494615040, option_139789494615312, option_139789494615584], **{'doc': u'', 'properties': (), 'name': 'file89'})
    option_139789494624336 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494624608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494624880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494625152 = UnicodeOption(**{'default': u'/etc/ssh/ssh_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494625424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494625696 = UnicodeOption(**{'default': u'/etc/ssh/ssh_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494625968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494626240 = UnicodeOption(**{'default': u'/var/lib/creole/ssh_config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494626512 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494596944 = OptionDescription(children=[option_139789494624336, option_139789494624608, option_139789494624880, option_139789494625152, option_139789494625424, option_139789494625696, option_139789494625968, option_139789494626240, option_139789494626512], **{'doc': u'', 'properties': (), 'name': 'file90'})
    option_139789494626784 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494627056 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494627328 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494627600 = UnicodeOption(**{'default': u'/etc/pam.d/common-password', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494627872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494632528 = UnicodeOption(**{'default': u'/etc/pam.d/common-password', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494632800 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494633072 = UnicodeOption(**{'default': u'/var/lib/creole/common-password', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494633344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494597264 = OptionDescription(children=[option_139789494626784, option_139789494627056, option_139789494627328, option_139789494627600, option_139789494627872, option_139789494632528, option_139789494632800, option_139789494633072, option_139789494633344], **{'doc': u'', 'properties': (), 'name': 'file91'})
    option_139789494633616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494633888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494634160 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494634432 = UnicodeOption(**{'default': u'/etc/DIR_COLORS', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494634704 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494634976 = UnicodeOption(**{'default': u'/etc/DIR_COLORS', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494635248 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494635520 = UnicodeOption(**{'default': u'/var/lib/creole/DIR_COLORS', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494635792 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494597584 = OptionDescription(children=[option_139789494633616, option_139789494633888, option_139789494634160, option_139789494634432, option_139789494634704, option_139789494634976, option_139789494635248, option_139789494635520, option_139789494635792], **{'doc': u'', 'properties': (), 'name': 'file92'})
    option_139789494636064 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494120528 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494120800 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494121072 = UnicodeOption(**{'default': u'/etc/pythonrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494121344 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494121616 = UnicodeOption(**{'default': u'/etc/pythonrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494121888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494122160 = UnicodeOption(**{'default': u'/var/lib/creole/pythonrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494122432 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494597904 = OptionDescription(children=[option_139789494636064, option_139789494120528, option_139789494120800, option_139789494121072, option_139789494121344, option_139789494121616, option_139789494121888, option_139789494122160, option_139789494122432], **{'doc': u'', 'properties': (), 'name': 'file93'})
    option_139789494122704 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494122976 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494123248 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494123520 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494123792 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494124064 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494128720 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494128992 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494129264 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494598224 = OptionDescription(children=[option_139789494122704, option_139789494122976, option_139789494123248, option_139789494123520, option_139789494123792, option_139789494124064, option_139789494128720, option_139789494128992, option_139789494129264], **{'doc': u'', 'properties': (), 'name': 'file94'})
    option_139789494129536 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494129808 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494130080 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494130352 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494130624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494130896 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494131168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494131440 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494131712 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494598544 = OptionDescription(children=[option_139789494129536, option_139789494129808, option_139789494130080, option_139789494130352, option_139789494130624, option_139789494130896, option_139789494131168, option_139789494131440, option_139789494131712], **{'doc': u'', 'properties': (), 'name': 'file95'})
    option_139789494131984 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494132256 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494136912 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494137184 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494137456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494137728 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494138000 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494138272 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494138544 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494598864 = OptionDescription(children=[option_139789494131984, option_139789494132256, option_139789494136912, option_139789494137184, option_139789494137456, option_139789494137728, option_139789494138000, option_139789494138272, option_139789494138544], **{'doc': u'', 'properties': (), 'name': 'file96'})
    option_139789494138816 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494139088 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494139360 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494139632 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494139904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494140176 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494140448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494145104 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494145376 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494599184 = OptionDescription(children=[option_139789494138816, option_139789494139088, option_139789494139360, option_139789494139632, option_139789494139904, option_139789494140176, option_139789494140448, option_139789494145104, option_139789494145376], **{'doc': u'', 'properties': (), 'name': 'file97'})
    option_139789494145648 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494145920 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494146192 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494146464 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494146736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494147008 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494147280 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494147552 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494147824 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494153296 = OptionDescription(children=[option_139789494145648, option_139789494145920, option_139789494146192, option_139789494146464, option_139789494146736, option_139789494147008, option_139789494147280, option_139789494147552, option_139789494147824], **{'doc': u'', 'properties': (), 'name': 'file98'})
    option_139789494148096 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494148368 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494148640 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494157392 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494157664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494157936 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494158208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494158480 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494158752 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494153616 = OptionDescription(children=[option_139789494148096, option_139789494148368, option_139789494148640, option_139789494157392, option_139789494157664, option_139789494157936, option_139789494158208, option_139789494158480, option_139789494158752], **{'doc': u'', 'properties': (), 'name': 'file99'})
    option_139789494159024 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494159296 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494159568 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494159840 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494160112 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494160384 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494160656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494160928 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494165584 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494153936 = OptionDescription(children=[option_139789494159024, option_139789494159296, option_139789494159568, option_139789494159840, option_139789494160112, option_139789494160384, option_139789494160656, option_139789494160928, option_139789494165584], **{'doc': u'', 'properties': (), 'name': 'file100'})
    option_139789494165856 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494166128 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494166400 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494166672 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494166944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494167216 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494167488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494167760 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494168032 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494154256 = OptionDescription(children=[option_139789494165856, option_139789494166128, option_139789494166400, option_139789494166672, option_139789494166944, option_139789494167216, option_139789494167488, option_139789494167760, option_139789494168032], **{'doc': u'', 'properties': (), 'name': 'file101'})
    option_139789494168304 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494168576 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494168848 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494169120 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494173776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494174048 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494174320 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494174592 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494174864 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494154576 = OptionDescription(children=[option_139789494168304, option_139789494168576, option_139789494168848, option_139789494169120, option_139789494173776, option_139789494174048, option_139789494174320, option_139789494174592, option_139789494174864], **{'doc': u'', 'properties': (), 'name': 'file102'})
    option_139789494175136 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494175408 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494175680 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494175952 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494176224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494176496 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494176768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494177040 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494177312 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494154896 = OptionDescription(children=[option_139789494175136, option_139789494175408, option_139789494175680, option_139789494175952, option_139789494176224, option_139789494176496, option_139789494176768, option_139789494177040, option_139789494177312], **{'doc': u'', 'properties': (), 'name': 'file103'})
    option_139789494181968 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494182240 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494182512 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494182784 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494183056 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494183328 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494183600 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494183872 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494184144 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494155216 = OptionDescription(children=[option_139789494181968, option_139789494182240, option_139789494182512, option_139789494182784, option_139789494183056, option_139789494183328, option_139789494183600, option_139789494183872, option_139789494184144], **{'doc': u'', 'properties': (), 'name': 'file104'})
    option_139789494184416 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494184688 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494184960 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494185232 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494185504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494194256 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494194528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494194800 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494195072 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494155536 = OptionDescription(children=[option_139789494184416, option_139789494184688, option_139789494184960, option_139789494185232, option_139789494185504, option_139789494194256, option_139789494194528, option_139789494194800, option_139789494195072], **{'doc': u'', 'properties': (), 'name': 'file105'})
    option_139789494195344 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494195616 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494195888 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494196160 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494196432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494196704 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494196976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494197248 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494197520 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494155856 = OptionDescription(children=[option_139789494195344, option_139789494195616, option_139789494195888, option_139789494196160, option_139789494196432, option_139789494196704, option_139789494196976, option_139789494197248, option_139789494197520], **{'doc': u'', 'properties': (), 'name': 'file106'})
    option_139789494197792 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494206544 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494206816 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494207088 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494207360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494207632 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494207904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494208176 = UnicodeOption(**{'default': u'/var/lib/creole/02eoleproxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494208448 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494156176 = OptionDescription(children=[option_139789494197792, option_139789494206544, option_139789494206816, option_139789494207088, option_139789494207360, option_139789494207632, option_139789494207904, option_139789494208176, option_139789494208448], **{'doc': u'', 'properties': (), 'name': 'file107'})
    option_139789494208720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494208992 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494209264 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494209536 = UnicodeOption(**{'default': u'/etc/modprobe.d/blacklist-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494209808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494210080 = UnicodeOption(**{'default': u'/etc/modprobe.d/blacklist-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494214736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494215008 = UnicodeOption(**{'default': u'/var/lib/creole/blacklist-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494215280 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494156496 = OptionDescription(children=[option_139789494208720, option_139789494208992, option_139789494209264, option_139789494209536, option_139789494209808, option_139789494210080, option_139789494214736, option_139789494215008, option_139789494215280], **{'doc': u'', 'properties': (), 'name': 'file108'})
    option_139789494215552 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494215824 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494216096 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494216368 = UnicodeOption(**{'default': u'rngd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494216640 = UnicodeOption(**{'default': u'/etc/default/rng-tools', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494216912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494217184 = UnicodeOption(**{'default': u'/etc/default/rng-tools', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494217456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494217728 = UnicodeOption(**{'default': u'/var/lib/creole/rng-tools', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494218000 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497985232}]})
    option_139789494156816 = OptionDescription(children=[option_139789494215552, option_139789494215824, option_139789494216096, option_139789494216368, option_139789494216640, option_139789494216912, option_139789494217184, option_139789494217456, option_139789494217728, option_139789494218000], **{'doc': u'', 'properties': (), 'name': 'file109'})
    option_139789494218272 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494222928 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494223200 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494223472 = UnicodeOption(**{'default': u'/etc/issue', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494223744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494224016 = UnicodeOption(**{'default': u'/etc/issue', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494224288 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494224560 = UnicodeOption(**{'default': u'/var/lib/creole/issue', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494224832 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494227024 = OptionDescription(children=[option_139789494218272, option_139789494222928, option_139789494223200, option_139789494223472, option_139789494223744, option_139789494224016, option_139789494224288, option_139789494224560, option_139789494224832], **{'doc': u'', 'properties': (), 'name': 'file110'})
    option_139789494225104 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494225376 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494225648 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494225920 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494226192 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494226464 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494235216 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494235488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494235760 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494236032 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494227344 = OptionDescription(children=[option_139789494225104, option_139789494225376, option_139789494225648, option_139789494225920, option_139789494226192, option_139789494226464, option_139789494235216, option_139789494235488, option_139789494235760, option_139789494236032], **{'doc': u'', 'properties': (), 'name': 'file111'})
    option_139789494236304 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494236576 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494236848 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494237120 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494237392 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494237664 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494237936 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494238208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494238480 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494238752 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494227664 = OptionDescription(children=[option_139789494236304, option_139789494236576, option_139789494236848, option_139789494237120, option_139789494237392, option_139789494237664, option_139789494237936, option_139789494238208, option_139789494238480, option_139789494238752], **{'doc': u'', 'properties': (), 'name': 'file112'})
    option_139789494243408 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494243680 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494243952 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494244224 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494244496 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494244768 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494245040 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494245312 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494245584 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494245856 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494227984 = OptionDescription(children=[option_139789494243408, option_139789494243680, option_139789494243952, option_139789494244224, option_139789494244496, option_139789494244768, option_139789494245040, option_139789494245312, option_139789494245584, option_139789494245856], **{'doc': u'', 'properties': (), 'name': 'file113'})
    option_139789494246128 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494246400 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494246672 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494246944 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494251600 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494251872 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494252144 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494252416 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494252688 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494252960 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494228304 = OptionDescription(children=[option_139789494246128, option_139789494246400, option_139789494246672, option_139789494246944, option_139789494251600, option_139789494251872, option_139789494252144, option_139789494252416, option_139789494252688, option_139789494252960], **{'doc': u'', 'properties': (), 'name': 'file114'})
    option_139789494253232 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494253504 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494253776 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494254048 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494254320 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494254592 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494254864 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494255136 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494259792 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494260064 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494228624 = OptionDescription(children=[option_139789494253232, option_139789494253504, option_139789494253776, option_139789494254048, option_139789494254320, option_139789494254592, option_139789494254864, option_139789494255136, option_139789494259792, option_139789494260064], **{'doc': u'', 'properties': (), 'name': 'file115'})
    option_139789494260336 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494260608 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494260880 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494261152 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494261424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494261696 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494261968 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494262240 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494262512 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494262784 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494228944 = OptionDescription(children=[option_139789494260336, option_139789494260608, option_139789494260880, option_139789494261152, option_139789494261424, option_139789494261696, option_139789494261968, option_139789494262240, option_139789494262512, option_139789494262784], **{'doc': u'', 'properties': (), 'name': 'file116'})
    option_139789494263056 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494263328 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494272080 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494272352 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494272624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494272896 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494273168 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494273440 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494273712 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494273984 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494229264 = OptionDescription(children=[option_139789494263056, option_139789494263328, option_139789494272080, option_139789494272352, option_139789494272624, option_139789494272896, option_139789494273168, option_139789494273440, option_139789494273712, option_139789494273984], **{'doc': u'', 'properties': (), 'name': 'file117'})
    option_139789494274256 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494274528 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494274800 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494275072 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494275344 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494275616 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494280272 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494280544 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494280816 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494281088 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494229584 = OptionDescription(children=[option_139789494274256, option_139789494274528, option_139789494274800, option_139789494275072, option_139789494275344, option_139789494275616, option_139789494280272, option_139789494280544, option_139789494280816, option_139789494281088], **{'doc': u'', 'properties': (), 'name': 'file118'})
    option_139789494281360 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494281632 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494281904 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494282176 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494282448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494282720 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494282992 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494283264 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494283536 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494283808 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494229904 = OptionDescription(children=[option_139789494281360, option_139789494281632, option_139789494281904, option_139789494282176, option_139789494282448, option_139789494282720, option_139789494282992, option_139789494283264, option_139789494283536, option_139789494283808], **{'doc': u'', 'properties': (), 'name': 'file119'})
    option_139789494288464 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494288736 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494289008 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494289280 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494289552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494289824 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494290096 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494290368 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494290640 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494290912 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494230224 = OptionDescription(children=[option_139789494288464, option_139789494288736, option_139789494289008, option_139789494289280, option_139789494289552, option_139789494289824, option_139789494290096, option_139789494290368, option_139789494290640, option_139789494290912], **{'doc': u'', 'properties': (), 'name': 'file120'})
    option_139789494291184 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494291456 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494291728 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494292000 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494300752 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494301024 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494301296 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494301568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494301840 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494302112 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494230544 = OptionDescription(children=[option_139789494291184, option_139789494291456, option_139789494291728, option_139789494292000, option_139789494300752, option_139789494301024, option_139789494301296, option_139789494301568, option_139789494301840, option_139789494302112], **{'doc': u'', 'properties': (), 'name': 'file121'})
    option_139789494302384 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494302656 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494302928 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494303200 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494303472 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494303744 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494304016 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494304288 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494308944 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494309216 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494313040 = OptionDescription(children=[option_139789494302384, option_139789494302656, option_139789494302928, option_139789494303200, option_139789494303472, option_139789494303744, option_139789494304016, option_139789494304288, option_139789494308944, option_139789494309216], **{'doc': u'', 'properties': (), 'name': 'file122'})
    option_139789494309488 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494309760 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494310032 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494310304 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494310576 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494310848 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494311120 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494311392 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494311664 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494311936 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494313360 = OptionDescription(children=[option_139789494309488, option_139789494309760, option_139789494310032, option_139789494310304, option_139789494310576, option_139789494310848, option_139789494311120, option_139789494311392, option_139789494311664, option_139789494311936], **{'doc': u'', 'properties': (), 'name': 'file123'})
    option_139789494312208 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494312480 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494321232 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494321504 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494321776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494322048 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494322320 = UnicodeOption(**{'default': u'/etc/update-motd.d/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494322592 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494322864 = UnicodeOption(**{'default': u'/var/lib/creole/00-header', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494323136 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494313680 = OptionDescription(children=[option_139789494312208, option_139789494312480, option_139789494321232, option_139789494321504, option_139789494321776, option_139789494322048, option_139789494322320, option_139789494322592, option_139789494322864, option_139789494323136], **{'doc': u'', 'properties': (), 'name': 'file124'})
    option_139789494323408 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494323680 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494323952 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494324224 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494324496 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494324768 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494329424 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494329696 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494329968 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494330240 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494314000 = OptionDescription(children=[option_139789494323408, option_139789494323680, option_139789494323952, option_139789494324224, option_139789494324496, option_139789494324768, option_139789494329424, option_139789494329696, option_139789494329968, option_139789494330240], **{'doc': u'', 'properties': (), 'name': 'file125'})
    option_139789494330512 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494330784 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494331056 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494331328 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494331600 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494331872 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494332144 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494332416 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494332688 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494332960 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494314320 = OptionDescription(children=[option_139789494330512, option_139789494330784, option_139789494331056, option_139789494331328, option_139789494331600, option_139789494331872, option_139789494332144, option_139789494332416, option_139789494332688, option_139789494332960], **{'doc': u'', 'properties': (), 'name': 'file126'})
    option_139789494337616 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494337888 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494338160 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494338432 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494338704 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494338976 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494339248 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494339520 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494339792 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494340064 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494314640 = OptionDescription(children=[option_139789494337616, option_139789494337888, option_139789494338160, option_139789494338432, option_139789494338704, option_139789494338976, option_139789494339248, option_139789494339520, option_139789494339792, option_139789494340064], **{'doc': u'', 'properties': (), 'name': 'file127'})
    option_139789494340336 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494340608 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494340880 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494341152 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494345808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494346080 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494346352 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494346624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494346896 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494347168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494314960 = OptionDescription(children=[option_139789494340336, option_139789494340608, option_139789494340880, option_139789494341152, option_139789494345808, option_139789494346080, option_139789494346352, option_139789494346624, option_139789494346896, option_139789494347168], **{'doc': u'', 'properties': (), 'name': 'file128'})
    option_139789494347440 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494347712 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494347984 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494348256 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494348528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494348800 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494349072 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494349344 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494358096 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494358368 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494315280 = OptionDescription(children=[option_139789494347440, option_139789494347712, option_139789494347984, option_139789494348256, option_139789494348528, option_139789494348800, option_139789494349072, option_139789494349344, option_139789494358096, option_139789494358368], **{'doc': u'', 'properties': (), 'name': 'file129'})
    option_139789494358640 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494358912 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494359184 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494359456 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494359728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494360000 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494360272 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494360544 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494360816 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494361088 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494315600 = OptionDescription(children=[option_139789494358640, option_139789494358912, option_139789494359184, option_139789494359456, option_139789494359728, option_139789494360000, option_139789494360272, option_139789494360544, option_139789494360816, option_139789494361088], **{'doc': u'', 'properties': (), 'name': 'file130'})
    option_139789494361360 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494361632 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494366288 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494366560 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494366832 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494367104 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494367376 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494367648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494367920 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494368192 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494315920 = OptionDescription(children=[option_139789494361360, option_139789494361632, option_139789494366288, option_139789494366560, option_139789494366832, option_139789494367104, option_139789494367376, option_139789494367648, option_139789494367920, option_139789494368192], **{'doc': u'', 'properties': (), 'name': 'file131'})
    option_139789494368464 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494368736 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494369008 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494369280 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494369552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494369824 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494378576 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494378848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494379120 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494379392 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494316240 = OptionDescription(children=[option_139789494368464, option_139789494368736, option_139789494369008, option_139789494369280, option_139789494369552, option_139789494369824, option_139789494378576, option_139789494378848, option_139789494379120, option_139789494379392], **{'doc': u'', 'properties': (), 'name': 'file132'})
    option_139789494379664 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494379936 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494380208 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494380480 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494380752 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494381024 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494381296 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494381568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494381840 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494382112 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494316560 = OptionDescription(children=[option_139789494379664, option_139789494379936, option_139789494380208, option_139789494380480, option_139789494380752, option_139789494381024, option_139789494381296, option_139789494381568, option_139789494381840, option_139789494382112], **{'doc': u'', 'properties': (), 'name': 'file133'})
    option_139789493862480 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493862752 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493863024 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493863296 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493863568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493863840 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493864112 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493864384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493864656 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493864928 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493870672 = OptionDescription(children=[option_139789493862480, option_139789493862752, option_139789493863024, option_139789493863296, option_139789493863568, option_139789493863840, option_139789493864112, option_139789493864384, option_139789493864656, option_139789493864928], **{'doc': u'', 'properties': (), 'name': 'file134'})
    option_139789493865200 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493865472 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493865744 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493866016 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493878864 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493879136 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493879408 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493879680 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493879952 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493880224 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493870992 = OptionDescription(children=[option_139789493865200, option_139789493865472, option_139789493865744, option_139789493866016, option_139789493878864, option_139789493879136, option_139789493879408, option_139789493879680, option_139789493879952, option_139789493880224], **{'doc': u'', 'properties': (), 'name': 'file135'})
    option_139789493880496 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493880768 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493881040 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493881312 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493881584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493881856 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493882128 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493882400 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493887056 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493887328 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493871312 = OptionDescription(children=[option_139789493880496, option_139789493880768, option_139789493881040, option_139789493881312, option_139789493881584, option_139789493881856, option_139789493882128, option_139789493882400, option_139789493887056, option_139789493887328], **{'doc': u'', 'properties': (), 'name': 'file136'})
    option_139789493887600 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493887872 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493888144 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493888416 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493888688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493888960 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493889232 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493889504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493889776 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493890048 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493871632 = OptionDescription(children=[option_139789493887600, option_139789493887872, option_139789493888144, option_139789493888416, option_139789493888688, option_139789493888960, option_139789493889232, option_139789493889504, option_139789493889776, option_139789493890048], **{'doc': u'', 'properties': (), 'name': 'file137'})
    option_139789493890320 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493890592 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493895248 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493895520 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493895792 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493896064 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493896336 = UnicodeOption(**{'default': u'/etc/update-motd.d/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493896608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493896880 = UnicodeOption(**{'default': u'/var/lib/creole/10-help-text', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493897152 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493871952 = OptionDescription(children=[option_139789493890320, option_139789493890592, option_139789493895248, option_139789493895520, option_139789493895792, option_139789493896064, option_139789493896336, option_139789493896608, option_139789493896880, option_139789493897152], **{'doc': u'', 'properties': (), 'name': 'file138'})
    option_139789493897424 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493897696 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493897968 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493898240 = UnicodeOption(**{'default': u'/etc/network/if-up.d/openssh-server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493898512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493898784 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493903440 = UnicodeOption(**{'default': u'/etc/network/if-up.d/openssh-server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493903712 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493903984 = UnicodeOption(**{'default': u'/var/lib/creole/openssh-server', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493904256 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493872272 = OptionDescription(children=[option_139789493897424, option_139789493897696, option_139789493897968, option_139789493898240, option_139789493898512, option_139789493898784, option_139789493903440, option_139789493903712, option_139789493903984, option_139789493904256], **{'doc': u'', 'properties': (), 'name': 'file139'})
    option_139789493904528 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493904800 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493905072 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493905344 = UnicodeOption(**{'default': u'/etc/sysctl.d/10-console-messages.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493905616 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493905888 = UnicodeOption(**{'default': u'/etc/sysctl.d/10-console-messages.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493906160 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493906432 = UnicodeOption(**{'default': u'/var/lib/creole/10-console-messages.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493906704 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493872592 = OptionDescription(children=[option_139789493904528, option_139789493904800, option_139789493905072, option_139789493905344, option_139789493905616, option_139789493905888, option_139789493906160, option_139789493906432, option_139789493906704], **{'doc': u'', 'properties': (), 'name': 'file140'})
    option_139789493906976 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493911632 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493911904 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493912176 = UnicodeOption(**{'default': u'/etc/cron.d/schedule', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493912448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493912720 = UnicodeOption(**{'default': u'/etc/cron.d/schedule', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493912992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493913264 = UnicodeOption(**{'default': u'/var/lib/creole/cron-schedule', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493913536 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493872912 = OptionDescription(children=[option_139789493906976, option_139789493911632, option_139789493911904, option_139789493912176, option_139789493912448, option_139789493912720, option_139789493912992, option_139789493913264, option_139789493913536], **{'doc': u'', 'properties': (), 'name': 'file141'})
    option_139789493913808 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493914080 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493914352 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493914624 = UnicodeOption(**{'default': u'/etc/logrotate.d/00-eole-options', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493914896 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493915168 = UnicodeOption(**{'default': u'/etc/logrotate.d/00-eole-options', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493919824 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493920096 = UnicodeOption(**{'default': u'/var/lib/creole/logrotate_global_options', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493920368 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493873232 = OptionDescription(children=[option_139789493913808, option_139789493914080, option_139789493914352, option_139789493914624, option_139789493914896, option_139789493915168, option_139789493919824, option_139789493920096, option_139789493920368], **{'doc': u'', 'properties': (), 'name': 'file142'})
    option_139789493920640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493920912 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493921184 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493921456 = UnicodeOption(**{'default': u'/var/lib/eole/config/zstats.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493921728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493922000 = UnicodeOption(**{'default': u'/var/lib/eole/config/zstats.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493922272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493922544 = UnicodeOption(**{'default': u'/var/lib/creole/zstats.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493922816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493873552 = OptionDescription(children=[option_139789493920640, option_139789493920912, option_139789493921184, option_139789493921456, option_139789493921728, option_139789493922000, option_139789493922272, option_139789493922544, option_139789493922816], **{'doc': u'', 'properties': (), 'name': 'file143'})
    option_139789493923088 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493923360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493928016 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493928288 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493928560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493928832 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493929104 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493929376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493929648 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493929920 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493873872 = OptionDescription(children=[option_139789493923088, option_139789493923360, option_139789493928016, option_139789493928288, option_139789493928560, option_139789493928832, option_139789493929104, option_139789493929376, option_139789493929648, option_139789493929920], **{'doc': u'', 'properties': (), 'name': 'file144'})
    option_139789493930192 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493930464 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493930736 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493931008 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493931280 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493931552 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493936208 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493936480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493936752 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493937024 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493874192 = OptionDescription(children=[option_139789493930192, option_139789493930464, option_139789493930736, option_139789493931008, option_139789493931280, option_139789493931552, option_139789493936208, option_139789493936480, option_139789493936752, option_139789493937024], **{'doc': u'', 'properties': (), 'name': 'file145'})
    option_139789493937296 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493937568 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493937840 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493938112 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493938384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493938656 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493938928 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493939200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493939472 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493939744 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493944400 = OptionDescription(children=[option_139789493937296, option_139789493937568, option_139789493937840, option_139789493938112, option_139789493938384, option_139789493938656, option_139789493938928, option_139789493939200, option_139789493939472, option_139789493939744], **{'doc': u'', 'properties': (), 'name': 'file146'})
    option_139789493948496 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493948768 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493949040 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493949312 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493949584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493949856 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493950128 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493950400 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493950672 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493950944 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493944720 = OptionDescription(children=[option_139789493948496, option_139789493948768, option_139789493949040, option_139789493949312, option_139789493949584, option_139789493949856, option_139789493950128, option_139789493950400, option_139789493950672, option_139789493950944], **{'doc': u'', 'properties': (), 'name': 'file147'})
    option_139789493951216 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493951488 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493951760 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493952032 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493956688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493956960 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493957232 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493957504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493957776 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493958048 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493945040 = OptionDescription(children=[option_139789493951216, option_139789493951488, option_139789493951760, option_139789493952032, option_139789493956688, option_139789493956960, option_139789493957232, option_139789493957504, option_139789493957776, option_139789493958048], **{'doc': u'', 'properties': (), 'name': 'file148'})
    option_139789493958320 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493958592 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493958864 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493959136 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493959408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493959680 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493959952 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493960224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493968976 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493969248 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493945360 = OptionDescription(children=[option_139789493958320, option_139789493958592, option_139789493958864, option_139789493959136, option_139789493959408, option_139789493959680, option_139789493959952, option_139789493960224, option_139789493968976, option_139789493969248], **{'doc': u'', 'properties': (), 'name': 'file149'})
    option_139789493969520 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493969792 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493970064 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493970336 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493970608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493970880 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493971152 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493971424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493971696 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493971968 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493945680 = OptionDescription(children=[option_139789493969520, option_139789493969792, option_139789493970064, option_139789493970336, option_139789493970608, option_139789493970880, option_139789493971152, option_139789493971424, option_139789493971696, option_139789493971968], **{'doc': u'', 'properties': (), 'name': 'file150'})
    option_139789493972240 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493972512 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493981264 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493981536 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493981808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493982080 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493982352 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493982624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493982896 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493983168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493946000 = OptionDescription(children=[option_139789493972240, option_139789493972512, option_139789493981264, option_139789493981536, option_139789493981808, option_139789493982080, option_139789493982352, option_139789493982624, option_139789493982896, option_139789493983168], **{'doc': u'', 'properties': (), 'name': 'file151'})
    option_139789493983440 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493983712 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493983984 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493984256 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493984528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493984800 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493989456 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493989728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493990000 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493990272 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493946320 = OptionDescription(children=[option_139789493983440, option_139789493983712, option_139789493983984, option_139789493984256, option_139789493984528, option_139789493984800, option_139789493989456, option_139789493989728, option_139789493990000, option_139789493990272], **{'doc': u'', 'properties': (), 'name': 'file152'})
    option_139789493990544 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493990816 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493991088 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493991360 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493991632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493991904 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493992176 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493992448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493992720 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493992992 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493946640 = OptionDescription(children=[option_139789493990544, option_139789493990816, option_139789493991088, option_139789493991360, option_139789493991632, option_139789493991904, option_139789493992176, option_139789493992448, option_139789493992720, option_139789493992992], **{'doc': u'', 'properties': (), 'name': 'file153'})
    option_139789493997648 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493997920 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493998192 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493998464 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493998736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493999008 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493999280 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493999552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493999824 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494000096 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493946960 = OptionDescription(children=[option_139789493997648, option_139789493997920, option_139789493998192, option_139789493998464, option_139789493998736, option_139789493999008, option_139789493999280, option_139789493999552, option_139789493999824, option_139789494000096], **{'doc': u'', 'properties': (), 'name': 'file154'})
    option_139789494000368 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494000640 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494000912 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494001184 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494005840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494006112 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494006384 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494006656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494006928 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494007200 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493947280 = OptionDescription(children=[option_139789494000368, option_139789494000640, option_139789494000912, option_139789494001184, option_139789494005840, option_139789494006112, option_139789494006384, option_139789494006656, option_139789494006928, option_139789494007200], **{'doc': u'', 'properties': (), 'name': 'file155'})
    option_139789494007472 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494007744 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494008016 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494008288 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494008560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494008832 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494009104 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494009376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494014032 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494014304 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493947600 = OptionDescription(children=[option_139789494007472, option_139789494007744, option_139789494008016, option_139789494008288, option_139789494008560, option_139789494008832, option_139789494009104, option_139789494009376, option_139789494014032, option_139789494014304], **{'doc': u'', 'properties': (), 'name': 'file156'})
    option_139789494014576 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494014848 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494015120 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494015392 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494015664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494015936 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494016208 = UnicodeOption(**{'default': u'/usr/sbin/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494016480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494016752 = UnicodeOption(**{'default': u'/var/lib/creole/policy-rc.d', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494017024 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493947920 = OptionDescription(children=[option_139789494014576, option_139789494014848, option_139789494015120, option_139789494015392, option_139789494015664, option_139789494015936, option_139789494016208, option_139789494016480, option_139789494016752, option_139789494017024], **{'doc': u'', 'properties': (), 'name': 'file157'})
    option_139789494017296 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494017568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494022224 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494022496 = UnicodeOption(**{'default': u'/etc/ntp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494022768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494023040 = UnicodeOption(**{'default': u'/etc/ntp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494023312 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494023584 = UnicodeOption(**{'default': u'/var/lib/creole/ntp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494023856 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494030416 = OptionDescription(children=[option_139789494017296, option_139789494017568, option_139789494022224, option_139789494022496, option_139789494022768, option_139789494023040, option_139789494023312, option_139789494023584, option_139789494023856], **{'doc': u'', 'properties': (), 'name': 'file158'})
    option_139789494024128 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494024400 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494024672 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494024944 = UnicodeOption(**{'default': u'/etc/default/ntpdate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494025216 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494025488 = UnicodeOption(**{'default': u'/etc/default/ntpdate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494025760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494034512 = UnicodeOption(**{'default': u'/var/lib/creole/ntpdate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494034784 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494030736 = OptionDescription(children=[option_139789494024128, option_139789494024400, option_139789494024672, option_139789494024944, option_139789494025216, option_139789494025488, option_139789494025760, option_139789494034512, option_139789494034784], **{'doc': u'', 'properties': (), 'name': 'file159'})
    option_139789494035056 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494035328 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494035600 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494035872 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.sbin.ntpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494036144 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494036416 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.sbin.ntpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494036688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494036960 = UnicodeOption(**{'default': u'/var/lib/creole/ntpd.apparmor.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494037232 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494031056 = OptionDescription(children=[option_139789494035056, option_139789494035328, option_139789494035600, option_139789494035872, option_139789494036144, option_139789494036416, option_139789494036688, option_139789494036960, option_139789494037232], **{'doc': u'', 'properties': (), 'name': 'file160'})
    option_139789494037504 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494037776 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494038048 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494042704 = UnicodeOption(**{'default': u'/etc/network/if-up.d/ntpdate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494042976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494043248 = UnicodeOption(**{'default': u'755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494043520 = UnicodeOption(**{'default': u'/etc/network/if-up.d/ntpdate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494043792 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494044064 = UnicodeOption(**{'default': u'/var/lib/creole/ntpdate.if-up', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494044336 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789494031376 = OptionDescription(children=[option_139789494037504, option_139789494037776, option_139789494038048, option_139789494042704, option_139789494042976, option_139789494043248, option_139789494043520, option_139789494043792, option_139789494044064, option_139789494044336], **{'doc': u'', 'properties': (), 'name': 'file161'})
    option_139789494044608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494044880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494045152 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494045424 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494045696 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/90-lxc_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494045968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494046240 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494050896 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/90-lxc_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494051168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494051440 = UnicodeOption(**{'default': u'/var/lib/creole/90-lxc_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494051712 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494031696 = OptionDescription(children=[option_139789494044608, option_139789494044880, option_139789494045152, option_139789494045424, option_139789494045696, option_139789494045968, option_139789494046240, option_139789494050896, option_139789494051168, option_139789494051440, option_139789494051712], **{'doc': u'', 'properties': (), 'name': 'file162'})
    option_139789494051984 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494052256 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494052528 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494052800 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494053072 = UnicodeOption(**{'default': u'/etc/eole/.VirtEnabled.lock', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494053344 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494053616 = UnicodeOption(**{'default': u'/etc/eole/.VirtEnabled.lock', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494053888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494054160 = UnicodeOption(**{'default': u'/var/lib/creole/container.lockfile', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494054432 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494032016 = OptionDescription(children=[option_139789494051984, option_139789494052256, option_139789494052528, option_139789494052800, option_139789494053072, option_139789494053344, option_139789494053616, option_139789494053888, option_139789494054160, option_139789494054432], **{'doc': u'', 'properties': (), 'name': 'file163'})
    option_139789494063184 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494063456 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494063728 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494064000 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494064272 = UnicodeOption(**{'default': u'/etc/lxc/default.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494064544 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494064816 = UnicodeOption(**{'default': u'/etc/lxc/default.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494065088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494065360 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494065632 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494032336 = OptionDescription(children=[option_139789494063184, option_139789494063456, option_139789494063728, option_139789494064000, option_139789494064272, option_139789494064544, option_139789494064816, option_139789494065088, option_139789494065360, option_139789494065632], **{'doc': u'', 'properties': (), 'name': 'file164'})
    option_139789494065904 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494066176 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494066448 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494066720 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494071376 = UnicodeOption(**{'default': u'/etc/init/lxc.override', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494071648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494071920 = UnicodeOption(**{'default': u'/etc/init/lxc.override', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494072192 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494072464 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.override', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494072736 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494032656 = OptionDescription(children=[option_139789494065904, option_139789494066176, option_139789494066448, option_139789494066720, option_139789494071376, option_139789494071648, option_139789494071920, option_139789494072192, option_139789494072464, option_139789494072736], **{'doc': u'', 'properties': (), 'name': 'file165'})
    option_139789494073008 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494073280 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494073552 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494073824 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494074096 = UnicodeOption(**{'default': u'/etc/dnsmasq.d/lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494074368 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494074640 = UnicodeOption(**{'default': u'/etc/dnsmasq.d/lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494074912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494079568 = UnicodeOption(**{'default': u'/var/lib/creole/dnsmasq-lxc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494079840 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494032976 = OptionDescription(children=[option_139789494073008, option_139789494073280, option_139789494073552, option_139789494073824, option_139789494074096, option_139789494074368, option_139789494074640, option_139789494074912, option_139789494079568, option_139789494079840], **{'doc': u'', 'properties': (), 'name': 'file166'})
    option_139789494080112 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494080384 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494080656 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494080928 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494081200 = UnicodeOption(**{'default': u'/etc/default/lxc-net', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494081472 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494081744 = UnicodeOption(**{'default': u'/etc/default/lxc-net', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494082016 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494082288 = UnicodeOption(**{'default': u'/var/lib/creole/lxc-net.default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494082560 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789494033296 = OptionDescription(children=[option_139789494080112, option_139789494080384, option_139789494080656, option_139789494080928, option_139789494081200, option_139789494081472, option_139789494081744, option_139789494082016, option_139789494082288, option_139789494082560], **{'doc': u'', 'properties': (), 'name': 'file167'})
    option_139789494082832 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494083104 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494087760 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494088032 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494088304 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494088576 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494088848 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494089120 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494089392 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494089664 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494089936 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494033616 = OptionDescription(children=[option_139789494082832, option_139789494083104, option_139789494087760, option_139789494088032, option_139789494088304, option_139789494088576, option_139789494088848, option_139789494089120, option_139789494089392, option_139789494089664, option_139789494089936], **{'doc': u'', 'properties': (), 'name': 'file168'})
    option_139789494090208 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494090480 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494090752 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494091024 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494091296 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494095952 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494096224 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494096496 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494096768 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494097040 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494097312 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494033936 = OptionDescription(children=[option_139789494090208, option_139789494090480, option_139789494090752, option_139789494091024, option_139789494091296, option_139789494095952, option_139789494096224, option_139789494096496, option_139789494096768, option_139789494097040, option_139789494097312], **{'doc': u'', 'properties': (), 'name': 'file169'})
    option_139789494097584 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494097856 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494098128 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494098400 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494098672 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494098944 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494099216 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494099488 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494108240 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494108512 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494108784 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494116432 = OptionDescription(children=[option_139789494097584, option_139789494097856, option_139789494098128, option_139789494098400, option_139789494098672, option_139789494098944, option_139789494099216, option_139789494099488, option_139789494108240, option_139789494108512, option_139789494108784], **{'doc': u'', 'properties': (), 'name': 'file170'})
    option_139789494109056 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789494109328 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789494109600 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789494109872 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789494110144 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789494110416 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789494110688 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789494110960 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789494111232 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789494111504 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789494111776 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494116752 = OptionDescription(children=[option_139789494109056, option_139789494109328, option_139789494109600, option_139789494109872, option_139789494110144, option_139789494110416, option_139789494110688, option_139789494110960, option_139789494111232, option_139789494111504, option_139789494111776], **{'doc': u'', 'properties': (), 'name': 'file171'})
    option_139789493600336 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493600608 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493600880 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493601152 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493601424 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493601696 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493601968 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493602240 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493602512 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493602784 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493603056 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494117072 = OptionDescription(children=[option_139789493600336, option_139789493600608, option_139789493600880, option_139789493601152, option_139789493601424, option_139789493601696, option_139789493601968, option_139789493602240, option_139789493602512, option_139789493602784, option_139789493603056], **{'doc': u'', 'properties': (), 'name': 'file172'})
    option_139789493603328 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493603600 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493603872 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493608528 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493608800 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493609072 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493609344 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493609616 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493609888 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493610160 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493610432 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494117392 = OptionDescription(children=[option_139789493603328, option_139789493603600, option_139789493603872, option_139789493608528, option_139789493608800, option_139789493609072, option_139789493609344, option_139789493609616, option_139789493609888, option_139789493610160, option_139789493610432], **{'doc': u'', 'properties': (), 'name': 'file173'})
    option_139789493610704 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493610976 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493611248 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493611520 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493611792 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493612064 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493616720 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493616992 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493617264 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493617536 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493617808 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494117712 = OptionDescription(children=[option_139789493610704, option_139789493610976, option_139789493611248, option_139789493611520, option_139789493611792, option_139789493612064, option_139789493616720, option_139789493616992, option_139789493617264, option_139789493617536, option_139789493617808], **{'doc': u'', 'properties': (), 'name': 'file174'})
    option_139789493618080 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493618352 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493618624 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493618896 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493619168 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493619440 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493619712 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493619984 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493620256 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493624912 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493625184 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494118032 = OptionDescription(children=[option_139789493618080, option_139789493618352, option_139789493618624, option_139789493618896, option_139789493619168, option_139789493619440, option_139789493619712, option_139789493619984, option_139789493620256, option_139789493624912, option_139789493625184], **{'doc': u'', 'properties': (), 'name': 'file175'})
    option_139789493625456 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493625728 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493626000 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493626272 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493626544 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493626816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493627088 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493627360 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493627632 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493627904 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493628176 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494118352 = OptionDescription(children=[option_139789493625456, option_139789493625728, option_139789493626000, option_139789493626272, option_139789493626544, option_139789493626816, option_139789493627088, option_139789493627360, option_139789493627632, option_139789493627904, option_139789493628176], **{'doc': u'', 'properties': (), 'name': 'file176'})
    option_139789493628448 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493637200 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493637472 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493637744 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493638016 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493638288 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493638560 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493638832 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493639104 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493639376 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493639648 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494118672 = OptionDescription(children=[option_139789493628448, option_139789493637200, option_139789493637472, option_139789493637744, option_139789493638016, option_139789493638288, option_139789493638560, option_139789493638832, option_139789493639104, option_139789493639376, option_139789493639648], **{'doc': u'', 'properties': (), 'name': 'file177'})
    option_139789493639920 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493640192 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493640464 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493640736 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493645392 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493645664 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493645936 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493646208 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493646480 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493646752 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493647024 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494118992 = OptionDescription(children=[option_139789493639920, option_139789493640192, option_139789493640464, option_139789493640736, option_139789493645392, option_139789493645664, option_139789493645936, option_139789493646208, option_139789493646480, option_139789493646752, option_139789493647024], **{'doc': u'', 'properties': (), 'name': 'file178'})
    option_139789493647296 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493647568 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493647840 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493648112 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493648384 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493648656 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493648928 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493653584 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493653856 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493654128 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493654400 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494119312 = OptionDescription(children=[option_139789493647296, option_139789493647568, option_139789493647840, option_139789493648112, option_139789493648384, option_139789493648656, option_139789493648928, option_139789493653584, option_139789493653856, option_139789493654128, option_139789493654400], **{'doc': u'', 'properties': (), 'name': 'file179'})
    option_139789493654672 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493654944 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493655216 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493655488 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493655760 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493656032 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493656304 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493656576 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493656848 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493657120 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493665872 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494119632 = OptionDescription(children=[option_139789493654672, option_139789493654944, option_139789493655216, option_139789493655488, option_139789493655760, option_139789493656032, option_139789493656304, option_139789493656576, option_139789493656848, option_139789493657120, option_139789493665872], **{'doc': u'', 'properties': (), 'name': 'file180'})
    option_139789493666144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493666416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493666688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493666960 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493667232 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493667504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493667776 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493668048 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493668320 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493668592 = UnicodeOption(**{'default': u'/var/lib/creole/50-nat_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493668864 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789494119952 = OptionDescription(children=[option_139789493666144, option_139789493666416, option_139789493666688, option_139789493666960, option_139789493667232, option_139789493667504, option_139789493667776, option_139789493668048, option_139789493668320, option_139789493668592, option_139789493668864], **{'doc': u'', 'properties': (), 'name': 'file181'})
    option_139789493669136 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493669408 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493674064 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493674336 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493674608 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493674880 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493675152 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493675424 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493675696 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493675968 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493676240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493682256 = OptionDescription(children=[option_139789493669136, option_139789493669408, option_139789493674064, option_139789493674336, option_139789493674608, option_139789493674880, option_139789493675152, option_139789493675424, option_139789493675696, option_139789493675968, option_139789493676240], **{'doc': u'', 'properties': (), 'name': 'file182'})
    option_139789493676512 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493676784 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493677056 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493677328 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493677600 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493686352 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493686624 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493686896 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493687168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493687440 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493687712 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493682576 = OptionDescription(children=[option_139789493676512, option_139789493676784, option_139789493677056, option_139789493677328, option_139789493677600, option_139789493686352, option_139789493686624, option_139789493686896, option_139789493687168, option_139789493687440, option_139789493687712], **{'doc': u'', 'properties': (), 'name': 'file183'})
    option_139789493687984 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493688256 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493688528 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493688800 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493689072 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493689344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493689616 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493689888 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493698640 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493698912 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493699184 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493682896 = OptionDescription(children=[option_139789493687984, option_139789493688256, option_139789493688528, option_139789493688800, option_139789493689072, option_139789493689344, option_139789493689616, option_139789493689888, option_139789493698640, option_139789493698912, option_139789493699184], **{'doc': u'', 'properties': (), 'name': 'file184'})
    option_139789493699456 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493699728 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493700000 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493700272 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493700544 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493700816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493701088 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493701360 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493701632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493701904 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493702176 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493683216 = OptionDescription(children=[option_139789493699456, option_139789493699728, option_139789493700000, option_139789493700272, option_139789493700544, option_139789493700816, option_139789493701088, option_139789493701360, option_139789493701632, option_139789493701904, option_139789493702176], **{'doc': u'', 'properties': (), 'name': 'file185'})
    option_139789493710928 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493711200 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493711472 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493711744 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493712016 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493712288 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493712560 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493712832 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493713104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493713376 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493713648 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493683536 = OptionDescription(children=[option_139789493710928, option_139789493711200, option_139789493711472, option_139789493711744, option_139789493712016, option_139789493712288, option_139789493712560, option_139789493712832, option_139789493713104, option_139789493713376, option_139789493713648], **{'doc': u'', 'properties': (), 'name': 'file186'})
    option_139789493713920 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493714192 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493714464 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493719120 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493719392 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493719664 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493719936 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493720208 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493720480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493720752 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493721024 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493683856 = OptionDescription(children=[option_139789493713920, option_139789493714192, option_139789493714464, option_139789493719120, option_139789493719392, option_139789493719664, option_139789493719936, option_139789493720208, option_139789493720480, option_139789493720752, option_139789493721024], **{'doc': u'', 'properties': (), 'name': 'file187'})
    option_139789493721296 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493721568 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493721840 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493722112 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493722384 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493722656 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493731408 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493731680 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493731952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493732224 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493732496 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493684176 = OptionDescription(children=[option_139789493721296, option_139789493721568, option_139789493721840, option_139789493722112, option_139789493722384, option_139789493722656, option_139789493731408, option_139789493731680, option_139789493731952, option_139789493732224, option_139789493732496], **{'doc': u'', 'properties': (), 'name': 'file188'})
    option_139789493732768 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493733040 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493733312 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493733584 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493733856 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493734128 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493734400 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493734672 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493734944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493739600 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493739872 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493684496 = OptionDescription(children=[option_139789493732768, option_139789493733040, option_139789493733312, option_139789493733584, option_139789493733856, option_139789493734128, option_139789493734400, option_139789493734672, option_139789493734944, option_139789493739600, option_139789493739872], **{'doc': u'', 'properties': (), 'name': 'file189'})
    option_139789493740144 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493740416 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493740688 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493740960 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493741232 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493741504 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493741776 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493742048 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493742320 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493742592 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493742864 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493684816 = OptionDescription(children=[option_139789493740144, option_139789493740416, option_139789493740688, option_139789493740960, option_139789493741232, option_139789493741504, option_139789493741776, option_139789493742048, option_139789493742320, option_139789493742592, option_139789493742864], **{'doc': u'', 'properties': (), 'name': 'file190'})
    option_139789493743136 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493751888 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493752160 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493752432 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493752704 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493752976 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493753248 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493753520 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493753792 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493754064 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493754336 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493685136 = OptionDescription(children=[option_139789493743136, option_139789493751888, option_139789493752160, option_139789493752432, option_139789493752704, option_139789493752976, option_139789493753248, option_139789493753520, option_139789493753792, option_139789493754064, option_139789493754336], **{'doc': u'', 'properties': (), 'name': 'file191'})
    option_139789493754608 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493754880 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493755152 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493755424 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493760080 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493760352 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493760624 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493760896 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493761168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493761440 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493761712 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493685456 = OptionDescription(children=[option_139789493754608, option_139789493754880, option_139789493755152, option_139789493755424, option_139789493760080, option_139789493760352, option_139789493760624, option_139789493760896, option_139789493761168, option_139789493761440, option_139789493761712], **{'doc': u'', 'properties': (), 'name': 'file192'})
    option_139789493761984 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493762256 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493762528 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493762800 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493763072 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493763344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493763616 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493768272 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493768544 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493768816 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493769088 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493685776 = OptionDescription(children=[option_139789493761984, option_139789493762256, option_139789493762528, option_139789493762800, option_139789493763072, option_139789493763344, option_139789493763616, option_139789493768272, option_139789493768544, option_139789493768816, option_139789493769088], **{'doc': u'', 'properties': (), 'name': 'file193'})
    option_139789493769360 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493769632 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493769904 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493770176 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493770448 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493770720 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493770992 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493771264 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493771536 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493771808 = UnicodeOption(**{'default': u'/var/lib/creole/00-container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493776464 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493780560 = OptionDescription(children=[option_139789493769360, option_139789493769632, option_139789493769904, option_139789493770176, option_139789493770448, option_139789493770720, option_139789493770992, option_139789493771264, option_139789493771536, option_139789493771808, option_139789493776464], **{'doc': u'', 'properties': (), 'name': 'file194'})
    option_139789493776736 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493777008 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493777280 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493777552 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493777824 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493778096 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493778368 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493778640 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493778912 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493779184 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493780880 = OptionDescription(children=[option_139789493776736, option_139789493777008, option_139789493777280, option_139789493777552, option_139789493777824, option_139789493778096, option_139789493778368, option_139789493778640, option_139789493778912, option_139789493779184], **{'doc': u'', 'properties': (), 'name': 'file195'})
    option_139789493779456 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493779728 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493780000 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493792848 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493793120 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493793392 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493793664 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493793936 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493794208 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493794480 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493781200 = OptionDescription(children=[option_139789493779456, option_139789493779728, option_139789493780000, option_139789493792848, option_139789493793120, option_139789493793392, option_139789493793664, option_139789493793936, option_139789493794208, option_139789493794480], **{'doc': u'', 'properties': (), 'name': 'file196'})
    option_139789493794752 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493795024 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493795296 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493795568 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493795840 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493796112 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493796384 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493801040 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493801312 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493801584 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493781520 = OptionDescription(children=[option_139789493794752, option_139789493795024, option_139789493795296, option_139789493795568, option_139789493795840, option_139789493796112, option_139789493796384, option_139789493801040, option_139789493801312, option_139789493801584], **{'doc': u'', 'properties': (), 'name': 'file197'})
    option_139789493801856 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493802128 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493802400 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493802672 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493802944 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493803216 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493803488 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493803760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493804032 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493804304 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493781840 = OptionDescription(children=[option_139789493801856, option_139789493802128, option_139789493802400, option_139789493802672, option_139789493802944, option_139789493803216, option_139789493803488, option_139789493803760, option_139789493804032, option_139789493804304], **{'doc': u'', 'properties': (), 'name': 'file198'})
    option_139789493804576 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493813328 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493813600 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493813872 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493814144 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493814416 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493814688 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493814960 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493815232 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493815504 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493782160 = OptionDescription(children=[option_139789493804576, option_139789493813328, option_139789493813600, option_139789493813872, option_139789493814144, option_139789493814416, option_139789493814688, option_139789493814960, option_139789493815232, option_139789493815504], **{'doc': u'', 'properties': (), 'name': 'file199'})
    option_139789493815776 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493816048 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493816320 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493816592 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493816864 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493821520 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493821792 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493822064 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493822336 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493822608 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493782480 = OptionDescription(children=[option_139789493815776, option_139789493816048, option_139789493816320, option_139789493816592, option_139789493816864, option_139789493821520, option_139789493821792, option_139789493822064, option_139789493822336, option_139789493822608], **{'doc': u'', 'properties': (), 'name': 'file200'})
    option_139789493822880 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493823152 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493823424 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493823696 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493823968 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493824240 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493824512 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493824784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493825056 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493833808 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493782800 = OptionDescription(children=[option_139789493822880, option_139789493823152, option_139789493823424, option_139789493823696, option_139789493823968, option_139789493824240, option_139789493824512, option_139789493824784, option_139789493825056, option_139789493833808], **{'doc': u'', 'properties': (), 'name': 'file201'})
    option_139789493834080 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493834352 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493834624 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493834896 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493835168 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493835440 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493835712 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493835984 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493836256 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493836528 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493783120 = OptionDescription(children=[option_139789493834080, option_139789493834352, option_139789493834624, option_139789493834896, option_139789493835168, option_139789493835440, option_139789493835712, option_139789493835984, option_139789493836256, option_139789493836528], **{'doc': u'', 'properties': (), 'name': 'file202'})
    option_139789493836800 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493837072 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493837344 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493842000 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493842272 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493842544 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493842816 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493843088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493843360 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493843632 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493783440 = OptionDescription(children=[option_139789493836800, option_139789493837072, option_139789493837344, option_139789493842000, option_139789493842272, option_139789493842544, option_139789493842816, option_139789493843088, option_139789493843360, option_139789493843632], **{'doc': u'', 'properties': (), 'name': 'file203'})
    option_139789493843904 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493844176 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493844448 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493844720 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493844992 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493845264 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493845536 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493850192 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493850464 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493850736 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493783760 = OptionDescription(children=[option_139789493843904, option_139789493844176, option_139789493844448, option_139789493844720, option_139789493844992, option_139789493845264, option_139789493845536, option_139789493850192, option_139789493850464, option_139789493850736], **{'doc': u'', 'properties': (), 'name': 'file204'})
    option_139789493851008 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493851280 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493851552 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493851824 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493852096 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493852368 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493852640 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493852912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493853184 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493853456 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493784080 = OptionDescription(children=[option_139789493851008, option_139789493851280, option_139789493851552, option_139789493851824, option_139789493852096, option_139789493852368, option_139789493852640, option_139789493852912, option_139789493853184, option_139789493853456], **{'doc': u'', 'properties': (), 'name': 'file205'})
    option_139789493853728 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493342288 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493342560 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493342832 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493343104 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493343376 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493343648 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493343920 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493344192 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493344464 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493346384 = OptionDescription(children=[option_139789493853728, option_139789493342288, option_139789493342560, option_139789493342832, option_139789493343104, option_139789493343376, option_139789493343648, option_139789493343920, option_139789493344192, option_139789493344464], **{'doc': u'', 'properties': (), 'name': 'file206'})
    option_139789493344736 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493345008 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493345280 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493345552 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493345824 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493354576 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493354848 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493355120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493355392 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493355664 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493346704 = OptionDescription(children=[option_139789493344736, option_139789493345008, option_139789493345280, option_139789493345552, option_139789493345824, option_139789493354576, option_139789493354848, option_139789493355120, option_139789493355392, option_139789493355664], **{'doc': u'', 'properties': (), 'name': 'file207'})
    option_139789493355936 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493356208 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493356480 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493356752 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493357024 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493357296 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493357568 = UnicodeOption(**{'default': u'/etc/eole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493357840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493358112 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493362768 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789493347024 = OptionDescription(children=[option_139789493355936, option_139789493356208, option_139789493356480, option_139789493356752, option_139789493357024, option_139789493357296, option_139789493357568, option_139789493357840, option_139789493358112, option_139789493362768], **{'doc': u'', 'properties': (), 'name': 'file208'})
    option_139789493363040 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493363312 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493363584 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493363856 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493364128 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493364400 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493364672 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493364944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493365216 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493365488 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493347344 = OptionDescription(children=[option_139789493363040, option_139789493363312, option_139789493363584, option_139789493363856, option_139789493364128, option_139789493364400, option_139789493364672, option_139789493364944, option_139789493365216, option_139789493365488], **{'doc': u'', 'properties': (), 'name': 'file209'})
    option_139789493365760 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493366032 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493366304 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493375056 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493375328 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493375600 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493375872 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493376144 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493376416 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493376688 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493347664 = OptionDescription(children=[option_139789493365760, option_139789493366032, option_139789493366304, option_139789493375056, option_139789493375328, option_139789493375600, option_139789493375872, option_139789493376144, option_139789493376416, option_139789493376688], **{'doc': u'', 'properties': (), 'name': 'file210'})
    option_139789493376960 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493377232 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493377504 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493377776 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493378048 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493378320 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493378592 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493383248 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493383520 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493383792 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493347984 = OptionDescription(children=[option_139789493376960, option_139789493377232, option_139789493377504, option_139789493377776, option_139789493378048, option_139789493378320, option_139789493378592, option_139789493383248, option_139789493383520, option_139789493383792], **{'doc': u'', 'properties': (), 'name': 'file211'})
    option_139789493384064 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493384336 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493384608 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493384880 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493385152 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493385424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493385696 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493385968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493386240 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493386512 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493348304 = OptionDescription(children=[option_139789493384064, option_139789493384336, option_139789493384608, option_139789493384880, option_139789493385152, option_139789493385424, option_139789493385696, option_139789493385968, option_139789493386240, option_139789493386512], **{'doc': u'', 'properties': (), 'name': 'file212'})
    option_139789493386784 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493391440 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493391712 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493391984 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493392256 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493392528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493392800 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493393072 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493393344 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493393616 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493348624 = OptionDescription(children=[option_139789493386784, option_139789493391440, option_139789493391712, option_139789493391984, option_139789493392256, option_139789493392528, option_139789493392800, option_139789493393072, option_139789493393344, option_139789493393616], **{'doc': u'', 'properties': (), 'name': 'file213'})
    option_139789493393888 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493394160 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493394432 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493394704 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493394976 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493399632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493399904 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493400176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493400448 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493400720 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493348944 = OptionDescription(children=[option_139789493393888, option_139789493394160, option_139789493394432, option_139789493394704, option_139789493394976, option_139789493399632, option_139789493399904, option_139789493400176, option_139789493400448, option_139789493400720], **{'doc': u'', 'properties': (), 'name': 'file214'})
    option_139789493400992 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493401264 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493401536 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493401808 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493402080 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493402352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493402624 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493402896 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493403168 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493407824 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493349264 = OptionDescription(children=[option_139789493400992, option_139789493401264, option_139789493401536, option_139789493401808, option_139789493402080, option_139789493402352, option_139789493402624, option_139789493402896, option_139789493403168, option_139789493407824], **{'doc': u'', 'properties': (), 'name': 'file215'})
    option_139789493408096 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493408368 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493408640 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493408912 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493409184 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493409456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493409728 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493410000 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493410272 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493410544 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493349584 = OptionDescription(children=[option_139789493408096, option_139789493408368, option_139789493408640, option_139789493408912, option_139789493409184, option_139789493409456, option_139789493409728, option_139789493410000, option_139789493410272, option_139789493410544], **{'doc': u'', 'properties': (), 'name': 'file216'})
    option_139789493410816 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493411088 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493411360 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493416016 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493416288 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493416560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493416832 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493417104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493417376 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493417648 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493349904 = OptionDescription(children=[option_139789493410816, option_139789493411088, option_139789493411360, option_139789493416016, option_139789493416288, option_139789493416560, option_139789493416832, option_139789493417104, option_139789493417376, option_139789493417648], **{'doc': u'', 'properties': (), 'name': 'file217'})
    option_139789493417920 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493418192 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493418464 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493418736 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493419008 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493419280 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493419552 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493424208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493424480 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493424752 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493432400 = OptionDescription(children=[option_139789493417920, option_139789493418192, option_139789493418464, option_139789493418736, option_139789493419008, option_139789493419280, option_139789493419552, option_139789493424208, option_139789493424480, option_139789493424752], **{'doc': u'', 'properties': (), 'name': 'file218'})
    option_139789493425024 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493425296 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493425568 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493425840 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493426112 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493426384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493426656 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493426928 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493427200 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493427472 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493432720 = OptionDescription(children=[option_139789493425024, option_139789493425296, option_139789493425568, option_139789493425840, option_139789493426112, option_139789493426384, option_139789493426656, option_139789493426928, option_139789493427200, option_139789493427472], **{'doc': u'', 'properties': (), 'name': 'file219'})
    option_139789493427744 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493440592 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493440864 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493441136 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493441408 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493441680 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493441952 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493442224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493442496 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493442768 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493433040 = OptionDescription(children=[option_139789493427744, option_139789493440592, option_139789493440864, option_139789493441136, option_139789493441408, option_139789493441680, option_139789493441952, option_139789493442224, option_139789493442496, option_139789493442768], **{'doc': u'', 'properties': (), 'name': 'file220'})
    option_139789493443040 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493443312 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493443584 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493443856 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493444128 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493448784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493449056 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493449328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493449600 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493449872 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493433360 = OptionDescription(children=[option_139789493443040, option_139789493443312, option_139789493443584, option_139789493443856, option_139789493444128, option_139789493448784, option_139789493449056, option_139789493449328, option_139789493449600, option_139789493449872], **{'doc': u'', 'properties': (), 'name': 'file221'})
    option_139789493450144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493450416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493450688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493450960 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493451232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493451504 = UnicodeOption(**{'default': u'/etc/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493451776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493452048 = UnicodeOption(**{'default': u'/var/lib/creole/hosts.deny', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493452320 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493433680 = OptionDescription(children=[option_139789493450144, option_139789493450416, option_139789493450688, option_139789493450960, option_139789493451232, option_139789493451504, option_139789493451776, option_139789493452048, option_139789493452320], **{'doc': u'', 'properties': (), 'name': 'file222'})
    option_139789493465168 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493465440 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493465712 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493465984 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493466256 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493466528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493466800 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493467072 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493467344 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493467616 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493434000 = OptionDescription(children=[option_139789493465168, option_139789493465440, option_139789493465712, option_139789493465984, option_139789493466256, option_139789493466528, option_139789493466800, option_139789493467072, option_139789493467344, option_139789493467616], **{'doc': u'', 'properties': (), 'name': 'file223'})
    option_139789493467888 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493468160 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493468432 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493468704 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493473360 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493473632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493473904 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493474176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493474448 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493474720 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493434320 = OptionDescription(children=[option_139789493467888, option_139789493468160, option_139789493468432, option_139789493468704, option_139789493473360, option_139789493473632, option_139789493473904, option_139789493474176, option_139789493474448, option_139789493474720], **{'doc': u'', 'properties': (), 'name': 'file224'})
    option_139789493474992 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493475264 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493475536 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493475808 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493476080 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493476352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493476624 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493476896 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493481552 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493481824 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493434640 = OptionDescription(children=[option_139789493474992, option_139789493475264, option_139789493475536, option_139789493475808, option_139789493476080, option_139789493476352, option_139789493476624, option_139789493476896, option_139789493481552, option_139789493481824], **{'doc': u'', 'properties': (), 'name': 'file225'})
    option_139789493482096 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493482368 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493482640 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493482912 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493483184 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493483456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493483728 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493484000 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493484272 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493484544 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493434960 = OptionDescription(children=[option_139789493482096, option_139789493482368, option_139789493482640, option_139789493482912, option_139789493483184, option_139789493483456, option_139789493483728, option_139789493484000, option_139789493484272, option_139789493484544], **{'doc': u'', 'properties': (), 'name': 'file226'})
    option_139789493484816 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493485088 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493493840 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493494112 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493494384 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493494656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493494928 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493495200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493495472 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493495744 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493435280 = OptionDescription(children=[option_139789493484816, option_139789493485088, option_139789493493840, option_139789493494112, option_139789493494384, option_139789493494656, option_139789493494928, option_139789493495200, option_139789493495472, option_139789493495744], **{'doc': u'', 'properties': (), 'name': 'file227'})
    option_139789493496016 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493496288 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493496560 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493496832 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493497104 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493497376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493502032 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493502304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493502576 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493502848 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493435600 = OptionDescription(children=[option_139789493496016, option_139789493496288, option_139789493496560, option_139789493496832, option_139789493497104, option_139789493497376, option_139789493502032, option_139789493502304, option_139789493502576, option_139789493502848], **{'doc': u'', 'properties': (), 'name': 'file228'})
    option_139789493503120 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493503392 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493503664 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493503936 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493504208 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493504480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493504752 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493505024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493505296 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493505568 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493435920 = OptionDescription(children=[option_139789493503120, option_139789493503392, option_139789493503664, option_139789493503936, option_139789493504208, option_139789493504480, option_139789493504752, option_139789493505024, option_139789493505296, option_139789493505568], **{'doc': u'', 'properties': (), 'name': 'file229'})
    option_139789493510224 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493510496 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493510768 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493511040 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493511312 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493511584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493511856 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493512128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493512400 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493512672 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493518416 = OptionDescription(children=[option_139789493510224, option_139789493510496, option_139789493510768, option_139789493511040, option_139789493511312, option_139789493511584, option_139789493511856, option_139789493512128, option_139789493512400, option_139789493512672], **{'doc': u'', 'properties': (), 'name': 'file230'})
    option_139789493512944 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493513216 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493513488 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493513760 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493522512 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493522784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493523056 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493523328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493523600 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493523872 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493518736 = OptionDescription(children=[option_139789493512944, option_139789493513216, option_139789493513488, option_139789493513760, option_139789493522512, option_139789493522784, option_139789493523056, option_139789493523328, option_139789493523600, option_139789493523872], **{'doc': u'', 'properties': (), 'name': 'file231'})
    option_139789493524144 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493524416 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493524688 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493524960 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493525232 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493525504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493525776 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493526048 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493530704 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493530976 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493519056 = OptionDescription(children=[option_139789493524144, option_139789493524416, option_139789493524688, option_139789493524960, option_139789493525232, option_139789493525504, option_139789493525776, option_139789493526048, option_139789493530704, option_139789493530976], **{'doc': u'', 'properties': (), 'name': 'file232'})
    option_139789493531248 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493531520 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493531792 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493532064 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493532336 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493532608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493532880 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493533152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493533424 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493533696 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493519376 = OptionDescription(children=[option_139789493531248, option_139789493531520, option_139789493531792, option_139789493532064, option_139789493532336, option_139789493532608, option_139789493532880, option_139789493533152, option_139789493533424, option_139789493533696], **{'doc': u'', 'properties': (), 'name': 'file233'})
    option_139789493533968 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493534240 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493542992 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493543264 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493543536 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493543808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493544080 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493544352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493544624 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493544896 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493519696 = OptionDescription(children=[option_139789493533968, option_139789493534240, option_139789493542992, option_139789493543264, option_139789493543536, option_139789493543808, option_139789493544080, option_139789493544352, option_139789493544624, option_139789493544896], **{'doc': u'', 'properties': (), 'name': 'file234'})
    option_139789493545168 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493545440 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493545712 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493545984 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493546256 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493546528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493551184 = UnicodeOption(**{'default': u'../config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493551456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493551728 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.config', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493552000 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493520016 = OptionDescription(children=[option_139789493545168, option_139789493545440, option_139789493545712, option_139789493545984, option_139789493546256, option_139789493546528, option_139789493551184, option_139789493551456, option_139789493551728, option_139789493552000], **{'doc': u'', 'properties': (), 'name': 'file235'})
    option_139789493552272 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493552544 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493552816 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493553088 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493553360 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493553632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493553904 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493554176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493554448 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493554720 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493520336 = OptionDescription(children=[option_139789493552272, option_139789493552544, option_139789493552816, option_139789493553088, option_139789493553360, option_139789493553632, option_139789493553904, option_139789493554176, option_139789493554448, option_139789493554720], **{'doc': u'', 'properties': (), 'name': 'file236'})
    option_139789493559376 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493559648 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493559920 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493560192 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493560464 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493560736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493561008 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493561280 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493561552 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493561824 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493520656 = OptionDescription(children=[option_139789493559376, option_139789493559648, option_139789493559920, option_139789493560192, option_139789493560464, option_139789493560736, option_139789493561008, option_139789493561280, option_139789493561552, option_139789493561824], **{'doc': u'', 'properties': (), 'name': 'file237'})
    option_139789493562096 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493562368 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493562640 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493562912 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493571664 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493571936 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493572208 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493572480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493572752 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493573024 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493520976 = OptionDescription(children=[option_139789493562096, option_139789493562368, option_139789493562640, option_139789493562912, option_139789493571664, option_139789493571936, option_139789493572208, option_139789493572480, option_139789493572752, option_139789493573024], **{'doc': u'', 'properties': (), 'name': 'file238'})
    option_139789493573296 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493573568 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493573840 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493574112 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493574384 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493574656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493574928 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493575200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493579856 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493580128 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493521296 = OptionDescription(children=[option_139789493573296, option_139789493573568, option_139789493573840, option_139789493574112, option_139789493574384, option_139789493574656, option_139789493574928, option_139789493575200, option_139789493579856, option_139789493580128], **{'doc': u'', 'properties': (), 'name': 'file239'})
    option_139789493580400 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493580672 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493580944 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493581216 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493581488 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493581760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493582032 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493582304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493582576 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493582848 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493521616 = OptionDescription(children=[option_139789493580400, option_139789493580672, option_139789493580944, option_139789493581216, option_139789493581488, option_139789493581760, option_139789493582032, option_139789493582304, option_139789493582576, option_139789493582848], **{'doc': u'', 'properties': (), 'name': 'file240'})
    option_139789493583120 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493583392 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493076048 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493076320 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493076592 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493076864 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493077136 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493077408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493077680 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493077952 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493521936 = OptionDescription(children=[option_139789493583120, option_139789493583392, option_139789493076048, option_139789493076320, option_139789493076592, option_139789493076864, option_139789493077136, option_139789493077408, option_139789493077680, option_139789493077952], **{'doc': u'', 'properties': (), 'name': 'file241'})
    option_139789493078224 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493078496 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493078768 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493079040 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493079312 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493079584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493084240 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493084512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493084784 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493085056 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493092432 = OptionDescription(children=[option_139789493078224, option_139789493078496, option_139789493078768, option_139789493079040, option_139789493079312, option_139789493079584, option_139789493084240, option_139789493084512, option_139789493084784, option_139789493085056], **{'doc': u'', 'properties': (), 'name': 'file242'})
    option_139789493085328 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493085600 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493085872 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493086144 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493086416 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493086688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493086960 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493087232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493087504 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493087776 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493092752 = OptionDescription(children=[option_139789493085328, option_139789493085600, option_139789493085872, option_139789493086144, option_139789493086416, option_139789493086688, option_139789493086960, option_139789493087232, option_139789493087504, option_139789493087776], **{'doc': u'', 'properties': (), 'name': 'file243'})
    option_139789493100624 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493100896 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493101168 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493101440 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493101712 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493101984 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493102256 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493102528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493102800 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493103072 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493093072 = OptionDescription(children=[option_139789493100624, option_139789493100896, option_139789493101168, option_139789493101440, option_139789493101712, option_139789493101984, option_139789493102256, option_139789493102528, option_139789493102800, option_139789493103072], **{'doc': u'', 'properties': (), 'name': 'file244'})
    option_139789493103344 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493103616 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493103888 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493104160 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493108816 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493109088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493109360 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493109632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493109904 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493110176 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493093392 = OptionDescription(children=[option_139789493103344, option_139789493103616, option_139789493103888, option_139789493104160, option_139789493108816, option_139789493109088, option_139789493109360, option_139789493109632, option_139789493109904, option_139789493110176], **{'doc': u'', 'properties': (), 'name': 'file245'})
    option_139789493110448 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493110720 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493110992 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493111264 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493111536 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493111808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493112080 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493112352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493117008 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493117280 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493093712 = OptionDescription(children=[option_139789493110448, option_139789493110720, option_139789493110992, option_139789493111264, option_139789493111536, option_139789493111808, option_139789493112080, option_139789493112352, option_139789493117008, option_139789493117280], **{'doc': u'', 'properties': (), 'name': 'file246'})
    option_139789493117552 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493117824 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493118096 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493118368 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493118640 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493118912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493119184 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493119456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493119728 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493120000 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493094032 = OptionDescription(children=[option_139789493117552, option_139789493117824, option_139789493118096, option_139789493118368, option_139789493118640, option_139789493118912, option_139789493119184, option_139789493119456, option_139789493119728, option_139789493120000], **{'doc': u'', 'properties': (), 'name': 'file247'})
    option_139789493120272 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493120544 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493129296 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493129568 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493129840 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493130112 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493130384 = UnicodeOption(**{'default': u'../fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493130656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493130928 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.fstab', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493131200 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493094352 = OptionDescription(children=[option_139789493120272, option_139789493120544, option_139789493129296, option_139789493129568, option_139789493129840, option_139789493130112, option_139789493130384, option_139789493130656, option_139789493130928, option_139789493131200], **{'doc': u'', 'properties': (), 'name': 'file248'})
    option_139789493131472 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493131744 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493132016 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493132288 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493132560 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493132832 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493137488 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493137760 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493138032 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493138304 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493138576 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493094672 = OptionDescription(children=[option_139789493131472, option_139789493131744, option_139789493132016, option_139789493132288, option_139789493132560, option_139789493132832, option_139789493137488, option_139789493137760, option_139789493138032, option_139789493138304, option_139789493138576], **{'doc': u'', 'properties': (), 'name': 'file249'})
    option_139789493138848 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493139120 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493139392 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493139664 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493139936 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493140208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493140480 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493140752 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493141024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493145680 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493145952 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493094992 = OptionDescription(children=[option_139789493138848, option_139789493139120, option_139789493139392, option_139789493139664, option_139789493139936, option_139789493140208, option_139789493140480, option_139789493140752, option_139789493141024, option_139789493145680, option_139789493145952], **{'doc': u'', 'properties': (), 'name': 'file250'})
    option_139789493146224 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493146496 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493146768 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493147040 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493147312 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493147584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493147856 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493148128 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493148400 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493148672 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493148944 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493095312 = OptionDescription(children=[option_139789493146224, option_139789493146496, option_139789493146768, option_139789493147040, option_139789493147312, option_139789493147584, option_139789493147856, option_139789493148128, option_139789493148400, option_139789493148672, option_139789493148944], **{'doc': u'', 'properties': (), 'name': 'file251'})
    option_139789493149216 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493157968 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493158240 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493158512 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493158784 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493159056 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493159328 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493159600 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493159872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493160144 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493160416 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493095632 = OptionDescription(children=[option_139789493149216, option_139789493157968, option_139789493158240, option_139789493158512, option_139789493158784, option_139789493159056, option_139789493159328, option_139789493159600, option_139789493159872, option_139789493160144, option_139789493160416], **{'doc': u'', 'properties': (), 'name': 'file252'})
    option_139789493160688 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493160960 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493161232 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493161504 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493166160 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493166432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493166704 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493166976 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493167248 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493167520 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493167792 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493095952 = OptionDescription(children=[option_139789493160688, option_139789493160960, option_139789493161232, option_139789493161504, option_139789493166160, option_139789493166432, option_139789493166704, option_139789493166976, option_139789493167248, option_139789493167520, option_139789493167792], **{'doc': u'', 'properties': (), 'name': 'file253'})
    option_139789493168064 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493168336 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493168608 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493168880 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493169152 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493169424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493169696 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493174352 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493174624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493174896 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493175168 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493178448 = OptionDescription(children=[option_139789493168064, option_139789493168336, option_139789493168608, option_139789493168880, option_139789493169152, option_139789493169424, option_139789493169696, option_139789493174352, option_139789493174624, option_139789493174896, option_139789493175168], **{'doc': u'', 'properties': (), 'name': 'file254'})
    option_139789493175440 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493175712 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493175984 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493176256 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493176528 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493176800 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493177072 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493177344 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493177616 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493177888 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493186640 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493178768 = OptionDescription(children=[option_139789493175440, option_139789493175712, option_139789493175984, option_139789493176256, option_139789493176528, option_139789493176800, option_139789493177072, option_139789493177344, option_139789493177616, option_139789493177888, option_139789493186640], **{'doc': u'', 'properties': (), 'name': 'file255'})
    option_139789493186912 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493187184 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493187456 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493187728 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493188000 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493188272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493188544 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493188816 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493189088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493189360 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493189632 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493179088 = OptionDescription(children=[option_139789493186912, option_139789493187184, option_139789493187456, option_139789493187728, option_139789493188000, option_139789493188272, option_139789493188544, option_139789493188816, option_139789493189088, option_139789493189360, option_139789493189632], **{'doc': u'', 'properties': (), 'name': 'file256'})
    option_139789493189904 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493190176 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493198928 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493199200 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493199472 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493199744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493200016 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493200288 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493200560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493200832 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493201104 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493179408 = OptionDescription(children=[option_139789493189904, option_139789493190176, option_139789493198928, option_139789493199200, option_139789493199472, option_139789493199744, option_139789493200016, option_139789493200288, option_139789493200560, option_139789493200832, option_139789493201104], **{'doc': u'', 'properties': (), 'name': 'file257'})
    option_139789493201376 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493201648 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493201920 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493202192 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493202464 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493207120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493207392 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493207664 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493207936 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493208208 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493208480 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493179728 = OptionDescription(children=[option_139789493201376, option_139789493201648, option_139789493201920, option_139789493202192, option_139789493202464, option_139789493207120, option_139789493207392, option_139789493207664, option_139789493207936, option_139789493208208, option_139789493208480], **{'doc': u'', 'properties': (), 'name': 'file258'})
    option_139789493208752 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493209024 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493209296 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493209568 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493209840 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493210112 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493210384 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493210656 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493215312 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493215584 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493215856 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493180048 = OptionDescription(children=[option_139789493208752, option_139789493209024, option_139789493209296, option_139789493209568, option_139789493209840, option_139789493210112, option_139789493210384, option_139789493210656, option_139789493215312, option_139789493215584, option_139789493215856], **{'doc': u'', 'properties': (), 'name': 'file259'})
    option_139789493216128 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493216400 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493216672 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493216944 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493217216 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493217488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493217760 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493218032 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493218304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493218576 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493218848 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493180368 = OptionDescription(children=[option_139789493216128, option_139789493216400, option_139789493216672, option_139789493216944, option_139789493217216, option_139789493217488, option_139789493217760, option_139789493218032, option_139789493218304, option_139789493218576, option_139789493218848], **{'doc': u'', 'properties': (), 'name': 'file260'})
    option_139789493227600 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493227872 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493228144 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493228416 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493228688 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493228960 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493229232 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493229504 = UnicodeOption(**{'default': u'../devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493229776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493230048 = UnicodeOption(**{'default': u'/var/lib/creole/lxc.devices.hook', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493230320 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493180688 = OptionDescription(children=[option_139789493227600, option_139789493227872, option_139789493228144, option_139789493228416, option_139789493228688, option_139789493228960, option_139789493229232, option_139789493229504, option_139789493229776, option_139789493230048, option_139789493230320], **{'doc': u'', 'properties': (), 'name': 'file261'})
    option_139789493230592 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493230864 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493231136 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493235792 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493236064 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493236336 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493236608 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493236880 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493237152 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493237424 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493181008 = OptionDescription(children=[option_139789493230592, option_139789493230864, option_139789493231136, option_139789493235792, option_139789493236064, option_139789493236336, option_139789493236608, option_139789493236880, option_139789493237152, option_139789493237424], **{'doc': u'', 'properties': (), 'name': 'file262'})
    option_139789493237696 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493237968 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493238240 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493238512 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493238784 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493239056 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493239328 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493243984 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493244256 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493244528 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493181328 = OptionDescription(children=[option_139789493237696, option_139789493237968, option_139789493238240, option_139789493238512, option_139789493238784, option_139789493239056, option_139789493239328, option_139789493243984, option_139789493244256, option_139789493244528], **{'doc': u'', 'properties': (), 'name': 'file263'})
    option_139789493244800 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493245072 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493245344 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493245616 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493245888 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493246160 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493246432 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493246704 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493246976 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493247248 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493181648 = OptionDescription(children=[option_139789493244800, option_139789493245072, option_139789493245344, option_139789493245616, option_139789493245888, option_139789493246160, option_139789493246432, option_139789493246704, option_139789493246976, option_139789493247248], **{'doc': u'', 'properties': (), 'name': 'file264'})
    option_139789493247520 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493256272 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493256544 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493256816 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493257088 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493257360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493257632 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493257904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493258176 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493258448 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493181968 = OptionDescription(children=[option_139789493247520, option_139789493256272, option_139789493256544, option_139789493256816, option_139789493257088, option_139789493257360, option_139789493257632, option_139789493257904, option_139789493258176, option_139789493258448], **{'doc': u'', 'properties': (), 'name': 'file265'})
    option_139789493258720 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493258992 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493259264 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493259536 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493259808 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493264464 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493264736 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493265008 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493265280 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493265552 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493268560 = OptionDescription(children=[option_139789493258720, option_139789493258992, option_139789493259264, option_139789493259536, option_139789493259808, option_139789493264464, option_139789493264736, option_139789493265008, option_139789493265280, option_139789493265552], **{'doc': u'', 'properties': (), 'name': 'file266'})
    option_139789493265824 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493266096 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493266368 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493266640 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493266912 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493267184 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493267456 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493267728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493268000 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493276752 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493268880 = OptionDescription(children=[option_139789493265824, option_139789493266096, option_139789493266368, option_139789493266640, option_139789493266912, option_139789493267184, option_139789493267456, option_139789493267728, option_139789493268000, option_139789493276752], **{'doc': u'', 'properties': (), 'name': 'file267'})
    option_139789493277024 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493277296 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493277568 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493277840 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493278112 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493278384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493278656 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493278928 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493279200 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493279472 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493269200 = OptionDescription(children=[option_139789493277024, option_139789493277296, option_139789493277568, option_139789493277840, option_139789493278112, option_139789493278384, option_139789493278656, option_139789493278928, option_139789493279200, option_139789493279472], **{'doc': u'', 'properties': (), 'name': 'file268'})
    option_139789493279744 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493280016 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493280288 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493289040 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493289312 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493289584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493289856 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493290128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493290400 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493290672 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493269520 = OptionDescription(children=[option_139789493279744, option_139789493280016, option_139789493280288, option_139789493289040, option_139789493289312, option_139789493289584, option_139789493289856, option_139789493290128, option_139789493290400, option_139789493290672], **{'doc': u'', 'properties': (), 'name': 'file269'})
    option_139789493290944 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493291216 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493291488 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493291760 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493292032 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493292304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493292576 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493297232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493297504 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493297776 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493269840 = OptionDescription(children=[option_139789493290944, option_139789493291216, option_139789493291488, option_139789493291760, option_139789493292032, option_139789493292304, option_139789493292576, option_139789493297232, option_139789493297504, option_139789493297776], **{'doc': u'', 'properties': (), 'name': 'file270'})
    option_139789493298048 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493298320 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493298592 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493298864 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493299136 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493299408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493299680 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493299952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493300224 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493300496 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493270160 = OptionDescription(children=[option_139789493298048, option_139789493298320, option_139789493298592, option_139789493298864, option_139789493299136, option_139789493299408, option_139789493299680, option_139789493299952, option_139789493300224, option_139789493300496], **{'doc': u'', 'properties': (), 'name': 'file271'})
    option_139789493300768 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493305424 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493305696 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493305968 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493306240 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493306512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493306784 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493307056 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493307328 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493307600 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493270480 = OptionDescription(children=[option_139789493300768, option_139789493305424, option_139789493305696, option_139789493305968, option_139789493306240, option_139789493306512, option_139789493306784, option_139789493307056, option_139789493307328, option_139789493307600], **{'doc': u'', 'properties': (), 'name': 'file272'})
    option_139789493307872 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493308144 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493308416 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493308688 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493308960 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493313616 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493313888 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493314160 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493314432 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493314704 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493270800 = OptionDescription(children=[option_139789493307872, option_139789493308144, option_139789493308416, option_139789493308688, option_139789493308960, option_139789493313616, option_139789493313888, option_139789493314160, option_139789493314432, option_139789493314704], **{'doc': u'', 'properties': (), 'name': 'file273'})
    option_139789493314976 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493315248 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493315520 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493315792 = UnicodeOption(**{'default': u'container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493316064 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493316336 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493316608 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493316880 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493317152 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf.container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493325904 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789493271120 = OptionDescription(children=[option_139789493314976, option_139789493315248, option_139789493315520, option_139789493315792, option_139789493316064, option_139789493316336, option_139789493316608, option_139789493316880, option_139789493317152, option_139789493325904], **{'doc': u'', 'properties': (), 'name': 'file274'})
    option_139789493326176 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493326448 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493326720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493326992 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493327264 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493327536 = UnicodeOption(**{'default': u'/etc/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493327808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493328080 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493328352 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493271440 = OptionDescription(children=[option_139789493326176, option_139789493326448, option_139789493326720, option_139789493326992, option_139789493327264, option_139789493327536, option_139789493327808, option_139789493328080, option_139789493328352], **{'doc': u'', 'properties': (), 'name': 'file275'})
    option_139789493328624 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493328896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493329168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493329440 = UnicodeOption(**{'default': u'/etc/logrotate.d/rsyslog', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492813904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492814176 = UnicodeOption(**{'default': u'/etc/logrotate.d/rsyslog', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492814448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492814720 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492814992 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493271760 = OptionDescription(children=[option_139789493328624, option_139789493328896, option_139789493329168, option_139789493329440, option_139789492813904, option_139789492814176, option_139789492814448, option_139789492814720, option_139789492814992], **{'doc': u'', 'properties': (), 'name': 'file276'})
    option_139789492815264 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492815536 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492815808 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492816080 = UnicodeOption(**{'default': u'certfiles', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492816352 = UnicodeOption(**{'default': u'/etc/rsyslog.d/ssl/rsyslog-request.tmpl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492816624 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492816896 = UnicodeOption(**{'default': u'/etc/rsyslog.d/ssl/rsyslog-request.tmpl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492817168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492817440 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog-request.tmpl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492822096 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493272080 = OptionDescription(children=[option_139789492815264, option_139789492815536, option_139789492815808, option_139789492816080, option_139789492816352, option_139789492816624, option_139789492816896, option_139789492817168, option_139789492817440, option_139789492822096], **{'doc': u'', 'properties': (), 'name': 'file277'})
    option_139789492822368 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492822640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492822912 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492823184 = UnicodeOption(**{'default': u'certfiles', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492823456 = UnicodeOption(**{'default': u'/etc/rsyslog.d/ssl/rsyslog-ca.tmpl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492823728 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492824000 = UnicodeOption(**{'default': u'/etc/rsyslog.d/ssl/rsyslog-ca.tmpl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492824272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492824544 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog-ca.tmpl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492824816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492830288 = OptionDescription(children=[option_139789492822368, option_139789492822640, option_139789492822912, option_139789492823184, option_139789492823456, option_139789492823728, option_139789492824000, option_139789492824272, option_139789492824544, option_139789492824816], **{'doc': u'', 'properties': (), 'name': 'file278'})
    option_139789492825088 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492825360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492825632 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492834384 = UnicodeOption(**{'default': u'/etc/rsyslog.d/aggregation/all.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492834656 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492834928 = UnicodeOption(**{'default': u'/etc/rsyslog.d/aggregation/all.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492835200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492835472 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_aggregation.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492835744 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492830608 = OptionDescription(children=[option_139789492825088, option_139789492825360, option_139789492825632, option_139789492834384, option_139789492834656, option_139789492834928, option_139789492835200, option_139789492835472, option_139789492835744], **{'doc': u'', 'properties': (), 'name': 'file279'})
    option_139789492836016 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492836288 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492836560 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492836832 = UnicodeOption(**{'default': u'/etc/rsyslog.d/templates/eole_templates.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492837104 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492837376 = UnicodeOption(**{'default': u'/etc/rsyslog.d/templates/eole_templates.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492837648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492837920 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_templates.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492846672 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492830928 = OptionDescription(children=[option_139789492836016, option_139789492836288, option_139789492836560, option_139789492836832, option_139789492837104, option_139789492837376, option_139789492837648, option_139789492837920, option_139789492846672], **{'doc': u'', 'properties': (), 'name': 'file280'})
    option_139789492846944 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492847216 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492847488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492847760 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-views/errors.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492848032 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492848304 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-views/errors.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492848576 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492848848 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_views_errors.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492849120 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492831248 = OptionDescription(children=[option_139789492846944, option_139789492847216, option_139789492847488, option_139789492847760, option_139789492848032, option_139789492848304, option_139789492848576, option_139789492848848, option_139789492849120], **{'doc': u'', 'properties': (), 'name': 'file281'})
    option_139789492849392 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492849664 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492849936 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492850208 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-views/auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492854864 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492855136 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-views/auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492855408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492855680 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_views_auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492855952 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492831568 = OptionDescription(children=[option_139789492849392, option_139789492849664, option_139789492849936, option_139789492850208, option_139789492854864, option_139789492855136, option_139789492855408, option_139789492855680, option_139789492855952], **{'doc': u'', 'properties': (), 'name': 'file282'})
    option_139789492856224 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492856496 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492856768 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492857040 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492857312 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492857584 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492857856 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492858128 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_rsyslog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492858400 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492831888 = OptionDescription(children=[option_139789492856224, option_139789492856496, option_139789492856768, option_139789492857040, option_139789492857312, option_139789492857584, option_139789492857856, option_139789492858128, option_139789492858400], **{'doc': u'', 'properties': (), 'name': 'file283'})
    option_139789492863056 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492863328 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492863600 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492863872 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/cron.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492864144 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492864416 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/cron.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492864688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492864960 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_cron.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492865232 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492832208 = OptionDescription(children=[option_139789492863056, option_139789492863328, option_139789492863600, option_139789492863872, option_139789492864144, option_139789492864416, option_139789492864688, option_139789492864960, option_139789492865232], **{'doc': u'', 'properties': (), 'name': 'file284'})
    option_139789492865504 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492865776 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492866048 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492866320 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/iptables.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492866592 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492871248 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/iptables.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492871520 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492871792 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_iptables.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492872064 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492832528 = OptionDescription(children=[option_139789492865504, option_139789492865776, option_139789492866048, option_139789492866320, option_139789492866592, option_139789492871248, option_139789492871520, option_139789492871792, option_139789492872064], **{'doc': u'', 'properties': (), 'name': 'file285'})
    option_139789492872336 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492872608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492872880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492873152 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/gpm.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492873424 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492873696 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/gpm.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492873968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492874240 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_gpm.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492874512 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492832848 = OptionDescription(children=[option_139789492872336, option_139789492872608, option_139789492872880, option_139789492873152, option_139789492873424, option_139789492873696, option_139789492873968, option_139789492874240, option_139789492874512], **{'doc': u'', 'properties': (), 'name': 'file286'})
    option_139789492874784 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492879440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492879712 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492879984 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/zephir.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492880256 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492880528 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/zephir.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492880800 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492881072 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_zephir.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492881344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492833168 = OptionDescription(children=[option_139789492874784, option_139789492879440, option_139789492879712, option_139789492879984, option_139789492880256, option_139789492880528, option_139789492880800, option_139789492881072, option_139789492881344], **{'doc': u'', 'properties': (), 'name': 'file287'})
    option_139789492881616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492881888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492882160 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492882432 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492882704 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492882976 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492887632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492887904 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492888176 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492833488 = OptionDescription(children=[option_139789492881616, option_139789492881888, option_139789492882160, option_139789492882432, option_139789492882704, option_139789492882976, option_139789492887632, option_139789492887904, option_139789492888176], **{'doc': u'', 'properties': (), 'name': 'file288'})
    option_139789492888448 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492888720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492888992 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492889264 = UnicodeOption(**{'default': u'/etc/rsyslog.d/default_dispatching.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492889536 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492889808 = UnicodeOption(**{'default': u'/etc/rsyslog.d/default_dispatching.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492890080 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492890352 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_default_dispatching.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492890624 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492833808 = OptionDescription(children=[option_139789492888448, option_139789492888720, option_139789492888992, option_139789492889264, option_139789492889536, option_139789492889808, option_139789492890080, option_139789492890352, option_139789492890624], **{'doc': u'', 'properties': (), 'name': 'file289'})
    option_139789492890896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492891168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492899920 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492900192 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/xmlrpc.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492900464 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492900736 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/xmlrpc.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492901008 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492901280 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_xmlrpc.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492901552 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492904016 = OptionDescription(children=[option_139789492890896, option_139789492891168, option_139789492899920, option_139789492900192, option_139789492900464, option_139789492900736, option_139789492901008, option_139789492901280, option_139789492901552], **{'doc': u'', 'properties': (), 'name': 'file290'})
    option_139789492901824 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492902096 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492902368 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492902640 = UnicodeOption(**{'default': u'pppoe', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492902912 = UnicodeOption(**{'default': u'/etc/ppp/peers/dsl-provider', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492903184 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492903456 = UnicodeOption(**{'default': u'/etc/ppp/peers/dsl-provider', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492912208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492912480 = UnicodeOption(**{'default': u'/var/lib/creole/dsl-provider', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492912752 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'pppoe', 'action': 'disabled', 'inverse': True, "option": option_139789497938272}]})
    option_139789492904336 = OptionDescription(children=[option_139789492901824, option_139789492902096, option_139789492902368, option_139789492902640, option_139789492902912, option_139789492903184, option_139789492903456, option_139789492912208, option_139789492912480, option_139789492912752], **{'doc': u'', 'properties': (), 'name': 'file291'})
    option_139789492913024 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492913296 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492913568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492913840 = UnicodeOption(**{'default': u'pppoe', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492914112 = UnicodeOption(**{'default': u'/etc/ppp/pap-secrets', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492914384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492914656 = UnicodeOption(**{'default': u'/etc/ppp/pap-secrets', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492914928 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492915200 = UnicodeOption(**{'default': u'/var/lib/creole/pap-secrets', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492915472 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'pppoe', 'action': 'disabled', 'inverse': True, "option": option_139789497938272}]})
    option_139789492904656 = OptionDescription(children=[option_139789492913024, option_139789492913296, option_139789492913568, option_139789492913840, option_139789492914112, option_139789492914384, option_139789492914656, option_139789492914928, option_139789492915200, option_139789492915472], **{'doc': u'', 'properties': (), 'name': 'file292'})
    option_139789492915744 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492924496 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492924768 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492925040 = UnicodeOption(**{'default': u'pppoe', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492925312 = UnicodeOption(**{'default': u'/etc/ppp/chap-secrets', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492925584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492925856 = UnicodeOption(**{'default': u'/etc/ppp/chap-secrets', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492926128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492926400 = UnicodeOption(**{'default': u'/var/lib/creole/chap-secrets', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492926672 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'pppoe', 'action': 'disabled', 'inverse': True, "option": option_139789497938272}]})
    option_139789492904976 = OptionDescription(children=[option_139789492915744, option_139789492924496, option_139789492924768, option_139789492925040, option_139789492925312, option_139789492925584, option_139789492925856, option_139789492926128, option_139789492926400, option_139789492926672], **{'doc': u'', 'properties': (), 'name': 'file293'})
    option_139789492926944 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492927216 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492927488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492927760 = UnicodeOption(**{'default': u'pppoe', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492928032 = UnicodeOption(**{'default': u'/etc/ppp/ip-up.d/0000usepeerdns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492932688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492932960 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492933232 = UnicodeOption(**{'default': u'/etc/ppp/ip-up.d/0000usepeerdns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492933504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492933776 = UnicodeOption(**{'default': u'/var/lib/creole/0000usepeerdns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492934048 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'pppoe', 'action': 'disabled', 'inverse': True, "option": option_139789497938272}]})
    option_139789492905296 = OptionDescription(children=[option_139789492926944, option_139789492927216, option_139789492927488, option_139789492927760, option_139789492928032, option_139789492932688, option_139789492932960, option_139789492933232, option_139789492933504, option_139789492933776, option_139789492934048], **{'doc': u'', 'properties': (), 'name': 'file294'})
    option_139789492934320 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492934592 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492934864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492935136 = UnicodeOption(**{'default': u'dhclient', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492935408 = UnicodeOption(**{'default': u'/etc/dhcp/dhclient.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492935680 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492935952 = UnicodeOption(**{'default': u'/etc/dhcp/dhclient.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492936224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492940880 = UnicodeOption(**{'default': u'/var/lib/creole/dhclient.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492941152 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'dhcp', 'action': 'disabled', 'inverse': True, "option": option_139789497938272}]})
    option_139789492905616 = OptionDescription(children=[option_139789492934320, option_139789492934592, option_139789492934864, option_139789492935136, option_139789492935408, option_139789492935680, option_139789492935952, option_139789492936224, option_139789492940880, option_139789492941152], **{'doc': u'', 'properties': (), 'name': 'file295'})
    option_139789492941424 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492941696 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492941968 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492942240 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492942512 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/10-chain_static_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492942784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492943056 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492943328 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/10-chain_static_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492943600 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492943872 = UnicodeOption(**{'default': u'/var/lib/creole/10-chain_static_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492944144 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789492905936 = OptionDescription(children=[option_139789492941424, option_139789492941696, option_139789492941968, option_139789492942240, option_139789492942512, option_139789492942784, option_139789492943056, option_139789492943328, option_139789492943600, option_139789492943872, option_139789492944144], **{'doc': u'', 'properties': (), 'name': 'file296'})
    option_139789492944416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492953168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492953440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492953712 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492953984 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/40-icmp_static_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492954256 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492954528 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492954800 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/40-icmp_static_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492955072 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492955344 = UnicodeOption(**{'default': u'/var/lib/creole/40-icmp_static_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492955616 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789492906256 = OptionDescription(children=[option_139789492944416, option_139789492953168, option_139789492953440, option_139789492953712, option_139789492953984, option_139789492954256, option_139789492954528, option_139789492954800, option_139789492955072, option_139789492955344, option_139789492955616], **{'doc': u'', 'properties': (), 'name': 'file297'})
    option_139789492955888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492956160 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492956432 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492956704 = UnicodeOption(**{'default': u'era_not_installed', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492961360 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/99-end_static_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492961632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492961904 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492962176 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/99-end_static_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492962448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492962720 = UnicodeOption(**{'default': u'/var/lib/creole/99-end_static_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492962992 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497966112}]})
    option_139789492906576 = OptionDescription(children=[option_139789492955888, option_139789492956160, option_139789492956432, option_139789492956704, option_139789492961360, option_139789492961632, option_139789492961904, option_139789492962176, option_139789492962448, option_139789492962720, option_139789492962992], **{'doc': u'', 'properties': (), 'name': 'file298'})
    option_139789492963264 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492963536 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492963808 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492964080 = UnicodeOption(**{'default': u'active_bonding', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492964352 = UnicodeOption(**{'default': u'/etc/modprobe.d/bonding.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492964624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492964896 = UnicodeOption(**{'default': u'/etc/modprobe.d/bonding.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492973648 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492973920 = UnicodeOption(**{'default': u'/var/lib/creole/bonding.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492974192 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497596112}]})
    option_139789492906896 = OptionDescription(children=[option_139789492963264, option_139789492963536, option_139789492963808, option_139789492964080, option_139789492964352, option_139789492964624, option_139789492964896, option_139789492973648, option_139789492973920, option_139789492974192], **{'doc': u'', 'properties': (), 'name': 'file299'})
    option_139789492974464 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492974736 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492975008 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492975280 = UnicodeOption(**{'default': u'/usr/sbin/ouvre.firewall', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492975552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492975824 = UnicodeOption(**{'default': u'755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492976096 = UnicodeOption(**{'default': u'/usr/sbin/ouvre.firewall', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492976368 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492976640 = UnicodeOption(**{'default': u'/var/lib/creole/ouvre.firewall', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492976912 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492907216 = OptionDescription(children=[option_139789492974464, option_139789492974736, option_139789492975008, option_139789492975280, option_139789492975552, option_139789492975824, option_139789492976096, option_139789492976368, option_139789492976640, option_139789492976912], **{'doc': u'', 'properties': (), 'name': 'file300'})
    option_139789492977184 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492985936 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492986208 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492986480 = UnicodeOption(**{'default': u'/usr/lib/eole/forteresse.sh', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492986752 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492987024 = UnicodeOption(**{'default': u'/usr/lib/eole/forteresse.sh', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492987296 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492987568 = UnicodeOption(**{'default': u'/var/lib/creole/forteresse.sh', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492987840 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492907536 = OptionDescription(children=[option_139789492977184, option_139789492985936, option_139789492986208, option_139789492986480, option_139789492986752, option_139789492987024, option_139789492987296, option_139789492987568, option_139789492987840], **{'doc': u'', 'properties': (), 'name': 'file301'})
    option_139789492988112 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492988384 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492988656 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492988928 = UnicodeOption(**{'default': u'/var/lib/eole/config/persistent-net.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492989200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492989472 = UnicodeOption(**{'default': u'/var/lib/eole/config/persistent-net.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492994128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492994400 = UnicodeOption(**{'default': u'/var/lib/creole/persistent-net.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492994672 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493002320 = OptionDescription(children=[option_139789492988112, option_139789492988384, option_139789492988656, option_139789492988928, option_139789492989200, option_139789492989472, option_139789492994128, option_139789492994400, option_139789492994672], **{'doc': u'', 'properties': (), 'name': 'file302'})
    option_139789492994944 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492995216 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492995488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492995760 = UnicodeOption(**{'default': u'routes', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492996032 = UnicodeOption(**{'default': u'/etc/eole/static-routes', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492996304 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492996576 = UnicodeOption(**{'default': u'/etc/eole/static-routes', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492996848 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492997120 = UnicodeOption(**{'default': u'/var/lib/creole/static-routes', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492997392 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497381760}]})
    option_139789493002640 = OptionDescription(children=[option_139789492994944, option_139789492995216, option_139789492995488, option_139789492995760, option_139789492996032, option_139789492996304, option_139789492996576, option_139789492996848, option_139789492997120, option_139789492997392], **{'doc': u'', 'properties': (), 'name': 'file303'})
    option_139789492997664 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493010512 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493010784 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493011056 = UnicodeOption(**{'default': u'/etc/eole/ssl/certif-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493011328 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493011600 = UnicodeOption(**{'default': u'/etc/eole/ssl/certif-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493011872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493012144 = UnicodeOption(**{'default': u'/var/lib/creole/certif-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493012416 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493002960 = OptionDescription(children=[option_139789492997664, option_139789493010512, option_139789493010784, option_139789493011056, option_139789493011328, option_139789493011600, option_139789493011872, option_139789493012144, option_139789493012416], **{'doc': u'', 'properties': (), 'name': 'file304'})
    option_139789493012688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493012960 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493013232 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493013504 = UnicodeOption(**{'default': u'/etc/eole/ssl/client-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493013776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493014048 = UnicodeOption(**{'default': u'/etc/eole/ssl/client-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493018704 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493018976 = UnicodeOption(**{'default': u'/var/lib/creole/client-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493019248 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493003280 = OptionDescription(children=[option_139789493012688, option_139789493012960, option_139789493013232, option_139789493013504, option_139789493013776, option_139789493014048, option_139789493018704, option_139789493018976, option_139789493019248], **{'doc': u'', 'properties': (), 'name': 'file305'})
    option_139789493019520 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493019792 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493020064 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493020336 = UnicodeOption(**{'default': u'/etc/eole/ssl/ca-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493020608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493020880 = UnicodeOption(**{'default': u'/etc/eole/ssl/ca-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493021152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493021424 = UnicodeOption(**{'default': u'/var/lib/creole/ca-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493021696 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493003600 = OptionDescription(children=[option_139789493019520, option_139789493019792, option_139789493020064, option_139789493020336, option_139789493020608, option_139789493020880, option_139789493021152, option_139789493021424, option_139789493021696], **{'doc': u'', 'properties': (), 'name': 'file306'})
    option_139789493021968 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493022240 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493030992 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493031264 = UnicodeOption(**{'default': u'le_active', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493031536 = UnicodeOption(**{'default': u'/etc/eole/ssl/nginx-minimal.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493031808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493032080 = UnicodeOption(**{'default': u'/etc/eole/ssl/nginx-minimal.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493032352 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493032624 = UnicodeOption(**{'default': u'/var/lib/creole/nginx-minimal.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493032896 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789493003920 = OptionDescription(children=[option_139789493021968, option_139789493022240, option_139789493030992, option_139789493031264, option_139789493031536, option_139789493031808, option_139789493032080, option_139789493032352, option_139789493032624, option_139789493032896], **{'doc': u'', 'properties': (), 'name': 'file307'})
    option_139789493033168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493033440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493033712 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493033984 = UnicodeOption(**{'default': u'le_active', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789493034256 = UnicodeOption(**{'default': u'/usr/share/eole/letsencrypt/hooks-post.d/00-copy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493034528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493043280 = UnicodeOption(**{'default': u'755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789493043552 = UnicodeOption(**{'default': u'/usr/share/eole/letsencrypt/hooks-post.d/00-copy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493043824 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493044096 = UnicodeOption(**{'default': u'/var/lib/creole/le-00-copy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493044368 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'letsencrypt', 'action': 'disabled', 'inverse': True, "option": option_139789497939360}]})
    option_139789493004240 = OptionDescription(children=[option_139789493033168, option_139789493033440, option_139789493033712, option_139789493033984, option_139789493034256, option_139789493034528, option_139789493043280, option_139789493043552, option_139789493043824, option_139789493044096, option_139789493044368], **{'doc': u'', 'properties': (), 'name': 'file308'})
    option_139789493044640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493044912 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493045184 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493045456 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493045728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493046000 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493046272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493046544 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493046816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493004560 = OptionDescription(children=[option_139789493044640, option_139789493044912, option_139789493045184, option_139789493045456, option_139789493045728, option_139789493046000, option_139789493046272, option_139789493046544, option_139789493046816], **{'doc': u'', 'properties': (), 'name': 'file309'})
    option_139789493051472 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493051744 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493052016 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493052288 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493052560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493052832 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493053104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493053376 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493053648 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493004880 = OptionDescription(children=[option_139789493051472, option_139789493051744, option_139789493052016, option_139789493052288, option_139789493052560, option_139789493052832, option_139789493053104, option_139789493053376, option_139789493053648], **{'doc': u'', 'properties': (), 'name': 'file310'})
    option_139789493053920 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493054192 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493054464 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493054736 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493055008 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493059664 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493059936 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493060208 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493060480 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493005200 = OptionDescription(children=[option_139789493053920, option_139789493054192, option_139789493054464, option_139789493054736, option_139789493055008, option_139789493059664, option_139789493059936, option_139789493060208, option_139789493060480], **{'doc': u'', 'properties': (), 'name': 'file311'})
    option_139789493060752 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493061024 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493061296 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493061568 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493061840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493062112 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493062384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493062656 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493062928 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493005520 = OptionDescription(children=[option_139789493060752, option_139789493061024, option_139789493061296, option_139789493061568, option_139789493061840, option_139789493062112, option_139789493062384, option_139789493062656, option_139789493062928], **{'doc': u'', 'properties': (), 'name': 'file312'})
    option_139789493063200 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493067856 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493068128 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493068400 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493068672 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493068944 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789493069216 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789493069488 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789493069760 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789493005840 = OptionDescription(children=[option_139789493063200, option_139789493067856, option_139789493068128, option_139789493068400, option_139789493068672, option_139789493068944, option_139789493069216, option_139789493069488, option_139789493069760], **{'doc': u'', 'properties': (), 'name': 'file313'})
    option_139789493070032 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789493070304 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789493070576 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789493070848 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789493071120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789493071392 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492551760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492552032 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492552304 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492559952 = OptionDescription(children=[option_139789493070032, option_139789493070304, option_139789493070576, option_139789493070848, option_139789493071120, option_139789493071392, option_139789492551760, option_139789492552032, option_139789492552304], **{'doc': u'', 'properties': (), 'name': 'file314'})
    option_139789492552576 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492552848 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492553120 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492553392 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492553664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492553936 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492554208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492554480 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492554752 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492560272 = OptionDescription(children=[option_139789492552576, option_139789492552848, option_139789492553120, option_139789492553392, option_139789492553664, option_139789492553936, option_139789492554208, option_139789492554480, option_139789492554752], **{'doc': u'', 'properties': (), 'name': 'file315'})
    option_139789492555024 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492555296 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492564048 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492564320 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492564592 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492564864 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492565136 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492565408 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492565680 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492560592 = OptionDescription(children=[option_139789492555024, option_139789492555296, option_139789492564048, option_139789492564320, option_139789492564592, option_139789492564864, option_139789492565136, option_139789492565408, option_139789492565680], **{'doc': u'', 'properties': (), 'name': 'file316'})
    option_139789492565952 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492566224 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492566496 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492566768 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492567040 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492567312 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492567584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492576336 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492576608 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492560912 = OptionDescription(children=[option_139789492565952, option_139789492566224, option_139789492566496, option_139789492566768, option_139789492567040, option_139789492567312, option_139789492567584, option_139789492576336, option_139789492576608], **{'doc': u'', 'properties': (), 'name': 'file317'})
    option_139789492576880 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492577152 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492577424 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492577696 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492577968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492578240 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492578512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492578784 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492579056 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492561232 = OptionDescription(children=[option_139789492576880, option_139789492577152, option_139789492577424, option_139789492577696, option_139789492577968, option_139789492578240, option_139789492578512, option_139789492578784, option_139789492579056], **{'doc': u'', 'properties': (), 'name': 'file318'})
    option_139789492579328 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492579600 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492579872 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492584528 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492584800 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492585072 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492585344 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492585616 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492585888 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492561552 = OptionDescription(children=[option_139789492579328, option_139789492579600, option_139789492579872, option_139789492584528, option_139789492584800, option_139789492585072, option_139789492585344, option_139789492585616, option_139789492585888], **{'doc': u'', 'properties': (), 'name': 'file319'})
    option_139789492586160 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492586432 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492586704 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492586976 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492587248 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492587520 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492587792 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492588064 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492592720 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492561872 = OptionDescription(children=[option_139789492586160, option_139789492586432, option_139789492586704, option_139789492586976, option_139789492587248, option_139789492587520, option_139789492587792, option_139789492588064, option_139789492592720], **{'doc': u'', 'properties': (), 'name': 'file320'})
    option_139789492592992 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492593264 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492593536 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492593808 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492594080 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492594352 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492594624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492594896 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492595168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492562192 = OptionDescription(children=[option_139789492592992, option_139789492593264, option_139789492593536, option_139789492593808, option_139789492594080, option_139789492594352, option_139789492594624, option_139789492594896, option_139789492595168], **{'doc': u'', 'properties': (), 'name': 'file321'})
    option_139789492595440 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492595712 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492595984 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492596256 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492600912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492601184 = UnicodeOption(**{'default': u'/etc/apt/apt.conf.d/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492601456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492601728 = UnicodeOption(**{'default': u'/var/lib/creole/03-eole-changelogs', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492602000 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492562512 = OptionDescription(children=[option_139789492595440, option_139789492595712, option_139789492595984, option_139789492596256, option_139789492600912, option_139789492601184, option_139789492601456, option_139789492601728, option_139789492602000], **{'doc': u'', 'properties': (), 'name': 'file322'})
    option_139789492602272 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492602544 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492602816 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492603088 = UnicodeOption(**{'default': u'apt_cacher', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492603360 = UnicodeOption(**{'default': u'/etc/apt-cacher-ng/acng.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492603632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492603904 = UnicodeOption(**{'default': u'/etc/apt-cacher-ng/acng.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492604176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492604448 = UnicodeOption(**{'default': u'/var/lib/creole/acng.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492609104 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497954928}]})
    option_139789492562832 = OptionDescription(children=[option_139789492602272, option_139789492602544, option_139789492602816, option_139789492603088, option_139789492603360, option_139789492603632, option_139789492603904, option_139789492604176, option_139789492604448, option_139789492609104], **{'doc': u'', 'properties': (), 'name': 'file323'})
    option_139789492609376 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492609648 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492609920 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492610192 = UnicodeOption(**{'default': u'apt_cacher', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492610464 = UnicodeOption(**{'default': u'/etc/apt-cacher-ng/backends_eole', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492610736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492611008 = UnicodeOption(**{'default': u'/etc/apt-cacher-ng/backends_eole', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492611280 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492611552 = UnicodeOption(**{'default': u'/var/lib/creole/backends_eole', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492611824 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497954928}]})
    option_139789492563152 = OptionDescription(children=[option_139789492609376, option_139789492609648, option_139789492609920, option_139789492610192, option_139789492610464, option_139789492610736, option_139789492611008, option_139789492611280, option_139789492611552, option_139789492611824], **{'doc': u'', 'properties': (), 'name': 'file324'})
    option_139789492612096 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492612368 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492612640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492617296 = UnicodeOption(**{'default': u'apt_cacher', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492617568 = UnicodeOption(**{'default': u'/etc/apt-cacher-ng/backends_ubuntu', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492617840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492618112 = UnicodeOption(**{'default': u'/etc/apt-cacher-ng/backends_ubuntu', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492618384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492618656 = UnicodeOption(**{'default': u'/var/lib/creole/backends_ubuntu', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492618928 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497954928}]})
    option_139789492563472 = OptionDescription(children=[option_139789492612096, option_139789492612368, option_139789492612640, option_139789492617296, option_139789492617568, option_139789492617840, option_139789492618112, option_139789492618384, option_139789492618656, option_139789492618928], **{'doc': u'', 'properties': (), 'name': 'file325'})
    option_139789492619200 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492619472 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492619744 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492620016 = UnicodeOption(**{'default': u'/etc/systemd/system/apt-cacher-ng.service.d/override.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492620288 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492620560 = UnicodeOption(**{'default': u'/etc/systemd/system/apt-cacher-ng.service.d/override.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492620832 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492625488 = UnicodeOption(**{'default': u'/var/lib/creole/apt-cache-ng.override.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492625760 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492629584 = OptionDescription(children=[option_139789492619200, option_139789492619472, option_139789492619744, option_139789492620016, option_139789492620288, option_139789492620560, option_139789492620832, option_139789492625488, option_139789492625760], **{'doc': u'', 'properties': (), 'name': 'file326'})
    option_139789492626032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492626304 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492626576 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492626848 = UnicodeOption(**{'default': u'/usr/share/era/backend/data/active_tags', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492627120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492627392 = UnicodeOption(**{'default': u'/usr/share/era/backend/data/active_tags', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492627664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492627936 = UnicodeOption(**{'default': u'/var/lib/creole/active_tags', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492628208 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492629904 = OptionDescription(children=[option_139789492626032, option_139789492626304, option_139789492626576, option_139789492626848, option_139789492627120, option_139789492627392, option_139789492627664, option_139789492627936, option_139789492628208], **{'doc': u'', 'properties': (), 'name': 'file327'})
    option_139789492628480 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492628752 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492629024 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492641872 = UnicodeOption(**{'default': u'/usr/share/ead2/backend/config/perms/perm_era.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492642144 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492642416 = UnicodeOption(**{'default': u'/usr/share/ead2/backend/config/perms/perm_era.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492642688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492642960 = UnicodeOption(**{'default': u'/var/lib/creole/perm_era.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492643232 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492630224 = OptionDescription(children=[option_139789492628480, option_139789492628752, option_139789492629024, option_139789492641872, option_139789492642144, option_139789492642416, option_139789492642688, option_139789492642960, option_139789492643232], **{'doc': u'', 'properties': (), 'name': 'file328'})
    option_139789492643504 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492643776 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492644048 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492644320 = UnicodeOption(**{'default': u'/etc/iproute2/rt_tables', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492644592 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492644864 = UnicodeOption(**{'default': u'/etc/iproute2/rt_tables', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492645136 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492645408 = UnicodeOption(**{'default': u'/var/lib/creole/rt_tables', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492650064 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492630544 = OptionDescription(children=[option_139789492643504, option_139789492643776, option_139789492644048, option_139789492644320, option_139789492644592, option_139789492644864, option_139789492645136, option_139789492645408, option_139789492650064], **{'doc': u'', 'properties': (), 'name': 'file329'})
    option_139789492650336 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492650608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492650880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492651152 = UnicodeOption(**{'default': u'agregation', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492651424 = UnicodeOption(**{'default': u'/etc/agregation.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492651696 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492651968 = UnicodeOption(**{'default': u'/etc/agregation.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492652240 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492652512 = UnicodeOption(**{'default': u'/var/lib/creole/agregation.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492652784 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497574000}]})
    option_139789492630864 = OptionDescription(children=[option_139789492650336, option_139789492650608, option_139789492650880, option_139789492651152, option_139789492651424, option_139789492651696, option_139789492651968, option_139789492652240, option_139789492652512, option_139789492652784], **{'doc': u'', 'properties': (), 'name': 'file330'})
    option_139789492653056 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492653328 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492653600 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492666448 = UnicodeOption(**{'default': u'agregation', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492666720 = UnicodeOption(**{'default': u'/etc/logrotate.d/agregation', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492666992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492667264 = UnicodeOption(**{'default': u'/etc/logrotate.d/agregation', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492667536 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492667808 = UnicodeOption(**{'default': u'/var/lib/creole/agregation.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492668080 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497574000}]})
    option_139789492631184 = OptionDescription(children=[option_139789492653056, option_139789492653328, option_139789492653600, option_139789492666448, option_139789492666720, option_139789492666992, option_139789492667264, option_139789492667536, option_139789492667808, option_139789492668080], **{'doc': u'', 'properties': (), 'name': 'file331'})
    option_139789492668352 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492668624 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492668896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492669168 = UnicodeOption(**{'default': u'eoledb', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492669440 = UnicodeOption(**{'default': u'/etc/eole/eole-db.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492669712 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492669984 = UnicodeOption(**{'default': u'/etc/eole/eole-db.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492674640 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492674912 = UnicodeOption(**{'default': u'/var/lib/creole/eole-db.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492675184 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955200}]})
    option_139789492631504 = OptionDescription(children=[option_139789492668352, option_139789492668624, option_139789492668896, option_139789492669168, option_139789492669440, option_139789492669712, option_139789492669984, option_139789492674640, option_139789492674912, option_139789492675184], **{'doc': u'', 'properties': (), 'name': 'file332'})
    option_139789492675456 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492675728 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492676000 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492676272 = UnicodeOption(**{'default': u'force_clamav', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492676544 = UnicodeOption(**{'default': u'/etc/clamav/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492676816 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492677088 = UnicodeOption(**{'default': u'/etc/clamav/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492677360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492677632 = UnicodeOption(**{'default': u'/var/lib/creole/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492677904 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497483344}]})
    option_139789492631824 = OptionDescription(children=[option_139789492675456, option_139789492675728, option_139789492676000, option_139789492676272, option_139789492676544, option_139789492676816, option_139789492677088, option_139789492677360, option_139789492677632, option_139789492677904], **{'doc': u'', 'properties': (), 'name': 'file333'})
    option_139789492678176 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492682832 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492683104 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492683376 = UnicodeOption(**{'default': u'clamav_proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492683648 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/clamav/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492683920 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492684192 = UnicodeOption(**{'default': u'/etc/clamav/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492684464 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492684736 = UnicodeOption(**{'default': u'/var/lib/creole/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492685008 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497484160}]})
    option_139789492632144 = OptionDescription(children=[option_139789492678176, option_139789492682832, option_139789492683104, option_139789492683376, option_139789492683648, option_139789492683920, option_139789492684192, option_139789492684464, option_139789492684736, option_139789492685008], **{'doc': u'', 'properties': (), 'name': 'file334'})
    option_139789492685280 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492685552 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492685824 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492686096 = UnicodeOption(**{'default': u'antivirus_ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492686368 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/clamav/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492691024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492691296 = UnicodeOption(**{'default': u'/etc/clamav/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492691568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492691840 = UnicodeOption(**{'default': u'/var/lib/creole/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492692112 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497484432}]})
    option_139789492632464 = OptionDescription(children=[option_139789492685280, option_139789492685552, option_139789492685824, option_139789492686096, option_139789492686368, option_139789492691024, option_139789492691296, option_139789492691568, option_139789492691840, option_139789492692112], **{'doc': u'', 'properties': (), 'name': 'file335'})
    option_139789492692384 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492692656 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492692928 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492693200 = UnicodeOption(**{'default': u'clamav_mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492693472 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/clamav/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492693744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492694016 = UnicodeOption(**{'default': u'/etc/clamav/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492694288 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492694560 = UnicodeOption(**{'default': u'/var/lib/creole/clamd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492699216 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497476512}]})
    option_139789492632784 = OptionDescription(children=[option_139789492692384, option_139789492692656, option_139789492692928, option_139789492693200, option_139789492693472, option_139789492693744, option_139789492694016, option_139789492694288, option_139789492694560, option_139789492699216], **{'doc': u'', 'properties': (), 'name': 'file336'})
    option_139789492699488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492699760 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492700032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492700304 = UnicodeOption(**{'default': u'clamav', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492700576 = UnicodeOption(**{'default': u'/etc/clamav/freshclam.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492700848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492701120 = UnicodeOption(**{'default': u'/etc/clamav/freshclam.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492701392 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492701664 = UnicodeOption(**{'default': u'/var/lib/creole/freshclam.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492701936 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}]})
    option_139789492633104 = OptionDescription(children=[option_139789492699488, option_139789492699760, option_139789492700032, option_139789492700304, option_139789492700576, option_139789492700848, option_139789492701120, option_139789492701392, option_139789492701664, option_139789492701936], **{'doc': u'', 'properties': (), 'name': 'file337'})
    option_139789492702208 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492702480 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492702752 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492707408 = UnicodeOption(**{'default': u'clam_container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492707680 = UnicodeOption(**{'default': u'/etc/sudoers.d/clamav', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492707952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492708224 = UnicodeOption(**{'default': u'0440', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492708496 = UnicodeOption(**{'default': u'/etc/sudoers.d/clamav', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492708768 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492709040 = UnicodeOption(**{'default': u'/var/lib/creole/clamav.sudoers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492709312 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789492715600 = OptionDescription(children=[option_139789492702208, option_139789492702480, option_139789492702752, option_139789492707408, option_139789492707680, option_139789492707952, option_139789492708224, option_139789492708496, option_139789492708768, option_139789492709040, option_139789492709312], **{'doc': u'', 'properties': (), 'name': 'file338'})
    option_139789492709584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492709856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492710128 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492710400 = UnicodeOption(**{'default': u'clamav', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492710672 = UnicodeOption(**{'default': u'/etc/eole/clamd-reload.cnf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492710944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492719696 = UnicodeOption(**{'default': u'/etc/eole/clamd-reload.cnf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492719968 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492720240 = UnicodeOption(**{'default': u'/var/lib/creole/clamd-reload.cnf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492720512 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}]})
    option_139789492715920 = OptionDescription(children=[option_139789492709584, option_139789492709856, option_139789492710128, option_139789492710400, option_139789492710672, option_139789492710944, option_139789492719696, option_139789492719968, option_139789492720240, option_139789492720512], **{'doc': u'', 'properties': (), 'name': 'file339'})
    option_139789492720784 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492721056 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492721328 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492721600 = UnicodeOption(**{'default': u'clamav', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492721872 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.sbin.clamd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492722144 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492722416 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.sbin.clamd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492722688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492722960 = UnicodeOption(**{'default': u'/var/lib/creole/usr.sbin.clamd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492723232 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}]})
    option_139789492716240 = OptionDescription(children=[option_139789492720784, option_139789492721056, option_139789492721328, option_139789492721600, option_139789492721872, option_139789492722144, option_139789492722416, option_139789492722688, option_139789492722960, option_139789492723232], **{'doc': u'', 'properties': (), 'name': 'file340'})
    option_139789492731984 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492732256 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492732528 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492732800 = UnicodeOption(**{'default': u'clamav', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492733072 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.bin.freshclam', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492733344 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492733616 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.bin.freshclam', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492733888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492734160 = UnicodeOption(**{'default': u'/var/lib/creole/freshclam.apparmor.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492734432 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497955472}]})
    option_139789492716560 = OptionDescription(children=[option_139789492731984, option_139789492732256, option_139789492732528, option_139789492732800, option_139789492733072, option_139789492733344, option_139789492733616, option_139789492733888, option_139789492734160, option_139789492734432], **{'doc': u'', 'properties': (), 'name': 'file341'})
    option_139789492734704 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492734976 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492735248 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492735520 = UnicodeOption(**{'default': u'/etc/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492740176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492740448 = UnicodeOption(**{'default': u'/etc/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492740720 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492740992 = UnicodeOption(**{'default': u'/var/lib/creole/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492741264 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492716880 = OptionDescription(children=[option_139789492734704, option_139789492734976, option_139789492735248, option_139789492735520, option_139789492740176, option_139789492740448, option_139789492740720, option_139789492740992, option_139789492741264], **{'doc': u'', 'properties': (), 'name': 'file342'})
    option_139789492741536 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492741808 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492742080 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492742352 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492742624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492742896 = UnicodeOption(**{'default': u'/etc/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492743168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492743440 = UnicodeOption(**{'default': u'/var/lib/creole/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492743712 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492717200 = OptionDescription(children=[option_139789492741536, option_139789492741808, option_139789492742080, option_139789492742352, option_139789492742624, option_139789492742896, option_139789492743168, option_139789492743440, option_139789492743712], **{'doc': u'', 'properties': (), 'name': 'file343'})
    option_139789492748368 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492748640 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492748912 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492749184 = UnicodeOption(**{'default': u'client_ldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492749456 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492749728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492750000 = UnicodeOption(**{'default': u'/etc/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492750272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492750544 = UnicodeOption(**{'default': u'/var/lib/creole/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492750816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492717520 = OptionDescription(children=[option_139789492748368, option_139789492748640, option_139789492748912, option_139789492749184, option_139789492749456, option_139789492749728, option_139789492750000, option_139789492750272, option_139789492750544, option_139789492750816], **{'doc': u'', 'properties': (), 'name': 'file344'})
    option_139789492751088 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492751360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492751632 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492751904 = UnicodeOption(**{'default': u'/etc/nsswitch.conf.default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492760656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492760928 = UnicodeOption(**{'default': u'/etc/nsswitch.conf.default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492761200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492761472 = UnicodeOption(**{'default': u'/var/lib/creole/nsswitch.conf.default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492761744 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492717840 = OptionDescription(children=[option_139789492751088, option_139789492751360, option_139789492751632, option_139789492751904, option_139789492760656, option_139789492760928, option_139789492761200, option_139789492761472, option_139789492761744], **{'doc': u'', 'properties': (), 'name': 'file345'})
    option_139789492762016 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492762288 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492762560 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492762832 = UnicodeOption(**{'default': u'ldap_nss_active', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492763104 = UnicodeOption(**{'default': u'/etc/nsswitch.conf.noldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492763376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492763648 = UnicodeOption(**{'default': u'/etc/nsswitch.conf.noldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492763920 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492764192 = UnicodeOption(**{'default': u'/var/lib/creole/nsswitch.conf.noldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492768848 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497485520}]})
    option_139789492718160 = OptionDescription(children=[option_139789492762016, option_139789492762288, option_139789492762560, option_139789492762832, option_139789492763104, option_139789492763376, option_139789492763648, option_139789492763920, option_139789492764192, option_139789492768848], **{'doc': u'', 'properties': (), 'name': 'file346'})
    option_139789492769120 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492769392 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492769664 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492769936 = UnicodeOption(**{'default': u'client_ldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492770208 = UnicodeOption(**{'default': u'/etc/ldap/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492770480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492770752 = UnicodeOption(**{'default': u'/etc/ldap/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492771024 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492771296 = UnicodeOption(**{'default': u'/var/lib/creole/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492771568 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492718480 = OptionDescription(children=[option_139789492769120, option_139789492769392, option_139789492769664, option_139789492769936, option_139789492770208, option_139789492770480, option_139789492770752, option_139789492771024, option_139789492771296, option_139789492771568], **{'doc': u'', 'properties': (), 'name': 'file347'})
    option_139789492771840 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492772112 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492772384 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492781136 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ldap/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492781408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492781680 = UnicodeOption(**{'default': u'/etc/ldap/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492781952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492782224 = UnicodeOption(**{'default': u'/var/lib/creole/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492782496 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492718800 = OptionDescription(children=[option_139789492771840, option_139789492772112, option_139789492772384, option_139789492781136, option_139789492781408, option_139789492781680, option_139789492781952, option_139789492782224, option_139789492782496], **{'doc': u'', 'properties': (), 'name': 'file348'})
    option_139789492782768 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492783040 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492783312 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492783584 = UnicodeOption(**{'default': u'client_ldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492783856 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/ldap/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492784128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492784400 = UnicodeOption(**{'default': u'/etc/ldap/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492784672 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492789328 = UnicodeOption(**{'default': u'/var/lib/creole/ldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492789600 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492719120 = OptionDescription(children=[option_139789492782768, option_139789492783040, option_139789492783312, option_139789492783584, option_139789492783856, option_139789492784128, option_139789492784400, option_139789492784672, option_139789492789328, option_139789492789600], **{'doc': u'', 'properties': (), 'name': 'file349'})
    option_139789492789872 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492790144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492790416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492790688 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492790960 = UnicodeOption(**{'default': u'/usr/share/ead2/backend/config/perms/perm_dhcp.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492791232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492791504 = UnicodeOption(**{'default': u'/usr/share/ead2/backend/config/perms/perm_dhcp.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492791776 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492792048 = UnicodeOption(**{'default': u'/var/lib/creole/perm_dhcp.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492792320 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}]})
    option_139789492797520 = OptionDescription(children=[option_139789492789872, option_139789492790144, option_139789492790416, option_139789492790688, option_139789492790960, option_139789492791232, option_139789492791504, option_139789492791776, option_139789492792048, option_139789492792320], **{'doc': u'', 'properties': (), 'name': 'file350'})
    option_139789492792592 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492792864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492801616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492801888 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492802160 = UnicodeOption(**{'default': u'/var/lib/eole/config/eoledhcp.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492802432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492802704 = UnicodeOption(**{'default': u'/var/lib/eole/config/eoledhcp.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492802976 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492803248 = UnicodeOption(**{'default': u'/var/lib/creole/eoledhcp.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492803520 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}]})
    option_139789492797840 = OptionDescription(children=[option_139789492792592, option_139789492792864, option_139789492801616, option_139789492801888, option_139789492802160, option_139789492802432, option_139789492802704, option_139789492802976, option_139789492803248, option_139789492803520], **{'doc': u'', 'properties': (), 'name': 'file351'})
    option_139789492803792 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492804064 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492804336 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492804608 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492804880 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/dhcp/dhcpd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492805152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492285520 = UnicodeOption(**{'default': u'/etc/dhcp/dhcpd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492285792 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492286064 = UnicodeOption(**{'default': u'/var/lib/creole/dhcpd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492286336 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}]})
    option_139789492798160 = OptionDescription(children=[option_139789492803792, option_139789492804064, option_139789492804336, option_139789492804608, option_139789492804880, option_139789492805152, option_139789492285520, option_139789492285792, option_139789492286064, option_139789492286336], **{'doc': u'', 'properties': (), 'name': 'file352'})
    option_139789492286608 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492286880 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492287152 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492287424 = UnicodeOption(**{'default': u'tftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492287696 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/default/tftpd-hpa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492287968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492288240 = UnicodeOption(**{'default': u'/etc/default/tftpd-hpa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492288512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492288784 = UnicodeOption(**{'default': u'/var/lib/creole/tftpd-hpa.default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492289056 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956288}]})
    option_139789492798480 = OptionDescription(children=[option_139789492286608, option_139789492286880, option_139789492287152, option_139789492287424, option_139789492287696, option_139789492287968, option_139789492288240, option_139789492288512, option_139789492288784, option_139789492289056], **{'doc': u'', 'properties': (), 'name': 'file353'})
    option_139789492297808 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492298080 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492298352 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492298624 = UnicodeOption(**{'default': u'dhrelay', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492298896 = UnicodeOption(**{'default': u'/etc/default/isc-dhcp-relay', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492299168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492299440 = UnicodeOption(**{'default': u'/etc/default/isc-dhcp-relay', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492299712 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492299984 = UnicodeOption(**{'default': u'/var/lib/creole/isc-dhcp-relay', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492300256 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956560}]})
    option_139789492798800 = OptionDescription(children=[option_139789492297808, option_139789492298080, option_139789492298352, option_139789492298624, option_139789492298896, option_139789492299168, option_139789492299440, option_139789492299712, option_139789492299984, option_139789492300256], **{'doc': u'', 'properties': (), 'name': 'file354'})
    option_139789492300528 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492300800 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492301072 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492301344 = UnicodeOption(**{'default': u'esu', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492310096 = UnicodeOption(**{'default': u'/home/esu/Base/DomainConf.xml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492310368 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492310640 = UnicodeOption(**{'default': u'/home/esu/Base/DomainConf.xml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492310912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492311184 = UnicodeOption(**{'default': u'/var/lib/creole/DomainConf.xml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492311456 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956832}]})
    option_139789492799120 = OptionDescription(children=[option_139789492300528, option_139789492300800, option_139789492301072, option_139789492301344, option_139789492310096, option_139789492310368, option_139789492310640, option_139789492310912, option_139789492311184, option_139789492311456], **{'doc': u'', 'properties': (), 'name': 'file355'})
    option_139789492311728 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492312000 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492312272 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492312544 = UnicodeOption(**{'default': u'esu', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492312816 = UnicodeOption(**{'default': u'/home/esu/Console/esuconf.xml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492313088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492313360 = UnicodeOption(**{'default': u'/home/esu/Console/esuconf.xml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492313632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492322384 = UnicodeOption(**{'default': u'/var/lib/creole/esuconf.xml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492322656 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956832}]})
    option_139789492799440 = OptionDescription(children=[option_139789492311728, option_139789492312000, option_139789492312272, option_139789492312544, option_139789492312816, option_139789492313088, option_139789492313360, option_139789492313632, option_139789492322384, option_139789492322656], **{'doc': u'', 'properties': (), 'name': 'file356'})
    option_139789492322928 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492323200 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492323472 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492323744 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/smbldap-tools/smbldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492324016 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492324288 = UnicodeOption(**{'default': u'/etc/smbldap-tools/smbldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492324560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492324832 = UnicodeOption(**{'default': u'/var/lib/creole/smbldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492325104 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492799760 = OptionDescription(children=[option_139789492322928, option_139789492323200, option_139789492323472, option_139789492323744, option_139789492324016, option_139789492324288, option_139789492324560, option_139789492324832, option_139789492325104], **{'doc': u'', 'properties': (), 'name': 'file357'})
    option_139789492325376 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492325648 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492325920 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492330576 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/smbldap-tools/smbldap_bind.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492330848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492331120 = UnicodeOption(**{'default': u'/etc/smbldap-tools/smbldap_bind.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492331392 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492331664 = UnicodeOption(**{'default': u'/var/lib/creole/smbldap_bind.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492331936 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492800080 = OptionDescription(children=[option_139789492325376, option_139789492325648, option_139789492325920, option_139789492330576, option_139789492330848, option_139789492331120, option_139789492331392, option_139789492331664, option_139789492331936], **{'doc': u'', 'properties': (), 'name': 'file358'})
    option_139789492332208 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492332480 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492332752 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492333024 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/nsswitch.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492333296 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492333568 = UnicodeOption(**{'default': u'/etc/nsswitch.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492333840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492334112 = UnicodeOption(**{'default': u'/var/lib/creole/nsswitch.conf.default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492342864 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492800400 = OptionDescription(children=[option_139789492332208, option_139789492332480, option_139789492332752, option_139789492333024, option_139789492333296, option_139789492333568, option_139789492333840, option_139789492334112, option_139789492342864], **{'doc': u'', 'properties': (), 'name': 'file359'})
    option_139789492343136 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492343408 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492343680 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492343952 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/nsswitch.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492344224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492344496 = UnicodeOption(**{'default': u'/etc/nsswitch.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492344768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492345040 = UnicodeOption(**{'default': u'/var/lib/creole/nsswitch-proxy_auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492345312 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492800720 = OptionDescription(children=[option_139789492343136, option_139789492343408, option_139789492343680, option_139789492343952, option_139789492344224, option_139789492344496, option_139789492344768, option_139789492345040, option_139789492345312], **{'doc': u'', 'properties': (), 'name': 'file360'})
    option_139789492345584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492345856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492346128 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492346400 = UnicodeOption(**{'default': u'smb_quotawarn', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492351056 = UnicodeOption(**{'default': u'/etc/warnquota.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492351328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492351600 = UnicodeOption(**{'default': u'/etc/warnquota.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492351872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492352144 = UnicodeOption(**{'default': u'/var/lib/creole/warnquota.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492352416 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497062272}]})
    option_139789492801040 = OptionDescription(children=[option_139789492345584, option_139789492345856, option_139789492346128, option_139789492346400, option_139789492351056, option_139789492351328, option_139789492351600, option_139789492351872, option_139789492352144, option_139789492352416], **{'doc': u'', 'properties': (), 'name': 'file361'})
    option_139789492352688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492352960 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492353232 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492353504 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/fichier.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492353776 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492354048 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/fichier.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492354320 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492354592 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-fichier.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492363344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492367440 = OptionDescription(children=[option_139789492352688, option_139789492352960, option_139789492353232, option_139789492353504, option_139789492353776, option_139789492354048, option_139789492354320, option_139789492354592, option_139789492363344], **{'doc': u'', 'properties': (), 'name': 'file362'})
    option_139789492363616 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492363888 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492364160 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492364432 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/samba/smb.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492364704 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492364976 = UnicodeOption(**{'default': u'/etc/samba/smb.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492365248 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492365520 = UnicodeOption(**{'default': u'/var/lib/creole/smb.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492365792 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492367760 = OptionDescription(children=[option_139789492363616, option_139789492363888, option_139789492364160, option_139789492364432, option_139789492364704, option_139789492364976, option_139789492365248, option_139789492365520, option_139789492365792], **{'doc': u'', 'properties': (), 'name': 'file363'})
    option_139789492366064 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492366336 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492366608 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492366880 = UnicodeOption(**{'default': u'ntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492375632 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/samba/smb.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492375904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492376176 = UnicodeOption(**{'default': u'/etc/samba/smb.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492376448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492376720 = UnicodeOption(**{'default': u'/var/lib/creole/smb-proxy_auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492376992 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}, {'expected': u'NTLM/SMB', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789492368080 = OptionDescription(children=[option_139789492366064, option_139789492366336, option_139789492366608, option_139789492366880, option_139789492375632, option_139789492375904, option_139789492376176, option_139789492376448, option_139789492376720, option_139789492376992], **{'doc': u'', 'properties': (), 'name': 'file364'})
    option_139789492377264 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492377536 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492377808 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492378080 = UnicodeOption(**{'default': u'smb_trash', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492378352 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/samba/recycle.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492378624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492378896 = UnicodeOption(**{'default': u'/etc/samba/recycle.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492379168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492383824 = UnicodeOption(**{'default': u'/var/lib/creole/recycle.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492384096 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497061456}]})
    option_139789492368400 = OptionDescription(children=[option_139789492377264, option_139789492377536, option_139789492377808, option_139789492378080, option_139789492378352, option_139789492378624, option_139789492378896, option_139789492379168, option_139789492383824, option_139789492384096], **{'doc': u'', 'properties': (), 'name': 'file365'})
    option_139789492384368 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492384640 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492384912 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492385184 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/bastion/data/80-eole_fichier_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492385456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492385728 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492386000 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/80-eole_fichier_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492386272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492386544 = UnicodeOption(**{'default': u'/var/lib/creole/80-eole_fichier_rules', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492386816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492368720 = OptionDescription(children=[option_139789492384368, option_139789492384640, option_139789492384912, option_139789492385184, option_139789492385456, option_139789492385728, option_139789492386000, option_139789492386272, option_139789492386544, option_139789492386816], **{'doc': u'', 'properties': (), 'name': 'file366'})
    option_139789492387088 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492387360 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492392016 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492392288 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/logrotate.d/samba', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492392560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492392832 = UnicodeOption(**{'default': u'/etc/logrotate.d/samba', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492393104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492393376 = UnicodeOption(**{'default': u'/var/lib/creole/samba.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492393648 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492369040 = OptionDescription(children=[option_139789492387088, option_139789492387360, option_139789492392016, option_139789492392288, option_139789492392560, option_139789492392832, option_139789492393104, option_139789492393376, option_139789492393648], **{'doc': u'', 'properties': (), 'name': 'file367'})
    option_139789492393920 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492394192 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492394464 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492394736 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/pam.d/login', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492395008 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492395280 = UnicodeOption(**{'default': u'/etc/pam.d/login', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492395552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492400208 = UnicodeOption(**{'default': u'/var/lib/creole/login', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492400480 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492369360 = OptionDescription(children=[option_139789492393920, option_139789492394192, option_139789492394464, option_139789492394736, option_139789492395008, option_139789492395280, option_139789492395552, option_139789492400208, option_139789492400480], **{'doc': u'', 'properties': (), 'name': 'file368'})
    option_139789492400752 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492401024 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492401296 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492401568 = UnicodeOption(**{'default': u'nscd_files', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492401840 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/nscd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492402112 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492402384 = UnicodeOption(**{'default': u'/etc/nscd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492402656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492402928 = UnicodeOption(**{'default': u'/var/lib/creole/nscd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492403200 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492369680 = OptionDescription(children=[option_139789492400752, option_139789492401024, option_139789492401296, option_139789492401568, option_139789492401840, option_139789492402112, option_139789492402384, option_139789492402656, option_139789492402928, option_139789492403200], **{'doc': u'', 'properties': (), 'name': 'file369'})
    option_139789492403472 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492403744 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492412496 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492412768 = UnicodeOption(**{'default': u'/etc/nut/nut.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492413040 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492413312 = UnicodeOption(**{'default': u'/etc/nut/nut.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492413584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492413856 = UnicodeOption(**{'default': u'/var/lib/creole/nut.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492414128 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492370000 = OptionDescription(children=[option_139789492403472, option_139789492403744, option_139789492412496, option_139789492412768, option_139789492413040, option_139789492413312, option_139789492413584, option_139789492413856, option_139789492414128], **{'doc': u'', 'properties': (), 'name': 'file370'})
    option_139789492414400 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492414672 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492414944 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492415216 = UnicodeOption(**{'default': u'nutmaster', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492415488 = UnicodeOption(**{'default': u'/etc/nut/ups.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492415760 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789492416032 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492420688 = UnicodeOption(**{'default': u'640', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492420960 = UnicodeOption(**{'default': u'/etc/nut/ups.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492421232 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'owner'})
    option_139789492421504 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492421776 = UnicodeOption(**{'default': u'/var/lib/creole/ups.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492422048 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789492370320 = OptionDescription(children=[option_139789492414400, option_139789492414672, option_139789492414944, option_139789492415216, option_139789492415488, option_139789492415760, option_139789492416032, option_139789492420688, option_139789492420960, option_139789492421232, option_139789492421504, option_139789492421776, option_139789492422048], **{'doc': u'', 'properties': (), 'name': 'file371'})
    option_139789492422320 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492422592 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492422864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492423136 = UnicodeOption(**{'default': u'nutmaster', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492423408 = UnicodeOption(**{'default': u'/etc/nut/upsd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492423680 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789492423952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492424224 = UnicodeOption(**{'default': u'640', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492432976 = UnicodeOption(**{'default': u'/etc/nut/upsd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492433248 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'owner'})
    option_139789492433520 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492433792 = UnicodeOption(**{'default': u'/var/lib/creole/upsd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492434064 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789492370640 = OptionDescription(children=[option_139789492422320, option_139789492422592, option_139789492422864, option_139789492423136, option_139789492423408, option_139789492423680, option_139789492423952, option_139789492424224, option_139789492432976, option_139789492433248, option_139789492433520, option_139789492433792, option_139789492434064], **{'doc': u'', 'properties': (), 'name': 'file372'})
    option_139789492434336 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492434608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492434880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492435152 = UnicodeOption(**{'default': u'nutmaster', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492435424 = UnicodeOption(**{'default': u'/etc/nut/upsd.users', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492435696 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789492435968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492436240 = UnicodeOption(**{'default': u'640', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492436512 = UnicodeOption(**{'default': u'/etc/nut/upsd.users', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492445264 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'owner'})
    option_139789492445536 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492445808 = UnicodeOption(**{'default': u'/var/lib/creole/upsd.users', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492446080 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497094496}]})
    option_139789492370960 = OptionDescription(children=[option_139789492434336, option_139789492434608, option_139789492434880, option_139789492435152, option_139789492435424, option_139789492435696, option_139789492435968, option_139789492436240, option_139789492436512, option_139789492445264, option_139789492445536, option_139789492445808, option_139789492446080], **{'doc': u'', 'properties': (), 'name': 'file373'})
    option_139789492446352 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492446624 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492446896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492447168 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492447440 = UnicodeOption(**{'default': u'/etc/nut/upsmon.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492447712 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789492447984 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492448256 = UnicodeOption(**{'default': u'640', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492448528 = UnicodeOption(**{'default': u'/etc/nut/upsmon.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492448800 = UnicodeOption(**{'default': u'nut', 'doc': u'', 'multi': False, 'properties': (), 'name': 'owner'})
    option_139789492453456 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492453728 = UnicodeOption(**{'default': u'/var/lib/creole/upsmon.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492454000 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957104}]})
    option_139789492457552 = OptionDescription(children=[option_139789492446352, option_139789492446624, option_139789492446896, option_139789492447168, option_139789492447440, option_139789492447712, option_139789492447984, option_139789492448256, option_139789492448528, option_139789492448800, option_139789492453456, option_139789492453728, option_139789492454000], **{'doc': u'', 'properties': (), 'name': 'file374'})
    option_139789492454272 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492454544 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492454816 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492455088 = UnicodeOption(**{'default': u'saltstack', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492455360 = UnicodeOption(**{'default': u'/etc/salt/minion', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492455632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492455904 = UnicodeOption(**{'default': u'/etc/salt/minion', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492456176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492456448 = UnicodeOption(**{'default': u'/var/lib/creole/minion', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492456720 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963664}]})
    option_139789492457872 = OptionDescription(children=[option_139789492454272, option_139789492454544, option_139789492454816, option_139789492455088, option_139789492455360, option_139789492455632, option_139789492455904, option_139789492456176, option_139789492456448, option_139789492456720], **{'doc': u'', 'properties': (), 'name': 'file375'})
    option_139789492456992 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492465744 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492466016 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492466288 = UnicodeOption(**{'default': u'saltstack', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492466560 = UnicodeOption(**{'default': u'/etc/salt/minion.d/smtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492466832 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492467104 = UnicodeOption(**{'default': u'/etc/salt/minion.d/smtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492467376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492467648 = UnicodeOption(**{'default': u'/var/lib/creole/minion.smtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492467920 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963664}]})
    option_139789492458192 = OptionDescription(children=[option_139789492456992, option_139789492465744, option_139789492466016, option_139789492466288, option_139789492466560, option_139789492466832, option_139789492467104, option_139789492467376, option_139789492467648, option_139789492467920], **{'doc': u'', 'properties': (), 'name': 'file376'})
    option_139789492468192 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492468464 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492468736 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492469008 = UnicodeOption(**{'default': u'saltstack', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492469280 = UnicodeOption(**{'default': u'/etc/salt/master', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492473936 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492474208 = UnicodeOption(**{'default': u'/etc/salt/master', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492474480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492474752 = UnicodeOption(**{'default': u'/var/lib/creole/master', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492475024 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963664}]})
    option_139789492458512 = OptionDescription(children=[option_139789492468192, option_139789492468464, option_139789492468736, option_139789492469008, option_139789492469280, option_139789492473936, option_139789492474208, option_139789492474480, option_139789492474752, option_139789492475024], **{'doc': u'', 'properties': (), 'name': 'file377'})
    option_139789492475296 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492475568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492475840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492476112 = UnicodeOption(**{'default': u'saltstack', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492476384 = UnicodeOption(**{'default': u'/etc/salt/master.d/cherry.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492476656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492476928 = UnicodeOption(**{'default': u'/etc/salt/master.d/cherry.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492477200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492477472 = UnicodeOption(**{'default': u'/var/lib/creole/cherry.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492482128 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963664}]})
    option_139789492458832 = OptionDescription(children=[option_139789492475296, option_139789492475568, option_139789492475840, option_139789492476112, option_139789492476384, option_139789492476656, option_139789492476928, option_139789492477200, option_139789492477472, option_139789492482128], **{'doc': u'', 'properties': (), 'name': 'file378'})
    option_139789492482400 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492482672 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492482944 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492483216 = UnicodeOption(**{'default': u'saltstack', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492483488 = UnicodeOption(**{'default': u'/etc/salt/master.d/ext_auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492483760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492484032 = UnicodeOption(**{'default': u'/etc/salt/master.d/ext_auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492484304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492484576 = UnicodeOption(**{'default': u'/var/lib/creole/ext_auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492484848 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963664}]})
    option_139789492459152 = OptionDescription(children=[option_139789492482400, option_139789492482672, option_139789492482944, option_139789492483216, option_139789492483488, option_139789492483760, option_139789492484032, option_139789492484304, option_139789492484576, option_139789492484848], **{'doc': u'', 'properties': (), 'name': 'file379'})
    option_139789492485120 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492485392 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492485664 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492490320 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492490592 = UnicodeOption(**{'default': u'/etc/strongswan.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492490864 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492491136 = UnicodeOption(**{'default': u'/etc/strongswan.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492491408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492491680 = UnicodeOption(**{'default': u'/var/lib/creole/strongswan.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492491952 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492459472 = OptionDescription(children=[option_139789492485120, option_139789492485392, option_139789492485664, option_139789492490320, option_139789492490592, option_139789492490864, option_139789492491136, option_139789492491408, option_139789492491680, option_139789492491952], **{'doc': u'', 'properties': (), 'name': 'file380'})
    option_139789492492224 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492492496 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492492768 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492493040 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492493312 = UnicodeOption(**{'default': u'/etc/strongswan.d/starter.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492493584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492493856 = UnicodeOption(**{'default': u'/etc/strongswan.d/starter.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492498512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492498784 = UnicodeOption(**{'default': u'/var/lib/creole/starter.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492499056 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492459792 = OptionDescription(children=[option_139789492492224, option_139789492492496, option_139789492492768, option_139789492493040, option_139789492493312, option_139789492493584, option_139789492493856, option_139789492498512, option_139789492498784, option_139789492499056], **{'doc': u'', 'properties': (), 'name': 'file381'})
    option_139789492499328 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492499600 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492499872 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492500144 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492500416 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492500688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492500960 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492501232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492501504 = UnicodeOption(**{'default': u'/var/lib/creole/charon.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492501776 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492460112 = OptionDescription(children=[option_139789492499328, option_139789492499600, option_139789492499872, option_139789492500144, option_139789492500416, option_139789492500688, option_139789492500960, option_139789492501232, option_139789492501504, option_139789492501776], **{'doc': u'', 'properties': (), 'name': 'file382'})
    option_139789492502048 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492510800 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492511072 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492511344 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492511616 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon-logging.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492511888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492512160 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon-logging.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492512432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492512704 = UnicodeOption(**{'default': u'/var/lib/creole/charon-logging.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492512976 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492460432 = OptionDescription(children=[option_139789492502048, option_139789492510800, option_139789492511072, option_139789492511344, option_139789492511616, option_139789492511888, option_139789492512160, option_139789492512432, option_139789492512704, option_139789492512976], **{'doc': u'', 'properties': (), 'name': 'file383'})
    option_139789492513248 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492513520 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492513792 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492514064 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492514336 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon/revocation.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492518992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492519264 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon/revocation.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492519536 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492519808 = UnicodeOption(**{'default': u'/var/lib/creole/revocation.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492520080 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492460752 = OptionDescription(children=[option_139789492513248, option_139789492513520, option_139789492513792, option_139789492514064, option_139789492514336, option_139789492518992, option_139789492519264, option_139789492519536, option_139789492519808, option_139789492520080], **{'doc': u'', 'properties': (), 'name': 'file384'})
    option_139789492520352 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492520624 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492520896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492521168 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492521440 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon/kernel-netlink.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492521712 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492521984 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon/kernel-netlink.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492522256 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492522528 = UnicodeOption(**{'default': u'/var/lib/creole/kernel-netlink.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492527184 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492461072 = OptionDescription(children=[option_139789492520352, option_139789492520624, option_139789492520896, option_139789492521168, option_139789492521440, option_139789492521712, option_139789492521984, option_139789492522256, option_139789492522528, option_139789492527184], **{'doc': u'', 'properties': (), 'name': 'file385'})
    option_139789492527456 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492527728 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492528000 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492528272 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492528544 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon/farp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492528816 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492529088 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon/farp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492529360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492529632 = UnicodeOption(**{'default': u'/var/lib/creole/farp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492529904 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492543568 = OptionDescription(children=[option_139789492527456, option_139789492527728, option_139789492528000, option_139789492528272, option_139789492528544, option_139789492528816, option_139789492529088, option_139789492529360, option_139789492529632, option_139789492529904], **{'doc': u'', 'properties': (), 'name': 'file386'})
    option_139789492530176 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492530448 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492530720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492027472 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492027744 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon/dhcp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492028016 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492028288 = UnicodeOption(**{'default': u'/etc/strongswan.d/charon/dhcp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492028560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492028832 = UnicodeOption(**{'default': u'/var/lib/creole/dhcp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492029104 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492543888 = OptionDescription(children=[option_139789492530176, option_139789492530448, option_139789492530720, option_139789492027472, option_139789492027744, option_139789492028016, option_139789492028288, option_139789492028560, option_139789492028832, option_139789492029104], **{'doc': u'', 'properties': (), 'name': 'file387'})
    option_139789492029376 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492029648 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492029920 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492030192 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492030464 = UnicodeOption(**{'default': u'/etc/ipsec.d/ipsec_updown', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492030736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492031008 = UnicodeOption(**{'default': u'755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492035664 = UnicodeOption(**{'default': u'/etc/ipsec.d/ipsec_updown', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492035936 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492036208 = UnicodeOption(**{'default': u'/var/lib/creole/ipsec_updown', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492036480 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492544208 = OptionDescription(children=[option_139789492029376, option_139789492029648, option_139789492029920, option_139789492030192, option_139789492030464, option_139789492030736, option_139789492031008, option_139789492035664, option_139789492035936, option_139789492036208, option_139789492036480], **{'doc': u'', 'properties': (), 'name': 'file388'})
    option_139789492036752 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492037024 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492037296 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492037568 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492037840 = UnicodeOption(**{'default': u'/etc/ipsec.d/ip_xfrm_policy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492038112 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492038384 = UnicodeOption(**{'default': u'755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789492038656 = UnicodeOption(**{'default': u'/etc/ipsec.d/ip_xfrm_policy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492038928 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492039200 = UnicodeOption(**{'default': u'/var/lib/creole/ip_xfrm_policy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492043856 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492544528 = OptionDescription(children=[option_139789492036752, option_139789492037024, option_139789492037296, option_139789492037568, option_139789492037840, option_139789492038112, option_139789492038384, option_139789492038656, option_139789492038928, option_139789492039200, option_139789492043856], **{'doc': u'', 'properties': (), 'name': 'file389'})
    option_139789492044128 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492044400 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492044672 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492044944 = UnicodeOption(**{'default': u'rvp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492045216 = UnicodeOption(**{'default': u'/usr/share/eole/test-rvp_more_ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492045488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492045760 = UnicodeOption(**{'default': u'/usr/share/eole/test-rvp_more_ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492046032 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492046304 = UnicodeOption(**{'default': u'/var/lib/creole/test-rvp_more_ip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492046576 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963936}]})
    option_139789492544848 = OptionDescription(children=[option_139789492044128, option_139789492044400, option_139789492044672, option_139789492044944, option_139789492045216, option_139789492045488, option_139789492045760, option_139789492046032, option_139789492046304, option_139789492046576], **{'doc': u'', 'properties': (), 'name': 'file390'})
    option_139789492046848 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492047120 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492047392 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492052048 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/ports.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492052320 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492052592 = UnicodeOption(**{'default': u'/etc/apache2/ports.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492052864 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492053136 = UnicodeOption(**{'default': u'/var/lib/creole/ports.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492053408 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492545168 = OptionDescription(children=[option_139789492046848, option_139789492047120, option_139789492047392, option_139789492052048, option_139789492052320, option_139789492052592, option_139789492052864, option_139789492053136, option_139789492053408], **{'doc': u'', 'properties': (), 'name': 'file391'})
    option_139789492053680 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492053952 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492054224 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492054496 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492054768 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/apache2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492055040 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492055312 = UnicodeOption(**{'default': u'/etc/apache2/apache2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492055584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492060240 = UnicodeOption(**{'default': u'/var/lib/creole/apache2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492060512 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789492545488 = OptionDescription(children=[option_139789492053680, option_139789492053952, option_139789492054224, option_139789492054496, option_139789492054768, option_139789492055040, option_139789492055312, option_139789492055584, option_139789492060240, option_139789492060512], **{'doc': u'', 'properties': (), 'name': 'file392'})
    option_139789492060784 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492061056 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492061328 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492061600 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492061872 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/conf-available/security.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492062144 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492062416 = UnicodeOption(**{'default': u'/etc/apache2/conf-available/security.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492062688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492062960 = UnicodeOption(**{'default': u'/var/lib/creole/apache-security', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492063232 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789492545808 = OptionDescription(children=[option_139789492060784, option_139789492061056, option_139789492061328, option_139789492061600, option_139789492061872, option_139789492062144, option_139789492062416, option_139789492062688, option_139789492062960, option_139789492063232], **{'doc': u'', 'properties': (), 'name': 'file393'})
    option_139789492063504 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492063776 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492068432 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492068704 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/sites-available/default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492068976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492069248 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492069520 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492069792 = UnicodeOption(**{'default': u'/var/lib/creole/apache-default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492070064 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492546128 = OptionDescription(children=[option_139789492063504, option_139789492063776, option_139789492068432, option_139789492068704, option_139789492068976, option_139789492069248, option_139789492069520, option_139789492069792, option_139789492070064], **{'doc': u'', 'properties': (), 'name': 'file394'})
    option_139789492070336 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492070608 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492070880 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492071152 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/sites-available/eole', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492071424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492071696 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/eole', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492071968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492076624 = UnicodeOption(**{'default': u'/var/lib/creole/apache-eole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492076896 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492546448 = OptionDescription(children=[option_139789492070336, option_139789492070608, option_139789492070880, option_139789492071152, option_139789492071424, option_139789492071696, option_139789492071968, option_139789492076624, option_139789492076896], **{'doc': u'', 'properties': (), 'name': 'file395'})
    option_139789492077168 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492077440 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492077712 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492077984 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/sites-available/Vhost', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492078256 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492078528 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/Vhost', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492078800 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492079072 = UnicodeOption(**{'default': u'/var/lib/creole/Vhost.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492079344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492546768 = OptionDescription(children=[option_139789492077168, option_139789492077440, option_139789492077712, option_139789492077984, option_139789492078256, option_139789492078528, option_139789492078800, option_139789492079072, option_139789492079344], **{'doc': u'', 'properties': (), 'name': 'file396'})
    option_139789492079616 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492079888 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492080160 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492084816 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/sites-available/Vhost-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492085088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492085360 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/Vhost-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492085632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492085904 = UnicodeOption(**{'default': u'/var/lib/creole/Vhost-ssl.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492086176 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492547088 = OptionDescription(children=[option_139789492079616, option_139789492079888, option_139789492080160, option_139789492084816, option_139789492085088, option_139789492085360, option_139789492085632, option_139789492085904, option_139789492086176], **{'doc': u'', 'properties': (), 'name': 'file397'})
    option_139789492086448 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492086720 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492086992 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492087264 = UnicodeOption(**{'default': u';', 'doc': u'', 'multi': False, 'properties': (), 'name': 'del_comment'})
    option_139789492087536 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492087808 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/php/7.0/apache2/php.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492088080 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492088352 = UnicodeOption(**{'default': u'/etc/php/7.0/apache2/php.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492093008 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492093280 = UnicodeOption(**{'default': u'/var/lib/creole/php.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492093552 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789492101200 = OptionDescription(children=[option_139789492086448, option_139789492086720, option_139789492086992, option_139789492087264, option_139789492087536, option_139789492087808, option_139789492088080, option_139789492088352, option_139789492093008, option_139789492093280, option_139789492093552], **{'doc': u'', 'properties': (), 'name': 'file398'})
    option_139789492093824 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492094096 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492094368 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492094640 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492094912 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/php/7.0/mods-available/eole.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492095184 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492095456 = UnicodeOption(**{'default': u'/etc/php/7.0/mods-available/eole.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492095728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492096000 = UnicodeOption(**{'default': u'/var/lib/creole/php-eole.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492096272 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789492101520 = OptionDescription(children=[option_139789492093824, option_139789492094096, option_139789492094368, option_139789492094640, option_139789492094912, option_139789492095184, option_139789492095456, option_139789492095728, option_139789492096000, option_139789492096272], **{'doc': u'', 'properties': (), 'name': 'file399'})
    option_139789492096544 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492105296 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492105568 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492105840 = UnicodeOption(**{'default': u'apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492106112 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/mods-available/mpm_prefork.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492106384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492106656 = UnicodeOption(**{'default': u'/etc/apache2/mods-available/mpm_prefork.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492106928 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492107200 = UnicodeOption(**{'default': u'/var/lib/creole/mpm_prefork.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492107472 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789492101840 = OptionDescription(children=[option_139789492096544, option_139789492105296, option_139789492105568, option_139789492105840, option_139789492106112, option_139789492106384, option_139789492106656, option_139789492106928, option_139789492107200, option_139789492107472], **{'doc': u'', 'properties': (), 'name': 'file400'})
    option_139789492107744 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492108016 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492108288 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492108560 = UnicodeOption(**{'default': u'remoteip', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492108832 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/mods-available/remoteip.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492117584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492117856 = UnicodeOption(**{'default': u'/etc/apache2/mods-available/remoteip.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492118128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492118400 = UnicodeOption(**{'default': u'/var/lib/creole/remoteip.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492118672 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497143920}]})
    option_139789492102160 = OptionDescription(children=[option_139789492107744, option_139789492108016, option_139789492108288, option_139789492108560, option_139789492108832, option_139789492117584, option_139789492117856, option_139789492118128, option_139789492118400, option_139789492118672], **{'doc': u'', 'properties': (), 'name': 'file401'})
    option_139789492118944 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492119216 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492119488 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492119760 = UnicodeOption(**{'default': u'ssoclient', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492120032 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/php/configCAS/cas.inc.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492120304 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492120576 = UnicodeOption(**{'default': u'/usr/share/php/configCAS/cas.inc.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492120848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492121120 = UnicodeOption(**{'default': u'/var/lib/creole/cas.inc.php.tmpl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492125776 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}]})
    option_139789492102480 = OptionDescription(children=[option_139789492118944, option_139789492119216, option_139789492119488, option_139789492119760, option_139789492120032, option_139789492120304, option_139789492120576, option_139789492120848, option_139789492121120, option_139789492125776], **{'doc': u'', 'properties': (), 'name': 'file402'})
    option_139789492126048 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492126320 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492126592 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492126864 = UnicodeOption(**{'default': u'couriersso', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492127136 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/php/configCAS/cas.inc.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492127408 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492127680 = UnicodeOption(**{'default': u'/usr/share/php/configCAS/cas.inc.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492127952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492128224 = UnicodeOption(**{'default': u'/var/lib/creole/cas.inc.php.tmpl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492128496 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496747696}]})
    option_139789492102800 = OptionDescription(children=[option_139789492126048, option_139789492126320, option_139789492126592, option_139789492126864, option_139789492127136, option_139789492127408, option_139789492127680, option_139789492127952, option_139789492128224, option_139789492128496], **{'doc': u'', 'properties': (), 'name': 'file403'})
    option_139789492128768 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492129040 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492129312 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492142160 = UnicodeOption(**{'default': u'ssoclient', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492142432 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/php/CAS/eoleCASConfig.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492142704 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492142976 = UnicodeOption(**{'default': u'/usr/share/php/CAS/eoleCASConfig.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492143248 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492143520 = UnicodeOption(**{'default': u'/var/lib/creole/eoleCASConfig.php.tmpl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492143792 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}]})
    option_139789492103120 = OptionDescription(children=[option_139789492128768, option_139789492129040, option_139789492129312, option_139789492142160, option_139789492142432, option_139789492142704, option_139789492142976, option_139789492143248, option_139789492143520, option_139789492143792], **{'doc': u'', 'properties': (), 'name': 'file404'})
    option_139789492144064 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492144336 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492144608 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492144880 = UnicodeOption(**{'default': u'bareoswebui', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492145152 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/bareos-webui/directors.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492145424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492145696 = UnicodeOption(**{'default': u'/etc/bareos-webui/directors.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492150352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492150624 = UnicodeOption(**{'default': u'/var/lib/creole/bareoswebui-directors.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492150896 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145280}]})
    option_139789492103440 = OptionDescription(children=[option_139789492144064, option_139789492144336, option_139789492144608, option_139789492144880, option_139789492145152, option_139789492145424, option_139789492145696, option_139789492150352, option_139789492150624, option_139789492150896], **{'doc': u'', 'properties': (), 'name': 'file405'})
    option_139789492151168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492151440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492151712 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492151984 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.sbin.named', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492152256 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492152528 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.sbin.named', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492152800 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492153072 = UnicodeOption(**{'default': u'/var/lib/creole/named.apparmor.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492153344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492103760 = OptionDescription(children=[option_139789492151168, option_139789492151440, option_139789492151712, option_139789492151984, option_139789492152256, option_139789492152528, option_139789492152800, option_139789492153072, option_139789492153344], **{'doc': u'', 'properties': (), 'name': 'file406'})
    option_139789492153616 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492153888 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492158544 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492158816 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/bind/named.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492159088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492159360 = UnicodeOption(**{'default': u'/etc/bind/named.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492159632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492159904 = UnicodeOption(**{'default': u'/var/lib/creole/named.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492160176 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492104080 = OptionDescription(children=[option_139789492153616, option_139789492153888, option_139789492158544, option_139789492158816, option_139789492159088, option_139789492159360, option_139789492159632, option_139789492159904, option_139789492160176], **{'doc': u'', 'properties': (), 'name': 'file407'})
    option_139789492160448 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492160720 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492160992 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492161264 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/bind/forward.zones', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492161536 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492161808 = UnicodeOption(**{'default': u'/etc/bind/forward.zones', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492162080 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492166736 = UnicodeOption(**{'default': u'/var/lib/creole/forward.zones', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492167008 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492104400 = OptionDescription(children=[option_139789492160448, option_139789492160720, option_139789492160992, option_139789492161264, option_139789492161536, option_139789492161808, option_139789492162080, option_139789492166736, option_139789492167008], **{'doc': u'', 'properties': (), 'name': 'file408'})
    option_139789492167280 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492167552 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492167824 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492168096 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/default/bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492168368 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492168640 = UnicodeOption(**{'default': u'/etc/default/bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492168912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492169184 = UnicodeOption(**{'default': u'/var/lib/creole/bind9', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492169456 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492104720 = OptionDescription(children=[option_139789492167280, option_139789492167552, option_139789492167824, option_139789492168096, option_139789492168368, option_139789492168640, option_139789492168912, option_139789492169184, option_139789492169456], **{'doc': u'', 'properties': (), 'name': 'file409'})
    option_139789492169728 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492170000 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492170272 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492174928 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/systemd/system/bind9.service.d/override.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492175200 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492175472 = UnicodeOption(**{'default': u'/etc/systemd/system/bind9.service.d/override.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492175744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492176016 = UnicodeOption(**{'default': u'/var/lib/creole/bind9.override', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492176288 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492183120 = OptionDescription(children=[option_139789492169728, option_139789492170000, option_139789492170272, option_139789492174928, option_139789492175200, option_139789492175472, option_139789492175744, option_139789492176016, option_139789492176288], **{'doc': u'', 'properties': (), 'name': 'file410'})
    option_139789492176560 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492176832 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492177104 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492177376 = UnicodeOption(**{'default': u'agriates', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492177648 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/bind/agriates.zones', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492177920 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492178192 = UnicodeOption(**{'default': u'/etc/bind/agriates.zones', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492178464 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492187216 = UnicodeOption(**{'default': u'/var/lib/creole/agriates.zones', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492187488 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497655648}]})
    option_139789492183440 = OptionDescription(children=[option_139789492176560, option_139789492176832, option_139789492177104, option_139789492177376, option_139789492177648, option_139789492177920, option_139789492178192, option_139789492178464, option_139789492187216, option_139789492187488], **{'doc': u'', 'properties': (), 'name': 'file411'})
    option_139789492187760 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492188032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492188304 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492188576 = UnicodeOption(**{'default': u'/var/lib/eole/config/ead.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492188848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492189120 = UnicodeOption(**{'default': u'/var/lib/eole/config/ead.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492189392 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492189664 = UnicodeOption(**{'default': u'/var/lib/creole/ead.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492189936 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492183760 = OptionDescription(children=[option_139789492187760, option_139789492188032, option_139789492188304, option_139789492188576, option_139789492188848, option_139789492189120, option_139789492189392, option_139789492189664, option_139789492189936], **{'doc': u'', 'properties': (), 'name': 'file412'})
    option_139789492190208 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492190480 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492190752 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492195408 = UnicodeOption(**{'default': u'/etc/rsyslog.d/aggregation/squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492195680 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492195952 = UnicodeOption(**{'default': u'/etc/rsyslog.d/aggregation/squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492196224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492196496 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_aggregation_squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492196768 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492184080 = OptionDescription(children=[option_139789492190208, option_139789492190480, option_139789492190752, option_139789492195408, option_139789492195680, option_139789492195952, option_139789492196224, option_139789492196496, option_139789492196768], **{'doc': u'', 'properties': (), 'name': 'file413'})
    option_139789492197040 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492197312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492197584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492197856 = UnicodeOption(**{'default': u'/etc/rsyslog.d/aggregation/dansguardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492198128 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492198400 = UnicodeOption(**{'default': u'/etc/rsyslog.d/aggregation/dansguardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492198672 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492198944 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_aggregation_dansguardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492203600 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492184400 = OptionDescription(children=[option_139789492197040, option_139789492197312, option_139789492197584, option_139789492197856, option_139789492198128, option_139789492198400, option_139789492198672, option_139789492198944, option_139789492203600], **{'doc': u'', 'properties': (), 'name': 'file414'})
    option_139789492203872 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492204144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492204416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492204688 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492204960 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492205232 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492205504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492205776 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492206048 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492184720 = OptionDescription(children=[option_139789492203872, option_139789492204144, option_139789492204416, option_139789492204688, option_139789492204960, option_139789492205232, option_139789492205504, option_139789492205776, option_139789492206048], **{'doc': u'', 'properties': (), 'name': 'file415'})
    option_139789492206320 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492206592 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492206864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492207136 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/dansguardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492215888 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492216160 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/dansguardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492216432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492216704 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_dansguardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492216976 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492185040 = OptionDescription(children=[option_139789492206320, option_139789492206592, option_139789492206864, option_139789492207136, option_139789492215888, option_139789492216160, option_139789492216432, option_139789492216704, option_139789492216976], **{'doc': u'', 'properties': (), 'name': 'file416'})
    option_139789492217248 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492217520 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492217792 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492218064 = UnicodeOption(**{'default': u'mysql-bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492218336 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/mysql.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492218608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492218880 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/mysql.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492219152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492219424 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-mysql.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492228176 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971312}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789492185360 = OptionDescription(children=[option_139789492217248, option_139789492217520, option_139789492217792, option_139789492218064, option_139789492218336, option_139789492218608, option_139789492218880, option_139789492219152, option_139789492219424, option_139789492228176], **{'doc': u'', 'properties': (), 'name': 'file417'})
    option_139789492228448 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492228720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492228992 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492229264 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/debian-start.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492229536 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492229808 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/debian-start.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492230080 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492230352 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_debian-start.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492230624 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492185680 = OptionDescription(children=[option_139789492228448, option_139789492228720, option_139789492228992, option_139789492229264, option_139789492229536, option_139789492229808, option_139789492230080, option_139789492230352, option_139789492230624], **{'doc': u'', 'properties': (), 'name': 'file418'})
    option_139789492230896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492231168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492231440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492231712 = UnicodeOption(**{'default': u'/etc/logrotate.d/mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492244560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492244832 = UnicodeOption(**{'default': u'/etc/logrotate.d/mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492245104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492245376 = UnicodeOption(**{'default': u'/var/lib/creole/mysql.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492245648 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492186000 = OptionDescription(children=[option_139789492230896, option_139789492231168, option_139789492231440, option_139789492231712, option_139789492244560, option_139789492244832, option_139789492245104, option_139789492245376, option_139789492245648], **{'doc': u'', 'properties': (), 'name': 'file419'})
    option_139789492245920 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492246192 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492246464 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492246736 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.sbin.mysqld', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492247008 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492247280 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.sbin.mysqld', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492247552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492247824 = UnicodeOption(**{'default': u'/var/lib/creole/mysqld.apparmor.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492248096 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492186320 = OptionDescription(children=[option_139789492245920, option_139789492246192, option_139789492246464, option_139789492246736, option_139789492247008, option_139789492247280, option_139789492247552, option_139789492247824, option_139789492248096], **{'doc': u'', 'properties': (), 'name': 'file420'})
    option_139789492252752 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492253024 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492253296 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492253568 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492253840 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/mysql/mysql.conf.d/mysqld.cnf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492254112 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492254384 = UnicodeOption(**{'default': u'/etc/mysql/mysql.conf.d/mysqld.cnf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492254656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492254928 = UnicodeOption(**{'default': u'/var/lib/creole/mysqld.cnf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492255200 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971312}]})
    option_139789492186640 = OptionDescription(children=[option_139789492252752, option_139789492253024, option_139789492253296, option_139789492253568, option_139789492253840, option_139789492254112, option_139789492254384, option_139789492254656, option_139789492254928, option_139789492255200], **{'doc': u'', 'properties': (), 'name': 'file421'})
    option_139789492255472 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492255744 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492256016 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492256288 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492260944 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/mysql/conf.d/mysql_mode.cnf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492261216 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492261488 = UnicodeOption(**{'default': u'/etc/mysql/conf.d/mysql_mode.cnf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492261760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492262032 = UnicodeOption(**{'default': u'/var/lib/creole/mysql_mode.cnf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492262304 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971312}]})
    option_139789492269136 = OptionDescription(children=[option_139789492255472, option_139789492255744, option_139789492256016, option_139789492256288, option_139789492260944, option_139789492261216, option_139789492261488, option_139789492261760, option_139789492262032, option_139789492262304], **{'doc': u'', 'properties': (), 'name': 'file422'})
    option_139789492262576 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492262848 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492263120 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492263392 = UnicodeOption(**{'default': u'spamassassin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492263664 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/default/spamassassin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492263936 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492264208 = UnicodeOption(**{'default': u'/etc/default/spamassassin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492264480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492273232 = UnicodeOption(**{'default': u'/var/lib/creole/spamassassin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492273504 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496746064}]})
    option_139789492269456 = OptionDescription(children=[option_139789492262576, option_139789492262848, option_139789492263120, option_139789492263392, option_139789492263664, option_139789492263936, option_139789492264208, option_139789492264480, option_139789492273232, option_139789492273504], **{'doc': u'', 'properties': (), 'name': 'file423'})
    option_139789492273776 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492274048 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492274320 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492274592 = UnicodeOption(**{'default': u'spamassassin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492274864 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/mail/spamassassin/local.pre', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492275136 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492275408 = UnicodeOption(**{'default': u'/etc/mail/spamassassin/local.pre', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492275680 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492275952 = UnicodeOption(**{'default': u'/var/lib/creole/local.pre', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492276224 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496746064}]})
    option_139789492269776 = OptionDescription(children=[option_139789492273776, option_139789492274048, option_139789492274320, option_139789492274592, option_139789492274864, option_139789492275136, option_139789492275408, option_139789492275680, option_139789492275952, option_139789492276224], **{'doc': u'', 'properties': (), 'name': 'file424'})
    option_139789492276496 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492276768 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492281424 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492281696 = UnicodeOption(**{'default': u'spamassassin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492281968 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/mail/spamassassin/local.cf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492282240 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492282512 = UnicodeOption(**{'default': u'/etc/mail/spamassassin/local.cf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492282784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492283056 = UnicodeOption(**{'default': u'/var/lib/creole/local.cf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492283328 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496746064}]})
    option_139789492270096 = OptionDescription(children=[option_139789492276496, option_139789492276768, option_139789492281424, option_139789492281696, option_139789492281968, option_139789492282240, option_139789492282512, option_139789492282784, option_139789492283056, option_139789492283328], **{'doc': u'', 'properties': (), 'name': 'file425'})
    option_139789492283600 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492283872 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492284144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492284416 = UnicodeOption(**{'default': u'/etc/eole/eoleldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492284688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492284960 = UnicodeOption(**{'default': u'/etc/eole/eoleldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491769424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491769696 = UnicodeOption(**{'default': u'/var/lib/creole/eoleldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491769968 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492270416 = OptionDescription(children=[option_139789492283600, option_139789492283872, option_139789492284144, option_139789492284416, option_139789492284688, option_139789492284960, option_139789491769424, option_139789491769696, option_139789491769968], **{'doc': u'', 'properties': (), 'name': 'file426'})
    option_139789491770240 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491770512 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491770784 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491771056 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/eole/eoleldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491771328 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491771600 = UnicodeOption(**{'default': u'/etc/eole/eoleldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491771872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491772144 = UnicodeOption(**{'default': u'/var/lib/creole/eoleldap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491772416 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492270736 = OptionDescription(children=[option_139789491770240, option_139789491770512, option_139789491770784, option_139789491771056, option_139789491771328, option_139789491771600, option_139789491771872, option_139789491772144, option_139789491772416], **{'doc': u'', 'properties': (), 'name': 'file427'})
    option_139789491772688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491772960 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491777616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491777888 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491778160 = UnicodeOption(**{'default': u'/tmp/DB_CONFIG', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491778432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491778704 = UnicodeOption(**{'default': u'/tmp/DB_CONFIG', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491778976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491779248 = UnicodeOption(**{'default': u'/var/lib/creole/DB_CONFIG', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491779520 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497955744}]})
    option_139789492271056 = OptionDescription(children=[option_139789491772688, option_139789491772960, option_139789491777616, option_139789491777888, option_139789491778160, option_139789491778432, option_139789491778704, option_139789491778976, option_139789491779248, option_139789491779520], **{'doc': u'', 'properties': (), 'name': 'file428'})
    option_139789491779792 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491780064 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491780336 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491780608 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491780880 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/ldap/slapd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491781152 = UnicodeOption(**{'default': u'openldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789491789904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491790176 = UnicodeOption(**{'default': u'0640', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789491790448 = UnicodeOption(**{'default': u'/etc/ldap/slapd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491790720 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491790992 = UnicodeOption(**{'default': u'/var/lib/creole/slapd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491791264 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497955744}]})
    option_139789492271376 = OptionDescription(children=[option_139789491779792, option_139789491780064, option_139789491780336, option_139789491780608, option_139789491780880, option_139789491781152, option_139789491789904, option_139789491790176, option_139789491790448, option_139789491790720, option_139789491790992, option_139789491791264], **{'doc': u'', 'properties': (), 'name': 'file429'})
    option_139789491791536 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491791808 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491792080 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491792352 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491792624 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/default/slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491792896 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491793168 = UnicodeOption(**{'default': u'/etc/default/slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491793440 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491798096 = UnicodeOption(**{'default': u'/var/lib/creole/default.slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491798368 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497955744}]})
    option_139789492271696 = OptionDescription(children=[option_139789491791536, option_139789491791808, option_139789491792080, option_139789491792352, option_139789491792624, option_139789491792896, option_139789491793168, option_139789491793440, option_139789491798096, option_139789491798368], **{'doc': u'', 'properties': (), 'name': 'file430'})
    option_139789491798640 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491798912 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491799184 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491799456 = UnicodeOption(**{'default': u'slapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491799728 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/var/lib/eole/00_annuaire.ldif', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491800000 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491800272 = UnicodeOption(**{'default': u'/var/lib/eole/00_annuaire.ldif', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491800544 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491800816 = UnicodeOption(**{'default': u'/var/lib/creole/00_annuaire.ldif', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491801088 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'local', 'action': 'disabled', 'inverse': True, "option": option_139789497955744}]})
    option_139789492272016 = OptionDescription(children=[option_139789491798640, option_139789491798912, option_139789491799184, option_139789491799456, option_139789491799728, option_139789491800000, option_139789491800272, option_139789491800544, option_139789491800816, option_139789491801088], **{'doc': u'', 'properties': (), 'name': 'file431'})
    option_139789491801360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491801632 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491806288 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491806560 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491806832 = UnicodeOption(**{'default': u'/etc/bareos/query.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491807104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491807376 = UnicodeOption(**{'default': u'/etc/bareos/query.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491807648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491807920 = UnicodeOption(**{'default': u'/var/lib/creole/query.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491808192 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789492272336 = OptionDescription(children=[option_139789491801360, option_139789491801632, option_139789491806288, option_139789491806560, option_139789491806832, option_139789491807104, option_139789491807376, option_139789491807648, option_139789491807920, option_139789491808192], **{'doc': u'', 'properties': (), 'name': 'file432'})
    option_139789491808464 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491808736 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491809008 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491809280 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491809552 = UnicodeOption(**{'default': u'/etc/bareos/include-options.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491809824 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491814480 = UnicodeOption(**{'default': u'/etc/bareos/include-options.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491814752 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491815024 = UnicodeOption(**{'default': u'/var/lib/creole/include-options.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491815296 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789492272656 = OptionDescription(children=[option_139789491808464, option_139789491808736, option_139789491809008, option_139789491809280, option_139789491809552, option_139789491809824, option_139789491814480, option_139789491814752, option_139789491815024, option_139789491815296], **{'doc': u'', 'properties': (), 'name': 'file433'})
    option_139789491815568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491815840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491816112 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491816384 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491816656 = UnicodeOption(**{'default': u'/etc/bareos/bareos-fd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491816928 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491817200 = UnicodeOption(**{'default': u'/etc/bareos/bareos-fd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491817472 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491817744 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-fd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491818016 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491822672 = OptionDescription(children=[option_139789491815568, option_139789491815840, option_139789491816112, option_139789491816384, option_139789491816656, option_139789491816928, option_139789491817200, option_139789491817472, option_139789491817744, option_139789491818016], **{'doc': u'', 'properties': (), 'name': 'file434'})
    option_139789491826768 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491827040 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491827312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491827584 = UnicodeOption(**{'default': u'bareos_sd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491827856 = UnicodeOption(**{'default': u'/etc/bareos/bareos-sd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491828128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491828400 = UnicodeOption(**{'default': u'/etc/bareos/bareos-sd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491828672 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491828944 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-sd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491829216 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971040}]})
    option_139789491822992 = OptionDescription(children=[option_139789491826768, option_139789491827040, option_139789491827312, option_139789491827584, option_139789491827856, option_139789491828128, option_139789491828400, option_139789491828672, option_139789491828944, option_139789491829216], **{'doc': u'', 'properties': (), 'name': 'file435'})
    option_139789491829488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491829760 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491830032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491830304 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491834960 = UnicodeOption(**{'default': u'/etc/bareos/bareos-dir.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491835232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491835504 = UnicodeOption(**{'default': u'/etc/bareos/bareos-dir.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491835776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491836048 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-dir.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491836320 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491823312 = OptionDescription(children=[option_139789491829488, option_139789491829760, option_139789491830032, option_139789491830304, option_139789491834960, option_139789491835232, option_139789491835504, option_139789491835776, option_139789491836048, option_139789491836320], **{'doc': u'', 'properties': (), 'name': 'file436'})
    option_139789491836592 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491836864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491837136 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491837408 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491837680 = UnicodeOption(**{'default': u'/etc/sudoers.d/bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491837952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491838224 = UnicodeOption(**{'default': u'0440', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789491838496 = UnicodeOption(**{'default': u'/etc/sudoers.d/bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491847248 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491847520 = UnicodeOption(**{'default': u'/var/lib/creole/bareos.sudoers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491847792 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491823632 = OptionDescription(children=[option_139789491836592, option_139789491836864, option_139789491837136, option_139789491837408, option_139789491837680, option_139789491837952, option_139789491838224, option_139789491838496, option_139789491847248, option_139789491847520, option_139789491847792], **{'doc': u'', 'properties': (), 'name': 'file437'})
    option_139789491848064 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491848336 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491848608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491848880 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491849152 = UnicodeOption(**{'default': u'/etc/bareos/bconsole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491849424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491849696 = UnicodeOption(**{'default': u'/etc/bareos/bconsole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491849968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491850240 = UnicodeOption(**{'default': u'/var/lib/creole/bconsole.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491850512 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491823952 = OptionDescription(children=[option_139789491848064, option_139789491848336, option_139789491848608, option_139789491848880, option_139789491849152, option_139789491849424, option_139789491849696, option_139789491849968, option_139789491850240, option_139789491850512], **{'doc': u'', 'properties': (), 'name': 'file438'})
    option_139789491850784 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491855440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491855712 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491855984 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491856256 = UnicodeOption(**{'default': u'/etc/bareos/bareossupport.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491856528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491856800 = UnicodeOption(**{'default': u'/etc/bareos/bareossupport.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491857072 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491857344 = UnicodeOption(**{'default': u'/var/lib/creole/bareossupport.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491857616 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491824272 = OptionDescription(children=[option_139789491850784, option_139789491855440, option_139789491855712, option_139789491855984, option_139789491856256, option_139789491856528, option_139789491856800, option_139789491857072, option_139789491857344, option_139789491857616], **{'doc': u'', 'properties': (), 'name': 'file439'})
    option_139789491857888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491858160 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491858432 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491858704 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491858976 = UnicodeOption(**{'default': u'/etc/bareos/bareosschedule.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491867728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491868000 = UnicodeOption(**{'default': u'/etc/bareos/bareosschedule.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491868272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491868544 = UnicodeOption(**{'default': u'/var/lib/creole/bareosschedule.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491868816 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491824592 = OptionDescription(children=[option_139789491857888, option_139789491858160, option_139789491858432, option_139789491858704, option_139789491858976, option_139789491867728, option_139789491868000, option_139789491868272, option_139789491868544, option_139789491868816], **{'doc': u'', 'properties': (), 'name': 'file440'})
    option_139789491869088 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491869360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491869632 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491869904 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491870176 = UnicodeOption(**{'default': u'/etc/bareos/bareosschedulepost.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491870448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491870720 = UnicodeOption(**{'default': u'/etc/bareos/bareosschedulepost.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491870992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491871264 = UnicodeOption(**{'default': u'/var/lib/creole/bareosschedulepost.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491875920 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491824912 = OptionDescription(children=[option_139789491869088, option_139789491869360, option_139789491869632, option_139789491869904, option_139789491870176, option_139789491870448, option_139789491870720, option_139789491870992, option_139789491871264, option_139789491875920], **{'doc': u'', 'properties': (), 'name': 'file441'})
    option_139789491876192 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491876464 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491876736 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491877008 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491877280 = UnicodeOption(**{'default': u'/etc/bareos/bareos-restore.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491877552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491877824 = UnicodeOption(**{'default': u'/etc/bareos/bareos-restore.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491878096 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491878368 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-restore.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491878640 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491825232 = OptionDescription(children=[option_139789491876192, option_139789491876464, option_139789491876736, option_139789491877008, option_139789491877280, option_139789491877552, option_139789491877824, option_139789491878096, option_139789491878368, option_139789491878640], **{'doc': u'', 'properties': (), 'name': 'file442'})
    option_139789491878912 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491879184 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491879456 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491888208 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491888480 = UnicodeOption(**{'default': u'/etc/bareos/bat.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491888752 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491889024 = UnicodeOption(**{'default': u'/etc/bareos/bat.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491889296 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491889568 = UnicodeOption(**{'default': u'/var/lib/creole/bat.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491889840 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491825552 = OptionDescription(children=[option_139789491878912, option_139789491879184, option_139789491879456, option_139789491888208, option_139789491888480, option_139789491888752, option_139789491889024, option_139789491889296, option_139789491889568, option_139789491889840], **{'doc': u'', 'properties': (), 'name': 'file443'})
    option_139789491890112 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491890384 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491890656 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491890928 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491891200 = UnicodeOption(**{'default': u'/etc/bareos/bareos-config-lib.sh', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491891472 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789491891744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491896400 = UnicodeOption(**{'default': u'0440', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789491896672 = UnicodeOption(**{'default': u'/etc/bareos/bareos-config-lib.sh', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491896944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491897216 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-config-lib.sh', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491897488 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789491825872 = OptionDescription(children=[option_139789491890112, option_139789491890384, option_139789491890656, option_139789491890928, option_139789491891200, option_139789491891472, option_139789491891744, option_139789491896400, option_139789491896672, option_139789491896944, option_139789491897216, option_139789491897488], **{'doc': u'', 'properties': (), 'name': 'file444'})
    option_139789491897760 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491898032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491898304 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491898576 = UnicodeOption(**{'default': u'/etc/logrotate.d/bareos-common', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491898848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491899120 = UnicodeOption(**{'default': u'/etc/logrotate.d/bareos-common', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491899392 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491899664 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-common.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491899936 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491826192 = OptionDescription(children=[option_139789491897760, option_139789491898032, option_139789491898304, option_139789491898576, option_139789491898848, option_139789491899120, option_139789491899392, option_139789491899664, option_139789491899936], **{'doc': u'', 'properties': (), 'name': 'file445'})
    option_139789491904592 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491904864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491905136 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491905408 = UnicodeOption(**{'default': u'courier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491905680 = UnicodeOption(**{'default': u'/usr/share/eole/annuaire/password_files/mail.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491905952 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491906224 = UnicodeOption(**{'default': u'/usr/share/eole/annuaire/password_files/mail.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491906496 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491906768 = UnicodeOption(**{'default': u'/var/lib/creole/mail.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491907040 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496747424}]})
    option_139789491912784 = OptionDescription(children=[option_139789491904592, option_139789491904864, option_139789491905136, option_139789491905408, option_139789491905680, option_139789491905952, option_139789491906224, option_139789491906496, option_139789491906768, option_139789491907040], **{'doc': u'', 'properties': (), 'name': 'file446'})
    option_139789491907312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491907584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491907856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491908128 = UnicodeOption(**{'default': u'courier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491916880 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/courier.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491917152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491917424 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/courier.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491917696 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491917968 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-courier.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491918240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496747424}]})
    option_139789491913104 = OptionDescription(children=[option_139789491907312, option_139789491907584, option_139789491907856, option_139789491908128, option_139789491916880, option_139789491917152, option_139789491917424, option_139789491917696, option_139789491917968, option_139789491918240], **{'doc': u'', 'properties': (), 'name': 'file447'})
    option_139789491918512 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491918784 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491919056 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491919328 = UnicodeOption(**{'default': u'courier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491919600 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/courier/authdaemonrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491919872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491920144 = UnicodeOption(**{'default': u'/etc/courier/authdaemonrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491920416 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491925072 = UnicodeOption(**{'default': u'/var/lib/creole/authdaemonrc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491925344 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496747424}]})
    option_139789491913424 = OptionDescription(children=[option_139789491918512, option_139789491918784, option_139789491919056, option_139789491919328, option_139789491919600, option_139789491919872, option_139789491920144, option_139789491920416, option_139789491925072, option_139789491925344], **{'doc': u'', 'properties': (), 'name': 'file448'})
    option_139789491925616 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491925888 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491926160 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491926432 = UnicodeOption(**{'default': u'courier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491926704 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/courier/authldaprc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491926976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491927248 = UnicodeOption(**{'default': u'0600', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789491927520 = UnicodeOption(**{'default': u'/etc/courier/authldaprc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491927792 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491928064 = UnicodeOption(**{'default': u'/var/lib/creole/authldaprc', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491928336 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496747424}]})
    option_139789491913744 = OptionDescription(children=[option_139789491925616, option_139789491925888, option_139789491926160, option_139789491926432, option_139789491926704, option_139789491926976, option_139789491927248, option_139789491927520, option_139789491927792, option_139789491928064, option_139789491928336], **{'doc': u'', 'properties': (), 'name': 'file449'})
    option_139789491928608 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491937360 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491937632 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491937904 = UnicodeOption(**{'default': u'courier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491938176 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/courier/imapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491938448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491938720 = UnicodeOption(**{'default': u'/etc/courier/imapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491938992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491939264 = UnicodeOption(**{'default': u'/var/lib/creole/imapd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491939536 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496747424}]})
    option_139789491914064 = OptionDescription(children=[option_139789491928608, option_139789491937360, option_139789491937632, option_139789491937904, option_139789491938176, option_139789491938448, option_139789491938720, option_139789491938992, option_139789491939264, option_139789491939536], **{'doc': u'', 'properties': (), 'name': 'file450'})
    option_139789491939808 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491940080 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491940352 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491940624 = UnicodeOption(**{'default': u'courier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491940896 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/courier/imapd-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491945552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491945824 = UnicodeOption(**{'default': u'/etc/courier/imapd-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491946096 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491946368 = UnicodeOption(**{'default': u'/var/lib/creole/imapd-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491946640 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496747424}]})
    option_139789491914384 = OptionDescription(children=[option_139789491939808, option_139789491940080, option_139789491940352, option_139789491940624, option_139789491940896, option_139789491945552, option_139789491945824, option_139789491946096, option_139789491946368, option_139789491946640], **{'doc': u'', 'properties': (), 'name': 'file451'})
    option_139789491946912 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491947184 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491947456 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491947728 = UnicodeOption(**{'default': u'courier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491948000 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/courier/pop3d-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491948272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491948544 = UnicodeOption(**{'default': u'/etc/courier/pop3d-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491948816 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491949088 = UnicodeOption(**{'default': u'/var/lib/creole/pop3d-ssl', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491953744 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789496747424}]})
    option_139789491914704 = OptionDescription(children=[option_139789491946912, option_139789491947184, option_139789491947456, option_139789491947728, option_139789491948000, option_139789491948272, option_139789491948544, option_139789491948816, option_139789491949088, option_139789491953744], **{'doc': u'', 'properties': (), 'name': 'file452'})
    option_139789491954016 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491954288 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491954560 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491954832 = UnicodeOption(**{'default': u'couriersso', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491955104 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/courier/authprog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491955376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491955648 = UnicodeOption(**{'default': u'/etc/courier/authprog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491955920 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491956192 = UnicodeOption(**{'default': u'/var/lib/creole/authprog.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491956464 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496747696}]})
    option_139789491915024 = OptionDescription(children=[option_139789491954016, option_139789491954288, option_139789491954560, option_139789491954832, option_139789491955104, option_139789491955376, option_139789491955648, option_139789491955920, option_139789491956192, option_139789491956464], **{'doc': u'', 'properties': (), 'name': 'file453'})
    option_139789491956736 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491957008 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491957280 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491966032 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/sites-available/phpmyadmin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491966304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491966576 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/phpmyadmin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491966848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491967120 = UnicodeOption(**{'default': u'/var/lib/creole/apache-phpmyadmin.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491967392 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491915344 = OptionDescription(children=[option_139789491956736, option_139789491957008, option_139789491957280, option_139789491966032, option_139789491966304, option_139789491966576, option_139789491966848, option_139789491967120, option_139789491967392], **{'doc': u'', 'properties': (), 'name': 'file454'})
    option_139789491967664 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491967936 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491968208 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491968480 = UnicodeOption(**{'default': u'myadmin', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491968752 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/phpmyadmin/config-db.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491969024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491969296 = UnicodeOption(**{'default': u'/etc/phpmyadmin/config-db.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491969568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491974224 = UnicodeOption(**{'default': u'/var/lib/creole/config-db.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491974496 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145552}]})
    option_139789491915664 = OptionDescription(children=[option_139789491967664, option_139789491967936, option_139789491968208, option_139789491968480, option_139789491968752, option_139789491969024, option_139789491969296, option_139789491969568, option_139789491974224, option_139789491974496], **{'doc': u'', 'properties': (), 'name': 'file455'})
    option_139789491974768 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491975040 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491975312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491975584 = UnicodeOption(**{'default': u'/usr/share/ead2/backend/config/perms/perm_proxy.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491975856 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491976128 = UnicodeOption(**{'default': u'/usr/share/ead2/backend/config/perms/perm_proxy.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491976400 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491976672 = UnicodeOption(**{'default': u'/var/lib/creole/perm_proxy.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491976944 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491915984 = OptionDescription(children=[option_139789491974768, option_139789491975040, option_139789491975312, option_139789491975584, option_139789491975856, option_139789491976128, option_139789491976400, option_139789491976672, option_139789491976944], **{'doc': u'', 'properties': (), 'name': 'file456'})
    option_139789491977216 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491977488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491977760 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491990608 = UnicodeOption(**{'default': u'/var/lib/eole/config/amon.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491990880 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491991152 = UnicodeOption(**{'default': u'/var/lib/eole/config/amon.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491991424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491991696 = UnicodeOption(**{'default': u'/var/lib/creole/amon.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491991968 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491916304 = OptionDescription(children=[option_139789491977216, option_139789491977488, option_139789491977760, option_139789491990608, option_139789491990880, option_139789491991152, option_139789491991424, option_139789491991696, option_139789491991968], **{'doc': u'', 'properties': (), 'name': 'file457'})
    option_139789491992240 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491992512 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491992784 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491993056 = UnicodeOption(**{'default': u'/etc/logrotate.d/squid_rsyslog', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491993328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491993600 = UnicodeOption(**{'default': u'/etc/logrotate.d/squid_rsyslog', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491993872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491994144 = UnicodeOption(**{'default': u'/var/lib/creole/squid.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491998800 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492002896 = OptionDescription(children=[option_139789491992240, option_139789491992512, option_139789491992784, option_139789491993056, option_139789491993328, option_139789491993600, option_139789491993872, option_139789491994144, option_139789491998800], **{'doc': u'', 'properties': (), 'name': 'file458'})
    option_139789491999072 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491999344 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491999616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491999888 = UnicodeOption(**{'default': u'/etc/lightsquid/lightsquid.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492000160 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492000432 = UnicodeOption(**{'default': u'/etc/lightsquid/lightsquid.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492000704 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492000976 = UnicodeOption(**{'default': u'/var/lib/creole/lightsquid.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492001248 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789492003216 = OptionDescription(children=[option_139789491999072, option_139789491999344, option_139789491999616, option_139789491999888, option_139789492000160, option_139789492000432, option_139789492000704, option_139789492000976, option_139789492001248], **{'doc': u'', 'properties': (), 'name': 'file459'})
    option_139789492001520 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492001792 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492002064 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492002336 = UnicodeOption(**{'default': u'guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492011088 = UnicodeOption(**{'default': u'/etc/logrotate.d/guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492011360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492011632 = UnicodeOption(**{'default': u'/etc/logrotate.d/guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492011904 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492012176 = UnicodeOption(**{'default': u'/var/lib/creole/guardian.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492012448 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789492003536 = OptionDescription(children=[option_139789492001520, option_139789492001792, option_139789492002064, option_139789492002336, option_139789492011088, option_139789492011360, option_139789492011632, option_139789492011904, option_139789492012176, option_139789492012448], **{'doc': u'', 'properties': (), 'name': 'file460'})
    option_139789492012720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492012992 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492013264 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492013536 = UnicodeOption(**{'default': u'guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492013808 = UnicodeOption(**{'default': u'/usr/share/nginx/html/guardian.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492014080 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492014352 = UnicodeOption(**{'default': u'/usr/share/nginx/html/guardian.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492014624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492019280 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-template-nginx.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492019552 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789492003856 = OptionDescription(children=[option_139789492012720, option_139789492012992, option_139789492013264, option_139789492013536, option_139789492013808, option_139789492014080, option_139789492014352, option_139789492014624, option_139789492019280, option_139789492019552], **{'doc': u'', 'properties': (), 'name': 'file461'})
    option_139789492019824 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492020096 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789492020368 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789492020640 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789492020912 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/guardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789492021184 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789492021456 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/guardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789492021728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789492022000 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789492022272 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789492004176 = OptionDescription(children=[option_139789492019824, option_139789492020096, option_139789492020368, option_139789492020640, option_139789492020912, option_139789492021184, option_139789492021456, option_139789492021728, option_139789492022000, option_139789492022272], **{'doc': u'', 'properties': (), 'name': 'file462'})
    option_139789492022544 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789492022816 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491507280 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491507552 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491507824 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/guardianf1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491508096 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491508368 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/guardianf1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491508640 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491508912 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf1-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491509184 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789492004496 = OptionDescription(children=[option_139789492022544, option_139789492022816, option_139789491507280, option_139789491507552, option_139789491507824, option_139789491508096, option_139789491508368, option_139789491508640, option_139789491508912, option_139789491509184], **{'doc': u'', 'properties': (), 'name': 'file463'})
    option_139789491509456 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491509728 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491510000 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491510272 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491510544 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/guardianf2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491510816 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491519568 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/guardianf2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491519840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491520112 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf2-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491520384 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789492004816 = OptionDescription(children=[option_139789491509456, option_139789491509728, option_139789491510000, option_139789491510272, option_139789491510544, option_139789491510816, option_139789491519568, option_139789491519840, option_139789491520112, option_139789491520384], **{'doc': u'', 'properties': (), 'name': 'file464'})
    option_139789491520656 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491520928 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491521200 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491521472 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491521744 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/guardianf3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491522016 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491522288 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/guardianf3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491522560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491522832 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf3-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491523104 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789492005136 = OptionDescription(children=[option_139789491520656, option_139789491520928, option_139789491521200, option_139789491521472, option_139789491521744, option_139789491522016, option_139789491522288, option_139789491522560, option_139789491522832, option_139789491523104], **{'doc': u'', 'properties': (), 'name': 'file465'})
    option_139789491527760 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491528032 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491528304 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491528576 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491528848 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/guardianf4.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491529120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491529392 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/guardianf4.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491529664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491529936 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf4-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491530208 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789492005456 = OptionDescription(children=[option_139789491527760, option_139789491528032, option_139789491528304, option_139789491528576, option_139789491528848, option_139789491529120, option_139789491529392, option_139789491529664, option_139789491529936, option_139789491530208], **{'doc': u'', 'properties': (), 'name': 'file466'})
    option_139789491530480 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491530752 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491531024 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491531296 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491535952 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/guardianf5.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491536224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491536496 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/guardianf5.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491536768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491537040 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf5-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491537312 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789492005776 = OptionDescription(children=[option_139789491530480, option_139789491530752, option_139789491531024, option_139789491531296, option_139789491535952, option_139789491536224, option_139789491536496, option_139789491536768, option_139789491537040, option_139789491537312], **{'doc': u'', 'properties': (), 'name': 'file467'})
    option_139789491537584 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491537856 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491538128 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491538400 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491538672 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/guardianf6.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491538944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491539216 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/guardianf6.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491539488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491544144 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf6-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491544416 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789492006096 = OptionDescription(children=[option_139789491537584, option_139789491537856, option_139789491538128, option_139789491538400, option_139789491538672, option_139789491538944, option_139789491539216, option_139789491539488, option_139789491544144, option_139789491544416], **{'doc': u'', 'properties': (), 'name': 'file468'})
    option_139789491544688 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491544960 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491545232 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491545504 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491545776 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/guardianf7.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491546048 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491546320 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/guardianf7.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491546592 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491546864 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf7-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491547136 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789492006416 = OptionDescription(children=[option_139789491544688, option_139789491544960, option_139789491545232, option_139789491545504, option_139789491545776, option_139789491546048, option_139789491546320, option_139789491546592, option_139789491546864, option_139789491547136], **{'doc': u'', 'properties': (), 'name': 'file469'})
    option_139789491547408 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491547680 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491552336 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491552608 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491552880 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/guardianf8.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491553152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491553424 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/guardianf8.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491553696 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491553968 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf8-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491554240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789491560528 = OptionDescription(children=[option_139789491547408, option_139789491547680, option_139789491552336, option_139789491552608, option_139789491552880, option_139789491553152, option_139789491553424, option_139789491553696, option_139789491553968, option_139789491554240], **{'doc': u'', 'properties': (), 'name': 'file470'})
    option_139789491554512 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491554784 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491555056 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491555328 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491555600 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/contentscanners/clamdscan.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491555872 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491568720 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/contentscanners/clamdscan.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491568992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491569264 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-clamdscan-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491569536 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789491560848 = OptionDescription(children=[option_139789491554512, option_139789491554784, option_139789491555056, option_139789491555328, option_139789491555600, option_139789491555872, option_139789491568720, option_139789491568992, option_139789491569264, option_139789491569536], **{'doc': u'', 'properties': (), 'name': 'file471'})
    option_139789491569808 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491570080 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491570352 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491570624 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491570896 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/common/bannediplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491571168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491571440 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/common/bannediplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491571712 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491571984 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-bannediplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491572256 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789491561168 = OptionDescription(children=[option_139789491569808, option_139789491570080, option_139789491570352, option_139789491570624, option_139789491570896, option_139789491571168, option_139789491571440, option_139789491571712, option_139789491571984, option_139789491572256], **{'doc': u'', 'properties': (), 'name': 'file472'})
    option_139789491576912 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491577184 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491577456 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491577728 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491578000 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/common/exceptioniplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491578272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491578544 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/common/exceptioniplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491578816 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491579088 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-exceptioniplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491579360 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789491561488 = OptionDescription(children=[option_139789491576912, option_139789491577184, option_139789491577456, option_139789491577728, option_139789491578000, option_139789491578272, option_139789491578544, option_139789491578816, option_139789491579088, option_139789491579360], **{'doc': u'', 'properties': (), 'name': 'file473'})
    option_139789491579632 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491579904 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491580176 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491580448 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491585104 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/common/fancy.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491585376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491585648 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/common/fancy.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491585920 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491586192 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-fancy-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491586464 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789491561808 = OptionDescription(children=[option_139789491579632, option_139789491579904, option_139789491580176, option_139789491580448, option_139789491585104, option_139789491585376, option_139789491585648, option_139789491585920, option_139789491586192, option_139789491586464], **{'doc': u'', 'properties': (), 'name': 'file474'})
    option_139789491586736 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491587008 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491587280 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491587552 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491587824 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/common/filtergroupslist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491588096 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491588368 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/common/filtergroupslist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491588640 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491597392 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-filtergroupslist-filtre1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491597664 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789491562128 = OptionDescription(children=[option_139789491586736, option_139789491587008, option_139789491587280, option_139789491587552, option_139789491587824, option_139789491588096, option_139789491588368, option_139789491588640, option_139789491597392, option_139789491597664], **{'doc': u'', 'properties': (), 'name': 'file475'})
    option_139789491597936 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491598208 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491598480 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491598752 = UnicodeOption(**{'default': u'guardian1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491599024 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian0/common/ip.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491599296 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491599568 = UnicodeOption(**{'default': u'/etc/guardian/guardian0/common/ip.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491599840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491600112 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-ip-filtre1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491600384 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794656}]})
    option_139789491562448 = OptionDescription(children=[option_139789491597936, option_139789491598208, option_139789491598480, option_139789491598752, option_139789491599024, option_139789491599296, option_139789491599568, option_139789491599840, option_139789491600112, option_139789491600384], **{'doc': u'', 'properties': (), 'name': 'file476'})
    option_139789491600656 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491600928 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491613776 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491614048 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491614320 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/guardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491614592 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491614864 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/guardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491615136 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491615408 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491615680 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491562768 = OptionDescription(children=[option_139789491600656, option_139789491600928, option_139789491613776, option_139789491614048, option_139789491614320, option_139789491614592, option_139789491614864, option_139789491615136, option_139789491615408, option_139789491615680], **{'doc': u'', 'properties': (), 'name': 'file477'})
    option_139789491615952 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491616224 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491616496 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491616768 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491617040 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/guardianf1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491617312 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491621968 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/guardianf1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491622240 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491622512 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf1-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491622784 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491563088 = OptionDescription(children=[option_139789491615952, option_139789491616224, option_139789491616496, option_139789491616768, option_139789491617040, option_139789491617312, option_139789491621968, option_139789491622240, option_139789491622512, option_139789491622784], **{'doc': u'', 'properties': (), 'name': 'file478'})
    option_139789491623056 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491623328 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491623600 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491623872 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491624144 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/guardianf2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491624416 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491624688 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/guardianf2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491624960 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491625232 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf2-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491625504 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491563408 = OptionDescription(children=[option_139789491623056, option_139789491623328, option_139789491623600, option_139789491623872, option_139789491624144, option_139789491624416, option_139789491624688, option_139789491624960, option_139789491625232, option_139789491625504], **{'doc': u'', 'properties': (), 'name': 'file479'})
    option_139789491630160 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491630432 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491630704 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491630976 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491631248 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/guardianf3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491631520 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491631792 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/guardianf3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491632064 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491632336 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf3-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491632608 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491563728 = OptionDescription(children=[option_139789491630160, option_139789491630432, option_139789491630704, option_139789491630976, option_139789491631248, option_139789491631520, option_139789491631792, option_139789491632064, option_139789491632336, option_139789491632608], **{'doc': u'', 'properties': (), 'name': 'file480'})
    option_139789491632880 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491633152 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491633424 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491633696 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491638352 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/guardianf4.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491638624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491638896 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/guardianf4.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491639168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491639440 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf4-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491639712 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491564048 = OptionDescription(children=[option_139789491632880, option_139789491633152, option_139789491633424, option_139789491633696, option_139789491638352, option_139789491638624, option_139789491638896, option_139789491639168, option_139789491639440, option_139789491639712], **{'doc': u'', 'properties': (), 'name': 'file481'})
    option_139789491639984 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491640256 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491640528 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491640800 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491641072 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/guardianf5.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491641344 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491641616 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/guardianf5.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491641888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491646544 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf5-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491646816 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491658832 = OptionDescription(children=[option_139789491639984, option_139789491640256, option_139789491640528, option_139789491640800, option_139789491641072, option_139789491641344, option_139789491641616, option_139789491641888, option_139789491646544, option_139789491646816], **{'doc': u'', 'properties': (), 'name': 'file482'})
    option_139789491647088 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491647360 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491647632 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491647904 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491648176 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/guardianf6.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491648448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491648720 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/guardianf6.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491648992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491649264 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf6-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491649536 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491659152 = OptionDescription(children=[option_139789491647088, option_139789491647360, option_139789491647632, option_139789491647904, option_139789491648176, option_139789491648448, option_139789491648720, option_139789491648992, option_139789491649264, option_139789491649536], **{'doc': u'', 'properties': (), 'name': 'file483'})
    option_139789491649808 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491650080 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491662928 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491663200 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491663472 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/guardianf7.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491663744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491664016 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/guardianf7.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491664288 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491664560 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf7-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491664832 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491659472 = OptionDescription(children=[option_139789491649808, option_139789491650080, option_139789491662928, option_139789491663200, option_139789491663472, option_139789491663744, option_139789491664016, option_139789491664288, option_139789491664560, option_139789491664832], **{'doc': u'', 'properties': (), 'name': 'file484'})
    option_139789491665104 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491665376 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491665648 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491665920 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491666192 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/guardianf8.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491666464 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491675216 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/guardianf8.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491675488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491675760 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf8-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491676032 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491659792 = OptionDescription(children=[option_139789491665104, option_139789491665376, option_139789491665648, option_139789491665920, option_139789491666192, option_139789491666464, option_139789491675216, option_139789491675488, option_139789491675760, option_139789491676032], **{'doc': u'', 'properties': (), 'name': 'file485'})
    option_139789491676304 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491676576 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491676848 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491677120 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491677392 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/contentscanners/clamdscan.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491677664 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491677936 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/contentscanners/clamdscan.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491678208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491678480 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-clamdscan-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491678752 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491660112 = OptionDescription(children=[option_139789491676304, option_139789491676576, option_139789491676848, option_139789491677120, option_139789491677392, option_139789491677664, option_139789491677936, option_139789491678208, option_139789491678480, option_139789491678752], **{'doc': u'', 'properties': (), 'name': 'file486'})
    option_139789491683408 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491683680 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491683952 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491684224 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491684496 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/common/bannediplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491684768 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491685040 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/common/bannediplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491685312 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491685584 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-bannediplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491685856 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491660432 = OptionDescription(children=[option_139789491683408, option_139789491683680, option_139789491683952, option_139789491684224, option_139789491684496, option_139789491684768, option_139789491685040, option_139789491685312, option_139789491685584, option_139789491685856], **{'doc': u'', 'properties': (), 'name': 'file487'})
    option_139789491686128 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491686400 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491686672 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491686944 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491695696 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/common/exceptioniplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491695968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491696240 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/common/exceptioniplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491696512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491696784 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-exceptioniplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491697056 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491660752 = OptionDescription(children=[option_139789491686128, option_139789491686400, option_139789491686672, option_139789491686944, option_139789491695696, option_139789491695968, option_139789491696240, option_139789491696512, option_139789491696784, option_139789491697056], **{'doc': u'', 'properties': (), 'name': 'file488'})
    option_139789491697328 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491697600 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491697872 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491698144 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491698416 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/common/fancy.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491698688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491698960 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/common/fancy.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491699232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491703888 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-fancy-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491704160 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491661072 = OptionDescription(children=[option_139789491697328, option_139789491697600, option_139789491697872, option_139789491698144, option_139789491698416, option_139789491698688, option_139789491698960, option_139789491699232, option_139789491703888, option_139789491704160], **{'doc': u'', 'properties': (), 'name': 'file489'})
    option_139789491704432 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491704704 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491704976 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491705248 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491705520 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/common/filtergroupslist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491705792 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491706064 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/common/filtergroupslist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491706336 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491706608 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-filtergroupslist-filtre2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491706880 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491661392 = OptionDescription(children=[option_139789491704432, option_139789491704704, option_139789491704976, option_139789491705248, option_139789491705520, option_139789491705792, option_139789491706064, option_139789491706336, option_139789491706608, option_139789491706880], **{'doc': u'', 'properties': (), 'name': 'file490'})
    option_139789491707152 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491707424 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491716176 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491716448 = UnicodeOption(**{'default': u'guardian2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491716720 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian1/common/ip.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491716992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491717264 = UnicodeOption(**{'default': u'/etc/guardian/guardian1/common/ip.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491717536 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491717808 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-ip-filtre2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491718080 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811600}]})
    option_139789491661712 = OptionDescription(children=[option_139789491707152, option_139789491707424, option_139789491716176, option_139789491716448, option_139789491716720, option_139789491716992, option_139789491717264, option_139789491717536, option_139789491717808, option_139789491718080], **{'doc': u'', 'properties': (), 'name': 'file491'})
    option_139789491718352 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491718624 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491718896 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491719168 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491719440 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/guardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491719712 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491724368 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/guardian.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491724640 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491724912 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491725184 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491662032 = OptionDescription(children=[option_139789491718352, option_139789491718624, option_139789491718896, option_139789491719168, option_139789491719440, option_139789491719712, option_139789491724368, option_139789491724640, option_139789491724912, option_139789491725184], **{'doc': u'', 'properties': (), 'name': 'file492'})
    option_139789491725456 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491725728 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491726000 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491726272 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491726544 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/guardianf1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491726816 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491727088 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/guardianf1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491727360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491727632 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf1-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491727904 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491662352 = OptionDescription(children=[option_139789491725456, option_139789491725728, option_139789491726000, option_139789491726272, option_139789491726544, option_139789491726816, option_139789491727088, option_139789491727360, option_139789491727632, option_139789491727904], **{'doc': u'', 'properties': (), 'name': 'file493'})
    option_139789491736656 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491736928 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491737200 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491737472 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491737744 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/guardianf2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491738016 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491738288 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/guardianf2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491738560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491738832 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf2-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491739104 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491744848 = OptionDescription(children=[option_139789491736656, option_139789491736928, option_139789491737200, option_139789491737472, option_139789491737744, option_139789491738016, option_139789491738288, option_139789491738560, option_139789491738832, option_139789491739104], **{'doc': u'', 'properties': (), 'name': 'file494'})
    option_139789491739376 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491739648 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491739920 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491740192 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491748944 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/guardianf3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491749216 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491749488 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/guardianf3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491749760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491750032 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf3-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491750304 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491745168 = OptionDescription(children=[option_139789491739376, option_139789491739648, option_139789491739920, option_139789491740192, option_139789491748944, option_139789491749216, option_139789491749488, option_139789491749760, option_139789491750032, option_139789491750304], **{'doc': u'', 'properties': (), 'name': 'file495'})
    option_139789491750576 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491750848 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491751120 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491751392 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491751664 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/guardianf4.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491751936 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491752208 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/guardianf4.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491752480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491236944 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf4-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491237216 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491745488 = OptionDescription(children=[option_139789491750576, option_139789491750848, option_139789491751120, option_139789491751392, option_139789491751664, option_139789491751936, option_139789491752208, option_139789491752480, option_139789491236944, option_139789491237216], **{'doc': u'', 'properties': (), 'name': 'file496'})
    option_139789491237488 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491237760 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491238032 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491238304 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491238576 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/guardianf5.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491238848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491239120 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/guardianf5.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491239392 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491239664 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf5-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491239936 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491745808 = OptionDescription(children=[option_139789491237488, option_139789491237760, option_139789491238032, option_139789491238304, option_139789491238576, option_139789491238848, option_139789491239120, option_139789491239392, option_139789491239664, option_139789491239936], **{'doc': u'', 'properties': (), 'name': 'file497'})
    option_139789491240208 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491240480 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491245136 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491245408 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491245680 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/guardianf6.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491245952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491246224 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/guardianf6.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491246496 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491246768 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf6-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491247040 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491746128 = OptionDescription(children=[option_139789491240208, option_139789491240480, option_139789491245136, option_139789491245408, option_139789491245680, option_139789491245952, option_139789491246224, option_139789491246496, option_139789491246768, option_139789491247040], **{'doc': u'', 'properties': (), 'name': 'file498'})
    option_139789491247312 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491247584 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491247856 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491248128 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491248400 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/guardianf7.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491248672 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491257424 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/guardianf7.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491257696 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491257968 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf7-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491258240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491746448 = OptionDescription(children=[option_139789491247312, option_139789491247584, option_139789491247856, option_139789491248128, option_139789491248400, option_139789491248672, option_139789491257424, option_139789491257696, option_139789491257968, option_139789491258240], **{'doc': u'', 'properties': (), 'name': 'file499'})
    option_139789491258512 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491258784 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491259056 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491259328 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491259600 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/guardianf8.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491259872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491260144 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/guardianf8.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491260416 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491260688 = UnicodeOption(**{'default': u'/var/lib/creole/guardianf8-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491260960 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491746768 = OptionDescription(children=[option_139789491258512, option_139789491258784, option_139789491259056, option_139789491259328, option_139789491259600, option_139789491259872, option_139789491260144, option_139789491260416, option_139789491260688, option_139789491260960], **{'doc': u'', 'properties': (), 'name': 'file500'})
    option_139789491265616 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491265888 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491266160 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491266432 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491266704 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/contentscanners/clamdscan.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491266976 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491267248 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/contentscanners/clamdscan.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491267520 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491267792 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-clamdscan-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491268064 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491747088 = OptionDescription(children=[option_139789491265616, option_139789491265888, option_139789491266160, option_139789491266432, option_139789491266704, option_139789491266976, option_139789491267248, option_139789491267520, option_139789491267792, option_139789491268064], **{'doc': u'', 'properties': (), 'name': 'file501'})
    option_139789491268336 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491268608 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491268880 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491269152 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491277904 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/common/bannediplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491278176 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491278448 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/common/bannediplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491278720 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491278992 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-bannediplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491279264 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491747408 = OptionDescription(children=[option_139789491268336, option_139789491268608, option_139789491268880, option_139789491269152, option_139789491277904, option_139789491278176, option_139789491278448, option_139789491278720, option_139789491278992, option_139789491279264], **{'doc': u'', 'properties': (), 'name': 'file502'})
    option_139789491279536 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491279808 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491280080 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491280352 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491280624 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/common/exceptioniplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491280896 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491281168 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/common/exceptioniplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491281440 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491286096 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-exceptioniplist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491286368 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491747728 = OptionDescription(children=[option_139789491279536, option_139789491279808, option_139789491280080, option_139789491280352, option_139789491280624, option_139789491280896, option_139789491281168, option_139789491281440, option_139789491286096, option_139789491286368], **{'doc': u'', 'properties': (), 'name': 'file503'})
    option_139789491286640 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491286912 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491287184 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491287456 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491287728 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/common/fancy.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491288000 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491288272 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/common/fancy.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491288544 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491288816 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-fancy-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491289088 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491748048 = OptionDescription(children=[option_139789491286640, option_139789491286912, option_139789491287184, option_139789491287456, option_139789491287728, option_139789491288000, option_139789491288272, option_139789491288544, option_139789491288816, option_139789491289088], **{'doc': u'', 'properties': (), 'name': 'file504'})
    option_139789491289360 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491289632 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491294288 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491294560 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491294832 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/common/filtergroupslist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491295104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491295376 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/common/filtergroupslist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491295648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491295920 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-filtergroupslist-filtre3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491296192 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491748368 = OptionDescription(children=[option_139789491289360, option_139789491289632, option_139789491294288, option_139789491294560, option_139789491294832, option_139789491295104, option_139789491295376, option_139789491295648, option_139789491295920, option_139789491296192], **{'doc': u'', 'properties': (), 'name': 'file505'})
    option_139789491296464 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491296736 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491297008 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491297280 = UnicodeOption(**{'default': u'guardian3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491297552 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/guardian/guardian2/common/ip.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491297824 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491306576 = UnicodeOption(**{'default': u'/etc/guardian/guardian2/common/ip.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491306848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491307120 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-ip-filtre3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491307392 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496811872}]})
    option_139789491310672 = OptionDescription(children=[option_139789491296464, option_139789491296736, option_139789491297008, option_139789491297280, option_139789491297552, option_139789491297824, option_139789491306576, option_139789491306848, option_139789491307120, option_139789491307392], **{'doc': u'', 'properties': (), 'name': 'file506'})
    option_139789491307664 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491307936 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491308208 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491308480 = UnicodeOption(**{'default': u'guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491308752 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/e2guardian/languages/french/template.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491309024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491309296 = UnicodeOption(**{'default': u'/etc/e2guardian/languages/french/template.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491309568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491309840 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-template.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491310112 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789491310992 = OptionDescription(children=[option_139789491307664, option_139789491307936, option_139789491308208, option_139789491308480, option_139789491308752, option_139789491309024, option_139789491309296, option_139789491309568, option_139789491309840, option_139789491310112], **{'doc': u'', 'properties': (), 'name': 'file507'})
    option_139789491318864 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491319136 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491319408 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491319680 = UnicodeOption(**{'default': u'guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491319952 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/e2guardian/languages/french/template-liste_blanche.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491320224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491320496 = UnicodeOption(**{'default': u'/etc/e2guardian/languages/french/template-liste_blanche.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491320768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491321040 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-template-liste_blanche.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491321312 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789491311312 = OptionDescription(children=[option_139789491318864, option_139789491319136, option_139789491319408, option_139789491319680, option_139789491319952, option_139789491320224, option_139789491320496, option_139789491320768, option_139789491321040, option_139789491321312], **{'doc': u'', 'properties': (), 'name': 'file508'})
    option_139789491321584 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491321856 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491322128 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491322400 = UnicodeOption(**{'default': u'guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491327056 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/e2guardian/languages/french/template-moderateur.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491327328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491327600 = UnicodeOption(**{'default': u'/etc/e2guardian/languages/french/template-moderateur.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491327872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491328144 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-template-moderateur.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491328416 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789491311632 = OptionDescription(children=[option_139789491321584, option_139789491321856, option_139789491322128, option_139789491322400, option_139789491327056, option_139789491327328, option_139789491327600, option_139789491327872, option_139789491328144, option_139789491328416], **{'doc': u'', 'properties': (), 'name': 'file509'})
    option_139789491328688 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491328960 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491329232 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491329504 = UnicodeOption(**{'default': u'guardian', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491329776 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/e2guardian/languages/french/template-utilisateur_interdits.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491330048 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491330320 = UnicodeOption(**{'default': u'/etc/e2guardian/languages/french/template-utilisateur_interdits.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491330592 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491335248 = UnicodeOption(**{'default': u'/var/lib/creole/guardian-template-utilisateur_interdits.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491335520 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497941264}]})
    option_139789491311952 = OptionDescription(children=[option_139789491328688, option_139789491328960, option_139789491329232, option_139789491329504, option_139789491329776, option_139789491330048, option_139789491330320, option_139789491330592, option_139789491335248, option_139789491335520], **{'doc': u'', 'properties': (), 'name': 'file510'})
    option_139789491335792 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491336064 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491336336 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491336608 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/logrotate.d/winbind', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491336880 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491337152 = UnicodeOption(**{'default': u'/etc/logrotate.d/winbind', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491337424 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491337696 = UnicodeOption(**{'default': u'/var/lib/creole/winbind.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491337968 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491312272 = OptionDescription(children=[option_139789491335792, option_139789491336064, option_139789491336336, option_139789491336608, option_139789491336880, option_139789491337152, option_139789491337424, option_139789491337696, option_139789491337968], **{'doc': u'', 'properties': (), 'name': 'file511'})
    option_139789491338240 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491338512 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491338784 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491347536 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/pam.d/squid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491347808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491348080 = UnicodeOption(**{'default': u'/etc/pam.d/squid', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491348352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491348624 = UnicodeOption(**{'default': u'/var/lib/creole/squid.pam', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491348896 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491312592 = OptionDescription(children=[option_139789491338240, option_139789491338512, option_139789491338784, option_139789491347536, option_139789491347808, option_139789491348080, option_139789491348352, option_139789491348624, option_139789491348896], **{'doc': u'', 'properties': (), 'name': 'file512'})
    option_139789491349168 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491349440 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491349712 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491349984 = UnicodeOption(**{'default': u'#', 'doc': u'', 'multi': False, 'properties': (), 'name': 'del_comment'})
    option_139789491350256 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/squid/common-squid1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491350528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491350800 = UnicodeOption(**{'default': u'/etc/squid/common-squid1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491351072 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491359824 = UnicodeOption(**{'default': u'/var/lib/creole/common-squid1.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491360096 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491312912 = OptionDescription(children=[option_139789491349168, option_139789491349440, option_139789491349712, option_139789491349984, option_139789491350256, option_139789491350528, option_139789491350800, option_139789491351072, option_139789491359824, option_139789491360096], **{'doc': u'', 'properties': (), 'name': 'file513'})
    option_139789491360368 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491360640 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491360912 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491361184 = UnicodeOption(**{'default': u'#', 'doc': u'', 'multi': False, 'properties': (), 'name': 'del_comment'})
    option_139789491361456 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/squid/common-squid2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491361728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491362000 = UnicodeOption(**{'default': u'/etc/squid/common-squid2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491362272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491362544 = UnicodeOption(**{'default': u'/var/lib/creole/common-squid2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491362816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491313232 = OptionDescription(children=[option_139789491360368, option_139789491360640, option_139789491360912, option_139789491361184, option_139789491361456, option_139789491361728, option_139789491362000, option_139789491362272, option_139789491362544, option_139789491362816], **{'doc': u'', 'properties': (), 'name': 'file514'})
    option_139789491363088 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491363360 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491368016 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491368288 = UnicodeOption(**{'default': u'#', 'doc': u'', 'multi': False, 'properties': (), 'name': 'del_comment'})
    option_139789491368560 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/squid/squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491368832 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491369104 = UnicodeOption(**{'default': u'/etc/squid/squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491369376 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491369648 = UnicodeOption(**{'default': u'/var/lib/creole/01squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491369920 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491313552 = OptionDescription(children=[option_139789491363088, option_139789491363360, option_139789491368016, option_139789491368288, option_139789491368560, option_139789491368832, option_139789491369104, option_139789491369376, option_139789491369648, option_139789491369920], **{'doc': u'', 'properties': (), 'name': 'file515'})
    option_139789491370192 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491370464 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491370736 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491371008 = UnicodeOption(**{'default': u'#', 'doc': u'', 'multi': False, 'properties': (), 'name': 'del_comment'})
    option_139789491371280 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/squid/01inc-squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491371552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491376208 = UnicodeOption(**{'default': u'/etc/squid/01inc-squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491376480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491376752 = UnicodeOption(**{'default': u'/var/lib/creole/01inc-squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491377024 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491313872 = OptionDescription(children=[option_139789491370192, option_139789491370464, option_139789491370736, option_139789491371008, option_139789491371280, option_139789491371552, option_139789491376208, option_139789491376480, option_139789491376752, option_139789491377024], **{'doc': u'', 'properties': (), 'name': 'file516'})
    option_139789491377296 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491377568 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491377840 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491378112 = UnicodeOption(**{'default': u'#', 'doc': u'', 'multi': False, 'properties': (), 'name': 'del_comment'})
    option_139789491378384 = UnicodeOption(**{'default': u'squid2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491378656 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/squid/squid2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491378928 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491379200 = UnicodeOption(**{'default': u'/etc/squid/squid2.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491379472 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491379744 = UnicodeOption(**{'default': u'/var/lib/creole/02squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491384400 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}]})
    option_139789491314192 = OptionDescription(children=[option_139789491377296, option_139789491377568, option_139789491377840, option_139789491378112, option_139789491378384, option_139789491378656, option_139789491378928, option_139789491379200, option_139789491379472, option_139789491379744, option_139789491384400], **{'doc': u'', 'properties': (), 'name': 'file517'})
    option_139789491384672 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491384944 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491385216 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491385488 = UnicodeOption(**{'default': u'#', 'doc': u'', 'multi': False, 'properties': (), 'name': 'del_comment'})
    option_139789491385760 = UnicodeOption(**{'default': u'squid2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491386032 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/squid/02inc-squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491386304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491386576 = UnicodeOption(**{'default': u'/etc/squid/02inc-squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491386848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491387120 = UnicodeOption(**{'default': u'/var/lib/creole/02inc-squid.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491387392 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496793840}]})
    option_139789491392592 = OptionDescription(children=[option_139789491384672, option_139789491384944, option_139789491385216, option_139789491385488, option_139789491385760, option_139789491386032, option_139789491386304, option_139789491386576, option_139789491386848, option_139789491387120, option_139789491387392], **{'doc': u'', 'properties': (), 'name': 'file518'})
    option_139789491387664 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491387936 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491396688 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491396960 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/squid/domaines_nocache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491397232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491397504 = UnicodeOption(**{'default': u'/etc/squid/domaines_nocache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491397776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491398048 = UnicodeOption(**{'default': u'/var/lib/creole/domaines_nocache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491398320 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491392912 = OptionDescription(children=[option_139789491387664, option_139789491387936, option_139789491396688, option_139789491396960, option_139789491397232, option_139789491397504, option_139789491397776, option_139789491398048, option_139789491398320], **{'doc': u'', 'properties': (), 'name': 'file519'})
    option_139789491398592 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491398864 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491399136 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491399408 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/squid/domaines_noauth', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491399680 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491399952 = UnicodeOption(**{'default': u'/etc/squid/domaines_noauth', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491400224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491404880 = UnicodeOption(**{'default': u'/var/lib/creole/domaines_noauth', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491405152 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491393232 = OptionDescription(children=[option_139789491398592, option_139789491398864, option_139789491399136, option_139789491399408, option_139789491399680, option_139789491399952, option_139789491400224, option_139789491404880, option_139789491405152], **{'doc': u'', 'properties': (), 'name': 'file520'})
    option_139789491405424 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491405696 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491405968 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491406240 = UnicodeOption(**{'default': u'kerberos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491406512 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/samba/lmhosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491406784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491407056 = UnicodeOption(**{'default': u'/etc/samba/lmhosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491407328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491407600 = UnicodeOption(**{'default': u'/var/lib/creole/lmhosts', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491407872 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789491393552 = OptionDescription(children=[option_139789491405424, option_139789491405696, option_139789491405968, option_139789491406240, option_139789491406512, option_139789491406784, option_139789491407056, option_139789491407328, option_139789491407600, option_139789491407872], **{'doc': u'', 'properties': (), 'name': 'file521'})
    option_139789491408144 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491408416 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491417168 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491417440 = UnicodeOption(**{'default': u'kerberos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491417712 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/krb5.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491417984 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491418256 = UnicodeOption(**{'default': u'/etc/krb5.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491418528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491418800 = UnicodeOption(**{'default': u'/var/lib/creole/krb5.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491419072 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'NTLM/KERBEROS', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789491393872 = OptionDescription(children=[option_139789491408144, option_139789491408416, option_139789491417168, option_139789491417440, option_139789491417712, option_139789491417984, option_139789491418256, option_139789491418528, option_139789491418800, option_139789491419072], **{'doc': u'', 'properties': (), 'name': 'file522'})
    option_139789491419344 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491419616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491419888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491420160 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491420432 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/cups.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491420704 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491429456 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/cups.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491429728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491430000 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-cups.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491430272 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789491394192 = OptionDescription(children=[option_139789491419344, option_139789491419616, option_139789491419888, option_139789491420160, option_139789491420432, option_139789491420704, option_139789491429456, option_139789491429728, option_139789491430000, option_139789491430272], **{'doc': u'', 'properties': (), 'name': 'file523'})
    option_139789491430544 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491430816 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491431088 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491431360 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491431632 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/cups/cupsd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491431904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491432176 = UnicodeOption(**{'default': u'/etc/cups/cupsd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491432448 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491432720 = UnicodeOption(**{'default': u'/var/lib/creole/cupsd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491432992 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789491394512 = OptionDescription(children=[option_139789491430544, option_139789491430816, option_139789491431088, option_139789491431360, option_139789491431632, option_139789491431904, option_139789491432176, option_139789491432448, option_139789491432720, option_139789491432992], **{'doc': u'', 'properties': (), 'name': 'file524'})
    option_139789491437648 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491437920 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491438192 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491438464 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491438736 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/cups/cups-browsed.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491439008 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491439280 = UnicodeOption(**{'default': u'/etc/cups/cups-browsed.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491439552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491439824 = UnicodeOption(**{'default': u'/var/lib/creole/cups-browsed.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491440096 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789491394832 = OptionDescription(children=[option_139789491437648, option_139789491437920, option_139789491438192, option_139789491438464, option_139789491438736, option_139789491439008, option_139789491439280, option_139789491439552, option_139789491439824, option_139789491440096], **{'doc': u'', 'properties': (), 'name': 'file525'})
    option_139789491440368 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491440640 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491440912 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491441184 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491454032 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/cups/cups-files.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491454304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491454576 = UnicodeOption(**{'default': u'/etc/cups/cups-files.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491454848 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491455120 = UnicodeOption(**{'default': u'/var/lib/creole/cups-files.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491455392 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789491395152 = OptionDescription(children=[option_139789491440368, option_139789491440640, option_139789491440912, option_139789491441184, option_139789491454032, option_139789491454304, option_139789491454576, option_139789491454848, option_139789491455120, option_139789491455392], **{'doc': u'', 'properties': (), 'name': 'file526'})
    option_139789491455664 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491455936 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491456208 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491456480 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491456752 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/cups/cups-pdf.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491457024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491457296 = UnicodeOption(**{'default': u'/etc/cups/cups-pdf.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491457568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491466320 = UnicodeOption(**{'default': u'/var/lib/creole/cups-pdf.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491466592 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789491395472 = OptionDescription(children=[option_139789491455664, option_139789491455936, option_139789491456208, option_139789491456480, option_139789491456752, option_139789491457024, option_139789491457296, option_139789491457568, option_139789491466320, option_139789491466592], **{'doc': u'', 'properties': (), 'name': 'file527'})
    option_139789491466864 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491467136 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491467408 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491467680 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491467952 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/cups/client.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491468224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491468496 = UnicodeOption(**{'default': u'/etc/cups/client.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491468768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491469040 = UnicodeOption(**{'default': u'/var/lib/creole/cups-client.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491469312 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789491395792 = OptionDescription(children=[option_139789491466864, option_139789491467136, option_139789491467408, option_139789491467680, option_139789491467952, option_139789491468224, option_139789491468496, option_139789491468768, option_139789491469040, option_139789491469312], **{'doc': u'', 'properties': (), 'name': 'file528'})
    option_139789491469584 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491469856 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491474512 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491474784 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491475056 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/pam.d/cupsys', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491475328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491475600 = UnicodeOption(**{'default': u'/etc/pam.d/cupsys', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491475872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491476144 = UnicodeOption(**{'default': u'/var/lib/creole/cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491476416 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789491396112 = OptionDescription(children=[option_139789491469584, option_139789491469856, option_139789491474512, option_139789491474784, option_139789491475056, option_139789491475328, option_139789491475600, option_139789491475872, option_139789491476144, option_139789491476416], **{'doc': u'', 'properties': (), 'name': 'file529'})
    option_139789491476688 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491476960 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491477232 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491477504 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491477776 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/default/cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491478048 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491482704 = UnicodeOption(**{'default': u'/etc/default/cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491482976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491483248 = UnicodeOption(**{'default': u'/var/lib/creole/cups.default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491483520 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789491486800 = OptionDescription(children=[option_139789491476688, option_139789491476960, option_139789491477232, option_139789491477504, option_139789491477776, option_139789491478048, option_139789491482704, option_139789491482976, option_139789491483248, option_139789491483520], **{'doc': u'', 'properties': (), 'name': 'file530'})
    option_139789491483792 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491484064 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491484336 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491484608 = UnicodeOption(**{'default': u'cups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491484880 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/apparmor.d/local/usr.sbin.cupsd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491485152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491485424 = UnicodeOption(**{'default': u'/etc/apparmor.d/local/usr.sbin.cupsd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491485696 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491485968 = UnicodeOption(**{'default': u'/var/lib/creole/usr.sbin.cupsd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491486240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971856}]})
    option_139789491487120 = OptionDescription(children=[option_139789491483792, option_139789491484064, option_139789491484336, option_139789491484608, option_139789491484880, option_139789491485152, option_139789491485424, option_139789491485696, option_139789491485968, option_139789491486240], **{'doc': u'', 'properties': (), 'name': 'file531'})
    option_139789491494992 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491495264 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491495536 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491495808 = UnicodeOption(**{'default': u'wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491496080 = UnicodeOption(**{'default': u'/etc/nginx/sites-enabled/wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491496352 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491496624 = UnicodeOption(**{'default': u'/etc/nginx/sites-enabled/wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491496896 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491497168 = UnicodeOption(**{'default': u'/var/lib/creole/nginx.wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491497440 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789491487440 = OptionDescription(children=[option_139789491494992, option_139789491495264, option_139789491495536, option_139789491495808, option_139789491496080, option_139789491496352, option_139789491496624, option_139789491496896, option_139789491497168, option_139789491497440], **{'doc': u'', 'properties': (), 'name': 'file532'})
    option_139789491497712 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491497984 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491498256 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491498528 = UnicodeOption(**{'default': u'wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490978896 = UnicodeOption(**{'default': u'/var/www/wpad.eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490979168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490979440 = UnicodeOption(**{'default': u'644', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789490979712 = UnicodeOption(**{'default': u'/var/www/wpad.eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490979984 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490980256 = UnicodeOption(**{'default': u'/var/lib/creole/wpad.eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490980528 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789491487760 = OptionDescription(children=[option_139789491497712, option_139789491497984, option_139789491498256, option_139789491498528, option_139789490978896, option_139789490979168, option_139789490979440, option_139789490979712, option_139789490979984, option_139789490980256, option_139789490980528], **{'doc': u'', 'properties': (), 'name': 'file533'})
    option_139789490980800 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490981072 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490981344 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490981616 = UnicodeOption(**{'default': u'wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490981888 = UnicodeOption(**{'default': u'/var/www/wpad.eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490982160 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490982432 = UnicodeOption(**{'default': u'644', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789490987088 = UnicodeOption(**{'default': u'/var/www/wpad.eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490987360 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490987632 = UnicodeOption(**{'default': u'/var/lib/creole/wpad.eth1', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490987904 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789491488080 = OptionDescription(children=[option_139789490980800, option_139789490981072, option_139789490981344, option_139789490981616, option_139789490981888, option_139789490982160, option_139789490982432, option_139789490987088, option_139789490987360, option_139789490987632, option_139789490987904], **{'doc': u'', 'properties': (), 'name': 'file534'})
    option_139789490988176 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490988448 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490988720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490988992 = UnicodeOption(**{'default': u'wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490989264 = UnicodeOption(**{'default': u'/var/www/wpad.eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490989536 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490989808 = UnicodeOption(**{'default': u'644', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789490990080 = UnicodeOption(**{'default': u'/var/www/wpad.eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490990352 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490990624 = UnicodeOption(**{'default': u'/var/lib/creole/wpad.eth2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490995280 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789491488400 = OptionDescription(children=[option_139789490988176, option_139789490988448, option_139789490988720, option_139789490988992, option_139789490989264, option_139789490989536, option_139789490989808, option_139789490990080, option_139789490990352, option_139789490990624, option_139789490995280], **{'doc': u'', 'properties': (), 'name': 'file535'})
    option_139789490995552 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490995824 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490996096 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490996368 = UnicodeOption(**{'default': u'wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490996640 = UnicodeOption(**{'default': u'/var/www/wpad.eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490996912 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490997184 = UnicodeOption(**{'default': u'644', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789490997456 = UnicodeOption(**{'default': u'/var/www/wpad.eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490997728 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490998000 = UnicodeOption(**{'default': u'/var/lib/creole/wpad.eth3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490998272 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789491488720 = OptionDescription(children=[option_139789490995552, option_139789490995824, option_139789490996096, option_139789490996368, option_139789490996640, option_139789490996912, option_139789490997184, option_139789490997456, option_139789490997728, option_139789490998000, option_139789490998272], **{'doc': u'', 'properties': (), 'name': 'file536'})
    option_139789490998544 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490998816 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491007568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491007840 = UnicodeOption(**{'default': u'wpad', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491008112 = UnicodeOption(**{'default': u'/var/www/wpad.eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491008384 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491008656 = UnicodeOption(**{'default': u'644', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789491008928 = UnicodeOption(**{'default': u'/var/www/wpad.eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491009200 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491009472 = UnicodeOption(**{'default': u'/var/lib/creole/wpad.eth4', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491009744 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497972128}]})
    option_139789491489040 = OptionDescription(children=[option_139789490998544, option_139789490998816, option_139789491007568, option_139789491007840, option_139789491008112, option_139789491008384, option_139789491008656, option_139789491008928, option_139789491009200, option_139789491009472, option_139789491009744], **{'doc': u'', 'properties': (), 'name': 'file537'})
    option_139789491010016 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491010288 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491010560 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491010832 = UnicodeOption(**{'default': u'/etc/nginx/sites-enabled/default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491011104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491019856 = UnicodeOption(**{'default': u'/etc/nginx/sites-enabled/default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491020128 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491020400 = UnicodeOption(**{'default': u'/var/lib/creole/nginx.default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491020672 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491489360 = OptionDescription(children=[option_139789491010016, option_139789491010288, option_139789491010560, option_139789491010832, option_139789491011104, option_139789491019856, option_139789491020128, option_139789491020400, option_139789491020672], **{'doc': u'', 'properties': (), 'name': 'file538'})
    option_139789491020944 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491021216 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491021488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491021760 = UnicodeOption(**{'default': u'/etc/nginx/eole_proxy_params', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491022032 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491022304 = UnicodeOption(**{'default': u'/etc/nginx/eole_proxy_params', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491022576 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491022848 = UnicodeOption(**{'default': u'/var/lib/creole/nginx.proxy-params', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491023120 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491489680 = OptionDescription(children=[option_139789491020944, option_139789491021216, option_139789491021488, option_139789491021760, option_139789491022032, option_139789491022304, option_139789491022576, option_139789491022848, option_139789491023120], **{'doc': u'', 'properties': (), 'name': 'file539'})
    option_139789491023392 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491028048 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491028320 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491028592 = UnicodeOption(**{'default': u'/usr/share/nginx/html/nginx.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491028864 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491029136 = UnicodeOption(**{'default': u'/usr/share/nginx/html/nginx.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491029408 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491029680 = UnicodeOption(**{'default': u'/var/lib/creole/nginx.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491029952 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491490000 = OptionDescription(children=[option_139789491023392, option_139789491028048, option_139789491028320, option_139789491028592, option_139789491028864, option_139789491029136, option_139789491029408, option_139789491029680, option_139789491029952], **{'doc': u'', 'properties': (), 'name': 'file540'})
    option_139789491030224 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491030496 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491030768 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491031040 = UnicodeOption(**{'default': u'/var/www/index.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491031312 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491031584 = UnicodeOption(**{'default': u'/var/www/index.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491040336 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491040608 = UnicodeOption(**{'default': u'/var/lib/creole/nginx.no_proxy.html', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491040880 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491490320 = OptionDescription(children=[option_139789491030224, option_139789491030496, option_139789491030768, option_139789491031040, option_139789491031312, option_139789491031584, option_139789491040336, option_139789491040608, option_139789491040880], **{'doc': u'', 'properties': (), 'name': 'file541'})
    option_139789491041152 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491041424 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491041696 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491041968 = UnicodeOption(**{'default': u'/etc/logrotate.d/nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491042240 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491042512 = UnicodeOption(**{'default': u'/etc/logrotate.d/nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491042784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491043056 = UnicodeOption(**{'default': u'/var/lib/creole/nginx.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491043328 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491044432 = OptionDescription(children=[option_139789491041152, option_139789491041424, option_139789491041696, option_139789491041968, option_139789491042240, option_139789491042512, option_139789491042784, option_139789491043056, option_139789491043328], **{'doc': u'', 'properties': (), 'name': 'file542'})
    option_139789491043600 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491043872 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491052624 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491052896 = UnicodeOption(**{'default': u'/etc/systemd/system/nginx.service.d/override.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491053168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491053440 = UnicodeOption(**{'default': u'/etc/systemd/system/nginx.service.d/override.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491053712 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491053984 = UnicodeOption(**{'default': u'/var/lib/creole/nginx.override', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491054256 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491044752 = OptionDescription(children=[option_139789491043600, option_139789491043872, option_139789491052624, option_139789491052896, option_139789491053168, option_139789491053440, option_139789491053712, option_139789491053984, option_139789491054256], **{'doc': u'', 'properties': (), 'name': 'file543'})
    option_139789491054528 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491054800 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491055072 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491055344 = UnicodeOption(**{'default': u'proftpd_container', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491055616 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/11-proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491055888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491056160 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789491060816 = UnicodeOption(**{'default': u'/usr/share/eole/bastion/data/11-proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491061088 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491061360 = UnicodeOption(**{'default': u'/var/lib/creole/11-proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491061632 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}, {'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497889120}]})
    option_139789491045072 = OptionDescription(children=[option_139789491054528, option_139789491054800, option_139789491055072, option_139789491055344, option_139789491055616, option_139789491055888, option_139789491056160, option_139789491060816, option_139789491061088, option_139789491061360, option_139789491061632], **{'doc': u'', 'properties': (), 'name': 'file544'})
    option_139789491061904 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491062176 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491062448 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491062720 = UnicodeOption(**{'default': u'proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491062992 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/proftpd/proftpd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491063264 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491063536 = UnicodeOption(**{'default': u'/etc/proftpd/proftpd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491063808 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491064080 = UnicodeOption(**{'default': u'/var/lib/creole/proftpd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491064352 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}]})
    option_139789491045392 = OptionDescription(children=[option_139789491061904, option_139789491062176, option_139789491062448, option_139789491062720, option_139789491062992, option_139789491063264, option_139789491063536, option_139789491063808, option_139789491064080, option_139789491064352], **{'doc': u'', 'properties': (), 'name': 'file545'})
    option_139789491069008 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491069280 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491069552 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491069824 = UnicodeOption(**{'default': u'proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491070096 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/pam.d/proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491070368 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491070640 = UnicodeOption(**{'default': u'/etc/pam.d/proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491070912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491071184 = UnicodeOption(**{'default': u'/var/lib/creole/pam_proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491071456 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}]})
    option_139789491045712 = OptionDescription(children=[option_139789491069008, option_139789491069280, option_139789491069552, option_139789491069824, option_139789491070096, option_139789491070368, option_139789491070640, option_139789491070912, option_139789491071184, option_139789491071456], **{'doc': u'', 'properties': (), 'name': 'file546'})
    option_139789491071728 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491072000 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491072272 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491072544 = UnicodeOption(**{'default': u'ssoclient', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491077200 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/pam_cas.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491077472 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491077744 = UnicodeOption(**{'default': u'/etc/pam_cas.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491078016 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491078288 = UnicodeOption(**{'default': u'/var/lib/creole/pam_cas_auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491078560 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}]})
    option_139789491046032 = OptionDescription(children=[option_139789491071728, option_139789491072000, option_139789491072272, option_139789491072544, option_139789491077200, option_139789491077472, option_139789491077744, option_139789491078016, option_139789491078288, option_139789491078560], **{'doc': u'', 'properties': (), 'name': 'file547'})
    option_139789491078832 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491079104 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491079376 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491079648 = UnicodeOption(**{'default': u'ssoclient', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491079920 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/pam_cas.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491080192 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491080464 = UnicodeOption(**{'default': u'/etc/pam_cas.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491080736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491089488 = UnicodeOption(**{'default': u'/var/lib/creole/pam_cas_auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491089760 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}]})
    option_139789491046352 = OptionDescription(children=[option_139789491078832, option_139789491079104, option_139789491079376, option_139789491079648, option_139789491079920, option_139789491080192, option_139789491080464, option_139789491080736, option_139789491089488, option_139789491089760], **{'doc': u'', 'properties': (), 'name': 'file548'})
    option_139789491090032 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491090304 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491090576 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491090848 = UnicodeOption(**{'default': u'/var/lib/eole/robot.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491091120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491091392 = UnicodeOption(**{'default': u'/var/lib/eole/robot.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491091664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491091936 = UnicodeOption(**{'default': u'/var/lib/creole/robot.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491092208 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491046672 = OptionDescription(children=[option_139789491090032, option_139789491090304, option_139789491090576, option_139789491090848, option_139789491091120, option_139789491091392, option_139789491091664, option_139789491091936, option_139789491092208], **{'doc': u'', 'properties': (), 'name': 'file549'})
    option_139789491092480 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491092752 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491093024 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491097680 = UnicodeOption(**{'default': u'/etc/logrotate.d/sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491097952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491098224 = UnicodeOption(**{'default': u'/etc/logrotate.d/sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491098496 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491098768 = UnicodeOption(**{'default': u'/var/lib/creole/sympa.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491099040 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491046992 = OptionDescription(children=[option_139789491092480, option_139789491092752, option_139789491093024, option_139789491097680, option_139789491097952, option_139789491098224, option_139789491098496, option_139789491098768, option_139789491099040], **{'doc': u'', 'properties': (), 'name': 'file550'})
    option_139789491099312 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491099584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491099856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491100128 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/sympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491100400 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491100672 = UnicodeOption(**{'default': u'/etc/rsyslog.d/eole-traps/sympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491100944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491101216 = UnicodeOption(**{'default': u'/var/lib/creole/rsyslog_traps_sympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491105872 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491047312 = OptionDescription(children=[option_139789491099312, option_139789491099584, option_139789491099856, option_139789491100128, option_139789491100400, option_139789491100672, option_139789491100944, option_139789491101216, option_139789491105872], **{'doc': u'', 'properties': (), 'name': 'file551'})
    option_139789491106144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491106416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491106688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491106960 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491107232 = UnicodeOption(**{'default': u'/usr/share/ead2/backend/config/actions/actions_mail.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491107504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491107776 = UnicodeOption(**{'default': u'/usr/share/ead2/backend/config/actions/actions_mail.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491108048 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491108320 = UnicodeOption(**{'default': u'/var/lib/creole/actions_mail.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491108592 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789491047632 = OptionDescription(children=[option_139789491106144, option_139789491106416, option_139789491106688, option_139789491106960, option_139789491107232, option_139789491107504, option_139789491107776, option_139789491108048, option_139789491108320, option_139789491108592], **{'doc': u'', 'properties': (), 'name': 'file552'})
    option_139789491108864 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491109136 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491109408 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491114064 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491114336 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/sympa/topics.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491114608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491114880 = UnicodeOption(**{'default': u'/etc/sympa/topics.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491115152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491115424 = UnicodeOption(**{'default': u'/var/lib/creole/topics.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491115696 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789491047952 = OptionDescription(children=[option_139789491108864, option_139789491109136, option_139789491109408, option_139789491114064, option_139789491114336, option_139789491114608, option_139789491114880, option_139789491115152, option_139789491115424, option_139789491115696], **{'doc': u'', 'properties': (), 'name': 'file553'})
    option_139789491115968 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491116240 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491116512 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491116784 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491117056 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/sympa/list_aliases.tt2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491117328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491117600 = UnicodeOption(**{'default': u'/etc/sympa/list_aliases.tt2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491122256 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491122528 = UnicodeOption(**{'default': u'/var/lib/creole/list_aliases.tt2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491122800 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789491130448 = OptionDescription(children=[option_139789491115968, option_139789491116240, option_139789491116512, option_139789491116784, option_139789491117056, option_139789491117328, option_139789491117600, option_139789491122256, option_139789491122528, option_139789491122800], **{'doc': u'', 'properties': (), 'name': 'file554'})
    option_139789491123072 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491123344 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491123616 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491123888 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491124160 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/sympa/auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491124432 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789491124704 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491124976 = UnicodeOption(**{'default': u'/etc/sympa/auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491125248 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'owner'})
    option_139789491125520 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491125792 = UnicodeOption(**{'default': u'/var/lib/creole/sympa.auth.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491138640 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789491130768 = OptionDescription(children=[option_139789491123072, option_139789491123344, option_139789491123616, option_139789491123888, option_139789491124160, option_139789491124432, option_139789491124704, option_139789491124976, option_139789491125248, option_139789491125520, option_139789491125792, option_139789491138640], **{'doc': u'', 'properties': (), 'name': 'file555'})
    option_139789491138912 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491139184 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491139456 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491139728 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491140000 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/sympa/sympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491140272 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789491140544 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491140816 = UnicodeOption(**{'default': u'0644', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789491141088 = UnicodeOption(**{'default': u'/etc/sympa/sympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491141360 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'owner'})
    option_139789491141632 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491141904 = UnicodeOption(**{'default': u'/var/lib/creole/sympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491142176 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789491131088 = OptionDescription(children=[option_139789491138912, option_139789491139184, option_139789491139456, option_139789491139728, option_139789491140000, option_139789491140272, option_139789491140544, option_139789491140816, option_139789491141088, option_139789491141360, option_139789491141632, option_139789491141904, option_139789491142176], **{'doc': u'', 'properties': (), 'name': 'file556'})
    option_139789491150928 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491151200 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491151472 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491151744 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491152016 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/sympa/wwsympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491152288 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789491152560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491152832 = UnicodeOption(**{'default': u'0644', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789491153104 = UnicodeOption(**{'default': u'/etc/sympa/wwsympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491153376 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'owner'})
    option_139789491153648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491153920 = UnicodeOption(**{'default': u'/var/lib/creole/wwsympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491154192 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789491131408 = OptionDescription(children=[option_139789491150928, option_139789491151200, option_139789491151472, option_139789491151744, option_139789491152016, option_139789491152288, option_139789491152560, option_139789491152832, option_139789491153104, option_139789491153376, option_139789491153648, option_139789491153920, option_139789491154192], **{'doc': u'', 'properties': (), 'name': 'file557'})
    option_139789491154464 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491159120 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491159392 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491159664 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491159936 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/sites-available/wwsympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491160208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491160480 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/wwsympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491160752 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491161024 = UnicodeOption(**{'default': u'/var/lib/creole/apache-sympa.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491161296 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789491131728 = OptionDescription(children=[option_139789491154464, option_139789491159120, option_139789491159392, option_139789491159664, option_139789491159936, option_139789491160208, option_139789491160480, option_139789491160752, option_139789491161024, option_139789491161296], **{'doc': u'', 'properties': (), 'name': 'file558'})
    option_139789491161568 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491161840 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491162112 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491162384 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491162656 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/var/www/sympa/wwsympa.fcgi', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491171408 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'group'})
    option_139789491171680 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491171952 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789491172224 = UnicodeOption(**{'default': u'/var/www/sympa/wwsympa.fcgi', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491172496 = UnicodeOption(**{'default': u'sympa', 'doc': u'', 'multi': False, 'properties': (), 'name': 'owner'})
    option_139789491172768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491173040 = UnicodeOption(**{'default': u'/var/lib/creole/wwsympa.fcgi', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491173312 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496748784}]})
    option_139789491132048 = OptionDescription(children=[option_139789491161568, option_139789491161840, option_139789491162112, option_139789491162384, option_139789491162656, option_139789491171408, option_139789491171680, option_139789491171952, option_139789491172224, option_139789491172496, option_139789491172768, option_139789491173040, option_139789491173312], **{'doc': u'', 'properties': (), 'name': 'file559'})
    option_139789491173584 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491173856 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491174128 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491174400 = UnicodeOption(**{'default': u'/home/client_scribe/install.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491174672 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491174944 = UnicodeOption(**{'default': u'/home/client_scribe/install.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491179600 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491179872 = UnicodeOption(**{'default': u'/var/lib/creole/install.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491180144 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491132368 = OptionDescription(children=[option_139789491173584, option_139789491173856, option_139789491174128, option_139789491174400, option_139789491174672, option_139789491174944, option_139789491179600, option_139789491179872, option_139789491180144], **{'doc': u'', 'properties': (), 'name': 'file560'})
    option_139789491180416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491180688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491180960 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491181232 = UnicodeOption(**{'default': u'/home/client_scribe/liste_fwregles.eol', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491181504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491181776 = UnicodeOption(**{'default': u'/home/client_scribe/liste_fwregles.eol', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491182048 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491182320 = UnicodeOption(**{'default': u'/var/lib/creole/liste_fwregles.eol', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491182592 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491132688 = OptionDescription(children=[option_139789491180416, option_139789491180688, option_139789491180960, option_139789491181232, option_139789491181504, option_139789491181776, option_139789491182048, option_139789491182320, option_139789491182592], **{'doc': u'', 'properties': (), 'name': 'file561'})
    option_139789491182864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491183136 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491187792 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491188064 = UnicodeOption(**{'default': u'/home/a/admin/perso/IntegrDom/joineole.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491188336 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491188608 = UnicodeOption(**{'default': u'/home/a/admin/perso/IntegrDom/joineole.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491188880 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491189152 = UnicodeOption(**{'default': u'/var/lib/creole/joineole.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491189424 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491133008 = OptionDescription(children=[option_139789491182864, option_139789491183136, option_139789491187792, option_139789491188064, option_139789491188336, option_139789491188608, option_139789491188880, option_139789491189152, option_139789491189424], **{'doc': u'', 'properties': (), 'name': 'file562'})
    option_139789491189696 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491189968 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491190240 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491190512 = UnicodeOption(**{'default': u'/usr/share/eole/mysql/controlevnc/gen/controlevnc.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491190784 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491191056 = UnicodeOption(**{'default': u'/usr/share/eole/mysql/controlevnc/gen/controlevnc.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491191328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491195984 = UnicodeOption(**{'default': u'/var/lib/creole/controlevnc.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491196256 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491133328 = OptionDescription(children=[option_139789491189696, option_139789491189968, option_139789491190240, option_139789491190512, option_139789491190784, option_139789491191056, option_139789491191328, option_139789491195984, option_139789491196256], **{'doc': u'', 'properties': (), 'name': 'file563'})
    option_139789491196528 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491196800 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491197072 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491197344 = UnicodeOption(**{'default': u'/usr/share/eole/mysql/controlevnc/updates/controlevnc.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491197616 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491197888 = UnicodeOption(**{'default': u'/usr/share/eole/mysql/controlevnc/updates/controlevnc.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491198160 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491198432 = UnicodeOption(**{'default': u'/var/lib/creole/controlevnc-update.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491198704 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491133648 = OptionDescription(children=[option_139789491196528, option_139789491196800, option_139789491197072, option_139789491197344, option_139789491197616, option_139789491197888, option_139789491198160, option_139789491198432, option_139789491198704], **{'doc': u'', 'properties': (), 'name': 'file564'})
    option_139789491198976 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491199248 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491199520 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491204176 = UnicodeOption(**{'default': u'/usr/share/eole/applications/passwords/controlevnc.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491204448 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491204720 = UnicodeOption(**{'default': u'/usr/share/eole/applications/passwords/controlevnc.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491204992 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491205264 = UnicodeOption(**{'default': u'/var/lib/creole/controlevnc.ini', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491205536 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491133968 = OptionDescription(children=[option_139789491198976, option_139789491199248, option_139789491199520, option_139789491204176, option_139789491204448, option_139789491204720, option_139789491204992, option_139789491205264, option_139789491205536], **{'doc': u'', 'properties': (), 'name': 'file565'})
    option_139789491205808 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491206080 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491206352 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491206624 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/usr/share/eole/controlevnc/config.py', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491206896 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491207168 = UnicodeOption(**{'default': u'/usr/share/eole/controlevnc/config.py', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491207440 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491207712 = UnicodeOption(**{'default': u'/var/lib/creole/vnc_config.py', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491212368 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491216464 = OptionDescription(children=[option_139789491205808, option_139789491206080, option_139789491206352, option_139789491206624, option_139789491206896, option_139789491207168, option_139789491207440, option_139789491207712, option_139789491212368], **{'doc': u'', 'properties': (), 'name': 'file566'})
    option_139789491212640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491212912 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491213184 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491213456 = UnicodeOption(**{'default': u'/etc/logrotate.d/freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491213728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491214000 = UnicodeOption(**{'default': u'/etc/logrotate.d/freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491214272 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491214544 = UnicodeOption(**{'default': u'/var/lib/creole/freeradius.logrotate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491214816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491216784 = OptionDescription(children=[option_139789491212640, option_139789491212912, option_139789491213184, option_139789491213456, option_139789491213728, option_139789491214000, option_139789491214272, option_139789491214544, option_139789491214816], **{'doc': u'', 'properties': (), 'name': 'file567'})
    option_139789491215088 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491215360 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491215632 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491215904 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491228752 = UnicodeOption(**{'default': u'/etc/freeradius/clients.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491229024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491229296 = UnicodeOption(**{'default': u'/etc/freeradius/clients.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491229568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789491229840 = UnicodeOption(**{'default': u'/var/lib/creole/freerad-clients.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789491230112 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789491217104 = OptionDescription(children=[option_139789491215088, option_139789491215360, option_139789491215632, option_139789491215904, option_139789491228752, option_139789491229024, option_139789491229296, option_139789491229568, option_139789491229840, option_139789491230112], **{'doc': u'', 'properties': (), 'name': 'file568'})
    option_139789491230384 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789491230656 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789491230928 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789491231200 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789491231472 = UnicodeOption(**{'default': u'/etc/freeradius/eap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789491231744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789491232016 = UnicodeOption(**{'default': u'/etc/freeradius/eap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789491232288 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490716752 = UnicodeOption(**{'default': u'/var/lib/creole/freerad-eap.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490717024 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789491217424 = OptionDescription(children=[option_139789491230384, option_139789491230656, option_139789491230928, option_139789491231200, option_139789491231472, option_139789491231744, option_139789491232016, option_139789491232288, option_139789490716752, option_139789490717024], **{'doc': u'', 'properties': (), 'name': 'file569'})
    option_139789490717296 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490717568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490717840 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490718112 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490718384 = UnicodeOption(**{'default': u'/etc/freeradius/huntgroups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490718656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490718928 = UnicodeOption(**{'default': u'/etc/freeradius/huntgroups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490719200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490719472 = UnicodeOption(**{'default': u'/var/lib/creole/freerad-huntgroups', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490719744 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789491217744 = OptionDescription(children=[option_139789490717296, option_139789490717568, option_139789490717840, option_139789490718112, option_139789490718384, option_139789490718656, option_139789490718928, option_139789490719200, option_139789490719472, option_139789490719744], **{'doc': u'', 'properties': (), 'name': 'file570'})
    option_139789490720016 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490720288 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490729040 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490729312 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490729584 = UnicodeOption(**{'default': u'/etc/freeradius/radiusd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490729856 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490730128 = UnicodeOption(**{'default': u'/etc/freeradius/radiusd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490730400 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490730672 = UnicodeOption(**{'default': u'/var/lib/creole/radiusd.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490730944 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789491218064 = OptionDescription(children=[option_139789490720016, option_139789490720288, option_139789490729040, option_139789490729312, option_139789490729584, option_139789490729856, option_139789490730128, option_139789490730400, option_139789490730672, option_139789490730944], **{'doc': u'', 'properties': (), 'name': 'file571'})
    option_139789490731216 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490731488 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490731760 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490732032 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490732304 = UnicodeOption(**{'default': u'/etc/freeradius/users', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490732576 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490737232 = UnicodeOption(**{'default': u'/etc/freeradius/users', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490737504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490737776 = UnicodeOption(**{'default': u'/var/lib/creole/freerad-users', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490738048 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789491218384 = OptionDescription(children=[option_139789490731216, option_139789490731488, option_139789490731760, option_139789490732032, option_139789490732304, option_139789490732576, option_139789490737232, option_139789490737504, option_139789490737776, option_139789490738048], **{'doc': u'', 'properties': (), 'name': 'file572'})
    option_139789490738320 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490738592 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490738864 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490739136 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490739408 = UnicodeOption(**{'default': u'/etc/freeradius/modules/ldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490739680 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490739952 = UnicodeOption(**{'default': u'/etc/freeradius/modules/ldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490740224 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490740496 = UnicodeOption(**{'default': u'/var/lib/creole/freerad-ldap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490740768 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789491218704 = OptionDescription(children=[option_139789490738320, option_139789490738592, option_139789490738864, option_139789490739136, option_139789490739408, option_139789490739680, option_139789490739952, option_139789490740224, option_139789490740496, option_139789490740768], **{'doc': u'', 'properties': (), 'name': 'file573'})
    option_139789490745424 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490745696 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490745968 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490746240 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490746512 = UnicodeOption(**{'default': u'/etc/freeradius/modules/mschap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490746784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490747056 = UnicodeOption(**{'default': u'/etc/freeradius/modules/mschap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490747328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490747600 = UnicodeOption(**{'default': u'/var/lib/creole/freerad-mschap', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490747872 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789491219024 = OptionDescription(children=[option_139789490745424, option_139789490745696, option_139789490745968, option_139789490746240, option_139789490746512, option_139789490746784, option_139789490747056, option_139789490747328, option_139789490747600, option_139789490747872], **{'doc': u'', 'properties': (), 'name': 'file574'})
    option_139789490748144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490748416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490748688 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490748960 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490753616 = UnicodeOption(**{'default': u'/etc/freeradius/sites-enabled/default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490753888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490754160 = UnicodeOption(**{'default': u'/etc/freeradius/sites-enabled/default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490754432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490754704 = UnicodeOption(**{'default': u'/var/lib/creole/freerad-default', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490754976 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789491219344 = OptionDescription(children=[option_139789490748144, option_139789490748416, option_139789490748688, option_139789490748960, option_139789490753616, option_139789490753888, option_139789490754160, option_139789490754432, option_139789490754704, option_139789490754976], **{'doc': u'', 'properties': (), 'name': 'file575'})
    option_139789490755248 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490755520 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490755792 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490756064 = UnicodeOption(**{'default': u'freeradius', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490756336 = UnicodeOption(**{'default': u'/etc/freeradius/sites-enabled/inner-tunnel', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490756608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490756880 = UnicodeOption(**{'default': u'/etc/freeradius/sites-enabled/inner-tunnel', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490757152 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490761808 = UnicodeOption(**{'default': u'/var/lib/creole/freerad-inner-tunnel', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490762080 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789496794384}]})
    option_139789491219664 = OptionDescription(children=[option_139789490755248, option_139789490755520, option_139789490755792, option_139789490756064, option_139789490756336, option_139789490756608, option_139789490756880, option_139789490757152, option_139789490761808, option_139789490762080], **{'doc': u'', 'properties': (), 'name': 'file576'})
    option_139789490762352 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490762624 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490762896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490763168 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490763440 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490763712 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490763984 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490764256 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490764528 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789491219984 = OptionDescription(children=[option_139789490762352, option_139789490762624, option_139789490762896, option_139789490763168, option_139789490763440, option_139789490763712, option_139789490763984, option_139789490764256, option_139789490764528], **{'doc': u'', 'properties': (), 'name': 'file577'})
    option_139789490764800 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490765072 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490765344 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490774096 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490774368 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490774640 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490774912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490775184 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490775456 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490782288 = OptionDescription(children=[option_139789490764800, option_139789490765072, option_139789490765344, option_139789490774096, option_139789490774368, option_139789490774640, option_139789490774912, option_139789490775184, option_139789490775456], **{'doc': u'', 'properties': (), 'name': 'file578'})
    option_139789490775728 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490776000 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490776272 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490776544 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490776816 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490777088 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490777360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490777632 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490786384 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490782608 = OptionDescription(children=[option_139789490775728, option_139789490776000, option_139789490776272, option_139789490776544, option_139789490776816, option_139789490777088, option_139789490777360, option_139789490777632, option_139789490786384], **{'doc': u'', 'properties': (), 'name': 'file579'})
    option_139789490786656 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490786928 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490787200 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490787472 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490787744 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490788016 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490788288 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490788560 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490788832 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490782928 = OptionDescription(children=[option_139789490786656, option_139789490786928, option_139789490787200, option_139789490787472, option_139789490787744, option_139789490788016, option_139789490788288, option_139789490788560, option_139789490788832], **{'doc': u'', 'properties': (), 'name': 'file580'})
    option_139789490789104 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490789376 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490789648 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490789920 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490794576 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490794848 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490795120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490795392 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490795664 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490783248 = OptionDescription(children=[option_139789490789104, option_139789490789376, option_139789490789648, option_139789490789920, option_139789490794576, option_139789490794848, option_139789490795120, option_139789490795392, option_139789490795664], **{'doc': u'', 'properties': (), 'name': 'file581'})
    option_139789490795936 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490796208 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490796480 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490796752 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490797024 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490797296 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490797568 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490797840 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490798112 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490783568 = OptionDescription(children=[option_139789490795936, option_139789490796208, option_139789490796480, option_139789490796752, option_139789490797024, option_139789490797296, option_139789490797568, option_139789490797840, option_139789490798112], **{'doc': u'', 'properties': (), 'name': 'file582'})
    option_139789490806864 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490807136 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490807408 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490807680 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490807952 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490808224 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490808496 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490808768 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490809040 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490783888 = OptionDescription(children=[option_139789490806864, option_139789490807136, option_139789490807408, option_139789490807680, option_139789490807952, option_139789490808224, option_139789490808496, option_139789490808768, option_139789490809040], **{'doc': u'', 'properties': (), 'name': 'file583'})
    option_139789490809312 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490809584 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490809856 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490810128 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490810400 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490815056 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490815328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490815600 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490815872 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490784208 = OptionDescription(children=[option_139789490809312, option_139789490809584, option_139789490809856, option_139789490810128, option_139789490810400, option_139789490815056, option_139789490815328, option_139789490815600, option_139789490815872], **{'doc': u'', 'properties': (), 'name': 'file584'})
    option_139789490816144 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490816416 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490816688 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490816960 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490817232 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490817504 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490817776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490818048 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490818320 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490784528 = OptionDescription(children=[option_139789490816144, option_139789490816416, option_139789490816688, option_139789490816960, option_139789490817232, option_139789490817504, option_139789490817776, option_139789490818048, option_139789490818320], **{'doc': u'', 'properties': (), 'name': 'file585'})
    option_139789490818592 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490823248 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490823520 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490823792 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490824064 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490824336 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490824608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490824880 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490825152 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490784848 = OptionDescription(children=[option_139789490818592, option_139789490823248, option_139789490823520, option_139789490823792, option_139789490824064, option_139789490824336, option_139789490824608, option_139789490824880, option_139789490825152], **{'doc': u'', 'properties': (), 'name': 'file586'})
    option_139789490825424 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490825696 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490825968 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490826240 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490826512 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490826784 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490831440 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490831712 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490831984 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490785168 = OptionDescription(children=[option_139789490825424, option_139789490825696, option_139789490825968, option_139789490826240, option_139789490826512, option_139789490826784, option_139789490831440, option_139789490831712, option_139789490831984], **{'doc': u'', 'properties': (), 'name': 'file587'})
    option_139789490832256 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490832528 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490832800 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490833072 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490833344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490833616 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490833888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490834160 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490834432 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490785488 = OptionDescription(children=[option_139789490832256, option_139789490832528, option_139789490832800, option_139789490833072, option_139789490833344, option_139789490833616, option_139789490833888, option_139789490834160, option_139789490834432], **{'doc': u'', 'properties': (), 'name': 'file588'})
    option_139789490834704 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490834976 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490839632 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490839904 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490840176 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490840448 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490840720 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490840992 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490841264 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490785808 = OptionDescription(children=[option_139789490834704, option_139789490834976, option_139789490839632, option_139789490839904, option_139789490840176, option_139789490840448, option_139789490840720, option_139789490840992, option_139789490841264], **{'doc': u'', 'properties': (), 'name': 'file589'})
    option_139789490841536 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490841808 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490842080 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490842352 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490842624 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490842896 = UnicodeOption(**{'default': u'/etc/ssmtp/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490843168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490847824 = UnicodeOption(**{'default': u'/var/lib/creole/ssmtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490848096 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490851920 = OptionDescription(children=[option_139789490841536, option_139789490841808, option_139789490842080, option_139789490842352, option_139789490842624, option_139789490842896, option_139789490843168, option_139789490847824, option_139789490848096], **{'doc': u'', 'properties': (), 'name': 'file590'})
    option_139789490848368 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490848640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490848912 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490849184 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490849456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490849728 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490850000 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490850272 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490850544 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490852240 = OptionDescription(children=[option_139789490848368, option_139789490848640, option_139789490848912, option_139789490849184, option_139789490849456, option_139789490849728, option_139789490850000, option_139789490850272, option_139789490850544], **{'doc': u'', 'properties': (), 'name': 'file591'})
    option_139789490850816 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490851088 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490851360 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490864208 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490864480 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490864752 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490865024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490865296 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490865568 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490852560 = OptionDescription(children=[option_139789490850816, option_139789490851088, option_139789490851360, option_139789490864208, option_139789490864480, option_139789490864752, option_139789490865024, option_139789490865296, option_139789490865568], **{'doc': u'', 'properties': (), 'name': 'file592'})
    option_139789490865840 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490866112 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490866384 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490866656 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490866928 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490867200 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490867472 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490867744 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490872400 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490852880 = OptionDescription(children=[option_139789490865840, option_139789490866112, option_139789490866384, option_139789490866656, option_139789490866928, option_139789490867200, option_139789490867472, option_139789490867744, option_139789490872400], **{'doc': u'', 'properties': (), 'name': 'file593'})
    option_139789490872672 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490872944 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490873216 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490873488 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490873760 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490874032 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490874304 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490874576 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490874848 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490853200 = OptionDescription(children=[option_139789490872672, option_139789490872944, option_139789490873216, option_139789490873488, option_139789490873760, option_139789490874032, option_139789490874304, option_139789490874576, option_139789490874848], **{'doc': u'', 'properties': (), 'name': 'file594'})
    option_139789490875120 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490875392 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490875664 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490875936 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490880592 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490880864 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490881136 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490881408 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490881680 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490853520 = OptionDescription(children=[option_139789490875120, option_139789490875392, option_139789490875664, option_139789490875936, option_139789490880592, option_139789490880864, option_139789490881136, option_139789490881408, option_139789490881680], **{'doc': u'', 'properties': (), 'name': 'file595'})
    option_139789490881952 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490882224 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490882496 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490882768 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490883040 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490883312 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490883584 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490883856 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490884128 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490853840 = OptionDescription(children=[option_139789490881952, option_139789490882224, option_139789490882496, option_139789490882768, option_139789490883040, option_139789490883312, option_139789490883584, option_139789490883856, option_139789490884128], **{'doc': u'', 'properties': (), 'name': 'file596'})
    option_139789490892880 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490893152 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490893424 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490893696 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490893968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490894240 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490894512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490894784 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490895056 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490854160 = OptionDescription(children=[option_139789490892880, option_139789490893152, option_139789490893424, option_139789490893696, option_139789490893968, option_139789490894240, option_139789490894512, option_139789490894784, option_139789490895056], **{'doc': u'', 'properties': (), 'name': 'file597'})
    option_139789490895328 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490895600 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490895872 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490896144 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490896416 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490901072 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490901344 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490901616 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490901888 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490854480 = OptionDescription(children=[option_139789490895328, option_139789490895600, option_139789490895872, option_139789490896144, option_139789490896416, option_139789490901072, option_139789490901344, option_139789490901616, option_139789490901888], **{'doc': u'', 'properties': (), 'name': 'file598'})
    option_139789490902160 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490902432 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490902704 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490902976 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490903248 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490903520 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490903792 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490904064 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490904336 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490854800 = OptionDescription(children=[option_139789490902160, option_139789490902432, option_139789490902704, option_139789490902976, option_139789490903248, option_139789490903520, option_139789490903792, option_139789490904064, option_139789490904336], **{'doc': u'', 'properties': (), 'name': 'file599'})
    option_139789490904608 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490909264 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490909536 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490909808 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490910080 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490910352 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490910624 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490910896 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490911168 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490855120 = OptionDescription(children=[option_139789490904608, option_139789490909264, option_139789490909536, option_139789490909808, option_139789490910080, option_139789490910352, option_139789490910624, option_139789490910896, option_139789490911168], **{'doc': u'', 'properties': (), 'name': 'file600'})
    option_139789490911440 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490911712 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490911984 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490912256 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490912528 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490912800 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490917456 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490917728 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490918000 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490855440 = OptionDescription(children=[option_139789490911440, option_139789490911712, option_139789490911984, option_139789490912256, option_139789490912528, option_139789490912800, option_139789490917456, option_139789490917728, option_139789490918000], **{'doc': u'', 'properties': (), 'name': 'file601'})
    option_139789490918272 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490918544 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490918816 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490919088 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490919360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490919632 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490919904 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490920176 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490920448 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490925648 = OptionDescription(children=[option_139789490918272, option_139789490918544, option_139789490918816, option_139789490919088, option_139789490919360, option_139789490919632, option_139789490919904, option_139789490920176, option_139789490920448], **{'doc': u'', 'properties': (), 'name': 'file602'})
    option_139789490920720 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490920992 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490929744 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490930016 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490930288 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490930560 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490930832 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490931104 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490931376 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490925968 = OptionDescription(children=[option_139789490920720, option_139789490920992, option_139789490929744, option_139789490930016, option_139789490930288, option_139789490930560, option_139789490930832, option_139789490931104, option_139789490931376], **{'doc': u'', 'properties': (), 'name': 'file603'})
    option_139789490931648 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490931920 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490932192 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490932464 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490932736 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490933008 = UnicodeOption(**{'default': u'/etc/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490933280 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490937936 = UnicodeOption(**{'default': u'/var/lib/creole/mailname', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490938208 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490926288 = OptionDescription(children=[option_139789490931648, option_139789490931920, option_139789490932192, option_139789490932464, option_139789490932736, option_139789490933008, option_139789490933280, option_139789490937936, option_139789490938208], **{'doc': u'', 'properties': (), 'name': 'file604'})
    option_139789490938480 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490938752 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490939024 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490939296 = UnicodeOption(**{'default': u'exim-bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490939568 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/mail.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490939840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490940112 = UnicodeOption(**{'default': u'/etc/bareos/bareosfichiers.d/mail.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490940384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490940656 = UnicodeOption(**{'default': u'/var/lib/creole/bareos-mail.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490940928 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789490926608 = OptionDescription(children=[option_139789490938480, option_139789490938752, option_139789490939024, option_139789490939296, option_139789490939568, option_139789490939840, option_139789490940112, option_139789490940384, option_139789490940656, option_139789490940928], **{'doc': u'', 'properties': (), 'name': 'file605'})
    option_139789490941200 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490941472 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490946128 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490946400 = UnicodeOption(**{'default': u'exim', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490946672 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/exim4/eole.d/exim-vars.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490946944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490947216 = UnicodeOption(**{'default': u'/etc/exim4/eole.d/exim-vars.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490947488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490947760 = UnicodeOption(**{'default': u'/var/lib/creole/exim-vars.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490948032 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490926928 = OptionDescription(children=[option_139789490941200, option_139789490941472, option_139789490946128, option_139789490946400, option_139789490946672, option_139789490946944, option_139789490947216, option_139789490947488, option_139789490947760, option_139789490948032], **{'doc': u'', 'properties': (), 'name': 'file606'})
    option_139789490948304 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490948576 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490948848 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490949120 = UnicodeOption(**{'default': u'exim', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490949392 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/exim4/eole.d/transport/smtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490949664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490954320 = UnicodeOption(**{'default': u'/etc/exim4/eole.d/transport/smtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490954592 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490954864 = UnicodeOption(**{'default': u'/var/lib/creole/smtp.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490955136 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490927248 = OptionDescription(children=[option_139789490948304, option_139789490948576, option_139789490948848, option_139789490949120, option_139789490949392, option_139789490949664, option_139789490954320, option_139789490954592, option_139789490954864, option_139789490955136], **{'doc': u'', 'properties': (), 'name': 'file607'})
    option_139789490955408 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490955680 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490955952 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490956224 = UnicodeOption(**{'default': u'exim', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490956496 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/exim4/eole.d/rewrite.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490956768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490957040 = UnicodeOption(**{'default': u'/etc/exim4/eole.d/rewrite.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490957312 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490957584 = UnicodeOption(**{'default': u'/var/lib/creole/exim-rewrite.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490957856 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490927568 = OptionDescription(children=[option_139789490955408, option_139789490955680, option_139789490955952, option_139789490956224, option_139789490956496, option_139789490956768, option_139789490957040, option_139789490957312, option_139789490957584, option_139789490957856], **{'doc': u'', 'properties': (), 'name': 'file608'})
    option_139789490962512 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490962784 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490963056 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490963328 = UnicodeOption(**{'default': u'ldapsso', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490963600 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/pam.d/exim', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490963872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490964144 = UnicodeOption(**{'default': u'/etc/pam.d/exim', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490964416 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490964688 = UnicodeOption(**{'default': u'/var/lib/creole/pam_exim', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490964960 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497964208}]})
    option_139789490927888 = OptionDescription(children=[option_139789490962512, option_139789490962784, option_139789490963056, option_139789490963328, option_139789490963600, option_139789490963872, option_139789490964144, option_139789490964416, option_139789490964688, option_139789490964960], **{'doc': u'', 'properties': (), 'name': 'file609'})
    option_139789490965232 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490965504 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490965776 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490966048 = UnicodeOption(**{'default': u'/etc/nginx/sites-available/eoleflask.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490450512 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490450784 = UnicodeOption(**{'default': u'/etc/nginx/sites-available/eoleflask.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490451056 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490451328 = UnicodeOption(**{'default': u'/var/lib/creole/eoleflask.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490451600 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490928208 = OptionDescription(children=[option_139789490965232, option_139789490965504, option_139789490965776, option_139789490966048, option_139789490450512, option_139789490450784, option_139789490451056, option_139789490451328, option_139789490451600], **{'doc': u'', 'properties': (), 'name': 'file610'})
    option_139789490451872 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490452144 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490452416 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490452688 = UnicodeOption(**{'default': u'/etc/ewt/appdata/ead/config.json', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490452960 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490453232 = UnicodeOption(**{'default': u'/etc/ewt/appdata/ead/config.json', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490453504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490453776 = UnicodeOption(**{'default': u'/var/lib/creole/config_ead.json', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490454048 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490928528 = OptionDescription(children=[option_139789490451872, option_139789490452144, option_139789490452416, option_139789490452688, option_139789490452960, option_139789490453232, option_139789490453504, option_139789490453776, option_139789490454048], **{'doc': u'', 'properties': (), 'name': 'file611'})
    option_139789490466896 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490467168 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490467440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490467712 = UnicodeOption(**{'default': u'ead3_nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490467984 = UnicodeOption(**{'default': u'/etc/nginx/web.d/ead3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490468256 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490468528 = UnicodeOption(**{'default': u'/etc/nginx/web.d/ead3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490468800 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490469072 = UnicodeOption(**{'default': u'/var/lib/creole/ead3.nginx.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490469344 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963392}]})
    option_139789490928848 = OptionDescription(children=[option_139789490466896, option_139789490467168, option_139789490467440, option_139789490467712, option_139789490467984, option_139789490468256, option_139789490468528, option_139789490468800, option_139789490469072, option_139789490469344], **{'doc': u'', 'properties': (), 'name': 'file612'})
    option_139789490469616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490469888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490470160 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490470432 = UnicodeOption(**{'default': u'ead3_apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490475088 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/ead3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490475360 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490475632 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/ead3.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490475904 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490476176 = UnicodeOption(**{'default': u'/var/lib/creole/ead3.apache.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490476448 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963392}, {'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497957648}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490929168 = OptionDescription(children=[option_139789490469616, option_139789490469888, option_139789490470160, option_139789490470432, option_139789490475088, option_139789490475360, option_139789490475632, option_139789490475904, option_139789490476176, option_139789490476448], **{'doc': u'', 'properties': (), 'name': 'file613'})
    option_139789490476720 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490476992 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490477264 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490477536 = UnicodeOption(**{'default': u'ead3_apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490477808 = UnicodeOption(**{'default': u'/etc/nginx/sites-enabled/ead3_apache.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490478080 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490478352 = UnicodeOption(**{'default': u'/etc/nginx/sites-enabled/ead3_apache.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490478624 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490483280 = UnicodeOption(**{'default': u'/var/lib/creole/ead3.apache_nginx.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490483552 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497963392}, {'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497957648}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490487376 = OptionDescription(children=[option_139789490476720, option_139789490476992, option_139789490477264, option_139789490477536, option_139789490477808, option_139789490478080, option_139789490478352, option_139789490478624, option_139789490483280, option_139789490483552], **{'doc': u'', 'properties': (), 'name': 'file614'})
    option_139789490483824 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490484096 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490484368 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490484640 = UnicodeOption(**{'default': u'ead3_fileserver', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490484912 = UnicodeOption(**{'default': u'/etc/eole/flask/available/ead3fileserver.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490485184 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490485456 = UnicodeOption(**{'default': u'/etc/eole/flask/available/ead3fileserver.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490485728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490486000 = UnicodeOption(**{'default': u'/var/lib/creole/ead3fileserver.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490486272 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490487696 = OptionDescription(children=[option_139789490483824, option_139789490484096, option_139789490484368, option_139789490484640, option_139789490484912, option_139789490485184, option_139789490485456, option_139789490485728, option_139789490486000, option_139789490486272], **{'doc': u'', 'properties': (), 'name': 'file615'})
    option_139789490486544 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490486816 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490495568 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490495840 = UnicodeOption(**{'default': u'/srv/salt/top.sls', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490496112 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490496384 = UnicodeOption(**{'default': u'/srv/salt/top.sls', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490496656 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490496928 = UnicodeOption(**{'default': u'/var/lib/creole/top.sls', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490497200 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490488016 = OptionDescription(children=[option_139789490486544, option_139789490486816, option_139789490495568, option_139789490495840, option_139789490496112, option_139789490496384, option_139789490496656, option_139789490496928, option_139789490497200], **{'doc': u'', 'properties': (), 'name': 'file616'})
    option_139789490497472 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490497744 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490498016 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490498288 = UnicodeOption(**{'default': u'/var/lib/eole/config/ead3.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490498560 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490498832 = UnicodeOption(**{'default': u'/var/lib/eole/config/ead3.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490499104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490503760 = UnicodeOption(**{'default': u'/var/lib/creole/ead3.services.cfg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490504032 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490488336 = OptionDescription(children=[option_139789490497472, option_139789490497744, option_139789490498016, option_139789490498288, option_139789490498560, option_139789490498832, option_139789490499104, option_139789490503760, option_139789490504032], **{'doc': u'', 'properties': (), 'name': 'file617'})
    option_139789490504304 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490504576 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490504848 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490505120 = UnicodeOption(**{'default': u'eoleapps', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490505392 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/sites-available/eoleapps-apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490505664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490505936 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/eoleapps-apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490506208 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490506480 = UnicodeOption(**{'default': u'/var/lib/creole/apache-eoleapps.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490506752 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145824}]})
    option_139789490488656 = OptionDescription(children=[option_139789490504304, option_139789490504576, option_139789490504848, option_139789490505120, option_139789490505392, option_139789490505664, option_139789490505936, option_139789490506208, option_139789490506480, option_139789490506752], **{'doc': u'', 'properties': (), 'name': 'file618'})
    option_139789490507024 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490507296 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490511952 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490512224 = UnicodeOption(**{'default': u'eoleapps', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490512496 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/eole/flask/eoleapps.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490512768 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490513040 = UnicodeOption(**{'default': u'/etc/eole/flask/eoleapps.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490513312 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490513584 = UnicodeOption(**{'default': u'/var/lib/creole/eoleapps.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490513856 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145824}]})
    option_139789490488976 = OptionDescription(children=[option_139789490507024, option_139789490507296, option_139789490511952, option_139789490512224, option_139789490512496, option_139789490512768, option_139789490513040, option_139789490513312, option_139789490513584, option_139789490513856], **{'doc': u'', 'properties': (), 'name': 'file619'})
    option_139789490514128 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490514400 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490514672 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490514944 = UnicodeOption(**{'default': u'genconfig_nginx', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490515216 = UnicodeOption(**{'default': u'/etc/nginx/web.d/genconfig.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490515488 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490524240 = UnicodeOption(**{'default': u'/etc/nginx/web.d/genconfig.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490524512 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490524784 = UnicodeOption(**{'default': u'/var/lib/creole/genconfig.nginx.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490525056 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497974032}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957920}]})
    option_139789490489296 = OptionDescription(children=[option_139789490514128, option_139789490514400, option_139789490514672, option_139789490514944, option_139789490515216, option_139789490515488, option_139789490524240, option_139789490524512, option_139789490524784, option_139789490525056], **{'doc': u'', 'properties': (), 'name': 'file620'})
    option_139789490525328 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490525600 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490525872 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490526144 = UnicodeOption(**{'default': u'genconfig_apache', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490526416 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/genconfig.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490526688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490526960 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/genconfig.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490527232 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490527504 = UnicodeOption(**{'default': u'/var/lib/creole/genconfig.apache.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490527776 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497974032}, {'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789497957920}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490489616 = OptionDescription(children=[option_139789490525328, option_139789490525600, option_139789490525872, option_139789490526144, option_139789490526416, option_139789490526688, option_139789490526960, option_139789490527232, option_139789490527504, option_139789490527776], **{'doc': u'', 'properties': (), 'name': 'file621'})
    option_139789490536528 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490536800 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490537072 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490537344 = UnicodeOption(**{'default': u'proftpd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490537616 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/proftpd/conf.d/scribehorus', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490537888 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490538160 = UnicodeOption(**{'default': u'/etc/proftpd/conf.d/scribehorus', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490538432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490538704 = UnicodeOption(**{'default': u'/var/lib/creole/proftpd_scribehorus', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490538976 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497972944}]})
    option_139789490489936 = OptionDescription(children=[option_139789490536528, option_139789490536800, option_139789490537072, option_139789490537344, option_139789490537616, option_139789490537888, option_139789490538160, option_139789490538432, option_139789490538704, option_139789490538976], **{'doc': u'', 'properties': (), 'name': 'file622'})
    option_139789490539248 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490539520 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490539792 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490540064 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/var/lib/eole/01_fichier.ldif', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490544720 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490544992 = UnicodeOption(**{'default': u'/var/lib/eole/01_fichier.ldif', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490545264 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490545536 = UnicodeOption(**{'default': u'/var/lib/creole/01_fichier.ldif', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490545808 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490490256 = OptionDescription(children=[option_139789490539248, option_139789490539520, option_139789490539792, option_139789490540064, option_139789490544720, option_139789490544992, option_139789490545264, option_139789490545536, option_139789490545808], **{'doc': u'', 'properties': (), 'name': 'file623'})
    option_139789490546080 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490546352 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490546624 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490546896 = UnicodeOption(**{'default': u'/etc/host.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490547168 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490547440 = UnicodeOption(**{'default': u'/etc/host.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490547712 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490547984 = UnicodeOption(**{'default': u'/var/lib/creole/host.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490548256 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490490576 = OptionDescription(children=[option_139789490546080, option_139789490546352, option_139789490546624, option_139789490546896, option_139789490547168, option_139789490547440, option_139789490547712, option_139789490547984, option_139789490548256], **{'doc': u'', 'properties': (), 'name': 'file624'})
    option_139789490557008 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490557280 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490557552 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490557824 = UnicodeOption(**{'default': u'/etc/sudoers.d/amon', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490558096 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490558368 = UnicodeOption(**{'default': u'0440', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789490558640 = UnicodeOption(**{'default': u'/etc/sudoers.d/amon', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490558912 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490559184 = UnicodeOption(**{'default': u'/var/lib/creole/amon.sudo', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490559456 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490490896 = OptionDescription(children=[option_139789490557008, option_139789490557280, option_139789490557552, option_139789490557824, option_139789490558096, option_139789490558368, option_139789490558640, option_139789490558912, option_139789490559184, option_139789490559456], **{'doc': u'', 'properties': (), 'name': 'file625'})
    option_139789490559728 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490560000 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490560272 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490560544 = UnicodeOption(**{'default': u'/etc/cron.d/amon-firewall', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490565200 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490565472 = UnicodeOption(**{'default': u'/etc/cron.d/amon-firewall', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490565744 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490566016 = UnicodeOption(**{'default': u'/var/lib/creole/cron-root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490566288 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490569296 = OptionDescription(children=[option_139789490559728, option_139789490560000, option_139789490560272, option_139789490560544, option_139789490565200, option_139789490565472, option_139789490565744, option_139789490566016, option_139789490566288], **{'doc': u'', 'properties': (), 'name': 'file626'})
    option_139789490566560 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490566832 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490567104 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490567376 = UnicodeOption(**{'default': u'/etc/eole/ldapconf.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490567648 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490567920 = UnicodeOption(**{'default': u'/etc/eole/ldapconf.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490568192 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490568464 = UnicodeOption(**{'default': u'/var/lib/creole/ldapconf.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490568736 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490569616 = OptionDescription(children=[option_139789490566560, option_139789490566832, option_139789490567104, option_139789490567376, option_139789490567648, option_139789490567920, option_139789490568192, option_139789490568464, option_139789490568736], **{'doc': u'', 'properties': (), 'name': 'file627'})
    option_139789490577488 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490577760 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490578032 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490578304 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/eole/ldapconf.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490578576 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490578848 = UnicodeOption(**{'default': u'/etc/eole/ldapconf.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490579120 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490579392 = UnicodeOption(**{'default': u'/var/lib/creole/ldapconf.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490579664 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490569936 = OptionDescription(children=[option_139789490577488, option_139789490577760, option_139789490578032, option_139789490578304, option_139789490578576, option_139789490578848, option_139789490579120, option_139789490579392, option_139789490579664], **{'doc': u'', 'properties': (), 'name': 'file628'})
    option_139789490579936 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490580208 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490580480 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490580752 = UnicodeOption(**{'default': u'/etc/apparmor.d/tunables/home.d/homescribe', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490581024 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490585680 = UnicodeOption(**{'default': u'/etc/apparmor.d/tunables/home.d/homescribe', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490585952 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490586224 = UnicodeOption(**{'default': u'/var/lib/creole/homescribe', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490586496 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490570256 = OptionDescription(children=[option_139789490579936, option_139789490580208, option_139789490580480, option_139789490580752, option_139789490581024, option_139789490585680, option_139789490585952, option_139789490586224, option_139789490586496], **{'doc': u'', 'properties': (), 'name': 'file629'})
    option_139789490586768 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490587040 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490587312 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490587584 = UnicodeOption(**{'default': u'/var/lib/lxc/bdd/rootfs/var/lib/eole/02_scribe.ldif', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490587856 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490588128 = UnicodeOption(**{'default': u'/var/lib/eole/02_scribe.ldif', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490588400 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490588672 = UnicodeOption(**{'default': u'/var/lib/creole/02_scribe.ldif', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490588944 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490570576 = OptionDescription(children=[option_139789490586768, option_139789490587040, option_139789490587312, option_139789490587584, option_139789490587856, option_139789490588128, option_139789490588400, option_139789490588672, option_139789490588944], **{'doc': u'', 'properties': (), 'name': 'file630'})
    option_139789490589216 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490593872 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490594144 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490594416 = UnicodeOption(**{'default': u'eoe', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490594688 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/eole/flask/available/eoe.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490594960 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490595232 = UnicodeOption(**{'default': u'/etc/eole/flask/available/eoe.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490595504 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490595776 = UnicodeOption(**{'default': u'/var/lib/creole/eoe-eoe.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490596048 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497146096}]})
    option_139789490570896 = OptionDescription(children=[option_139789490589216, option_139789490593872, option_139789490594144, option_139789490594416, option_139789490594688, option_139789490594960, option_139789490595232, option_139789490595504, option_139789490595776, option_139789490596048], **{'doc': u'', 'properties': (), 'name': 'file631'})
    option_139789490596320 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490596592 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490596864 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490597136 = UnicodeOption(**{'default': u'eop', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490597408 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/eole/flask/available/eop.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490602064 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490602336 = UnicodeOption(**{'default': u'/etc/eole/flask/available/eop.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490602608 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490602880 = UnicodeOption(**{'default': u'/var/lib/creole/eop-eop.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490603152 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497146368}]})
    option_139789490571216 = OptionDescription(children=[option_139789490596320, option_139789490596592, option_139789490596864, option_139789490597136, option_139789490597408, option_139789490602064, option_139789490602336, option_139789490602608, option_139789490602880, option_139789490603152], **{'doc': u'', 'properties': (), 'name': 'file632'})
    option_139789490603424 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490603696 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490603968 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490604240 = UnicodeOption(**{'default': u'eop', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490604512 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/usr/share/eole/flask/eop/menus/motdepasse.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490604784 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490605056 = UnicodeOption(**{'default': u'/usr/share/eole/flask/eop/menus/motdepasse.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490605328 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490605600 = UnicodeOption(**{'default': u'/var/lib/creole/eop-motdepasse.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490614352 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497146368}]})
    option_139789490571536 = OptionDescription(children=[option_139789490603424, option_139789490603696, option_139789490603968, option_139789490604240, option_139789490604512, option_139789490604784, option_139789490605056, option_139789490605328, option_139789490605600, option_139789490614352], **{'doc': u'', 'properties': (), 'name': 'file633'})
    option_139789490614624 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490614896 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490615168 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490615440 = UnicodeOption(**{'default': u'websockify', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490615712 = UnicodeOption(**{'default': u'/var/lib/lxc/partage/rootfs/etc/default/eop-websockify', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490615984 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490616256 = UnicodeOption(**{'default': u'0755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789490616528 = UnicodeOption(**{'default': u'/etc/default/eop-websockify', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490616800 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490617072 = UnicodeOption(**{'default': u'/var/lib/creole/eop-websockify', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490617344 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497146368}]})
    option_139789490571856 = OptionDescription(children=[option_139789490614624, option_139789490614896, option_139789490615168, option_139789490615440, option_139789490615712, option_139789490615984, option_139789490616256, option_139789490616528, option_139789490616800, option_139789490617072, option_139789490617344], **{'doc': u'', 'properties': (), 'name': 'file634'})
    option_139789490617616 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490617888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490626640 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490626912 = UnicodeOption(**{'default': u'roundcube', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490627184 = UnicodeOption(**{'default': u'/etc/eole/eole-db.d/roundcube-db.yml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490627456 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490627728 = UnicodeOption(**{'default': u'/etc/eole/eole-db.d/roundcube-db.yml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490628000 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490628272 = UnicodeOption(**{'default': u'/var/lib/creole/roundcube-db.yml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490628544 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497143376}]})
    option_139789490572176 = OptionDescription(children=[option_139789490617616, option_139789490617888, option_139789490626640, option_139789490626912, option_139789490627184, option_139789490627456, option_139789490627728, option_139789490628000, option_139789490628272, option_139789490628544], **{'doc': u'', 'properties': (), 'name': 'file635'})
    option_139789490628816 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490629088 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490629360 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490629632 = UnicodeOption(**{'default': u'roundcube', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490629904 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/sites-available/roundcube', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490630176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490634832 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/roundcube', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490635104 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490635376 = UnicodeOption(**{'default': u'/var/lib/creole/roundcube-apache.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490635648 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497143376}]})
    option_139789490572496 = OptionDescription(children=[option_139789490628816, option_139789490629088, option_139789490629360, option_139789490629632, option_139789490629904, option_139789490630176, option_139789490634832, option_139789490635104, option_139789490635376, option_139789490635648], **{'doc': u'', 'properties': (), 'name': 'file636'})
    option_139789490635920 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490636192 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490636464 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490636736 = UnicodeOption(**{'default': u'roundcube', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490637008 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/var/www/html/roundcube/config/config.inc.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490637280 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490637552 = UnicodeOption(**{'default': u'/var/www/html/roundcube/config/config.inc.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490637824 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490638096 = UnicodeOption(**{'default': u'/var/lib/creole/roundcube-config.inc.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490638368 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497143376}]})
    option_139789490572816 = OptionDescription(children=[option_139789490635920, option_139789490636192, option_139789490636464, option_139789490636736, option_139789490637008, option_139789490637280, option_139789490637552, option_139789490637824, option_139789490638096, option_139789490638368], **{'doc': u'', 'properties': (), 'name': 'file637'})
    option_139789490651216 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490651488 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490651760 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490652032 = UnicodeOption(**{'default': u'roundcube', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490652304 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/var/www/html/roundcube/plugins/cas_authentication/config.inc.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490652576 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490652848 = UnicodeOption(**{'default': u'/var/www/html/roundcube/plugins/cas_authentication/config.inc.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490653120 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490653392 = UnicodeOption(**{'default': u'/var/lib/creole/roundcube-cas.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490653664 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497143376}]})
    option_139789490655312 = OptionDescription(children=[option_139789490651216, option_139789490651488, option_139789490651760, option_139789490652032, option_139789490652304, option_139789490652576, option_139789490652848, option_139789490653120, option_139789490653392, option_139789490653664], **{'doc': u'', 'properties': (), 'name': 'file638'})
    option_139789490653936 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490654208 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490654480 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490654752 = UnicodeOption(**{'default': u'cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490663504 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/etc/cntlm.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490663776 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490664048 = UnicodeOption(**{'default': u'/etc/cntlm.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490664320 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490664592 = UnicodeOption(**{'default': u'/var/lib/creole/cntlm.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490664864 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497574816}]})
    option_139789490655632 = OptionDescription(children=[option_139789490653936, option_139789490654208, option_139789490654480, option_139789490654752, option_139789490663504, option_139789490663776, option_139789490664048, option_139789490664320, option_139789490664592, option_139789490664864], **{'doc': u'', 'properties': (), 'name': 'file639'})
    option_139789490665136 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490665408 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490665680 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490665952 = UnicodeOption(**{'default': u'cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490666224 = UnicodeOption(**{'default': u'/var/lib/lxc/internet/rootfs/usr/sbin/eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490666496 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490666768 = UnicodeOption(**{'default': u'755', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789490667040 = UnicodeOption(**{'default': u'/usr/sbin/eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490671696 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490671968 = UnicodeOption(**{'default': u'/var/lib/creole/eole-cntlm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490672240 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': True, "option": option_139789497574816}]})
    option_139789490655952 = OptionDescription(children=[option_139789490665136, option_139789490665408, option_139789490665680, option_139789490665952, option_139789490666224, option_139789490666496, option_139789490666768, option_139789490667040, option_139789490671696, option_139789490671968, option_139789490672240], **{'doc': u'', 'properties': (), 'name': 'file640'})
    option_139789490672512 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490672784 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490673056 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490673328 = UnicodeOption(**{'default': u'pydio-bacula', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490673600 = UnicodeOption(**{'default': u'/etc/bacula/baculafichiers.d/pydio.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490673872 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490674144 = UnicodeOption(**{'default': u'/etc/bacula/baculafichiers.d/pydio.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490674416 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490674688 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-bacula.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490674960 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': ('disabled',), 'name': 'activate'})
    option_139789490656272 = OptionDescription(children=[option_139789490672512, option_139789490672784, option_139789490673056, option_139789490673328, option_139789490673600, option_139789490673872, option_139789490674144, option_139789490674416, option_139789490674688, option_139789490674960], **{'doc': u'', 'properties': (), 'name': 'file641'})
    option_139789490675232 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490679888 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490680160 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490680432 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490680704 = UnicodeOption(**{'default': u'/etc/eole/eole-db.d/pydio-db.yml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490680976 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490681248 = UnicodeOption(**{'default': u'/etc/eole/eole-db.d/pydio-db.yml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490681520 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490681792 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-db.yml', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490682064 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490656592 = OptionDescription(children=[option_139789490675232, option_139789490679888, option_139789490680160, option_139789490680432, option_139789490680704, option_139789490680976, option_139789490681248, option_139789490681520, option_139789490681792, option_139789490682064], **{'doc': u'', 'properties': (), 'name': 'file642'})
    option_139789490682336 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490682608 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490682880 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490683152 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490683424 = UnicodeOption(**{'default': u'/usr/share/eole/db/pydio/gen/pydio-create.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490688080 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490688352 = UnicodeOption(**{'default': u'/usr/share/eole/db/pydio/gen/pydio-create.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490688624 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490688896 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-create.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490689168 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490656912 = OptionDescription(children=[option_139789490682336, option_139789490682608, option_139789490682880, option_139789490683152, option_139789490683424, option_139789490688080, option_139789490688352, option_139789490688624, option_139789490688896, option_139789490689168], **{'doc': u'', 'properties': (), 'name': 'file643'})
    option_139789490689440 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490689712 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490689984 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490690256 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490690528 = UnicodeOption(**{'default': u'/usr/share/eole/db/pydio/updates/pydio-update-1.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490690800 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490691072 = UnicodeOption(**{'default': u'/usr/share/eole/db/pydio/updates/pydio-update-1.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490691344 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490691616 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-update-1.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490696272 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490657232 = OptionDescription(children=[option_139789490689440, option_139789490689712, option_139789490689984, option_139789490690256, option_139789490690528, option_139789490690800, option_139789490691072, option_139789490691344, option_139789490691616, option_139789490696272], **{'doc': u'', 'properties': (), 'name': 'file644'})
    option_139789490696544 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490696816 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490697088 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490697360 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490697632 = UnicodeOption(**{'default': u'/usr/share/eole/db/pydio/updates/pydio-update-2.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490697904 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490698176 = UnicodeOption(**{'default': u'/usr/share/eole/db/pydio/updates/pydio-update-2.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490698448 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490698720 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-update-2.sql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490698992 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490657552 = OptionDescription(children=[option_139789490696544, option_139789490696816, option_139789490697088, option_139789490697360, option_139789490697632, option_139789490697904, option_139789490698176, option_139789490698448, option_139789490698720, option_139789490698992], **{'doc': u'', 'properties': (), 'name': 'file645'})
    option_139789490699264 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490699536 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490699808 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490704464 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490704736 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/apache2/sites-available/pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490705008 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490705280 = UnicodeOption(**{'default': u'/etc/apache2/sites-available/pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490705552 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490705824 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-apache.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490706096 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490657872 = OptionDescription(children=[option_139789490699264, option_139789490699536, option_139789490699808, option_139789490704464, option_139789490704736, option_139789490705008, option_139789490705280, option_139789490705552, option_139789490705824, option_139789490706096], **{'doc': u'', 'properties': (), 'name': 'file646'})
    option_139789490706368 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490706640 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490706912 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490707184 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490707456 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/var/www/html/pydio/data/plugins/boot.conf/bootstrap.json', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490707728 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490708000 = UnicodeOption(**{'default': u'/var/www/html/pydio/data/plugins/boot.conf/bootstrap.json', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490188368 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490188640 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-bootstrap.json', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490188912 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490658192 = OptionDescription(children=[option_139789490706368, option_139789490706640, option_139789490706912, option_139789490707184, option_139789490707456, option_139789490707728, option_139789490708000, option_139789490188368, option_139789490188640, option_139789490188912], **{'doc': u'', 'properties': (), 'name': 'file647'})
    option_139789490189184 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490189456 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490189728 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490190000 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490190272 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/var/www/html/pydio/conf/bootstrap_conf.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490190544 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490190816 = UnicodeOption(**{'default': u'/var/www/html/pydio/conf/bootstrap_conf.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490191088 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490191360 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-bootstrap_conf.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490191632 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490658512 = OptionDescription(children=[option_139789490189184, option_139789490189456, option_139789490189728, option_139789490190000, option_139789490190272, option_139789490190544, option_139789490190816, option_139789490191088, option_139789490191360, option_139789490191632], **{'doc': u'', 'properties': (), 'name': 'file648'})
    option_139789490191904 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490204752 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490205024 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490205296 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490205568 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/var/www/html/pydio/conf/bootstrap_repositories.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490205840 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490206112 = UnicodeOption(**{'default': u'/var/www/html/pydio/conf/bootstrap_repositories.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490206384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490206656 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-bootstrap_repositories.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490206928 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490658832 = OptionDescription(children=[option_139789490191904, option_139789490204752, option_139789490205024, option_139789490205296, option_139789490205568, option_139789490205840, option_139789490206112, option_139789490206384, option_139789490206656, option_139789490206928], **{'doc': u'', 'properties': (), 'name': 'file649'})
    option_139789490207200 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490207472 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490207744 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490208016 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490208288 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/var/www/html/pydio/conf/EnvoleFilters.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490212944 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490213216 = UnicodeOption(**{'default': u'/var/www/html/pydio/conf/EnvoleFilters.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490213488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490213760 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-EnvoleFilters.php', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490214032 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490221136 = OptionDescription(children=[option_139789490207200, option_139789490207472, option_139789490207744, option_139789490208016, option_139789490208288, option_139789490212944, option_139789490213216, option_139789490213488, option_139789490213760, option_139789490214032], **{'doc': u'', 'properties': (), 'name': 'file650'})
    option_139789490214304 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490214576 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490214848 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490215120 = UnicodeOption(**{'default': u'pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'filelist'})
    option_139789490215392 = UnicodeOption(**{'default': u'/var/lib/lxc/reseau/rootfs/etc/cron.d/pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490215664 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490215936 = UnicodeOption(**{'default': u'/etc/cron.d/pydio', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490216208 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490216480 = UnicodeOption(**{'default': u'/var/lib/creole/pydio-cron.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490225232 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497155664}]})
    option_139789490221456 = OptionDescription(children=[option_139789490214304, option_139789490214576, option_139789490214848, option_139789490215120, option_139789490215392, option_139789490215664, option_139789490215936, option_139789490216208, option_139789490216480, option_139789490225232], **{'doc': u'', 'properties': (), 'name': 'file651'})
    option_139789490225504 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490225776 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490226048 = UnicodeOption(**{'default': u'root', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490226320 = UnicodeOption(**{'default': u'/etc/eole/bastion.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'full_name'})
    option_139789490226592 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'mkdir'})
    option_139789490226864 = UnicodeOption(**{'default': u'/etc/eole/bastion.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490227136 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'rm'})
    option_139789490227408 = UnicodeOption(**{'default': u'/var/lib/creole/bastion.conf', 'doc': u'', 'multi': False, 'properties': (), 'name': 'source'})
    option_139789490227680 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490221776 = OptionDescription(children=[option_139789490225504, option_139789490225776, option_139789490226048, option_139789490226320, option_139789490226592, option_139789490226864, option_139789490227136, option_139789490227408, option_139789490227680], **{'doc': u'', 'properties': (), 'name': 'file652'})
    option_139789490222096 = OptionDescription(children=[option_139789494987984, option_139789494988304, option_139789495029840, option_139789495030160, option_139789495030480, option_139789495030800, option_139789495031120, option_139789495031440, option_139789495031760, option_139789495032080, option_139789495032400, option_139789495032720, option_139789495033040, option_139789495033360, option_139789495115856, option_139789495116176, option_139789495116496, option_139789495116816, option_139789495117136, option_139789495117456, option_139789495117776, option_139789495118096, option_139789495118416, option_139789495118736, option_139789495119056, option_139789495119376, option_139789494685776, option_139789494686096, option_139789494686416, option_139789494686736, option_139789494687056, option_139789494687376, option_139789494687696, option_139789494688016, option_139789494688336, option_139789494688656, option_139789494688976, option_139789494689296, option_139789494767696, option_139789494768016, option_139789494768336, option_139789494768656, option_139789494768976, option_139789494769296, option_139789494769616, option_139789494769936, option_139789494770256, option_139789494770576, option_139789494770896, option_139789494771216, option_139789494857808, option_139789494858128, option_139789494858448, option_139789494858768, option_139789494859088, option_139789494859408, option_139789494859728, option_139789494860048, option_139789494860368, option_139789494860688, option_139789494861008, option_139789494861328, option_139789494411344, option_139789494411664, option_139789494411984, option_139789494412304, option_139789494412624, option_139789494412944, option_139789494413264, option_139789494413584, option_139789494413904, option_139789494414224, option_139789494414544, option_139789494414864, option_139789494509648, option_139789494509968, option_139789494510288, option_139789494510608, option_139789494510928, option_139789494511248, option_139789494511568, option_139789494511888, option_139789494512208, option_139789494512528, option_139789494512848, option_139789494513168, option_139789494595664, option_139789494595984, option_139789494596304, option_139789494596624, option_139789494596944, option_139789494597264, option_139789494597584, option_139789494597904, option_139789494598224, option_139789494598544, option_139789494598864, option_139789494599184, option_139789494153296, option_139789494153616, option_139789494153936, option_139789494154256, option_139789494154576, option_139789494154896, option_139789494155216, option_139789494155536, option_139789494155856, option_139789494156176, option_139789494156496, option_139789494156816, option_139789494227024, option_139789494227344, option_139789494227664, option_139789494227984, option_139789494228304, option_139789494228624, option_139789494228944, option_139789494229264, option_139789494229584, option_139789494229904, option_139789494230224, option_139789494230544, option_139789494313040, option_139789494313360, option_139789494313680, option_139789494314000, option_139789494314320, option_139789494314640, option_139789494314960, option_139789494315280, option_139789494315600, option_139789494315920, option_139789494316240, option_139789494316560, option_139789493870672, option_139789493870992, option_139789493871312, option_139789493871632, option_139789493871952, option_139789493872272, option_139789493872592, option_139789493872912, option_139789493873232, option_139789493873552, option_139789493873872, option_139789493874192, option_139789493944400, option_139789493944720, option_139789493945040, option_139789493945360, option_139789493945680, option_139789493946000, option_139789493946320, option_139789493946640, option_139789493946960, option_139789493947280, option_139789493947600, option_139789493947920, option_139789494030416, option_139789494030736, option_139789494031056, option_139789494031376, option_139789494031696, option_139789494032016, option_139789494032336, option_139789494032656, option_139789494032976, option_139789494033296, option_139789494033616, option_139789494033936, option_139789494116432, option_139789494116752, option_139789494117072, option_139789494117392, option_139789494117712, option_139789494118032, option_139789494118352, option_139789494118672, option_139789494118992, option_139789494119312, option_139789494119632, option_139789494119952, option_139789493682256, option_139789493682576, option_139789493682896, option_139789493683216, option_139789493683536, option_139789493683856, option_139789493684176, option_139789493684496, option_139789493684816, option_139789493685136, option_139789493685456, option_139789493685776, option_139789493780560, option_139789493780880, option_139789493781200, option_139789493781520, option_139789493781840, option_139789493782160, option_139789493782480, option_139789493782800, option_139789493783120, option_139789493783440, option_139789493783760, option_139789493784080, option_139789493346384, option_139789493346704, option_139789493347024, option_139789493347344, option_139789493347664, option_139789493347984, option_139789493348304, option_139789493348624, option_139789493348944, option_139789493349264, option_139789493349584, option_139789493349904, option_139789493432400, option_139789493432720, option_139789493433040, option_139789493433360, option_139789493433680, option_139789493434000, option_139789493434320, option_139789493434640, option_139789493434960, option_139789493435280, option_139789493435600, option_139789493435920, option_139789493518416, option_139789493518736, option_139789493519056, option_139789493519376, option_139789493519696, option_139789493520016, option_139789493520336, option_139789493520656, option_139789493520976, option_139789493521296, option_139789493521616, option_139789493521936, option_139789493092432, option_139789493092752, option_139789493093072, option_139789493093392, option_139789493093712, option_139789493094032, option_139789493094352, option_139789493094672, option_139789493094992, option_139789493095312, option_139789493095632, option_139789493095952, option_139789493178448, option_139789493178768, option_139789493179088, option_139789493179408, option_139789493179728, option_139789493180048, option_139789493180368, option_139789493180688, option_139789493181008, option_139789493181328, option_139789493181648, option_139789493181968, option_139789493268560, option_139789493268880, option_139789493269200, option_139789493269520, option_139789493269840, option_139789493270160, option_139789493270480, option_139789493270800, option_139789493271120, option_139789493271440, option_139789493271760, option_139789493272080, option_139789492830288, option_139789492830608, option_139789492830928, option_139789492831248, option_139789492831568, option_139789492831888, option_139789492832208, option_139789492832528, option_139789492832848, option_139789492833168, option_139789492833488, option_139789492833808, option_139789492904016, option_139789492904336, option_139789492904656, option_139789492904976, option_139789492905296, option_139789492905616, option_139789492905936, option_139789492906256, option_139789492906576, option_139789492906896, option_139789492907216, option_139789492907536, option_139789493002320, option_139789493002640, option_139789493002960, option_139789493003280, option_139789493003600, option_139789493003920, option_139789493004240, option_139789493004560, option_139789493004880, option_139789493005200, option_139789493005520, option_139789493005840, option_139789492559952, option_139789492560272, option_139789492560592, option_139789492560912, option_139789492561232, option_139789492561552, option_139789492561872, option_139789492562192, option_139789492562512, option_139789492562832, option_139789492563152, option_139789492563472, option_139789492629584, option_139789492629904, option_139789492630224, option_139789492630544, option_139789492630864, option_139789492631184, option_139789492631504, option_139789492631824, option_139789492632144, option_139789492632464, option_139789492632784, option_139789492633104, option_139789492715600, option_139789492715920, option_139789492716240, option_139789492716560, option_139789492716880, option_139789492717200, option_139789492717520, option_139789492717840, option_139789492718160, option_139789492718480, option_139789492718800, option_139789492719120, option_139789492797520, option_139789492797840, option_139789492798160, option_139789492798480, option_139789492798800, option_139789492799120, option_139789492799440, option_139789492799760, option_139789492800080, option_139789492800400, option_139789492800720, option_139789492801040, option_139789492367440, option_139789492367760, option_139789492368080, option_139789492368400, option_139789492368720, option_139789492369040, option_139789492369360, option_139789492369680, option_139789492370000, option_139789492370320, option_139789492370640, option_139789492370960, option_139789492457552, option_139789492457872, option_139789492458192, option_139789492458512, option_139789492458832, option_139789492459152, option_139789492459472, option_139789492459792, option_139789492460112, option_139789492460432, option_139789492460752, option_139789492461072, option_139789492543568, option_139789492543888, option_139789492544208, option_139789492544528, option_139789492544848, option_139789492545168, option_139789492545488, option_139789492545808, option_139789492546128, option_139789492546448, option_139789492546768, option_139789492547088, option_139789492101200, option_139789492101520, option_139789492101840, option_139789492102160, option_139789492102480, option_139789492102800, option_139789492103120, option_139789492103440, option_139789492103760, option_139789492104080, option_139789492104400, option_139789492104720, option_139789492183120, option_139789492183440, option_139789492183760, option_139789492184080, option_139789492184400, option_139789492184720, option_139789492185040, option_139789492185360, option_139789492185680, option_139789492186000, option_139789492186320, option_139789492186640, option_139789492269136, option_139789492269456, option_139789492269776, option_139789492270096, option_139789492270416, option_139789492270736, option_139789492271056, option_139789492271376, option_139789492271696, option_139789492272016, option_139789492272336, option_139789492272656, option_139789491822672, option_139789491822992, option_139789491823312, option_139789491823632, option_139789491823952, option_139789491824272, option_139789491824592, option_139789491824912, option_139789491825232, option_139789491825552, option_139789491825872, option_139789491826192, option_139789491912784, option_139789491913104, option_139789491913424, option_139789491913744, option_139789491914064, option_139789491914384, option_139789491914704, option_139789491915024, option_139789491915344, option_139789491915664, option_139789491915984, option_139789491916304, option_139789492002896, option_139789492003216, option_139789492003536, option_139789492003856, option_139789492004176, option_139789492004496, option_139789492004816, option_139789492005136, option_139789492005456, option_139789492005776, option_139789492006096, option_139789492006416, option_139789491560528, option_139789491560848, option_139789491561168, option_139789491561488, option_139789491561808, option_139789491562128, option_139789491562448, option_139789491562768, option_139789491563088, option_139789491563408, option_139789491563728, option_139789491564048, option_139789491658832, option_139789491659152, option_139789491659472, option_139789491659792, option_139789491660112, option_139789491660432, option_139789491660752, option_139789491661072, option_139789491661392, option_139789491661712, option_139789491662032, option_139789491662352, option_139789491744848, option_139789491745168, option_139789491745488, option_139789491745808, option_139789491746128, option_139789491746448, option_139789491746768, option_139789491747088, option_139789491747408, option_139789491747728, option_139789491748048, option_139789491748368, option_139789491310672, option_139789491310992, option_139789491311312, option_139789491311632, option_139789491311952, option_139789491312272, option_139789491312592, option_139789491312912, option_139789491313232, option_139789491313552, option_139789491313872, option_139789491314192, option_139789491392592, option_139789491392912, option_139789491393232, option_139789491393552, option_139789491393872, option_139789491394192, option_139789491394512, option_139789491394832, option_139789491395152, option_139789491395472, option_139789491395792, option_139789491396112, option_139789491486800, option_139789491487120, option_139789491487440, option_139789491487760, option_139789491488080, option_139789491488400, option_139789491488720, option_139789491489040, option_139789491489360, option_139789491489680, option_139789491490000, option_139789491490320, option_139789491044432, option_139789491044752, option_139789491045072, option_139789491045392, option_139789491045712, option_139789491046032, option_139789491046352, option_139789491046672, option_139789491046992, option_139789491047312, option_139789491047632, option_139789491047952, option_139789491130448, option_139789491130768, option_139789491131088, option_139789491131408, option_139789491131728, option_139789491132048, option_139789491132368, option_139789491132688, option_139789491133008, option_139789491133328, option_139789491133648, option_139789491133968, option_139789491216464, option_139789491216784, option_139789491217104, option_139789491217424, option_139789491217744, option_139789491218064, option_139789491218384, option_139789491218704, option_139789491219024, option_139789491219344, option_139789491219664, option_139789491219984, option_139789490782288, option_139789490782608, option_139789490782928, option_139789490783248, option_139789490783568, option_139789490783888, option_139789490784208, option_139789490784528, option_139789490784848, option_139789490785168, option_139789490785488, option_139789490785808, option_139789490851920, option_139789490852240, option_139789490852560, option_139789490852880, option_139789490853200, option_139789490853520, option_139789490853840, option_139789490854160, option_139789490854480, option_139789490854800, option_139789490855120, option_139789490855440, option_139789490925648, option_139789490925968, option_139789490926288, option_139789490926608, option_139789490926928, option_139789490927248, option_139789490927568, option_139789490927888, option_139789490928208, option_139789490928528, option_139789490928848, option_139789490929168, option_139789490487376, option_139789490487696, option_139789490488016, option_139789490488336, option_139789490488656, option_139789490488976, option_139789490489296, option_139789490489616, option_139789490489936, option_139789490490256, option_139789490490576, option_139789490490896, option_139789490569296, option_139789490569616, option_139789490569936, option_139789490570256, option_139789490570576, option_139789490570896, option_139789490571216, option_139789490571536, option_139789490571856, option_139789490572176, option_139789490572496, option_139789490572816, option_139789490655312, option_139789490655632, option_139789490655952, option_139789490656272, option_139789490656592, option_139789490656912, option_139789490657232, option_139789490657552, option_139789490657872, option_139789490658192, option_139789490658512, option_139789490658832, option_139789490221136, option_139789490221456, option_139789490221776], **{'doc': u'', 'properties': (), 'name': 'files'})
    option_139789490227952 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490228224 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490228496 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490228768 = IntOption(**{'multi': False, 'name': 'major', 'doc': u'', 'properties': ('force_default_on_freeze', 'frozen', 'hidden'), 'callback': eosfunc.cdrom_minormajor, 'callback_params': Params(args=(ParamValue("major"), ParamValue("/home")), kwargs={})})
    option_139789490241616 = IntOption(**{'multi': False, 'name': 'minor', 'doc': u'', 'properties': ('force_default_on_freeze', 'frozen', 'hidden'), 'callback': eosfunc.cdrom_minormajor, 'callback_params': Params(args=(ParamValue("minor"), ParamValue("/home")), kwargs={})})
    option_139789490241888 = UnicodeOption(**{'default': u'rwm', 'doc': u'', 'multi': False, 'properties': (), 'name': 'mode'})
    option_139789490242160 = UnicodeOption(**{'default': u'/home', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490242432 = UnicodeOption(**{'default': u'allow', 'doc': u'', 'multi': False, 'properties': (), 'name': 'permission'})
    option_139789490242704 = UnicodeOption(**{'multi': False, 'name': 'type', 'doc': u'', 'properties': ('force_default_on_freeze', 'frozen', 'hidden'), 'callback': eosfunc.device_type, 'callback_params': Params(args=(ParamValue("/home"),), kwargs={})})
    option_139789490222416 = OptionDescription(children=[option_139789490227952, option_139789490228224, option_139789490228496, option_139789490228768, option_139789490241616, option_139789490241888, option_139789490242160, option_139789490242432, option_139789490242704], **{'doc': u'', 'properties': (), 'name': 'disknod0'})
    option_139789490222736 = OptionDescription(children=[option_139789490222416], **{'doc': u'', 'properties': (), 'name': 'disknods'})
    option_139789490242976 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490243248 = UnicodeOption(**{'default': u'annuaire', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490243520 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490243792 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490244064 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490244336 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490245992 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490246552 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490244608 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490223056 = OptionDescription(children=[option_139789490242976, option_139789490243248, option_139789490243520, option_139789490243792, option_139789490244064, option_139789490244336, option_139789490245992, option_139789490246552, option_139789490244608], **{'doc': u'', 'properties': (), 'name': 'host0'})
    option_139789490244880 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490245152 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490249808 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490250080 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490250352 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490250624 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490249352 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490254184 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490250896 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490223376 = OptionDescription(children=[option_139789490244880, option_139789490245152, option_139789490249808, option_139789490250080, option_139789490250352, option_139789490250624, option_139789490249352, option_139789490254184, option_139789490250896], **{'doc': u'', 'properties': (), 'name': 'host1'})
    option_139789490251168 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490251440 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490251712 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490251984 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490252256 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490252528 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490256984 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490257544 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490252800 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490223696 = OptionDescription(children=[option_139789490251168, option_139789490251440, option_139789490251712, option_139789490251984, option_139789490252256, option_139789490252528, option_139789490256984, option_139789490257544, option_139789490252800], **{'doc': u'', 'properties': (), 'name': 'host2'})
    option_139789490253072 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490253344 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490262096 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490262368 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490262640 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490262912 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490260520 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490261080 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490263184 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490224016 = OptionDescription(children=[option_139789490253072, option_139789490253344, option_139789490262096, option_139789490262368, option_139789490262640, option_139789490262912, option_139789490260520, option_139789490261080, option_139789490263184], **{'doc': u'', 'properties': (), 'name': 'host3'})
    option_139789490263456 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490263728 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490264000 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490264272 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490264544 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490264816 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490268152 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490268712 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490265088 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490224336 = OptionDescription(children=[option_139789490263456, option_139789490263728, option_139789490264000, option_139789490264272, option_139789490264544, option_139789490264816, option_139789490268152, option_139789490268712, option_139789490265088], **{'doc': u'', 'properties': (), 'name': 'host4'})
    option_139789490265360 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490265632 = UnicodeOption(**{'default': u'dns', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490278480 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490278752 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490279024 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490279296 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490275784 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490276344 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490279568 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490224656 = OptionDescription(children=[option_139789490265360, option_139789490265632, option_139789490278480, option_139789490278752, option_139789490279024, option_139789490279296, option_139789490275784, option_139789490276344, option_139789490279568], **{'doc': u'', 'properties': (), 'name': 'host5'})
    option_139789490279840 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490280112 = UnicodeOption(**{'default': u'mysql', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490280384 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490280656 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490280928 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490281200 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490283416 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490283976 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490281472 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490286672 = OptionDescription(children=[option_139789490279840, option_139789490280112, option_139789490280384, option_139789490280656, option_139789490280928, option_139789490281200, option_139789490283416, option_139789490283976, option_139789490281472], **{'doc': u'', 'properties': (), 'name': 'host6'})
    option_139789490281744 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490282016 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490290768 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490291040 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490291312 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490291584 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490295144 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490295704 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490291856 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490286992 = OptionDescription(children=[option_139789490281744, option_139789490282016, option_139789490290768, option_139789490291040, option_139789490291312, option_139789490291584, option_139789490295144, option_139789490295704, option_139789490291856], **{'doc': u'', 'properties': (), 'name': 'host7'})
    option_139789490292128 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490292400 = UnicodeOption(**{'default': u'ftp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490292672 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490292944 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490293216 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490293488 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490298504 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490299240 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490293760 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490287312 = OptionDescription(children=[option_139789490292128, option_139789490292400, option_139789490292672, option_139789490292944, option_139789490293216, option_139789490293488, option_139789490298504, option_139789490299240, option_139789490293760], **{'doc': u'', 'properties': (), 'name': 'host8'})
    option_139789490294032 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490294304 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490303056 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490303328 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490303600 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490303872 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490302040 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490302600 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490304144 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490287632 = OptionDescription(children=[option_139789490294032, option_139789490294304, option_139789490303056, option_139789490303328, option_139789490303600, option_139789490303872, option_139789490302040, option_139789490302600, option_139789490304144], **{'doc': u'', 'properties': (), 'name': 'host9'})
    option_139789490304416 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490304688 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490304960 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490305232 = UnicodeOption(**{'default': u'bdd', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490305504 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490305776 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490309672 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490310232 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490306048 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490287952 = OptionDescription(children=[option_139789490304416, option_139789490304688, option_139789490304960, option_139789490305232, option_139789490305504, option_139789490305776, option_139789490309672, option_139789490310232, option_139789490306048], **{'doc': u'', 'properties': (), 'name': 'host10'})
    option_139789490306320 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490306592 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490319440 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490319712 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490319984 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490320256 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490313208 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490313768 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490320528 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490288272 = OptionDescription(children=[option_139789490306320, option_139789490306592, option_139789490319440, option_139789490319712, option_139789490319984, option_139789490320256, option_139789490313208, option_139789490313768, option_139789490320528], **{'doc': u'', 'properties': (), 'name': 'host11'})
    option_139789490320800 = UnicodeOption(**{'default': u'Serveur web sur l\u2019IP eth0', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490321072 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490321344 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490321616 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490321888 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490322160 = UnicodeOption(**{'default': u'web', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490324936 = SymLinkOption(**{'name': 'ip', "opt": option_139789498664080})
    option_139789490325496 = SymLinkOption(**{'name': 'name', "opt": option_139789497414256})
    option_139789490322432 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497957376}]})
    option_139789490288592 = OptionDescription(children=[option_139789490320800, option_139789490321072, option_139789490321344, option_139789490321616, option_139789490321888, option_139789490322160, option_139789490324936, option_139789490325496, option_139789490322432], **{'doc': u'', 'properties': (), 'name': 'host12'})
    option_139789490322704 = UnicodeOption(**{'default': u'serveurs d\u2019authentification SMB', 'doc': u'', 'multi': False, 'properties': (), 'name': 'comment'})
    option_139789490322976 = UnicodeOption(**{'default': u'proxy', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490327632 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490327904 = UnicodeOption(**{'default': u'internet', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490328176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'crossed'})
    option_139789490328448 = UnicodeOption(**{'default': u'auth_smb', 'doc': u'', 'multi': False, 'properties': (), 'name': 'hostlist'})
    option_139789490332568 = SymLinkOption(**{'name': 'ip', "opt": option_139789496895152})
    option_139789490333128 = SymLinkOption(**{'name': 'name', "opt": option_139789496894608})
    option_139789490328720 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'NTLM/SMB', 'action': 'disabled', 'inverse': True, "option": option_139789497574544}]})
    option_139789490288912 = OptionDescription(children=[option_139789490322704, option_139789490322976, option_139789490327632, option_139789490327904, option_139789490328176, option_139789490328448, option_139789490332568, option_139789490333128, option_139789490328720], **{'doc': u'', 'properties': (), 'name': 'host13'})
    option_139789490289232 = OptionDescription(children=[option_139789490223056, option_139789490223376, option_139789490223696, option_139789490224016, option_139789490224336, option_139789490224656, option_139789490286672, option_139789490286992, option_139789490287312, option_139789490287632, option_139789490287952, option_139789490288272, option_139789490288592, option_139789490288912], **{'doc': u'', 'properties': (), 'name': 'hosts'})
    option_139789490328992 = UnicodeOption(**{'default': u'fichier', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490329264 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490329536 = UnicodeOption(**{'default': u'partage', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490329808 = ChoiceOption(**{'multi': False, 'values': (u'auto', u'ext3', u'ext4', u'nfs', u'smb'), 'name': 'fstype', 'default': 'auto', 'doc': u'', 'properties': ()})
    option_139789490339920 = SymLinkOption(**{'name': 'name', "opt": option_139789497071824})
    option_139789490330080 = ChoiceOption(**{'multi': False, 'values': (u'bind', u'normal'), 'name': 'type', 'default': 'bind', 'doc': u'', 'properties': ()})
    option_139789490330352 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490289552 = OptionDescription(children=[option_139789490328992, option_139789490329264, option_139789490329536, option_139789490329808, option_139789490339920, option_139789490330080, option_139789490330352], **{'doc': u'', 'properties': (), 'name': 'fstab0'})
    option_139789490330624 = UnicodeOption(**{'default': u'mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container'})
    option_139789490330896 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'container_group'})
    option_139789490331168 = UnicodeOption(**{'default': u'reseau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'real_container'})
    option_139789490348112 = ChoiceOption(**{'multi': False, 'values': (u'auto', u'ext3', u'ext4', u'nfs', u'smb'), 'name': 'fstype', 'default': 'auto', 'doc': u'', 'properties': ()})
    option_139789490348384 = FilenameOption(**{'default': u'/home/mail', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490348656 = ChoiceOption(**{'multi': False, 'values': (u'bind', u'normal'), 'name': 'type', 'default': 'bind', 'doc': u'', 'properties': ()})
    option_139789490348928 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490289872 = OptionDescription(children=[option_139789490330624, option_139789490330896, option_139789490331168, option_139789490348112, option_139789490348384, option_139789490348656, option_139789490348928], **{'doc': u'', 'properties': (), 'name': 'fstab1'})
    option_139789490290192 = OptionDescription(children=[option_139789490289552, option_139789490289872], **{'doc': u'', 'properties': (), 'name': 'fstabs'})
    option_139789490352208 = OptionDescription(children=[option_139789496621008, option_139789496621328, option_139789495961552, option_139789495821968, option_139789495261136, option_139789494907600, option_139789494987664, option_139789490222096, option_139789490222736, option_139789490289232, option_139789490290192], **{'doc': u'', 'properties': (), 'name': 'containers'})
    option_139789490349200 = ChoiceOption(**{'multi': False, 'values': (u'Sauvegarder maintenant', u'Programmer une sauvegarde unique'), 'name': 'backuptype', 'default': u'Sauvegarder maintenant', 'doc': u'Type de sauvegarde', 'properties': ('mandatory', 'normal')})
    option_139789490349472 = ChoiceOption(**{'multi': False, 'values': (u'Totale', u'Diff\xe9rentielle', u'Incr\xe9mentale'), 'name': 'backupoption', 'default': u'Totale', 'doc': u'Choisir les options de sauvegarde', 'properties': ('mandatory', 'normal')})
    option_139789490349744 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23), 'name': 'hour', 'default': 0, 'doc': u'Heure', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Programmer une sauvegarde unique', 'action': 'disabled', 'inverse': True, "option": option_139789490349200}]})
    option_139789490350016 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59), 'name': 'minute', 'default': 0, 'doc': u'Minute', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Programmer une sauvegarde unique', 'action': 'disabled', 'inverse': True, "option": option_139789490349200}]})
    option_139789490350288 = DateOption(**{'doc': u'Jour', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'day', "requires": [{'expected': u'Programmer une sauvegarde unique', 'action': 'disabled', 'inverse': True, "option": option_139789490349200}]})
    option_139789490352528 = OptionDescription(children=[option_139789490349200, option_139789490349472, option_139789490349744, option_139789490350016, option_139789490350288], **{'doc': u'Sauvegarde', 'properties': ('basic',), 'name': 'sauvegarde'})
    option_139789490352528.impl_set_information("icon", None)
    option_139789490352848 = OptionDescription(children=[option_139789490352528], **{'doc': u'', 'properties': (), 'name': 'backuponce'})
    option_139789490350560 = FilenameOption(**{'default': u'/var/lib/eole/reports/rapport-bareos.txt', 'doc': u'filename', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'filename'})
    option_139789490350832 = UnicodeOption(**{'default': u'prolog', 'doc': u'language', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'language'})
    option_139789490353168 = OptionDescription(children=[option_139789490350560, option_139789490350832], **{'doc': u'Derni\xe8re sauvegarde', 'properties': ('basic',), 'name': 'options'})
    option_139789490353168.impl_set_information("icon", None)
    option_139789490353488 = OptionDescription(children=[option_139789490353168], **{'doc': u'', 'properties': (), 'name': 'backupreport'})
    option_139789490353808 = OptionDescription(children=[], **{'doc': u'backupstate', 'properties': ('basic',), 'name': 'backupstate'})
    option_139789490353808.impl_set_information("icon", None)
    option_139789490354128 = OptionDescription(children=[option_139789490353808], **{'doc': u'', 'properties': (), 'name': 'backupstate'})
    option_139789490351104 = UnicodeOption(**{'callback_params': None, 'multi': False, 'name': 'bareos_mon_password', 'doc': u'bareos_mon_password', 'properties': ('auto_freeze', 'basic'), 'callback': eosfunc.gen_random})
    option_139789490354448 = OptionDescription(children=[option_139789490351104], **{'doc': u'Moniteur', 'properties': ('hidden', 'basic'), 'name': 'monitor'})
    option_139789490354448.impl_set_information("icon", None)
    option_139789490351376 = ChoiceOption(**{'multi': False, 'values': (u'none', u'manual', u'usb', u'smb'), 'name': 'support_type', 'default': u'none', 'doc': u'Choix du type de support', 'properties': ('mandatory', 'normal')})
    option_139789490351648 = DomainnameOption(**{'type_': 'hostname', 'name': 'smb_machine', 'doc': u'Nom de domaine du serveur SMB', 'allow_ip': True, 'multi': False, 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'smb', 'action': 'disabled', 'inverse': True, "option": option_139789490351376}]})
    option_139789490368592 = IPOption(**{'multi': False, 'allow_reserved': True, 'name': 'smb_ip', 'doc': u'IP du serveur SMB', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'smb', 'action': 'disabled', 'inverse': True, "option": option_139789490351376}]})
    option_139789490368864 = UnicodeOption(**{'doc': u'Nom du partage SMB', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'smb_partage', "requires": [{'expected': u'smb', 'action': 'disabled', 'inverse': True, "option": option_139789490351376}]})
    option_139789490369136 = UnicodeOption(**{'doc': u'Identifiant SMB', 'multi': False, 'properties': ('normal',), 'name': 'smb_login', "requires": [{'expected': u'smb', 'action': 'disabled', 'inverse': True, "option": option_139789490351376}]})
    option_139789490369408 = PasswordOption(**{'doc': u'Mot de passe SMB', 'multi': False, 'properties': ('normal',), 'name': 'smb_password', "requires": [{'expected': u'smb', 'action': 'disabled', 'inverse': True, "option": option_139789490351376}]})
    option_139789490369680 = FilenameOption(**{'doc': u'Chemin USB', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'usb_path', "requires": [{'expected': u'usb', 'action': 'disabled', 'inverse': True, "option": option_139789490351376}]})
    option_139789490354768 = OptionDescription(children=[option_139789490351376, option_139789490351648, option_139789490368592, option_139789490368864, option_139789490369136, option_139789490369408, option_139789490369680], **{'doc': u'Type de support de sauvegarde', 'properties': ('basic',), 'name': 'support'})
    option_139789490354768.impl_set_information("icon", None)
    option_139789490369952 = EmailOption(**{'doc': u'Adresse pour notifier les sauvegardes r\xe9ussies', 'multi': True, 'properties': ('normal',), 'name': 'mail_ok'})
    option_139789490370224 = EmailOption(**{'doc': u'Adresse pour notifier les sauvegardes \xe9chou\xe9es', 'multi': True, 'properties': ('normal',), 'name': 'mail_error'})
    option_139789490355088 = OptionDescription(children=[option_139789490369952, option_139789490370224], **{'doc': u'Notification par courrier \xe9lectronique', 'properties': ('basic',), 'name': 'mail'})
    option_139789490355088.impl_set_information("icon", None)
    option_139789490370496 = ChoiceOption(**{'doc': u'Fr\xe9quence de sauvegarde', 'multi': True, 'values': (u'weekly', u'daily', u'monthly'), 'properties': ('normal',), 'name': 'job_type'})
    option_139789490370768 = ChoiceOption(**{'doc': u"Jour de d\xe9but d'ex\xe9cution de la sauvegarde (1:lundi)", 'multi': True, 'values': (u'1', u'2', u'3', u'4', u'5', u'6', u'7'), 'properties': ('normal',), 'name': 'day'})
    option_139789490371040 = ChoiceOption(**{'multi': True, 'values': (u'1', u'2', u'3', u'4', u'5', u'6', u'7'), 'name': 'end_day', 'doc': u"Jour de fin d'ex\xe9cution de la sauvegarde (1:lundi)", 'properties': ('normal',), "requires": [{'expected': u'daily', 'action': 'disabled', 'inverse': True, "option": option_139789490370496}]})
    option_139789490371312 = ChoiceOption(**{'doc': u'Heure de lancement de la sauvegarde', 'multi': True, 'values': (u'0', u'1', u'2', u'3', u'4', u'5', u'6', u'7', u'8', u'9', u'10', u'11', u'12', u'13', u'14', u'15', u'16', u'17', u'18', u'19', u'20', u'21', u'22', u'23'), 'properties': ('normal',), 'name': 'hour'})
    option_139789490371584 = ChoiceOption(**{'doc': u'Type de sauvegarde', 'multi': True, 'values': (u'Full', u'Incremental', u'Differential'), 'properties': ('normal',), 'name': 'level'})
    option_139789490355408 = Leadership(children=[option_139789490370496, option_139789490370768, option_139789490371040, option_139789490371312, option_139789490371584], **{'doc': u'Fr\xe9quence de sauvegarde', 'properties': ('normal',), 'name': 'job_type'})
    option_139789490355728 = OptionDescription(children=[option_139789490355408], **{'doc': u'Programmation des sauvegardes automatiques', 'properties': ('basic',), 'name': 'job'})
    option_139789490355728.impl_set_information("icon", None)
    option_139789490380880 = OptionDescription(children=[option_139789490354448, option_139789490354768, option_139789490355088, option_139789490355728], **{'doc': u'', 'properties': (), 'name': 'bareos'})
    option_139789490371856 = IntOption(**{'default': 0, 'doc': u'Dernier index utilis\xe9 lors de la r\xe9servation DHCP', 'multi': False, 'properties': ('mandatory', 'expert'), 'name': 'last_id'})
    option_139789490372128 = IntOption(**{'doc': u'Index de la r\xe9servation DHCP', 'multi': True, 'properties': ('normal',), 'name': 'id_dhcp'})
    option_139789490384976 = DomainnameOption(**{'type_': 'domainname', 'name': 'hostname', 'doc': u'Nom de la machine', 'allow_ip': True, 'allow_without_dot': True, 'multi': True, 'properties': ('mandatory', 'normal')})
    option_139789490385248 = UnicodeOption(**{'doc': u'Adresse MAC de la machine', 'multi': True, 'properties': ('mandatory', 'normal'), 'name': 'macaddress'})
    option_139789490385520 = ChoiceOption(**{'doc': u'La machine est sur une plage dynamique', 'multi': True, 'values': (u'oui', u'non'), 'properties': ('mandatory', 'normal'), 'name': 'dyn'})
    option_139789490385792 = IPOption(**{'multi': True, 'allow_reserved': True, 'name': 'ip', 'doc': u'IP \xe0 r\xe9server dans le r\xe9seau DHCP', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'oui', 'action': 'disabled', 'inverse': False, "option": option_139789490385520}]})
    option_139789490386064 = UnicodeOption(**{'doc': u'Nom de la plage dynamique', 'multi': True, 'properties': ('mandatory', 'normal'), 'name': 'name', "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789490385520}]})
    option_139789490381200 = Leadership(children=[option_139789490372128, option_139789490384976, option_139789490385248, option_139789490385520, option_139789490385792, option_139789490386064], **{'doc': u'Index de la r\xe9servation DHCP', 'properties': ('normal',), 'name': 'id_dhcp'})
    option_139789490381520 = OptionDescription(children=[option_139789490371856, option_139789490381200], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}], **{'doc': u'dhcp', 'properties': ('basic',), 'name': 'dhcp'})
    option_139789490381520.impl_set_information("icon", None)
    option_139789490381840 = OptionDescription(children=[option_139789490381520], **{'doc': u'', 'properties': (), 'name': 'dhcp'})
    option_139789490386336 = ChoiceOption(**{'multi': False, 'values': (u'oui', u'non'), 'name': 'dhcp_activation_ead3', 'default': 'non', 'doc': u"Activer l'action DHCP EAD3 (et d\xe9sactiver celle de l'EAD2) ?", 'properties': ('mandatory', 'normal')})
    option_139789490382160 = OptionDescription(children=[option_139789490386336], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}], **{'doc': u"Activation de l'action EAD3 de gestion DHCP", 'properties': ('basic',), 'name': 'dhcp_activation'})
    option_139789490382160.impl_set_information("icon", None)
    option_139789490382480 = OptionDescription(children=[option_139789490382160], **{'doc': u'', 'properties': (), 'name': 'dhcpactivation'})
    option_139789490382800 = OptionDescription(children=[], **{'doc': u'', 'properties': (), 'name': 'fileserver'})
    option_139789490386608 = FilenameOption(**{'default': u'/tmp/majlist', 'doc': u'filename', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'filename'})
    option_139789490383120 = OptionDescription(children=[option_139789490386608], **{'doc': u'Paquets \xe0 mettre \xe0 jour', 'properties': ('basic',), 'name': 'options'})
    option_139789490383120.impl_set_information("icon", None)
    option_139789490383440 = OptionDescription(children=[option_139789490383120], **{'doc': u'', 'properties': (), 'name': 'majlist'})
    option_139789490386880 = ChoiceOption(**{'multi': False, 'values': (u'Faire une mise \xe0 jour du serveur instantan\xe9ment', u'Programmer une mise \xe0 jour unique du serveur', u'Faire une mise \xe0 jour du serveur la nuit qui vient'), 'name': 'typemaj', 'default': u'Faire une mise \xe0 jour du serveur la nuit qui vient', 'doc': u'Type de la mise \xe0 jour', 'properties': ('mandatory', 'normal')})
    option_139789490387152 = ChoiceOption(**{'multi': False, 'values': (u'Uniquement la mise \xe0 jour', u'Mise \xe0 jour et reconfigure', u'Mise \xe0 jour, reconfigure et red\xe9marrage du serveur'), 'name': 'majoption', 'default': u'Mise \xe0 jour, reconfigure et red\xe9marrage du serveur', 'doc': u'Choisir les options de mise \xe0 jour', 'properties': ('mandatory', 'normal'), "requires": [{'expected': u'Programmer une mise \xe0 jour unique du serveur', 'action': 'disabled', 'inverse': True, "option": option_139789490386880}, {'expected': u'Faire une mise \xe0 jour du serveur instantan\xe9ment', 'action': 'disabled', 'inverse': True, "option": option_139789490386880}]})
    option_139789490387424 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23), 'name': 'hour', 'default': 0, 'doc': u'Heure', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Programmer une mise \xe0 jour unique du serveur', 'action': 'disabled', 'inverse': True, "option": option_139789490386880}]})
    option_139789490387696 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59), 'name': 'minute', 'default': 0, 'doc': u'Minute', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Programmer une mise \xe0 jour unique du serveur', 'action': 'disabled', 'inverse': True, "option": option_139789490386880}]})
    option_139789490387968 = DateOption(**{'doc': u'Jour', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'day', "requires": [{'expected': u'Programmer une mise \xe0 jour unique du serveur', 'action': 'disabled', 'inverse': True, "option": option_139789490386880}]})
    option_139789490383760 = OptionDescription(children=[option_139789490386880, option_139789490387152, option_139789490387424, option_139789490387696, option_139789490387968], **{'doc': u'Mise \xe0 jour unique', 'properties': ('basic',), 'name': 'mise_a_jour'})
    option_139789490383760.impl_set_information("icon", None)
    option_139789490384080 = OptionDescription(children=[option_139789490383760], **{'doc': u'', 'properties': (), 'name': 'majonce'})
    option_139789490388240 = FilenameOption(**{'default': u'/var/lib/eole/reports/rapport-maj.log', 'doc': u'filename', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'filename'})
    option_139789490388512 = UnicodeOption(**{'default': u'prolog', 'doc': u'language', 'multi': False, 'properties': ('mandatory', 'normal'), 'name': 'language'})
    option_139789490384400 = OptionDescription(children=[option_139789490388240, option_139789490388512], **{'doc': u'Derni\xe8re mise \xe0 jour', 'properties': ('basic',), 'name': 'options'})
    option_139789490384400.impl_set_information("icon", None)
    option_139789490401360 = OptionDescription(children=[option_139789490384400], **{'doc': u'', 'properties': (), 'name': 'majreport'})
    option_139789490401680 = OptionDescription(children=[], **{'doc': u'', 'properties': (), 'name': 'manageservices'})
    option_139789490405456 = ChoiceOption(**{'multi': False, 'values': (u'Red\xe9marrer le serveur instantan\xe9ment', u'Red\xe9marrer le serveur la nuit qui vient', u'Red\xe9marrer le serveur \xe0 une certaine date'), 'name': 'typeex', 'default': u'Red\xe9marrer le serveur la nuit qui vient', 'doc': u"Type d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789490405728 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23), 'name': 'hour', 'default': 0, 'doc': u'Heure', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Red\xe9marrer le serveur \xe0 une certaine date', 'action': 'disabled', 'inverse': True, "option": option_139789490405456}]})
    option_139789490406000 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59), 'name': 'minute', 'default': 0, 'doc': u'Minute', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Red\xe9marrer le serveur \xe0 une certaine date', 'action': 'disabled', 'inverse': True, "option": option_139789490405456}]})
    option_139789490406272 = DateOption(**{'doc': u'Jour', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'day', "requires": [{'expected': u'Red\xe9marrer le serveur \xe0 une certaine date', 'action': 'disabled', 'inverse': True, "option": option_139789490405456}]})
    option_139789490402000 = OptionDescription(children=[option_139789490405456, option_139789490405728, option_139789490406000, option_139789490406272], **{'doc': u'Programmer le red\xe9marrage du serveur', 'properties': ('basic',), 'name': 'reboot'})
    option_139789490402000.impl_set_information("icon", None)
    option_139789490402320 = OptionDescription(children=[option_139789490402000], **{'doc': u'', 'properties': (), 'name': 'reboot'})
    option_139789490406544 = ChoiceOption(**{'multi': False, 'values': (u'Reconfigurer le serveur instantan\xe9ment', u'Reconfigurer le serveur la nuit qui vient', u'Reconfigurer le serveur \xe0 une certaine date'), 'name': 'typeex', 'default': u'Reconfigurer le serveur la nuit qui vient', 'doc': u"Type d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789490406816 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23), 'name': 'hour', 'default': 0, 'doc': u'Heure', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Reconfigurer le serveur \xe0 une certaine date', 'action': 'disabled', 'inverse': True, "option": option_139789490406544}]})
    option_139789490407088 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59), 'name': 'minute', 'default': 0, 'doc': u'Minute', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'Reconfigurer le serveur \xe0 une certaine date', 'action': 'disabled', 'inverse': True, "option": option_139789490406544}]})
    option_139789490407360 = DateOption(**{'doc': u'Jour', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'day', "requires": [{'expected': u'Reconfigurer le serveur \xe0 une certaine date', 'action': 'disabled', 'inverse': True, "option": option_139789490406544}]})
    option_139789490402640 = OptionDescription(children=[option_139789490406544, option_139789490406816, option_139789490407088, option_139789490407360], **{'doc': u'Programmer le reconfigure du serveur', 'properties': ('basic',), 'name': 'reconfigure'})
    option_139789490402640.impl_set_information("icon", None)
    option_139789490402960 = OptionDescription(children=[option_139789490402640], **{'doc': u'', 'properties': (), 'name': 'reconfigure'})
    option_139789490407632 = ChoiceOption(**{'multi': False, 'values': (1, 2, 3, 4, 5), 'name': 'hour', 'doc': u'Heure', 'properties': ('force_store_value', 'basic'), 'callback': eosfunc.random_int, 'callback_params': Params(args=(ParamValue(1), ParamValue(5)), kwargs={})})
    option_139789490407904 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59), 'name': 'minute', 'doc': u'Minute', 'properties': ('force_store_value', 'basic'), 'callback': eosfunc.random_int, 'callback_params': Params(args=(ParamValue(0), ParamValue(59)), kwargs={})})
    option_139789490408176 = ChoiceOption(**{'multi': False, 'values': (1, 2, 3, 4, 5, 6, 7), 'name': 'weekday', 'doc': u'Jour des t\xe2ches hebdomadaires (1 : lundi)', 'properties': ('force_store_value', 'basic'), 'callback': eosfunc.random_int, 'callback_params': Params(args=(ParamValue(1), ParamValue(7)), kwargs={})})
    option_139789490408448 = ChoiceOption(**{'multi': False, 'values': (1, 2, 3, 4, 5, 6, 7), 'name': 'monthday', 'doc': u'Jour des t\xe2ches mensuelles la premi\xe8re semaine du mois (1 : lundi)', 'properties': ('force_store_value', 'basic'), 'callback': eosfunc.random_int, 'callback_params': Params(args=(ParamValue(1), ParamValue(7)), kwargs={'exclude': ParamOption(option_139789490408176, False)})})
    option_139789490408448.impl_add_consistency("not_equal", option_139789490408176, **{})
    option_139789490403280 = OptionDescription(children=[option_139789490407632, option_139789490407904, option_139789490408176, option_139789490408448], **{'doc': u"Heure et jour d'ex\xe9cution des t\xe2ches planifi\xe9es", 'properties': ('basic',), 'name': 'schedule'})
    option_139789490403280.impl_set_information("icon", None)
    option_139789490408720 = UnicodeOption(**{'default': u"Exportation de l'annuaire LDAP", 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490408992 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("local"),), kwargs={'mismatch': ParamValue("none"), 'condition_1': ParamOption(option_139789497955744, False), 'match': ParamValue("daily")})})
    option_139789490413648 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'pre', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490403600 = OptionDescription(children=[option_139789490408720, option_139789490408992, option_139789490413648], **{'doc': u"Exportation de l'annuaire LDAP", 'properties': ('basic',), 'name': 'annuaire'})
    option_139789490403600.impl_set_information("icon", None)
    option_139789490413920 = UnicodeOption(**{'default': u'Compactage de la base de donn\xe9e de Bareos', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490414192 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("monthly"), 'condition_1': ParamOption(option_139789497971584, False), 'match': ParamValue("none")})})
    option_139789490414464 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490403920 = OptionDescription(children=[option_139789490413920, option_139789490414192, option_139789490414464], **{'doc': u'Compactage de la base de donn\xe9es de Bareos', 'properties': ('basic',), 'name': 'bareos'})
    option_139789490403920.impl_set_information("icon", None)
    option_139789490414736 = UnicodeOption(**{'default': u'Exportation des quotas et du SID Samba', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490415008 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'default': 'daily', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789490415280 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'pre', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490404240 = OptionDescription(children=[option_139789490414736, option_139789490415008, option_139789490415280], **{'doc': u'Exportation des quotas et du SID Samba', 'properties': ('basic',), 'name': 'fichier'})
    option_139789490404240.impl_set_information("icon", None)
    option_139789490415552 = UnicodeOption(**{'default': u'Nettoyage des virus mis en quarantaine', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490415824 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'default': 'daily', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789490416096 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490404560 = OptionDescription(children=[option_139789490415552, option_139789490415824, option_139789490416096], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497483616}], **{'doc': u'scannedonly', 'properties': ('basic',), 'name': 'scannedonly'})
    option_139789490404560.impl_set_information("icon", None)
    option_139789490416368 = UnicodeOption(**{'default': u'Purge des corbeilles Samba', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490416640 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'default': 'daily', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789490416912 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490404880 = OptionDescription(children=[option_139789490416368, option_139789490416640, option_139789490416912], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497061456}], **{'doc': u'Purge des corbeilles Samba', 'properties': ('basic',), 'name': 'smbtrash'})
    option_139789490404880.impl_set_information("icon", None)
    option_139789490417184 = UnicodeOption(**{'default': u'Envoi des courriels li\xe9s aux d\xe9passement de quotas', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490434128 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'default': 'daily', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789490434400 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490438224 = OptionDescription(children=[option_139789490417184, option_139789490434128, option_139789490434400], requires=[{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497062272}], **{'doc': u'warnquota', 'properties': ('basic',), 'name': 'warnquota'})
    option_139789490438224.impl_set_information("icon", None)
    option_139789490434672 = UnicodeOption(**{'default': u"V\xe9rification de l'int\xe9grit\xe9 des caches Samba", 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490434944 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'default': 'daily', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789490435216 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490438544 = OptionDescription(children=[option_139789490434672, option_139789490434944, option_139789490435216], **{'doc': u"V\xe9rification de l'int\xe9grit\xe9 des caches Samba", 'properties': ('basic',), 'name': 'tdbbackup'})
    option_139789490438544.impl_set_information("icon", None)
    option_139789490435488 = UnicodeOption(**{'default': u'Mise \xe0 jour du serveur', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490435760 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'default': 'weekly', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789490436032 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490438864 = OptionDescription(children=[option_139789490435488, option_139789490435760, option_139789490436032], **{'doc': u'Mise \xe0 jour automatique', 'properties': ('basic',), 'name': 'majauto'})
    option_139789490438864.impl_set_information("icon", None)
    option_139789490436304 = UnicodeOption(**{'default': u'Mise \xe0 jour blacklist du serveur', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490436576 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'doc': u'day', 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("daily"), 'condition_1': ParamOption(option_139789497941264, False), 'match': ParamValue("none")})})
    option_139789490436848 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490439184 = OptionDescription(children=[option_139789490436304, option_139789490436576, option_139789490436848], **{'doc': u'Mise \xe0 jour des listes noires du serveur', 'properties': ('basic',), 'name': 'majblacklist'})
    option_139789490439184.impl_set_information("icon", None)
    option_139789490437120 = UnicodeOption(**{'default': u'Exportation des bases MySQL', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789490437392 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("daily"), 'condition_1': ParamOption(option_139789497971312, False), 'match': ParamValue("none")})})
    option_139789490437664 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'pre', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490439504 = OptionDescription(children=[option_139789490437120, option_139789490437392, option_139789490437664], **{'doc': u'Exportation des bases de donn\xe9es MySQL', 'properties': ('basic',), 'name': 'mysql'})
    option_139789490439504.impl_set_information("icon", None)
    option_139789489926224 = UnicodeOption(**{'default': u'Purge du cache eole-guardian', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789489926496 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'doc': u'day', 'properties': ('normal',), 'callback': eosfunc.calc_multi_condition, 'callback_params': Params(args=(ParamValue("non"),), kwargs={'mismatch': ParamValue("daily"), 'condition_1': ParamOption(option_139789497941264, False), 'match': ParamValue("none")})})
    option_139789489926768 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490439824 = OptionDescription(children=[option_139789489926224, option_139789489926496, option_139789489926768], **{'doc': u'Purge du cache eole-guardian', 'properties': ('basic',), 'name': 'purgecache'})
    option_139789490439824.impl_set_information("icon", None)
    option_139789489927040 = UnicodeOption(**{'default': u'Red\xe9marrage du serveur', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789489927312 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'default': 'none', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789489927584 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490440144 = OptionDescription(children=[option_139789489927040, option_139789489927312, option_139789489927584], **{'doc': u'Red\xe9marrage automatique', 'properties': ('hidden', 'basic'), 'name': 'z_rebootauto'})
    option_139789490440144.impl_set_information("icon", None)
    option_139789489927856 = UnicodeOption(**{'default': u'Arr\xeat du serveur', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789489928128 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'default': 'none', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789489928400 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490440464 = OptionDescription(children=[option_139789489927856, option_139789489928128, option_139789489928400], **{'doc': u'Arr\xeat automatique', 'properties': ('hidden', 'basic'), 'name': 'z_shutdownauto'})
    option_139789490440464.impl_set_information("icon", None)
    option_139789489928672 = UnicodeOption(**{'default': u'Reconfigure du serveur', 'doc': u'description', 'multi': False, 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal'), 'name': 'description'})
    option_139789489928944 = ChoiceOption(**{'multi': False, 'values': (u'none', u'daily', u'weekly', u'monthly'), 'name': 'day', 'default': 'none', 'doc': u"P\xe9riodicit\xe9 d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789489929216 = ChoiceOption(**{'multi': False, 'values': (u'pre', u'post'), 'name': 'mode', 'default': 'post', 'doc': u'mode', 'properties': ('force_default_on_freeze', 'frozen', 'hidden', 'mandatory', 'normal')})
    option_139789490440784 = OptionDescription(children=[option_139789489928672, option_139789489928944, option_139789489929216], **{'doc': u'Reconfigure automatique', 'properties': ('hidden', 'basic'), 'name': 'y_reconfigureauto'})
    option_139789490440784.impl_set_information("icon", None)
    option_139789490441104 = OptionDescription(children=[option_139789490403280, option_139789490403600, option_139789490403920, option_139789490404240, option_139789490404560, option_139789490404880, option_139789490438224, option_139789490438544, option_139789490438864, option_139789490439184, option_139789490439504, option_139789490439824, option_139789490440144, option_139789490440464, option_139789490440784], **{'doc': u'', 'properties': (), 'name': 'schedule'})
    option_139789490441424 = OptionDescription(children=[], **{'doc': u'', 'properties': (), 'name': 'scheduleonce'})
    option_139789489929488 = ChoiceOption(**{'multi': False, 'values': (u'\xc9teindre le serveur instantan\xe9ment', u'\xc9teindre le serveur la nuit qui vient', u'\xc9teindre le serveur \xe0 une certaine date'), 'name': 'typeex', 'default': u'\xc9teindre le serveur la nuit qui vient', 'doc': u"Type d'ex\xe9cution", 'properties': ('mandatory', 'normal')})
    option_139789489929760 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23), 'name': 'hour', 'default': 0, 'doc': u'Heure', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'\xc9teindre le serveur \xe0 une certaine date', 'action': 'disabled', 'inverse': True, "option": option_139789489929488}]})
    option_139789489934416 = ChoiceOption(**{'multi': False, 'values': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59), 'name': 'minute', 'default': 0, 'doc': u'Minute', 'properties': ('mandatory', 'basic'), "requires": [{'expected': u'\xc9teindre le serveur \xe0 une certaine date', 'action': 'disabled', 'inverse': True, "option": option_139789489929488}]})
    option_139789489934688 = DateOption(**{'doc': u'Jour', 'multi': False, 'properties': ('mandatory', 'basic'), 'name': 'day', "requires": [{'expected': u'\xc9teindre le serveur \xe0 une certaine date', 'action': 'disabled', 'inverse': True, "option": option_139789489929488}]})
    option_139789490441744 = OptionDescription(children=[option_139789489929488, option_139789489929760, option_139789489934416, option_139789489934688], **{'doc': u"Programmer l'arr\xeat du serveur", 'properties': ('basic',), 'name': 'shutdown'})
    option_139789490441744.impl_set_information("icon", None)
    option_139789489942608 = OptionDescription(children=[option_139789490441744], **{'doc': u'', 'properties': (), 'name': 'shutdown'})
    option_139789489934960 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'actionlist'})
    option_139789489935232 = UnicodeOption(**{'default': u'Ex\xe9cuter une sauvegarde (maintenant ou programm\xe9e)', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489935504 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789489935776 = UnicodeOption(**{'default': u'icons/x-office-calendar.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789489936048 = UnicodeOption(**{'default_multi': u'Appliquer', 'multi': True, 'name': 'input', 'default': [u'Appliquer'], 'doc': u'', 'properties': ()})
    option_139789489936320 = UnicodeOption(**{'default': u'backuponce', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789489936592 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789489936864 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789489937136 = UnicodeOption(**{'default_multi': u'sauvegarde', 'multi': True, 'name': 'tag', 'default': [u'sauvegarde'], 'doc': u'', 'properties': ()})
    option_139789489937408 = UnicodeOption(**{'default': u'Ex\xe9cuter une sauvegarde', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789489937680 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'form', 'doc': u'', 'properties': ()})
    option_139789489937952 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789489942928 = OptionDescription(children=[option_139789489934960, option_139789489935232, option_139789489935504, option_139789489935776, option_139789489936048, option_139789489936320, option_139789489936592, option_139789489936864, option_139789489937136, option_139789489937408, option_139789489937680, option_139789489937952], **{'doc': u'', 'properties': (), 'name': 'action0'})
    option_139789489950800 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'actionlist'})
    option_139789489951072 = UnicodeOption(**{'default': u'Visualiser le journal de la derni\xe8re sauvegarde', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489951344 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789489951616 = UnicodeOption(**{'default': u'icons/edit-find.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789489951888 = UnicodeOption(**{'default': u'backupreport', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789489952160 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789489952432 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789489952704 = UnicodeOption(**{'default_multi': u'log', 'multi': True, 'name': 'tag', 'default': [u'log', u'sauvegarde'], 'doc': u'', 'properties': ()})
    option_139789489952976 = UnicodeOption(**{'default': u'Rapport de sauvegarde', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789489953248 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'reader', 'doc': u'', 'properties': ()})
    option_139789489953520 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789489943248 = OptionDescription(children=[option_139789489950800, option_139789489951072, option_139789489951344, option_139789489951616, option_139789489951888, option_139789489952160, option_139789489952432, option_139789489952704, option_139789489952976, option_139789489953248, option_139789489953520], **{'doc': u'', 'properties': (), 'name': 'action1'})
    option_139789489953792 = UnicodeOption(**{'default': u'backupstate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'actionlist'})
    option_139789489954064 = FilenameOption(**{'default': u'bareos-webui', 'doc': u'', 'multi': False, 'properties': (), 'name': 'apache_path'})
    option_139789489954336 = UnicodeOption(**{'default': u"Acc\xe9der \xe0 bareos-web pour voir l'\xe9tat des sauvegardes (en cours, en attente, r\xe9ussies, en \xe9chec)", 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489963088 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789489963360 = UnicodeOption(**{'default': u'backup.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789489963632 = UnicodeOption(**{'default': u'backupstate', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789489963904 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789489964176 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789489964448 = UnicodeOption(**{'default_multi': u'sauvegarde', 'multi': True, 'name': 'tag', 'default': [u'sauvegarde', u'bareos', u'bareoswebui'], 'doc': u'', 'properties': ()})
    option_139789489964720 = UnicodeOption(**{'default': u"Afficher l'\xe9tat des sauvegardes", 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789489964992 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'apache', 'doc': u'', 'properties': ()})
    option_139789489965264 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497145280}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789489943568 = OptionDescription(children=[option_139789489953792, option_139789489954064, option_139789489954336, option_139789489963088, option_139789489963360, option_139789489963632, option_139789489963904, option_139789489964176, option_139789489964448, option_139789489964720, option_139789489964992, option_139789489965264], **{'doc': u'', 'properties': (), 'name': 'action2'})
    option_139789489965536 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'actionlist'})
    option_139789489965808 = UnicodeOption(**{'default': u'Configurer la sauvegarde (type de support, notification par courrier \xe9lectronique, programmation)', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489966080 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789489966352 = UnicodeOption(**{'default': u'configuration.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789489966624 = UnicodeOption(**{'default_multi': u'Configurer', 'multi': True, 'name': 'input', 'default': [u'Configurer'], 'doc': u'', 'properties': ()})
    option_139789489975376 = UnicodeOption(**{'default': u'bareos', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789489975648 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789489975920 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789489976192 = UnicodeOption(**{'default_multi': u'configuration', 'multi': True, 'name': 'tag', 'default': [u'configuration', u'sauvegarde'], 'doc': u'', 'properties': ()})
    option_139789489976464 = UnicodeOption(**{'default': u'Configurer la sauvegarde', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789489976736 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'form', 'doc': u'', 'properties': ()})
    option_139789489977008 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}, {'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497971584}]})
    option_139789489943888 = OptionDescription(children=[option_139789489965536, option_139789489965808, option_139789489966080, option_139789489966352, option_139789489966624, option_139789489975376, option_139789489975648, option_139789489975920, option_139789489976192, option_139789489976464, option_139789489976736, option_139789489977008], **{'doc': u'', 'properties': (), 'name': 'action3'})
    option_139789489977280 = UnicodeOption(**{'default': u'Sauvegarde', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789489977552 = UnicodeOption(**{'default': u'#99e6ff', 'doc': u'', 'multi': False, 'properties': (), 'name': 'color'})
    option_139789489977824 = UnicodeOption(**{'default': u'icons/media-floppy.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789489978096 = UnicodeOption(**{'default': u'Gestion de la sauvegarde', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489944208 = OptionDescription(children=[option_139789489942928, option_139789489943248, option_139789489943568, option_139789489943888, option_139789489977280, option_139789489977552, option_139789489977824, option_139789489978096], **{'doc': u'Sauvegarde', 'properties': (), 'name': 'sauvegarde'})
    option_139789489978368 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'actionlist'})
    option_139789489978640 = UnicodeOption(**{'default': u"G\xe9rer la configuration DHCP via la visualisation des baux en cours et la r\xe9servation d'IP", 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489978912 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789489983568 = UnicodeOption(**{'default': u'icons/document-properties.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789489983840 = UnicodeOption(**{'default': u'dhcp', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789489984112 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789489984384 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789489984656 = UnicodeOption(**{'default_multi': u'dhcp', 'multi': True, 'name': 'tag', 'default': [u'dhcp', u'r\xe9seau'], 'doc': u'', 'properties': ()})
    option_139789489984928 = UnicodeOption(**{'default': u'Param\xe9trer le DHCP', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789489985200 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'custom', 'doc': u'', 'properties': ()})
    option_139789489985472 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}]})
    option_139789489944528 = OptionDescription(children=[option_139789489978368, option_139789489978640, option_139789489978912, option_139789489983568, option_139789489983840, option_139789489984112, option_139789489984384, option_139789489984656, option_139789489984928, option_139789489985200, option_139789489985472], **{'doc': u'', 'properties': (), 'name': 'action0'})
    option_139789489985744 = UnicodeOption(**{'default': u'dhcpactivation', 'doc': u'', 'multi': False, 'properties': (), 'name': 'actionlist'})
    option_139789489986016 = UnicodeOption(**{'default': u'Gestion de la Compatibilit\xe9 EAD2/EAD3 pour le DHCP', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489986288 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789489986560 = UnicodeOption(**{'default': u'icons/document-properties.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789489986832 = UnicodeOption(**{'default_multi': u'Valider', 'multi': True, 'name': 'input', 'default': [u'Valider'], 'doc': u'', 'properties': ()})
    option_139789489987104 = UnicodeOption(**{'default': u'dhcpactivation', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789489991760 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789489992032 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789489992304 = UnicodeOption(**{'default_multi': u'dhcp', 'multi': True, 'name': 'tag', 'default': [u'dhcp', u'r\xe9seau', u'compatibilit\xe9', u'EAD2', u'EAD3'], 'doc': u'', 'properties': ()})
    option_139789489992576 = UnicodeOption(**{'default': u'DHCP : Compatibilit\xe9 EAD2/EAD3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789489992848 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'form', 'doc': u'', 'properties': ()})
    option_139789489993120 = BoolOption(**{'multi': False, 'name': 'activate', 'default': True, 'doc': u'', 'properties': (), "requires": [{'expected': u'non', 'action': 'disabled', 'inverse': False, "option": option_139789497956016}]})
    option_139789489944848 = OptionDescription(children=[option_139789489985744, option_139789489986016, option_139789489986288, option_139789489986560, option_139789489986832, option_139789489987104, option_139789489991760, option_139789489992032, option_139789489992304, option_139789489992576, option_139789489992848, option_139789489993120], **{'doc': u'', 'properties': (), 'name': 'action1'})
    option_139789489993392 = UnicodeOption(**{'default': u'R\xe9seau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789489993664 = UnicodeOption(**{'default': u'#3ca5d2', 'doc': u'', 'multi': False, 'properties': (), 'name': 'color'})
    option_139789489993936 = UnicodeOption(**{'default': u'icons/applications-internet.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789489994208 = UnicodeOption(**{'default': u'Gestion du r\xe9seau', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489945168 = OptionDescription(children=[option_139789489944528, option_139789489944848, option_139789489993392, option_139789489993664, option_139789489993936, option_139789489994208], **{'doc': u'R\xe9seau', 'properties': (), 'name': 'reseau'})
    option_139789489994480 = UnicodeOption(**{'default': u'fileserver', 'doc': u'', 'multi': False, 'properties': (), 'name': 'actionlist'})
    option_139789489994752 = UnicodeOption(**{'default': u'T\xe9l\xe9versement/t\xe9l\xe9chargement de fichiers pour les actions EAD3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489995024 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789489995296 = UnicodeOption(**{'default': u'icons/system-file-manager.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490008144 = UnicodeOption(**{'default': u'fileserver', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490008416 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490008688 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490008960 = UnicodeOption(**{'default_multi': u'fichiers', 'multi': True, 'name': 'tag', 'default': [u'fichiers', u't\xe9l\xe9chargement', u't\xe9l\xe9versement'], 'doc': u'', 'properties': ()})
    option_139789490009232 = UnicodeOption(**{'default': u'G\xe9rer les fichiers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490009504 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'custom', 'doc': u'', 'properties': ()})
    option_139789490009776 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789489945488 = OptionDescription(children=[option_139789489994480, option_139789489994752, option_139789489995024, option_139789489995296, option_139789490008144, option_139789490008416, option_139789490008688, option_139789490008960, option_139789490009232, option_139789490009504, option_139789490009776], **{'doc': u'', 'properties': (), 'name': 'action0'})
    option_139789490010048 = UnicodeOption(**{'default': u'Fichiers', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490010320 = UnicodeOption(**{'default': u'#fcce74', 'doc': u'', 'multi': False, 'properties': (), 'name': 'color'})
    option_139789490010592 = UnicodeOption(**{'default': u'icons/system-file-manager.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490010864 = UnicodeOption(**{'default': u'Stockage de fichiers pour les actions EAD3', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789489945808 = OptionDescription(children=[option_139789489945488, option_139789490010048, option_139789490010320, option_139789490010592, option_139789490010864], **{'doc': u'Fichiers', 'properties': (), 'name': 'fichiers'})
    option_139789490011136 = UnicodeOption(**{'default': u'Afficher la liste des paquets disponibles', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490011408 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789490011680 = UnicodeOption(**{'default': u'icons/system-software-update.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490020432 = UnicodeOption(**{'default': u'majlist', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490020704 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490020976 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490021248 = UnicodeOption(**{'default_multi': u'maj', 'multi': True, 'name': 'tag', 'default': [u'maj', u'syst\xe8me'], 'doc': u'', 'properties': ()})
    option_139789490021520 = UnicodeOption(**{'default': u'Paquets \xe0 mettre \xe0 jour', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490021792 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'reader', 'doc': u'', 'properties': ()})
    option_139789490022064 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789489946128 = OptionDescription(children=[option_139789490011136, option_139789490011408, option_139789490011680, option_139789490020432, option_139789490020704, option_139789490020976, option_139789490021248, option_139789490021520, option_139789490021792, option_139789490022064], **{'doc': u'', 'properties': (), 'name': 'action0'})
    option_139789490022336 = UnicodeOption(**{'default': u'Programmer une mise \xe0 jour (maintenant, cette nuit, \xe0 ex\xe9cution programm\xe9e)', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490022608 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789490022880 = UnicodeOption(**{'default': u'icons/x-office-calendar.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490023152 = UnicodeOption(**{'default_multi': u'Appliquer', 'multi': True, 'name': 'input', 'default': [u'Appliquer'], 'doc': u'', 'properties': ()})
    option_139789490023424 = UnicodeOption(**{'default': u'majonce', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490023696 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490023968 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490028624 = UnicodeOption(**{'default_multi': u'maj-auto', 'multi': True, 'name': 'tag', 'default': [u'maj-auto', u'update', u'mise \xe0 jour', u'schedule'], 'doc': u'', 'properties': ()})
    option_139789490028896 = UnicodeOption(**{'default': u'Mise \xe0 jour', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490029168 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'form', 'doc': u'', 'properties': ()})
    option_139789490029440 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490032720 = OptionDescription(children=[option_139789490022336, option_139789490022608, option_139789490022880, option_139789490023152, option_139789490023424, option_139789490023696, option_139789490023968, option_139789490028624, option_139789490028896, option_139789490029168, option_139789490029440], **{'doc': u'', 'properties': (), 'name': 'action1'})
    option_139789490029712 = UnicodeOption(**{'default': u'Afficher le journal de la derni\xe8re mise \xe0 jour', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490029984 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789490030256 = UnicodeOption(**{'default': u'icons/edit-find.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490030528 = UnicodeOption(**{'default': u'majreport', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490030800 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490031072 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490031344 = UnicodeOption(**{'default_multi': u'log', 'multi': True, 'name': 'tag', 'default': [u'log', u'maj', u'maj-auto', u'mise \xe0 jour'], 'doc': u'', 'properties': ()})
    option_139789490031616 = UnicodeOption(**{'default': u'Rapport de mise \xe0 jour', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490031888 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'reader', 'doc': u'', 'properties': ()})
    option_139789490032160 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490033040 = OptionDescription(children=[option_139789490029712, option_139789490029984, option_139789490030256, option_139789490030528, option_139789490030800, option_139789490031072, option_139789490031344, option_139789490031616, option_139789490031888, option_139789490032160], **{'doc': u'', 'properties': (), 'name': 'action2'})
    option_139789490040912 = UnicodeOption(**{'default': u'Mise \xe0 jour', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490041184 = UnicodeOption(**{'default': u'#fca474', 'doc': u'', 'multi': False, 'properties': (), 'name': 'color'})
    option_139789490041456 = UnicodeOption(**{'default': u'icons/applications-internet.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490041728 = UnicodeOption(**{'default': u'Gestion de la mise \xe0 jour', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490033360 = OptionDescription(children=[option_139789489946128, option_139789490032720, option_139789490033040, option_139789490040912, option_139789490041184, option_139789490041456, option_139789490041728], **{'doc': u'Mise \xe0 jour', 'properties': (), 'name': 'mise_a_jour'})
    option_139789490042000 = UnicodeOption(**{'default': u'Gestion des services', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490042272 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789490042544 = UnicodeOption(**{'default': u'icons/preferences-system-session.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490042816 = UnicodeOption(**{'default': u'manageservices', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490043088 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490043360 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490043632 = UnicodeOption(**{'default_multi': u'services', 'multi': True, 'name': 'tag', 'default': [u'services'], 'doc': u'', 'properties': ()})
    option_139789490043904 = UnicodeOption(**{'default': u'Services', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490044176 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'custom', 'doc': u'', 'properties': ()})
    option_139789490044448 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490033680 = OptionDescription(children=[option_139789490042000, option_139789490042272, option_139789490042544, option_139789490042816, option_139789490043088, option_139789490043360, option_139789490043632, option_139789490043904, option_139789490044176, option_139789490044448], **{'doc': u'', 'properties': (), 'name': 'action0'})
    option_139789490049104 = UnicodeOption(**{'default': u'Programmer le red\xe9marrage du serveur', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490049376 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789490049648 = UnicodeOption(**{'default': u'icons/system-shutdown.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490049920 = UnicodeOption(**{'default_multi': u'Appliquer', 'multi': True, 'name': 'input', 'default': [u'Appliquer'], 'doc': u'', 'properties': ()})
    option_139789490050192 = UnicodeOption(**{'default': u'reboot', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490050464 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490050736 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490051008 = UnicodeOption(**{'default_multi': u'reboot', 'multi': True, 'name': 'tag', 'default': [u'reboot', u'red\xe9marrage'], 'doc': u'', 'properties': ()})
    option_139789490051280 = UnicodeOption(**{'default': u'Red\xe9marrer', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490051552 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'form', 'doc': u'', 'properties': ()})
    option_139789490051824 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490034000 = OptionDescription(children=[option_139789490049104, option_139789490049376, option_139789490049648, option_139789490049920, option_139789490050192, option_139789490050464, option_139789490050736, option_139789490051008, option_139789490051280, option_139789490051552, option_139789490051824], **{'doc': u'', 'properties': (), 'name': 'action1'})
    option_139789490052096 = UnicodeOption(**{'default': u'Programmer le reconfigure du serveur', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490052368 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789490052640 = UnicodeOption(**{'default': u'icons/preferences-system-session.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490057296 = UnicodeOption(**{'default_multi': u'Appliquer', 'multi': True, 'name': 'input', 'default': [u'Appliquer'], 'doc': u'', 'properties': ()})
    option_139789490057568 = UnicodeOption(**{'default': u'reconfigure', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490057840 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490058112 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490058384 = UnicodeOption(**{'default_multi': u'reconfigure', 'multi': True, 'name': 'tag', 'default': [u'reconfigure'], 'doc': u'', 'properties': ()})
    option_139789490058656 = UnicodeOption(**{'default': u'Reconfigurer', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490058928 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'form', 'doc': u'', 'properties': ()})
    option_139789490059200 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490034320 = OptionDescription(children=[option_139789490052096, option_139789490052368, option_139789490052640, option_139789490057296, option_139789490057568, option_139789490057840, option_139789490058112, option_139789490058384, option_139789490058656, option_139789490058928, option_139789490059200], **{'doc': u'', 'properties': (), 'name': 'action2'})
    option_139789490059472 = UnicodeOption(**{'default': u"Programmer l'arr\xeat du serveur", 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490059744 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789490060016 = UnicodeOption(**{'default': u'icons/system-shutdown.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490060288 = UnicodeOption(**{'default_multi': u'Appliquer', 'multi': True, 'name': 'input', 'default': [u'Appliquer'], 'doc': u'', 'properties': ()})
    option_139789490060560 = UnicodeOption(**{'default': u'shutdown', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490060832 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490065488 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490065760 = UnicodeOption(**{'default_multi': u'shutdown', 'multi': True, 'name': 'tag', 'default': [u'shutdown', u'arr\xeat', u'\xe9teindre', u'halt', u'poweroff'], 'doc': u'', 'properties': ()})
    option_139789490066032 = UnicodeOption(**{'default': u'\xc9teindre', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490066304 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'form', 'doc': u'', 'properties': ()})
    option_139789490066576 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490034640 = OptionDescription(children=[option_139789490059472, option_139789490059744, option_139789490060016, option_139789490060288, option_139789490060560, option_139789490060832, option_139789490065488, option_139789490065760, option_139789490066032, option_139789490066304, option_139789490066576], **{'doc': u'', 'properties': (), 'name': 'action3'})
    option_139789490066848 = UnicodeOption(**{'default': u'Syst\xe8me', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490067120 = UnicodeOption(**{'default': u'#ddc9e6', 'doc': u'', 'multi': False, 'properties': (), 'name': 'color'})
    option_139789490067392 = UnicodeOption(**{'default': u'system.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490067664 = UnicodeOption(**{'default': u'Gestion du serveur', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490034960 = OptionDescription(children=[option_139789490033680, option_139789490034000, option_139789490034320, option_139789490034640, option_139789490066848, option_139789490067120, option_139789490067392, option_139789490067664], **{'doc': u'Syst\xe8me', 'properties': (), 'name': 'systeme'})
    option_139789490067936 = UnicodeOption(**{'default': u'Param\xe9trer les t\xe2ches planifi\xe9es (heure, jour)', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490068208 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789490068480 = UnicodeOption(**{'default': u'icons/x-office-calendar.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490068752 = UnicodeOption(**{'default_multi': u'Programmer', 'multi': True, 'name': 'input', 'default': [u'Programmer'], 'doc': u'', 'properties': ()})
    option_139789490069024 = UnicodeOption(**{'default': u'schedule', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490073680 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490073952 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490074224 = UnicodeOption(**{'default_multi': u'maj', 'multi': True, 'name': 'tag', 'default': [u'maj', u'schedule'], 'doc': u'', 'properties': ()})
    option_139789490074496 = UnicodeOption(**{'default': u'T\xe2ches planifi\xe9es', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490074768 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'form', 'doc': u'', 'properties': ()})
    option_139789490075040 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490035280 = OptionDescription(children=[option_139789490067936, option_139789490068208, option_139789490068480, option_139789490068752, option_139789490069024, option_139789490073680, option_139789490073952, option_139789490074224, option_139789490074496, option_139789490074768, option_139789490075040], **{'doc': u'', 'properties': (), 'name': 'action0'})
    option_139789490075312 = UnicodeOption(**{'default': u'Liste des t\xe2ches planifi\xe9es cette nuit', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490075584 = UnicodeOption(**{'default_multi': u'ead', 'multi': True, 'name': 'ewtapp', 'default': [u'ead'], 'doc': u'', 'properties': ()})
    option_139789490075856 = UnicodeOption(**{'default': u'icons/preferences-desktop-screensaver.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490076128 = UnicodeOption(**{'default': u'scheduleonce', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490076400 = UnicodeOption(**{'default_multi': u'ead_admin', 'multi': True, 'name': 'profile', 'default': [u'ead_admin'], 'doc': u'', 'properties': ()})
    option_139789490076672 = BoolOption(**{'default': False, 'doc': u'', 'multi': False, 'properties': (), 'name': 'save'})
    option_139789490076944 = UnicodeOption(**{'default_multi': u'schedule', 'multi': True, 'name': 'tag', 'default': [u'schedule'], 'doc': u'', 'properties': ()})
    option_139789490077216 = UnicodeOption(**{'default': u'T\xe2ches uniques', 'doc': u'', 'multi': False, 'properties': (), 'name': 'title'})
    option_139789490085968 = ChoiceOption(**{'multi': False, 'values': (u'form', u'custom', u'external', u'reader', u'apache'), 'name': 'type', 'default': 'custom', 'doc': u'', 'properties': ()})
    option_139789490086240 = BoolOption(**{'default': True, 'doc': u'', 'multi': False, 'properties': (), 'name': 'activate'})
    option_139789490035600 = OptionDescription(children=[option_139789490075312, option_139789490075584, option_139789490075856, option_139789490076128, option_139789490076400, option_139789490076672, option_139789490076944, option_139789490077216, option_139789490085968, option_139789490086240], **{'doc': u'', 'properties': (), 'name': 'action1'})
    option_139789490086512 = UnicodeOption(**{'default': u'T\xe2ches planifi\xe9es', 'doc': u'', 'multi': False, 'properties': (), 'name': 'name'})
    option_139789490086784 = UnicodeOption(**{'default': u'#8cd98c', 'doc': u'', 'multi': False, 'properties': (), 'name': 'color'})
    option_139789490087056 = UnicodeOption(**{'default': u'icons/appointment-new.svg', 'doc': u'', 'multi': False, 'properties': (), 'name': 'image'})
    option_139789490087328 = UnicodeOption(**{'default': u'Gestion des t\xe2ches planifi\xe9es', 'doc': u'', 'multi': False, 'properties': (), 'name': 'description'})
    option_139789490035920 = OptionDescription(children=[option_139789490035280, option_139789490035600, option_139789490086512, option_139789490086784, option_139789490087056, option_139789490087328], **{'doc': u'T\xe2ches planifi\xe9es', 'properties': (), 'name': 'taches_planifiees'})
    option_139789490036240 = OptionDescription(children=[option_139789489944208, option_139789489945168, option_139789489945808, option_139789490033360, option_139789490034960, option_139789490035920], **{'doc': u'actions', 'properties': (), 'name': 'actions'})
    option_139789490094160 = OptionDescription(children=[option_139789496574672, option_139789490352208, option_139789490352848, option_139789490353488, option_139789490354128, option_139789490380880, option_139789490381840, option_139789490382480, option_139789490382800, option_139789490383440, option_139789490384080, option_139789490401360, option_139789490401680, option_139789490402320, option_139789490402960, option_139789490441104, option_139789490441424, option_139789489942608, option_139789490036240], **{'doc': u'', 'properties': (), 'name': 'baseoption'})
    return Config(option_139789490094160, persistent=True, session_id="amonecole", display_name=display_name)
